#!/usr/bin/perl -I. -I.. -I/opt/novell/xad/lib64/perl -I/opt/novell/xad/lib/perl

################################################################################
# Domain Services for Windows Provisioning Tool
#
# See the file LICENSE for redistribution information.
#
# Copyright (c) 2001-2009
#	Novell, Inc.  All rights reserved.
################################################################################
#
#   NAME:
#         child_install.pm
#
#   FACILITY:
#         Additional Domain Controller Installation
#
#   ABSTRACT:
#
#   REVISION: $Id$
#
#################################################################################

package adc_install;

use NDS::stage;
#use NDS::replops;
#use NDS::import;
use XAD::registry;
use XAD::dns;
use XAD::nds;
use XAD::secure;
use Locale::gettext;
use File::Copy;
use File::Path qw(mkpath);
use Logger qw(:DEFAULT);
use util;


if (registry::getReg("INSTALL_TYPE") eq "frd") {
	#Based on the install type the hierarchy is made
	require Install::frd_install;
	our @ISA = qw(frd_install);
}
elsif (registry::getReg("INSTALL_TYPE") eq "child") {
	require Install::child_install;
	our @ISA = qw(child_install);
}

my $self = {};

sub new
{
	my $class = shift;
	$self = {@_};
	bless($self, $class);

	#Calls the constructor of the parent class
	if ($ISA[0] eq "frd_install") {
		frd_install::new();
	}
	elsif ($ISA[0] eq "child_install") {
		child_install::new();
	}

	Logger::set_filename("/var/opt/novell/xad/log/ndsdcinit.log");
	refresh_reg();

	return $self;
}

sub croak { die "$0: @_: $!\n" }

#Updates the xad.ini with Mapped Domain NC value
sub stage_default_config
{

	Log(MESG, ">>>Updating domain configuration");

	my $xad_ini_file = registry::getReg("LIVE_ETCDIR")."/xad.ini";
	$nds_admin_server_context = registry::getReg("NDSADMINSERVERCONTEXT");
	$nds_admin_server_context =~ s/\./,/g;
	$nds_admin_server_context =~ s/,dc=/,DC=/g;
	$nds_admin_server_context =~ s/st=/s=/gi;

	util::file_append_str("Mapped Domain NC = $nds_admin_server_context\n", $xad_ini_file);

	#Refreh the registry, based on this value lot of other value will change
	refresh_reg();

	eval {
		#For Forest Root Domain there is no TPO. uid, gid are assigned simply
		if (registry::getReg("INSTALL_TYPE") ne "frd") {

			my $domain_name = registry::getReg("Root Domain");
			my $domain_provider = `/opt/novell/xad/sbin/provision -q -q -l 60 --locate-dc $domain_name`;
			if ($?) {
				my $error = $? >> 8;
				die("Failed to get the domain provider for domain name [$domain_name] [$error]");
			}
			chomp($domain_provider);
			my $ldapadmin = registry::getReg("LDAPADMINNAME");

			my @out = `/usr/bin/ldapsearch -h $domain_provider -Y GSS-SPNEGO -b "$ldapadmin"  -LLL -s base uidNumber`;
			if ($?) {
				die("Could not get the Administrator uid number");
			}

			my @grepout = grep(/uidNumber/i,  @out);
			@grepout[0] =~ /uidNumber:\s*(\d+)/i;
			my $uidNumber = $1;

			if ($uidNumber != "") {
				my $trust_posix_offset = int($uidNumber) - 500;
				my $xad_ini_file = registry::getReg("LIVE_ETCDIR")."/xad.ini";

				util::file_append_str("TRUSTPOSIXOFFSET = $trust_posix_offset\n", $xad_ini_file);

				#Calculate the values admin UID, GID which depends on trustposixoffset.
				registry::setReg("TRUSTPOSIXOFFSET", $trust_posix_offset);
				registry::setReg("ADMINISTRATOR_UID", $trust_posix_offset + 500);
				registry::setReg("DOMAIN_ADMINS_GID", $trust_posix_offset + 512);

			}
			else {
				die("Could not get the tpo value");
			}
		}
	};

	if ($@) { Log(ERR, $@) }

	Log(MESG, "   Domain configuration successfully updated");
}

sub stage_config
{
	my $templatedir = registry::getReg("TEMPLATEDIR");
	my $dsstatedir = registry::getReg("DSSTATEDIR");

	#This is to generate the crossref object's reference into the ncp server object.
	Log ("   Generating crossrefreference.ldif");

	stage::gen_ldif("$templatedir/config-bl.ini", '>', "$dsstatedir/config/crossrefreference.ldif", "--ncpdn");
}

sub stage_domain
{
	Log(MESG, ">>>Preparing Domain Naming Context");

	decide_domain_zones();

	$self->SUPER::stage_domain();
}

#Configures the samba
sub stage_samba
{

	my $tempdir = registry::getReg("TEMPDIR");

	if (-e $tempdir."/stage_samba") {
		Log("The stage of the SMB server configuration was already done.");
		return 0;
	}

	stage::stage_samba();

	my $templatedir = registry::getReg("TEMPLATEDIR");
	my $stagedir = registry::getReg("STAGEDIR");
	my $live_dsstatedir = registry::getReg("LIVE_DSSTATEDIR");
	my $live_stagedir = registry::getReg("LIVE_STAGEDIR");
	my $smb_proxy = registry::getReg("SMB_PROXY");

	eval{

		Log("   Creating $stagedir/etc/samba/smb-adc.conf");
		stage::reg_substitute("$templatedir/smb-adc.conf", "$stagedir/etc/samba/smb.conf");

		Log("   Copying /etc/samba/smb.conf to $live_dsstatedir/backup/smb.conf.org");
		copy ("/etc/samba/smb.conf", "$live_dsstatedir/backup/smb.conf.org");

		Log("   Copying $stagedir/etc/samba/smb.conf to $live_stagedir/etc/samba/smb.conf");
		copy ("$stagedir/etc/samba/smb.conf", "$live_stagedir/etc/samba/smb.conf");

	};

	if ($@) { Log(ERR, $@) }

	`touch $tempdir/stage_samba`;
}

#Configures sysvol
sub stage_sysvol
{
	my $live_local_statedir = registry::getReg("LIVE_LOCALSTATEDIR");
	my $dns_root = registry::getReg("DNSROOT");
	my $administrator_uid = registry::getReg("ADMINISTRATOR_UID");
	my $domain_admins_gid = registry::getReg("DOMAIN_ADMINS_GID");
	my $localdc_fqdn = registry::getReg("DNS Host Name");
	my $pdcmaster_fqdn = registry::getReg("PDCMASTERFQDN");

	eval{

		stage::stage_sysvol();

		Log(" Creating msdfs Link $live_local_statedir/msdfs/");
		`(cd $live_local_statedir/msdfs; ln -sf "msdfs:$localdc_fqdn\\sysvol-msdfs,$pdcmaster_fqdn\\sysvol" $live_local_statedir/msdfs/$dns_root)`;

		Log("   Changing ownership for $live_local_statedir/msdfs to $administrator_uid:$domain_admins_gid");
		`chown -R $administrator_uid:$domain_admins_gid $live_local_statedir/msdfs`;

	};

	if ($@) {  Log(ERR, $@); }

	Log(MESG, "   Sysvol Successfully configured for ADC");

}

#Get the new site if it is required
sub stage_site
{

	Log(MESG, ">>>Configuring Site");

	stage::stage_site();

	Log(MESG, "   Site configuration prepared");
}

sub configure_nldap_settings
{
	nds::configure_nldap_settings();
}

sub configure_zone_object
{

	Log(MESG, ">>>Creating DNS Zone object for Aditional domain controller");

	decide_domain_zones();

	eval{

		my $ip_address = registry::getReg("IPADDRESS");
		my $ldap_temp_adminname = registry::getReg("LDAPTEMPADMINNAME");

		# Set the LDAPADMINNAME & ADM_PASSWD.
		my $nds_adminname = registry::getReg("NDSEXISTINGADMINNAME");
		my $ldap_adminname = $nds_adminname;
		$ldap_adminname =~ s/\.([a-z,A-Z,0-9]*)=/,$1=/g;
		registry::setReg("LDAPADMINNAME", $ldap_adminname);
		$ENV{ADM_PASSWD} = $ENV{NDSEXISTINGADMINPASSWD};

		registry::setReg("LDAPADD_FLAGS", "-h $ip_address -x -D $ldap_temp_adminname -a -w '$ENV{'ADM_PASSWD'}'");

		# Fetch the Master/Designated DNS Server Name for the zone. There will be
		# one forward zone and one or more reverse zones in the domain. In case
		# more than one DNS servers exist in the domain different DNS servers can
		# work as Master/Designated server for different zones. So, here let us
		# use the Master/Designated server of the forward zone as DNS Master.

		# AD Integrated (primary) DNS entries are stored in Active Directory
		# rather than a normal zone file leading to more than one primary AD
		# DNS servers coexist due to AD replication. But Novell DNS does not
		# support more than one Master/Designated DNS server for the zone.

		if (! nds::check_certificate()) {
			nds::get_certificate();
		}

	};

	if ($@) { Log(ERR, $@) }

	dns::configure_zone_object();

	Log(MESG,"   DNS Zone objects are created");
}

sub configure_zone_rr_object
{

	Log(MESG, ">>>Creating the DNS Resource Records object");

	decide_domain_zones();

	eval {

		my $dns_server_ip = registry::getReg("DNS_SERVER_IP");
		my $ldap_adminname = registry::getReg("LDAPADMINNAME");
		my $dns_zone = registry::getReg("DNSZONE");
		my $fwdzone_context = registry::getReg("FWDZONE_CONTEXT");

		# Set the LDAPADMINNAME & ADM_PASSWD.
		my $nds_adminname = registry::getReg("NDSEXISTINGADMINNAME");
		my $ldap_adminname = $nds_adminname;
		$ldap_adminname =~ s/\.([a-z,A-Z,0-9]*)=/,$1=/g;
		registry::setReg("LDAPADMINNAME", $ldap_adminname);
		$ENV{ADM_PASSWD} = $ENV{NDSEXISTINGADMINPASSWD};

		my $soa_value = `ADM_PASSWD='$ENV{ADM_PASSWD}' /opt/novell/xad/share/dcinit/provisionTools.sh soa-query -p $dns_server_ip -a '$ldap_adminname' -c "cn=$dns_zone,$fwdzone_context"`;

		chomp($soa_value);
		registry::setReg("SOAVALUE", $soa_value);
		dns::configure_zone_rr_object();
		dns::configure_delegate_zone_object();
	};

	if ($@) {Log(ERR, $@)}

	Log(MESG, "   DNS Resource Records are created");
}

sub configure_nldap_objects
{

	my $ldapi_url = registry::getReg("LDAPI_URL");
	my $ldap_temp_adminname = registry::getReg("LDAPTEMPADMINNAME");

	registry::setReg("LDAPMODIFY_FLAGS", " -H $ldapi_url -x -D '$ldap_temp_adminname' -w '$ENV{'ADM_PASSWD'}'");

	nds::configure_nldap_objects();
}

sub refresh_nldap
{

	registry::setReg("NDSADMINNAME", registry::getReg("NDSTEMPADMINNAME"));

	nds::refresh_nldap();
}

sub create_site
{
	my $ldapi_url = registry::getReg("LDAPI_URL");
	my $ldap_temp_adminname = registry::getReg("LDAPTEMPADMINNAME");

	registry::setReg("LDAPMODIFY_FLAGS", " -H $ldapi_url -x -D '$ldap_temp_adminname'".
			" -w '$ENV{'ADM_PASSWD'}'");

	nds::create_site();
}

####################################################################
# Partition and replica related functions                          #
####################################################################


#Here we add the replica of the domain partition to the local server
sub add_domain_replica
{

	my $ip_address = registry::getReg("IPADDRESS");
	my $ldap_actual_domainnc = registry::getReg("LDAPACTUALDOMAINNC");
	my $install_machinename = registry::getReg("INSTALLMACHINENAME");
	my $ldap_server_context = registry::getReg("LDAPSERVERCONTEXT");
	my $ldap_admin = registry::getReg("LDAPACTUALADMINNAME");

	Log(MESG, ">>>Adding Domain Replica");

	eval{
		replops::add_replica(h=>$ip_address, p=>389,
				s=>"CN=$install_machinename,$ldap_server_context",
				n=>$ldap_actual_domainnc,
				t=>"writable",
				D=>$ldap_admin,
				w=>"env:ADM_PASSWD");
	};

	if ($@) { Log(ERR, $@) }

	Log(MESG, "   Domain Replica successfully added");

}


#Here we add the replica of the configuration partition to the local server
sub add_config_replica
{

	my $ip_address = registry::getReg("IPADDRESS");
	my $install_machinename = registry::getReg("INSTALLMACHINENAME");
	my $ldap_server_context = registry::getReg("LDAPSERVERCONTEXT");
	my $ldap_admin = registry::getReg("LDAPACTUALADMINNAME");
	my $actual_forenstnc = `/opt/novell/xad/share/dcinit/provisionTools.sh actual-forest-nc`;
	my $config_schema_replica = registry::getReg("CONFIG_SCHEMA_REPLICAS");
	chomp($actual_forenstnc);

	Log(MESG, ">>>Adding Config Replica");

	eval{
		if ($config_schema_replica ne "" and $config_schema_replica eq "TRUE") {
			replops::add_replica(h=>$ip_address, p=>389,
					s=>"CN=$install_machinename,$ldap_server_context",
					n=>"CN=Configuration,$actual_forenstnc",
					t=>"writable",
					D=>$ldap_admin,
					w=>"env:ADM_PASSWD");
		}else {
			Log(MESG, "   Config Replica addition is ignored");
		}
	};

	if ($@) { Log(ERR, $@) }

	Log(MESG, "   Config Replica successfully added");

}

#Here we add the replica of the schema partition to the local server
sub add_schema_replica
{

	my $ip_address = registry::getReg("IPADDRESS");
	my $install_machinename = registry::getReg("INSTALLMACHINENAME");
	my $ldap_server_context = registry::getReg("LDAPSERVERCONTEXT");
	my $ldap_admin = registry::getReg("LDAPACTUALADMINNAME");
	my $actual_forenstnc = `/opt/novell/xad/share/dcinit/provisionTools.sh actual-forest-nc`;
	my $config_schema_replica = registry::getReg("CONFIG_SCHEMA_REPLICAS");
	chomp($actual_forenstnc);

	Log(MESG, ">>>Adding Schema Replica");

	eval{
		if ($config_schema_replica ne "" and $config_schema_replica eq "TRUE") {
			replops::add_replica(h=>$ip_address, p=>389,
					s=>"CN=$install_machinename,$ldap_server_context",
					n=>"CN=Schema,CN=Configuration,$actual_forenstnc",
					t=>"writable",
					D=>$ldap_admin,
					w=>"env:ADM_PASSWD");
		} else {
			Log(MESG, "   Schema Relica addition is ignored");
		}

	};

	if ($@) { Log(ERR, $@) }

	Log(MESG, "   Schema Replica successfully added");

}

####################################################################
# Utility modules 											                      #
####################################################################


sub update_nsswitch
{
	util::update_nsswitch();
}

sub restart_services
{
	util::restart_services();
}

sub update_resolv_conf {

	util::update_resolv_conf();
}

sub update_cron_tab
{
	util::update_cron_tab();
}

sub nldap_enable_requiretls
{
	nds::nldap_enable_requiretls();
}

sub decide_domain_zones
{
	# Calculate the forward and reverse zone contexts.
	eval {
		my $ldap_existing_adminname = registry::getReg("LDAPEXISTINGADMINNAME");
		my $dnslocator_obj = registry::getReg("DNS_LOCATOR_OBJECT");
		my $forest_name = registry::getReg("Forest Root");
		my $actual_forestnc = registry::getReg("ACTUALFORESTNC");
		my $dns_server_context = registry::getReg("DNSSERVER_CONTEXT");
		my $frd_provider = `/opt/novell/xad/sbin/provision -q -q -l 60 --locate-dc '$forest_name'`;
		if ($?) {
			my $error = $? >> 8;
			die("Failed to get the forest root domain provider [$forest_name] [$error]");
		}
		chomp($frd_provider);

		my $default_zone_context = $dns_server_context;
		if ($default_zone_context eq "") {
			#
			# If dns_server_context is not available with us, there is something wrong
			# then, we will go with FRD default container by default as fall back
			#
			die("Could not get DNS Server context.");
		}

		my @dns_zones  = `LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem /opt/novell/eDirectory/bin/ldapsearch -D '$ldap_existing_adminname' -w '$ENV{NDSEXISTINGADMINPASSWD}' -H ldaps://$frd_provider:636 -E 2.16.840.1.113719.1.513.4.5 -b '$dnslocator_obj' -s base dNIPDNSZones -N -LLL 2>/dev/null | sed -e ' /^ / {; H; d; }; /^ /! {; x; s/\\n //; }; ' -e  's/st=/s=/gi' | grep -i "dNIPDNSZones: "`;
		chomp(@dns_zones);
		if(@dns_zones) {
			#Set the forward zone context.
			my $fwd_zone = registry::getReg("DNS_ZONE");
			my ($fwdzone_context)  = grep {/dNIPDNSZones: cn=$fwd_zone/i} @dns_zones;
			$fwdzone_context =~ s/^dNIPDNSZones: cn=$fwd_zone,//gi;
			if($fwdzone_context ne ""){
				registry::setReg("IS_FWDZONE_EXIST", "TRUE");
				registry::setReg("FWDZONE_CONTEXT", $fwdzone_context);
			}
			else {
				die("Could not locate forward zone of the domain");
				registry::setReg("IS_FWDZONE_EXIST", "FALSE");
				registry::setReg("FWDZONE_CONTEXT", "$default_zone_context");
			}

			#Set the reverse zone context.


			my $rev_zone = registry::getReg("IP_ADDRESS_UNDERSCORE_FORM");
			my ($revzone_context)  = grep {/dNIPDNSZones: cn=$rev_zone/i} @dns_zones;
			$revzone_context =~ s/^dNIPDNSZones: cn=$rev_zone,//gi;
			if($revzone_context ne ""){
				registry::setReg("IS_REVZONE_EXIST", "TRUE");
				registry::setReg("REVZONE_CONTEXT", $revzone_context);
			}
			else {
				Log(WARN, "Could not locate reverse zone for the DC");
				registry::setReg("IS_REVZONE_EXIST", "FALSE");
				#
				# Here we need to take the following into consideration,
				# It is mandatory for ADC FWDZONE to exist whereas
				# the REVERSE zone may not. Reverse zone may have
				# to be newly created bcos of ADC being in new subnet
				# Hence, we default to DNS Server context as a
				# place holder for new zone. For e.g., non DNS CDC
				# That will make our logic cleaner that, any new zone
				# created will be with DNS SERVER Context. This will
				# make sure that DNS always has the ZONE with-in its replica.
				#
				registry::setReg("REVZONE_CONTEXT", "$default_zone_context");
			}
		}
		else {
			die("Failed to fetch dNIPDNSZones from DNS_LOCATOR_OBJECT");
		}

		# Set the REVZONE_DNSMASTER for the reverse zone.
		# We set designated dns server of FRD zone as the REVZONE_DNSMASTER.
		# Here setting the FWDZONE_DNSMASTER for the forward zone is not required as the zone is expected to be exist.
		my $frd_zone = $forest_name;
		my $fwd_zone_context = registry::getReg("FWDZONE_CONTEXT");
		$frd_zone =~ s/\./_/g;
		my $frd_designated_dns = `LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem /opt/novell/eDirectory/bin/ldapsearch -D '$ldap_existing_adminname' -w '$ENV{NDSEXISTINGADMINPASSWD}' -H ldaps://$frd_provider:636 -b 'cn=$frd_zone,$fwd_zone_context' -s base dNIPDesignatedServer -N -LLL 2>/dev/null | sed ' /^ / {; H; d; }; /^ /! {; x; s/\\n //; }; ' | grep -i "dNIPDesignatedServer: " | awk -F "dNIPDesignatedServer: " '{print \$2}'`;
		chomp($frd_designated_dns);
		if ($frd_designated_dns eq "") {
			Log(WARN, "Could not fetch the FRD zone designated name server DN, assigning the local machine as the master. After the provisioning please change the name-server entry in the SOA record of this domain's zone");
			registry::setReg("REVZONE_DNSMASTER", registry::getReg("DNSHOSTNAME"));
		}
		else {
			my $revzone_dnsmaster = `LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem /opt/novell/eDirectory/bin/ldapsearch -D '$ldap_existing_adminname' -w '$ENV{NDSEXISTINGADMINPASSWD}' -H ldaps://$frd_provider:636 -b "$frd_designated_dns" -s base dNIPServerDNSNames -LLL | sed ' /^ / {; H; d; }; /^ /! {; x; s/\\n //; }; ' | grep "dNIPServerDNSNames: " | awk -F "dNIPServerDNSNames: " '{print \$2}'`;
			chomp($revzone_dnsmaster);
			if ($revzone_dnsmaster eq "") {
				Log(WARN, "Could not fetch the FRD zone name server dns name, assigning the local machine as the master. After the provisioning please change the name-server entry in the SOA record of this domain's zone");
				registry::setReg("REVZONE_DNSMASTER", registry::getReg("DNSHOSTNAME"));
			}
			else {
				registry::setReg("REVZONE_DNSMASTER", $revzone_dnsmaster);
			}
		}
		Log("The reverse zone dns master is ".registry::getReg("REVZONE_DNSMASTER"));

	};

	if ($@) { Log(ERR, $@) }

}

sub refresh_reg
{

	registry::clearReg();

	#For Sites and Subnets, Need to obtain the site name
	my $domain_name = registry::getReg("Root Domain");
	my $sitename = util::find_sitename($domain_name);
	registry::setReg("Site", $sitename);

	#Do before doing a reg_init
	my $out = `/opt/novell/xad/share/dcinit/provisionTools.sh get-default-container`;
	if ($? == 0) {
		chomp($out);
		registry::setReg("DEFAULT_CONTAINER", $out);
	}
	else {
		registry::setReg("DEFAULT_CONTAINER", "ou=OESSystemObjects");
	}

	if (defined $ENV{XADDOMAINADMINNAME}) {
		my $adminName = $ENV{XADDOMAINADMINNAME};
		$adminName =~ /cn=([^\.]*)/;
		my $DCINIT_CONFIG = "/etc/opt/novell/xad/xad.ini";

		`grep "Administrator SAM Name" $DCINIT_CONFIG`;
		if ($?) {
			open(CONFIG,">>$DCINIT_CONFIG") || croak("could not open $DCINIT_CONFIG\n");
			print CONFIG "Administrator SAM Name = $1\n";
			registry::setReg("Administrator SAM Name", $1);
		}
	}
	my $ncp_dn = "cn=".registry::getReg("DNS Unqualified Host Name").",".
			registry::getReg("DEFAULT_CONTAINER").",".registry::getReg("Mapped Domain NC");
	$ncp_dn =~ s/st=/s=/gi;
	registry::setReg("NCP_DN", $ncp_dn);

	registry::reg_init();

	registry::setReg("LDAPADMINNAME", registry::getReg("Mapped Administrator DN"));

	my $dns_root = registry::getReg("DNSROOT");
	my $ldap_adminname = registry::getReg("LDAPADMINNAME");
	$ldap_adminname =~ s/st=/s=/gi;
	my $provider = registry::getReg("PROVIDER");
	Log ("Excuting /opt/novell/xad/share/dcinit/ndsConfigServerContext.sh --add-dc '$dns_root' '$ldap_adminname' $provider");
	my $nds_server_context = `/opt/novell/xad/share/dcinit/ndsConfigServerContext.sh --add-dc '$dns_root' '$ldap_adminname' $provider`;
	chomp($nds_server_context);
	$nds_server_context =~ s/st=/s=/gi;
	registry::setReg("NDSSERVERCONTEXT", $nds_server_context);

	my $nds_admin_server_context = $nds_server_context;
	my $default_contaier = registry::getReg("DEFAULT_CONTAINER");
	$nds_admin_server_context =~ s/^$default_contaier.//;
	$nds_admin_server_context =~ s/st=/s=/gi;
	registry::setReg("NDSADMINSERVERCONTEXT", $nds_admin_server_context);

	my $ldap_actual_domainnc = $nds_admin_server_context;
	$ldap_actual_domainnc =~ s/\.([a-z,A-Z,0-9]*)=/,$1=/g;
	registry::setReg("LDAPACTUALDOMAINNC", $ldap_actual_domainnc);

	my $ldap_server_context = $nds_server_context;
	$ldap_server_context =~ s/\.([a-z,A-Z,0-9]*)=/,$1=/g;
	registry::setReg("LDAPSERVERCONTEXT", $ldap_server_context);

	registry::setReg("NDSADMINNAME", "CN=".registry::getReg("Administrator SAM Name").".CN=Users.".registry::getReg("NDSADMINSERVERCONTEXT"));
	registry::setReg("LDAPACTUALADMINNAME", "CN=".registry::getReg("Administrator SAM Name").",CN=Users,".registry::getReg("LDAPACTUALDOMAINNC"));

	my $ldap_existing_adminname = registry::getReg("NDSEXISTINGADMINNAME");
	$ldap_existing_adminname =~ s/\.([a-z,A-Z,0-9]*)=/,$1=/g;
	registry::setReg("LDAPEXISTINGADMINNAME", $ldap_existing_adminname);

	registry::setReg("LDAPTEMPADMINNAME", "CN=".registry::getReg("Administrator SAM Name").",CN=Users,".registry::getReg("MAPPEDDOMAINNC"));
	registry::setReg("NDSTEMPADMINNAME", "CN=".registry::getReg("Administrator SAM Name").".CN=Users.".registry::getReg("NDSADMINSERVERCONTEXT"));
	registry::setReg("LDAP_NLDAP_EXOP_OID", "2.16.840.1.113719.1.27.100.9");
	registry::setReg("LDAPCOMMON_FLAGS", " -h $provider -Y GSS-SPNEGO");

	my $actual_forestnc = `/opt/novell/xad/share/dcinit/provisionTools.sh actual-forest-nc`;
	#Store it in registry only if we could find the result
	if ($? == 0) {
		chomp($actual_forestnc);
		registry::setReg("ACTUALFORESTNC", $actual_forestnc);
	}

	eval {


		if (registry::getReg("INSTALL_TYPE") eq "frd") {
			registry::setReg("ADMINISTRATOR_UID", 1049076);
			registry::setReg("DOMAIN_ADMINS_GID", 1049088);
		}
		else {
			#Calculate the values admin UID, GID which depends on trustposixoffset.
			my $trust_posix_offset = registry::getReg("TRUSTPOSIXOFFSET");

			if ($trust_posix_offset ne "") {
				registry::setReg("ADMINISTRATOR_UID", $trust_posix_offset + 500);
				registry::setReg("DOMAIN_ADMINS_GID", $trust_posix_offset + 512);
				registry::setReg("GP_CREATOR_OWNER_GID",
					$trust_posix_offset + 520);
			}
		}

		my $mapped_domainnc = registry::getReg("MAPPEDDOMAINNC");
		my $domain_provider = `/opt/novell/xad/sbin/provision -q -q -l 60 --locate-dc '$dns_root'`;
		if ($?) {
			my $error = $? >> 8;
			die("Failed to get the domain provider for domain name [$dns_root] [$error]");
		}
		chomp($domain_provider);

		my $pdc_server  = `/usr/bin/ldapsearch -H ldaps://$domain_provider:636 -x -D '$ldap_adminname' -w '$ENV{ADM_PASSWD}' -b '$mapped_domainnc' -s base -LLL 2>/dev/null | sed ' /^ / {; H; d; }; /^ /! {; x; s/\\n //; }; ' | grep -i 'fSMORoleOwner: ' | awk -F ": " '{print \$2}' | awk -F "cn=NTDS Settings," '{print \$2}' |tr '[:upper:]' '[:lower:]'`;
		chomp ($pdc_server);
		my $pdc_master = `/usr/bin/ldapsearch -H ldaps://$domain_provider:636 -x -D '$ldap_adminname' -w '$ENV{ADM_PASSWD}' -b '$pdc_server' -s base -LLL 2>/dev/null | sed ' /^ / {; H; d; }; /^ /! {; x; s/\\n //; }; ' | grep -i 'dnsHostName: ' | awk -F ": " '{print \$2}'`;
		chomp($pdc_master);

		registry::setReg("PDCMASTERFQDN", "$pdc_master");

		registry::compute_pre_install_variables();

		registry::setReg("Domain Administrator Principal",
				"'".registry::getReg("SAMACCOUNTNAME_DOMAIN")."'@".registry::getReg("Kerberos Realm"));
	};

	if ($@) { Log(ERR, $@) }

}
