#!/usr/bin/perl -I. -I/opt/novell/xad/lib64/perl -I/opt/novell/xad/lib/perl
################################################################################
# Pre-Post Check module for Domain Services for Windows Provisioning
#
# See the file LICENSE for redistribution information.
#
# Copyright (c) 2001-2009
# Novell, Inc.  All rights reserved.
################################################################################
#
#   NAME: check.pm
#   FACILITY: A pre-post check module which is called during provisioning
#
#################################################################################

$ENV_PATH = "/opt/novell/xad/sbin:/opt/novell/xad/bin:/opt/novell/xad/share/dcinit:/opt/novell/eDirectory/sbin:/opt/novell/eDirectory/bin:";
$ENV{'PATH'} = $ENV_PATH.$ENV{'PATH'};

package check;

use v5.10;# at least for Perl 5.10
no warnings 'experimental::smartmatch'; #Avoid known warnings with given-when

use XAD::registry;
use Logger qw(:DEFAULT);
use Install::frd_install;
use Install::child_install;
use Install::check_util;
use File::Temp;
use helpers::ndsServerHealthCheck;
use helpers::containerCheck;
use wins_common;

my $type = registry::getReg("INSTALL_TYPE");
my $is_replica = registry::getReg("IS_REPLICA");
my $adminDN;
my $Ldapsearch;
my $mapped;
my $sedPattern = '\' /^ / {; H; d; }; /^ /! {; x; s/\n //; }; \'';

sub _init
{
	$ENV{'TEMP_ADM_PASSWD'} = $ENV{'ADM_PASSWD'};
	my $lib = registry::getReg("_LIB");
	$Ldapsearch = "LD_PRELOAD=/opt/novell/xad/$lib/libadmpasswd.so.2 /opt/novell/eDirectory/bin/ldapsearch";
	$mapped = registry::getReg("MAPPED");

	my $pipe_status_file = mktemp("/tmp/pipe_status.XXXXXXXX");
	if(defined($pipe_status_file)) {
			$ENV{'PIPE_STATUS_FILE'} = $pipe_status_file;
	}

	if ($is_replica eq "TRUE") {
		$adminDN = registry::getReg("LDAPADMINNAME");
		if (not defined $ENV{'ADM_PASSWD'}) {
			Log (ERR, "Exiting. Did you forget to export the ADM_PASSWD");
		}
	} elsif ($type eq "frd") {
		if ($mapped) {
			$adminDN = registry::getReg("LDAPDOMAINADMINNAME");
			if (not defined $ENV{'NDSEXISTINGADMINPASSWD'}) {
				Log (ERR, "Exiting. Did you forget to export the NDSEXISTINGADMINPASSWD");
			}
			$ENV{'ADM_PASSWD'} = $ENV{'NDSEXISTINGADMINPASSWD'};
		}else {
			$adminDN = registry::getReg("LDAPADMINNAME");
			if (not defined $ENV{'ADM_PASSWD'}) {
				Log (ERR, "Exiting. Did you forget to export the ADM_PASSWD");
			}
		}
	} elsif ($type eq "child") {
		if ($mapped) {
			$adminDN = registry::getReg("LDAPDOMAINADMINNAME");
			if (not defined $ENV{'NDSEXISTINGADMINPASSWD'}) {
				Log (ERR, "Exiting. Did you forget to export the NDSEXISTINGADMINPASSWD");
			}
			$ENV{'ADM_PASSWD'} = $ENV{'NDSEXISTINGADMINPASSWD'};
		} else {
			$adminDN = registry::getReg("LDAPADMINNAME");
			if (not defined $ENV{'NDSEXISTINGADMINPASSWD'}) {
				Log (ERR, "Exiting. Did you forget to export the NDSEXISTINGADMINPASSWD");
			}
			$ENV{'ADM_PASSWD'} = $ENV{'NDSEXISTINGADMINPASSWD'};
		}
	}

}

my $_LIB = registry::getReg("_Lib");
my $SASL_PATH = "/opt/novell/xad/$_LIB/sasl2";
$ENV{'_LIB'} = $_LIB;
$ENV{'SASL_PATH'} = $SASL_PATH;
my $debug = 0;

my %Keymap = ( 	'STAGING'			=>	1,
		'SCHEMAADD'			=>	2,
		'CONFIGEDIR'			=>	3,
		'PRECHECK'                      =>      4,
		'DNSCONFIGURATION'		=>	5,
		'SLAPICONFIG'			=>	6,
		'DOMAINPARTITIONCREATION'	=>	7,
		'ADDDOMAINREPLICA'		=>	8,
		'DOMAINOBJADDITION'		=>	9,
		'CONFIGPARTITIONCREATION'	=>	10,
		'SCHEMAPARTITIONCREATION'	=>	11,
		'CONFIGOBJECTADDITION'		=>	12,
		'DOMAINJOIN'			=>	13,
		'ACLCONFIGURATION'		=>	14,
		'RESTARTDSFW'			=>	15,
		'SETDEFAULTPASSWORD'		=>	16,
		'ENABLELOCALKRB'		=>	17,
		'SAMIFICATION'			=>	18,
		'MIGRATENKDC'			=>	19,
		'CREATETRUST'			=>	20,
		'CRONTABENTRIES'		=>	21,
		'CLEANUP'			=>	22,
		'SITECONFIGURATION'		=>	23
	);

#***************************************
#PROVISIONING STEPS
#***************************************
#1.  [YaST] STAGING => provision_staging.pl
#2.  [YaST] SCHEMAADD => provision_schema.pl
#3.  [YaST] CONFIGEDIR => provision_config_edir.pl
#4.  [NA] PRECHECK = /opt/novell/xad/share/dcinit/provision/provision_precheck.pl
#5.  DNSCONFIGURATION = /opt/novell/xad/share/dcinit/provision/provision_dns.pl
#6.  SLAPICONFIG = /opt/novell/xad/share/dcinit/provision/provision_config_slapi.pl
#7.  DOMAINOBJADDITION = /opt/novell/xad/share/dcinit/provision/provision_add_domainobj.pl
#8.  CONFIGOBJECTADDITION = /opt/novell/xad/share/dcinit/provision/provision_add_configobj.pl
#9.  DOMAINPARTITIONCREATION = /opt/novell/xad/share/dcinit/provision/provision_partition_domain.pl
#10. CONFIGPARTITIONCREATION = /opt/novell/xad/share/dcinit/provision/provision_partition_configuration.pl
#11. SCHEMAPARTITIONCREATION = /opt/novell/xad/share/dcinit/provision/provision_partition_schema.pl
#12. DOMAINJOIN = /opt/novell/xad/share/dcinit/provision/provision_domain_join.pl
#13. ACLCONFIGURATION = /opt/novell/xad/share/dcinit/provision/provision_config_acl.pl
#14. RESTARTDSFW = /opt/novell/xad/share/dcinit/provision/provision_restart_dsfw.pl
#15. SETDEFAULTPASSWORD = /opt/novell/xad/share/dcinit/provision/provision_set_cred_foraccounts.pl
#16. ENABLELOCALKRB = /opt/novell/xad/share/dcinit/provision/provision_enable_local_krb.pl
#17. CRONTABENTRIES = /opt/novell/xad/share/dcinit/provision/provision_crontab_entry_add.pl
#18. CLEANUP = /opt/novell/xad/share/dcinit/provision/provision_cleanup.pl
#***************************************

sub post_check_staging {
	my ($status, $msg);

	($status, $msg) = check_util::stage_dir();
	return (0, $msg) if (!$status);

	($status, $msg) = check_util::stage_default_config();
	return (0, $msg) if (!$status);

	($status, $msg) = check_util::stage_domainguid();
	return (0, $msg) if (!$status);

	($status, $msg) = check_util::stage_ntp();
	return (0, $msg) if (!$status);

	($status, $msg) = check_util::stage_libldap();
	return (0, $msg) if (!$status);

	($status, $msg) = check_util::stage_rc();
	return (0, $msg) if (!$status);

	($status, $msg) = check_util::stage_linkengine();
	return (0, $msg) if (!$status);

	($status, $msg) = check_util::stage_schema_prepare();
	return (0, $msg) if (!$status);

	($status, $msg) = check_util::stage_schema_aggregate();
	return (0, $msg) if (!$status);

	($status, $msg) = check_util::stage_config();
	return (0, $msg) if (!$status);

	($status, $msg) = check_util::stage_domain();
	return (0, $msg) if (!$status);

	($status, $msg) = check_util::stage_ndsd();
	return (0, $msg) if (!$status);

	($status, $msg) = check_util::stage_krb5();
	return (0, $msg) if (!$status);

	($status, $msg) = check_util::stage_samba();
	return (0, $msg) if (!$status);

	($status, $msg) = check_util::stage_xadsd();
	return (0, $msg) if (!$status);

	($status, $msg) = check_util::stage_to_live();
	return (0, $msg) if (!$status);

	return (1);
}

#load_samspm, extend_nam_schema, extend_schema;
sub post_check_schema_add {
	#YAST does it.
	my ($status, $msg);
	($status, $msg) = check_util::check_schema_extension();
	return (0, $msg) if (!$status);

	return (1);
}

#configure_nmas_methods, configure_nldap_settings;
sub post_check_edir_config {
	#YAST does it.
	my ($status, $msg);

	($status, $msg) = check_util::check_config_edir();
	return (0, $msg) if (!$status);

	return (1);
}

sub post_check_pre_check {
	#Not Applicable
	return (1);
}

sub post_check_WINS() {
	Log(INFO, "Doing WINS post check");

	my $result = wins_common::check_WINS_dat();
	if ($result != 0) {
		Log(INFO, "WINS dat file not present on this server");
		return $result;
	}

	$result = wins_common::verify_WINS_srv();
	if ($result != 0) {
		Log(INFO, "WINS SRV record not found on this server");
		return $result;
	}

	Log(INFO, "WINS configure Post check successful");

	return 0;
}

sub post_check_dns_config {
	#my $searchBase = registry::getReg("DNS_LOCATOR_OBJECT");
	my $searchBase = registry::getReg("FWDZONE_CONTEXT");
	my $dns_server_ip = registry::getReg("DNS_SERVER_IP");
	my $searchFilter = "(objectclass=DNIP:DNS Zone)";
	my $rootDomain = registry::getReg("Root Domain");
	my $isreplica = registry::getReg("IS_REPLICA");
	$rootDomain =~ s/\./_/g;

	# Check Forward Zone update
	if ($isreplica eq "FALSE") {
		my @ldapOutput = `$Ldapsearch -x -h $dns_server_ip -b "$searchBase" -s one "$searchFilter" dn -E "2.16.840.1.113719.1.513.4.5" -LLL | sed -e $sedPattern -e \'s/st=/s=/gi\'; echo \${PIPESTATUS[0]} > $ENV{'PIPE_STATUS_FILE'}`;
		my $ldap_status = `cat $ENV{'PIPE_STATUS_FILE'}`;
		chomp $ldap_status;

		return (0, "Ldapsearch Failed [Inside post_check_dns_config]") if ($ldap_status);

		return (0, "Forward zone entry is missing") if (!grep(/dn: cn=$rootDomain,$searchBase/i, @ldapOutput));
	}

	# Check Reverese Zone update
	my $rev_zone = registry::getReg("IP_ADDRESS_UNDERSCORE_FORM");
	my $ptr_rrset = registry::getReg("REVERS_PTR_IP");
	$ptr_rrset =~ s/\./_/g;
	my $revzone_context = registry::getReg("REVZONE_CONTEXT");
	$nds_admin_server_context =~ s/st=/s=/gi;
	my $frd_provider;
	if($type eq "frd" and $isreplica eq "FALSE") {
		$frd_provider = $dns_server_ip;
	}
	else {
		my $frd_name = registry::getReg("Forest Root");
		$frd_provider = `/usr/bin/dig -t SRV _ldap._tcp.pdc._msdcs.$frd_name  +short | cut -d " " -f4 | sed 's/\.\$//'`;
		if ($? || $frd_provider eq "") {
			my $error = $? >> 8;
			die("Failed to get the domain provider for domain name [$frd_name] [$error]");
		}
		chomp($frd_provider);
	}

	my @ldapOutput = `$Ldapsearch -x -h $frd_provider -b "cn=$rev_zone,$revzone_context" -s one dn -E "2.16.840.1.113719.1.513.4.5" -LLL | sed -e $sedPattern -e \'s/st=/s=/gi\'; echo \${PIPESTATUS[0]} > $ENV{'PIPE_STATUS_FILE'}`;
	my $ldap_status = `cat $ENV{'PIPE_STATUS_FILE'}`;
	chomp $ldap_status;

	return (0, "Ldapsearch Failed [Inside post_check_dns_config]") if ($ldap_status);

	return (0, "Reverse zone entry is missing") if (!grep(/dn: cn=$ptr_rrset,cn=$rev_zone,$revzone_context/i, @ldapOutput));

	if ( lc(registry::getReg("WINS")) eq 'yes') {
		if (post_check_WINS() != 0) {
			return (0, "WINS configuration failed");
		}
	}

	return (1);

}

#configure_nldap_objects, refresh_nldap, validate_loaded_plugin;
sub post_check_slapi_config {
	my $defaultDomainNC = registry::getReg("MAPPEDDOMAINNC");
	my $hostName = registry::getReg("InstallMachineName");
	my $defaultContainer = registry::getReg("DEFAULT_CONTAINER");
        my $searchBase1 = "CN=LDAP Group - $hostName,$defaultContainer,$defaultDomainNC";
        my $searchBase2 = "CN=LDAP Server - $hostName,$defaultContainer,$defaultDomainNC";
        my $searchBase3 = "CN=$hostName,$defaultContainer,$defaultDomainNC";
        my $searchScope = "base";
        my $searchAttribute1 = "ldapClassList";
        my $searchAttribute2 = "ldapAttributeList";
        my $searchAttribute3 = "extensionInfo";
        my $searchAttribute4 = "CachedAttrsOnExtRefs";
        my $searchAttribute5 = "indexDefinition";
        my $ipAddress = registry::getReg("IPADDRESS");

	#Validating /domain/nldap-delete-classlist.ldif
        my @ldapOutput = `$Ldapsearch -x -D '$adminDN' -w '$ENV{'ADM_PASSWD'}' -b '$searchBase1' -s base $searchAttribute1 -E "2.16.840.1.113719.1.513.4.5" -LLL| sed $sedPattern; echo \${PIPESTATUS[0]} > $ENV{'PIPE_STATUS_FILE'}`;
	my $ldap_status = `cat $ENV{'PIPE_STATUS_FILE'}`;
	chomp $ldap_status;
	return (0, "Ldapsearch 1 Failed [Inside post_check_slapi_config]") if ($ldap_status);
	return (0, "nldap delete classlist operation failed") if (!grep(/NDSName=User.*LDAP.*inetOrg.*/i, @ldapOutput) || !grep(/NDSName=Group.*LDAP.*group.*/i, @ldapOutput));

	#Validating domain/nldap.ldif
	@ldapOutput = `$Ldapsearch -x -D '$adminDN' -w '$ENV{'ADM_PASSWD'}' -b '$searchBase1' -s base $searchAttribute1 $searchAttribute2 -E "2.16.840.1.113719.1.513.4.5" -LLL | sed $sedPattern; echo \${PIPESTATUS[0]} > $ENV{'PIPE_STATUS_FILE'}`;
	my $ldap_status = `cat $ENV{'PIPE_STATUS_FILE'}`;
	chomp $ldap_status;
	return (0, "Ldapsearch 2 Failed [Inside post_check_slapi_config]") if ($ldap_status);
	return (0, "nldap add ldapClassList or ldapAttributeList operation failed") if (!grep(/NDSName=User.*LDAPNames=user.*inetOrgPerson/i, @ldapOutput) || !grep(/NDSName=Group.*LDAPNames=group.*groupOfNames.*groupOfUniqueNam/i, @ldapOutput) || !grep(/NDSName=msDS-User-Account-Control/i, @ldapOutput));

	#Validating domain/nldap.ldif
	@ldapOutput = `$Ldapsearch -x -D '$adminDN' -w '$ENV{'ADM_PASSWD'}' -b '$searchBase2' -s base $searchAttribute3 -E "2.16.840.1.113719.1.513.4.5" -LLL | sed $sedPattern; echo \${PIPESTATUS[0]} > $ENV{'PIPE_STATUS_FILE'}`;
	my $ldap_status = `cat $ENV{'PIPE_STATUS_FILE'}`;
	chomp $ldap_status;
	return (0, "Ldapsearch 3 Failed [Inside post_check_slapi_config]") if ($ldap_status);
	return (0, "nldap add extensionInfo operation failed") if (!grep(/0.*object.*nad_object_init.*nad-plugin/i, @ldapOutput) || !grep(/7.*extendedop.*whoami_extop_init.*whoami-plugin/i, @ldapOutput));

	#Validating domain/nldap.ldif
	@ldapOutput = `$Ldapsearch -x -D '$adminDN' -w '$ENV{'ADM_PASSWD'}' -b '$searchBase3' -s base $searchAttribute4 $searchAttribute5 -E "2.16.840.1.113719.1.513.4.5" -LLL | sed $sedPattern; echo \${PIPESTATUS[0]} > $ENV{'PIPE_STATUS_FILE'}`;
	my $ldap_status = `cat $ENV{'PIPE_STATUS_FILE'}`;
	chomp $ldap_status;
	return (0, "Ldapsearch 4 Failed [Inside post_check_slapi_config]") if ($ldap_status);
	return (0, "nldap add CachedAttrsOnExtRefs or indexDefinition operation failed") if (!grep(/fRSPrimaryMember/i, @ldapOutput) || !grep(/0.*nETBIOSName.*nETBIOSName/i, @ldapOutput) || !grep(/0.*userPrincipalName.*userPrincipalName/i, @ldapOutput));

	return (1);

}

sub post_check_domain_obj_add {
	my $defaultDomainNC = registry::getReg("MAPPEDDOMAINNC");
	my $searchScope = "base";
	my $ipAddress = registry::getReg("IPADDRESS");

	#Validating /domain/domaindns-associate.ldif
	my @ldapOutput = `$Ldapsearch -x -b '$defaultDomainNC' -s $searchScope objectclass -E "2.16.840.1.113719.1.513.4.5" -LLL | sed $sedPattern; echo \${PIPESTATUS[0]} > $ENV{'PIPE_STATUS_FILE'}`;
	my $ldap_status = `cat $ENV{'PIPE_STATUS_FILE'}`;
	chomp $ldap_status;
	return (0, "Ldapsearch 1 Failed [Inside post_check_domain_obj_add]") if ($ldap_status);
	return (0, "domain_dns_associate_online operation failed") if (!grep(/domainDNS/, @ldapOutput));

	#Validating /domain/domain-object-modify.ldif
	@ldapOutput = `$Ldapsearch -x -D '$adminDN' -w '$ENV{'ADM_PASSWD'}' -b '$defaultDomainNC' -s $searchScope -E "2.16.840.1.113719.1.513.4.5" -LLL gPLink msDS-AllUsersTrustQuota nextRid | sed $sedPattern; echo \${PIPESTATUS[0]} > $ENV{'PIPE_STATUS_FILE'}`;
	my $ldap_status = `cat $ENV{'PIPE_STATUS_FILE'}`;
	chomp $ldap_status;
	return (0, "Ldapsearch 2 Failed [Inside post_check_domain_obj_add]") if ($ldap_status);
	return (0, "domain_object_modify_online operation failed") if (!grep(/gPLink/, @ldapOutput) || !grep(/msDS-AllUsersTrustQuota/, @ldapOutput) || !grep(/nextRid/, @ldapOutput));

	#Validating /domain/domain.ldif
	@ldapOutput = `$Ldapsearch -x -D '$adminDN' -w '$ENV{'ADM_PASSWD'}' -b "$defaultDomainNC" -s one -E "2.16.840.1.113719.1.513.4.5" -LLL "(|(cn=Builtin)(cn=Users)(cn=Computers))" dn | sed -e $sedPattern -e \'s/st=/s=/gi\'; echo \${PIPESTATUS[0]} > $ENV{'PIPE_STATUS_FILE'}` ;
	my $ldap_status = `cat $ENV{'PIPE_STATUS_FILE'}`;
	chomp $ldap_status;
	return (0, "Ldapsearch 3 Failed [Inside post_check_domain_obj_add]") if ($ldap_status);
	return (0, "domain_online operation failed") if (!grep(/cn=Builtin,$defaultDomainNC/i, @ldapOutput) || !grep(/cn=Users,$defaultDomainNC/i, @ldapOutput) || !grep(/cn=Computers,$defaultDomainNC/i, @ldapOutput));

	#Validating /domain/forest.ldif
	if ($type eq "frd") {
		@ldapOutput = `$Ldapsearch -x -D '$adminDN' -w '$ENV{'ADM_PASSWD'}' -b "cn=Enterprise Admins,cn=Users,$defaultDomainNC" -s base -E "2.16.840.1.113719.1.513.4.5" -LLL | sed $sedPattern; echo \${PIPESTATUS[0]} > $ENV{'PIPE_STATUS_FILE'}`;
		my $ldap_status = `cat $ENV{'PIPE_STATUS_FILE'}`;
		chomp $ldap_status;
		return (0, "Ldapsearch 4 Failed [Inside post_check_domain_obj_add]") if ($ldap_status);
		return (0, "forest_object_online operation failed") if (!grep(/uamPosixGroup/, @ldapOutput) || !grep(/samAccountName.*Enterprise Admins/i, @ldapOutput));
	}

	return (1);
}

#forest_bl_online, domain_bl_online, config_bl_online, schema_bl_online;
sub post_check_config_obj_add {
	my $defaultDomainNC = registry::getReg("MAPPEDDOMAINNC");
	my $forestNC = registry::getReg("ACTUALFORESTNC");
	my $ipAddress = registry::getReg("IPADDRESS");
	my $hostName = registry::getReg("InstallMachineName");
	my $DCName = registry::getReg("Install_DCName");
	my $sitename = registry::getReg("Site");

	#Post check of Configuration Object Add
	if ($type eq "frd") {
		#Validating /config/config.ldif [config_online]
		my @ldapOutput = `$Ldapsearch -x -D '$adminDN' -w '$ENV{'ADM_PASSWD'}' -b "cn=Configuration,$forestNC" -s sub "(|(CN=DisplaySpecifiers)(CN=user-Display))" dn -E "2.16.840.1.113719.1.513.4.5" -LLL | sed $sedPattern; echo \${PIPESTATUS[0]} > $ENV{'PIPE_STATUS_FILE'}`;
		my $ldap_status = `cat $ENV{'PIPE_STATUS_FILE'}`;
		chomp $ldap_status;
		return (0, "Ldapsearch 1 Failed [Inside post_check_config_obj_add]") if ($ldap_status);
		return (0, "config_online operation failed") if (!grep(/cn=DisplaySpecifiers,cn=Configuration.*/i, @ldapOutput) || !grep(/cn=user-Display,cn=409,cn=DisplaySpecifiers,cn=Configuration.*/i, @ldapOutput));

		#Validating /schema/schema.ldif [schema_online]
		@ldapOutput = `$Ldapsearch -x -D '$adminDN' -w '$ENV{'ADM_PASSWD'}' -b "cn=Schema,cn=Configuration,$forestNC" -s sub "(|(cn=Trust-Partner)(cn=User-Account-Control))" dn -E "2.16.840.1.113719.1.513.4.5" -LLL | sed $sedPattern; echo \${PIPESTATUS[0]} > $ENV{'PIPE_STATUS_FILE'}`;
		my $ldap_status = `cat $ENV{'PIPE_STATUS_FILE'}`;
		chomp $ldap_status;
		return (0, "Ldapsearch 2 Failed [Inside post_check_config_obj_add]") if ($ldap_status);
		return (0, "schema_online operation failed") if (!grep(/cn=Trust-Partner,cn=Schema,cn=Configuration.*/i, @ldapOutput) || !grep(/cn=User-Account-Control,cn=Schema,cn=Configuration.*/i, @ldapOutput));

	} elsif ($type eq "child") {
		#Validating config/config-addl.ldif
		@ldapOutput = `$Ldapsearch -x -D '$adminDN' -w '$ENV{'ADM_PASSWD'}' -b "CN=$DCName,CN=Servers,CN=$sitename,CN=Sites,CN=Configuration,$forestNC" -s base -E "2.16.840.1.113719.1.513.4.5" -LLL dnsHostName | sed $sedPattern; echo \${PIPESTATUS[0]} > $ENV{'PIPE_STATUS_FILE'}`;
		my $ldap_status = `cat $ENV{'PIPE_STATUS_FILE'}`;
		chomp $ldap_status;
		return (0, "Ldapsearch 3 Failed [Inside post_check_config_obj_add]") if ($ldap_status);
		return (0, "config_online operation failed") if (!grep(/dnsHostName.*$hostName.*/i, @ldapOutput));
	}
	#Post check of BL Object Add
	my $administratorsDN = "CN=Administrators,CN=Builtin," . registry::getReg("MAPPEDDOMAINNC");
	my $enterpriseadminsDN = "CN=Enterprise Admins,CN=Users," . registry::getReg("MAPPEDDOMAINNC");
	my ($ldapOutput1, $ldapOutput2);

	if ($type eq "frd") {
		#Validating /domain/forest-bl.ldif
		@ldapOutput1 = `$Ldapsearch -x -D '$adminDN' -w '$ENV{'ADM_PASSWD'}' -b "$administratorsDN" -s base -E "2.16.840.1.113719.1.513.4.5" -LLL member | sed -e $sedPattern -e \'s/st=/s=/gi\'; echo \${PIPESTATUS[0]} > $ENV{'PIPE_STATUS_FILE'}`;
		my $ldap_status = `cat $ENV{'PIPE_STATUS_FILE'}`;
		chomp $ldap_status;
		return (0, "Ldapsearch 1 Failed [Inside post_check_config_obj_add]") if ($ldap_status);
		@ldapOutput2 = `$Ldapsearch -x -D '$adminDN' -w '$ENV{'ADM_PASSWD'}' -b "$enterpriseadminsDN" -s base -E "2.16.840.1.113719.1.513.4.5" -LLL member | sed -e $sedPattern -e \'s/st=/s=/gi\'; echo \${PIPESTATUS[0]} > $ENV{'PIPE_STATUS_FILE'}`;
		my $ldap_status = `cat $ENV{'PIPE_STATUS_FILE'}`;
		chomp $ldap_status;
		return (0, "Ldapsearch 2 Failed [Inside post_check_config_obj_add]") if ($ldap_status);
		return (0, "forest_bl_online operation failed") if (!grep(/cn=Administrators,cn=Builtin,$defaultDomainNC/i, @ldapOutput1) || !grep(/cn=Administrator,cn=Users,$defaultDomainNC/i, @ldapOutput2));

		#Validating /config/config-bl.ldif
		@ldapOutput1 = `$Ldapsearch -x -D '$adminDN' -w '$ENV{'ADM_PASSWD'}' -b "CN=Enterprise Schema,CN=Partitions,CN=Configuration,$forestNC" -s base -E "2.16.840.1.113719.1.513.4.5" -LLL dn | sed $sedPattern; echo \${PIPESTATUS[0]} > $ENV{'PIPE_STATUS_FILE'}`;
		my $ldap_status = `cat $ENV{'PIPE_STATUS_FILE'}`;
		chomp $ldap_status;
		return (0, "Ldapsearch 4 Failed [Inside post_check_config_obj_add]") if ($ldap_status);
		return (0, "config_bl_online operation failed") if (!grep(/dn:/, @ldapOutput1));

		#Validating /schema/schema-bl.ldif
		@ldapOutput1 = `$Ldapsearch -x -D '$adminDN' -w '$ENV{'ADM_PASSWD'}' -b "CN=Organization,CN=Schema,CN=Configuration,$forestNC" -s base -E "2.16.840.1.113719.1.513.4.5" -LLL defaultObjectCategory | sed $sedPattern; echo \${PIPESTATUS[0]} > $ENV{'PIPE_STATUS_FILE'}`;
		my $ldap_status = `cat $ENV{'PIPE_STATUS_FILE'}`;
		chomp $ldap_status;
		return (0, "Ldapsearch 5 Failed [Inside post_check_config_obj_add]") if ($ldap_status);
		return (0, "schema_bl_online operation failed") if (!grep(/defaultObjectCategory/, @ldapOutput1));

	} elsif ($type eq "child") {
		#Validating /domain/domain-bl.ldif
		@ldapOutput1 = `$Ldapsearch -x -D '$adminDN' -w '$ENV{'ADM_PASSWD'}' -b "$defaultDomainNC" -s base -E "2.16.840.1.113719.1.513.4.5" -LLL rIDManagerReference fSMORoleOwner | sed $sedPattern; echo \${PIPESTATUS[0]} > $ENV{'PIPE_STATUS_FILE'}`;
		my $ldap_status = `cat $ENV{'PIPE_STATUS_FILE'}`;
		chomp $ldap_status;
		return (0, "Ldapsearch 6 Failed [Inside post_check_config_obj_add]") if ($ldap_status);
		return (0, "domain_bl_online operation failed") if (!grep(/cn=RID Manager.*,cn=System,.*/i, @ldapOutput1) || !grep(/cn=NTDS Settings,cn=$DCName,cn=Servers,cn=$sitename.*/i, @ldapOutput1));

		#Validating config-addl-bl.ldif
		@ldapOutput1 = `$Ldapsearch -x -D '$adminDN' -w '$ENV{'ADM_PASSWD'}' -b "CN=$DCName,CN=Servers,CN=$sitename,CN=Sites,CN=Configuration,$forestNC" -s base -E "2.16.840.1.113719.1.513.4.5" -LLL serverReference | sed $sedPattern; echo \${PIPESTATUS[0]} > $ENV{'PIPE_STATUS_FILE'}`;
		my $ldap_status = `cat $ENV{'PIPE_STATUS_FILE'}`;
		chomp $ldap_status;
		return (0, "Ldapsearch 7 Failed [Inside post_check_config_obj_add]") if ($ldap_status);
		return (0, "config_bl_online operation failed") if (!grep(/cn=$DCName,ou=Domain Controllers,.*/i, @ldapOutput1));

	}

	return (1);
}

sub check_partition_status {
	my ($partitionName, $partitionNC) = @_;
	my $ipAddress = registry::getReg("IPADDRESS");
	my $port = 524;
	my $tmpfile = `/bin/mktemp`;
	my $cnt = 0;
	my $msg = "";
	my $status = 0; # Lets assume the replica state is NOT 'ON'

	chomp($tmpfile);
	$partitionNC =~  s/\,([a-z,A-Z,0-9]*)=/.$1=/g;
	$partitionNC =~  s/^/\./g;
	$partitionNC =~  s/st=/s=/g;

	Log("Executing ... ndsstat -h $ipAddress:$port -p '$partitionNC' | grep \"^\.CN\" | sed 's/^\\.CN=.*\\. //g' | awk '{print \$2}'");

	while ($cnt <= 60) {

		`ndsstat -h $ipAddress:$port -p '$partitionNC' > $tmpfile 2>&1`;

		if ($? || (!(-e $tmpfile))) {
			$msg = "Failed to get state of $partitionName Partition";
			last;
		}
		my @nsOutput = `cat $tmpfile | grep "^\.CN" |  sed 's/^\\.CN=.*\\. //g' | awk '{print \$2}'`;

		foreach $nsout (@nsOutput) {
			chomp($nsout);
			if ($nsout eq "") {
				next;
			}
			if (lc($nsout) ne "on") {
				$status = 0;
				$msg = "State of $partitionName Partition is NOT ON";
				Log(MESGNONEWLN,".");
				last;
			}
			$status = 1;
		}

		if ($status == 1) {
			$msg = "State of $partition partition is ON";
			last;
		} else {
			sleep(1);
			$cnt += 1;
		}
	}

	if (-e $tmpfile) {
		Logger::unset_mask(Logger::LOG_TIMESTAMP);

		open FH, "<$tmpfile";
		while (<FH>) {
			chomp($_);
			Log("$_");
		}
		close(FH);

		Logger::set_mask(Logger::LOG_TIMESTAMP);
		unlink ($tmpfile);
	}

	return ($status, $msg);
}

sub post_check_domain_partition {
	my ($status, $msg);
	my $installmachinename = registry::getReg("INSTALLMACHINENAME");
	my $partitionName = "Domain";
	my $ndsConfigGet = "/opt/novell/xad/share/dcinit/ndsConfigGet.sh";
	my $domainName = registry::getReg("DOMAINNAME");
	my $treeName = `$ndsConfigGet "n4u.base.tree-name"`;
	chomp ($treeName);
	my $domainPartitionNC = $domainName.".T=$treeName.";
	($status, $msg) = &check_partition_status("$partitionName", "$domainPartitionNC");
	$domainPartitionNC =~  s/\,([a-z,A-Z,0-9]*)=/.$1=/g;
	$domainPartitionNC =~  s/^/\./g;
	if ($type eq "frd") {
		my $output = `ndsstat -p '$domainPartitionNC' | grep -i .CN=$installmachinename | awk -F " " '{print \$2}' `;
		chomp($output);
		return (0, "The local server [$installmachinename] is NOT holding the master replica") if ($output ne "Master");
	}
	return ($status, $msg);
}

sub post_check_domain_replica_add {
	my $treeName = `/opt/novell/xad/share/dcinit/ndsConfigGet.sh "n4u.base.tree-name"`;
	my $mappeddomainnc = registry::getReg("MAPPEDDOMAINNC");
	my $installmachinename = registry::getReg("INSTALLMACHINENAME");
	my ($partitionname);

	$mappeddomainnc =~  s/\,([a-z,A-Z,0-9]*)=/.$1=/g;
	$mappeddomainnc =~  s/^/\./g;

	chomp($treeName, $installmachinename);
	$partitionname = $mappeddomainnc.".T=$treeName.";

	print "PN[$partitionname], MDNC[$mappeddomainnc], TN [$treeName] , MN [$installmachinename]\n" if $debug;

	$output = `ndsstat -p '$partitionname' | grep -i .CN=$installmachinename | sed 's/^\\.CN=.*\\. //g' | awk -F " " \'{print \$1}\' `;
	chomp($output);
	print "The output is [$output] \n" if $debug ;

	return (0, "Could not found the server in the partition replica list") if ($?);
	return (0, "The local server [$installmachinename] does not have Read/Write/Master replica") if ((!grep(/Read.Write/, $output)) && (!grep(/Master/, $output)));

	return (1);
}

sub post_check_config_partition {
	my ($status, $msg);
	my $partitionName = "Configuration";
	my $ndsConfigGet = "/opt/novell/xad/share/dcinit/ndsConfigGet.sh";
	my $forestNC = registry::getReg("ACTUALFORESTNC");
	my $confignc = "CN=Configuration,".$forestNC;
	my $treeName = `$ndsConfigGet "n4u.base.tree-name"`;
	chomp ($treeName);
	my $configPartitionNC = $confignc.".T=$treeName.";
	($status, $msg) = &check_partition_status("$partitionName", "$configPartitionNC");

	return ($status, $msg);
}

sub post_check_schema_partition {
	my ($status, $msg);
	my $partitionName = "Schema";
	my $ndsConfigGet = "/opt/novell/xad/share/dcinit/ndsConfigGet.sh";
	my $forestNC = registry::getReg("ACTUALFORESTNC");
	my $schemanc = "CN=Schema,CN=Configuration,".$forestNC;
	my $treeName = `$ndsConfigGet "n4u.base.tree-name"`;
	chomp ($treeName);
	my $schemaPartitionNC = $schemanc.".T=$treeName.";
	($status, $msg) = &check_partition_status("$partitionName", "$schemaPartitionNC");

	return ($status, $msg);
}

sub post_check_domain_join {

	return (1);
}

#compute_effective_acls, import_nds_domain_acl, import_nds_super_rights_acls, import_nds_admin_acls, import_domain_server_reference;
sub post_check_acl_config {
	my $hostName = registry::getReg("InstallMachineName");
	my $DCName = registry::getReg("Install_DCName");
	my $defaultDomainNC = registry::getReg("MAPPEDDOMAINNC");
	my $forestNC = registry::getReg("ACTUALFORESTNC");
	my $ipAddress = registry::getReg("IPADDRESS");
	my $live_dsstatedir = registry::getReg("LIVE_DSSTATEDIR");
	my $defaultContainer = registry::getReg("DEFAULT_CONTAINER");
	my ($ldapOutput1, $ldapOutput2);
	my $sitename  = registry::getReg("Site");

	if ($type eq "frd" and registry::getReg("IS_REPLICA") eq "FALSE") {

		return (0, "$live_dsstatedir/domain/nds-domain-acls.ldif does not exist") if (!(-f "$live_dsstatedir/domain/nds-domain-acls.ldif"));

		#TODO
		#Validating /domain/nds-super-rights-acls.ldif
		#$ldapOutput1 = `$Ldapsearch -x -D '$adminDN' -w '$ENV{'ADM_PASSWD'}' -b "CN=Policies,CN=System,$defaultDomainNC" -s base -LLL ACL | grep ACL: | wc -l`;
		#return (0, "ACL CONFIGURATION FAILED:[import_nds_super_rights_acls operation failed]") if ($ldapOutput1 < 2);

		#Validating /domain/nds-admin-acls.ldif
		@ldapOutput1 = `$Ldapsearch -x -D '$adminDN' -w '$ENV{'ADM_PASSWD'}' -b "$defaultDomainNC" -s base -E "2.16.840.1.113719.1.513.4.5" -LLL ACL | sed $sedPattern; echo \${PIPESTATUS[0]} > $ENV{'PIPE_STATUS_FILE'}`;
		my $ldap_status = `cat $ENV{'PIPE_STATUS_FILE'}`;
		chomp $ldap_status;
		return (0, "Ldapsearch 1 Failed [Inside post_check_acl_config]") if ($ldap_status);
		@ldapOutput2 = `$Ldapsearch -x -D '$adminDN' -w '$ENV{'ADM_PASSWD'}' -b "CN=Schema,CN=Configuration,$forestNC" -s base -E "2.16.840.1.113719.1.513.4.5" -LLL ACL | sed $sedPattern; echo \${PIPESTATUS[0]} > $ENV{'PIPE_STATUS_FILE'}`;
		my $ldap_status = `cat $ENV{'PIPE_STATUS_FILE'}`;
		chomp $ldap_status;
		return (0, "Ldapsearch 2 Failed [Inside post_check_acl_config]") if ($ldap_status);

		if (!grep (/32.subtree.CN=Schema Admins,.*/i, @ldapOutput2)
		   || !grep (/16.*subtree.CN=Schema Admins,.*/i, @ldapOutput2)) {
			return (0, "Missing 'Schema Admins' ACL on Schema Partition");
		}
		#Validating domain/computer-container-policy.ldif
		@ldapOutput1 = `$Ldapsearch -x -D '$adminDN' -w '$ENV{'ADM_PASSWD'}'\\
				-b "CN=Computers,$defaultDomainNC" -s base -E "2.16.840.1.113719.1.513.4.5" -LLL\\
				nspmPasswordPolicyDN | \\
				sed $sedPattern; echo \${PIPESTATUS[0]} \\
				> $ENV{'PIPE_STATUS_FILE'}`;
		my $ldap_status = `cat $ENV{'PIPE_STATUS_FILE'}`;
		chomp $ldap_status;
		return (0, "Ldapsearch 3 Failed [Inside post_check_acl_config]") if ($ldap_status);
		if (!grep(/\s*nspmPasswordPolicyDN:\s*CN=Default/i, @ldapOutput1)) {
			return (0, "Missing 'Default Password Policy' in "
					."nspmPasswordPolicyDN on Computers container");
		}

		#Validating domain/nds-domain-acls.ldif
		@ldapOutput1 = `$Ldapsearch -x -D '$adminDN' -w '$ENV{'ADM_PASSWD'}' -b "CN=Configuration,$forestNC" -s base -E "2.16.840.1.113719.1.513.4.5" -LLL ACL | sed $sedPattern; echo \${PIPESTATUS[0]} > $ENV{'PIPE_STATUS_FILE'}`;
		my $ldap_status = `cat $ENV{'PIPE_STATUS_FILE'}`;
		chomp $ldap_status;
		return (0, "Ldapsearch 4 Failed [Inside post_check_acl_config]") if ($ldap_status);
		if (!grep (/32.*subtree.*CN=Enterprise Admins.*/i, @ldapOutput1)) {
			return (0, "Missing 'Enterprise Admins' ACL on Configuration"
					." Partition");
		}

		if (!$mapped) {
			#Validating domain/nds-domain-lum-acls.ldif
			@ldapOutput1 = `$Ldapsearch -x -D '$adminDN' -w '$ENV{'ADM_PASSWD'}' -b "$defaultDomainNC" -s base -E "2.16.840.1.113719.1.513.4.5" -LLL ACL | sed $sedPattern; echo \${PIPESTATUS[0]} > $ENV{'PIPE_STATUS_FILE'}`;
			my $ldap_status = `cat $ENV{'PIPE_STATUS_FILE'}`;
			chomp $ldap_status;
			return (0, "Ldapsearch 5 Failed [Inside post_check_acl_config]") if ($ldap_status);

			my @parsed_result = grep (/(\d)*.*subtree.*Public.*gidNumber/i, @ldapOutput1);
			$acl_mask = $parsed_result[0];
			$acl_mask =~ s/.subtree.*Public.*gidNumber//i;
			$acl_mask =~ s/ACL:\s*//i;
			$acl_mask = $acl_mask & 2;

			return (0, "import_nds_domain_acl operation failed for lum") if ($acl_mask ne 2);
		}
	}
	#Validating /domain/nds-domain.ldif
	@ldapOutput1 = `$Ldapsearch -x -D '$adminDN' -w '$ENV{'ADM_PASSWD'}' -b "CN=Policies,CN=System,$defaultDomainNC" -s base -E "2.16.840.1.113719.1.513.4.5" -LLL ACL | sed $sedPattern; echo \${PIPESTATUS[0]} > $ENV{'PIPE_STATUS_FILE'}`;
	my $ldap_status = `cat $ENV{'PIPE_STATUS_FILE'}`;
	chomp $ldap_status;
	return (0, "Ldapsearch 6 Failed [Inside post_check_acl_config]") if ($ldap_status);
	if (!grep(/ACL:/i, @ldapOutput1)) {
		return (0, "Missing ACLs on 'CN=Policies' container");
	}

	#Validating /domain/nds-domain-server-reference.ldif
	@ldapOutput1 = `$Ldapsearch -x -D '$adminDN' -w '$ENV{'ADM_PASSWD'}' -b "CN=$hostName,$defaultContainer,$defaultDomainNC" -s base -E "2.16.840.1.113719.1.513.4.5" -LLL serverReference | sed $sedPattern; echo \${PIPESTATUS[0]} > $ENV{'PIPE_STATUS_FILE'}`;
	my $ldap_status = `cat $ENV{'PIPE_STATUS_FILE'}`;
	chomp $ldap_status;
	return (0, "Ldapsearch 7 Failed [Inside post_check_acl_config]") if ($ldap_status);
	if (!grep(/serverReference.*cn=$DCName,cn=Servers,cn=$sitename.*/i,
			@ldapOutput1)) {
		chomp(@ldapOutput1);
		return (0, "Missing server reference on NCP server object: [@ldapOutput1]");
	}

	#Validating /domain/nds-domain-container.ldif
	@ldapOutput2 = `$Ldapsearch -x -D '$adminDN' -w '$ENV{'ADM_PASSWD'}' -b "$defaultContainer,$defaultDomainNC" -s base -E "2.16.840.1.113719.1.513.4.5" -LLL showInAdvancedViewOnly | sed $sedPattern; echo \${PIPESTATUS[0]} > $ENV{'PIPE_STATUS_FILE'}`;
	my $ldap_status = `cat $ENV{'PIPE_STATUS_FILE'}`;
	chomp $ldap_status;
	return (0, "Ldapsearch 8 Failed [Inside post_check_acl_config]") if ($ldap_status);
	if (!grep(/showInAdvancedViewOnly: TRUE/i, @ldapOutput2)) {
		chomp(@ldapOutput2);
		return (0, "Missing update of Default container [$defaultContainer]: "
			   ."[@ldapOutput2]");
	}

	return (1);
}

#update_nsswitch, restart_services
sub post_check_status_dsfw_services {
	my $dns_master = registry::getReg("DNS Master");

	return (0, "Service ndsd is down") if (system("/usr/bin/systemctl status ndsd.service>/dev/null 2>&1"));
	if ($dns_master eq "TRUE") {
#TBD check for novell-named
#		return (0, "Service novell-named is down") if (system("/usr/sbin/rcnovell-named status >/dev/null 2>&1"));
	}

	return (0, "Service nscd is down") if (system("/usr/sbin/rcnscd status >/dev/null 2>&1"));
	return (0, "Service rpcd is down") if (system("/opt/novell/xad/sbin/rcrpcd status >/dev/null 2>&1"));
	return (0, "Service xadkrb5kdc is down") if (system("/opt/novell/xad/sbin/rcxad-krb5kdc status >/dev/null 2>&1"));
	return (0, "Service xadkpasswdd is down") if (system("/opt/novell/xad/sbin/rcxad-kpasswdd status >/dev/null 2>&1"));
	return (0, "Service xadsd is down") if (system("/opt/novell/xad/sbin/rcxadsd status >/dev/null 2>&1"));
#TBD check for rcsmb, rcwinbind, rcsmb
#	return (0, "Service nmb is down") if (system("/usr/sbin/rcnmb status >/dev/null 2>&1"));
#	return (0, "Service winbind is down") if (system("/usr/sbin/rcwinbind status >/dev/null 2>&1"));
#	return (0, "Service smb is down") if (system("/usr/sbin/rcsmb status >/dev/null 2>&1"));

	# checking that init script is enabled for xadsd, xad-krb5kdc, rpcd and xad-krb5kdc
	return (1);
}

#wait_for_samspm, bootstrap_krbtgt, bootstrap_machine, bootstrap_administrator
sub post_check_account_creds {
	my $cout = `ndstrace -c modules | grep 'samspm.*Running' | wc -l`;
	return (0, "samspm not loaded") if ($cout != 1);

	#TODO
	#Not sure how to check bootstrap_krbtgt, bootstrap_machine, bootstrap_administrator

	return (1);
}

#enable_krb5_local_lookup, acquire_domain_ticket, sync_gpo_nmas
sub post_check_local_krb {
	my $krbfile = registry::getReg(LIVE_ETCDIR)."/krb5.conf";
	my $krout = grep (/^\s*#\s*kdc\s*=\s*.*/, `grep -e "^\#" $krbfile`);
	return (0, "In $krbfile \'kdc\' lookup entry is commented") if ($krout != 0);

	($status, $msg) = check_util::check_creds() ;
	return (0, $msg) if (!$status);

	return (1);
}

#
# If AD SAM Keys are missing or samaccountname is set to a random value,
# we log the user details to dsfw-changes.log for the Admin to correct it.
# Post check will always pass though.
#
sub post_check_samification {
	my $live_dsstatedir = registry::getReg("LIVE_DSSTATEDIR");
	my $mapped = registry::getReg("MAPPED");
	my $domain_nc = registry::getReg("Mapped Domain NC");
	my $machine_name = registry::getReg("INSTALLMACHINENAME");

	#Logging the list of existing users that are samified and also those that are not samified
	my $users_log_file = "/var/opt/novell/xad/log/dsfw-changes.log";

	my @non_samified_users;
	my @non_unique_users;
	my $non_samified_users_count = 0;
	my $non_unique_users_count = 0;
	my $ldap_out_file = mktemp("/tmp/tmp.XXXXXXXX");

	my $admin_name = registry::getReg("LDAPADMINNAME");
	my $is_mapped = registry::getReg("MAPPED");
	my $install_type = registry::getReg("INSTALL_TYPE");
	my $cntl1 = Net::LDAP::Control->new(
				   type	 => "2.16.840.1.113719.1.513.4.6",
				   value	=> "Violate domain boundary",
				   critical => 0
				 );

	my $cntl2 = Net::LDAP::Control->new(
				   type	 => "2.16.840.1.113719.1.513.4.8",
				   value	=> "Search Partition Boundary",
				   critical => 0
				 );
	my $crossref_dn = registry::getReg("CrossRef_DN");
	my ($ld, $cn, $samaccountname) = "";

	eval{
		Log(MESG, ">>>verifying if existing user objects are samified");

		if (! (defined $ENV{ADM_PASSWD_DOMAIN} || defined $ENV{ADM_PASSWD})) {
			die("The credentials needed for samification is not supplied.");
		}

		if (defined $ENV{ADM_PASSWD_DOMAIN}) {
			$ENV{ADM_PASSWD} = $ENV{ADM_PASSWD_DOMAIN};
		}

		if ($install_type eq "frd" && $is_mapped eq 0) {
			$ENV{NDSEXISTINGADMINPASSWD} = $ENV{ADM_PASSWD};
		}

		if (!(defined $ENV{NDSEXISTINGADMINPASSWD}) && $is_mapped eq 1) {
			die("The tree administrator credentials is not supplied");
		}

		$ld = import::connectandbind (admin=>$admin_name, pwd=>$ENV{NDSEXISTINGADMINPASSWD});

		my $mesg = $ld->search (base=>$crossref_dn,
				filter=> "(objectclass=*)",
				scope=>'base',
				timelimit=>30,
				attrs=>['domainpartitionlist'],
				deref=>'never');
		$mesg->code && die("LDAP search failed on [$crossref_dn] with error code: [ ".
			$mesg->code ." ]and error message: [ ". $mesg->error ." ]");

		my @partitions = $mesg->entry->get_value('domainpartitionlist');
		if (scalar @partitions == 0) {
			Log("Found no additional partitions in domain for samfication validation");
			push(@partitions, $domain_nc);
		}

		foreach my $samifying_partition (@partitions) {
			my $filter = "(objectclass=user)";
			Log(">>> Examining partition [ $samifying_partition ] for pre-existing users..");
			if ( lc($samifying_partition) eq lc($domain_nc)) {
				# Skip user objects that are created by us in default domain NC.
				$filter = "(&(objectclass=user)(!(|(name=guest)(name=Administrator)".
					"(name=OESCommonProxy_*)(name=krbtgt)(name=$machine_name)".
					"(name=xad_provisioning_user))))";
			}
			my $mesg = $ld->search (base=>$samifying_partition,
					filter=>$filter,
					scope=>'sub',
					timelimit=>30,
					attrs=>['dn', 'cn', 'samaccountname', 'unicodepwd', 'supplementalcredentials'],
					deref=>'never',
					control=>[$cntl1, $cntl2]);
			$mesg->code && die("LDAP search failed on [$samifying_partition] ".
				"with error code: [ ".$mesg->code." ] and error message: [ ". $mesg->error." ]");
			my @entries = $mesg->entries;
			foreach $entry (@entries) {
				my @attrs = $entry->attributes();
				$samaccountname = $entry->get_value(samaccountname);
				$cn = $entry->get_value(cn);
				# Look for accounts without AD Keys
				if(not grep (/unicodepwd::/i, @attrs) or not grep (/supplementalcredentials::/i, @attrs)) {
					$non_samified_users[$non_samified_users_count++] = $entry->dn();
				}
				if (lc($cn) ne lc($samaccountname)) {
					# Look for accounts that does not have a valid samaccountname
					$non_unique_users[$non_unique_users_count++] = $entry->dn();
				}

			}
		}
		$ld->unbind;
	};

	if ($@) {
			Log(WARN, $@);
			Log(ERR, "   Unable to validate samification of objects in domain");
		}

	# Logging of user lists created in previous section
	open(USERLOGFILE,">>$users_log_file") || Log("Could not open log $users_log_file for logging") && return 1;

	# Logging users that does not have unicodepwd or supplementalcredentials
	if($non_samified_users_count > 0) {
		# Reference log file for existing users in provisioning log
		Log("WARNING: There are [$non_samified_users_count] users missing AD SAM Keys and that needs Administrator's attention");
		Log("WARNING: User DNs are listed in $users_log_file");
		print USERLOGFILE "\nList of users missing AD SAM Keys\(missing unicodepwd or supplementalcredentials attribute\):-\n";

		for($i = 0; $i < $non_samified_users_count; $i++) {
			print USERLOGFILE "$non_samified_users[$i]\n";
		}
	}

	print USERLOGFILE "\n";
	#Logging users that are not Samified
	if($non_unique_users_count > 0) {
		#Reference of log file for existing users in provisioning log
		Log("WARNING: There are [$non_unique_users_count] users with invalid sAMAccountName that needs Administrator's attention");
		Log("WARNING: User DNs are listed in $users_log_file");
		print USERLOGFILE "\nList of users that has invalid sAMAccountName:-\n";

		for($i = 0; $i < $non_unique_users_count; $i++) {
			print USERLOGFILE "$non_unique_users[$i]\n";
		}
	}

	close(USERLOGFILE);
	return (1);
}

sub post_check_migrate_nkdc {

	#TODO
	return (1);
}

sub post_check_trust_creation {

	#TODO
	return (1);
}

#update_cron_tab
sub post_check_cron_tab_entry {
	Log (INFO, "Crontab postcheck started..");
	my @output;
	@output = `/usr/bin/crontab -l`;

	return (0, "The crontab entry for gposync not created") if (!grep("/opt/novell/xad/sbin/gposync", @output));

	return (0, "The crontab entry for the krb5kdc refresh not created") if (!grep("/sbin/killproc -HUP /opt/novell/xad/sbin/krb5kdc", @output));

	($status, $msg) = check_util::stage_sysvol();
	return (0, $msg) if (!$status);

	($status, $msg) = check_util::stage_gpo();
	return (0, $msg) if (!$status);

	return (1);
}

#delete_temporary_admin, stage_clean
sub post_check_cleanup {

	#Crontab Entry Postcheck
	&post_check_cron_tab_entry();

	my $ldapconfig = registry::getReg("LDAPCONFIG");
	my $ndstreename = `/opt/novell/xad/share/dcinit/ndsConfigGet.sh "n4u.base.tree-name"`;
	my $live_etcdir = registry::getReg("LIVE_ETCDIR");
	my $nc = registry::getReg("MAPPEDDOMAINNC");
	my ($adminname, $adminpwd) ;

	if ($type eq "frd" && !($mapped)) {
		$adminname = registry::getReg("Mapped Administrator DN");
	} else {
		$adminname = $adminDN;
	}
	$adminname =~ s/\,([a-z,A-Z,0-9]*)=/.$1=/g;

	my $stagedir = registry::getReg("STAGEDIR");
	my $tmpdir = registry::getReg("LIVE_LOCALSTATEDIR")."/tmp";

	chomp($ldapconfig, $ndstreename, $ndstreename, $live_etcdir, $stagedir, $tmpdir);

	printf "$ldapconfig -V --config-file $live_etcdir/nds.conf -t $ndstreename -a '$adminname' -w '$ENV{'ADM_PASSWD'}' \n" if $debug;
	print "stagedir $stagedir, tempdir $tmpdir\n" if $debug;

	@output = `LANG=POSIX $ldapconfig -V --config-file $live_etcdir/nds.conf -t $ndstreename -a '$adminname' -w '$ENV{'ADM_PASSWD'}'`;

	#chomp($output);
	print "The state of Require TLS is [@output] \n" if $debug;
	if (registry::getReg("Require TLS") eq "TRUE") {
		return (0, "The TLS for Simple Bind is not set to \"yes\"") if (!grep(/Require TLS for Simple Binds with Password: yes/i, @output));
	}elsif (registry::getReg("Require TLS") eq "FALSE") {
		return (0, "The TLS for Simple Bind is not set to \"no\"") if (!grep(/Require TLS for Simple Binds with Password: no/i, @output));
	}


	#Check for the temporary admin is deleted, only for nnm frd setup
	if (!$mapped) {
		my @ldapOutput = `$Ldapsearch -w '$ENV{'ADM_PASSWD'}' -b $nc -s one dn -x -E "2.16.840.1.113719.1.513.4.5" -LLL | sed $sedPattern; echo \${PIPESTATUS[0]} > $ENV{'PIPE_STATUS_FILE'}`;
		my $ldap_status = `cat $ENV{'PIPE_STATUS_FILE'}`;
		chomp $ldap_status;
		return (0, "Ldapsearch Failed [Inside post_check_cleanup]") if ($ldap_status);
		return (0, "The temporary user [XAD_PROVISIONING_USER] has not deleted") if (grep(/xad_provisioning_user/, @ldapOutput));
	}

	return (0, "The directory $stagedir is not cleaned up") if (-f $stagedir);
	return (0, "The directory $tmpdir is not cleaned up") if (-f $tmpdir);

	my $ip_address = registry::getReg("IP Address");
	my $dns_master = registry::getReg("DNS Master");
	chomp($ip_address, $dns_master);
	if ($dns_master eq "TRUE") {
		`grep -q $ip_address /etc/resolv.conf`;
		return (0, "The resolv.conf is not properly updated") if ($?);
	}

	return (1);
}

sub post_check_site {
	my ($status, $msg);
	my ($ndstreename, $forestNC, $sitesDN, $serverPartitionNC);
	my $ndsConfigGet = "/opt/novell/xad/share/dcinit/ndsConfigGet.sh";
	my $provisionTools = "/opt/novell/xad/share/dcinit/provisionTools.sh";
	my $partitionName = registry::getReg("InstallMachineName");
	my $ldapadminname = registry::getReg("LDAPADMINNAME");
	my $mapped_domain_nc = registry::getReg("MAPPEDDOMAINNC");
	my $certFile_der = "/etc/opt/novell/certs/SSCert.der";
	my $certFile_pem = "/etc/opt/novell/certs/SSCert.pem";
	my $count =0;
	my $ldap_out_file = mktemp("/tmp/ldap_out.XXXXXXXX");
	my $DCName = registry::getReg("Install_DCName");

	$ndstreename = `$ndsConfigGet "n4u.base.tree-name"`;
	chomp ($ndstreename);

	$forestNC = registry::getReg("ACTUALFORESTNC");
	chomp($forestNC);
	if (!defined($forestNC) or $forestNC eq "") {
		$forestNC = `$provisionTools "actual-forest-nc"`;
		chomp($forestNC);
	}
	$forestNC =~ s/s=/st=/gi;
	Log ("Forest NC [$forestNC]");

	$sitesDN = "CN=Sites,CN=Configuration,".$forestNC;
	Log(INFO, "Checking partition status: forestNC[$forestNC]\n\t\tserverNC[$sitesDN]");

	if (lc(registry::getReg("LAUNCH_CONFIG_WIZARD")) eq "yes") {
		if ($ENV{'LDAPEXISTINGADMINNAME'} ne "") {
			$ldapadminname = $ENV{'LDAPEXISTINGADMINNAME'};
		}
	}

	#Obtain defaultNamingContext
	Log(INFO, "Executing ...LDAPTLS_CACERT=$certFile_pem /opt/novell/eDirectory/bin/ldapsearch -D '$ldapadminname'".
			" -w '$ENV{'ADM_PASSWD'}' -s base -b '$mapped_domain_nc'".
			" -E 2.16.840.1.113719.1.513.4.5 -LLL defaultNamingContext");
	my @error_output = `LDAPTLS_CACERT=$certFile_pem /opt/novell/eDirectory/bin/ldapsearch -D '$ldapadminname' \\
				-w '$ENV{'ADM_PASSWD'}' -s base -b '$mapped_domain_nc' \\
				-E 2.16.840.1.113719.1.513.4.5 -LLL defaultNamingContext > $ldap_out_file`;
	if ($? != 0) {
		chomp(@error_output);
		unlink($ldap_out_file);
		Log(ERR, "LDAP Search failed with error [@error_output]");
	}
        my $default_naming_context = `cat $ldap_out_file \\
					| sed ' /^ / {; H; d; }; /^ /! {; x; s/\\n //; }; '| grep -i "^dn:"`;
        chomp($default_naming_context);
        $default_naming_context =~ s/^dn:\s+//g;

	my $default_container = `/opt/novell/xad/share/dcinit/provisionTools.sh get-default-container`;
        if ($? == 0) {
                chomp($default_container);
        }
        else {
               $default_container="ou=OESSystemObjects";
        }

	#Prepare ncp server object context
        $servernc = "cn=$partitionName,$default_container,$default_naming_context";

	#Obtain msDS server object context
        Log(INFO, "Executing... LDAPTLS_CACERT=$certFile_pem /opt/novell/eDirectory/bin/ldapsearch -D '$ldapadminname'".
			" -w '$ENV{'ADM_PASSWD'}' -s sub -b '$servernc' serverReference -E 2.16.840.1.113719.1.513.4.5 -LLL");
	@error_output = `LDAPTLS_CACERT=$certFile_pem /opt/novell/eDirectory/bin/ldapsearch -D '$ldapadminname' \\
			-w '$ENV{'ADM_PASSWD'}' -s sub -b '$servernc' \\
			serverReference -E "2.16.840.1.113719.1.513.4.5" -LLL > $ldap_out_file`;
	if ($? != 0) {
		chomp(@error_output);
		unlink($ldap_out_file);
		Log(ERR, "LDAP Search failed with error [@error_output]");
	}
        my $serverPartitionName = `cat $ldap_out_file \\
					| sed ' /^ / {; H; d; }; /^ /! {; x; s/\\n //g; }; ' \\
					| grep -i serverreference`;
        $serverPartitionName =~ s/^serverreference:\s+//gi;
        chomp($serverPartitionName);
	$serverPartitionName =~ s/,/\./g;
	$serverPartitionName .= ".T=$ndstreename.";

	($status, $msg) = &check_partition_status("$partitionName", "$serverPartitionName");
	return ($status, $msg) if (!$status);

	do{
		Log(INFO, "Executing... LDAPTLS_CACERT=$certFile_pem /opt/novell/eDirectory/bin/ldapsearch -D '$ldapadminname'".
				" -w '$ENV{'ADM_PASSWD'}' -s sub -b '$sitesDN'".
				" '(cn=$DCName)' dn -E 2.16.840.1.113719.1.513.4.5 -LLL");
		@error_output = `LDAPTLS_CACERT=$certFile_pem /opt/novell/eDirectory/bin/ldapsearch -D '$ldapadminname' \\
				-w '$ENV{'ADM_PASSWD'}' -s sub -b '$sitesDN' \\
				 '(cn=$DCName)' dn -E "2.16.840.1.113719.1.513.4.5" -LLL > $ldap_out_file`;
		if ($? != 0) {
			chomp(@error_output);
			unlink($ldap_out_file);
			Log(ERR, "LDAP Search failed with error [@error_output]");
		}
		$serverPartitionNC = `cat $ldap_out_file \\
					| sed ' /^ / {; H; d; }; /^ /! {; x; s/\\n //g; }; ' \\
					| grep '^dn:'`;
		chomp($serverPartitionNC);
		sleep(5);
		$count = $count + 1;
	}while($serverPartitionNC eq "" and $count < 10);

	if (-f $ldap_out_file) {
		unlink($ldap_out_file);
	}

	return (0, "The AD server object partition is not complete.") if($serverPartitionNC eq "");

	return (1);
}

##################################################
#POST-CHECKS FOR PROVISIONING
##################################################

sub post_check {
	my $step = shift;
	my ($status, $msg);
	my $stage = $Keymap{"$step"};
	_init();
	given("$stage")
		{
			when ("$Keymap{'STAGING'}")			{ ($status, $msg) = &post_check_staging(); }
			when ("$Keymap{'SCHEMAADD'}")			{ ($status, $msg) = &post_check_schema_add(); }
			when ("$Keymap{'CONFIGEDIR'}")			{ ($status, $msg) = &post_check_edir_config(); }
			when ("$Keymap{'PRECHECK'}")			{ ($status, $msg) = &post_check_pre_check(); }
			when ("$Keymap{'DNSCONFIGURATION'}")		{ ($status, $msg) = &post_check_dns_config(); }
			when ("$Keymap{'SLAPICONFIG'}")			{ ($status, $msg) = &post_check_slapi_config(); }
			when ("$Keymap{'DOMAINPARTITIONCREATION'}")	{ ($status, $msg) = &post_check_domain_partition(); }
			when ("$Keymap{'ADDDOMAINREPLICA'}")		{ ($status, $msg) = &post_check_domain_replica_add(); }
			when ("$Keymap{'DOMAINOBJADDITION'}")		{ ($status, $msg) = &post_check_domain_obj_add(); }
			when ("$Keymap{'CONFIGPARTITIONCREATION'}")	{ ($status, $msg) = &post_check_config_partition(); }
			when ("$Keymap{'SCHEMAPARTITIONCREATION'}")	{ ($status, $msg) = &post_check_schema_partition(); }
			when ("$Keymap{'CONFIGOBJECTADDITION'}")		{ ($status, $msg) = &post_check_config_obj_add(); }
			when ("$Keymap{'DOMAINJOIN'}")			{ ($status, $msg) = &post_check_domain_join(); }
			when ("$Keymap{'ACLCONFIGURATION'}")		{ ($status, $msg) = &post_check_acl_config(); }
			when ("$Keymap{'RESTARTDSFW'}")			{ ($status, $msg) = &post_check_status_dsfw_services(); }
			when ("$Keymap{'SETDEFAULTPASSWORD'}")		{ ($status, $msg) = &post_check_account_creds(); }
			when ("$Keymap{'ENABLELOCALKRB'}")		{ ($status, $msg) = &post_check_local_krb(); }
			when ("$Keymap{'SAMIFICATION'}")			{ ($status, $msg) = &post_check_samification(); }
			when ("$Keymap{'MIGRATENKDC'}")			{ ($status, $msg) = &post_check_migrate_nkdc(); }
			when ("$Keymap{'CREATETRUST'}")			{ ($status, $msg) = &post_check_trust_creation(); }
			when ("$Keymap{'CLEANUP'}")			{ ($status, $msg) = &post_check_cleanup(); }
			when ("$Keymap{'SITECONFIGURATION'}")		{ ($status, $msg) = &post_check_site(); }
			default						{ ($status, $msg) = (0, "NO OPTION FOUND"); }
                };
	$ENV{'ADM_PASSWD'} = $ENV{'TEMP_ADM_PASSWD'};
	delete ($ENV{'TEMP_ADM_PASSWD'});

	unlink($ENV{'PIPE_STATUS_FILE'});
	delete ($ENV{'PIPE_STATUS_FILE'});

	return ($status, $msg);

}

##################################################
#PRE-CHECKS FOR PROVISIONING
##################################################
#Remove the Bad Address Cache from local server.
sub badAddressCacheRemove {
	my $nds_etcdir = registry::getReg("LIVE_NDSETCDIR");

	Log ("Removing the Bad Address Cache from the local server");

	# Load the dstrace
	system("ndsstat --config-file $nds_etcdir/conf/nds.conf -c \"load dstrace\">/dev/null 2>&1");
	if ($?) {
		Log ("Failed to load the dstrace");
		return;
	}

	# Remove the bad address cache.
	system("ndsstat --config-file $nds_etcdir/conf/nds.conf -c \"set ndstrace=*u\"");
	Log ("Failed to remove the bad address cache") if ($?);

	# Unload the dstrace
	system("ndsstat --config-file $nds_etcdir/conf/nds.conf -c \"unload dstrace\">/dev/null 2>&1");
	if ($?) {
		Log ("Failed to unload the dstrace");
		return;
	}

	Log ("Removed the Bad Address Cache from the local server Successfully");
}

#Validate Replica Status for Time Synchronization.
sub validateReplica {
	my ($partition, $error, %listOfPartitions);
	($error, %listOfPartitions) = helpers::partitionList();
	if ($error) {
		Log(ERR, "Failed to get list of partitions");
	}
	foreach my $partition (keys %listOfPartitions) {
		chomp($partition);
		my $error = helpers::replicaStatus($adminDN, $ENV{'ADM_PASSWD'}, $listOfPartitions{$partition}, $partition);
		Log(ERR, "Replica status check failed for partition [$partition]. For more detail see Healthcheck log.") if ($error);
	}
}

sub pre_check_staging {
	#YaST does it.
	return (1, "PRE-CHECK OF STAGING PASSED");
}

#load_samspm, extend_nam_schema, extend_schema;
sub pre_check_schema_add {
	#YAST does it.
	my $nds_libdir = registry::getReg("NDSROOT") . '/' . registry::getReg("_LIB");
	my $nds_etcdir = registry::getReg("LIVE_NDSETCDIR");
	my $live_etcdir = registry::getReg("LIVE_ETCDIR");
	my $is_samspm_entry = `cat $nds_etcdir/conf/ndsmodules.conf | grep -ic samspm >/dev/null 2>&1`;

	return (0, "$nds_libdir/nds-modules/libsamspm.so does not exist") if (!(-f "$nds_libdir/nds-modules/libsamspm.so"));

	return (0, "samspm entry exists in $live_etcdir/ndsmodules.conf") if ($is_samspm_entry != 0);
	return (0, "/var/lib/novell-lum/NAM.SCH does not exist") if (!(-f "/var/lib/novell-lum/NAM.SCH"));
	return (0, "$live_etcdir/msds.sch does not exist") if (!(-f "$live_etcdir/msds.sch"));

	return (1);
}

sub pre_check_edir_config {
	#YaST does it.
	return (1);
}

sub pre_check_pre_check {
	#Not Applicable.
	return (1);
}

sub pre_check_dns_config {
	my $live_dsstatedir = registry::getReg("LIVE_DSSTATEDIR");
	my $dnsfile  = "$live_dsstatedir/dns/final-dns.ldif";
	my $zonefile = "$live_dsstatedir/dns/zone-object.ldif";
	my $revzonefile = "$live_dsstatedir/dns/reverse-zone-object.ldif";
	my $zonemodfile = "$live_dsstatedir/dns/DNS-modify.ldif";
	my $revzonemodfile = "$live_dsstatedir/dns/DNS-modify-reverse-zones.ldif";
	return (0, "$zonefile does NOT exist") if (!(-f "$zonefile"));
	return (0, "$revzonefile does NOT exist") if (!(-f "$revzonefile"));
	return (0, "$zonemodfile does NOT exist") if (!(-f "$zonemodfile"));
	return (0, "$revzonemodfile does NOT exist") if (!(-f "$revzonemodfile"));

	return (1);
}

#configure_nldap_objects, refresh_nldap, validate_loaded_plugin;
sub pre_check_slapi_config {
	my $live_dsstatedir = registry::getReg("LIVE_DSSTATEDIR");

	return (0, "$live_dsstatedir/domain/nldap-delete-classlist.ldif does NOT exist") if (!(-f "$live_dsstatedir/domain/nldap-delete-classlist.ldif"));
	return (0, "$live_dsstatedir/domain/nldap.ldif does NOT exist") if (!(-f "$live_dsstatedir/domain/nldap.ldif"));

	return (1);
}

#domaindns_associate_online, domain_object_modify_online, domain_online, forest_objects_online
sub pre_check_domain_obj_add {
	my $live_dsstatedir = registry::getReg("LIVE_DSSTATEDIR");

	return (0, "$live_dsstatedir/domain/domaindns-associate.ldif does not exist") if (!(-f "$live_dsstatedir/domain/domaindns-associate.ldif"));
	return (0, "$live_dsstatedir/domain/domain-object-modify.ldif does not exist") if (!(-f "$live_dsstatedir/domain/domain-object-modify.ldif"));
	return (0, "$live_dsstatedir/domain/domain-object-modify-lockout-policy.ldif does not exist") if (!(-f "$live_dsstatedir/domain/domain-object-modify-lockout-policy.ldif"));
	return (0, "$live_dsstatedir/domain/domain.ldif does not exist") if (!(-f "$live_dsstatedir/domain/domain.ldif"));
	return (0, "$live_dsstatedir/domain/forest.ldif does not exist") if (!(-f "$live_dsstatedir/domain/forest.ldif"));

	return (1);
}

#config_online, schema_online
#forest_bl_online, domain_bl_online, config_bl_online, schema_bl_online;
sub pre_check_config_obj_add {
	my $live_dsstatedir = registry::getReg("LIVE_DSSTATEDIR");

	#Check for Configuration Object specific *.ldif files
	return (0, "$live_dsstatedir/schema/schema.ldif does not exist") if (!(-f "$live_dsstatedir/schema/schema.ldif"));
	if ($type eq "frd") {
		return (0, "$live_dsstatedir/config/config.ldif does not exist") if (!(-f "$live_dsstatedir/config/config.ldif"));
	}
	elsif($type eq "child") {
		return (0, "$live_dsstatedir/config/config-addl.ldif does not exist") if (!(-f "$live_dsstatedir/config/config-addl.ldif"));
	}
	#Check for Backlink Object specific *.ldif files
	return (0, "$live_dsstatedir/domain/forest-bl.ldif deos not exist") if (!(-f "$live_dsstatedir/domain/forest-bl.ldif"));
	return (0, "$live_dsstatedir/domain/domain-bl.ldif deos not exist") if (!(-f "$live_dsstatedir/domain/domain-bl.ldif"));
	if ($type eq "frd") {
		return (0, "$live_dsstatedir/config/config-bl.ldif deos not exist") if (!(-f "$live_dsstatedir/config/config-bl.ldif"));
	}elsif ($type eq "child") {
		return (0, "$live_dsstatedir/config/config-addl-bl.ldif deos not exist") if (!(-f "$live_dsstatedir/config/config-addl-bl.ldif"));
	}
	return (0, "$live_dsstatedir/schema/schema-bl.ldif deos not exist") if (!(-f "$live_dsstatedir/schema/schema-bl.ldif"));

	return (1);
}

sub pre_check_domain_partition {

	#TODO
	#Not sure what should be checked
	my $ipAddress = registry::getReg("IPADDRESS");
	my $port = 524;

	my @nsout = `ndsstat -h $ipAddress:$port -r | grep "^\.CN" | sed 's/^\\.CN=.*\\. //g' | awk '{print \$2}'`;
	chomp (@nsout);
	return (0, "Failed to get state of Partitions") if ($?);
	foreach my $entry (@nsout) {
		return (0, "All Replicas are not in ON state") if ($entry ne "On");
	}

	#Validate Replica Status for Time Synchronization.
	validateReplica();
	#Bad Address Cache reset for NM Installation
	&badAddressCacheRemove() if (!($type eq "frd" && !$mapped));

	return (1);
}

sub pre_check_domain_replica_add {
	my $ipAddress = registry::getReg("IPADDRESS");
	my $port = 524;
	my $Treename =  `/opt/novell/xad/share/dcinit/ndsConfigGet.sh "n4u.base.tree-name"`;
	my $mappedDomNC = lc(`/opt/novell/xad/share/dcinit/printConfigKey.pl "Mapped Domain NC"`);
	chomp ($Treename, $mappedDomNC);
	my $partitionNC = "." . lc ($mappedDomNC) . ".T=$Treename.";
	$partitionNC =~ s/,/\./g;
	# Check if RO replica of domain partition already exists
	my $ncp_dn=`/opt/novell/eDirectory/bin/ndsstat | grep "Server Name" | awk -F": " '{print \$2}'`;
	chomp($ncp_dn);
	`/opt/novell/eDirectory/bin/ndsstat -d -p '$partitionNC' | grep '$ncp_dn'  | awk '{print $2}' | grep -iq "Read-Only"`;
	if(!$?) {
		return (0, "Please remove the 'Read-Only' replica of this domain partition on [$ncp_dn] and retry");
	}
	my @nsout = `ndsstat -h $ipAddress:$port -r | grep "^\.CN" | sed 's/^\\.CN=.*\\. //g' | awk '{print \$2}'`;
	return (0, "Failed to get state of Partitions") if ($?);
	foreach my $entry (@nsout) {
		chomp($entry);
		return (0, "All Replicas are not in ON state") if ($entry ne "On");
	}

	#Validate Replica Status for Time Synchronization.
	validateReplica();

	#Bad Address Cache reset for NM Installation
	&badAddressCacheRemove() if (!($type eq "frd" && !$mapped));

	return (1);
}

sub pre_check_config_partition {

	#TODO
	#Not sure what should be checked
	my $ipAddress = registry::getReg("IPADDRESS");
	my $port = 524;

	my @nsout = `ndsstat -h $ipAddress:$port -r | grep "^\.CN" | sed 's/^\\.CN=.*\\. //g' | awk '{print \$2}'`;
	chomp (@nsout);
	return (0, "Failed to get state of Partitions") if ($?);
	foreach my $entry (@nsout) {
		return (0, "All Replicas are not in ON state") if ($entry ne "On");
	}

	#Validate Replica Status for Time Synchronization.
	validateReplica();

	#Bad Address Cache reset for NM Installation
	&badAddressCacheRemove() if (!($type eq "frd" && !$mapped));

	return (1);
}

sub pre_check_schema_partition {

	#TODO
	#Not sure what should be checked
	my $ipAddress = registry::getReg("IPADDRESS");
	my $port = 524;

	my @nsout = `ndsstat -h $ipAddress:$port -r | grep "^\.CN" | sed 's/^\\.CN=.*\\. //g' | awk '{print \$2}'`;
	chomp (@nsout);
	return (0, "Failed to get state of Partitions") if ($?);
	foreach my $entry (@nsout) {
		return (0, "All Replicas are not in ON state") if ($entry ne "On");
	}

	#Validate Replica Status for Time Synchronization.
	validateReplica();

	#Do Bad Address Cache reset for NM Installation
	&badAddressCacheRemove() if (!($type eq "frd" && !$mapped));

	return (1);
}

sub pre_check_domain_join {

	#Validate Replica Status for Time Synchronization.
	validateReplica();

	return (1);
}

#compute_effective_acls, import_nds_domain_acl, import_nds_super_rights_acls, import_nds_admin_acls, import_domain_server_reference;
sub pre_check_acl_config {
	my $templatedir = registry::getReg("TEMPLATEDIR");
	my $live_dsstatedir = registry::getReg("LIVE_DSSTATEDIR");

	if ($type eq "frd" || $type eq "child") {
		$error = helpers::checkDomainPartitionList(registry::getReg("LDAPADMINNAME"), $ENV{'ADM_PASSWD'});
		if ($error) {
			return (0, "The validation for selected partitions failed.");
		}
	}

	if ($type eq "frd") {
		return (0, "$live_dsstatedir/domain/nds-super-rights-acls.ldif does not exist") if (!(-f "$live_dsstatedir/domain/nds-super-rights-acls.ldif"));
		return (0, "$live_dsstatedir/domain/nds-admin-acls.ldif does not exist") if (!(-f "$live_dsstatedir/domain/nds-admin-acls.ldif"));
		return (0, "$live_dsstatedir/domain/computer-container-policy.ldif does not exist") if (!(-f "$live_dsstatedir/domain/computer-container-policy.ldif"));

	} else {
		return (0, "$templatedir/nds-domain-acls.ini does not exist") if (!(-f "$templatedir/nds-domain-acls.ini"));
		return (0, "$templatedir/nds-domain-lum-acls.ini does not exist") if (!(-f "$templatedir/nds-domain-lum-acls.ini"));
	}

	return (0, "$live_dsstatedir/domain/nds-domain.ldif does not exist") if (!(-f "$live_dsstatedir/domain/nds-domain.ldif"));
	return (0, "$live_dsstatedir/domain/nds-domain-server-reference.ldif does not exist") if (!(-f "$live_dsstatedir/domain/nds-domain-server-reference.ldif"));
	return (0, "$live_dsstatedir/domain/nds-domain-container.ldif does not exist") if (!(-f "$live_dsstatedir/domain/nds-domain-container.ldif"));

	return (1);

}

#update_nsswitch, restart_services
sub pre_check_status_dsfw_services {

	#return (0, "/usr/sbin/rcndsd does not exist") if (!(-f "/usr/sbin/rcndsd"));
	return (0, "/usr/sbin/rcnovell-named does not exist") if (!(-f "/usr/sbin/rcnovell-named"));
	return (0, "/usr/sbin/rcnovell-xregd does not exist") if (!(-f "/usr/sbin/rcnovell-xregd"));
	#TBD check for rcsmb, rcwinbind, rcsmb systemd scripts
	#return (0, "/usr/sbin/rcwinbind does not exist") if (!(-f "/usr/sbin/rcwinbind"));
	#return (0, "/usr/sbin/rcsmb does not exist") if (!(-f "/usr/sbin/rcsmb"));
	#return (0, "/usr/sbin/rcnmb does not exist") if (!(-f "/usr/sbin/rcnmb"));
	return (0, "/usr/sbin/rcxad-krb5kdc does not exist") if (!(-f "/opt/novell/xad/sbin/rcxad-krb5kdc"));
	return (0, "/usr/sbin/rcxad-kpasswdd does not exist") if (!(-f "/opt/novell/xad/sbin/rcxad-kpasswdd"));
	return (0, "/usr/sbin/rcxadsd does not exist") if (!(-f "/opt/novell/xad/sbin/rcxadsd"));

	return (1);
}

#wait_for_samspm, bootstrap_krbtgt, bootstrap_machine, bootstrap_administrator
sub pre_check_account_creds {
	return (0, "Service ndsd is down") if (system("/usr/bin/systemctl status ndsd.service >/dev/null 2>&1"));

	my $XADROOT = registry::getReg("XADROOT");

	return (0, "$XADROOT/sbin/setpassword does not exist") if (!(-f "$XADROOT/sbin/setpassword"));

	return (1);
}

#enable_krb5_local_lookup, acquire_domain_ticket, sync_gpo_nmas
sub pre_check_local_krb {
	return (0, "Service ndsd is down") if (system("/usr/bin/systemctl status ndsd.service >/dev/null 2>&1"));

	my $krbfile = registry::getReg("LIVE_ETCDIR")."/krb5.conf";
	my $GPO2NMAS = registry::getReg("GPO2NMAS");

	return (0, "$krbfile does not exist") if (!(-f "$krbfile"));
	return (0, "$GPO2NMAS does not exist") if (!(-f "$GPO2NMAS"));

	return (1);
}

sub pre_check_samification {

	#TODO
	return (0, "Service ndsd is down") if (system("/usr/bin/systemctl status ndsd.service >/dev/null 2>&1"));

	return (1);
}

sub pre_check_migrate_nkdc {

	#TODO
	return (1);
}

sub pre_check_trust_creation {

	return (0, "Service ndsd is down") if (system("/usr/bin/systemctl status ndsd.service >/dev/null 2>&1"));

	my $parent_name = registry::getReg("Parent Domain");
	my $parent_provider = `/opt/novell/xad/sbin/provision -q -q -l 60 --locate-dc $parent_name`;

	if ($? != 0) {
		return (0, "Could not get the parent domain details. Check the dns server is reachable and your resolv.conf has the correct dns located which can resolve $parent_name.");
	}

	#Validate Replica Status for Time Synchronization.
	validateReplica();

	return (1);
}


#update_cron_tab
sub pre_check_cron_tab_entry {
	Log (INFO, "Crontab precheck started..");
	my $status;
	my @n = `/usr/bin/crontab -l`;
	if (grep (/gposync/, @n) || grep(/krb5kdc/, @n)) {
		Log (WARN, "Entries already exist");
		return (1);
	}

	return (1);
}

#delete_temporary_admin, stage_clean
sub pre_check_cleanup {

	#TODO
	#Not sure what should be checked
#
	&pre_check_cron_tab_entry();
	return (1);
}

sub pre_check_site {
	#TODO
	#Not sure what should be checked
	my $ipAddress = registry::getReg("IPADDRESS");
	my $port = 524;
	my @nsout = `ndsstat -h $ipAddress:$port -r | grep "^\.CN" | sed 's/^\\.CN=.*\\. //g' | awk '{print \$2}'`;
	chomp (@nsout);
	return (0, "Failed to get state of Partitions") if ($?);
	foreach my $entry (@nsout) {
		return (0, "All Replicas are not in ON state") if ($entry ne "On");
	}

	if (lc(registry::getReg("LAUNCH_CONFIG_WIZARD")) eq "yes") {
		if ($ENV{'LDAPEXISTINGADMINNAME'} ne "") {
			$adminDN = $ENV{'LDAPEXISTINGADMINNAME'};
		}
	}

	#Validate Replica Status for Time Synchronization.
	validateReplica();

	#Do Bad Address Cache reset for NM Installation
	&badAddressCacheRemove() if (!($type eq "frd" && !$mapped));

	return (1);
}


sub pre_check {
	my $step = shift;
	my ($status, $msg);
	my $stage = $Keymap{"$step"};
	_init();
	given("$stage") {
			when ("$Keymap{'STAGING'}")			{ ($status, $msg) = &pre_check_staging(); }
			when ("$Keymap{'SCHEMAADD'}")			{ ($status, $msg) = &pre_check_schema_add(); }
			when ("$Keymap{'CONFIGEDIR'}")			{ ($status, $msg) = &pre_check_edir_config(); }
			when ("$Keymap{'PRECHECK'}")			{ ($status, $msg) = &pre_check_pre_check(); }
			when ("$Keymap{'DNSCONFIGURATION'}")		{ ($status, $msg) = &pre_check_dns_config(); }
			when ("$Keymap{'SLAPICONFIG'}")			{ ($status, $msg) = &pre_check_slapi_config(); }
			when ("$Keymap{'DOMAINPARTITIONCREATION'}")	{ ($status, $msg) = &pre_check_domain_partition(); }
			when ("$Keymap{'ADDDOMAINREPLICA'}")		{ ($status, $msg) = &pre_check_domain_replica_add(); }
			when ("$Keymap{'DOMAINOBJADDITION'}")		{ ($status, $msg) = &pre_check_domain_obj_add(); }
			when ("$Keymap{'CONFIGPARTITIONCREATION'}")	{ ($status, $msg) = &pre_check_config_partition(); }
			when ("$Keymap{'SCHEMAPARTITIONCREATION'}")	{ ($status, $msg) = &pre_check_schema_partition(); }
			when ("$Keymap{'CONFIGOBJECTADDITION'}")		{ ($status, $msg) = &pre_check_config_obj_add(); }
			when ("$Keymap{'DOMAINJOIN'}")			{ ($status, $msg) = &pre_check_domain_join(); }
			when ("$Keymap{'ACLCONFIGURATION'}")		{ ($status, $msg) = &pre_check_acl_config(); }
			when ("$Keymap{'RESTARTDSFW'}")			{ ($status, $msg) = &pre_check_status_dsfw_services(); }
			when ("$Keymap{'SETDEFAULTPASSWORD'}")		{ ($status, $msg) = &pre_check_account_creds(); }
			when ("$Keymap{'ENABLELOCALKRB'}")		{ ($status, $msg) = &pre_check_local_krb(); }
			when ("$Keymap{'SAMIFICATION'}")			{ ($status, $msg) = &pre_check_samification(); }
			when ("$Keymap{'MIGRATENKDC'}")			{ ($status, $msg) = &pre_check_migrate_nkdc(); }
			when ("$Keymap{'CREATETRUST'}")			{ ($status, $msg) = &pre_check_trust_creation(); }
			when ("$Keymap{'CLEANUP'}")			{ ($status, $msg) = &pre_check_cleanup(); }
			when ("$Keymap{'SITECONFIGURATION'}")		{ ($status, $msg) = &pre_check_site(); }
			default						{ ($status, $msg) = (0, "NO OPTION FOUND"); }
                };
	$ENV{'ADM_PASSWD'} = $ENV{'TEMP_ADM_PASSWD'};
	delete ($ENV{'TEMP_ADM_PASSWD'});

	unlink($ENV{'PIPE_STATUS_FILE'});
	delete ($ENV{'PIPE_STATUS_FILE'});

	return ($status, $msg);
}

1;
