#!/usr/bin/perl -I. -I/opt/novell/xad/lib64/perl -I/opt/novell/xad/lib/perl

################################################################################
# Pre-Post Check Utility for Domain Services for Windows Provisioning
#
# See the file LICENSE for redistribution information.

# Copyright (c) 2001-2009
# Novell, Inc.  All rights reserved.
################################################################################
#
#   NAME: check_util.pm
#   FACILITY: A utility module which is used by check.pm
#
#################################################################################

package check_util;

#use Test::Simple tests => 48;
use XAD::registry;
use XAD::dns;
use Logger qw(:DEFAULT);
#use File::Find;
use Net::LDAP;
#use Net::LDAP::Schema;
#use Net::LDAP::LDIF;
#use Net::LDAP::Filter;
use util;
use File::Find;
use File::Temp;

my $inst;
my $type;
my $debug = 0;
my $mapped = 0;
my $myLocalhost = "127.0.0.1";

$type = registry::getReg("INSTALL_TYPE");
#FIXME:
#The tree name and the admin name does not come until i do refresh_reg
#So calling it
if ($type eq "frd") {
	$inst = frd_install->new();
} elsif ($type eq "child") {
	$inst = child_install->new();
	}


sub stage_dir {
	print ("\n==>Inside stage_dir");

	my $ndsroot = registry::getReg("NDSROOT");
	my $etcdir  = registry::getReg("ETCDIR");
	my $stagedir = registry::getReg("STAGEDIR");
	my $localstatedir = registry::getReg("LOCALSTATEDIR");
	my $dsstatedir = registry::getReg("DSSTATEDIR");
	my $tempdir = registry::getReg("TEMPDIR");

	return (0, "$stagedir NOT created") if (!(-d $stagedir));
	return (0, "$etcdir NOT created") if (!($etcdir));
	return (0, "$etcdir/openldap NOT created") if (!(-d "$etcdir/openldap"));
	return (0, "$localstatedir NOT created") if (!(-d "$localstatedir"));
	return (0, "$localstatedir/log NOT created") if (!(-d "$localstatedir/log"));
	return (0, "$localstatedir/rpc/PIPE NOT created") if (!(-d "$localstatedir/rpc/PIPE"));
	return (0, "$localstatedir/run NOT created") if (!(-d "$localstatedir/run"));
	return (0, "$localstatedir/run/ntp NOT created") if (!(-d "$localstatedir/run/ntp"));
	return (0, "$stagedir/etc/sysconfig NOT created") if (!(-d "$stagedir/etc/sysconfig"));
	return (0, "$stagedir/etc/sysconfig/novell NOT created") if (!(-d "$stagedir/etc/sysconfig/novell"));
	return (0, "$stagedir/etc$ndsroot/conf NOT created") if (!(-d "$stagedir/etc".$ndsroot."/conf"));
	return (0, "$dsstatedir created") if (!(-d "$dsstatedir"));

	my $var_perm = `ls -ld $dsstatedir | cut -d' ' -f1`;
	return (0, "$dsstatedir permission NOT assigned as 750") if (!($var_perm =~ /drwxr-x---/));

	my $var_grp = `ls -ld $dsstatedir | cut -d' ' -f4`;
	return (0, "$dsstatedir is NOT assigned to named group") if (!($var_grp =~ /named/));

	return (0, "$dsstatedir/config NOT created") if (!(-d "$dsstatedir/config"));
	my $var_perm = `ls -ld $dsstatedir/config | cut -d' ' -f1`;
	return (0, "$dsstatedir/config permission is NOT 700") if (!($var_perm =~ /drwx------/));

	return (0, "$dsstatedir/domain NOT created") if (!(-d "$dsstatedir/domain"));
	my $var_perm = `ls -ld $dsstatedir/domain | cut -d' ' -f1`;
	return (0, "$dsstatedir/domain permission is NOT 700") if (!($var_perm =~ /drwx------/));

	return (0, "$dsstatedir/schema NOT created") if (!(-d "$dsstatedir/schema"));
	my $var_perm = `ls -ld $dsstatedir/schema | cut -d' ' -f1`;
	return (0, "$dsstatedir/schema permission is NOT 700") if (!($var_perm =~ /drwx------/));

	return (0, "$dsstatedir/samify NOT created") if (!(-d "$dsstatedir/samify"));
	my $var_perm = `ls -ld $dsstatedir/samify | cut -d' ' -f1`;
	return (0, "$dsstatedir/samify permission is NOT 700") if (!($var_perm =~ /drwx------/));

	return (0, "$tempdir NOT created") if (!(-d "$tempdir"));
	my $var_perm = `ls -ld $tempdir | cut -d' ' -f1`;
	return (0, "$tempdir permission is NOT 700") if (!($var_perm =~ /drwx------/));

	return (0, "$dsstatedir/linkengine NOT created") if (!(-d "$dsstatedir/linkengine"));
	my $var_perm = `ls -ld $dsstatedir/linkengine | cut -d' ' -f1`;
	return (0, "$dsstatedir/linkengine permission is NOT 700") if (!($var_perm =~ /drwx------/));
	return (1, "stage_dir is complete");
}

sub stage_default_config{
	print ("\n==>Inside stage_default_config");

	my $source = registry::getReg("DCINIT_CONFIG");
	my $dest = registry::getReg("LIVE_ETCDIR");

	return (0, "$dest NOT created") if (!(-d $dest));
	return (0, "$dest/xad.ini is NOT created") if (!(-f "$dest/xad.ini"));
	return (1, "stage_default_config is complete");

}

sub stage_domainguid{
	print ("\n==>Inside stage_domainguid");

 	my $ip = registry::getReg("IPADDRESS");
 	my $mappeddomainnc = registry::getReg("MAPPEDDOMAINNC");
 	my $LIVE_ETCDIR = registry::getReg("LIVE_ETCDIR");

	my $dguid = `egrep '^Domain GUID.*' $LIVE_ETCDIR/xad.ini`;
	return (0, "Grepping the $LIVE_ETCDIR/xad.ini for domainguid is NOT successfull") if (!($? == 0));

	$dguid =~ s/^Domain GUID = //;
	return (0, "domain guid is $dguid") if (!($dguid));

	return (0, "domain guid is NOT in correct format") if (!($dguid =~ /^[a-zA-Z0-9]{8}(-[a-zA-Z0-9]{4}){3}-[a-zA-Z0-9]{12}$/));
	return (1, "stage_domain is complete");

}

sub stage_ntp {
	print ("\n==>Inside stage_ntp");

	my $ntp_conf_file = "/etc/chrony.conf";
	return (0, "$ntp_conf_file NOT exists") if (!(-f "/etc/chrony.conf"));
	`grep -q '\$REGISTRY' /etc/chrony.conf`;
	return (0, "All the REGISTRY values are NOT replaced in /etc/chrony.conf") if (!($? != 0));
	return (1, "stage_ntp is complete");

}

sub stage_libldap {
	print ("\n==>Inside stage_libldap");

	my $ldapconf = registry::getReg("ETCDIR")."/openldap/ldap.conf";
	`grep -q '\$REGISTRY' $ldapconf`;
	return (0, "All the REGISTRY values are NOT replaced in $ldapconf") if (!($? != 0));
	return (1, "stage_libldap is complete");

}

sub stage_rc{
	print ("\n==>Inside stage_rc");

	my ($var_perm, $out);
	my $templatedir = registry::getReg("TEMPLATEDIR");
	my $stagedir = registry::getReg("STAGEDIR");
	my $xadroot = registry::getReg("XADROOT");

	return (0, "$stagedir/etc/init.d NOT created") if (!(-d "$stagedir/etc/init.d"));

	#rpcd
	`grep -q '\$REGISTRY' $stagedir/etc/init.d/rpcd`;
	return (0, "All the REGISTRY values are NOT replaced in $stagedir/etc/init.d/rpcd") if (!($? != 0));

	$var_perm = `ls -ld $stagedir/etc/init.d/rpcd | cut -d' ' -f1`;
	return (0, "$stagedir/etc/init.d/rpcd permission NOT assigned as 755") if (!($var_perm =~ /-rwxr-xr-x/));

	$out = `ls -l $xadroot/sbin/rcrpcd | awk '{print \$NF}'`;
	chomp($out);
	return (0, "$xadroot/sbin/rcrpcd is NOT pointing to /etc/init.d/rpcd") if (!($out eq "/etc/init.d/rpcd"));


	#krb5
	`grep -q '\$REGISTRY' $stagedir/etc/init.d/xad-krb5kdc`;
	return (0, "All the REGISTRY values are NOT replaced in $stagedir/etc/init.d/xad-krb5kdc") if (!($? != 0));

	$var_perm = `ls -ld $stagedir/etc/init.d/xad-krb5kdc | cut -d' ' -f1`;
	return (0, "$stagedir/etc/init.d/xad-krb5kdc permission NOT assigned as 755") if (!($var_perm =~ /-rwxr-xr-x/));

	$out = `ls -l $xadroot/sbin/rcxad-krb5kdc | awk '{print \$NF}'`;
	chomp($out);
	return (0, "$xadroot/sbin/rcxad-krb5kdc is NOT pointing to /etc/init.d/xad-krb5kdc") if (!($out eq "/etc/init.d/xad-krb5kdc"));


	#kpasswd
	`grep -q '\$REGISTRY' $stagedir/etc/init.d/xad-kpasswdd`;
	return (0, "All the REGISTRY values are NOT replaced in $stagedir/etc/init.d/xad-kpasswdd $?") if (!($? != 0));

	$var_perm = `ls -ld $stagedir/etc/init.d/xad-kpasswdd | cut -d' ' -f1`;
	return (0, "$stagedir/etc/init.d/xad-kpasswdd permission NOT assigned as 755") if (!($var_perm =~ /-rwxr-xr-x/));

	$out = `ls -l $xadroot/sbin/rcxad-kpasswdd | awk '{print \$NF}'`;
	chomp($out);
	return (0, "$xadroot/sbin/rcxad-kpasswdd is NOT pointing to /etc/init.d/xad-kpasswdd") if (!( $out eq "/etc/init.d/xad-kpasswdd"));


	#xadsd
	`grep -q '\$REGISTRY' $stagedir/etc/init.d/xadsd`;
	return (0, "All the REGISTRY values are NOT replaced in $stagedir/etc/init.d/xadsd") if (!($? != 0));

	$var_perm = `ls -ld $stagedir/etc/init.d/xadsd | cut -d' ' -f1`;
	return (0, "$stagedir permission NOT assigned as 755") if (!($var_perm =~ /-rwxr-xr-x/));

	$out = `ls -l $xadroot/sbin/rcxadsd | awk '{print \$NF}'`;
	chomp($out);
	return (0, "$xadroot/sbin/rcxadsd is NOT pointing to /etc/init.d/xadsd") if (!( $out eq "/etc/init.d/xadsd"));
	return (1, "stage_rc is complete");

}

sub stage_linkengine {
	print ("\n==>Inside stage_linkengine");

	my $linkengineldif = registry::getReg("DSSTATEDIR")."/linkengine/linkengine.ldif";

	return (0, "$linkengineldif NOT exists") if (!(-f $linkengineldif));

	`grep -q '\$REGISTRY' $linkengineldif`;
	return (0, "All the REGISTRY values are NOT replaced in $linkengineldif") if (!($? != 0));
	return (1, "stage_linkengine is complete");

}

sub stage_schema_prepare {
	print ("\n==>Inside stage_schema_prepare");

	$schemaldif = registry::getReg("DSSTATEDIR")."/schema/schema.ldif";
	$schemablldif = registry::getReg("DSSTATEDIR")."/schema/schema-bl.ldif";

	return (0, "$schemaldif NOT exists") if (!(-f $schemaldif));
	return (0, "$schemablldif NOT exists") if (!(-f $schemablldif));

	`grep -q '\$REGISTRY' $schemaldif`;
	return (0, "All the REGISTRY values are NOT replaced in $schemaldif") if (!($? != 0));

	`grep -q '\$REGISTRY' $schemablldif`;
	return (0, "All the REGISTRY values are NOT replaced in $schemablldif") if (!($? != 0));
	return (1, "stage_schema_prepare is complete");

}

sub stage_schema_aggregate {
	print ("\n==>Inside stage_schema_aggregate");

	$msdsfile = registry::getReg("ETCDIR")."/msds.sch";

	return (0, "$msdsfile NOT exists") if (!(-f $msdsfile));

	`grep -q '\$REGISTRY' $msdsfile`;
	return (0, "All the REGISTRY values are NOT replaced in $msdsfile") if (!($? != 0));

	#Some more test case for the class mapping?
	return (1, "stage_schema_aggregate is complete");

}

sub stage_config{
	print ("\n==>Inside stage_config");

	my $configldif ;
	my $configblldif ;

	if ($type eq "frd") {
		$configldif = registry::getReg("DSSTATEDIR")."/config/config.ldif";
		$configblldif = registry::getReg("DSSTATEDIR")."/config/config-bl.ldif";
	} elsif ($type eq "child") {
		$configldif = registry::getReg("DSSTATEDIR")."/config/config-addl.ldif";
		$configblldif = registry::getReg("DSSTATEDIR")."/config/config-addl-bl.ldif";
	}

	return (0, "$configldif NOT exists") if (!(-f $configldif));
	return (0, "$configblldif NOT exists") if (!(-f $configblldif));

	`grep -q '\$REGISTRY' $configldif`;
	return (0, "All the REGISTRY values are NOT replaced in $configldif") if (!($? != 0));

	`grep -q '\$REGISTRY' $configblldif`;
	return (0, "All the REGISTRY values are NOT replaced in $configblldif") if (!($? != 0));
	return (1, "stage_config is complete");

}

sub stage_domain{
	print ("\n==>Inside stage_domain");

	return (1, "stage_domain is complete");
	#TODO
	#Add test case here;
}

sub stage_ndsd{
	print ("\n==>Inside stage_ndsd");

	my $livendsconf = registry::getReg("LIVE_NDSCONFDIR")."/nds.conf";
	my $ndsconf = registry::getReg("NDSCONFDIR")."/nds.conf";
	my $tempndsconf = registry::getReg("TEMPLATEDIR")."/nds.conf";
	my $liveetcdir = registry::getReg("LIVE_ETCDIR");

	return (0, "$ndsconf NOT exist") if (!(-f $ndsconf));

	`grep -q '\$REGISTRY' $ndsconf`;
	return (0, "All the REGISTRY values are NOT replaced in $ndsconf") if (!($? != 0));

	return (0, "$liveetcdir NOT created") if (!(-d $liveetcdir));

	$out = `ls -l $liveetcdir/nds.conf | awk '{print \$NF}'`;
	chomp($out);
	return (0, "$liveetcdir/nds.conf is NOT pointing to $livendsconf") if (!($out eq $livendsconf));
	return (1, "stage_ndsd is complete");

}

sub stage_krb5 {
	print ("\n==>Inside stage_krb5");

	my $dsstatedir = registry::getReg("DSSTATEDIR");
	my $tempdir = registry::getReg("TEMPLATEDIR");
	my $etcdir = registry::getReg("ETCDIR");
	my $stagedir = registry::getReg("STAGEDIR");
	my $liveetcdir = registry::getReg("LIVE_ETCDIR");
	my $out;

	return (0, "$dsstatedir/krb5kdc NOT created") if (!(-d "$dsstatedir/krb5kdc"));

	$out = `ls -ld $dsstatedir/krb5kdc | cut -d' ' -f1`;
	return (0, "$$dsstatedir/krb5kdc permission NOT assigned as 750") if (!($out =~ /drwxr-x---/));

	$out = `ls -ld $dsstatedir/krb5kdc | cut -d' ' -f4`;
	return (0, "$dsstatedir/krb5kdc is NOT assigned to named group") if (!($out =~ /named/));

	`grep -q '\$REGISTRY' $etcdir/krb5.conf`;
	return (0, "All the REGISTRY values are NOT replaced in $etcdir/krb5.conf") if (!($? != 0));

	`grep -q '\$REGISTRY' $dsstatedir/krb5kdc/kdc.conf`;
	return (0, "All the REGISTRY values are NOT replaced in $dsstatedir/krb5kdc/kdc.conf") if (!($? != 0));

	return (0, "$etcdir/gss NOT created") if (!(-d "$etcdir/gss"));

	$out = `ls -ld $etcdir/gss | cut -d' ' -f1`;
	return (0, "$$dsstatedir/krb5kdc permission NOT assigned as 755") if (!($out =~ /drwxr-xr-x/));

	`grep -q '\$REGISTRY' $etcdir/gss/mech`;
	return (0, "All the REGISTRY values are NOT replaced in $etcdir/gss/mech") if (!($? != 0));

	$out = `ls -l $stagedir/etc/krb5.conf | awk '{print \$NF}'`;
	chomp($out);
	return (0, "$stagedir/etc/krb5.conf is NOT pointing to ../etc/opt/novell/xad/krb5.conf") if (!($out eq "../etc/opt/novell/xad/krb5.conf"));

	return (1, "stage_krb5 is complete");

}

sub stage_samba {
	print ("\n==>Inside stage_samba");

	my $stagedir = registry::getReg("STAGEDIR");
	my $localstatedir = registry::getReg("LOCALSTATEDIR");
	my $templatedir = registry::getReg("TEMPLATEDIR");
	my $live_dsstatedir = registry::getReg("LIVE_DSSTATEDIR");
	my $tempdir = registry::getReg("TEMPDIR");
	my $live_stagedir = registry::getReg("LIVE_STAGEDIR");

	return (0, "$stagedir/etc/samba NOT created") if (!(-d "$stagedir/etc/samba"));
	return (0, "$localstatedir/profiles NOT created") if (!(-d "$localstatedir/profiles"));

	return (0, "All the REGISTRY values are NOT replaced in $stagedir/etc/sysconfig/samba") if (!(util::chkfor_reg("$stagedir/etc/sysconfig/samba") != 0));

	return (0, "$stagedir/etc/samba/MACHINE.SID is NOT created") if (!(-f "$stagedir/etc/samba/MACHINE.SID"));

	return (0, "$stagedir/etc/samba/MACHINE.SID NOT contains the SID") if (!(util::grepfor(registry::getReg("Domain SID"))));

	if ($type eq "frd") {
		return (0, "All the REGISTRY values are NOT replaced in $stagedir/etc/samba/smb.conf") if (util::chkfor_reg("$stagedir/etc/samba/smb.conf") ==0);
	}

	return (0, "$live_dsstatedir/backup NOT created") if (!(-d "$live_dsstatedir/backup"));
	return (0, "$live_dsstatedir/backup/smb.conf.org NOT exists") if (!(-f "$live_dsstatedir/backup/smb.conf.org"));
	return (0, "$live_stagedir/etc/samba/smb.conf NOT exists") if (!(-f "$live_stagedir/etc/samba/smb.conf"));

	return (1, "stage_samba is complete");
}

sub stage_sysvol {
	print ("\n==>Inside stage_sysvol");

	my $LIVELOCALSTATEDIR = registry::getReg("LIVE_LOCALSTATEDIR");
	my $DNSROOT = registry::getReg("DNSROOT");
	my $ADMINISTRATOR_UID = registry::getReg("ADMINISTRATOR_UID");
	my $DOMAIN_ADMINS_GID = registry::getReg("DOMAIN_ADMINS_GID");


	return (0, "$LIVELOCALSTATEDIR/sysvol NOT created") if (!(-d "$LIVELOCALSTATEDIR/sysvol"));
	return (0, "$LIVELOCALSTATEDIR/sysvol/domain NOT created") if (!(-d "$LIVELOCALSTATEDIR/sysvol/domain"));
	return (0, "$LIVELOCALSTATEDIR/sysvol/domain/Policies NOT created") if (!(-d "$LIVELOCALSTATEDIR/sysvol/domain/Policies"));
	return (0, "$LIVELOCALSTATEDIR/sysvol/domain/scripts NOT created") if (!(-d "$LIVELOCALSTATEDIR/sysvol/domain/scripts"));
	return (0, "$LIVELOCALSTATEDIR/sysvol/domain/scripts/Default User NOT created") if (!(-d "$LIVELOCALSTATEDIR/sysvol/domain/scripts/Default User"));
	return (0, "$LIVELOCALSTATEDIR/sysvol/sysvol NOT created") if (!(-d "$LIVELOCALSTATEDIR/sysvol/sysvol"));

	return (0, "$LIVELOCALSTATEDIR/sysvol/staging NOT created") if (!(-d "$LIVELOCALSTATEDIR/sysvol/staging"));
	return (0, "$LIVELOCALSTATEDIR/sysvol/staging areas NOT created") if (!(-d "$LIVELOCALSTATEDIR/sysvol/staging areas"));

	return (0, "$LIVELOCALSTATEDIR/sysvol is NOT assigned right ownership ($ADMINISTRATOR_UID)") if (!(util::get_owner("$LIVELOCALSTATEDIR/sysvol") =~ /$ADMINISTRATOR_UID/));
	return (0, "$LIVELOCALSTATEDIR/sysvol is NOT assigned right group ($DOMAIN_ADMINS_GID))") if (!(util::get_grp("$LIVELOCALSTATEDIR/sysvol") =~ /$DOMAIN_ADMINS_GID/));
	return (1, "stage_sysvol is complete");

}

sub stage_gpo {

	my $live_domaingpodir = registry::getReg("LIVE_DOMAINGPODIR");
	my $gpodir = registry::getReg("GPODIR");
	my $tempdir = registry::getReg("TEMPDIR");
	my $ADMINISTRATOR_UID = registry::getReg("ADMINISTRATOR_UID");
	my $DOMAIN_ADMINS_GID = registry::getReg("DOMAIN_ADMINS_GID");


	return (0, "$live_domaingpodir is NOT created") if (!(-d $live_domaingpodir));
	return (0, "All the REGISTRY values are NOT replaced in $live_domaingpodir/gpt.ini") if (!(util::chkfor_reg("$live_domaingpodir/gpt.ini") != 0));
	return (0, "$live_domaingpodir/MACHINE NOT created") if (!(-d "$live_domaingpodir/MACHINE"));
	return (0, "$live_domaingpodir/MACHINE/Microsoft/Windows NT/SecEdit NOT created") if (!(-d "$live_domaingpodir/MACHINE/Microsoft/Windows NT/SecEdit"));
	return (0, "$live_domaingpodir/USER NOT created") if (!(-d "$live_domaingpodir/USER"));
	return (0, "$live_domaingpodir is NOT assigned right ownership ($ADMINISTRATOR_UID)") if (!(util::get_owner("$live_domaingpodir") =~ /$ADMINISTRATOR_UID/));
	return (0, "$live_domaingpodir is NOT assigned right group ($DOMAIN_ADMINS_GID))") if (!(util::get_grp("$live_domaingpodir") =~ /$DOMAIN_ADMINS_GID/));
	return (1, "stage_gpo is complete");

}

sub stage_xadsd {
	print ("\n==>Inside stage_xadsd");

	my $etcdir = registry::getReg("ETCDIR");
	my $stagedir = registry::getReg("STAGEDIR");

	return (0, "All the REGISTRY values are NOT replaced in $etcdir/xadss.conf") if (util::chkfor_reg("$etcdir/xadss.conf") == 0);
	return (0, "All the REGISTRY values are NOT replaced in $stagedir/etc/sysconfig/novell/xadsd") if (util::chkfor_reg("$stagedir/etc/sysconfig/novell/xadsd") == 0);
	return (1, "stage_xadsd is complete");

}

sub stage_to_live{
	print ("\n==>Inside stage_to_live");

	#Read the content before we call the operation
	my $flist;
	my $stagedir = registry::getReg("STAGEDIR");

	sub wanted{
		if ($_ eq "." or $_ eq "..") { return; }
		my $name = $File::Find::name;
		#Avoid all tmp dirs which are created as part of touch command
		return if($name =~ /^$stagedir\/var\/opt\/novell\/xad\/tmp/);
		my @statlist = lstat($File::Find::name);
		$name =~ s/^$stagedir//;
               			#permission    #uid            #gid
		$flist{$name} = [$statlist[2], $statlist[4], $statlist[5] ];

	}


	find(\&wanted, $stagedir);

	foreach (keys %flist) {
		print "$_\n";
		my @statlist = lstat($_);
		return (0, "$_ copied from stage to live NOT successful") if (!(-e $_));
		return (0, "$_ file's permissions and ownership are NOT preserved while moving to live directory") if (!(@statlist[2] == $flist{$_}[0] && @statlist[4] == $flist{$_}[1] && @statlist[5] == $flist{$_}[2]));
	}

}

sub check_creds  {
	my $dcName = registry::getReg("Install_DCName");
	my $rootDomain = registry::getReg("Root Domain");
	my $tmp_file = mktemp("/tmp/tmp.XXXXXXXXXX");
	my $count = 0;
	my $rc = 0;
	$ENV{KRB5CCNAME} = $tmp_file;

	do {
		
		@output=`/opt/novell/xad/bin/kinit -k -t /var/opt/novell/xad/ds/krb5kdc/krb5.keytab $dcName\$ 2>&1`;
		$rc=$? >> 8;
		chomp($output [0]);
		if ($rc != 0){
			Log ("Kinit Failed: $output[0]. Error code:-$rc");
			sleep (5);
		}
		$count++;
	}
	while ($? != 0 and $count < 5);
	return (0, "Failed to obtain ticket for $dcName") if ($?);

	@output = `/opt/novell/xad/bin/klist`;
	chomp($output [1]);
	return (0, "Could not find the Default principal for $dcName\$") if (!grep(/Default principal:(\s)$dcName\$(\S*)/i, $output[1]));

	`/opt/novell/xad/bin/kdestroy`;

	return (1);
}

sub check_schema_extension{
	print ("\n==>Inside check_schema_extension");

	my ($status, $msg);
	($status, $msg) = check_samspm();
	($status, $msg) = check_nam_schema();
	($status, $msg) = check_msds_schema();
	return ($status, $msg);

}

sub check_samspm{
	print ("\n==>Inside check_samspm");

	my $ndsstat = registry::getReg("NDSSTAT");
	my $live_etcdir = registry::getReg("LIVE_ETCDIR");

	my $out = `$ndsstat --config-file $live_etcdir/nds.conf -c "load samspm"`;

	return (0, "samspm NOT loaded") if ($out!~/Module samspm already loaded/);
}

sub check_nam_schema{
	print ("\n==>Inside check_nam_schema");

	my $file = "/var/lib/novell-lum/NAM.SCH";

	#Load schema from the eDirectory
	#It works for me. Is there a way where local eDirectory will not have schema?
	my $ldap = Net::LDAP->new("$myLocalhost");

	#Anonymous bind. Works fine for me to fetch the required schema.
	#Mostly schema is public readable.
	$ldap->bind();

	#fetch the schema list.
	$schema = $ldap->schema();

	#LDAP::LDIf is not reading the NAM.ldif properly.
	#ldif->readentry pops out an error. Hence reading the .sch file line by line.

	if ( ! -f $file){
		return(0, "Could not find the $file file. Something seriously wrong here.");
	}

	open (FILE, $file) or return (0, "Could not open NAM.SCH");

	while (<FILE>) {
		if ($_ =~ /^--attributeTypes:.*NAME '(.*)' DESC/) {
			$attr_href = $schema->attribute($1);
			return (0, "Attribute $1 does not exist") if !$attr_href;
		}
		elsif ($_ =~ /^--objectClasses:.*NAME '(.*)' DES/) {
			$obj_href = $schema->objectclass($1);
			return (0, "ObjectClass $1 does not exist") if !$obj_href;
		}
		else {
			next;
		}
	}
}

sub check_msds_schema{
	print ("\n==>Inside check_msds_schema");
	my $file = registry::getReg("LIVE_ETCDIR")."/msds.sch";

	#It works for me. Is there a way where local eDirectory will not have schema?
	my $ldap = Net::LDAP->new("$myLocalhost");

	#Anonymous bind. Works fine for me to fetch the required schema.
	#Mostly schema is public readable.
	$ldap->bind();

	#fetch the schema list.
	$schema = $ldap->schema();

	if ( ! -f $file){
		return (0, "Could not find the $file file. Something seriously wrong here.");
	}

	open (FILE, $file) or return (0, "Could not open msds.sch");

	while (<FILE>) {
		if ($_ =~ /^"(.+)" ATTRIBUTE ::=/) {
			#The semicolons and spaces are not represented by the ds.
			#ldapsearch -b cn=schema -s base -x | grep msdsComputer. There is no msds:computer
			$name = $1;
			$name =~ s/[: ]//g;
			$attr_href = $schema->attribute(lc($name));
			return (0, "Attribute $name does not exist") if !$attr_href;
		}
		if ($_ =~ /^"(.+)" OBJECT-CLASS ::=/) {
			$name = $1;
			$name =~ s/[: ]//g;
			#FIXME: There is a schema mapping happenening for User and interorgperson which seems to fail the test case.
			$obj_href = $schema->objectclass(lc($name));
			return (0, "ObjectClass $name does not exist") if !$obj_href;
		}

	}

}

sub check_config_edir{
	print ("\n==>Inside check_config_edir");

	my $out = `ldapsearch -b" " -x -s base | grep -i sasl`;

	return (0, "MSKRB5 method is NOT loaded") if ($out!~/GSS_MSKRB/);
	return (0, "SPNEGO methos is NOT loaded") if ($out!~/GSS-SPNEGO/);
	return (0, "Guess IPC External method is NOT loaded") if ($out!~/NMAS_LOGIN/);

	{
	#FIXME:
	#The tree name and the admin name does not come until i do refresh_reg
	#So calling it
		if ($type eq "frd") {
		my $inst = frd_install->new();
		}
	}

	my $ldapconfig = registry::getReg("LDAPCONFIG");
	my $live_etcdir = registry::getReg("LIVE_ETCDIR");
	my $ndstreename = registry::getReg("NDSTREENAME");
	my $ndsadminname = registry::getReg("NDSADMINNAME");
	my $ldap_url = registry::getReg("LDAPI_URL");


	if (not defined $ENV{ADM_PASSWD}) {
		return(0, "Existing. Did you forgot to export your ADM_PASSWD variable?");
	}

	$out = `$ldapconfig get --config-file $live_etcdir/nds.conf -t $ndstreename -a $ndsadminname -w $ENV{'ADM_PASSWD'} | grep -ic "Require TLS for Simple Binds with Password: no"`;
	return (0, "Require TLS...option is NOT set to no") if ($out!~/Require TLS for Simple Binds with Password: no/);
	return (0, "ldapinterface is NOT set to proper") if ($out!~/ldapInterfaces: ldap:\/\/:389 ldaps:\/\/:636 $ldap_url cldap:\/\/ ldap:\/\/:3268 ldaps:\/\/:3269/);

	return (1, "nds configuration is proper");
}

sub check_domain_replica_add {
	print ("\n==>Inside check_domain_replica_add");

	my $treeName = `/opt/novell/xad/share/dcinit/ndsConfigGet.sh "n4u.base.tree-name"`;
	my $mappeddomainnc=registry::getReg("MAPPEDDOMAINNC");
	my $installmachinename = registry::getReg("INSTALLMACHINENAME");
	my ($partitionname) ;

	$mappeddomainnc =~  s/\,([a-z,A-Z,0-9]*)=/.$1=/g;
	$mappeddomainnc =~  s/^/\./g;

	chomp($treeName, $installmachinename);
	$partitionname = $mappeddomainnc.".T=$treeName.";

	print "PN[$partitionname], MDNC[$mappeddomainnc], TN [$treeName] , MN [$installmachinename]\n" if $debug;

	my $output = `/opt/novell/eDirectory/bin/ndsstat -p $partitionname | grep -i .CN=$installmachinename | awk -F " " \'{print \$2}\' `;
	chomp($output);
	print "The output is [$output] \n" if $debug ;

	return (0, "Server in the partition replica list NOT found") if ($?);
	return (0, "The local server [$installmachinename] does NOT have the master replica") if ($output ne "Master");

	return (1, "Domain replica addition is successful");
}

1;
