#!/usr/bin/perl -I. -I.. -I/opt/novell/xad/lib64/perl -I/opt/novell/xad/lib/perl

################################################################################
# Domain Services for Windows Provisioning Tool
#
# See the file LICENSE for redistribution information.
#
# Copyright (c) 2001-2009
#	Novell, Inc.  All rights reserved.
################################################################################
#
#   NAME:
#         child_install.pm
#
#   FACILITY:
#         Child Domain Installation
#
#   ABSTRACT:
#
#   REVISION: $Id$
#
#################################################################################

package child_install;

use NDS::stage;
use NDS::replops;
use NDS::import;
use XAD::registry;
use XAD::dns;
use XAD::secure;
use Locale::gettext;
use File::Copy;
use File::Path qw(mkpath);
use Logger qw(:DEFAULT);
use util;
use XAD::nds;
use File::Glob ':glob';

our @ISA = qw(stage dns nds replops import secure);
#use base qw(NDS::stage);

my $self = {};

sub new
{
	my $class = shift;
	$self = {@_};
	bless($self, $class);

	Logger::set_filename("/var/opt/novell/xad/log/ndsdcinit.log");
	refresh_reg();

	return $self;
}

sub croak { die "$0: @_: $!\n" }


#Prepare the required directories for staging
sub stage_dir
{

	Log(MESG, ">>>Preparing staging directory tree");

	$self->SUPER::stage_dir();

	Log(MESG, "   Staging directories are successfully created");
}


#Copies the xad.ini to the stage etc dir
sub stage_default_config
{

	Log(MESG, ">>>Saving copy of domain configuration");

	#Trustposixoffset calculatation
	my $forest_root = registry::getReg("FORESTROOT");
	my $forestroot_domain_provider = `/opt/novell/xad/sbin/provision -q -q -l 60 --locate-dc $forest_root`;
	if ($?) {
		my $error = $? >> 8;
		die ("Failed to get the domain provider for domain [$forest_root] [$error]");
	}

	my $actual_forestnc = registry::getReg("ACTUALFORESTNC");
	my $trust_posix_offset;
	my $tree_name = registry::getReg("NDSTREENAME");
	my $ndap_forest_nc = $actual_forestnc;
	my $ip;
	$ndap_forest_nc =~ s/,/\./g;

	chomp($forestroot_domain_provider);

	eval {

		my @out = `/opt/novell/eDirectory/bin/ndsstat -h $forestroot_domain_provider -p ".cn=configuration.$ndap_forest_nc.T=$tree_name." -d`;
		if ($? != 0) {
			Log(WARN, "Could not get the configuration partition details");
		}

		my $ncp_dn = "";
		foreach (@out)
		{
			#Take the Master replica (check the replica type field)
		        if($_ =~ /^\.(.*)\.\s+ Master/)
		        {
				$ncp_dn = ".$1.";
				Log("Found the ncp server $ncp_dn who holds the master replica of the configuration partition.");
				last;
			}
		}

		chomp($ncp_dn);

		if (!($ncp_dn =~ /^\.cn/i)) {
			Log(WARN, "Coult not get the ncp server dn of the server holding the replica of the configuration partition");
		}
		#Remove the initial dot
		$ncp_dn =~ s/^\.//;

		#Remove the tree name
		$ncp_dn =~ s/\.T=$tree_name\.//;

		#Convert nds format to ldap format
		$ncp_dn =~ s/\./,/g;

		if ($ncp_dn =~ /^cn/i) {
			$ENV{'TEMP_ADM_PASSWD'} = $ENV{'ADM_PASSWD'};
			$ENV{'ADM_PASSWD'} = $ENV{'NDSEXISTINGADMINPASSWD'};
			$ip = util::getIpFromNcp($ncp_dn, $forestroot_domain_provider);
			$ENV{'ADM_PASSWD'} = $ENV{'TEMP_ADM_PASSWD'};
		}
		else {
			$ip = $forestroot_domain_provider;
		}

		Log("Obtaining the TPO value by contacting the server $ip");

		my $ldap_adminname = registry::getReg("LDAPADMINNAME");
		my @out = `LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem /opt/novell/eDirectory/bin/ldapsearch -H ldaps://$ip:636 -x -D '$ldap_adminname' -w '$ENV{'NDSEXISTINGADMINPASSWD'}' -b 'CN=Partitions,CN=Configuration,$actual_forestnc' -LLL`;

		if ($? != 0) {
			die("Could not get trust posix offset value, ldapsearch failed. \n @out");
		}

		my $search_tpo = "";
		my @grep_out  = grep(/usnintersite:/i, @out);

		if (defined $grep_out[0]) {
			#We have got the tpo value
			chomp($grep_out[0]);
			$grep_out[0] =~ /usnintersite:\s*(.*)/i;
			$search_tpo = $1;
		}

		if ($search_tpo ne "") {
			$trust_posix_offset = $search_tpo + 5242880;
		}
		else {
			$trust_posix_offset = 6291456;
		}

		my $xad_ini_file = registry::getReg("LIVE_ETCDIR")."/xad.ini";

		util::file_append_str("TRUSTPOSIXOFFSET = $trust_posix_offset\n", $xad_ini_file);

		#Calculate the values admin UID, GID which depends on trustposixoffset.
		registry::setReg("TRUSTPOSIXOFFSET", $trust_posix_offset);
		registry::setReg("ADMINISTRATOR_UID", $trust_posix_offset + 500);
		registry::setReg("DOMAIN_ADMINS_GID", $trust_posix_offset + 512);

		#NOTE: Nothing to do here. xad.ini file is created in the ndsdcinit.pl itself.
		#$self->SUPER::stage_default_config();
	};

	if ($@) { Log(ERR, $@) }

	Log(MESG, "   Domain configuration successfully saved");
}


#Get the domain guid and replace it in xad.ini
sub stage_domainguid
{

	Log(MESG, ">>>Configuring Domain Name Server");

	$self->SUPER::stage_domainguid();

	Log(MESG, "   Domain GUID is update in the configuration file");
}


#Prepare the ntp configuration file
sub stage_ntp
{

	Log(MESG, ">>>Configuring Network Time");

	$self->SUPER::stage_ntp();

	Log(MESG, "   NTP configuration is updated");
}


#Prepare the openldap.conf.
#FIXME: Does it required?
sub stage_libldap
{

	Log(MESG, ">>>Configuring OpenLDAP");

	$self->SUPER::stage_libldap();

	Log(MESG, "   Configured OpenLDAP");
}


#Generates the rc scripts and links them
sub stage_rc
{

	Log(MESG, ">>>Creating startup scripts");

	$self->SUPER::stage_rc();

	Log(MESG,"   Startup scripts successfully created");

}


#linkengine.ldif is generated
sub stage_linkengine
{

	Log(MESG, ">>>Generating linkengine.ldif");

	$self->SUPER::stage_linkengine();

	Log(MESG, "   Generated linkengine.ldif");

}


#schema and schema-bl ldif files are generated here
sub stage_schema_prepare
{

	Log(MESG, ">>>Preparing Schema Naming Context");

	$self->SUPER::stage_schema_prepare();

	Log(MESG, "   Schema is prepared\n");
}

sub schema_aggregate
{

	Log(MESG,">>>Generating Aggregated Schema");

	$self->SUPER::schema_aggregate();

	Log(MESG,"   msds.sch schema is Aggregated");
}

#Generates config-addl.ldif, config-addl-bl.ldif
sub stage_config
{

	Log(MESG, ">>>Preparing Configuration Naming Context");

	my $tempdir = registry::getReg("TEMPDIR");

	if (-e $tempdir."/stage_config") {

		Log("The stage of preparation of the configuration naming context was already done.");
		return 0;
	}

	$self->SUPER::stage_config();

	Log(MESG, "   Configuration ldif files are generated");

	`touch $tempdir/stage_config`;
}

#Generate all domain related ldif files
sub stage_domain
{

	Log(MESG, ">>>Preparing Domain Naming Context");

	if (registry::getReg("IS_REPLICA") eq "FALSE") {
		decide_domain_zones();
	}

	my $tempdir = registry::getReg("TEMPDIR");

	if (-e $tempdir."/stage_domain") {
		Log("The stage of preparing the domain naming context was already done.");
		return 0;
	}

	eval{

		$self->SUPER::stage_domain();

		my $domain_ldif = registry::getReg("DSSTATEDIR")."/domain/domain.ldif";
		my $admin_ldif = registry::getReg("DSSTATEDIR")."/domain/administrator.ldif";
		my $trust_posix_offset = registry::getReg("TRUSTPOSIXOFFSET");

		if (registry::getReg("IS_REPLICA") eq "FALSE") {

			my $rc = system("/opt/novell/xad/share/dcinit/provisionTools.sh update-tpo -f $domain_ldif -t $trust_posix_offset");
			if ($rc != 0) {
				die("Could not update the trust posix offset value in $domain_ldif file");
			}

			$rc = system("/opt/novell/xad/share/dcinit/provisionTools.sh update-tpo -f $admin_ldif -t $trust_posix_offset");
			if ($rc != 0) {
				die("Could not update the trust posix offset value in $admin_ldif file");
			}

		}

		my $ds_statedir = registry::getReg("DSSTATEDIR");
		my $template_dir = registry::getReg("TEMPLATEDIR");

		#egrep -v (nextRid|objectSid|serverState)
		open(INFILE, "$ds_statedir/domain/domain-object.ldif") or
			die("Could not open $ds_statedir/domain/domain-object.ldif");

		open(OUTFILE, ">$ds_statedir/domain/domain-obj.ldif") or
			die("Could not open $ds_statedir/domain/domain-obj.ldif for writing");

		while( <INFILE> ) {

			if ( grep( ! /(gpo|^a)/, $_) ) {
				print OUTFILE $_;
			}
		}
		close (INFILE);
		close(OUTFILE);

		unlink("$ds_statedir/domain/domain-object.ldif");
		move("$ds_statedir/domain/domain-obj.ldif", "$ds_statedir/domain/domain-object.ldif");


		Log("   Generating temporary-admin.ldif");
		stage::gen_ldif("$template_dir/temporary-admin.ini", '>',
				"$ds_statedir/domain/temporary-admin.ldif", "--defaultrootdomain");

		Log("   Generating temporary-admin-permissions.ldif");
		stage::gen_ldif("$template_dir/temporary-admin-permissions.ini", '>',
				"$ds_statedir/domain/temporary-admin-permissions.ldif", "--rootnc");

		Log("   Generating domain-temporary-container.ldif");
		stage::gen_ldif("$template_dir/domain-temporary-container.ini", '>',
				"$ds_statedir/domain/domain-temporary-container.ldif", "--defaultrootdomain");

		Log("   Generating nds-server-addl-first.ldif");
		stage::gen_ldif("$template_dir/nds-server-addl-first.ini", '>',
				"$ds_statedir/domain/nds-server-addl-first.ldif", "--forestparentnc");

		Log("   Generating nds-server-addl-second.ldif");
		stage::gen_ldif("$template_dir/nds-server-addl-second.ini", '>',
				"$ds_statedir/domain/nds-server-addl-second.ldif",  "--ndstempcontainer");


	};

	if ($@) { Log(ERR, $@) }

	Log(MESG, "   Domain Naming Context is prepared successfully");

	`touch $tempdir/stage_domain`;

}

#Add the additional parameters to ndsd.conf
sub stage_ndsd
{

	Log(MESG, ">>>Staging ndsd");

	$self->SUPER::stage_ndsd();

	Log("   successfully staged ndsd");

}

#Configure Kerberos
sub stage_krb5
{

	Log(MESG, ">>>Configuring Kerberos");

	$self->SUPER::stage_krb5();

	Log(MESG, "Kerberos successfully Configured");
}


#Configures the samba
sub stage_samba
{

	Log(MESG, ">>>Configuring SMB Server");

	my $tempdir = registry::getReg("TEMPDIR");

	if (-e $tempdir."/stage_samba") {
		Log("The stage of the SMB server configuration was already done.");
		return 0;
	}

	$self->SUPER::stage_samba();

	my $templatedir = registry::getReg("TEMPLATEDIR");
	my $stagedir = registry::getReg("STAGEDIR");
	my $live_dsstatedir = registry::getReg("LIVE_DSSTATEDIR");
	my $live_stagedir = registry::getReg("LIVE_STAGEDIR");

	eval{

		Log("   Creating $stagedir/etc/samba/smb.conf");
		stage::reg_substitute("$templatedir/smb.conf", "$stagedir/etc/samba/smb.conf");
		util::make_netlogon_share_writable("$stagedir/etc/samba/smb.conf");

		Log("   Copying /etc/samba/smb.conf to $live_dsstatedir/backup/smb.conf.org");
		copy("/etc/samba/smb.conf", "$live_dsstatedir/backup/smb.conf.org");

		Log("   Copying $stagedir/etc/samba/smb.conf to $live_stagedir/etc/samba/smb.conf");
		copy("$stagedir/etc/samba/smb.conf", "$live_stagedir/etc/samba/smb.conf");

	};

	if ($@) { Log(ERR, $@) }

	Log(MESG, "   Successfully staged samba");

	`touch $tempdir/stage_samba`;
}

sub stage_sysvol
{

	my $tempdir = registry::getReg("TEMPDIR");

	if (-e $tempdir."/stage_sysvol") {

		Log("The stage of the sysvol creation was already done");
		return 0;
	}

	$self->SUPER::stage_sysvol();


	`touch $tempdir/stage_sysvol`;

}

sub stage_gpo
{

	Log(MESG, ">>>Configuring Group Policy Objects");

	$self->SUPER::stage_gpo();

	Log(MESG, ">>>Group Policy Objects are successfully configured");
}

#xadsd configuration is taken care
sub stage_xadsd
{

	Log(">>>Configuring Domain Services configuration");

	$self->SUPER::stage_xadsd();

	Log("   Domain Services configuration is prepared");
}

#Get the new site if it is required
sub stage_site
{

	Log(MESG, ">>>Configuring Site");

	$self->SUPER::stage_site();

	Log(MESG, "   Site configuration prepared");
}

#Bring the staging directories to live
sub stage_to_live
{

	Log(MESG, ">>>Bringing the stage directories to live");

	$self->SUPER::stage_to_live();

	Log(MESG, "   Stage directories are live now");
}

sub configure_nldap_settings
{

	registry::setReg("NDSADMINNAME", registry::getReg("NDSTEMPADMINNAME"));
	$ENV{ADM_PASSWD} = $ENV{NDSEXISTINGADMINPASSWD};
	$self->SUPER::configure_nldap_settings();


}
sub configure_zone_object
{

	Log(MESG, ">>>Creating DNS Zone object");

	if (registry::getReg("IS_REPLICA") eq "FALSE") {
		decide_domain_zones();
	}

	eval{

		my $ip_address = registry::getReg("IPADDRESS");
		my $ldap_temp_adminname = registry::getReg("LDAPTEMPADMINNAME");
		my $ldap_adminname = registry::getReg("LDAPADMINNAME");

		$ENV{ADM_PASSWD} = $ENV{NDSEXISTINGADMINPASSWD};
		registry::setReg("LDAPADD_FLAGS", "-h $ip_address -x -D $ldap_temp_adminname -a -w '$ENV{'ADM_PASSWD'}'");

		if (! nds::check_certificate()) {
			nds::get_certificate();
		}

	};

	if ($@) { Log(ERR, $@) }

	$self->SUPER::configure_zone_object();

	Log(MESG,"   DNS Zone objects are created");
}

sub configure_zone_rr_object
{

	Log(MESG, ">>>Creating the DNS Resource Records object");

	if (registry::getReg("IS_REPLICA") eq "FALSE") {
		decide_domain_zones();
	}

	$self->SUPER::configure_zone_rr_object();
	$self->SUPER::configure_delegate_zone_object();

	Log(MESG, "   DNS Resource Records are created");
}

sub configure_nldap_objects
{

	my $ldapi_url = registry::getReg("LDAPI_URL");
	my $ldap_temp_adminname = registry::getReg("LDAPTEMPADMINNAME");

	$ENV{ADM_PASSWD} = $ENV{NDSEXISTINGADMINPASSWD};
	registry::setReg("LDAPMODIFY_FLAGS", " -H $ldapi_url -x -D '$ldap_temp_adminname' -w '$ENV{'ADM_PASSWD'}'");

	$self->SUPER::configure_nldap_objects();
}

sub refresh_nldap
{

	$ENV{ADM_PASSWD} = $ENV{NDSEXISTINGADMINPASSWD};
	registry::setReg("NDSADMINNAME", registry::getReg("NDSTEMPADMINNAME"));

	$self->SUPER::refresh_nldap();
}

####################################################################
# Partition and replica related functions                          #
####################################################################

#Creates the domain partition.
#This will be executed only for NNM setups.
sub create_domain_nc
{

	Log(MESG, ">>>Creating Domain Naming Context");

	my $mappeddomainnc = registry::getReg("MAPPEDDOMAINNC");
	my $defaultrootdomain = registry::getReg("Domain NC");

	if ($mappeddomainnc ne $defaultrootdomain) {
		Log("Skipping it since the Installation is on the existing eDirectory");
		return;
	}

	my $ipaddress = registry::getReg("IPADDRESS");
	my $ldap_temp_adminname = registry::getReg("LDAPTEMPADMINNAME");

	sleep(40); #Waits 40 secs before partitioning to sync up
	$self->SUPER::create_domain_nc($ipaddress, $mappeddomainnc, $ldap_temp_adminname, "env:NDSEXISTINGADMINPASSWD");

	eval{

		my $parent_domain = registry::getReg("PARENTDOMAIN");
		my $parent_domain_provider = `/opt/novell/xad/sbin/provision -q -q -l 60 --locate-dc $parent_domain`;
		if ($?) {
			my $error = $? >> 8;
			die ("Failed to get the domain provider for domain [$parent_domain] [$error]");
		}
		chomp($parent_domain_provider);

		$parent_domain_provider =~ /^(.*?)\./;
		my $parent_machine_name = $1;

		#Wait until the split is complete
		Log(MESG, "Waiting until the domain NC split is complete");

		my $nds_partition_name = registry::getReg("NDSPARTITIONNAME");
		my $domainnc_split_complete = 0;
		while ($domainnc_split_complete != 1) {
			sleep 1;
			my $out = `/opt/novell/eDirectory/bin/ndsstat -h $parent_domain_provider -p $nds_partition_name | grep -i "\.cn=$parent_machine_name\."`;

			#The second field is replica state
			if (lc([split(/\s+/,$out)]->[2]) eq "on"){
				$domainnc_split_complete = 1;
			}
		}

		if ($domainnc_split_complete == 0) {
			die("Domain NC is not split completely, hence terminating the Installation...");
		}
	};

	if ($@) { Log(ERR, $@) }

	Log(MESG, "   Domain Naming Context created successfully");

}

#Here we add the replica of the domain partition to the local server
sub add_domain_replica
{

	my $ip_address = registry::getReg("IPADDRESS");
	my $mappeddomainnc = registry::getReg("MAPPEDDOMAINNC");
	my $install_machinename = registry::getReg("INSTALLMACHINENAME");
	my $nds_tempcontainer = registry::getReg("NDSTEMPCONTAINER");
	my $ldap_temp_admin = registry::getReg("LDAPTEMPADMINNAME");

	Log(MESG, ">>>Adding Domain Replica");

	eval{
		replops::add_replica(h=>$ip_address, p=>389,
				s=>"CN=$install_machinename,$nds_tempcontainer",
				n=>$mappeddomainnc,
				t=>"writable",
				D=>$ldap_temp_admin,
				w=>"env:NDSEXISTINGADMINPASSWD");
	};

	if ($@) { Log(ERR, $@) }

	Log(MESG, "   Domain Replica successfully added");

}

#Make the domain replica as master
sub make_master_nc
{

	Log(MESG, ">>>Making the local replica as the master replica");
	my $ip_address = registry::getReg("IPADDRESS");
	my $install_machinename = registry::getReg("INSTALLMACHINENAME");
	my $nds_tempcontainer = registry::getReg("NDSTEMPCONTAINER");
	my $mappeddomainnc = registry::getReg("MAPPEDDOMAINNC");
	my $ldap_temp_admin = registry::getReg("LDAPTEMPADMINNAME");

	eval{

		my $rc = replops::make_master_replica(h=>$ip_address,p=>389,
						s=>"CN=$install_machinename,$nds_tempcontainer",
						n=>"$mappeddomainnc",
						D=>"$ldap_temp_admin",
						w=>"env:NDSEXISTINGADMINPASSWD");
		if ($rc != 0) {
			die("Could not Make as master replica");
		}
	};

	if ($@) { Log(ERR, $@) }
	sleep 20;
	Log(MESG, "   Local replica role changed to master");

}

#Here we add the replica of the configuration partition to the local server
sub add_config_replica
{

	my $ip_address = registry::getReg("IPADDRESS");
	my $install_machinename = registry::getReg("INSTALLMACHINENAME");
	my $nds_tempcontainer = registry::getReg("NDSTEMPCONTAINER");
	my $ldap_temp_admin = registry::getReg("LDAPTEMPADMINNAME");
	my $actual_forestnc = `/opt/novell/xad/share/dcinit/provisionTools.sh actual-forest-nc`;
	chomp($actual_forestnc);

	Log(MESG, ">>>Adding Config Replica");

	eval{
		replops::add_replica(h=>$ip_address, p=>389,
				s=>"CN=$install_machinename,$nds_tempcontainer",
				n=>"CN=Configuration,$actual_forestnc",
				t=>"writable",
				D=>$ldap_temp_admin,
				w=>"env:NDSEXISTINGADMINPASSWD");
	};

	if ($@) { Log(ERR, $@) }

	Log(MESG, "   Config Replica successfully added");

}

#Here we add the replica of the schema partition to the local server
sub add_schema_replica
{

	my $ip_address = registry::getReg("IPADDRESS");
	my $install_machinename = registry::getReg("INSTALLMACHINENAME");
	my $nds_tempcontainer = registry::getReg("NDSTEMPCONTAINER");
	my $ldap_temp_admin = registry::getReg("LDAPTEMPADMINNAME");
	my $actual_forestnc = `/opt/novell/xad/share/dcinit/provisionTools.sh actual-forest-nc`;
	chomp($actual_forestnc);

	Log(MESG, ">>>Adding Schema Replica");

	eval{
		replops::add_replica(h=>$ip_address, p=>389,
				s=>"CN=$install_machinename,$nds_tempcontainer",
				n=>"CN=Schema,CN=Configuration,$actual_forestnc",
				t=>"writable",
				D=>$ldap_temp_admin,
				w=>"env:NDSEXISTINGADMINPASSWD");
	};

	if ($@) { Log(ERR, $@) }

	Log(MESG, "   Schema Replica successfully added");

}

####################################################################
# Import functions for domain/forest objects                       #
####################################################################

#Adds the configuration Partition objects
sub config_online
{

	my $ldapi_url = registry::getReg("LDAPI_URL");
	my $ldap_temp_adminname = registry::getReg("LDAPTEMPADMINNAME");

	# Refreshing the local LDAP server so that the crossref plugin
	# could read the crossref objects under configuration partition.

	Log("Calling refresh_nldap() to refresh local LDAP server before adding the crossref object for local domain.");
	refresh_nldap();

	$ENV{ADM_PASSWD} = $ENV{NDSEXISTINGADMINPASSWD};
	registry::setReg("LDAPADD_FLAGS", " -H $ldapi_url -x -D '$ldap_temp_adminname' -a -w '$ENV{'ADM_PASSWD'}'");

	my $config_ldif = registry::getReg("LIVE_DSSTATEDIR")."/config/config-addl.ldif";

	$self->SUPER::config_online($config_ldif);

}

sub domaindns_associate_online
{

	my $ldapi_url = registry::getReg("LDAPI_URL");
	my $ldap_temp_adminname = registry::getReg("LDAPTEMPADMINNAME");

	$ENV{ADM_PASSWD} = $ENV{NDSEXISTINGADMINPASSWD};
	registry::setReg("LDAPMODIFY_FLAGS", " -H $ldapi_url -x -D '$ldap_temp_adminname' -a -w '$ENV{'ADM_PASSWD'}'");

	$self->SUPER::domaindns_associate_online();

}

sub domain_object_modify_online
{

#	The below flags is already set by domaindns_associate_online
#	$ENV{ADM_PASSWD} = $ENV{NDSEXISTINGADMINPASSWD};
#	registry::setReg("LDAPMODIFY_FLAGS", " -H $ldapi_url -x -D $ldap_temp_adminname -a -w $ENV{'ADM_PASSWD'}");

	$self->SUPER::domain_object_modify_online();

}

sub domain_online
{

	my $ldapi_url = registry::getReg("LDAPI_URL");
	my $ldap_temp_adminname = registry::getReg("LDAPTEMPADMINNAME");

	$ENV{ADM_PASSWD} = $ENV{NDSEXISTINGADMINPASSWD};
	registry::setReg("LDAPADD_FLAGS", " -H $ldapi_url -x -D '$ldap_temp_adminname' -a -w '$ENV{'ADM_PASSWD'}'");

	$self->SUPER::domain_online();

}

sub import_nds_domain_acl
{

	Log(MESG, ">>>Configuring default ACLs and NMAS password policies");

	$ENV{ADM_PASSWD} = $ENV{NDSEXISTINGADMINPASSWD};
 	my $ldap_modify_flags = " -H ".registry::getReg("LDAPI_URL").
			     " -x -D '". registry::getReg("LDAPTEMPADMINNAME")."' -w '$ENV{'ADM_PASSWD'}'";

	registry::setReg("LDAPMODIFY_FLAGS", $ldap_modify_flags);


	$self->SUPER::import_nds_domain_acl();

	Log(MESG, "   Configured the default ACLs and NMAS password policies");

}

#Update the server reference
sub import_domain_server_reference
{

	#FIXME: check is_replica == FALSE

	$ENV{ADM_PASSWD} = $ENV{NDSEXISTINGADMINPASSWD};
 	my $ldap_modify_flags = " -H ".registry::getReg("LDAPI_URL").
			     " -x -D '". registry::getReg("LDAPTEMPADMINNAME")."' -w '$ENV{'ADM_PASSWD'}'";

	registry::setReg("LDAPMODIFY_FLAGS", $ldap_modify_flags);

	$self->SUPER::import_domain_server_reference();
}

#TPO value is updated upon cn=Partition
sub set_trustposixoffset
{

	my $ldapi_url = registry::getReg("LDAPI_URL");
	my $ldap_temp_adminname = registry::getReg("LDAPTEMPADMINNAME");
	my $trust_posix_offset = registry::getReg("TRUSTPOSIXOFFSET");
	my $ldap_modify = registry::getReg("LDAPMODIFY");
	$ENV{ADM_PASSWD} = $ENV{NDSEXISTINGADMINPASSWD};
 	my $ldap_modify_flags = " -H $ldapi_url -x -D '$ldap_temp_adminname' -a -w '$ENV{'ADM_PASSWD'}'";

	registry::setReg("LDAPMODIFY_FLAGS", $ldap_modify_flags);
	$ENV{KRB5CCNAME} = registry::getReg("KRB5CCNAME_DOMAIN");

	Log(MESG, ">>>Configuring the trust posix offset value");

	eval{

		if($trust_posix_offset eq "") {
			Log(WARN, "Could not find the TRUSTPOSIXOFFSET value, please check the xad.ini");
			return;
		}

		Log("Updating the Trusted Posix Offset value");

		my $tpo_file = registry::getReg("LIVE_DSSTATEDIR")."/domain/trust-posix-offset.ldif";
		my $str;

		#Delete the file if exists, and create the file
		unlink($tpo_file) if (-e $tpo_file);
		`touch $tpo_file`;

		$str = "dn: CN=Partitions,CN=Configuration,".registry::getReg("LDAPFORESTNC")."\n";
		util::file_append_str($str, $tpo_file);

		util::file_append_str("changetype: modify\n", $tpo_file);
		util::file_append_str("replace: USNIntersite\n", $tpo_file);
		util::file_append_str("USNIntersite: $trust_posix_offset\n", $tpo_file);
		util::file_append_str("\n\n", $tpo_file);

		$str = "dn: ". registry::getReg("MAPPEDDOMAINNC")."\n";
		util::file_append_str($str, $tpo_file);

		util::file_append_str("changetype: modify\n", $tpo_file);
		util::file_append_str("replace: trustPosixOffset\n", $tpo_file);
		util::file_append_str("trustPosixOffset: $trust_posix_offset\n", $tpo_file);

		Log(MESG, "Importing $tpo_file");
		#Added for ldif modification
		my $rc = import::import_file(host=>$ldapi_url,
			admin=>$ldap_temp_adminname,
			pwd=>$ENV{'ADM_PASSWD'},
			file=>$tpo_file,
			cont_if_exist=>"true");

		if ($rc != 0) {
			die("Could not import $tpo_file");
		}
	};

	if ($@) { Log(ERR, $@) }

	Log(MESG, "   The trust posix offset value is updated successfully");
}

#Imports configuration back link objects to the server
sub config_bl_online
{

	my $ldap_modify = registry::getReg("LDAPMODIFY");
	my $config_bl_ldif = registry::getReg("LIVE_DSSTATEDIR")."/config/config-addl-bl.ldif";

	$ENV{ADM_PASSWD} = $ENV{NDSEXISTINGADMINPASSWD};
 	my $ldap_add_flags = " -H ".registry::getReg("LDAPI_URL").
			     " -x -D '".registry::getReg("LDAPTEMPADMINNAME")."' -a -w '$ENV{'ADM_PASSWD'}'";

	registry::setReg("LDAPADD_FLAGS", $ldap_add_flags);

	Log(MESG, ">>>Importing Configuration Naming Context (Back-Links) into LDAP Server");

	eval{

		Log("Importing $config_bl_ldif");

		my $rc = import::import_file(host=>registry::getReg("LDAPI_URL"),
			admin=>registry::getReg("LDAPTEMPADMINNAME"),
			pwd=>$ENV{'ADM_PASSWD'},
			file=>$config_bl_ldif,
			cont_if_exist=>"true");

		if ($rc != 0) {
			die("Could not execute ldapadd for $LIVE_DSSTATEDIR/config/config-addl-bl.ldif");
		}

	};

 	if ($@) { Log(ERR, $@) }

	Log(MESG, "   Configuration Naming Context (Back-Links) are imported successfully into LDAP Server");

}

#Back linked domain objects are added to the server
sub domain_bl_online
{

	$ENV{ADM_PASSWD} = $ENV{NDSEXISTINGADMINPASSWD};
 	my $ldap_modify_flags = " -H ".registry::getReg("LDAPI_URL").
			     " -x -D '".registry::getReg("LDAPTEMPADMINNAME")."' -w '$ENV{'ADM_PASSWD'}'";

	registry::setReg("LDAPMODIFY_FLAGS", $ldap_modify_flags);

	$self->SUPER::domain_bl_online();

}

sub touch_all_users_groups
{
	my $domain_nc = registry::getReg("DEFAULTROOTDOMAIN");
	my $adminDN = registry::getReg("LDAPADMINNAME");
	my $adminDNQuotes = "\'$adminDN\'";
	my $ldap_search_flags = " -w '$ENV{NDSEXISTINGADMINPASSWD}' -x -D ".$adminDNQuotes;
	my $ldap_modify_flags = " -w '$ENV{NDSEXISTINGADMINPASSWD}' -D ".$adminDNQuotes;

	my $ldap_search_final_flags = $ldap_search_flags . " -H ldaps://". registry::getReg("IP Address").":636";
	my $ldap_modify_final_flags = $ldap_modify_flags . " -H ldaps://". registry::getReg("IP Address").":636";
	touch_users_groups();

	my $parent_domain = registry::getReg("Parent Domain");
	my $parent_domain_provider = `/opt/novell/xad/sbin/provision -q -q -l 60 --locate-dc $parent_domain`;
	if ($?) {
		my $error = $? >> 8;
		die("Failed to get the domain provider for domain [$parent_domain] [$error]");
	}
	chomp($parent_domain_provider);
	$ldap_search_final_flags = $ldap_search_flags . " -H ldaps://". $parent_domain_provider. ":636";
	$ldap_modify_final_flags = $ldap_modify_flags . " -H ldaps://". $parent_domain_provider. ":636";
	$domain_nc =~  s/^([a-z,A-Z,0-9]*)=([a-z,A-Z,0-9]*),//g; #Figures out the parent mapped container.
	touch_users_groups();

	sub touch_users_groups
	{
		eval {
			my $ldapsearch = "/opt/novell/eDirectory/bin/ldapsearch";
			my $ldapmodify = "/opt/novell/eDirectory/bin/ldapmodify";
			my $dsstatedir = registry::getReg("DSSTATEDIR");
			my $template_dir = registry::getReg("TEMPLATEDIR");

			Log ("Executing $ldapsearch $ldap_search_final_flags -b $domain_nc \"(&(objectclass=user))\" dn | sed ' /^ / {; H; d; }; /^ /! {; x; s/\\n //; }; ' > $dsstatedir/samify/intermediate_file1");
			`LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem $ldapsearch $ldap_search_final_flags -b $domain_nc "(&(objectclass=user))" dn | sed ' /^ / {; H; d; }; /^ /! {; x; s/\\n //; }; ' > $dsstatedir/samify/intermediate_file1`;
			`grep 'dn: *' $dsstatedir/samify/intermediate_file1 | grep -v '^#.*' > $dsstatedir/samify/intermediate_file2`;

			`/opt/novell/xad/share/dcinit/makeLdif.pl -t $template_dir/samify_user.ldif -a $dsstatedir/samify/intermediate_file2 -o $dsstatedir/samify/final_usr.ldif`;

			$rc = system("LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem $ldapmodify $ldap_modify_final_flags -f $dsstatedir/samify/final_usr.ldif");
			if ($rc != 0) {
				die ("Failed to touch the users");
			}

			`LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem $ldapsearch $ldap_search_final_flags -b $domain_nc "(&(objectclass=group))" dn | sed ' /^ / {; H; d; }; /^ /! {; x; s/\\n //; }; ' > $dsstatedir/samify/intermediate_file1`;
			`grep 'dn: *' $dsstatedir/samify/intermediate_file1 | grep -v '^#.*' > $dsstatedir/samify/intermediate_file2`;

			`/opt/novell/xad/share/dcinit/makeLdif.pl -t $template_dir/samify_grp.ldif  -a $dsstatedir/samify/intermediate_file2 -o $dsstatedir/samify/final_grp.ldif`;

			$rc = system("LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem $ldapmodify $ldap_modify_final_flags -f $dsstatedir/samify/final_grp.ldif");
			if ($rc != 0) {
				die("Failed to touch the groups");
			}
		};

		if ($@) {
			Log(ERR, $@);
		}


	}


}

sub add_custom_objects
{
	shift;
	my $stage = shift;
	$self->SUPER::add_custom_objects($stage);
}

sub update_nsswitch
{
	util::update_nsswitch();
}

sub restart_services
{
	util::restart_services();
}

sub update_resolv_conf {

	util::update_resolv_conf();
}

sub enable_krb5_local_lookup
{
	registry::setReg("ADM_PRINCIPAL", registry::getReg("Domain Administrator Principal"));
	$self->SUPER::enable_krb5_local_lookup();
}

sub acquire_domain_ticket
{

	if (not defined $ENV{ADM_PASSWD_DOMAIN}) {
		die("The domain password is not supplied to get the ticket.");
	}

	$ENV{ADM_PASSWD} = $ENV{ADM_PASSWD_DOMAIN};

	my $adm_principal = registry::getReg("Domain Administrator Principal");

	if (registry::getReg("IS_REPLICA") eq "FALSE") {
		$adm_principal = uc($adm_principal);
	}

	registry::setReg("ADM_PRINCIPAL", $adm_principal);

	$self->SUPER::acquire_domain_ticket();
}

sub migrate_nkdc_realm
{
	my $ldaptemp_admin = registry::getReg("LDAPTEMPADMINNAME");

	registry::setReg("LDAPADMINNAME", $ldaptemp_admin);
	$self->SUPER::migrate_nkdc_realm();
}

sub sync_gpo_nmas
{
	$self->SUPER::sync_gpo_nmas();
}

sub update_cron_tab
{
	util::update_cron_tab();
}

sub nldap_enable_requiretls
{

	#$ENV{ADM_PASSWD} = $ENV{NDSEXISTINGADMINPASSWD};
	#registry::setReg("NDSADMINNAME", registry::getReg("NDSTEMPADMINNAME"));

	# use the local domain administrator to set enable require TLS as usage
	# of tree admin would take us to other servers for admin dn resolution

	$ENV{ADM_PASSWD} = $ENV{ADM_PASSWD_DOMAIN};
	my $Local_Admin_DN = registry::getReg("Mapped Administrator DN");
	$Local_Admin_DN =~ s/,/\./g;
	registry::setReg("NDSADMINNAME", $Local_Admin_DN);

	$self->SUPER::nldap_enable_requiretls();

}

sub create_site
{
	my $ldapi_url = registry::getReg("LDAPI_URL");
	my $ldap_temp_adminname = registry::getReg("LDAPTEMPADMINNAME");

	$ENV{ADM_PASSWD} = $ENV{NDSEXISTINGADMINPASSWD};
	registry::setReg("LDAPMODIFY_FLAGS", " -H $ldapi_url -x -D '$ldap_temp_adminname'".
			" -w '$ENV{'ADM_PASSWD'}'");

	$self->SUPER::create_site();
}

sub cleanup{

	delete_temporary_admin();
	$self->SUPER::stage_clean();

}

sub decide_domain_zones
{
	# Calculate the forward and reverse zone contexts.
	eval {
		my $ldap_existing_adminname = registry::getReg("LDAPTEMPADMINNAME");
		my $dnslocator_obj = registry::getReg("DNS_LOCATOR_OBJECT");
		my $dnsserver_context = registry::getReg("DNSSERVER_CONTEXT");
		my $forest_name = registry::getReg("Forest Root");
		my $actual_forestnc = registry::getReg("ACTUALFORESTNC");
		my $domain_nc = registry::getReg("Mapped Domain NC");
		my $default_container;
		my $default_zone_context;

		my $frd_provider = `/opt/novell/xad/sbin/provision -q -q -l 60 --locate-dc '$forest_name'`;
		if ($?) {
			my $error = $? >> 8;
			die("Failed to get the forest root domain provider [$forest_name] [$error]");
		}
		chomp($frd_provider);

		if (registry::getReg("DNS Master") eq "TRUE") {
			$default_zone_context = "$dnsserver_context";
		} else {
			$default_container = `/opt/novell/xad/share/dcinit/provisionTools.sh get-default-container -d $forest_name`;
			if($? ne 0) {
				die("Could not get default container.");
			}
			chomp($default_container);
			$default_zone_context = "$default_container,$actual_forestnc";
		}

		my @dns_zones  = `LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem /opt/novell/eDirectory/bin/ldapsearch -D '$ldap_existing_adminname' -w '$ENV{NDSEXISTINGADMINPASSWD}' -H ldaps://$frd_provider:636 -b '$dnslocator_obj' -s base dNIPDNSZones -E '2.16.840.1.113719.1.513.4.5' -LLL 2>/dev/null | sed -e ' /^ / {; H; d; }; /^ /! {; x; s/\\n //; }; ' -e  's/st=/s=/gi' | grep -i "dNIPDNSZones: "`;
		chomp(@dns_zones);
		if(@dns_zones) {
			#Set the forward zone context.
			my $fwd_zone = registry::getReg("DNS_ZONE");
			my ($fwdzone_context)  = grep {/dNIPDNSZones: cn=$fwd_zone/i} @dns_zones;
			$fwdzone_context =~ s/^dNIPDNSZones: cn=$fwd_zone,//gi;
			if($fwdzone_context ne "" ){
				registry::setReg("IS_FWDZONE_EXIST", "TRUE");
				registry::setReg("FWDZONE_CONTEXT", $fwdzone_context);
			}
			else {
				registry::setReg("IS_FWDZONE_EXIST", "FALSE");
				registry::setReg("FWDZONE_CONTEXT", "$default_zone_context");
			}

			#Set the reverse zone context.

			my $rev_zone = registry::getReg("IP_ADDRESS_UNDERSCORE_FORM");
			my ($revzone_context)  = grep {/dNIPDNSZones: cn=$rev_zone/i} @dns_zones;
			$revzone_context =~ s/^dNIPDNSZones: cn=$rev_zone,//gi;
			if($revzone_context ne "" ){
				registry::setReg("IS_REVZONE_EXIST", "TRUE");
				registry::setReg("REVZONE_CONTEXT", $revzone_context);
			}
			else {
				Log(WARN, "Could not locate reverse zone for the DC");
				registry::setReg("IS_REVZONE_EXIST", "FALSE");
				registry::setReg("REVZONE_CONTEXT", "$default_zone_context");
			}
		}
		else {
			die("Failed to fetch dNIPDNSZones from DNS_LOCATOR_OBJECT");
		}

		# Set the FWDZONE_DNSMASTER for the forward zone.
		my $dns_server_ip = registry::getReg("DNS_SERVER_IP");
		my $dns_server = registry::getReg("DNSSERVER");
		my $fwd_zone_context = registry::getReg("FWDZONE_CONTEXT");
		if (lc(registry::getReg("DNS Master")) eq "false") {
			Log("Getting the DNS Master, /opt/novell/xad/share/dcinit/provisionTools.sh get-dns-master -p $dns_server_ip -a '$ldap_existing_adminname' -c '$dns_server'");
			my $dns_master = `ADM_PASSWD='$ENV{NDSEXISTINGADMINPASSWD}' /opt/novell/xad/share/dcinit/provisionTools.sh get-dns-master -p $dns_server_ip -a '$ldap_existing_adminname' -c "$dns_server"`;
			chomp($dns_master);
			if ($dns_master ne "") {
				registry::setReg("FWDZONE_DNSMASTER", $dns_master);
			}
			else {
				Log(WARN, "Could not calculate the DNS Master, assigning the local machine as the master. After the provisioning please change the name-server entry in the SOA record of this domain's zone");
				registry::setReg("FWDZONE_DNSMASTER", registry::getReg("DNSHOSTNAME"));
			}
		}
		else {
			registry::setReg("FWDZONE_DNSMASTER", registry::getReg("DNSHOSTNAME"));
		}
		Log("The forward zone dns master is ".registry::getReg("FWDZONE_DNSMASTER"));

		# Set the REVZONE_DNSMASTER for the reverse zone.
		# We set designated dns server of FRD zone as the REVZONE_DNSMASTER.
		my $frd_zone = $forest_name;
		$frd_zone =~ s/\./_/g;
		my $frd_designated_dns = `LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem /opt/novell/eDirectory/bin/ldapsearch -D '$ldap_existing_adminname' -w '$ENV{NDSEXISTINGADMINPASSWD}' -H ldaps://$frd_provider:636 -b 'cn=$frd_zone,$fwd_zone_context' -s base dNIPDesignatedServer -N -LLL 2>/dev/null | sed ' /^ / {; H; d; }; /^ /! {; x; s/\\n //; }; ' | grep -i "dNIPDesignatedServer: " | awk -F "dNIPDesignatedServer: " '{print \$2}'`;
		chomp($frd_designated_dns);
		if ($frd_designated_dns eq "") {
			Log(WARN, "Could not fetch the FRD zone designated name server DN, assigning the local machine as the master. After the provisioning please change the name-server entry in the SOA record of this domain's zone");
			registry::setReg("REVZONE_DNSMASTER", registry::getReg("DNSHOSTNAME"));
		}
		else {
			my $revzone_dnsmaster = `LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem /opt/novell/eDirectory/bin/ldapsearch -D '$ldap_existing_adminname' -w '$ENV{NDSEXISTINGADMINPASSWD}' -H ldaps://$frd_provider:636 -b "$frd_designated_dns" -s base dNIPServerDNSNames -LLL | sed ' /^ / {; H; d; }; /^ /! {; x; s/\\n //; }; ' | grep "dNIPServerDNSNames: " | awk -F "dNIPServerDNSNames: " '{print \$2}'`;
			chomp($revzone_dnsmaster);
			if ($revzone_dnsmaster eq "") {
				Log(WARN, "Could not fetch the FRD zone name server dns name, assigning the local machine as the master. After the provisioning please change the name-server entry in the SOA record of this domain's zone");
				registry::setReg("REVZONE_DNSMASTER", registry::getReg("DNSHOSTNAME"));
			}
			else {
				registry::setReg("REVZONE_DNSMASTER", $revzone_dnsmaster);
			}
		}
		Log("The reverse zone dns master is ".registry::getReg("REVZONE_DNSMASTER"));

	};

	if ($@) { Log(ERR, $@) }

}

sub refresh_reg
{

	if (registry::getReg("IS_REPLICA") eq "TRUE") {
		#Additional DC does not inherits this method
		#We should not calculate the below variable when its Additional DC
		return;
	}

	registry::clearReg();
	my $parent_domain = registry::getReg("PARENTDOMAIN");
	my $sitename = util::find_sitename($parent_domain);
	registry::setReg("Site", $sitename);
	registry::reg_init();

	eval {
		my $parent_domain = registry::getReg("PARENTDOMAIN");
		my $parent_domain_provider = `/opt/novell/xad/sbin/provision -q -q -l 60 --locate-dc $parent_domain`;
		if ($?) {
			my $error = $? >> 8;
			die ("Failed to get the domain provider for domain [$parent_domain] [$error]");
		}

		chomp($parent_domain_provider);
		#Common for all except ADC
		registry::setReg("NDSSERVERCONTEXT", registry::getReg("NDS Server Context"));

		my $nds_existing_adminname = registry::getReg("NDSEXISTINGADMINNAME");
		registry::setReg("NDSADMINNAME",  $nds_existing_adminname);

		my $temp = $nds_existing_adminname;
		$temp =~ s/\.([a-z,A-Z,0-9]*)=/,$1=/g;
		registry::setReg("LDAPADMINNAME", $temp);

		registry::setReg("LDAPCOMMON_FLAGS", " -h $parent_domain_provider -Y GSS-SPNEGO");
		registry::compute_pre_install_variables();

		# Write the NDSADMINNAME and LDAPADMINNAME in the xad.ini file.
		my $DCINIT_CONFIG = "/etc/opt/novell/xad/xad.ini";
		open(CONFIG,">>$DCINIT_CONFIG") || croak("could not open $DCINIT_CONFIG\n");

		`grep NDSADMINNAME $DCINIT_CONFIG`;
		if ($?) {
			my $ndsadminname = registry::getReg("NDSADMINNAME");
			print CONFIG "NDSADMINNAME = $ndsadminname\n";
		}

		`grep LDAPADMINNAME $DCINIT_CONFIG`;
		if ($?) {
			my $ldapadminname = registry::getReg("LDAPADMINNAME");
			print CONFIG "LDAPADMINNAME = $ldapadminname\n";
		}

		close(CONFIG);

		if (!(registry::getReg("NDSPARENTDOMAINADMIN"))) {
			if (defined $ENV{XADPARENTDOMAINADMIN}) {
				my $parent_admin_dn = $ENV{XADPARENTDOMAINADMIN};
				$parent_admin_dn =~ /cn=([^\.]*)/i;
				registry::setReg("NDSPARENTADMINNAME", "CN=$1.CN=Users.".registry::getReg("PARENTDOMAINNC"));
			}else {
				registry::setReg("NDSPARENTADMINNAME", "CN=Administrator.CN=Users.".registry::getReg("PARENTDOMAINNC"));
			}
		}
		else {
			registry::setReg("NDSPARENTADMINNAME", registry::getReg("NDSPARENTDOMAINADMIN"));
		}


		if(registry::getReg("MAPPED") eq "1") {

			my $dns_root = registry::getReg("DNSROOT");
			my $nds_parent_adminname = registry::getReg("NDSPARENTADMINNAME");

			my $temp = `ADM_PASSWD='$ENV{ADM_PASSWD_PARENT}' /opt/novell/xad/share/dcinit/ndsConfigServerContext.sh $dns_root '$nds_parent_adminname' $parent_domain_provider`;
			chomp($temp);

			registry::setReg("NDSADMINSERVERCONTEXT", $temp);

		}
		else {
			my $temp = registry::getReg("MAPPEDDOMAINNC");
			$temp =~ s/,/./g;
			registry::setReg("NDSADMINSERVERCONTEXT", $temp);
		}

		registry::setReg("NDSTEMPADMINNAME", registry::getReg("NDSEXISTINGADMINNAME"));

		$temp = registry::getReg("NDSEXISTINGADMINNAME");
		$temp =~ s/\.([a-z,A-Z,0-9]*)=/,$1=/g;
		registry::setReg("LDAPTEMPADMINNAME", $temp);

		my $actual_forestnc = `/opt/novell/xad/share/dcinit/provisionTools.sh actual-forest-nc`;
		if ($? == 0) {
			#Store it in registry only if we could find the result
			chomp($actual_forestnc);
			registry::setReg("ACTUALFORESTNC", $actual_forestnc);
		}


		my $trust_posix_offset = registry::getReg("TRUSTPOSIXOFFSET");
		if ($trust_posix_offset ne "") {
			registry::setReg("ADMINISTRATOR_UID", $trust_posix_offset + 500);
			registry::setReg("DOMAIN_ADMINS_GID", $trust_posix_offset + 512);
			registry::setReg("GP_CREATOR_OWNER_GID",
				$trust_posix_offset + 520);
		}
	};

	if ($@) { Log(ERR, $@) }

}
