#!/usr/bin/perl -I. -I.. -I/opt/novell/xad/lib64/perl -I/opt/novell/xad/lib/perl

################################################################################
# Domain Services for Windows Provisioning Tool
#
# See the file LICENSE for redistribution information.
#
# Copyright (c) 2001-2009
#	Novell, Inc.  All rights reserved.
################################################################################
#
#   NAME:
#         frd_install.pm
#
#   FACILITY:
#         FRD Installation steps
#
#   ABSTRACT:
#
#   REVISION: $Id$
#
#################################################################################

package frd_install;

use NDS::stage;
use NDS::replops;
use NDS::import;
use XAD::registry;
use XAD::dns;
use XAD::secure;
use Locale::gettext;
use File::Copy;
use File::Path qw(mkpath);
use Logger qw(:DEFAULT);
use util;
use XAD::nds;
use File::Glob ':glob';

our @ISA = qw(stage dns nds replops import secure);
#use base qw(NDS::stage);

my $self = {};
my $myLocalhost = "127.0.0.1";

sub new {

	my $class = shift;
	$self = {@_};
	bless($self, $class);
    
	Logger::set_filename("/var/opt/novell/xad/log/ndsdcinit.log");
	refresh_reg();
	
	return $self;

}

sub croak { die "$0: @_: $!\n" }


#Prepare the required directory for staging
sub stage_dir {
	Log(MESG,">>>Preparing staging directory tree");

	$self->SUPER::stage_dir();

	Log(MESG,"   Staging directories are successfully created");
}


#Copies the xad.ini to the stage etc dir 
sub stage_default_config {

	#NOTE: Nothing to do here. xad.ini file is created in the ndsdcinit.pl itself.
	#This is maintained to keep the inheritance in order.
	#$self->SUPER::stage_default_config();

}


#Get the domain guid and replace it in xad.ini
sub stage_domainguid {
	Log(MESG,">>>Configuring Domain Name Server");

	$self->SUPER::stage_domainguid();

	Log(MESG,"   Domain GUID is update in the configuration file");
}


#Prepare the ntp configuration file
sub stage_ntp {
	Log(MESG,">>>Configuring Network Time");

	$self->SUPER::stage_ntp();

	Log(MESG,"   NTP configuration is updated");
}


#Prepare the openldap.conf.
#FIXME: Does it required?
sub stage_libldap {
	Log(MESG,">>>Configuring OpenLDAP");

	$self->SUPER::stage_libldap();

	Log(MESG,"   Configured OpenLDAP");
}


#Generates the rc scripts and links them
sub stage_rc{
	Log(MESG, ">>>Creating startup scripts");

	$self->SUPER::stage_rc();

	Log(MESG,"   Startup scripts successfully created");

}


#linkengine.ldif is generated
sub stage_linkengine{
	Log(MESG, ">>>Generating linkengine.ldif");

	$self->SUPER::stage_linkengine();

	Log(MESG, "   Generated linkengine.ldif");

}


#schema and schema-bl ldif files are generated here
sub stage_schema_prepare{
	Log(MESG,">>>Preparing Schema Naming Context");

	$self->SUPER::stage_schema_prepare();

	Log(MESG,"   Schema is prepared\n");
}

sub schema_aggregate {
	Log(MESG,">>>Generating Aggregated Schema\n");

	$self->SUPER::schema_aggregate();

	Log(MESG,"   msds.sch schema is Aggregated\n");
}

#Generates config.ldif, config-bl.ldif
sub stage_config {
	Log(MESG, ">>>Preparing Configuration Naming Context");

	my $templatedir = registry::getReg("TEMPLATEDIR");
	my $dsstatedir = registry::getReg("DSSTATEDIR");
	my $tempdir = registry::getReg("TEMPDIR");

	if (-e $tempdir."/stage_config") {
		Log("The stage of preparation of the configuration naming context was already done.");
		return 0;
	}
eval{
	#We do nothing here
	#$self->SUPER::stage_config();

	stage::gen_ldif("$templatedir/config.ini", '>', "$dsstatedir/config/config.ldif", "--defaultconfignc");
	stage::gen_ldif("$templatedir/config.ini", '>>', "$dsstatedir/config/config.ldif", "--defaultfirstmachine");
	stage::gen_ldif("$templatedir/config-bl.ini", '>', "$dsstatedir/config/config-bl.ldif", "--defaultconfignc");
	stage::gen_ldif("$templatedir/config-bl.ini", '>>', "$dsstatedir/config/config-bl.ldif", "--defaultfirstmachine");
	stage::gen_ldif("$templatedir/config-bl.ini", '>', "$dsstatedir/config/crossrefreference.ldif", "--ncpdn");

};
	if ($@) { Log(ERR, $@) }

	Log(MESG,"   Configuration ldif files are generated at $dsstatedir/config/{config,config-bl}.ldif\n");
	`touch $tempdir/stage_config`;
}

#Overridden from stage::
#Generate all domain related ldif files
sub stage_domain
{

	Log(MESG,">>>Preparing Domain Naming Context");

        if (registry::getReg("IS_REPLICA") eq "FALSE") {
                decide_domain_zones();
        }

	my $tempdir = registry::getReg("TEMPDIR");
	if (-e $tempdir."/stage_domain") {
		Log("The stage of preparing the domain naming context was already done.");
		return 0;
	}

	$self->SUPER::stage_domain();


	`touch $tempdir/stage_domain`;
	Log(MESG,"   Domain Naming Context is prepared successfully");

}

#Add the additional parameters to ndsd.conf
sub stage_ndsd {
	Log(MESG,">>>Staging ndsd");

	$self->SUPER::stage_ndsd();

	Log("   Successfully staged ndsd");

}

#Configure Kerberos
sub stage_krb5{
	Log(MESG, ">>>Configuring Kerberos");

	$self->SUPER::stage_krb5();

	Log(MESG, "Kerberos successfully Configured");
}

#Overridden from stage
#Configures the samba
sub stage_samba {
	Log(MESG,">>>Configuring SMB Server");
	my $tempdir = registry::getReg("TEMPDIR");

	if (-e $tempdir."/stage_samba") {
		Log("The stage of the SMB server configuration was already done.");
		return 0;
	}

	$self->SUPER::stage_samba();

	my $templatedir = registry::getReg("TEMPLATEDIR");
	my $stagedir = registry::getReg("STAGEDIR");
	my $live_dsstatedir = registry::getReg("LIVE_DSSTATEDIR");
	my $live_stagedir = registry::getReg("LIVE_STAGEDIR");

eval{
	Log("   Creating $stagedir/etc/samba/smb.conf");
	stage::reg_substitute("$templatedir/smb.conf", "$stagedir/etc/samba/smb.conf");
	util::make_netlogon_share_writable("$stagedir/etc/samba/smb.conf");

	Log("   Copying /etc/samba/smb.conf to $live_dsstatedir/backup/smb.conf.org");
	copy ("/etc/samba/smb.conf", "$live_dsstatedir/backup/smb.conf.org");

	Log("   Copying $stagedir/etc/samba/smb.conf to $live_stagedir/etc/samba/smb.conf");
	copy ("$stagedir/etc/samba/smb.conf", "$live_stagedir/etc/samba/smb.conf");

	Log(MESG,"   Successfully staged samba");
};
	if ($@) { Log(ERR, $@) }
	`touch $tempdir/stage_samba`;
}

sub stage_sysvol
{

	my $tempdir = registry::getReg("TEMPDIR");

	if (-e $tempdir."/stage_sysvol") {

		Log("The stage of the sysvol creation was already done");
		return 0;
	}

	$self->SUPER::stage_sysvol();

	`touch $tempdir/stage_sysvol`;

}

sub stage_gpo {
	Log(MESG,">>>Configuring Group Policy Objects");

	$self->SUPER::stage_gpo();

	Log(MESG,"   Group Policy Objects are successfully configured");
}

#xadsd configuration is taken care
sub stage_xadsd {
	Log(">>>Configuring Domain Services configuration");

	$self->SUPER::stage_xadsd();

	Log("   Domain Services configuration is prepared");
}

#Get the new site if it is required
sub stage_site
{

	Log(MESG, ">>>Configuring Site");

	#Not required

	Log(MESG, "   Site configuration prepared");
}

#Bring the staging directories to live
sub stage_to_live{
	Log(MESG, ">>>Bringing the stage directories to live");

	$self->SUPER::stage_to_live();

	Log(MESG, "   Stage directories are live now");
}
sub staging {
	print ">>>Starting the staging \n";
	stage_dir();
	stage_default_config(); #we need nothing here for frd
	replace_domainguid();
	stage_ntp();

	#FIXME: Does it required?
	stage_libldap();

	stage_rc();
	stage_linkengine();
	stage_schema_prepare();
	schema_aggregate();
	stage_config();
	stage_domain();
	stage_ndsd();
	stage_krb5();
	stage_samba();
	stage_sysvol();
	stage_gpo();
	stage_xadsd();
	stage_to_live();
}

sub configure_zone_object {
	Log(MESG,">>>Creating DNS Zone object");

	decide_domain_zones();

	eval{
		if (! nds::check_certificate()) {
			nds::get_certificate();
		}

	};

	if ($@) { Log(ERR, $@) }

	$self->SUPER::configure_zone_object();

	Log(MESG,"   DNS Zone objects are created");
}

sub configure_zone_rr_object {
	Log(MESG, ">>>Creating the DNS Resource Records object");

	decide_domain_zones();

	my $live_dsstatedir = registry::getReg("LIVE_DSSTATEDIR");
	my $templatedir = registry::getReg("TEMPLATEDIR");
	my $dns_master = registry::getReg("DNSHOSTNAME");
	my $dnsroot = registry::getReg("DNSROOT");
	my $ipaddress = registry::getReg("IPADDRESS");
	my $dnsserver_context = registry::getReg("DNSSERVER_CONTEXT");
	my $dns_server_ip = registry::getReg("DNS_SERVER_IP");
	my $is_replica = registry::getReg("IS_REPLICA");
	my $fwdzone_context = registry::getReg("FWDZONE_CONTEXT");

eval{
	if (-d "$live_dsstatedir/dns/RR") {
		my @list = <$live_dsstatedir/dns/RR/*.ldif>;
		foreach (@list) { unlink $_; }
	}
	else {
		mkpath("$live_dsstatedir/dns/RR");
	}

	if (-f "$live_dsstatedir/dns/RR/$dnsroot.zone") {
		Log("Removing $live_dsstatedir/dns/RR/$dnsroot.zone");
		unlink("$live_dsstatedir/dns/RR/$dnsroot.zone");
	}
	Log(MESG, "   Generating $live_dsstatedir/dns/RR/$dnsroot.zone");
	stage::reg_substitute("$templatedir/dns.zone", "$live_dsstatedir/dns/RR/$dnsroot.zone");
	`sed -i "s/{DNSMaster}/$dns_master/g"  $live_dsstatedir/dns/RR/$dnsroot.zone`;
	`sed -i "s/{DNSServerIP}/$dns_server_ip/g"  $live_dsstatedir/dns/RR/$dnsroot.zone`;

	if (-f "$live_dsstatedir/dns/RR/$ipaddress.zone") {
		Log("Removing $live_dsstatedir/dns/RR/$ipaddress.zone");
		unlink("$live_dsstatedir/dns/RR/$ipaddress.zone");
	}
	Log(MESG, "   Generating $live_dsstatedir/dns/RR/$ipaddress.zone");
	stage::reg_substitute("$templatedir/dc-reverse.zone", "$live_dsstatedir/dns/RR/$ipaddress.zone");
	`sed -i "s/{DNSMaster}/$dns_master/g"  $live_dsstatedir/dns/RR/$ipaddress.zone`;

	Log(MESG, "   Generating ldifs from $live_dsstatedir/dns/RR/$dnsroot.zone\n");
	dns::Process_DNS_Files("$live_dsstatedir/dns/RR/$dnsroot.zone", "$fwdzone_context", "$is_replica", "add");

	Log(MESG, "   Generating ldifs from $live_dsstatedir/dns/RR/$ipaddress.zone, $fwdzone_context, $is_replica, add");
	dns::Process_DNS_Files("$live_dsstatedir/dns/RR/$ipaddress.zone", "$fwdzone_context", "$is_replica", "add");


	if (! nds::check_certificate()) {
		nds::get_certificate();
	}
};
	if ($@) { Log(ERR, $@) }

	$self->SUPER::configure_zone_rr_object();

	Log(MESG, "   DNS Resource Records are created");
}

sub configure_dns{

	configure_zone_object();
	configure_zone_rr_object();

}

sub configure_nds{

	#Load samspm using ndsstat
	$self->SUPER::load_samspm();
	$self->SUPER::extend_nam_schema();
	$self->SUPER::extend_schema();

	#Configures IPC,mskrb,spnego methods and also triggers the backlinker
	$self->SUPER::configure_nmas_methods();
	$self->SUPER::configure_nldap_settings();
}

####################################################################
# Partition and replica related functions                          #
####################################################################

#Creates the domain partition.
#This will be executed only for NNM setups.
sub create_domain_nc {

	Log(MESG, ">>>Creating Domain Naming Context");

	my $mappeddomainnc = registry::getReg("MAPPEDDOMAINNC");
	my $defaultrootdomain = registry::getReg("Domain NC");

	if ($mappeddomainnc ne $defaultrootdomain) {
		Log("Skipping it since the Installation is on the existing eDirectory");
		return;
	}

	my $ipaddress = registry::getReg("IPADDRESS");
	my $ldapadminname = registry::getReg("LDAPADMINNAME");

	#Log(MESG, ">>>Waiting for 80 seconds before partitioning");
	sleep(80); #Waits 80 secs before partitioning to sync up
	$self->SUPER::create_domain_nc($ipaddress, $mappeddomainnc, $ldapadminname, "env:ADM_PASSWD");

	Log(MESG, "   Domain Naming Context created successfully");
}

#Here we add the replica of the domain partition to the local server
sub add_domain_replica
{

	my $NDSPARTITIONNAME = registry::getReg("NDSPARTITIONNAME");
	my $INSTALLMACHINENAME = registry::getReg("INSTALLMACHINENAME");

	Log(MESG, ">>>Adding Domain Replica");

	eval{

		my $ip_addres = registry::getReg("IPADDRESS");
		my $NDSTEMPCONTAINER = registry::getReg("NDSTEMPCONTAINER");
		my $MAPPEDDOMAINNC = registry::getReg("MAPPEDDOMAINNC");
		my $LDAPADMINNAME = registry::getReg("LDAPADMINNAME");
		my $XADROOT = registry::getReg("XADROOT");


		my $rc = replops::add_replica(h=>$ip_addres,p=>389,
					s=>"CN=$INSTALLMACHINENAME,$NDSTEMPCONTAINER",
					n=>"$MAPPEDDOMAINNC",
					t=>"writable",
					D=>"$LDAPADMINNAME",
					w=>$ENV{ADM_PASSWD});

		if ($rc != 0) {
			die("Could not add the domain replica");
		}
	};

	if ($@) { Log(ERR, $@) }
	Log(MESG, "   Domain Replica successfully added");
}

sub make_master_nc
{

	if (registry::getReg("MAPPED")) {

		Log(MESG, ">>>Making the local replica as the master replica");
		my $INSTALLMACHINENAME = registry::getReg("INSTALLMACHINENAME");
		my $NDSTEMPCONTAINER = registry::getReg("NDSTEMPCONTAINER");
		my $MAPPEDDOMAINNC = registry::getReg("MAPPEDDOMAINNC");
		my $LDAPADMINNAME = registry::getReg("LDAPADMINNAME");
		my $XADROOT = registry::getReg("XADROOT");

		Log("Executing.. $XADROOT/sbin/replops -h $myLocalhost -p 389 -s CN=$INSTALLMACHINENAME,$NDSTEMPCONTAINER -t master -n $MAPPEDDOMAINNC -D $LDAPADMINNAME -w env:ADM_PASSWD --change-replica-type");

		eval{

			my $rc = replops::make_master_replica(h=>"$myLocalhost",p=>389,
							s=>"CN=$INSTALLMACHINENAME,$NDSTEMPCONTAINER",
							n=>"$MAPPEDDOMAINNC",
							D=>"$LDAPADMINNAME",
							w=>"env:ADM_PASSWD");

			if ($rc != 0) {
				die("Could not Make as master replica");
			}
		};

		if ($@) { Log(ERR, $@) }
		sleep 20;
		Log(MESG, "   Local replica role changed to master");
	}

}

####################################################################
# Partition and replica related functions                          #
####################################################################
sub configure_nldap {

	$self->SUPER::configure_nldap_objects();
	$self->SUPER::refresh_nldap();
	Log("Waiting for LDAP server to respond");
	sleep(40); #Waits for 40 secs after the refresh
	$self->SUPER::validate_loaded_plugin();
}

####################################################################
# Import functions for domain/forest objects                       #
####################################################################

#Adds the configuration Partition objects
sub config_online
{

	my $config_ldif = registry::getReg("LIVE_DSSTATEDIR")."/config/config.ldif";

	$self->SUPER::config_online($config_ldif);

}

sub import_nds_domain_acl
{

	Log(MESG, ">>>Configuring default ACLs and NMAS password policies");

	$self->SUPER::import_nds_domain_acl();

	my $LDAPMODIFY = registry::getReg("LDAPMODIFY");
	my $LDAPMODIFY_FLAGS = registry::getReg("LDAPMODIFY_FLAGS");
	my $LIVE_DSSTATEDIR = registry::getReg("LIVE_DSSTATEDIR");
	my $nds_dom_acls_ldif = "$LIVE_DSSTATEDIR/domain/nds-domain-acls.ldif";
	my $nds_dom_lum_acls_ldif = "$LIVE_DSSTATEDIR/domain/nds-domain-lum-acls.ldif";
	my $rc;

eval{

	Log("Importing $LIVE_DSSTATEDIR/domain/nds-domain-acls.ldif");

	my $rc = import::import_file(host=>registry::getReg("LDAPI_URL"),
		admin=>registry::getReg("LDAPADMINNAME"),
		file=>$nds_dom_acls_ldif,
		cont_if_exist=>"true");

	if ($rc != 0) {
		die("Could not import $nds_dom_acls_ldif");
	}

	my $MAPPEDDOMAINNC = registry::getReg("MAPPEDDOMAINNC");
	my $DEFAULTROOTDOMAIN = registry::getReg("Domain NC");
};
 	if ($@) { Log(WARN, $@) }
	Log(MESG, "   Configured default ACLs and NMAS password policies successfully");

}

sub nds_import{

	$self->SUPER::domaindns_associate_online();
	$self->SUPER::domain_object_modify_online();
	$self->SUPER::domain_online();
	$self->SUPER::forest_objects_online();
	$self->SUPER::config_online();
	$self->SUPER::schema_online();

	$self->SUPER::forest_bl_online();
	$self->SUPER::domain_bl_online();
	$self->SUPER::config_bl_online();
	$self->SUPER::schema_bl_online();


	$self->SUPER::compute_effective_acls();

	import_nds_domain_acl();
#	import_nds_policies(); #Only for Name mapped scneario
	$self->SUPER::import_nds_super_rights_acls();
	$self->SUPER::import_nds_admin_acls();
	$self->SUPER::import_domain_server_reference();

#FIXME: This should move above just after creating the cn=schema and cn=configuration
	$self->SUPER::create_config_nc();
	$self->SUPER::create_schema_nc();

}

sub update_nsswitch{

	util::update_nsswitch();

}

sub restart_services{

	util::restart_services();
}

sub update_resolv_conf {

	util::update_resolv_conf();
}

sub nds_bootstrap{
	$self->SUPER::wait_for_samspm();
	$self->SUPER::bootstrap_krbtgt();
	$self->SUPER::bootstrap_machine();
	$self->SUPER::bootstrap_administrator();
}

sub migrate_nkdc_realm {

	$self->SUPER::migrate_nkdc_realm()
}

sub enable_krb5_local_lookup{

	registry::setReg("ADM_PRINCIPAL", registry::getReg("Domain Administrator Principal"));
	$self->SUPER::enable_krb5_local_lookup();
}

sub acquire_domain_ticket{

	if (not defined $ENV{ADM_PASSWD}) {
		$ENV{ADM_PASSWD} = $ENV{ADM_PASSWD_DOMAIN};
	}
	registry::setReg("ADM_PRINCIPAL", registry::getReg("Domain Administrator Principal"));

	$self->SUPER::acquire_domain_ticket();
}

sub sync_gpo_nmas{
	$self->SUPER::sync_gpo_nmas();
}

sub update_cron_tab{

	util::update_cron_tab();
}

sub delete_temporary_admin{

	my $NDSEXISTINGADMINNAME = registry::getReg("NDSEXISTINGADMINNAME");
	my $LDAPDELETE = registry::getReg("LDAPDELETE");
	my $LDAPADMINNAME = registry::getReg("LDAPADMINNAME");

	my $LDAPDELETE_FLAGS = " -Y GSS-SPNEGO";
	my $ADM_PRINCIPAL=registry::getReg("Domain Administrator Principal");
	

	if ($NDSEXISTINGADMINNAME eq "") {
		Log(MESG, ">>>Deleting the temporary administrator");
		my $rc = system("/opt/novell/xad/bin/kinit -E $ADM_PRINCIPAL");
		if($rc != 0) { 
			die("Could not get the initial credential") 
		}
		
		my $rc = system("$LDAPDELETE $LDAPDELETE_FLAGS $LDAPADMINNAME");
		if ($rc != 0) {
			$rc = $rc >> 8;
			Log(WARN, "Could not delete the temporary administrator [$rc]");
		}
		Log(MESG, "   Deletion of the temporary administrator is successful");
	}


}

sub nldap_enable_requiretls{
	if (defined $ENV{NDSEXISTINGADMINPASSWD}){
		#Name mapped first domain. They do not have ADM_PASSWD_DOMAIN
		$ENV{ADM_PASSWD_DOMAIN} = $ENV{ADM_PASSWD};
		$ENV{ADM_PASSWD} = $ENV{NDSEXISTINGADMINPASSWD};
	}

	$self->SUPER::nldap_enable_requiretls();
	if ($? != 0) {
		Log(WARN,"could not disable the option TLS for Simple Binds with Password=yes");
	}

	if (defined $ENV{NDSEXISTINGADMINPASSWD}){
		$ENV{ADM_PASSWD} = $ENV{ADM_PASSWD_DOMAIN};
	}

}

sub add_custom_objects
{
	shift;
	my $stage = shift;
	$self->SUPER::add_custom_objects($stage);
}

sub cleanup{

	delete_temporary_admin();
	$self->SUPER::stage_clean();

}

sub decide_domain_zones
{
	my $password;
	if(registry::getReg("MAPPED")) {
		$password = $ENV{NDSEXISTINGADMINPASSWD};
	}
	else {
		$password = $ENV{ADM_PASSWD};
	}
	# Calculate the forward and reverse zone contexts. 
	eval {
		my $ldap_existing_adminname = registry::getReg("LDAPADMINNAME");
		my $dnslocator_obj = registry::getReg("DNS_LOCATOR_OBJECT");
		my $forest_name = registry::getReg("Forest Root");
		my $actual_forestnc = registry::getReg("ACTUALFORESTNC");
		my $frd_default_container = "ou=OESSystemObjects";
		my $dns_server_ip = registry::getReg("DNS_SERVER_IP");
		#my $frd_provider = `/opt/novell/xad/sbin/provision -q -q -l 60 --locate-dc '$forest_name'`;
		#if ($?) {
		#	my $error = $? >> 8;
		#	die("Failed to get the forest root domain provider [$forest_name] [$error]");
		#}
		#chomp($frd_provider);

		my @dns_zones  = `LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem /opt/novell/eDirectory/bin/ldapsearch -H ldaps://$dns_server_ip:636 -D '$ldap_existing_adminname' -w '$password' -b '$dnslocator_obj' -s base dNIPDNSZones -N -LLL 2>/dev/null | sed -e ' /^ / {; H; d; }; /^ /! {; x; s/\\n //; }; ' -e  's/st=/s=/gi' | grep -i "dNIPDNSZones: "`;
		chomp(@dns_zones);
		if(@dns_zones) {
			#Set the forward zone context.
			my $fwd_zone = registry::getReg("DNS_ZONE");
			my ($fwdzone_context)  = grep {/dNIPDNSZones: cn=$fwd_zone/i} @dns_zones;
			$fwdzone_context =~ s/^dNIPDNSZones: cn=$fwd_zone,//gi;
			if($fwdzone_context ne ""){
				registry::setReg("IS_FWDZONE_EXIST", "TRUE");
				registry::setReg("FWDZONE_CONTEXT", $fwdzone_context);
			}
			else {
				Log(WARN, "Could not locate forward zone of the domain");
				registry::setReg("IS_FWDZONE_EXIST", "FALSE");
				registry::setReg("FWDZONE_CONTEXT", "$frd_default_container,$actual_forestnc");
			}
			
			#Set the reverse zone context.
			#$frd_default_container = `/opt/novell/xad/share/dcinit/provisionTools.sh get-default-container -d $forest_name`;
			#if($? ne 0) {
			#	die("Could not get FRD default container.");
			#}
			#chomp($frd_default_container);

			my $rev_zone = registry::getReg("IP_ADDRESS_UNDERSCORE_FORM");
			my ($revzone_context)  = grep {/dNIPDNSZones: cn=$rev_zone/i} @dns_zones;
			$revzone_context =~ s/^dNIPDNSZones: cn=$rev_zone,//gi;
			if($revzone_context ne ""){
				registry::setReg("IS_REVZONE_EXIST", "TRUE");
				registry::setReg("REVZONE_CONTEXT", $revzone_context);
			}
			else {
				Log(WARN, "Could not locate reverse zone for the DC");
				registry::setReg("IS_REVZONE_EXIST", "FALSE");
				registry::setReg("REVZONE_CONTEXT", "$frd_default_container,$actual_forestnc");
			}
		}
		else {
			die("Failed to fetch dNIPDNSZones from DNS_LOCATOR_OBJECT");
		}

		# Set the REVZONE_DNSMASTER for the reverse zone.
		# We set designated dns server of FRD zone as the REVZONE_DNSMASTER.
		# Here setting the FWDZONE_DNSMASTER for the forward zone is not required as the zone is expected to be exist.
		my $frd_zone = $forest_name;
		my $fwd_zone_context = registry::getReg("FWDZONE_CONTEXT");
		$frd_zone =~ s/\./_/g;
		my $frd_designated_dns = `/opt/novell/eDirectory/bin/ldapsearch -D '$ldap_existing_adminname' -w '$password' -h $dns_server_ip -e /etc/opt/novell/certs/SSCert.der -p 636 -b 'cn=$frd_zone,$fwd_zone_context' -s base dNIPDesignatedServer -N -LLL 2>/dev/null | sed ' /^ / {; H; d; }; /^ /! {; x; s/\\n //; }; ' | grep -i "dNIPDesignatedServer: " | awk -F "dNIPDesignatedServer: " '{print \$2}'`;
		chomp($frd_designated_dns);
		if ($frd_designated_dns eq "") {
			Log(WARN, "Could not fetch the FRD zone designated name server DN, assigning the local machine as the master. After the provisioning please change the name-server entry in the SOA record of this domain's zone");
			registry::setReg("REVZONE_DNSMASTER", registry::getReg("DNSHOSTNAME"));
		}
		else {
			my $revzone_dnsmaster = `/opt/novell/eDirectory/bin/ldapsearch -D '$ldap_existing_adminname' -w '$password' -h $dns_server_ip -e /etc/opt/novell/certs/SSCert.der -p 636 -b "$frd_designated_dns" -s base dNIPServerDNSNames -LLL | sed ' /^ / {; H; d; }; /^ /! {; x; s/\\n //; }; ' | grep "dNIPServerDNSNames: " | awk -F "dNIPServerDNSNames: " '{print \$2}'`;
			chomp($revzone_dnsmaster);
			if ($revzone_dnsmaster eq "") {
				Log(WARN, "Could not fetch the FRD zone name server dns name, assigning the local machine as the master. After the provisioning please change the name-server entry in the SOA record of this domain's zone");
				registry::setReg("REVZONE_DNSMASTER", registry::getReg("DNSHOSTNAME"));
			}
			else {
				registry::setReg("REVZONE_DNSMASTER", $revzone_dnsmaster);
			}
		}
		Log("The reverse zone dns master is ".registry::getReg("REVZONE_DNSMASTER"));

	};

	if ($@) { Log(ERR, $@) }

}

sub refresh_reg
{

	if (registry::getReg("IS_REPLICA") eq "TRUE") {
		#Additional DC does not inherits this method
		#We should not calculate the below variable when its Additional DC
		return;
	}

	registry::clearReg();
	registry::reg_init();
	registry::setReg("ADMINISTRATOR_UID", 1049076);
	registry::setReg("DOMAIN_ADMINS_GID", 1049088);

	#Common for all except ADC
	registry::setReg("NDSSERVERCONTEXT", registry::getReg("NDS Server Context"));


	my $NDSEXISTINGADMINNAME = registry::getReg("NDSEXISTINGADMINNAME");
	if ($NDSEXISTINGADMINNAME eq "") {
		registry::setReg("NDSADMINNAME", "CN=XAD_PROVISIONING_USER.".registry::getReg("NDSSERVERCONTEXT"));
		registry::setReg("LDAPADMINNAME", "CN=XAD_PROVISIONING_USER,".registry::getReg("Domain NC"));
	}
	else {
		registry::setReg("NDSADMINNAME",  $NDSEXISTINGADMINNAME);
		my $temp = $NDSEXISTINGADMINNAME;
		$temp =~ s/\.([a-z,A-Z,0-9]*)=/,$1=/g;
		registry::setReg("LDAPADMINNAME", $temp);
	}
	registry::setReg("LDAPCOMMON_FLAGS", "-H ".registry::getReg("LDAPI_URL")." -x -D '".registry::getReg("LDAPADMINNAME")."' -w '$ENV{'ADM_PASSWD'}'");
	registry::compute_pre_install_variables();

	# Write the NDSADMINNAME and LDAPADMINNAME in the xad.ini file.
	my $DCINIT_CONFIG = "/etc/opt/novell/xad/xad.ini";
	open(CONFIG,">>$DCINIT_CONFIG") || croak("could not open $DCINIT_CONFIG\n");

	`grep NDSADMINNAME $DCINIT_CONFIG`;
	if ($?) {
		my $ndsadminname = registry::getReg("NDSADMINNAME");
		print CONFIG "NDSADMINNAME = $ndsadminname\n";
	}

	`grep LDAPADMINNAME $DCINIT_CONFIG`;
	if ($?) {
		my $ldapadminname = registry::getReg("LDAPADMINNAME");
		print CONFIG "LDAPADMINNAME = $ldapadminname\n";
	}

	close(CONFIG);

#	registry::setReg("LDAPTEMPADMINNAME", "CN=Administrator,CN=Users,".registry::getReg("MAPPEDDOMAINNC"));
#	registry::setReg("NDSTEMPADMINNAME", "CN=Administrator.CN=Users.".registry::getReg("NDSADMINSERVERCONTEXT"));
	registry::setReg("ACTUALFORESTNC", registry::getReg("MAPPEDDOMAINNC"));

	# Set the FWDZONE_DNSMASTER for the forward zone.
	registry::setReg("FWDZONE_DNSMASTER", registry::getReg("DNSHOSTNAME"));
	
	# Set the REVZONE_DNSMASTER for the reverse zone.
	registry::setReg("REVZONE_DNSMASTER", registry::getReg("DNSHOSTNAME"));
}

sub start_install {
	$self = shift; #gets the instance of calling object

	print ">>> Refreshing the registry\n";
	refresh_reg();
	print ">>> starting frd Installation\n";

	staging();
	configure_dns();
	configure_nds();
	configure_partition();
	configure_nldap();
	nds_import();

	update_nsswitch();
	restart_services();
	nds_bootstrap();
	enable_krb5_local_lookup();
	acquire_domain_ticket();
	sync_gpo_nmas();
	update_cron_tab();
	cleanup();
}
