#!/usr/bin/perl -I. -I.. -I/opt/novell/xad/lib64/perl -I/opt/novell/xad/lib/perl

################################################################################
# Domain Services for Windows Provisioning Tool
#
# See the file LICENSE for redistribution information.
#
# Copyright (c) 2001-2009
#	Novell, Inc.  All rights reserved.
################################################################################
#
#   NAME:
#         dns.pm
#
#   FACILITY:
#         Provides all the dns related work
#
#   ABSTRACT:
#
#
#   REVISION: $Id$
#
#################################################################################

package import;

use v5.10; # at least for Perl 5.10
no warnings 'experimental::smartmatch'; # Avoid known warnings with given-when

use Logger qw(:DEFAULT);
use XAD::registry;
use XAD::nds;
use Net::LDAP;
use Net::LDAPS;
use Net::LDAP::Util qw(ldap_error_text
                         ldap_error_name
                         ldap_error_desc
                        );                                              #SSL bind 
use Net::LDAP::LDIF;
use Net::LDAP::Control;
use Net::LDAP::Constant qw( LDAP_CONTROL_MATCHEDVALS );
use util;
use util_update;
use MIME::Base64 qw(encode_base64 decode_base64);
use dsfw_ldif;
my $myLocalhost = "127.0.0.1";

sub connectandbind
{
	my %args = @_;
	my $host  = $args{host} || "$myLocalhost";
	my $port  = $args{port} || 389;
	my $admin = $args{admin}|| registry::getReg("LDAPADMINNAME");
	my $pwd   = $args{pwd}  || $ENV{ADM_PASSWD};
	my $cert  = $args{cert} || "";
	my $urix  = $args{uri} || 0;
	my $ssl = $args{ssl} || "no";
	my $scheme = lc($ssl) eq "yes"? "ldaps":"ldap";
	my $ld;

	Log("connect and bind.. $host, $port, $admin, $cert");

	eval {
		$ld = Net::LDAP->new($host, scheme=>$scheme, port=>$port, timeout=>120) or die("Could not create connection");
		#Added for certificate 
		if($cert ne ""){
			my $mesg = $ld->start_tls(verify=>'require', sslversion=>'SSLv23:!SSLv2:!SSLv3', cafile=>$cert);
			(not $urix) && $mesg->code && die $mesg->error;
		}

		my $msg = $ld->bind($admin, password=>$pwd);
		$msg->code && die $msg->error;

		if (ldap_error_name($msg) ne "LDAP_SUCCESS") {
			die(ldap_error_text($msg));
		}
	};

	if($@){
		Log(INFO, $@);
		return 0;
	}

	return $ld;
}

sub import_file_old
{
	my %args = @_;
	my $file  = $args{file} || die("No file name is mentioned for import");
	my $cont_if_exist = $args{cont_if_exist} || "false";
	my $edir_schematic_cntl = $args{edir_schematic_cntl} || "false";
	my $ld = connectandbind(@_);

	$cont_if_exist = lc($cont_if_exist);
	$edir_schematic_cntl = lc($edir_schematic_cntl);

	my $ldif = Net::LDAP::LDIF->new( $file, "r", onerror => 'warn' );
	my ($entry, $msg, $cntl);

	if ($edir_schematic_cntl eq "true") {
		 $cntl = Net::LDAP::Control->new(
		   type     => "2.16.840.1.113719.1.513.4.5",
		   value    => "eDirectory semantics control",
		   critical => 0
		 );
	}

	while( not $ldif->eof ( ) ) {
		$entry = $ldif->read_entry ( );
		if ( $ldif->error ( ) ) {
			Log(WARN, $ldif->error ());
			Log(WARN, $ldif->error_lines());
			return -1; 
		} else {

			if ($edir_schematic_cntl eq "false") {
				$msg=$entry->update($ld);
			}
			else {
				$msg=$entry->update($ld, control=>$cntl);
			}
			if (ldap_error_name($msg) ne "LDAP_SUCCESS") {
				Log(WARN, "Could not process the entry ");
				Log(WARN, $ldif->current_lines());
				Log(WARN, ldap_error_name($msg).":".ldap_error_text($msg));

                                if ($cont_if_exist eq "true" and (ldap_error_name($msg) eq "LDAP_ALREADY_EXISTS"
                                                                or ldap_error_name($msg) eq "LDAP_TYPE_OR_VALUE_EXISTS")) {
                                        Log(INFO, "proceeding with the next entry");
                                }
                                else {
                                        return -1;
                                }
                        }
                        else {
                                Log("Processing entry ".$entry->dn());
                        }

		}
	}
        $ld->unbind();
        $ld->disconnect();
	$ldif->done();
	return 0;
}

sub ndsimport_file
{

	my ($host, $port, $admin, $pwd , $cert, $file) = (@_);

	Log("Executing../opt/novell/eDirectory/bin/ldapmodify -h $host -x -D $admin -e /etc/opt/novell/certs/SSCert.der -p 636 -W -a -f $file");
	my $rc = system("/opt/novell/eDirectory/bin/ldapmodify -h $host -x -D $admin -e /etc/opt/novell/certs/SSCert.der -p 636 -w $pwd -a -f $file");

	return $rc;
}


#Associates domainDNS aux class with the domain partition object
sub domaindns_associate_online
{

	my $ldapmodify = registry::getReg("LDAPMODIFY");
	my $ldapmodifyflags = registry::getReg("LDAPMODIFY_FLAGS");
	my $live_dsstatedir = registry::getReg("LIVE_DSSTATEDIR");
        #Added for ldif modification
        my $ldapadminname = registry::getReg("LDAPADMINNAME");
        my $domaindns_assoct_ldif="$live_dsstatedir/domain/domaindns-associate.ldif";

	Log(MESG,">>>Associating DomainDNS object class with domain object");

	#To sync the schema, limber is triggered
	nds::trigger_limber();

	eval{
		#Added for ldif files
		Log(MESG, "Importing $domaindns_assoct_ldif");
		my $rc = import_file(host=>registry::getReg("LDAPI_URL"),
			admin=>$ldapadminname,
			file=>$domaindns_assoct_ldif,
			cont_if_exist=>"true");
		if ($rc != 0) {
			die("Could not import $domaindns_assoct_ldif");
		}
	};

	if ($@) { Log(ERR, $@) }

	Log(MESG, "   DomainDNS object class is associated with domain object successfully");
}

#Updates the domain container objects
sub domain_object_modify_online
{

	my $NATIVE_REPL = registry::getReg("NATIVE_REPL");
	my $LDAPMODIFY = registry::getReg("LDAPMODIFY");
	my $LDAPMODIFY_FLAGS = registry::getReg("LDAPMODIFY_FLAGS");
	my $LIVE_DSSTATEDIR = registry::getReg("LIVE_DSSTATEDIR");
        #Added for ldif modification
        my $ldapadminname = registry::getReg("LDAPADMINNAME");
        my $domainobj_mod_ldif = "$LIVE_DSSTATEDIR/domain/domain-object-modify.ldif";
        my $domainobj_mod_lockpol_ldif = "$LIVE_DSSTATEDIR/domain/domain-object-modify-lockout-policy.ldif";


	Log(MESG, ">>> Modifying Domain Object");

	eval{

		#@if [ "X$(NATIVE_REPL)" = "XTRUE" ]; FIXME: Need to check this variable
		Log(MESG, "Importing $domainobj_mod_ldif");

		#Added for ldif modification
		my $rc = import_file(host=>registry::getReg("LDAPI_URL"),
			admin=>$ldapadminname,
			file=>$domainobj_mod_ldif,
			cont_if_exist=>"true");

		if ($rc != 0) {
			die("Could not import $domainobj_mod_ldif");
		}

		if ( -s "$LIVE_DSSTATEDIR/domain/domain-object-modify-lockout-policy.ldif"){

			Log(MESG, "Importing $domainobj_mod_lockpol_ldif");
			my $rc = import_file(host=>registry::getReg("LDAPI_URL"),
				admin=>$ldapadminname,
				file=>$domainobj_mod_lockpol_ldif,
				cont_if_exist=>"true");
			if ($rc != 0) {
				die("Could not import $domainobj_mod_lockpol_ldif");
			}
		}
	};

	if ($@) { Log(ERR, $@) }

	Log(MESG, "   Domain Objects are modified successfully");

}



#Adds objects under domain partitioning
sub domain_online
{

	my $LDAPMODIFY = registry::getReg("LDAPMODIFY");
	my $LDAPMODIFY_FLAGS = registry::getReg("LDAPMODIFY_FLAGS");
	my $LDAPADD_FLAGS = registry::getReg("LDAPADD_FLAGS");
	my $LIVE_DSSTATEDIR = registry::getReg("LIVE_DSSTATEDIR");
	my $NATIVE_REPL = registry::getReg("NATIVE_REPL");
        #Added for ldif modification
        my $ldapadminname = registry::getReg("LDAPADMINNAME");
        my $admin_delete_ldif = "$LIVE_DSSTATEDIR/domain/administrator-delete.ldif";
        my $domain_ldif = "$LIVE_DSSTATEDIR/domain/domain.ldif";
	my $ldap_no_such_attribute = 16;


	Log(MESG, ">>> Importing Domain Naming Context into LDAP Server");

	eval{
		#
		# Perl module seems to fail for administrator-delete.ldif. Temporarily switching to ldapmodify
		#
		Log("Modifying Administrator");
		my $error_info = `$LDAPMODIFY $LDAPMODIFY_FLAGS -f $LIVE_DSSTATEDIR/domain/administrator-delete.ldif 2>&1`;
		my $error_value = ($rc>>8);
		if($error_value != $ldap_no_such_attribute){
			Log(INFO, "Could not import $LIVE_DSSTATEDIR/domain/administrator-delete.ldif, error returned $error_value:[$error_info]");
		}

		#Added for ldif modification
		Log(MESG, "Importing $domain_ldif");

		if( $NATIVE_REPL eq "TRUE" ) {
			#Added for ldif modification
			my $rc = import_file(host=>registry::getReg("LDAPI_URL"),
				admin=>$ldapadminname,
				file=>$domain_ldif,
				cont_if_exist=>"true");
			if($rc != 0) {
				die("Could not import $domain_ldif");
			}
		}
		else{
			my $rc = import_file(host=>registry::getReg("LDAPI_URL"),
				admin=>$ldapadminname,
				file=>$domain_ldif,
				cont_if_exist=>"true");
			if($rc != 0) {
				die("Could not import $domain_ldif");
			}
		}

	};

	if ($@) {
		Log(ERR, $@);
	}
	Log(MESG, "   Domain Naming Context is imported into LDAP Server successfully");
}

sub add_custom_objects
{
	$LDAPMODIFY = registry::getReg("LDAPMODIFY");
	my $LIVE_DSSTATEDIR = registry::getReg("LIVE_DSSTATEDIR");
	my $LDAPMODIFY_FLAGS = registry::getReg("LDAPMODIFY_FLAGS");
	my $LDAPADD_FLAGS = registry::getReg("LDAPADD_FLAGS");
	my $dsstatedir = registry::getReg("DSSTATEDIR");
	my $ldapadminname = registry::getReg("LDAPADMINNAME");
	my $ldap_errmsg = "Undefined attribute type";
        #Added for ldif modification
        my $crossref_ldif = "$LIVE_DSSTATEDIR/config/crossrefreference.ldif";
	shift;
	my $stage = shift;

	eval {
		if ($stage eq "stage1") {
			if (-e "$LIVE_DSSTATEDIR/config/crossrefreference.ldif") {
				Log(MESG, "Creating crossref reference to ncp server object");
				#Added for ldif modification
				Log(MESG, "Importing $crossref_ldif");

				my $rc = import_file(host=>registry::getReg("LDAPI_URL"),
					admin=>$ldapadminname,
					file=>$crossref_ldif,
					cont_if_exist=>"true");

				if($rc != 0) {
					die("Could not import $crossref_ldif");
				}

				#my $rc = `$cmdLdap 2>&1`;
				#my $cmdStatus = $?;
				#chomp $rc;
				#if(!grep(/$ldap_errmsg/, $rc) && ($cmdStatus>>8)!=0 ){
				#        die("Could not execute ldapmodify for $dsstatedir/config/crossrefreference.ldif [$rc]");
				#}
			}
		}

		if (-e "$LIVE_DSSTATEDIR/domain/custom_domain_boundary.ldif") {
		# splitting this file for now as it is failig with -614
		# a retry works!
			my $file1 = `/bin/mktemp`;
			my $line;

			open INFILE, "<$LIVE_DSSTATEDIR/domain/custom_domain_boundary.ldif";
			open OUTFILE1, ">$file1" ;
	
			if ($stage eq "stage1") {
				while( $line = <INFILE> ) {
					print OUTFILE1 $line;
					if( $line =~ /^$/ ) {
						last;
					}
				}
				close( OUTFILE1 );

		    		Log(MESG, "   Creating customized domain name space");
				#Added for ldif modification
				Log(MESG, "Importing $file1");
				my $rc = import_file(admin=>$ldapadminname,
					pwd=>$ENV{'ADM_PASSWD'},
					file=>$file1,
					cont_if_exist=>"true");
				unlink( $file1 );
				if( $rc != 0 ) {
					die("Could not import stage 1 $file1");
				}

			} elsif ($stage eq "stage2") {

				while( $line = <INFILE> ) {
					if( $line =~ /^$/ ) {
						last;
					}
				}

				while( $line = <INFILE> ) {
					print OUTFILE1 $line;
				}

				close( OUTFILE1 );
				
				my $counter = 0;
				my $rc = 0;
				do {
					#Added for ldif modification
					Log(MESG, "Importing $file1");
					my $rc = import_file(admin=>$ldapadminname,
						pwd=>$ENV{'ADM_PASSWD'},
						file=>$file1,
						cont_if_exist=>"true");
					if($rc != 0) {
						Log(MESG, "Could not import stage 2 $file1 - Retrying");
					}
					sleep(5);
					$counter++;
				} while( $rc != 0 && $counter <= 24);
# Ignore this return status
				$rc = 0; 
			}
		}
	};
 	if ($@) { Log(ERR, $@) }

}

#Creates the configuration and schema container
sub add_partition_container
{
	my $file = registry::getReg("LIVE_DSSTATEDIR")."/domain/partition-container.ldif";

	Log(MESG, ">>>Creating Configuration schema containers");
	eval{
		my $rc = import_file(file=>$file, cont_if_exist=>"true");
		if ($rc != 0) {
			die("Could not import the ldif file [$file]");
		}
	};
 	if ($@) { Log(ERR, $@) }
	Log(MESG, "   Configuration schema containers are created");

}

#Adds the forest wide objects under the configuration partition
sub forest_objects_online
{

	my $LDAPMODIFY = registry::getReg("LDAPMODIFY");
	my $LDAPMODIFY_FLAGS = registry::getReg("LDAPMODIFY_FLAGS");
	my $LDAPADD_FLAGS = registry::getReg("LDAPADD_FLAGS");
	my $LIVE_DSSTATEDIR = registry::getReg("LIVE_DSSTATEDIR");
	my $NATIVE_REPL = registry::getReg("NATIVE_REPL");
        #Added for ldif modifications
        my $ldapadminname = registry::getReg("LDAPADMINNAME");
        my $forest_ldif = "$LIVE_DSSTATEDIR/domain/forest.ldif";

	Log(MESG, ">>>Importing Forest Naming Context objects into LDAP Server");

eval{

	Log("Importing $LIVE_DSSTATEDIR/domain/forest.ldif");

	if ($NATIVE_REPL eq "TRUE" ){
		#Added for ldif modification
		my $rc=import_file(host=>registry::getReg("LDAPI_URL"),
			admin=>$ldapadminname,
			file=>$forest_ldif,
			cont_if_exist=>"true");
		if ($rc != 0) {
			die("Could not import $forest_ldif");
		}
	}
	else {
		#Added for ldif modification
		my $rc=import_file(host=>registry::getReg("LDAPI_URL"),
			admin=>$ldapadminname,
			file=>$forest_ldif,
			cont_if_exist=>"true");
		if ($rc != 0) {
			die("Could not import $forest_ldif");
		}
	}
};
 	if ($@) { Log(ERR, $@) }
	Log(MESG, "   Forest Naming Context objects are imported successfully into LDAP Server");
}

#Adds objects under configuration partition
sub config_online
{

	my $self = shift; #Gets the object instance

	#A frd installation would pass config.ldif and child installation would pass config-addl.ldif....
	my $file = shift;

	my $LDAPMODIFY = registry::getReg("LDAPMODIFY");
	my $LDAPMODIFY_FLAGS = registry::getReg("LDAPMODIFY_FLAGS");
	my $LDAPADD_FLAGS = registry::getReg("LDAPADD_FLAGS");

	my $NATIVE_REPL = registry::getReg("NATIVE_REPL");
        #Added for ldif modifications
        my $ldapadminname = registry::getReg("LDAPADMINNAME");


	Log(MESG, ">>>Importing Configuration Naming Context into LDAP Server");

	eval{

		Log("Importing $file");

		if ($NATIVE_REPL eq "TRUE" ) {
			#Added for ldif modification
			my $rc = import_file(host=>registry::getReg("LDAPI_URL"),
				admin=>$ldapadminname,
				file=>$file,
				cont_if_exist=>"true");

			if ($rc != 0) {
				die("Could not import $file");
			}
		}
		else {
			#Added for ldif modification
			Log("Executing ldapadd for $file");
			my $rc = import_file(host=>registry::getReg("LDAPI_URL"),
				admin=>$ldapadminname,
				file=>$file,
				cont_if_exist=>"true");

			if ($rc != 0) {
				die("Could not import $file");
			}
		}
	};

 	if ($@) { Log(ERR, $@) }

	Log(MESG, "   Configuration Naming Context objects are imported successfully into LDAP Server");
}


#Adds AD schema under schema partition
sub schema_online
{

	my $LDAPMODIFY = registry::getReg("LDAPMODIFY");
	my $LDAPMODIFY_FLAGS = registry::getReg("LDAPMODIFY_FLAGS");
	my $LDAPADD_FLAGS = registry::getReg("LDAPADD_FLAGS");
	my $LIVE_DSSTATEDIR = registry::getReg("LIVE_DSSTATEDIR");
	my $NATIVE_REPL = registry::getReg("NATIVE_REPL");
        #Added for ldif modification
        my $ldapadminname = registry::getReg("LDAPADMINNAME");
        my $schema_ldif="$LIVE_DSSTATEDIR/schema/schema.ldif";

	Log(MESG, ">>>Importing Schema Naming Context into LDAP Server");

eval{

	Log(MESG, "Importing $schema_ldif");

	if ($NATIVE_REPL eq "TRUE" ){
		#Added for ldif modification
		my $rc = import_file(host=>registry::getReg("LDAPI_URL"),
			admin=>$ldapadminname,
			file=>$schema_ldif,
			cont_if_exist=>"true");
		if ($rc != 0) {
			#die("Could not execute ldapmodify for $LIVE_DSSTATEDIR/schema/schema.ldif");
			die("Could not import $schema_ldif");
		}

	}
	else {
		#Added for ldif modification
		my $rc = import_file(host=>registry::getReg("LDAPI_URL"),
			admin=>$ldapadminname,
			file=>$schema_ldif,
			cont_if_exist=>"true");
		if ($rc != 0) {
			die("Could not import $schema_ldif");
		}
	}
};
 	if ($@) { Log(ERR, $@) }
	Log(MESG, "   Schema Naming Context objects are imported successfully into LDAP Server");

}

#Import the back linked schema objects to the server
sub schema_bl_online
{
	my $LDAPMODIFY = registry::getReg("LDAPMODIFY");
	my $LDAPMODIFY_FLAGS = registry::getReg("LDAPMODIFY_FLAGS");
	my $LIVE_DSSTATEDIR = registry::getReg("LIVE_DSSTATEDIR");
        #Added for ldif modifications
        my $ldapadminname = registry::getReg("LDAPADMINNAME");
        my $schema_bl_ldif = "$LIVE_DSSTATEDIR/schema/schema-bl.ldif";


	Log(">>>Importing Schema Naming Context (Back-Links) into LDAP Server");

eval{

	#Added for ldif modification
	Log(MESG, "Importing $schema_bl_ldif");

	my $rc = import_file(host=>registry::getReg("LDAPI_URL"),
		admin=>$ldapadminname,
		file=>$schema_bl_ldif,
		cont_if_exist=>"true");
	if ($rc != 0) {
		die("Could not import $schema_bl_ldif");
	}

};
 	if ($@) { Log(ERR, $@) }
	Log(MESG, "   Schema Naming Context (Back-Links) are imported successfully into LDAP Server");

}

#Imports configuration back link objects to the server
sub config_bl_online
{
	my $LDAPMODIFY = registry::getReg("LDAPMODIFY");
	my $LDAPMODIFY_FLAGS = registry::getReg("LDAPMODIFY_FLAGS");
	my $LIVE_DSSTATEDIR = registry::getReg("LIVE_DSSTATEDIR");
        #Added for ldif modification
        my $ldapadminname = registry::getReg("LDAPADMINNAME");
        my $config_bl_ldif = "$LIVE_DSSTATEDIR/config/config-bl.ldif";


	Log(MESG, ">>>Importing Configuration Naming Context (Back-Links) into LDAP Server");

eval{

	Log(MESG, "Importing $config_bl_ldif");
	my $rc = import_file(host=>registry::getReg("LDAPI_URL"),
		admin=>$ldapadminname,
		file=>$config_bl_ldif,
		cont_if_exist=>"true");

	if ($rc != 0) {
		die("Could not import $config_bl_ldif");
	}

};
 	if ($@) { Log(ERR, $@) }
	Log(MESG, "   Configuration Naming Context (Back-Links) are imported successfully into LDAP Server");

}


#Back linked domain objects are added to the server
sub domain_bl_online
{

	my $ldap_modify = registry::getReg("LDAPMODIFY");
	my $ldap_modify_flags = registry::getReg("LDAPMODIFY_FLAGS");
	my $domain_bl_ldif = registry::getReg("LIVE_DSSTATEDIR")."/domain/domain-bl.ldif";
        #Added for ldif modification
        my $ldapadminname = registry::getReg("LDAPADMINNAME");

	if (-f $domain_bl_ldif){

		Log(MESG, ">>>Importing Domain Naming Context (Back-Links) into LDAP Server");

		eval{
			Log("Importing $domain_bl_ldif");
			my $rc = import_file(host=>registry::getReg("LDAPI_URL"),
				admin=>$ldapadminname,
				file=>$domain_bl_ldif,
				cont_if_exist=>"true");

			if ($rc != 0) {
				die("Could not import $domain_bl_ldif");
			}

		};

	 	if ($@) { Log(ERR, $@) }
		Log(MESG, "   Domain Naming Context (Back-Links) are imported successfully into LDAP Server");
	}

}

#Back linked forest objects are added to the server
sub forest_bl_online 
{
	my $LDAPMODIFY = registry::getReg("LDAPMODIFY");
	my $LDAPMODIFY_FLAGS = registry::getReg("LDAPMODIFY_FLAGS");
	my $LIVE_DSSTATEDIR = registry::getReg("LIVE_DSSTATEDIR");
        #Added for ldif modification
        my $ldapadminname = registry::getReg("LDAPADMINNAME");
        my $forest_bl_ldif = "$LIVE_DSSTATEDIR/domain/forest-bl.ldif";

	if (-f "$LIVE_DSSTATEDIR/domain/forest-bl.ldif"){

		Log(MESG, ">>>Importing Forest Naming Context (Back-Links) into LDAP Server");

		eval{
			#Added for ldif modification
			Log(MESG, "Importing $forest_bl_ldif");

			my $rc = import_file(host=>registry::getReg("LDAPI_URL"),
				admin=>$ldapadminname,
				file=>$forest_bl_ldif,
				cont_if_exist=>"true");

			if ($rc != 0) {
				die("Could not import $forest_bl_ldif");
			}

		};

	 	if ($@) { Log(ERR, $@) }
		Log(MESG, "   Forest Naming Context (Back-Links) are imported successfully into LDAP Server");
	}
}


sub import_nds_domain_acl
{

	my $ldap_modify = registry::getReg("LDAPMODIFY");
	my $ldap_modify_flags = registry::getReg("LDAPMODIFY_FLAGS");
	my $live_dsstatedir = registry::getReg("LIVE_DSSTATEDIR");
	my $rc;
        #Added for ldif modification
        my $nds_dom_ldif = "$live_dsstatedir/domain/nds-domain.ldif";
        my $comp_cont_polc_ldif = "$live_dsstatedir/domain/computer-container-policy.ldif";

	eval{
		#Added for ldif modification
		Log(MESG, "Importing $nds_dom_ldif");

		my $rc = import_file(host=>registry::getReg("LDAPI_URL"),
			admin=>registry::getReg("LDAPADMINNAME"),
			file=>$nds_dom_ldif,
			cont_if_exist=>"true");

		if ($rc != 0) {
			die("Could not import $nds_dom_ldif");
		}

		#FIXME: Check the variable is_replica == FALSE
		Log(MESG, "Importing $comp_cont_polc_ldif");

		my $rc = import_file(host=>registry::getReg("LDAPI_URL"),
			admin=>registry::getReg("LDAPADMINNAME"),
			file=>$comp_cont_polc_ldif,
			cont_if_exist=>"true");

		if ($rc != 0) {
			die("Could not import $comp_cont_polc_ldif");
		}

	};

 	if ($@) { Log(ERR, $@) }

}


sub import_nds_super_rights_acls
{

	my $LDAPMODIFY = registry::getReg("LDAPMODIFY");
	my $LDAPMODIFY_FLAGS = registry::getReg("LDAPMODIFY_FLAGS");
	my $LIVE_DSSTATEDIR = registry::getReg("LIVE_DSSTATEDIR");
        #Added for ldif modifications
        my $nds_surght_acls_ldif = "$LIVE_DSSTATEDIR/domain/nds-super-rights-acls.ldif";
        my $nds_domrght_acls_ldif = "$LIVE_DSSTATEDIR/domain/nds-domain-rights-acls.ldif";


eval{
	if (! registry::getReg("MAPPED")) {

		Log(MESG,">>>Configuring supervisor privileges");
		#Added for ldif modification
		Log(MESG, "Importing $nds_surght_acls_ldif");

		my $rc = import_file(host=>registry::getReg("LDAPI_URL"),
			admin=>registry::getReg("LDAPADMINNAME"),
			file=>$nds_surght_acls_ldif,
			cont_if_exist=>"true");

		if ($rc != 0) {
			die("Could not import $nds_surght_acls_ldif");
		}

	}
	else {
		Log(MESG, ">>>Configuring domain container privileges");
		#Added for ldif modification
		Log(MESG, "Importing $nds_domrght_acls_ldif");

		my $rc = import_file(host=>registry::getReg("LDAPI_URL"),
			admin=>registry::getReg("LDAPADMINNAME"),
			file=>$nds_domrght_acls_ldif,
			cont_if_exist=>"true");

		if ($rc != 0) {
			die("Could not import $nds_domright_acls_ldif");
		}

	}

};

 	if ($@) { Log(ERR, $@) }

	Log(MESG, "   Configured the privileges successfully");
}

#Configure default ACL for Schema and configuration partition
sub import_nds_admin_acls
{

	Log(MESG,">>>Configuring default ACLs for Configuration and Schema partitions");

	my $LDAPMODIFY = registry::getReg("LDAPMODIFY");
	my $LDAPMODIFY_FLAGS = registry::getReg("LDAPMODIFY_FLAGS");
	my $LIVE_DSSTATEDIR = registry::getReg("LIVE_DSSTATEDIR");
        #Added for ldif modifications
        my $nds_admin_acls_ldif = "$LIVE_DSSTATEDIR/domain/nds-admin-acls.ldif";


	eval{
		#Added for ldif modification
		Log(MESG, "Importing $nds_admin_acls_ldif");

		my $rc = import_file(host=>registry::getReg("LDAPI_URL"),
			admin=>registry::getReg("LDAPADMINNAME"),
			file=>$nds_admin_acls_ldif,
			cont_if_exist=>"true");

		if ($rc != 0) {
			die("could not import $nds_admin_acls_ldif");
		}
	};

 	if ($@) { Log(ERR, $@) }

	Log(MESG, "   Configured the default ACLs successfully");
}


sub import_domain_server_reference
{

	my $LDAPMODIFY = registry::getReg("LDAPMODIFY");
	my $LDAPMODIFY_FLAGS = registry::getReg("LDAPMODIFY_FLAGS");
	my $LIVE_DSSTATEDIR = registry::getReg("LIVE_DSSTATEDIR");
        #Added for ldif modifications
        my $nds_dom_srv_ref_ldif = "$LIVE_DSSTATEDIR/domain/nds-domain-server-reference.ldif";
        my $nds_dom_cont_ldif = "$LIVE_DSSTATEDIR/domain/nds-domain-container.ldif";


	Log(MESG,">>>Relating NCP Server object and AD Server object");

	eval{
		#Added for ldif modification
		Log(MESG, "Importing $nds_dom_srv_ref_ldif");

		my $rc = import_file(host=>registry::getReg("LDAPI_URL"),
			admin=>registry::getReg("LDAPADMINNAME"),
			file=>$nds_dom_srv_ref_ldif,
			cont_if_exist=>"true");
		if ($rc != 0) {
			die("Could not import $nds_dom_srv_ref_ldif");
		}

		Log(MESG, "Importing $nds_dom_cont_ldif");

		my $rc = import_file(host=>registry::getReg("LDAPI_URL"),
			admin=>registry::getReg("LDAPADMINNAME"),
			file=>$nds_dom_cont_ldif,
			cont_if_exist=>"true");
		if ($rc != 0) {
			die("Could not import $nds_dom_cont_ldif");
		}

	};

	if ($@) { Log(ERR, $@) }
	Log(MESG, "   NCP server object and AD server object related successfully");
}

sub import_adc_acls
{

	Log(MESG, ">>>Giving Rights to all the Servers");
	my $ldapmodify = registry::getReg("LDAPMODIFY");
	my $ldap_modify_flags = registry::getReg("LDAPMODIFY_FLAGS");
	my $acl_file = registry::getReg("LIVE_DSSTATEDIR")."/domain/nds-adc-acls.ldif";

	eval {

		if( -s $acl_file) {
			Log("Importing $acl_file");
			my $rc = import_file(host=>registry::getReg("LDAPI_URL"),
				admin=>registry::getReg("LDAPADMINNAME"),
				file=>$acl_file,
				cont_if_exist=>"true");

			if ($rc != 0) {
				die("Could not upload the acls");
			}
		}
	};

	if ($@) { Log(ERR, $@) }
}

sub import_nds_policies
{

	if (! int(registry::getReg("MAPPED"))) {
		#Should be called only for Name-Mapped installs
		return;
	}

	my $ldapsearch = registry::getReg("LDAPSEARCH");
	my $ldapmodify = registry::getReg("LDAPMODIFY");
	my $ldapi_url = registry::getReg("LDAPI_URL");
	my $mapped_domainnc = registry::getReg("MAPPEDDOMAINNC");
	my $policy_dn = "CN=Domain Password Policy,CN=Password Policies,CN=System,$mapped_domainnc";
	my $dsstatedir = registry::getReg("DSSTATEDIR");
	my $policy_ldif =  "$dsstatedir/domain/policy.ldif";
	my $templatedir = registry::getReg("TEMPLATEDIR");
	my $changes_log = "/var/opt/novell/xad/log/dsfw-changes.log";

	$ENV{ADM_PASSWD} = $ENV{NDSEXISTINGADMINPASSWD};
	my $ldapadminname = registry::getReg("NDSEXISTINGADMINNAME");
	$ldapadminname =~ s/\.([a-z,A-Z,0-9]*)=/,$1=/g;

	my $ldapsearch_flags = " -H $ldapi_url -x -LLL -D '$ldapadminname' -w '$ENV{'ADM_PASSWD'}'";
	my $ldapmodify_flags = " -H $ldapi_url -x -D '$ldapadminname' -w '$ENV{'ADM_PASSWD'}'";

	system("$ldapsearch $ldapsearch_flags -b '$mapped_domainnc' -s sub '(&(nspmPasswordPolicyDN=*)(!(cn=Builtin))(!(cn=Computers))(!(ou=Domain Controllers)))' dn | sed ' /^ / {; H; d; }; /^ /! {; x; s/\\n //; }; ' | sed -e 's/^#.*//' -e 's/[ ^I]*\$//' -e '/^\$/ d' > $dsstatedir/domain/policydn_file");
	system("sed \"s/\$/;$policy_dn/g\" $dsstatedir/domain/policydn_file > $dsstatedir/domain/makeLdif_file");
	system("makeLdif.pl -t $templatedir/passwd_policy.ldif -a $dsstatedir/domain/makeLdif_file -o $policy_ldif");

	Log(MESG, "Associating default password policy");
	Log("Importing $policy_ldif");
	my $rc = import_file(host=>registry::getReg("LDAPI_URL"),
		admin=>registry::getReg("LDAPADMINNAME"),
		file=>$policy_ldif,
		cont_if_exist=>"true");

	if ($rc != 0) {
		die("Could not associate default password policy");
	}

	util::file_append_str(">>> Modifying Password Policies in subtree", $changes_log);
	util::file_cat($policy_ldif, ">>", $changes_log);

}
#******************************For LDIF Modification***********************************#
#TODO Option for ldapi bind as type
sub import_file
{
	my %args = @_;
	my $file  = $args{file} || die("No file name is mentioned for import");
	my $cont_if_exist = $args{cont_if_exist} || "false";
	my $edir_schematic_cntl = $args{edir_schematic_cntl} || "false";
	my $ld = connectandbind(@_);

	if(not defined $ld){
		Log(INFO, "Failed to get the connection handle");
		return -1;
	}

	my $tempdir="/var/tmp/";
	my $tempfile=$tempdir."ldifEntry.fidl";
	my $rc = 0;

	$cont_if_exist = lc($cont_if_exist);
	$edir_schematic_cntl = lc($edir_schematic_cntl);

	my $ldif = dsfw_ldif->new( $file, "r", onerror => 'warn' );
	my ($entry, $msg, $cntl);

	if ($edir_schematic_cntl eq "true") {
		 $cntl = Net::LDAP::Control->new(
		   type     => "2.16.840.1.113719.1.513.4.5",
		   value    => "eDirectory semantics control",
		   critical => 0
		 );
	}

	#Start Reading the complete LDIF file
	while(not $ldif->eof()){
		$entry = $ldif->read_entry();
		if ( $ldif->error ( ) || not defined $entry) {
			Log(WARN, $ldif->error ());
			Log(WARN, $ldif->error_lines());
			#return -1; #commenting to bypass just the warning
			next;
		}

		#Write the entry to temporary file
		#Log(INFO, "Current Entry \n".$ldif->current_lines());
		open TEMP_HANDLE, ">$tempfile" or die("Could not open the file");
		print TEMP_HANDLE  $ldif->current_lines();
		close TEMP_HANDLE;

		#Local Variables
		my $command="";
		my $attrib = "";
		my $modify_present = 0;
		my $finalPhase = 0;
		my $noattrib = 0;
		my $default_on = 0;
		my @filedata = ();
		my @changearr = ();
		my @addparam=();
		my @delparam = ();
		my @replparam = ();
		my @controlrefs = ();

		#checking for whether the "changetype" string present or not
		open TEMP_HANDLE, "<$tempfile" or die("Could not open the file");
		@filedata = <TEMP_HANDLE>;
		close TEMP_HANDLE;
		@changearr = grep /^changetype:\s*/i, @filedata;

		#process the entry
		open TEMP_HANDLE, "<$tempfile" or die("Could not open the file");
		while($line=<TEMP_HANDLE>){
                        chomp $line;
                        #Multiple modifications in a single DN
			if($line && ($line eq "-")){
				$rc = LdapModification( hndl=>\$ld, ldifh=>\$ldif, dn=>$dn_name, cmd=>$command,
					add=>\@addparam, del=>\@delparam, repl=>\@replparam, ctrlref=>\@controlrefs, attrib=>$noattrib,
					modf=>$modify_present, cont_if_exist=>"true", defcmd=>$default_on, lastent=>"false");
				return $rc if($rc == -1);
				$finalPhase=0;
				$noattrib=0;
				@addparam=();
				@delparam=();
				@replparam=();
				next;
			}

			#directive DN
			if($line && ($line =~ /^dn:\s*([\$\(\)\{\}\-,_\@\#\\=\s\w\d]+)/)){
				$dn_name=$1, $noattrib=0, $modify_present=0;
				next;
			}

			#control directive under DN
			if($line && ($line =~ /^control:\s*([.\d]+)/)){
				my (%control_hash);
				%control_hash=("type",$1,"value",0,"critical",0);
				push @controlrefs,  Net::LDAP::Control->new(%control_hash);
				next;
			}

			#changetype directive under DN
			if (@changearr && $line && ($line =~ /^changetype:\s*/i)) {
				if($line =~ /:\s*(\w+)/){
					if($1 eq "modify"){
						$modify_present=1;
						next;
					}
					else{
						$command=$1;
						next;
					}
				}
			}

			if($modify_present){
				#Identify the command to perform on the attribute
				if($line =~ /(add|delete|replace|increment):\s*([\-;\w]+)/){
					$command=$1, $attrib=$2, $finalPhase=1;
					next;
				}

				if(!$command){
					$command="add", $finalPhase=1, $modify_present=0, $default_on=1;
				}
			}
			else{
				$finalPhase=1;
			}

			$command="add" if(!@changearr and !$command);   #Initially suggested for replace

			#separate the attribute($attrib) and it's value($line)
			$line=~s/\s*([\-\d\w]+):\s*//;
			$attrib=$1 if(!$modify_present);
			#decoding base64 format to binary
			if($line=~/^:\s*/){
				#Log(INFO, "Base64 decoding done on [".$line."]");
				$line=~s/^:\s*//;
				$line=decode_base64($line);
			}
			#next if($line eq $attrib);

			#push all the attributes into respective arrays
			if($finalPhase){
				push @addparam,$attrib,$line if($command eq "add");
				push @delparam,$attrib,$line if($command eq "delete");
				push @replparam,$attrib,$line if($command eq "replace");
				$noattrib ++;
			}
		}#End of file parsing While

		close TEMP_HANDLE;

		#ADDED NOW
		push @delparam,$attrib if(!@delparam && $attrib && $command);
		$rc = LdapModification(hndl=>\$ld, ldifh=>\$ldif, dn=>$dn_name, cmd=>$command,
			add=>\@addparam, del=>\@delparam, repl=>\@replparam, ctrlref=>\@controlrefs, attrib=>$noattrib,
			modf=>$modify_present, cont_if_exist=>"true", defcmd=>$default_on, lastent=>"true");
		return $rc if($rc == -1);

	}#End of While

        $ld->unbind();
        $ld->disconnect();
	$ldif->done();
	return $rc;
}

sub LdapModification{
	my %args = @_;
	my $ld = ${$args{hndl}};
	my $ldif = ${$args{ldifh}};
	my $dn_name = $args{dn};
	my $command = $args{cmd} || die("No command supplied");
	my @addparam = @{$args{add}};
	my @delparam = @{$args{del}};
	my @replparam = @{$args{repl}};
	my @controlrefs = @{$args{crtlref}};
	my $noattrib = $args{attrib} || 0;
	my $modify_present = $args{modf} || 0;
	my $cont_if_exist = $args{cont_if_exist} || "false";
	my $default_on = $args{defcmd} || 0;
	my $last_entry = $args{lastent} || "false";
	my @parameters = ();

	#Log(INFO, "DN : ".$dn_name);
	#Log(INFO, "COMMAND : ".$command);
	#Log(INFO, "ADD PARAM : @addparam");
	#Log(INFO, "DEL PARAM : @delparam");
	#Log(INFO, "REPLACE PARAM : @replparam");
	#Log(INFO, "ATTRIB : ".$noattrib);
	#Log(INFO, "CONTINUE IF EXIST : ".$cont_if_exist);
	#Log(INFO, "MODIFY PRESENT : ".$modify_present);
	#Log(INFO, "DEFAULT COMMAND : ".$default_on);

	#When there is no attribute provided
	if(!$noattrib){
		given($command){
			when("add") { push @addparam,$attrib,"[]"; }
			#case "delete" { push @delparam,$attrib,"[]"; }
			#ADDED NOW
			when("delete") { push @delparam,$attrib; }
			default { Log(WARN, $command." command not found"); }
		}
	}

	#Preparing the paramaeter
	if($modify_present || $default_on){
		if($#replparam > 0){
			push @parameters, 'replace';
			push @parameters, \@replparam;
		}

		if($#addparam > 0){
			push @parameters, 'add';
			push @parameters, \@addparam;
		}

		if($#delparam > 0){
			push @parameters, 'delete';
			push @parameters, \@delparam;
		}
	}
	else{
		if($command eq "add" && $#addparam>0){
			push @parameters, 'attrs';
			push @parameters, \@addparam;
		}

		if($command eq "delete" && $#delparam>0){
			push @parameters, 'attrs';
			push @parameters, \@delparam;
		}
	}

	#Apply the modification operation on the current DN
	if($#controlrefs > 0){
		$msg=$ld->modify($dn_name, changes => [@parameters], control => [@controlrefs]);
	}
	else{
		if($modify_present || $default_on){
			$msg=$ld->modify($dn_name, changes => [@parameters]);
		}
		else{
			given($command){
				when("add") { $msg=$ld->add($dn_name, @parameters); }
				when("delete") { $msg=$ld->delete($dn_name, @parameters); }
				default { Log(WARN, $command." command not found"); }
			}
		}
	}

	if (ldap_error_name($msg) ne "LDAP_SUCCESS") {
		if($cont_if_exist eq "true"){
			if(ldap_error_name($msg) eq "LDAP_TYPE_OR_VALUE_EXISTS" or
					ldap_error_name($msg) eq "LDAP_ALREADY_EXISTS"){
				#ParameterDisplay(dn=>$dn_name, param=>\@parameters, errstr1=>" already exists.");
				#Log(INFO, "{@parameters} already exists in <".$dn_name.">");
			}
			elsif(ldap_error_name($msg) eq "LDAP_NO_SUCH_ATTRIBUTE"){
				#ParameterDisplay(dn=>$dn_name, param=>\@parameters, errstr1=>" does not exists.");
				#Log(INFO, "{@parameters} does not exists in <".$dn_name.">");
			}
			else{
				Log(WARN, "Could not process the entry ");
				ParameterDisplay(dn=>$dn_name, param=>\@parameters, errstr1=>"Entry failed");
				Log(WARN, $ldif->current_lines());
				Log(WARN, ldap_error_name($msg).":".ldap_error_text($msg));
				return -1;
			}
		}
		else{
			Log(WARN, "Could not process the entry ");
			ParameterDisplay(dn=>$dn_name, param=>\@parameters, errstr1=>"Entry failed");
			Log(WARN, $ldif->current_lines());
			Log(WARN, ldap_error_name($msg).":".ldap_error_text($msg));
			return -1;
		}
	}
	else{
		#ParameterDisplay(dn=>$dn_name, param=>\@parameters, errstr1=>" ");
		Log(INFO, "Successfully processed [$dn_name]") if($last_entry eq "true");
	}

	return 0;
}

sub ParameterDisplay{
	my %args=@_;
	my $dn_name = $args{dn};
	my @parameters=@{$args{param}};
	my $err_str1 = $args{errstr1};
	my $LogString = "";

	for(my $x=0; $x<@parameters; $x+=2){
		$LogString .= "(".$parameters[$x].")";
		my $refarray=$parameters[$x+1];
		for(my $i=0; $i<@$refarray; $i+=2){
			$LogString .= " ".@$refarray[$i]." => ".@$refarray[$i+1]." ";
		}
	}
	$LogString .= $err_str1;
	Log(INFO, $LogString);
}


1;
