#!/usr/bin/perl -I. -I.. -I/opt/novell/xad/lib64/perl -I/opt/novell/xad/lib/perl

################################################################################
# Domain Services for Windows Provisioning Tool
#
# See the file LICENSE for redistribution information.
#
# Copyright (c) 2001-2009
#	Novell, Inc.  All rights reserved.
################################################################################
#
#   NAME:
#         replops.pm
#
#   FACILITY:
#         Provides all the replication/partition related work
#
#   ABSTRACT:
#
#   REVISION: $Id$
#
#################################################################################

package replops;

use XAD::registry;
use Logger qw(:DEFAULT);

sub split_partition{
	my ($ip, $mapnc, $ldapadmin, $pwd, $port, $serverFDN, $certFile) = @_;

	my $xadroot = registry::getReg("XADROOT");
	my $rc = 0;

	Log(MESG, "   splitting partition using command..");
	if($serverFDN eq "NULL")
	{
		# Its a split of domain partition, and the local server will not have a replica
		# as of now, hence we can't get the status of replica @ local server
		# hence using -c WITHOUT -s <NCPServerFDN>
		if(defined $certFile){ 
			Log(MESG, " $xadroot/sbin/replops -h $ip -c -p $port -n '$mapnc' ".
				"-f '$certFile' -D '$ldapadmin' -W --split-partition");
			$rc = system(" $xadroot/sbin/replops -h $ip -c -p $port -n '$mapnc' ".
				"-f '$certFile' -D '$ldapadmin' -w '$pwd' --split-partition");
		}
		else{
			Log(MESG, "   $xadroot/sbin/replops -h $ip -c -p $port -n '$mapnc' -D '$ldapadmin' -W --split-partition");
			$rc = system(" $xadroot/sbin/replops -h $ip -c -p $port -n '$mapnc' -D '$ldapadmin' -w '$pwd' --split-partition");
		}

	} else {
		
		# Its a split of configuration or schema partition, the local server would 
		# get a replica as a result of this split operation, hence checking replica
		# status on local server
		Log(MESG, "   $xadroot/sbin/replops -h $ip -p $port -n '$mapnc' -s '$serverFDN' -D '$ldapadmin' -W --split-partition");
		$rc = system("$xadroot/sbin/replops -h $ip -p $port -n '$mapnc' -s '$serverFDN' -D '$ldapadmin' -w '$pwd' --split-partition");
	}

	if ($rc != 0) {
		die("Could not split the partition");
	}

#	Log("Waiting for 20 seconds to syncup");
	sleep(20);
}

sub create_domain_nc
{

	eval{
		#FIXME:Shifts the object instance. Everyone who calls this method,
		#has to be a object instance.
		shift;
		
		# send the local NCP server LDAP formatted FDN as NULL
		my $NCPserverFDN = "NULL";

		#sends the port number too;

		split_partition(@_, 389, $NCPserverFDN);

	};

	if ($@) { Log(ERR, $@) }

}

#Creates the config partition by splitting up the domain partition
sub create_config_nc {

	my $ipaddress = registry::getReg("IPADDRESS");
	my $ldapadminname = registry::getReg("LDAPADMINNAME");
	my $localmachinename = registry::getReg("InstallMachineName");
	my $mappedNDStempcontainer = registry::getReg("Mapped NDS Temp Container");
	my $NCPserverFDN = "CN=$localmachinename" . ",$mappedNDStempcontainer";

	Log(MESG, ">>>Creating Configuration Naming Context");

eval{
	if (! registry::getReg("MAPPED")) {
		my $defaultconfignc = registry::getReg("DEFAULTCONFIGNC");
		split_partition($ipaddress, $defaultconfignc, $ldapadminname, "env:ADM_PASSWD", 389, $NCPserverFDN);
	}
	else {
		my $mappedconfignc = registry::getReg("MAPPEDCONFIGNC");
		split_partition($ipaddress, $mappedconfignc, $ldapadminname, "env:ADM_PASSWD", 389, $NCPserverFDN);
	}
};
	if ($@) { Log(ERR, $@) }
	Log(MESG, "   Configuration Naming Context created successfully");
}

#Creates the schema partition by splitting up the domain partition
sub create_schema_nc {

	my $ipaddress = registry::getReg("IPADDRESS");
	my $ldapadminname = registry::getReg("LDAPADMINNAME");
	my $localmachinename = registry::getReg("InstallMachineName");
	my $mappedNDStempcontainer = registry::getReg("Mapped NDS Temp Container");
	my $NCPserverFDN = "CN=$localmachinename" . ",$mappedNDStempcontainer";


	Log(MESG, ">>>Creating Schema Naming Context");

eval{
	if (! registry::getReg("MAPPED")) {
		my $schema = registry::getReg("SCHEMA");
		split_partition($ipaddress, $schema, $ldapadminname, "env:ADM_PASSWD", 389, $NCPserverFDN);
	}
	else {
		my $mappedschema = registry::getReg("MAPPEDSCHEMA");
		split_partition($ipaddress, $mappedschema, $ldapadminname, "env:ADM_PASSWD", 389, $NCPserverFDN);
	}
};
	if ($@) { Log(ERR, $@) }
	Log(MESG, "   Schema Naming Context created successfully");
}

#Adds replica of mentioned type
sub add_replica
{

	my %args          = @_;
	my $host          = $args{h} || die("No Host name provided for replops");
	my $port          = $args{p} || 389;
	my $ncp_serv_name = $args{s} || die("No ncp server name mentioned");
	my $replica_name  = $args{n} || die("No replica name mentioned");
	my $replica_type  = $args{t} || "writable";
	my $user_name     = $args{D} || die("No user name mentioned");
	my $pwd           = $args{w} || die("No credential available");

	my $xad_root = registry::getReg("XADROOT");

	Log("Executing.. $xad_root/sbin/replops -h $host -p $port -s '$ncp_serv_name' -n '$replica_name' -t $replica_type -D '$user_name' -W --add-replica");

	my $rc = system("$xad_root/sbin/replops -h $host -p $port -s '$ncp_serv_name' -n '$replica_name' -t $replica_type -D '$user_name' -w '$pwd' --add-replica");
	if($rc != 0) {
		die("replops failed while adding replica");
	}


}

#Make the mentioned replica as master
sub make_master_replica
{

	my %args          = @_;
	my $host          = $args{h} || die("No Host name provided for replops");
	my $port          = $args{p} || 389;
	my $ncp_serv_name = $args{s} || die("No ncp server name mentioned");
	my $replica_name  = $args{n} || die("No replica name mentioned");
	my $user_name     = $args{D} || die("No user name mentioned");
	my $pwd           = $args{w} || die("No credential available");

	my $xad_root = registry::getReg("XADROOT");

	Log("Executing.. $xad_root/sbin/replops -h $host -p $port -s '$ncp_serv_name' -n '$replica_name' -t master -D '$user_name' -W --change-replica-type");

	my $rc = system("$xad_root/sbin/replops -h $host -p $port -s '$ncp_serv_name' -n '$replica_name' -t \"master\" -D '$user_name' -w '$pwd' --change-replica-type");
	if($rc != 0) {
		die("replops failed while adding replica");
	}


}

#Removes the domain replica from other server
#Domain replica should not be left in other server who is not part of the domain
sub remove_domain_replica_from_other_server
{

	Log(MESG, "Deleting the domain replica from other server who is not part of the this domain");

	my $xad_root = registry::getReg("XADROOT");
	my $rc;

	eval{

		$ENV{ADM_PASSWD} = $ENV{NDSEXISTINGADMINPASSWD};

		if(registry::getReg("MAPPED")) {
			$rc = system("$xad_root/share/dcinit/removeReplica.sh --nm-child");
		}
		else {
			$rc = system("$xad_root/share/dcinit/removeReplica.sh --child");
		}

		if ($rc != 0) {
			die("Could not delete the domain replica from other server");
		}
	};

	if ($@) { Log(ERR, $@) }

	Log("Domain replica is removed from other servers successfully");

}
sub remove_config_replica{
	my $XADROOT = registry::getReg("XADROOT");

	if (registry::getReg("MAPPED")) {

	eval{
		Log(MESG, ">>>Removing config replica from non dsfw server");
		my $out = `$XADROOT/share/dcinit/removeReplica.sh --remove-config-replica`;

		if ($? != 0) {
			print $out;
			die("Could not remove read write config replica from non dsfw server");
		}

	};

		if ($@) { Log(ERR, $@) }
		Log("Config Replica is removed successfully from the non dsfw server");
	}

}

sub remove_schema_replica{
	my $XADROOT = registry::getReg("XADROOT");

	if (registry::getReg("MAPPED")) {

	eval{
		Log(MESG, ">>>Removing schema replica from non dsfw server");
		my $out = `$XADROOT/share/dcinit/removeReplica.sh --remove-schema-replica`;

		if ($? != 0) {
			print $out;
			die("Could not remove read write schema replica from non dsfw server");
		}

	};

		if ($@) { Log(ERR, $@) }
		Log("Schema Replica is removed successfully from the non dsfw server");
	}

}

1;
