#!/usr/bin/perl -I. -I.. -I/opt/novell/xad/lib64/perl -I/opt/novell/xad/lib/perl

################################################################################
# Domain Services for Windows Provisioning Tool
#
# See the file LICENSE for redistribution information.
#
# Copyright (c) 2001-2009
#	Novell, Inc.  All rights reserved.
################################################################################
#
#   NAME:
#         stage.pm
#
#   FACILITY:
#         Provides all the staging related work
#
#   ABSTRACT:
#	 Prepare the requierd configuration for Provisioning
#
#   REVISION: $Id$
#
#################################################################################

package stage;

use XAD::registry;
use XAD::schema;
use XAD::ntds;
use XAD::dclib;
use File::Copy;
use Logger qw(:DEFAULT);
use Carp qw(cluck);
use util;
use File::Path;

#our @ISA = qw(registry);

use File::Path qw(mkpath);
use File::Copy;
use Locale::gettext;

#Constructor for the stage
sub new {

	my $class = shift;
	my $self = {@_};
	bless($self, $class);
	reg_init();
	return $self;
}

sub croak { die "$0: @_: $!\n" } #Alternative logging for Logger::log

#Replacement for regSubstitute.pl
#Replace all the $REGISTRY with the current registry value 
# from the INI file to the target
#Inside framework better to call this rather than calling the backtick regSubstitute.pl script.
sub reg_substitute {
	my ($infile, $outfile) = (@_);

	open(IN, $infile) or Log(ERR,"could not open the input file $infile");
	open(OUT,">>$outfile") or Log(ERR, "could not open the output file $outfile");

	while (my $line = <IN>) {

		my $txt = XAD::regSubstitute($line);
		print OUT "$txt";
	}

	close IN;
	close OUT;


}

#Replacement for genldif.p
#Inside framework better to call this rather than calling the backtick script.
sub gen_ldif {
	my $fileName = shift;
	my $op = shift;
	my $outfile = shift;
	my $OFFLINE;

	if (!(-f $fileName)) {
		print STDERR sprintf(gettext("Could not open %s."), $fileName), "\n";
		exit 1;
	}

	if ($_[0] eq "--offline") {
		$OFFLINE = 1;
		shift;
	} else {
		$OFFLINE = 0;
	}

	my $arg;

	foreach $arg (@_[0..$#_]) {
		my $treeRoot;

		if ($arg =~ /^--/) {
			$treeRoot = uc(substr($arg, 2));
		} else {
			print STDERR gettext("Invalid parameter $_\n");
			exit 1;
		}

		{#Lets get the output of this module to msdsfile.
			local *STDOUT;
			open (STDOUT, $op, $outfile) or croak("Could not open the file $outfile");
			XAD::genDcSkelLdif($fileName, $treeRoot, 0, $OFFLINE);
		}

	}

	if (!(-e $outfile) || (-z $outfile)) {
		Log (ERR, "The LDIF file [$outfile] is either not generated or is empty");
	}
}

sub stage_dir {

	my $ndsroot = registry::getReg("NDSROOT");
	my $etcdir  = registry::getReg("ETCDIR");
	my $stagedir = registry::getReg("STAGEDIR");
	my $localstatedir = registry::getReg("LOCALSTATEDIR");
	my $dsstatedir = registry::getReg("DSSTATEDIR");
	my $tempdir = registry::getReg("TEMPDIR");

	if (-e $tempdir."/stage_dir") {
		Log("Creating the staging directories was already done.");
		return 0;
	}

	eval{
		mkpath( $stagedir );
		mkpath( $etcdir );
		mkpath( $etcdir."/openldap" );
		mkpath( $localstatedir );
		mkpath( $localstatedir."/rpc/PIPE" );
		mkpath( $localstatedir."/rpc/PIPE/np" );
		mkpath( $localstatedir."/run" );
		mkpath( $localstatedir."/run/ntp" );
		mkpath( $stagedir."/etc/sysconfig" );
		mkpath( $stagedir."/etc/sysconfig/novell" );
		mkpath( $stagedir."/etc/$ndsroot/conf" );
		mkpath( $dsstatedir, 0, 0750 );
		system( "chgrp named ".$dsstatedir );
		mkpath( $dsstatedir."/config", 0, 0700);
		mkpath( $dsstatedir."/domain", 0, 0700);
		mkpath( $dsstatedir."/schema", 0, 0700);
		mkpath( $dsstatedir."/samify", 0, 0700);
		mkpath( $tempdir, 0, 0700);

#	if ($main::Nds == 1) {
		mkpath ($dsstatedir."/linkengine", 0, 0700);
#	}
#	else {
#		print STDERR "Nds var is not defined\n";
#	}
		#Creating directory structure for ntp at mounted location
		if(registry::getReg('NcalRpcDir') ne ""){
			mkpath(registry::getReg('NcalRpcDir')."/PIPE");
			mkpath(registry::getReg('NcalRpcDir')."/PIPE/np");
			mkpath("/var/lib/chrony".registry::getReg('NcalRpcDir'));
		}
		#Creating directory structure /var/opt/novell/xad/rpc in ntp chroot jail
		mkpath( "/var/lib/chrony".registry::getReg("LIVE_LOCALSTATEDIR")."/rpc" );
	};
	if($@) {
		Log(ERR,$@); #Printing ERR message actually dies with proper error message and a backtrace
	}

	`touch $tempdir/stage_dir`;
}

sub stage_default_config {

#We do nothing here. xad.ini is already created by ndsdcinit.pl
# eval {
# 	if ( registry::getReg("DCINIT_CONFIG") ne "") {
# 		mkpath(registry::getReg("LIVE_ETCDIR"));
# 		my $source = registry::getReg("DCINIT_CONFIG");
# 		my $dest = registry::getReg("LIVE_ETCDIR")."/xad.ini";
# 		copy ($source, $dest) or Log(ERR, "Could not copy the xad.ini file to $dest");
# 	}
# };
# 	if ($@) {
# 		Log(ERR,$@);
# 	}
}

#Gets the domain guid from edir and replaces with xad.ini
sub stage_domainguid {
	my $tempdir = registry::getReg("TEMPDIR");
	my $retry_count = 0;

	eval{
	retry:
		my $ip = registry::getReg("IPADDRESS");
		my $mappeddomainnc = registry::getReg("Mapped Domain NC");
		my $LIVE_ETCDIR = registry::getReg("LIVE_ETCDIR");

		Log("	Getting Domain GUID");

		#Get the domain guid and also store in registry
		Log("	Executing... /opt/novell/xad/share/dcinit/provisionTools.sh get-domain-guid -p $ip -c $mappeddomainnc");
		my $DOMAINGUID=`/opt/novell/xad/share/dcinit/provisionTools.sh get-domain-guid -p $ip -c '$mappeddomainnc'`;
		chomp($DOMAINGUID);
		#registry::setReg("DOMAINGUID", $DOMAINGUID);

		if (not ($DOMAINGUID =~ /^[a-zA-Z0-9]{8}(-[a-zA-Z0-9]{4}){3}-[a-zA-Z0-9]{12}$/)) {
			die("DOMAINGUID $DOMAINGUID is not in correct format");
		}
		if ($DOMAINGUID =~ /^0{8}(-0{4}){3}-0{12}$/) {
			#All zeros, the ldap search failed.
			die("Could not retrive the domain guid, domain_guid=$DOMAINGUID");
		}
		Log("	DOMAINGUID = $DOMAINGUID");

		#Store (actually replace) the domain guid in xad.ini
		util::replace_line_in_file ('^Domain GUID.*', "Domain GUID = $DOMAINGUID", "$LIVE_ETCDIR/xad.ini");
	};

	if ($@) { #catch any error and trap all die
		if ($retry_count < 5) {
			Log("$@ \n This is may be due to the replication latency. Retrying...\n");
			sleep (15);
			$retry_count = $retry_count + 1;
			goto retry;
		}
		Log(ERR,$@);
	}
}

#Gets the domain sid from edir and replaces with xad.ini
sub stage_domainsid {
	my $retry_count = 0;
	my $ldap_existing_adminname = registry::getReg("LDAPTEMPADMINNAME");
	my $domain_name = registry::getReg("Root Domain");

	eval{
	retry:
		my $LIVE_ETCDIR = registry::getReg("LIVE_ETCDIR");
		Log("	Getting Domain SID");
		#Get the domain sid and also store in registry
		Log("	Executing... /opt/novell/xad/share/dcinit/provisionTools.sh get-domain-sid -a \"$ldap_existing_adminname\" -d \"$domain_name\"");
		my $DOMAINSID=`ADM_PASSWD=$ENV{'NDSEXISTINGADMINPASSWD'} /opt/novell/xad/share/dcinit/provisionTools.sh get-domain-sid -a "$ldap_existing_adminname" -d "$domain_name"`;
		chomp($DOMAINSID);
		Log("   DOMAINSID = $DOMAINSID");
		#Store (actually replace) the domain sid in xad.ini
		util::replace_line_in_file ('^Domain SID.*', "Domain SID = $DOMAINSID", "$LIVE_ETCDIR/xad.ini");

		#Register the new value of SID
		registry::reg_init();
	};
	if ($@) { #catch any error and trap all die
		if ($retry_count < 5) {
			Log("$@ \n This is may be due to the replication latency. Retrying...\n");
			sleep (15);
			$retry_count = $retry_count + 1;
			goto retry;
		}
		Log(ERR,$@);
	}
}
sub restart_ntp{
	if ( -e "/usr/lib/systemd/system/chronyd.service" ){
		`/usr/bin/systemctl restart chronyd.service`;
		if( $? != 0 ){
			Log(WARN,">>>The chrony service restart failed. Try starting chrony manually.");
		}else {
			Log(MESG,">>>The chrony service restart succeeded");
		}
	}
}

#Configure the chrony.conf File
sub stage_ntp {
	my $tempdir = registry::getReg("TEMPDIR");

	if (-e $tempdir."/stage_ntp") {
		Log("The configuration of the NTP server already done.");
		return 0;
	}
	eval{

		my $TEMPLATEDIR = registry::getReg("TEMPLATEDIR");

		#This call has to be overridden for the ADCs. The below is only for NON ADC.
		my $err= system("/opt/novell/xad/share/dcinit/provisionTools.sh ntpconfig -t $TEMPLATEDIR/ntp-master.conf");

		if ($err != 0) { die("provisionTools ntpconfig failed with err code $err");	}

		if (util::chkfor_reg("/etc/chrony.conf") == 0) { die("Not all the REGISTRY values are replaced in /etc/chrony.conf")};

	};
	if ($@) {
		Log(ERR, $@)   #catch any error and trap all die
	}else {
		Log(MESG,">>>Checking for NTP changes");

		# This line for checking the ntp changes now commenting because this line is no longer relevant in chrony.conf file for ntp service.
		# Now we are replacing this line with another line for checking ntp changes in chrony.conf file.
		# For this fix we have removed this line from /opt/novell/xad/share/dcinit/templates/ntp-master.conf and 
		# /opt/novell/xad/share/dcinit/templates/ntp-slave.conf file.
		# This fix is the part of D-6370.
		#$rc = system("/usr/bin/grep 'authprov /opt/novell/xad/lib64/libw32time.so 131072:4294967295 global' /etc/chrony.conf");

		#$rc = system("/usr/bin/grep 'restrict -4 default notrap nomodify nopeer noquery mssntp' /etc/chrony.conf");

		$rc = system("/usr/bin/grep 'ntpsigndsocket ' /etc/chrony.conf");
		if ($rc == 0){
			Log(MESG, "   Successfully updated DSfW specific chrony configuration. Restarting chrony service for the changes to take effect.");
			restart_ntp();
		}
	}

	`touch $tempdir/stage_ntp`;
}

#FIXME: Does it requierd?
sub stage_libldap{
	my $tempdir = registry::getReg("TEMPDIR");

	if (-e $tempdir."/stage_libldap") {
		Log("The configuration of open ldap was already done.");
		return 0;
	}
eval{
	my $tempconf = registry::getReg("TEMPLATEDIR")."/ldap.conf";
	my $ldapconf = registry::getReg("ETCDIR")."/openldap/ldap.conf";

	reg_substitute ($tempconf, $ldapconf);

	if (util::chkfor_reg("$ldapconf") == 0) { die("Not all the REGISTRY values are replaced in $ldapconf")};

};
	if ($@) {Log(ERR, $@)}

	`touch $tempdir/stage_libldap`;
}

#Generating the rcscripts and linking them
sub stage_rc {

	my $templatedir = registry::getReg("TEMPLATEDIR");
	my $stagedir = registry::getReg("STAGEDIR");
	my $xadroot = registry::getReg("XADROOT");
	my $tempdir = registry::getReg("TEMPDIR");

	if (-e $tempdir."/stage_rc") {
		Log("The creation of rc scripts and link was already done.");
		return 0;
	}

	eval{
		mkpath( $stagedir."/usr/lib/systemd/system" );
		mkpath( $stagedir."/opt/novell/xad/bin" );
                mkpath( $stagedir."/opt/novell/xad/sbin" );
                mkpath( $stagedir."/opt/novell/xad/libexec" );

		Log("   Creating rpcd systemd service");
		reg_substitute("$templatedir/rpcd.service", "$stagedir/usr/lib/systemd/system/rpcd.service");

		if(util::chkfor_reg("$stagedir/usr/lib/systemd/system/rpcd.service") == 0) {
			die("Not all values are replaced in $stagedir/usr/lib/systemd/system/rpcd.service");
		}

		Log("   Creating rpcd start script");
		reg_substitute("$templatedir/start-rpcd", "$stagedir/opt/novell/xad/bin/start-rpcd");

		if(util::chkfor_reg("$stagedir/opt/novell/xad/bin/start-rpcd") == 0) {
			die("Not all values are replaced in $stagedir/opt/novell/xad/bin/start-rpcd");
		}

		Log("   Creating rpcd stop script");
		reg_substitute("$templatedir/stop-rpcd", "$stagedir/opt/novell/xad/bin/stop-rpcd");

		if(util::chkfor_reg("$stagedir/opt/novell/xad/bin/stop-rpcd") == 0) {
			die("Not all values are replaced in $stagedir/opt/novell/xad/bin/stop-rpcd");
		}

		`ln -sf /usr/sbin/service $xadroot/sbin/rcrpcd`;
		chmod 0644, "$stagedir/usr/lib/systemd/system/rpcd.service";
                chmod 0755, "$stagedir/opt/novell/xad/bin/start-rpcd";
		chmod 0755, "$stagedir/opt/novell/xad/bin/stop-rpcd";

                Log("   Creating w32tm_client systemd service");
                reg_substitute("$templatedir/w32tm_client.service", "$stagedir/usr/lib/systemd/system/w32tm_client.service");

                if(util::chkfor_reg("$stagedir/usr/lib/systemd/system/w32tm_client.service") == 0) {
                        die("Not all values are replaced in $stagedir/usr/lib/systemd/system/w32tm_client.service");
                }

                Log("   Creating w32tm_client start script");
                reg_substitute("$templatedir/start-w32tm_client", "$stagedir/opt/novell/xad/sbin/start-w32tm_client");

                if(util::chkfor_reg("$stagedir/opt/novell/xad/sbin/start-w32tm_client") == 0) {
                        die("Not all values are replaced in $stagedir/opt/novell/xad/sbin/start-w32tm_client");
                }

                `ln -sf /usr/sbin/service $xadroot/sbin/rcw32tm_client`;
                chmod 0644, "$stagedir/usr/lib/systemd/system/w32tm_client.service";
                chmod 0755, "$stagedir/opt/novell/xad/sbin/start-w32tm_client";

		Log("   Creating krb5 systemd service in stage");
		if(-e "$stagedir/usr/lib/systemd/system/xad-krb5kdc.service"){
                	Log("   Staging krb5 systemd service, in stage_rc");
			`/usr/bin/rm $stagedir/usr/lib/systemd/system/xad-krb5kdc.service`;
                }

		reg_substitute("$templatedir/xad-krb5kdc.service", "$stagedir/usr/lib/systemd/system/xad-krb5kdc.service");

		if (util::chkfor_reg("$stagedir/usr/lib/systemd/system/xad-krb5kdc.service") == 0) {
			die("Not all values are replaced in the $stagedir/usr/lib/systemd/system/xad-krb5kdc.service");
		}

                Log("   Creating krb5 start script in stage_rc");
		if(-e "$stagedir/opt/novell/xad/sbin/start-xad-krb5kdc"){
                	Log("   Staging krb5 start script in stage_rc");
			`/usr/bin/rm $stagedir/opt/novell/xad/sbin/start-xad-krb5kdc`;
                }

                reg_substitute("$templatedir/start-xad-krb5kdc", "$stagedir/opt/novell/xad/sbin/start-xad-krb5kdc");

                if (util::chkfor_reg("$stagedir/opt/novell/xad/sbin/start-xad-krb5kdc") == 0) {
                        die("Not all values are replaced in the $stagedir/opt/novell/xad/sbin/start-xad-krb5kdc");
                }

		`ln -sf /usr/sbin/service $xadroot/sbin/rcxad-krb5kdc`;
		chmod 0644, "$stagedir/usr/lib/systemd/system/xad-krb5kdc.service";
                chmod 0755, "$stagedir/opt/novell/xad/sbin/start-xad-krb5kdc";

                Log("   Creating kpasswd systemd service");
                reg_substitute("$templatedir/xad-kpasswdd.service", "$stagedir/usr/lib/systemd/system/xad-kpasswdd.service");

                if (util::chkfor_reg("$stagedir/usr/lib/systemd/system/xad-kpasswdd.service") == 0) {
                        die("Not all values are replaced in the $stagedir/usr/lib/systemd/system/xad-kpasswdd.service");
                }

                Log("   Creating kpasswd start script");
                reg_substitute("$templatedir/start-xad-kpasswdd", "$stagedir/opt/novell/xad/libexec/start-xad-kpasswdd");

                if (util::chkfor_reg("$stagedir/opt/novell/xad/libexec/start-xad-kpasswdd") == 0) {
                        die("Not all values are replaced in the $stagedir/opt/novell/xad/libexec/start-xad-kpasswdd");
                }

                `ln -sf /usr/sbin/service $xadroot/sbin/rcxad-kpasswdd`;
                chmod 0644, "$stagedir/usr/lib/systemd/system/xad-kpasswdd.service";
                chmod 0755, "$stagedir/opt/novell/xad/libexec/start-xad-kpasswdd";

		Log("   Creating xadsd systemd service");
                reg_substitute("$templatedir/xadsd.service", "$stagedir/usr/lib/systemd/system/xadsd.service");

                if(util::chkfor_reg("$stagedir/usr/lib/systemd/system/xadsd.service") == 0) {
                        die("Not all values are replaced in $stagedir/usr/lib/systemd/system/xadsd.service");
                }

                Log("   Creating xadsd start script");
                reg_substitute("$templatedir/start-xadsd", "$stagedir/opt/novell/xad/libexec/start-xadsd");

                if(util::chkfor_reg("$stagedir/opt/novell/xad/libexec/start-xadsd") == 0) {
                        die("Not all values are replaced in $stagedir/opt/novell/xad/libexec/start-xadsd");
                }

                Log("   Creating xadsd stop script");
                reg_substitute("$templatedir/stop-xadsd", "$stagedir/opt/novell/xad/libexec/stop-xadsd");

                if(util::chkfor_reg("$stagedir/opt/novell/xad/libexec/stop-xadsd") == 0) {
                        die("Not all values are replaced in $stagedir/opt/novell/xad/libexec/stop-xadsd");
                }

                `ln -sf /usr/sbin/service $xadroot/sbin/rcxadsd`;
                chmod 0644, "$stagedir/usr/lib/systemd/system/xadsd.service";
                chmod 0755, "$stagedir/opt/novell/xad/libexec/start-xadsd";
                chmod 0755, "$stagedir/opt/novell/xad/libexec/stop-xadsd";

                Log("   Creating smb systemd service");
                reg_substitute("$templatedir/smb.service", "$stagedir/usr/lib/systemd/system/smb.service");

                if(util::chkfor_reg("$stagedir/usr/lib/systemd/system/smb.service") == 0) {
                        die("Not all values are replaced in $stagedir/usr/lib/systemd/system/smb.service");
                }

                chmod 0644, "$stagedir/usr/lib/systemd/system/smb.service";

                Log("   Creating winbind systemd service");
                reg_substitute("$templatedir/winbind.service", "$stagedir/usr/lib/systemd/system/winbind.service");

                if(util::chkfor_reg("$stagedir/usr/lib/systemd/system/winbind.service") == 0) {
                        die("Not all values are replaced in $stagedir/usr/lib/systemd/system/winbind.service");
                }

                chmod 0644, "$stagedir/usr/lib/systemd/system/winbind.service";

                Log("   Creating nmb systemd service");
                reg_substitute("$templatedir/nmb.service", "$stagedir/usr/lib/systemd/system/nmb.service");

                if(util::chkfor_reg("$stagedir/usr/lib/systemd/system/nmb.service") == 0) {
                        die("Not all values are replaced in $stagedir/usr/lib/systemd/system/nmb.service");
                }

                chmod 0644, "$stagedir/usr/lib/systemd/system/nmb.service";

		Log("   Modifying apache2 service file");
		my $apache_service_file = "/usr/lib/systemd/system/apache2.service";

		if(-f "$apache_service_file"){
			copy("$apache_service_file", "${stagedir}$apache_service_file") or Log(WARN, "Copy of apache2 service file to stage directory failed");
			my $output = `grep "After=" ${stagedir}$apache_service_file`;
			chomp($output);
			my $winbind_present = `grep "After=" ${stagedir}$apache_service_file | grep -i "winbind.service"`;
			if(($?>>8) != 0){
				util::replace_line_in_file ("$output", "$output winbind.service", "${stagedir}$apache_service_file");
				chmod 0644, "${stagedir}$apache_service_file";

			}
		}else{
			Log(WARN, "apache2 service file not found");
		}

	};
	if ($@) {Log(ERR, $@)}

	`touch $tempdir/stage_rc`;
}

sub stage_linkengine{

	my $tempdir = registry::getReg("TEMPDIR");

	if (-e $tempdir."/stage_linkengine") {
		Log("The preparation of link engine was already done");
		return 0;
	}
eval {
	my $templatedir = registry::getReg("TEMPLATEDIR");
	my $dsstatedir = registry::getReg("DSSTATEDIR");
	gen_ldif("$templatedir/linkengine.ini", '>', "$dsstatedir/linkengine/linkengine.ldif", "--offline", "--linkengine");

	if (util::chkfor_reg("$dsstatedir/linkengine/linkengine.ldif") == 0) { 
		die("Not all the REGISTRY values are replaced in $dsstatedir/linkengine/linkengine.ldif")
	};

};
	if ($@) { Log(ERR, $@) }
	
	`touch $tempdir/stage_linkengine`;
}

#schema and schema-bl ldif files are generated here
sub stage_schema_prepare {
	my $tempdir = registry::getReg("TEMPDIR");

	if (-e $tempdir."/stage_schema_prepare") {
		Log("The preparation of the schema was already done.");
		return 0;
	}
eval{
	my $templatedir = registry::getReg("TEMPLATEDIR");
	my $dsstatedir = registry::getReg("DSSTATEDIR");

	Log("   Generating schema.ldif");
	gen_ldif("$templatedir/schema.ini", '>', "$dsstatedir/schema/schema.ldif", "--offline", "--schema");

	if (util::chkfor_reg("$dsstatedir/schema/schema.ldif") == 0) { 
		die("Not all the REGISTRY values are replaced in $dsstatedir/schema/schema.ldif $?")
	};


	Log("   Generating schema-bl.ldif");
	gen_ldif("$templatedir/schema-bl.ini", '>', "$dsstatedir/schema/schema-bl.ldif", "--schema");

	if (util::chkfor_reg("$dsstatedir/schema/schema-bl.ldif") == 0) { 
		die("Not all the REGISTRY values are replaced in $dsstatedir/schema/schema-bl.ldif")
	};

};
	if ($@) { Log(ERR, $@) }
	`touch $tempdir/stage_schema_prepare`;
}

#This generates msds.sch from schema.ldif
sub schema_aggregate {

	my $msdsfile;
	my $tempdir = registry::getReg("TEMPDIR");

	if (-e $tempdir."/stage_schema_aggregate") {
		Log("The generation of the aggregated schema was already done.");
		return 0;
	}

eval{
	my $schemafile = registry::getReg("DSSTATEDIR")."/schema/schema.ldif";
	$msdsfile = registry::getReg("ETCDIR")."/msds.sch";

	{#Lets get the output of this module to msdsfile.
		local *STDOUT;
		open STDOUT, '>', $msdsfile or die("Could not open the file $msdsfile");
		schema::aggregateSchema( $schemafile, "--ndsschema");
	}

	if (util::chkfor_reg("$msdsfile") == 0) { die("Not all the REGISTRY values are replaced in $msdsfile")};

};
	if ($@) { Log(ERR, $@) }
	Log(MESG,"   $msdsfile is generated successfully\n");
	`touch $tempdir/stage_schema_aggregate`;
}

# This gets called only for for NON FRDs.
sub stage_config
{

	my $templatedir = registry::getReg("TEMPLATEDIR");
	my $dsstatedir = registry::getReg("DSSTATEDIR");

	eval{
		stage::gen_ldif("$templatedir/config-addl.ini", '>', "$dsstatedir/config/config-addl.ldif", "--defaultconfignc");
		stage::gen_ldif("$templatedir/config-addl-bl.ini", '>', "$dsstatedir/config/config-addl-bl.ldif", "--defaultconfignc");

		#This is to generate the crossref object's reference into the ncp server object.
		stage::gen_ldif("$templatedir/config-bl.ini", '>', "$dsstatedir/config/crossrefreference.ldif", "--ncpdn");

	};

	if ($@) { Log(ERR, $@) }
}


sub generate_upgrade_ldifs {
	Log("Generating upgrade related ldif files");

	}

sub stage_domain
{

	my $templatedir = registry::getReg("TEMPLATEDIR");
	my $dsstatedir = registry::getReg("DSSTATEDIR");
	my $dnsdhcpgroup = registry::getReg("DNSDHCP_GROUP");
	my $dnsserver = registry::getReg("DNSSERVER");
	my $dns_ncpserver = registry::getReg("DNS_NCPServer");
	my $fwdzone_context = registry::getReg("FWDZONE_CONTEXT");
	my $dns_locator = registry::getReg("DNS_LOCATOR_OBJECT");
	my $tempdir = registry::getReg("TEMPDIR");
	my $policyfile =  registry::getReg("DSSTATEDIR")."/domain/domain-object-modify-lockout-policy.ldif";
	my $dom_obj_file = registry::getReg("DSSTATEDIR"). "/domain/domain-object-modify.ldif";
	my $default_container = registry::getReg("DEFAULT_CONTAINER");
	my $revzone_context = registry::getReg("REVZONE_CONTEXT");

	mkpath  "$dsstatedir/dns";

	Log("   Generating domain-object.ldif");
	gen_ldif("$templatedir/domain-object.ini", '>', "$dsstatedir/domain/domain-object.ldif", "--defaultrootdomainobject");

	Log("   Generating domain-tcb.ldif");
	gen_ldif("$templatedir/domain-tcb.ini", '>', "$dsstatedir/domain/domain-tcb.ldif", "--defaultrootdomainobject");

	Log("   Generating domaindns-associate.ldif");
	gen_ldif("$templatedir/domaindns-associate.ini", '>', "$dsstatedir/domain/domaindns-associate.ldif", "--defaultrootdomainobject");

	Log("   Generating domain-object-modify.ldif");
	gen_ldif("$templatedir/domain-object-modify.ini",  '>', "$dsstatedir/domain/domain-object-modify.ldif", "--defaultrootdomainobject");
	`touch  $dsstatedir/domain/domain-object-modify-lockout-policy.ldif`;


	if (not registry::getReg("MAPPED")) {
		util::file_append_str("replace: intruderAttemptResetInterval\n", $policyfile);
		util::file_append_str("intruderAttemptResetInterval: 1800\n", $policyfile);
		util::file_append_str("-\nreplace: intruderLockoutResetInterval\n", $policyfile);
		util::file_append_str("intruderLockoutResetInterval: 1800\n", $policyfile);
		util::file_append_str("-\nreplace: loginIntruderLimit\n", $policyfile);
		util::file_append_str("loginIntruderLimit: -1\n", $policyfile);
	}
	else {
		my $LDAPSEARCH = registry::getReg("LDAPSEARCH");
		my $LDAPDOMAINADMINNAME = registry::getReg("LDAPDOMAINADMINNAME");
		my $MAPPEDDOMAINNC = registry::getReg("MAPPEDDOMAINNC");
		my $password = $ENV{'NDSEXISTINGADMINPASSWD'};
#		my $DSSTATEDIR = registry::getReg("DSSTATEDIR");
		my $puthyphen=0;

		`LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem /opt/novell/eDirectory/bin/ldapsearch -D "$LDAPDOMAINADMINNAME" -w "$password" -b "$MAPPEDDOMAINNC" -s base | grep 'intruderAttemptResetInterval' > /dev/null 2>&1;`;
		if ($? != 0 ) {
		      util::file_append_str("replace: intruderAttemptResetInterval\n", $policyfile);
		      util::file_append_str("intruderAttemptResetInterval: 1800\n", $policyfile);
		      $puthyphen++;
		}

		`LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem /opt/novell/eDirectory/bin/ldapsearch -D "$LDAPDOMAINADMINNAME" -w "$password" -b "$MAPPEDDOMAINNC" -s base | grep 'intruderLockoutResetInterval' > /dev/null 2>&1`;
		if ($? != 0 ) {
		       if($puthyphen != 0){
		       		util::file_append_str("-\n", $policyfile);
		       }
		       util::file_append_str("replace: intruderLockoutResetInterval\n", $policyfile);
                       util::file_append_str("intruderLockoutResetInterval: 1800\n", $policyfile);
		       $puthyphen++;
		}

		`LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem /opt/novell/eDirectory/bin/ldapsearch -D "$LDAPDOMAINADMINNAME" -w "$password" -b "$MAPPEDDOMAINNC" -e /etc/opt/novell/certs/SSCert.der -s base | grep 'loginIntruderLimit' > /dev/null 2>&1`;
		if ($? != 0 ) {
		       if($puthyphen != 0){
		       		util::file_append_str("-\n", $policyfile);
		       }
		        util::file_append_str("replace: loginIntruderLimit\n", $policyfile);
			util::file_append_str("loginIntruderLimit: -1\n", $policyfile);
		       $puthyphen++;
		}
	}

	if (-s $policyfile) {
		open (TEMPFILE, ">/tmp/TEMPLOCKOUTPOLICY.ldif") or die("Could not open the temporary file /tmp/TEMPLOCKOUTPOLICY.ldif");

		open DOMOBJ, $dom_obj_file or die("could not open the file $dom_obj_file");
		my $line1 = <DOMOBJ>;
		my $line2 = <DOMOBJ>;
		print TEMPFILE $line1;
		print TEMPFILE $line2;
		close DOMOBJ;

		open POLICYFILE, "$policyfile" or die("could not open the file $policyfile");
		while (<POLICYFILE>) {
			print TEMPFILE;
			
		}
		close(POLICYFILE);

		close TEMPFILE;

		unlink($policyfile) or die("Could not remove the file $policyfile");
		rename("/tmp/TEMPLOCKOUTPOLICY.ldif", $policyfile) or die("Could not rename the file /tmp/TEMPLOCKOUTPOLICY.ldif to $policyfile");
	}


	Log("   Generating domain.ldif");
	gen_ldif("$templatedir/domain.ini", '>',
		"$dsstatedir/domain/domain.ldif", "--defaultrootdomain");

	Log("	Generating the administrator");

	# gen_ldif("$templatedir/administrator.ini", '>',
	# 	 "$dsstatedir/domain/schema-mapping-for-administrator.ldif", "--ndstempcontainer");
	gen_ldif("$templatedir/administrator.ini", '>',
		 "$dsstatedir/domain/administrator-acl-on-root.ldif", "--rootnc");
	gen_ldif("$templatedir/administrator-delete.ini", '>',
		 "$dsstatedir/domain/administrator-delete.ldif", "--administrator");


	gen_ldif("$templatedir/administrator.ini", '>',
		"$dsstatedir/domain/administrator.ldif", "--defaultrootdomain");

	#Replace sn with space. The NAD plugins not loaded yet.
	system("sed -i 's/sn: replace-with-space/sn:: IA==/g' $dsstatedir/domain/administrator.ldif");
	if (-z "$dsstatedir/domain/administrator.ldif") {
		Log (ERR, "The LDIF file [$dsstatedir/domain/administrator.ldif] is empty");
	}


	Log("   Generating domain-bl.ldif");
	gen_ldif("$templatedir/domain-bl.ini", '>', "$dsstatedir/domain/domain-bl.ldif", "--defaultrootdomain");

	Log("   Generating partition container");
	gen_ldif("$templatedir/partition-container.ini", '>', "$dsstatedir/domain/partition-container.ldif", "--defaultconfignc");
	gen_ldif("$templatedir/partition-container.ini", '>>', "$dsstatedir/domain/partition-container.ldif", "--schema");

	Log("   Generating forest.ldif");
	gen_ldif("$templatedir/forest.ini", '>', "$dsstatedir/domain/forest.ldif", "--defaultrootdomain");

	Log("   Generating forest-bl.ldif");
	gen_ldif("$templatedir/forest-bl.ini", '>', "$dsstatedir/domain/forest-bl.ldif", "--defaultrootdomain");

	if (($fwdzone_context eq "") || ($revzone_context eq "")) {
		Log(ERR, "Forward and reverse  Zone context cannot be empty");
	}
	Log("   Generating zone-object.ldif");
	my $container_name = $default_container;
	$container_name =~ s/^ou=//;
	gen_ldif("$templatedir/zone-object.ini", '>', "$dsstatedir/dns/zone-object-temp.ldif", "--ndstempcontainer");
	`sed -e '/^dn: $default_container,/Id' $dsstatedir/dns/zone-object-temp.ldif | sed -e '/^OU: $container_name/Id' | sed -e "s/\#DNSDHCP-GROUP\#/\#$dnsdhcpgroup\#/g" | sed -e "s/{DNSServer}/$dnsserver/g" | sed -e "s/{DNS_NCPServer}/$dns_ncpserver/g" | sed -e "s/{DNSZONE_CONTEXT}/$fwdzone_context/g" > $dsstatedir/dns/zone-object.ldif`;
	unlink("$dsstatedir/dns/zone-object-temp.ldif");
	if (-z "$dsstatedir/dns/zone-object.ldif") {
		Log(ERR, "The LDIF file [$dsstatedir/dns/zone-object.ldif] generated is empty");
	}

	Log("   Generating DNS-modify.ldif");
	gen_ldif("$templatedir/DNS-modify.ini", '>', "$dsstatedir/dns/DNS-modify-temp.ldif", "--ndstempcontainer");
	`sed -e '/^dn: $default_container,/Id' $dsstatedir/dns/DNS-modify-temp.ldif | sed -e '/^OU: $container_name/Id' | sed -e "s/{DNSLocator}/$dns_locator/g" | sed -e "s/{DNSServer}/$dnsserver/g" | sed -e "s/{DNS_NCPServer}/$dns_ncpserver/g" | sed -e "s/{DNSZONE_CONTEXT}/$fwdzone_context/g" | sed -e 's/^-//g' > $dsstatedir/dns/DNS-modify.ldif`;
	unlink("$dsstatedir/dns/DNS-modify-temp.ldif");
	if (-z "$dsstatedir/dns/DNS-modify.ldif") {
		Log(ERR, "The LDIF file [$dsstatedir/dns/DNS-modify.ldif] generated is empty");
	}


	Log("   Generating DNS-modify-reverse-zones.ldif");
	gen_ldif("$templatedir/DNS-modify-reverse-zones.ini", '>', "$dsstatedir/dns/DNS-modify-reverse-zones-temp.ldif", "--ndstempcontainer");
	`sed -e '/^dn: $default_container,/Id' $dsstatedir/dns/DNS-modify-reverse-zones-temp.ldif | sed -e '/^OU: $container_name/Id' | sed -e "s/{DNSLocator}/$dns_locator/g" | sed -e "s/{DNSServer}/$dnsserver/g" | sed -e "s/{DNS_NCPServer}/$dns_ncpserver/g" | sed -e "s/{DNSZONE_CONTEXT}/$revzone_context/g" | sed -e 's/^-//g' > $dsstatedir/dns/DNS-modify-reverse-zones.ldif`;
	unlink("$dsstatedir/dns/DNS-modify-reverse-zones-temp.ldif");
	if (-z "$dsstatedir/dns/DNS-modify-reverse-zones.ldif") {
		Log(ERR, "The LDIF file [$dsstatedir/dns/DNS-modify-reverse-zones.ldif] generated is empty");
	}

	Log("   Generating reverse-zone-object.ldif");
	gen_ldif("$templatedir/reverse-zone-object.ini", '>', "$dsstatedir/dns/reverse-zone-object-temp.ldif", "--ndstempcontainer");
	`sed -e '/^dn: $default_container,/Id'  $dsstatedir/dns/reverse-zone-object-temp.ldif | sed -e '/^OU: $container_name/Id' | sed -e "s/\#DNSDHCP-GROUP\#/\#$dnsdhcpgroup\#/g" | sed -e "s/{DNSServer}/$dnsserver/g" | sed -e "s/{DNS_NCPServer}/$dns_ncpserver/g" | sed -e "s/{DNSZONE_CONTEXT}/$revzone_context/g" > $dsstatedir/dns/reverse-zone-object.ldif`;
	unlink("$dsstatedir/dns/reverse-zone-object-temp.ldif");
        if (-z "$dsstatedir/dns/reverse-zone-object.ldif") {
                Log(ERR, "The LDIF file [$dsstatedir/dns/reverse-zone-object.ldif] generated is empty");
        }

	#if NDS =TRUE
	Log("   Generating nldap.ldif");
	gen_ldif ("$templatedir/nldap.ini", '>', "$dsstatedir/domain/nldap.ldif", "--ndstempcontainer");

	Log("   Generating nldap-delete-classlist.ldif");
	gen_ldif ("$templatedir/nldap-delete-classlist.ini", '>', "$dsstatedir/domain/nldap-delete-classlist.ldif", "--ndstempcontainer");

	Log("   Generating nds-domain.ldif");
	gen_ldif("$templatedir/nds-domain.ini", '>', "$dsstatedir/domain/nds-domain.ldif", "--rootnc");

	Log("   Generating computer-container-policy.ldif");
	gen_ldif("$templatedir/computer-container-policy.ini", '>', "$dsstatedir/domain/computer-container-policy.ldif", "--defaultrootdomain");

	Log("   Generating nds-domain-server-reference.ldif");
	gen_ldif("$templatedir/nds-domain-server-reference.ini", '>', "$dsstatedir/domain/nds-domain-server-reference.ldif", "--ndstempcontainer");

	Log("   Generating nds-domain-container.ldif");
	gen_ldif("$templatedir/nds-domain-container.ini", '>', "$dsstatedir/domain/nds-domain-container.ldif", "--ndstempcontainer");

	Log("   Generating nds-admin-acls.ldif");
	gen_ldif("$templatedir/nds-admin-acls.ini", '>', "$dsstatedir/domain/nds-admin-acls.ldif", "--defaultparentnc");

	Log("   Generating nds-super-rights-acl.ldif");
	gen_ldif("$templatedir/nds-super-rights-acls.ini", '>', "$dsstatedir/domain/nds-super-rights-acls.ldif", "--rootnc");

	Log("   Generating nds-domain-rights-acls.ldif");
	gen_ldif("$templatedir/nds-domain-rights-acls.ini", '>', "$dsstatedir/domain/nds-domain-rights-acls.ldif", "--defaultrootdomain");

	Log("   Generating nds-adc-acls.ldif");
	gen_ldif("$templatedir/nds-adc-acls.ini", '>', "$dsstatedir/domain/nds-adc-acls.ldif", "--defaultconfignc");

	Log("   Generating nds-server.ldif");
	gen_ldif("$templatedir/nds-server.ini", '>', "$dsstatedir/domain/nds-server.ldif", "--defaultrootdomain");

	Log("   Generating nds-tree-key.ldif");
	gen_ldif("$templatedir/nds-tree-key.ini", '>', "$dsstatedir/domain/nds-tree-key.ldif`", "--security");

	Log(MESG,"   Domain related policies are generated");

	#if test "$(DEFAULTROOTDOMAIN)" != "$(LDAPFORESTNC)"; then has to be implemented for NON FRD

}

#Add the additional parameters to ndsd.conf
sub stage_ndsd {

	Log("   Adding additional parameters to ndsd.conf");

	my $livendsconf = registry::getReg("LIVE_NDSCONFDIR")."/nds.conf";
	my $ndsconf = registry::getReg("NDSCONFDIR")."/nds.conf";
	my $tempndsconf = registry::getReg("TEMPLATEDIR")."/nds.conf";
	my $liveetcdir = registry::getReg("LIVE_ETCDIR");
	my $tempdir = registry::getReg("TEMPDIR");

	if (-e $tempdir."/stage_ndsd") {
		Log("The configuration of ndsd was already done.");
		return 0;
	}
	
	#FIXME:Check if NDS is TRUE has to be checked?
eval{
	if (-f $livendsconf) {
		Log("   Copying $livendsconf to $ndsconf");
		copy ($livendsconf, $ndsconf);
	}

	Log("   Updating nds.conf");
	`provisionTools.sh "ndsconf-set"  -t $tempndsconf -s $ndsconf`;

	if ($? != 0) { die("provisionTool failed while calling for ndsconf-set") }


	Log("   Creating $liveetcdir and linking the conf file");
	mkpath ($liveetcdir) or Log(WARN,"Could not create the directory $liveetcdir");
	`(cd $liveetcdir; ln -sf $livendsconf .)`;

	my $tempndsmodule = registry::getReg("TEMPLATEDIR") . "/ndsmodules.conf";
	my $livendsmodule = registry::getReg("LIVE_NDSCONFDIR"). "/ndsmodules.conf";

	Log("   Creating $livendsmodule");
	reg_substitute($tempndsmodule, $livendsmodule);


	if (util::chkfor_reg("$livendsmodule") == 0) { die("Not all the REGISTRY values are replaced in $livendsmodule")};

};
	if ($@) { Log(ERR, $@) }

	`touch $tempdir/stage_ndsd`;
}

#Configuring kerberos
sub stage_krb5{

	my $dsstatedir = registry::getReg("DSSTATEDIR");
	my $tempdir = registry::getReg("TEMPDIR");
	my $templatedir = registry::getReg("TEMPLATEDIR");
	my $etcdir = registry::getReg("ETCDIR");
	my $stagedir = registry::getReg("STAGEDIR");
	my $liveetcdir = registry::getReg("LIVE_ETCDIR");
	my $keytab_file = registry::getReg("LIVE_DSSTATEDIR")."/krb5kdc/krb5.keytab";

	if (-e $tempdir."/stage_krb5") {
		Log("The kerberos configuration was already done.");
		return 0;
	}

eval{
	mkpath ("$dsstatedir/krb5kdc", 0, 0750);
	`chgrp named $dsstatedir/krb5kdc`;

	if ($? != 0) { Log(WARN, "Could not change the permission for $dsstatedir/krb5kdc"); }

	Log("   Generating $etcdir/krb5.conf");
	reg_substitute("$templatedir/krb5.conf", "$etcdir/krb5.conf");

	#validate the above generated file
	if (util::chkfor_reg("$etcdir/krb5.conf") == 0) { die("Not all the REGISTRY values are replaced in $etcdir/krb5.conf")};


	Log("   Generating $etcdir/kdc.conf");
	reg_substitute("$templatedir/kdc.conf", "$dsstatedir/krb5kdc/kdc.conf");

	if (util::chkfor_reg("$dsstatedir/krb5kdc/kdc.conf") == 0) { die("Not all the REGISTRY values are replaced in $dsstatedir/krb5kdc/kdc.conf")};


#	Kadmin is not beting used
#	Log("   Generating $templatedir/kadm5.conf");
#	reg_substitute("$templatedir/kadm5.acl", "$dsstatedir/krb5kdc/kadm5.acl");

#	`grep -q '\$REGISTRY'  $dsstatedir/krb5kdc/kadm5.acl`;
#	if ($? == 0) { die("Not all the REGISTRY values are replaced in $dsstatedir/krb5kdc/kadm5.acl")};

	mkpath("$etcdir/gss", 0, 0755);

	reg_substitute("$templatedir/mech", "$etcdir/gss/mech");

	if (util::chkfor_reg("$etcdir/gss/mech") == 0) { die("Not all the REGISTRY values are replaced in $etcdir/gss/mech")};

#	symlink("$stagedir/$liveetcdir/krb5.conf", "$stagedir/etc/krb5.conf");

	`(cd $stagedir/etc; ln -sf ..$liveetcdir/krb5.conf .)`;
	Log("   Linked $stagedir/$liveetcdir/krb5.conf to $stagedir/etc/krb5.conf");

	`(cd $stagedir/etc; ln -sf ..$keytab_file .)`;
	Log("   Linked $keytab_file to $stagedir/etc/krb5.keytab");

};
	if ($@) { Log(ERR, $@) }
	`touch $tempdir/stage_krb5`;
}

#Configuring samba
sub stage_samba {

	my $stagedir = registry::getReg("STAGEDIR");
	my $localstatedir = registry::getReg("LOCALSTATEDIR");
	my $templatedir = registry::getReg("TEMPLATEDIR");
	my $live_dsstatedir = registry::getReg("LIVE_DSSTATEDIR");
eval{
	mkpath "$stagedir/etc/samba" or Log(WARN,"Could not create the directory  $stagedir/etc/samba");
	mkpath "$localstatedir/profiles" or Log(WARN,"Could not create the directory $localstatedir/profiles");

	Log("   Creating $stagedir/etc/sysconfig/samba\n");
	reg_substitute("$templatedir/samba.sysconfig", "$stagedir/etc/sysconfig/samba");


	if (util::chkfor_reg("$stagedir/etc/sysconfig/samba") == 0) { 
		die("Not all the REGISTRY values are replaced in $stagedir/etc/sysconfig/samba")
	};


	`rm -f /etc/samba/secrets.tdb`;

	Log("   Writing $stagedir/etc/samba/MACHINE.SID\n");
	open FILE, ">$stagedir/etc/samba/MACHINE.SID" or die("Could not open the file $stagedir/etc/samba/MACHINE.SID");
	print FILE  registry::getReg("Domain SID");
	close FILE;
	Log("   Domain SID is ".registry::getReg("Domain SID")."\n");

	mkpath "$live_dsstatedir/backup" or Log(WARN,"could not create the directory $live_dsstatedir/backup");


};
	if ($@) { Log(ERR, $@) }

}

sub stage_sysvol
{
	my $DOMAIN_COMPUTERS_GID_OFFSET = 15;
	my $DOMAIN_USERS_GID_OFFSET = 13;
	my $live_local_statedir = registry::getReg("LIVE_LOCALSTATEDIR");
	my $dns_root = registry::getReg("DNSROOT");
	my $administrator_uid = registry::getReg("ADMINISTRATOR_UID");
	my $domain_admins_gid = registry::getReg("DOMAIN_ADMINS_GID");
	my $gp_creator_owner_gid = registry::getReg("GP_CREATOR_OWNER_GID");
	my $domain_computers_gid =$administrator_uid + $DOMAIN_COMPUTERS_GID_OFFSET;
	my $domain_users_gid=$administrator_uid + $DOMAIN_USERS_GID_OFFSET;

	Log(MESG,">>>Configuring System Volume");

	eval{

		Log("   Creating $live_local_statedir/msdfs");
		mkpath ("$live_local_statedir/msdfs");

		Log("   Changing ownership for $live_local_statedir/msdfs to $administrator_uid:$domain_admins_gid");
		`chown -R $administrator_uid:$domain_admins_gid $live_local_statedir/msdfs`;

		Log("   Creating $live_local_statedir/sysvol");
		mkpath ("$live_local_statedir/sysvol");
		`setfacl -m g:$domain_admins_gid:rwx $live_local_statedir/sysvol`;
		`setfacl -m g:$domain_computers_gid:r-x $live_local_statedir/sysvol`;
		`setfacl -m g:$domain_users_gid:r-x $live_local_statedir/sysvol`;
		`setfacl -m g:$gp_creator_owner_gid:rwx $live_local_statedir/sysvol`;
		`setfacl -m o:--- $live_local_statedir/sysvol`;
		`getfacl --access $live_local_statedir/sysvol | setfacl -d -M- $live_local_statedir/sysvol`;

		`setfacl -m d:g:$gp_creator_owner_gid:rwx $live_local_statedir/sysvol`;
		`setfacl -m d:g:$domain_computers_gid:r-x $live_local_statedir/sysvol`;
		`setfacl -m d:g:$domain_users_gid:r-x $live_local_statedir/sysvol`;
		`setfacl -m d:o:--- $live_local_statedir/sysvol`;
		Log("   Creating $live_local_statedir/sysvol/domain");
		mkpath("$live_local_statedir/sysvol/domain");

		Log("   Creating $live_local_statedir/sysvol/domain/Policies");
		mkpath("$live_local_statedir/sysvol/domain/Policies");

		`setfacl -m g:$gp_creator_owner_gid:rwx $live_local_statedir/sysvol/domain/Policies`;
		`chmod +t $live_local_statedir/sysvol/domain/Policies`;

		Log("   Creating $live_local_statedir/sysvol/domain/scripts");
		mkpath("$live_local_statedir/sysvol/domain/scripts");
		#Enabling "domain users" to access the files in [netlogon] share
		Log("   Executing \"chmod 771 $live_local_statedir/sysvol/domain/scripts\"");
		`chmod 771 $live_local_statedir/sysvol/domain/scripts`;

		Log("   Creating $live_local_statedir/sysvol/domain/scripts/Default User");
		mkpath("$live_local_statedir/sysvol/domain/scripts/Default User");

		Log("   Creating $live_local_statedir/sysvol/sysvol");
		mkpath("$live_local_statedir/sysvol/sysvol");

		Log("   Linking $dns_root");
		`(cd $live_local_statedir/sysvol/sysvol; ln -sf ../domain $dns_root)`;

		Log("   Linking $live_local_statedir/sysvol/sysvol/$dns_root");
		`(cd $live_local_statedir/sysvol/sysvol/$dns_root; ln -sf ../sysvol $live_local_statedir/sysvol/sysvol/$dns_root)`;

		Log("   Creating $live_local_statedir/sysvol/staging");
		mkpath ("$live_local_statedir/sysvol/staging");

		Log("   Creating $live_local_statedir/sysvol/staging areas");
		mkpath("$live_local_statedir/sysvol/staging areas");

		`(cd "$live_local_statedir/sysvol/staging areas"; ln -sf ../staging $dns_root)`;

		Log("   Changing ownership for $live_local_statedir/sysvol to $administrator_uid:$domain_admins_gid");
		`chown -R $administrator_uid:$domain_admins_gid $live_local_statedir/sysvol`;
		`chmod -R o-r $live_local_statedir/sysvol`;
		`chmod -R o-x $live_local_statedir/sysvol`;

	};

	if ($@) {  Log(ERR, $@); }

	Log(MESG, "   Sysvol Successfully configured");


}

sub stage_gpo {

	my $live_domaingpodir = registry::getReg("LIVE_DOMAINGPODIR");
	my $gpodir = registry::getReg("GPODIR");
	my $tempdir = registry::getReg("TEMPDIR");
	my $ADMINISTRATOR_UID = registry::getReg("ADMINISTRATOR_UID")+0; #Make sure its numeric, chown expects it as numeric
	my $DOMAIN_ADMINS_GID = registry::getReg("DOMAIN_ADMINS_GID")+0;

	if (-e $tempdir."/stage_gpo") {
		Log("The gpo creation was already done.");
		return 0;
	}

eval {
	mkpath("$live_domaingpodir");
	reg_substitute("$gpodir/gpt.ini", "$live_domaingpodir/gpt.ini");
	mkpath("$live_domaingpodir/MACHINE");
	mkpath("$live_domaingpodir/MACHINE/Microsoft/Windows NT/SecEdit");
	mkpath("$live_domaingpodir/USER");

	my $cnt = chown ($ADMINISTRATOR_UID, $DOMAIN_ADMINS_GID, "$live_domaingpodir");
	if($cnt != 1) { die ("could not change the ownership of $live_domaingpodir");}

};
	if ($@) { Log(ERR, $@) }
	`touch $tempdir/stage_gpo`;
}

#xadsd configuration is take care here
sub stage_xadsd {

	my $templatedir = registry::getReg("TEMPLATEDIR");
	my $etcdir = registry::getReg("ETCDIR");
	my $stagedir = registry::getReg("STAGEDIR");
	my $tempdir = registry::getReg("TEMPDIR");

	if (-e $tempdir."/stage_xadsd") {
		Log("The xadsd configuration already done.");
		return 0;
	}
eval {
	reg_substitute("$templatedir/xadss.conf", "$etcdir/xadss.conf");
	reg_substitute("$templatedir/xadsd.sysconfig", "$stagedir/etc/sysconfig/novell/xadsd");

	if (util::chkfor_reg("$etcdir/xadss.conf") == 0) { 
		die("Not all the REGISTRY values are replaced in $etcdir/xadss.conf")
	};
	if (util::chkfor_reg("$stagedir/etc/sysconfig/novell/xadsd") == 0) {
		die("Not all the REGISTRY values are replaced in $stagedir/etc/sysconfig/novell/xadsd")
	};

};
	if ($@) { Log(ERR, $@) }

	`touch $tempdir/stage_xadsd`;
}


sub stage_to_live{

	my $LIVE_LOCALSTATEDIR = registry::getReg("LIVE_LOCALSTATEDIR");
	my $LIVE_ETCDIR = registry::getReg("LIVE_ETCDIR");
	my $XADROOT = registry::getReg("XADROOT");
	my $STAGEDIR = registry::getReg("STAGEDIR");
	my $tempdir = registry::getReg("TEMPDIR");

	if (-e $tempdir."/stage_to_live") {
		Log("The configuration of stage-to-live was already done.");
		return 0;
	}

eval{
	Log("Removing $LIVE_LOCALSTATEDIR/samba");
	rmtree ("$LIVE_LOCALSTATEDIR/samba");

	Log("Removing $LIVE_ETCDIR/samba");
	rmtree ("$LIVE_ETCDIR/samba");

	Log("Removing /var".$XADROOT."/run");
	rmtree("/var".$XADROOT."/run");

	Log(MESG,"Making staging directory tree live");

#	`(cd $STAGEDIR; star -c --artype=exustar --acl . | (cd /; star -x --artype=exustar --acl ))`;
	`(cd $STAGEDIR; tar cpf - . | (cd /; tar xpf - ))`;

};
	if ($@) { Log(ERR, $@) }
	`touch $tempdir/stage_to_live`;
}

sub stage_clean{

	my $stagedir = registry::getReg("STAGEDIR");
	my $tmpdir = registry::getReg("LIVE_LOCALSTATEDIR")."/tmp";

	Log(MESG,">>>Cleaning staging directory tree");

eval{
	my $ndsdcrm_file = "/opt/novell/xad/sbin/ndsdcrm";
	rmtree($stagedir);
	rmtree($tmpdir);
	unlink($ndsdcrm_file);

};
	if ($@) { Log(WARN, $@) }
	Log(MESG, "  Staging directories are removed");
}

sub stage_site
{
	my $templatedir = registry::getReg("TEMPLATEDIR");
	my $dsstatedir = registry::getReg("DSSTATEDIR");
	my $forest_root = registry::getReg("FORESTROOT");
	my $new_site = registry::getReg("SITE");
	my $found = 0;
	Log(INFO, "New Site : $new_site");
	Log(INFO, "Forest Root : $forest_root");

	eval{
		#Check if the site exists or not.
		my $site_list = `/opt/novell/xad/share/dcinit/provisionTools.sh \\
					get-site-list -d $forest_root`;
		my $search_err = $?>>8;
		chomp($site_list);
		if ($search_err) {
			die("Could not get the sites [$search_err] [$site_list]");
		}

		Log(INFO, "List of sites : $site_list");
		my @list_of_sites = split(' ', $site_list);

		foreach my $site (@list_of_sites) {
			if ("$new_site" eq "$site") {
				$found = 1;
			}
		}

		#Site does not exists
		if (!$found) {
			#This is to generate the new site object's into the site object.
			stage::gen_ldif("$templatedir/config-site.ini", '>',
					"$dsstatedir/config/config-site.ldif",
					"--defaultconfignc");
		}

	};

	if ($@) {
		Log(ERR, $@);
	}
}

1;
