#!/usr/bin/perl
################################################################################
# Domain Services for Windows Provisioning Tool
#
# See the file LICENSE for redistribution information.
#
# Copyright (c) 2001-2004
#	Novell, Inc.  All rights reserved.
################################################################################
#
#   NAME:
#         registry.pl
#
#   FACILITY:
#
#         Domain Controller Initialization
#
#   ABSTRACT:
#
#         This module is responsible for the configuration defaults of a
#         newly created domain.
#
#   REVISION:
#
#         $Id$
#
#################################################################################

package XAD;

use XAD::registry;
use MIME::Base64;

my $rcsId = '$Id$';

#FIXME: Move this to some other place.
#Due to this registry is getting defined here recursively.
sub regSubstitute
{
	my ($text) = $_[0];

	# Regex gurus must know a better way
	if ($text =~ /^\$EMBEDDED:(\d+):/) {
		$text =~ s/^\$EMBEDDED/B/;
	} elsif ($text =~ /^\$EMBEDDED:/) {
		$text =~ s/^\$EMBEDDED://;
	}

	foreach (keys %registry::REGISTRY) {
		$text =~ s/\$\bREGISTRY=$_\b/$registry::REGISTRY{$_}/g;
	}
	return $text;
}

sub domainToDC
{
	my @dnc = split(/\./, ".$_[0]");

	return substr(join(",DC=", @dnc), 1);
}

sub reverseIPAddress
{
	my ($ip) = $_[0];
	my (@ip, $rev, @rev);
	
	my $out = `/sbin/ip addr`;
	chomp($out);
	$out =~ /$ip\/(\d*).*/;
	my $subnetmask = $1;

	if ($subnetmask == "") { 
	    $subnetmask = 24; #we assume default is class C
	}
	$numofoctet = int( ($subnetmask / 8) + .5); #flooring to upper value
	@ip = split /\./, $ip;
	@rev = reverse @ip;

	#Taking the zone name as required
	#If ip is 192.168.3.10 netmask is 24 then 3.168.192.in-addr.arpa will be zone name
	#and 10 will be a PTR record.
	$rev = join ('.', @rev[(4 - $numofoctet)..3], "in-addr.arpa.");

	#Remaining will be added as PTR record of the zone
	return ($rev, join('.',@rev[0..(3 - $numofoctet)]));
}
sub getHostName
{
	my $hostname = `/bin/hostname`;
	chop($hostname);
	$hostname =~ s/\.(.*)//g;

	return uc($hostname);
}

#FIXME: Move to some other place so that registry package does not get redfined
sub dumpRegistry
{
	foreach (keys %REGISTRY) {
		print "$_: $REGISTRY{$_}\n";
	}
}

sub unpackSid
{
	my $packedSid = $_[0];
	my ($Revision, $SubAuthorityCount, $IdentifierAuthority);
	my $i;

	($Revision, $SubAuthorityCount) = unpack("CC", $packedSid);
	$IdentifierAuthority = unpack("C", substr($packedSid, 7));

	$stringSid = "S-$Revision-$IdentifierAuthority";
	for ($i = 0; $i < $SubAuthorityCount; $i++) {
		$stringSid .= "-" . unpack("V", substr($packedSid, 8 + (4 * $i)));
	}

	return $stringSid;
}

# Pack a Security Identifier string into a little-endian
# binary blob.
sub packSid
{
	my $stringSid = $_[0];
	my @Data;
	my ($Revision, $SubAuthorityCount, $IdentifierAuthority, @SubAuthority);
	my $packedSid, $i;

	if (!($stringSid =~ /^S-/)) {
		return;
	}
	@Data = split(/-/, substr($stringSid, 2));
	$Revision = $Data[0];
	$IdentifierAuthority = $Data[1];
	$SubAuthorityCount = $#Data - 1;

	$packedSid = pack("C8", $Revision, $SubAuthorityCount, 0, 0, 0, 0, 0, $IdentifierAuthority);
	for ($i = 0; $i < $SubAuthorityCount; $i++) {
		$packedSid .= pack("V", $Data[2 + $i]);
	}

	return $packedSid;
}

sub sddl2hex
{
	my ($domainSid, $ownerRid, $groupRid, $stringSD) = @_;
	my $hexSD = `sddl2hex "$domainSid" "$ownerRid" "$groupRid" "$stringSD"`;
	if (!($hexSD =~ /\\x/)) {
		return;
	}

	chop($hexSD);

	return $hexSD;
}

sub packSecurityDescriptor
{
	my ($domainSid, $ownerRid, $groupRid, $stringSD) = @_;
	my $hexSD;
	my $packedSD;

	$hexSD = sddl2hex($domainSid, $ownerRid, $groupRid, $stringSD);
	$packedSD = pack("H*", $hexSD);

	return $packedSD;
}

sub unpackGuidLittleEndian
{
	my $packedGuid = $_[0];
	sprintf "%08x-%04x-%04x-%02x%02x-%02x%02x%02x%02x%02x%02x", unpack("VvvC8", $packedGuid);
}

sub unpackGuidBigEndian
{
	my $packedGuid = $_[0];
	sprintf "%08x-%04x-%04x-%02x%02x-%02x%02x%02x%02x%02x%02x", unpack("NnnC8", $packedGuid);
}

sub unpackGuid
{
	return unpackGuidLittleEndian(@_);
}

# Pack a GUID string into a little-endian binary blob.
sub packGuid
{
	my $stringGuid = $_[0];
	my ($Data1, $Data2, $Data3, $Data4, $Data5) = split(/-/, $stringGuid);
	my (@Data4, @Data5);

	$Data1 = hex($Data1);
	$Data2 = hex($Data2);
	$Data3 = hex($Data3);
	$Data4[0] = hex(substr($Data4, 0, 2));
	$Data4[1] = hex(substr($Data4, 2, 4));
	$Data5[0] = hex(substr($Data5, 0, 2));
	$Data5[1] = hex(substr($Data5, 2, 2));
	$Data5[2] = hex(substr($Data5, 4, 2));
	$Data5[3] = hex(substr($Data5, 6, 2));
	$Data5[4] = hex(substr($Data5, 8, 2));
	$Data5[5] = hex(substr($Data5, 10, 2));

	return pack("VvvC8", $Data1, $Data2, $Data3, @Data4, @Data5);
}

sub encodeBase64
{
	return encode_base64($_[0], "");
}


sub decodeBase64
{
	return decode_base64($_[0]);
}

sub getOperatingSystem
{
	#open(RELEASE, "/etc/SuSE-release") || die "Failed to open /etc/SuSE-release";
	#my $suseRelease = <RELEASE>;
	#chop($suseRelease);
	
	open(RELEASE, "/etc/os-release") || die "Failed to open /etc/os-release";
	my $suseRelease = "";
	while(<RELEASE>)
	{
		if ($_ =~ /^\s*PRETTY_NAME\s*=\s*/i) {
			chomp($_);
			chop($_);
			$suseRelease = (split /=/,$_)[1];
			$suseRelease =~ s/^.//s;
		}
	}

	close(RELEASE);

	return $suseRelease;
}

sub getOperatingSystemVersion
{
	my $uname = `/bin/uname -r`;
	chop($uname);

	return $uname;
}

1;

