#!/usr/bin/perl
################################################################################
# Domain Services for Windows Provisioning Tool
#
# See the file LICENSE for redistribution information.
#
# Copyright (c) 2001-2006
#	Novell, Inc.  All rights reserved.
################################################################################
#
#   NAME:
#         ndsschema.pm
#
#   FACILITY:
#
#         Domain Controller Initialization
#
#   ABSTRACT:
#
#         This module is responsible for mapping an Active Directory 
#         schema to an eDirectory schema.
#
#   REVISION:
#
#         $Id$
#
#################################################################################

package schema;

use XAD::dclib;
use XAD::ndsschemamap;

my $rcsId = '$Id$';

#
# NDS syntax table. Note that where multiple syntaxes are mapped to
# a single NDS syntax, there will often be an AD-aware schema plugin
# that will do validation and additional mapping of presentation
# syntaxes. So this mapping is not as lossy as it would seem from
# inspecting the table below.
#
$NDS_SYNTAXES{"2.5.5.1"} = "SYN_DIST_NAME";	# distinguishedName
#$NDS_SYNTAXES{"2.5.5.2"} = "SYN_CLASS_NAME";	# Object-Identifier
$NDS_SYNTAXES{"2.5.5.2"} = "SYN_CI_STRING";	# directoryString (Object-Identifier)
$NDS_SYNTAXES{"2.5.5.3"} = "SYN_CE_STRING";	# directoryString
$NDS_SYNTAXES{"2.5.5.4"} = "SYN_CI_STRING";	# directoryString
$NDS_SYNTAXES{"2.5.5.5:19"} = "SYN_PR_STRING";	# printableString
$NDS_SYNTAXES{"2.5.5.5:22"} = "SYN_CE_STRING";	# IA5String
$NDS_SYNTAXES{"2.5.5.6"} = "SYN_NU_STRING";	# numericString
$NDS_SYNTAXES{"2.5.5.7"} = "SYN_PATH";		# orName/dnWithOctetString
$NDS_SYNTAXES{"2.5.5.8"} = "SYN_BOOLEAN";	# boolean
$NDS_SYNTAXES{"2.5.5.9"} = "SYN_INTEGER";	# integer
$NDS_SYNTAXES{"2.5.5.10"} = "SYN_OCTET_STRING";	# octetString
$NDS_SYNTAXES{"2.5.5.11"} = "SYN_TIME";		# utcTime/generalizedTime
$NDS_SYNTAXES{"2.5.5.12"} = "SYN_CI_STRING";	# directoryString (Unicode)
$NDS_SYNTAXES{"2.5.5.13"} = "SYN_OCTET_STRING";	# presentationAddress
$NDS_SYNTAXES{"2.5.5.14"} = "SYN_OCTET_STRING";	# accessPoint
$NDS_SYNTAXES{"2.5.5.15"} = "SYN_OCTET_STRING";	# Object-Security-Descriptor
$NDS_SYNTAXES{"2.5.5.16"} = "SYN_INTEGER64";	# Large-Integer
$NDS_SYNTAXES{"2.5.5.17"} = "SYN_OCTET_STRING";	# Object-SID

#
# Adjust exceptions according to internally defined schema
# (these are for eDirectory)
#
@NDS_ATTRIBUTEID_EXCEPTIONS = (
	"2.5.4.0", # objectClass
	"2.5.4.1", # aliasedObjectName
	"2.5.4.3", # commonName
	"2.5.4.13", # description
	"2.5.4.34", # seeAlso
	"2.5.4.35", # userPassword
	"2.5.18.1", # createTimestamp
	"2.5.18.2", # modifyTimestamp
	"2.5.18.3", # creatorsName
	"2.5.18.4", # modifersName
	"2.5.21.5", # attributeTypes
	"2.5.21.6", # objectClasses
	"2.5.21.9", # structuralObjectClass
	"2.16.840.1.113719.1.8.4.2", # assistant
	"0.9.2342.19200300.100.1.38", # associatedName
	"0.9.2342.19200300.100.1.55", # audio
	"2.5.4.38", # authorityRevocationList
	"2.5.4.15", # businessCategory
	"2.5.4.6", # c
	"2.5.4.37", # cACertificate
	"2.16.840.1.113730.3.1.1", # carLicense
	"2.5.4.39", # certificateRevocationList
	"0.9.2342.19200300.100.1.43", # co
	"2.16.840.1.113719.1.8.4.5", # company
	"2.5.4.40", # crossCertificatePair
	"0.9.2342.19200300.100.1.25", # dc
	"2.5.4.53", # deltaRevocationList
	"2.16.840.1.113730.3.1.2", # departmentNumber
	"2.5.4.27", # destinationIndicator
	"2.16.840.1.113719.1.8.4.6", # directReports
	"2.16.840.1.113730.3.1.241", # displayName
	"2.16.840.1.113730.3.1.4", # employeeType
	"0.9.2342.19200300.100.1.20", # homePhone
	"0.9.2342.19200300.100.1.39", # homePostalAddress
	"2.5.4.51", # houseIdentifier
	"2.5.4.43", # initials
	"0.9.2342.19200300.100.1.60", # jpegPhoto
	"2.5.4.2", # knowledgeInformation
	"2.5.4.7", # l
	"0.9.2342.19200300.100.1.10", # manager
	"2.5.4.31", # member
	"0.9.2342.19200300.100.1.41", # mobile
	"2.5.4.10", # o
	"2.16.840.1.113719.1.1.4.1.602", # objectVersion
	"2.5.4.11", # ou
	"2.5.4.32", # owner
	"0.9.2342.19200300.100.1.42", # pager
	"0.9.2342.19200300.100.1.40", # personalTitle
	"0.9.2342.19200300.100.1.7", # photo
	"2.5.4.28", # preferredDeliveryMethod
	"2.16.840.1.113730.3.1.39", # preferredLanguage
	"2.5.4.29", # presentationAddress
	"2.5.4.26", # registeredAddress
	"2.16.840.1.113719.1.1.4.1.122", # revision
	"0.9.2342.19200300.100.1.6", # roomNumber
	"2.5.4.14", # searchGuide
	"2.16.840.1.113719.1.27.4.5", # searchTimeLimit
	"0.9.2342.19200300.100.1.21", # secretary
	"2.5.4.30", # supportedApplicationContext
	"2.5.4.22", # teletexTerminalIdentifier
	"2.5.4.21", # telexNumber
	"2.5.4.12", # title
	"2.5.4.31", # uniqueMember
	"2.5.4.36", # userCertificate
	"2.16.840.1.113730.3.1.216", # userPKCS12
	"2.16.840.1.113730.3.1.40", # userSMIMECertificate
	"2.5.4.24", # x121Address
	);

@NDS_DEMOTED_ATTRIBUTES = (
	"nTSecurityDescriptor",
	"defaultObjectCategory",
	"rIDNextRID",
	"rIDPreviousAllocationPool",
	);

@NDS_OPERATIONAL_ADDITIONS = (
#	"nTSecurityDescriptor",
	"replPropertyMetaData"
	);

@NDS_GOVERNSID_EXCEPTIONS = (
	"2.5.6.1", # alias
	"2.5.20.1", # subSchema
	);

# These are the NIS related classes which would be extended by using
# NAM.SCH, but as some other classes in MS schema derive their attributes
# from these classes, as these classes are defined as 'systemAuxiliaryClass'
# for them, we need to have their defination in lidf, but would not extend them.

@NDS_NISCLASS_EXCEPTIONS = (
	"mailRecipient",
	"posixAccount",
	"posixGroup",
	"ipHost",
	"ieee802Device",
	"bootableDevice",
	);
	
@NDS_AUXILIARY_ATTRIBUTE_EXCEPTIONS = (
	"domainDNS",
	"xadFlags",
	);
#
# These are attributes that either do not exist at the NDS layer
# or are reserved names/operational attributes in our implementation
#
# The difference with the attribute ID exceptions is that these
# attributes are not included in object class definitions either
# (which are referenced by LDAP display name)
#
@NDS_LDAPDISPLAYNAME_EXCEPTIONS = (
	"attributeTypes",
	"createTimeStamp",
	"modifyTimeStamp",
	"objectClass",
	"userPassword",
	"pwdLastSet",
	"structuralObjectClass",
	"subSchemaSubEntry"
	);

#
# Containers to add "container" containment rule to
#
@NDS_COMPAT_CONTAINERS = (
	"nspmPasswordPolicyContainer",
	);

@NDS_COUNTER_ATTRIBUTE_EXCEPTIONS = (
	"msDS-KeyVersionNumber",
	"revision",
#	"rIDNextRid",
#	"nextRid"
	);

#
# Attributes that we want to map we still define the schema for
# (for now at least)
#
@NDS_RFC2307_ATTRIBUTE_EXCEPTIONS = (
	"unixHomeDirectory"
	);

sub ldap2NdsOid
{
	my ($dotOid) = @_;
	my $braceOid;
	my @oidElements;

	@oidElements = split(/\./, $dotOid);
	$braceOid = "{" . join(" ", @oidElements) . "}";

	return $braceOid;
}

sub isInList
{
	my ($what, $list) = @_;
	my $iterator;

	foreach $iterator (@$list) {
		if (uc($iterator) eq uc($what)) {
			return 1;
		}
	}

	return 0;
}

sub isBoundableNdsSyntax
{
	my ($ndsSyntax) = @_;

	if ($ndsSyntax eq "SYN_DIST_NAME" ||
	    $ndsSyntax eq "SYN_PATH" ||
	    $ndsSyntax eq "SYN_INTEGER64") {
		return 0;
	}

	return 1;
}

sub emitNdsSchemaPreamble
{
	my ($ldapDisplayName, $type, $schUpdateOp, $pass) = @_;

	print "\"";
	if ($type eq "OBJECT-CLASS" && defined($NDS_CLASS_MAP{$ldapDisplayName})) {
		print $NDS_CLASS_MAP{$ldapDisplayName};
	} elsif ($type eq "ATTRIBUTE" && defined($NDS_ATTRIBUTE_MAP{$ldapDisplayName})) {
		print $NDS_ATTRIBUTE_MAP{$ldapDisplayName};
	} else {
		print substr($ldapDisplayName, 0, 32);
	}
	print  "\" $type ::=\n";
	print "{\n";
	print "\tOperation\t" . $schUpdateOp . ",\n";
}

sub emitNdsSchemaPostamble
{
	print "}\n\n";
}

sub emitNdsServerObjectExtensions
{
	my @mayContain = ( "serverReference" );

	&emitNdsSchemaPreamble("Server", "OBJECT-CLASS", "MODIFY");
print <<EOL
	MayContain	{"serverReference",
			 "crossRefReference"}
EOL
;
	&emitNdsSchemaPostamble();
}

sub emitNdsAttributeSchema
{
	my ($ldif) = @_;
	my $syntax, $ndsSyntax;
	my $omSyntax, $omObjectClass;
	my $isOperational;
	my $desc;
	my $attributeID;
	my $isOperational = 0;
	my $isNonReplicated = 0;
	my $schUpdateOp;
	my @flags;
	my $ldapDisplayName;

	if (!defined($ldif->{"ATTRIBUTESYNTAX"})) {
		die $ldif->{'DN'}->[0] . ": Attribute syntax undefined";
	}

	if (!defined($ldif->{"ATTRIBUTEID"})) {
		die $ldif->{'DN'}->[0] . ": Attribute OID undefined";
	}

	if (!defined($ldif->{"LDAPDISPLAYNAME"}->[0])) {
		die $ldif->{'DN'}->[0] . ": Attribute LDAP display name undefined";
	}

	return if &isInList($ldif->{"ATTRIBUTEID"}->[0], \@NDS_ATTRIBUTEID_EXCEPTIONS);
	return if &isInList($ldif->{"LDAPDISPLAYNAME"}->[0], \@NDS_LDAPDISPLAYNAME_EXCEPTIONS);
	

#	if ($ldif->{"ATTRIBUTEID"}->[0] eq "2.16.840.1.113730.3.1.34") {
#		# 'ref' attribute, MS think it is 'middleName'
#		# XXX this is a kludge
#		$attributeID = "2.16.840.1.113719.1.513.2.28";
#	} else {
#		$attributeID = $ldif->{"ATTRIBUTEID"}->[0];
#	}

	if (defined($NDS_OID_MAP{$ldif->{"ATTRIBUTEID"}->[0]}))
	{
		$attributeID = $NDS_OID_MAP{$ldif->{"ATTRIBUTEID"}->[0]};
	} else {
		$attributeID = $ldif->{"ATTRIBUTEID"}->[0];
	}


	if (!defined($ldif->{"LDAPDISPLAYNAME"})) {
		die $ldif->{'DN'}->[0] . ": LDAP display name undefined";
	}
	$ldapDisplayName = $ldif->{"LDAPDISPLAYNAME"}->[0];
	if (defined($NDS_ATTRIBUTE_MAP{$ldapDisplayName}) &&
	    !($NDS_ATTRIBUTE_MAP{$ldapDisplayName} =~ /^mSDS:/i || &isInList($ldapDisplayName, \@NDS_RFC2307_ATTRIBUTE_EXCEPTIONS)))
	{
		return;
	}

	$syntax = $ldif->{"ATTRIBUTESYNTAX"}->[0];
	$omSyntax = $ldif->{"OMSYNTAX"}->[0];
	$omObjectClass = XAD::encodeBase64($ldif->{"OMOBJECTCLASS"}->[0]);

	if (defined($NDS_SYNTAXES{$syntax})) {
		$ndsSyntax = $NDS_SYNTAXES{$syntax};
	} elsif (defined($NDS_SYNTAXES{"$syntax:$omSyntax"})) {
		$ndsSyntax = $NDS_SYNTAXES{"$syntax:$omSyntax"};
	} elsif (defined($NDS_SYNTAXES{"$syntax:$omObjectClass"})) {
		$ndsSyntax = $NDS_SYNTAXES{"$syntax:$omObjectClass"};
	} else {
		print STDERR "No NDS syntax map for $syntax\n";
		exit 1;
	}
	if ($ndsSyntax eq "SYN_INTEGER" && &isInList($ldapDisplayName, \@NDS_COUNTER_ATTRIBUTE_EXCEPTIONS)) {
		$ndsSyntax = "SYN_COUNTER";
	}

	$schUpdateOp = "ADD";

	if ($ldif->{"SYSTEMFLAGS"}->[0] & 0x00000004) {
		$isOperational = 1;
	} else {
		$isOperational = &isInList($ldapDisplayName, \@NDS_OPERATIONAL_ADDITIONS);
	}
	if ($ldif->{"SYSTEMFLAGS"}->[0] & 0x00000001) {
		$isNonReplicated = 1;
	} else {
		$isNonReplicated = &isInList($ldapDisplayName, \@NRA_EXCEPTIONS);
	}

	if (grep(/TRUE/i, @{$ldif->{"ISSINGLEVALUED"}})) {
		push(@flags, "DS_SINGLE_VALUED_ATTR");
	}
	if (defined($ldif->{"RANGEUPPER"}) && &isBoundableNdsSyntax($ndsSyntax)) {
		push(@flags, "DS_SIZED_ATTR");
	}
	if ($isOperational) {
		push(@flags, "DS_OPERATIONAL");
	}
	if ($isNonReplicated) {
		push(@flags, "DS_PER_REPLICA");
	}
	if (&isInList($ldapDisplayName, \@NDS_SERVER_READ_ATTRIBUTES)) {
		push(@flags, "DS_SERVER_READ");
	}
	if (&isInList($ldapDisplayName, \@NDS_HIDDEN_ATTRIBUTES)) {
		push(@flags, "DS_HIDDEN_ATTR");
		push(@flags, "DS_READ_ONLY_ATTR");
	}
	if (&isInList($ldapDisplayName, \@NDS_READ_ONLY_ATTRIBUTES)) {
		push(@flags, "DS_READ_ONLY_ATTR");
	}
	if ($ndsSyntax eq "SYN_INTEGER64") {
		$ndsSyntax = "SYN_OCTET_STRING";
		# Uncomment this when eDirectory supports 64-bit integers
		push(@flags, "DS_INTEGER64");
	}
	push(@flags, "DS_SYNC_IMMEDIATE");

	print "-- Attribute definition for " . $ldif->{"CN"}->[0] . "\n";

	&emitNdsSchemaPreamble($ldapDisplayName, "ATTRIBUTE", $schUpdateOp);

	print "\tSyntaxID\t" . $ndsSyntax . ",\n";
	print "\tFlags\t\t{" . join(", ", @flags) . "},\n";
	if (defined($ldif->{"RANGEUPPER"}) && &isBoundableNdsSyntax($ndsSyntax)) {
		my $rangeLower, $rangeUpper;

		$rangeLower = defined($ldif->{"RANGELOWER"}) ? $ldif->{"RANGELOWER"}->[0] : 0;
		$rangeUpper = $ldif->{"RANGEUPPER"}->[0];

		print "\tLowerBound\t$rangeLower,\n";
		print "\tUpperBound\t$rangeUpper,\n";
	}
	print "\tASN1ObjID\t" . &ldap2NdsOid($attributeID) . "\n";

	&emitNdsSchemaPostamble();
}

sub emitNdsObjectClassAttrs
{
	my ($ldif, $tag, $attrs, $except) = @_;
	my $ldapDisplayName, $first = 1;
	my %seen = ();
	my @uniq;

	if (scalar(@$attrs) == 0) {
		return;
	}

	# First, uniq the array because the coalescing of auxiliary classes
	# may have added duplicate attribute names. Sort it for good measure
	# too.
	foreach $ldapDisplayName (@$attrs) {
		$seen{$ldapDisplayName}++;
	}
	@uniq = sort keys %seen;

	foreach $ldapDisplayName (@uniq) {
		next if &isInList($ldapDisplayName, $except);

		if ($first) {
			print "\t$tag\t{";
			$first = 0;
		} else {
			print ",\n\t\t\t";
		}

		if (defined($NDS_ATTRIBUTE_MAP{$ldapDisplayName})) {
			$ldapDisplayName = $NDS_ATTRIBUTE_MAP{$ldapDisplayName};
		}

		# truncate attribute names to 32 characters for NDS compat
		print "\"" . substr($ldapDisplayName, 0, 32) . "\"";
	}

	if ($first == 0) {
		print "},\n";
	}
}

sub emitNdsContainmentRules
{
	my ($containedBy, $pass) = @_;
	my $superior, $first = 1,$prevSuperior="";
	my @tempContainedBy = ();

	if (scalar(@$containedBy) == 0) {
		return;
	}

	foreach $superior (@containedBy) {
		if ($superior ne "domainDNS" ) {
			push(@tempContainedBy, $superior);
		} else {
			push(@tempContainedBy, "organizationalUnit");
			push(@tempContainedBy, "organization");
			push(@tempContainedBy, "domain");
			push(@tempContainedBy, "locality");
			push(@tempContainedBy, "country");
		}
	}

	@tempContainedBy = sort(@tempContainedBy);
	@containedBy = ();

	foreach $superior (@tempContainedBy) {
		if ($superior ne $prevSuperior) {
			push(@containedBy, $superior);
		}
		$prevSuperior = $superior;
	}

	print "\tContainedBy\t{";
	foreach $superior (@$containedBy) {
		if ($first == 1) {
			$first = 0;
		} else {
			print ",\n\t\t\t";
		}
		if (defined($NDS_CLASS_MAP{$superior})) {
			$superior = $NDS_CLASS_MAP{$superior};
		}
		print "\"" . substr($superior, 0, 32) . "\"";
	}
	print "}";
	if ($pass == 0) {
		print ",";
	}
	print "\n";
}

sub emitNdsInheritenceRules
{
	my ($ldif, $subClassOf) = @_;
	my $sup, $first = 1;

	if (scalar(@$subClassOf) == 0) {
		return;
	}

	print "\tSubClassOf\t{";
	if (($ldif->{"LDAPDISPLAYNAME"}->[0] eq "inetOrgPerson") && (scalar(@$subClassOf) == 1))
	{
		@subClassOf = "Organizational Person";
	}
	foreach $sup (@$subClassOf) {
		if ($first == 1) {
			$first = 0;
		} else {
			print ",\n\t\t\t";
		}
		if (defined($NDS_CLASS_MAP{$sup})) {
			$sup = $NDS_CLASS_MAP{$sup};
		}
		print "\"" . substr($sup, 0, 32) . "\"";
	}
	print "},";
	print "\n";
}

sub flattenAuxClasses
{
	my ($key, $classes) = @_;
	my @auxiliaryClasses = ();
	my $ldifP;

	$ldifP = $classes->{$key};

	while (defined($ldifP) &&
		uc($ldifP->{"SUBCLASSOF"}->[0]) ne uc($ldifP->{"LDAPDISPLAYNAME"}->[0])) {
		my $auxClass, $superClass;

		$superClass = $ldifP->{"SUBCLASSOF"}->[0];

		foreach $auxClass (@{$ldifP->{'AUXILIARYCLASS'}}, @{$ldifP->{'SYSTEMAUXILIARYCLASS'}}) {
			push(@auxiliaryClasses, $auxClass);
			push(@auxiliaryClasses, &flattenAuxClasses($auxClass, $classes));
		}

		$ldifP = $classes->{$superClass};
	}

	return @auxiliaryClasses;
}

sub flattenContainers
{
	my ($key, $classes) = @_;
	my %possSuperiors = ();
	my @uniq;
	my $ldifP;

	$ldifP = $classes->{$key};

	while (defined($ldifP) &&
		uc($ldifP->{"SUBCLASSOF"}->[0]) ne uc($ldifP->{"LDAPDISPLAYNAME"}->[0])) {
		my $superior, $superClass;

		$superClass = $ldifP->{"SUBCLASSOF"}->[0];

		foreach $superior (@{$ldifP->{'POSSSUPERIORS'}}, @{$ldifP->{'SYSTEMPOSSSUPERIORS'}}) {
			next if &isInList($superior, \@NDS_NOT_CONTAINER_CLASSES);
			$possSuperiors{$superior}++;
		}

		$ldifP = $classes->{$superClass};
	}

	return sort keys %possSuperiors;
}

sub emitNdsObjectClassSchema
{
	my ($key, $classes, $pass) = @_;
	my $ldif = $classes->{$key}, $ldifP;
	my $type;
	my $desc;
	my @flags = (), @subClassOf = (), $rDNAttID;
	my $auxiliaryClass;
	my $isNdsSystemObjectClass = 0;
	my $schUpdateOp;
	my $ldapDisplayName;
	my $governsID;
	my @mayContain = (), @mustContain = (), @auxiliaryClasses = (), @containedBy = ("top");
	my @demotedAttributes = ();
	my $isTop = 0;
	my $isContainer;

	if (!defined($ldif->{"OBJECTCLASSCATEGORY"})) {
		die $ldif->{'DN'}->[0] . ": object category name undefined";
	}
	if (!defined($ldif->{"LDAPDISPLAYNAME"})) {
		die $ldif->{'DN'}->[0] . ": object class display name undefined";
	}
	$ldapDisplayName = $ldif->{"LDAPDISPLAYNAME"}->[0];
	if (!defined($ldif->{"GOVERNSID"})) {
		die $ldif->{'DN'}->[0] . ": object class OID undefined";
	}

	if(defined($NDS_OID_MAP{$ldif->{"GOVERNSID"}->[0]})) {
		$governsID = $NDS_OID_MAP{$ldif->{"GOVERNSID"}->[0]};
	} else {
		$governsID = $ldif->{"GOVERNSID"}->[0];
	}

	return if &isInList($governsID, \@NDS_GOVERNSID_EXCEPTIONS);
	return if &isInList($ldapDisplayName, \@NDS_NISCLASS_EXCEPTIONS);

	#
	# All object classes defined in the class map are marked as system
	# object classes, ie. they are existing classes to be extended
	# rather than added. The exception is classes that begin with the
	# prefix "mSDS:", which is used to disambiguate naming conflicts
	# with NDS classes.
	#
	if (defined($NDS_CLASS_MAP{$ldapDisplayName}) &&
	    !($NDS_CLASS_MAP{$ldapDisplayName} =~ /^mSDS:/i)) {
		$isNdsSystemObjectClass = 1;
	}

	$type = $ldif->{"OBJECTCLASSCATEGORY"}->[0];

	if (defined($ldif->{"DESCRIPTION"})) {
		$desc = $ldif->{"DESCRIPTION"}->[0];
	} elsif (defined($ldif->{"ADMINDESCRIPTION"})) {
		$desc = $ldif->{"ADMINDESCRIPTION"}->[0];
	}

	if ($isNdsSystemObjectClass == 1 || $pass > 0) {
		$schUpdateOp = "MODIFY";
	} else {
		$schUpdateOp = "ADD";
	}

	$isTop = ($ldapDisplayName eq "top");
	$isContainer = ($type != 3) && !(&isInList($ldapDisplayName, \@NDS_CONTAINER_EXCEPTIONS));

	if (&isInList($ldapDisplayName, \@NDS_AUXILIARY_CLASSES)) {
		$type = 3;
	}
	# NB 88-Class (0) not supported; will map to DS_EFFECTIVE_CLASS
	if ($type == 0 || $type == 1) {
		# structural
		push(@flags, "DS_EFFECTIVE_CLASS");
	} elsif ($type == 2) {
		# abstract
	} elsif ($type == 3) {
		# auxiliary
		push(@flags, "DS_AUXILIARY_CLASS");
	} else {
		die "Unknown object class category " . $type;
	}
	if ($isContainer) {
		push(@flags, "DS_CONTAINER_CLASS");
	}

	@subClassOf = ($ldif->{"SUBCLASSOF"}->[0]);

	# Generally all auxiliary classes would be subclasses of top
	if (&isInList($ldapDisplayName, \@NDS_AUXILIARY_CLASSES)) {
		@subClassOf = ("top");
	}
	$rDNAttID = $ldif->{"RDNATTID"}->[0];

	if ($desc ne "") {
		if ($pass == 0) {
			print "-- Object class definition for " . $desc . "\n";
		} elsif (($pass == 1) && ($ldapDisplayName ne "inetOrgPerson")) {
			print "-- Structure rule definition for " . $desc . "\n";
		}
	}
	if (($pass == 1) && ($ldapDisplayName ne "inetOrgPerson")) {
		&emitNdsSchemaPreamble($ldapDisplayName, "OBJECT-CLASS", $schUpdateOp, $pass);
	}
	elsif($pass == 0) {
		&emitNdsSchemaPreamble($ldapDisplayName, "OBJECT-CLASS", $schUpdateOp, $pass);
	}

	if ($pass == 0) {
		push(@mustContain, (@{$ldif->{"MUSTCONTAIN"}}, @{$ldif->{"SYSTEMMUSTCONTAIN"}}), $rDNAttID); # in case RDN is not explicitly listed
		push(@mayContain, (@{$ldif->{"MAYCONTAIN"}}, @{$ldif->{"SYSTEMMAYCONTAIN"}}));

		if (&isInList("userPassword", \@mustContain) || &isInList("userPassword", \@mayContain)) {
			push(@subClassOf, "ndsLoginProperties");
		}
		if ($ldapDisplayName eq "builtinDomain") {
			push(@subClassOf, "ndsContainerLoginProperties");
		}

		# push(@subClassOf, (@{$ldif->{'AUXILIARYCLASS'}}, @{$ldif->{'SYSTEMAUXILIARYCLASS'}}));
		&emitNdsInheritenceRules($ldif,\@subClassOf);

		if (scalar(@flags) != 0) {
			print "\tFlags\t\t{" . join(", ", @flags) . "},\n";
		}

		if ($schUpdateOp eq "ADD") {
			if ($type != 3) {
				&emitNdsContainmentRules(\@containedBy, $pass);
			}

			if ($rDNAttID ne "" && (!(&isInList($ldapDisplayName, \@NDS_AUXILIARY_ATTRIBUTE_EXCEPTIONS)))) {
				print "\tNamedBy\t\t{\"";
				if (defined($NDS_ATTRIBUTE_MAP{$rDNAttID})) {
					print $NDS_ATTRIBUTE_MAP{$rDNAttID};
				} else {
					print $rDNAttID;
				}
				print "\"},\n";
			}
		}

		# Flatten the auxiliary class list
		@auxiliaryClasses = &flattenAuxClasses($ldapDisplayName, $classes);

		foreach $auxiliaryClass (@auxiliaryClasses) {
			my $ldifP = $classes->{$auxiliaryClass};

			push(@mustContain, (@{$ldifP->{"MUSTCONTAIN"}}, @{$ldifP->{"SYSTEMMUSTCONTAIN"}}));
			push(@mayContain, (@{$ldifP->{"MAYCONTAIN"}}, @{$ldifP->{"SYSTEMMAYCONTAIN"}}));
		}

		if ($isNdsSystemObjectClass == 1) {
			# If we are modifying an eDirectory system class, then we need to demote
			# any mandatory attributes to optional otherwise we will break existing
			# deployments.
			push(@mayContain, @mustContain);
			@mustContain = ();
		}

		if (defined($NDS_REMOVE_ATTRIBUTE_MAP{$ldapDisplayName})) {
			
			my $attributesToRemove = $NDS_REMOVE_ATTRIBUTE_MP{$ldapDisplayName};
			my @attributesRemoveList = split(":", $attributesToRemove);
			
			my $value;

			foreach $value (@attributesRemoveList) {
				if(&isInList($value, \@mayContain)) {
					push(@demotedAttributes, $value);
				}
			}
		}	
	
		# Retain the old values of demotedAttributes and adding the ones
		# present in NDS_LDAPDISPLAYNAME_EXCEPTIONS		
		foreach $value (@NDS_LDAPDISPLAYNAME_EXCEPTIONS) {
			push(@demotedAttributes, $value);
		}

		my $dem;

		foreach $dem (@NDS_DEMOTED_ATTRIBUTES) {
			if (&isInList($dem, \@mustContain)) {
				push(@mayContain, $dem);
				push(@demotedAttributes, $dem);
				push(@demotedAttributes, $rDNAttId);
			}
		}

		if (!(&isInList($ldapDisplayName, \@NDS_AUXILIARY_ATTRIBUTE_EXCEPTIONS))) {
			&emitNdsObjectClassAttrs($ldif, "MustContain", \@mustContain, \@demotedAttributes);
		}
	
		# Flush the demotedAttributes array
		@demotedAttributes = ();
	
		if (defined($NDS_REMOVE_ATTRIBUTE_MAP{$ldapDisplayName})) {

                        my $attributesToRemove = $NDS_REMOVE_ATTRIBUTE_MAP{$ldapDisplayName};
                        my @attributesRemoveList = split(":", $attributesToRemove);

                        my $value;

                        foreach $value (@attributesRemoveList) {
                                if(&isInList($value, \@mayContain)) {
                                        push(@demotedAttributes, $value);
                                }
                        }
                }

		foreach $value (@NDS_LDAPDISPLAYNAME_EXCEPTIONS) {
                        push(@demotedAttributes, $value);
		}

		if ($isNdsSystemObjectClass == 1) {
			push(@demotedAttributes, $rDNAttId);
		}
		&emitNdsObjectClassAttrs($ldif, "MayContain", \@mayContain, \@demotedAttributes);

		print "\tASN1ObjID\t" . &ldap2NdsOid($governsID) . "\n";
	} elsif ($pass == 1) {
		@containedBy = &flattenContainers($ldapDisplayName, $classes);
		# assert checks that caller should have made
		die if (scalar(@containedBy) == 0);
		die if ($type == 3);
#		die if ($isNdsSystemObjectClass);

		if ($type != 3 && $ldapDisplayName ne "inetOrgPerson") {
			&emitNdsContainmentRules(\@containedBy, $pass);
		}
	}
	if (($pass == 1) && ( $ldapDisplayName ne "inetOrgPerson")) {
		&emitNdsSchemaPostamble();
	}
	elsif($pass == 0) {
		&emitNdsSchemaPostamble();
	}
}

sub emitNdsSchema
{
	my ($ldifEntries) = @_;
	my $ldif, $ldapDisplayName;
	my $copyrightMsg = "";
	my %classes;

	# Create an associative array containing the schema partition
	# indexed by LDAP display name
	foreach $ldif (@$ldifEntries) {
		next unless (&isKindOf($ldif, "classSchema"));
		next if (!defined($ldif->{"LDAPDISPLAYNAME"}));
		$classes{$ldif->{"LDAPDISPLAYNAME"}->[0]} = $ldif;
	}

	# print copyright message and version of ndsschema that
	#  generated this
	$CopyrightMsg = "-- Domain Services for Windows Schema Definitions\n-- Novell Inc.\n-- 1800 South, Novell Place\n\n-- CoPyRiGhT=(c) Copyright 2011, Novell, Inc.  All rights reserved\n";

	print "$CopyrightMsg";
	print "-- ident $rcsId\n\n";

	# Output a native NDS schema definition for the AD schema
	print "XADSchemaExtensions DEFINITIONS ::=\n";
	print "BEGIN\n\n";

	# attributeTypes
	foreach $ldif (@$ldifEntries) {
		if (&isKindOf($ldif, "attributeSchema")) {
			&emitNdsAttributeSchema($ldif);
		}
	}

	# objectClasses
	foreach $ldif (@$ldifEntries) {
		if (&isKindOf($ldif, "classSchema")) {
			&emitNdsObjectClassSchema($ldif->{"LDAPDISPLAYNAME"}->[0], {%classes}, 0);
		}
	}

	# dITStructureRules
	foreach $ldif (@$ldifEntries) {
		if (&isKindOf($ldif, "classSchema")) {
			my $ldapDisplayName = $ldif->{"LDAPDISPLAYNAME"}->[0];

			next if (&isInList($ldapDisplayName, \@NDS_AUXILIARY_CLASSES));
			next if (&isInList($ldapDisplayName, \@NDS_NOT_CONTAINBLE_CLASSES));
			next if ($ldif->{"OBJECTCLASSCATEGORY"}->[0] == 3); # skip aux classes
			next if (uc($ldapDisplayName) eq "TOP");

			# very inefficient because it computes it twice XXX
			my @s = &flattenContainers($ldapDisplayName, {%classes});
			next if (scalar(@s) == 0);

			&emitNdsObjectClassSchema($ldapDisplayName, {%classes}, 1);
		}
	}

	# eDirectory interop dITStructureRules
	foreach $ldapDisplayName (@NDS_COMPAT_CONTAINERS) {
		my @container = ("container");
		if (uc($ldapDisplayName) eq "ORGANIZATIONALUNIT") {
			push(@container, "domainDNS");
		}
		print "-- Structure rule compatibility definition for " . $NDS_CLASS_MAP{$ldapDisplayName} . "\n";
		&emitNdsSchemaPreamble($ldapDisplayName, "OBJECT-CLASS", "MODIFY", 1);
		&emitNdsContainmentRules(\@container, 1);
		&emitNdsSchemaPostamble();
	}

	&emitNdsServerObjectExtensions();

	print "END\n";
}

sub emitNdsAttributeIndex
{
	my ($ldif) = @_;
	my $searchFlags;

	if (!defined($ldif->{"LDAPDISPLAYNAME"})) {
		die $ldif->{'DN'}->[0] . ": attribute display name undefined";
	}

	my $ldapDisplayName = $ldif->{"LDAPDISPLAYNAME"}->[0];
	my $indexVersion = 0;
	my $indexState = 2;
	my $indexRule = 0;
	my $indexType = 0;
	my $indexValueState = 1;

	my $indexName = $ldapDisplayName;
	if ($indexRule == 1) {
		$indexName .= "Presence";
	} elsif ($indexRule == 2) {
		$indexName .= "Substring";
	} elsif ($indexRule != 0) {
		die "Invalid index rule $indexRule";
	}
	if (defined($NDS_ATTRIBUTE_MAP{$ldapDisplayName})) {
		$ldapDisplayName = $NDS_ATTRIBUTE_MAP{$ldapDisplayName};
	}

	$searchFlags = $ldif->{"SEARCHFLAGS"}->[0];
	# Don't index linked attributes
	if (uc($ldif->{"LINKID"}->[0]) ne "") {
		$searchFlags = 0;
	}

	# 0x1 - index over attribute
	# 0x2 - index over attribute and container
	# 0x4 - add attribute to ANR (ambiguous name resolution) set
	# 0x8 - preserve attribute when deleted
	# 0x16 - preserve attribute when copied
	if ($searchFlags & (0x1 | 0x2)) {
		print 'indexDefinition=' .
			$indexVersion . '$' . $indexName . '$' .
			$indexState . '$' . $indexRule . '$' . $indexType . '$' .
			$indexValueState . '$';
		# truncate attribute names to 32 characters for NDS compat
		print substr($ldapDisplayName, 0, 32) . "\n";
	}
}

1;
