#!/usr/bin/perl
################################################################################
# Domain Services for Windows Provisioning Tool
#
# See the file LICENSE for redistribution information.
#
# Copyright (c) 2001-2006
#	Novell, Inc.  All rights reserved.
################################################################################
#
#   NAME:
#         ndsschemamap.pm
#
#   FACILITY:
#
#         Domain Controller Initialization
#
#   ABSTRACT:
#
#         LDAP to eDirectory default schema mapping table.
#
#   REVISION:
#
#         $Id$
#
#################################################################################

package schema;

my $rcsId = '$Id$';

%NDS_ATTRIBUTE_MAP = (
	"accountBalance"		=> "Account Balance",
#	"assistant"			=> "assistant",			# different flags
	"aliasedObjectName"		=> "Aliased Object Name",
#	"audio"				=> "audio",			# different flags
#	"authorityRevocationList"	=> "authorityRevocationList",	# different flags
	"defaultQueue"			=> "Default Queue",
#	"cACertificate"			=> "cACertificate",		# different flags
#	"certificateRevocationList"	=> "certificateRevocationList",	# different flags
#	"co"				=> "co",			# different flags
#	"company"			=> "company",			# different flags
#	"crossCertificatePair"		=> "crossCertificatePair",	# different flags
#	"dc"				=> "dc",			# different flags
#	"deltaRevocationList"		=> "deltaRevocationList",	# different flags
	"detectIntruder"		=> "Detect Intruder",
#	"directReports"			=> "directReports",		# different flags
#	"displayName"			=> "displayName",		# different flags
#	"dmdName"			=> "dmdName",			# different flags
	"dsRevision"			=> "DS Revision",
	"employeeNumber"		=> "NSCP:employeeNumber",
#	"employeeType"			=> "employeeType",		# different flags
	"equivalentToMe"		=> "Equivalent To Me",
	"facsimileTelephoneNumber"	=> "Facsimile Telephone Number",
#	"fromEntry"			=> "fromEntry",			# different flags
	"fullName"			=> "Full Name",
	"generationQualifier"		=> "Generational Qualifier",
	"givenName"			=> "Given Name",
	"groupID"			=> "GID",
	"groupMembership"		=> "Group Membership",
	"homeDirectory"			=> "mSDS:HomeDirectory",
#	"homePhone"			=> "homePhone",			# different flags
#	"homePostalAddress"		=> "homePostalAddress",		# different flags
	"hostDevice"			=> "Host Device",
	"hostResourceName"		=> "Host Resource Name",
	"hostServer"			=> "Host Server",
#	"initials"			=> "initials",			# different flags
	"intruderAttemptResetInterval"	=> "Intruder Attempt Reset Interval",
	"intruderLockoutResetInterval"	=> "Intruder Lockout Reset Interval",
#	"knowledgeInformation"		=> "knowledgeInformation",	# different flags
#	"l"				=> "l",				# different flags
	"lastLoginTime"			=> "Last Login Time",
	"lockedByIntruder"		=> "Locked By Intruder",
	"lockoutAfterDetection"		=> "Lockout After Detection",
	"loginAllowedTimeMap"		=> "Login Allowed Time Map",
	"loginDisabled"			=> "Login Disabled",
	"loginExpirationTime"		=> "Login Expiration Time",
	"loginGraceLimit"		=> "Login Grace Limit",
	"loginGraceRemaining"		=> "Login Grace Remaining",
	"loginIntruderAddress"		=> "Login Intruder Address",
	"loginIntruderAttempts"		=> "Login Intruder Attempts",
	"loginIntruderLimit"		=> "Login Intruder Limit",
	"loginIntruderResetTime"	=> "Login Intruder Reset Time",
	"loginMaximumSimultaneous"	=> "Login Maximum Simultaneous",
	"loginScript"			=> "Login Script",
	"loginTime"			=> "Login Time",
	"mail"				=> "Internet EMail Address",
	"mailRoutingAddress"		=> "NSCP:mailRoutingAddress",
#	"manager"			=> "manager",			# different flags
	"memberCertificateDesc"		=> "NSCP:memberCertificateDesc",
	"messageServer"			=> "Message Server",
#	"mobile"			=> "mobile",			# different flags
	"multiLineDescription"		=> "Description",
	"networkAddress"		=> "Network Address",
	"networkAddressRestriction"	=> "Network Address Restriction",
	"ndsCrossCertificatePair"	=> "Cross Certificate Pair",
	"ndsHomeDirectory"		=> "Home Directory",
	"objectClass"			=> "Object Class",
	"pageDescriptionLanguage"	=> "Page Description Language",
	"passwordAllowChange"		=> "Password Allow Change",
	"passwordExpirationInterval"	=> "Password Expiration Interval",
	"passwordExpirationTime"	=> "Password Expiration Time",
	"passwordMinimumLength"		=> "Password Minimum Length",
	"passwordRequired"		=> "Password Required",
	"passwordUniqueRequired"	=> "Password Unique Required",
	"partitionControl"		=> "Partition Control",
	"physicalDeliveryOfficeName"	=> "Physical Delivery Office Name",
	"postalAddress"			=> "Postal Address",
	"postalCode"			=> "Postal Code",
	"postOfficeBox"			=> "Postal Office Box",
	"printerConfiguration"		=> "Printer Configuration",
	"printerControl"		=> "Printer Control",
	"printJobConfiguration"		=> "Print Job Configuration",
	"printServer"			=> "Print Server",
	"privateKey"			=> "Private Key",
	"queueDirectory"		=> "Queue Directory",
	"roleOccupant"			=> "Role Occupant",
	"securityEquals"		=> "Security Equals",
	"seeAlso"			=> "See Also",
	"serialNumber"			=> "Serial Number",
	"sn"				=> "Surname",
	"st"				=> "S",
	"street"			=> "SA",
	"supportedServices"		=> "Supported Services",
	"supportedConnections"		=> "Supported Connections",
	"supportedTypefaces"		=> "Supported Typefaces",
	"telephoneNumber"		=> "Telephone Number",
	"typeCreatorMap"		=> "Type Creator Map",
	"uid"				=> "uniqueID",
	"mailRecipient"			=> "msds:mailRecipient",	# different attributes
	"homePostalAddress"		=> "msds:homePostalAddress",	# different attributes
	"objectVersion"			=> "msds:objectVersion",	# different attributes
	"unixHomeDirectory"		=> "homeDirectory",		# special case
#	"uid"				=> "uid"			# different flags
	);

# OID mapping is necessary where there is a a conflict between the 
# OID used by eDirectory and the one used by AD.
%NDS_OID_MAP = (
	"1.2.840.113556.1.5.66"		=> "0.9.2342.19200300.100.4.13",	# "Domain" class
	"1.2.840.113556.1.4.145"	=> "2.16.840.1.113719.1.1.4.1.122",	# "revision" attribute
	"1.3.6.1.4.1.11.1.3.1.1.3"	=> "2.16.840.1.113719.1.27.4.5",	# "searchtimelimit" attribute
	"2.16.840.1.113730.3.140"	=> "2.16.840.1.113730.3.1.40",		# "usersmimecertificate" attribute
	"1.2.840.113556.1.4.652"	=> "2.16.840.1.113719.1.8.4.2",		# "assistant" attribute
	"1.2.840.113556.1.2.131"	=> "0.9.2342.19200300.100.1.43",	# "co" attribute
	"1.2.840.113556.1.2.146"	=> "2.16.840.1.113719.1.8.4.5",		# "company" attribute
	"1.2.840.113556.1.2.13"		=> "2.16.840.1.113730.3.1.241",		# "displayname" attribute
	"1.2.840.113556.1.2.613"	=> "2.16.840.1.113730.3.1.4",		# "employeetype" attribute
	"1.2.840.113556.1.2.598"	=> "2.5.4.54",				# "dmdName" attribute
	"1.2.840.113556.1.5.8"		=> "2.5.6.17",				# "Group" class
	"2.5.6.9"			=> "2.5.6.17",				# "Group-of-Names" class
	"1.2.840.113556.1.4.651"	=> "0.9.2342.19200300.100.1.22",	# "othermailbox" attribute
	"1.2.840.113556.1.2.436"	=> "2.16.840.1.113719.1.8.4.6",		# "directReports" attribute
	"1.2.840.113556.1.5.9"		=> "2.16.840.1.113730.3.2.2",		# "User" class
	"1.2.840.113556.1.2.615"	=> "0.9.2342.19200300.100.1.40"		# "personaltitle" attribute
	);

# Changes required in schema so as to allow the installation of DSfW
# using a container administrator's account. Following classes should 
# not have the attributes mentioned in the MUST or MAY contain list.
%NDS_REMOVE_ATTRIBUTE_MAP = (
	"applicationEntity"		=> "cn:presentationAddress",
	"applicationProcess"		=> "cn",
	"certificationAuthority"	=> "authorityRevocationList:certificateRevocationList:cACertificate",
	"country"			=> "c",
	"cRLDistributionPoint"		=> "cn",
	"device"			=> "cn",
	"domain"			=> "dc",
	"group"				=> "cn",
	"groupOfNames"			=> "cn",
	"groupOfUniqueNames"		=> "cn",
	"organization"			=> "o",
	"organizationalRole"		=> "cn",
	"organizationalUnit"		=> "ou",
	"person"			=> "cn:sn",
	"dSA"				=> "cn",
	"organizationalPerson"		=> "cn",
	"inetOrgPerson"			=> "cn",
	"user"				=> "cn"
	);

# Class mapping is necessary where there is a name conflict or the
# inheritence hierarchy differs, or where there is no conflict but
# we need to know to extend rather than attempt to replace the
# class definition.
#
# The following rules apply generally:
#
# (a) X.500 classes (and user/group) are extended with optional attributes
# (b) AD classes with conflicts are mapped to the NDS name "mSDS:<ClassName>"
# (c) NDS classes with conflicts are remapped to the LDAP name "nds<ClassName>"
#
# mSDS is a prefix used by AD system schema introduced in Windows
# 2003 to avoid conflicts with deployed schema. However the mSDS:
# prefix guarantees that we will not have any conflcits with future
# AD system schema because the colon character is illegal in LDAP
# object class names.
#
%NDS_CLASS_MAP = (
	"certificationAuthority"	=> "certificationAuthority",		# X.500
	"country"			=> "Country",				# X.500
	"computer"			=> "mSDS:Computer",			# AD
	"cRLDistributionPoint"		=> "cRLDistributionPoint",		# NDS
	"device"			=> "Device",				# X.500
	"dMD"				=> "mSDS:DMD",				# AD
	"group"				=> "Group",				# AD
	"locality"			=> "Locality",				# X.500
	"ndsComputer"			=> "Computer",				# NDS
	"ndsDmd"			=> "dmd",				# NDS
	"ndsServer"			=> "Server",				# NDS
	"ndsVolume"			=> "Volume",				# NDS
	"organization"			=> "Organization",			# X.500
	"organizationalPerson"		=> "Organizational Person",		# X.500
	"organizationalUnit"		=> "Organizational Unit",		# X.500
	"person"			=> "Person",				# X.500
	"server"			=> "mSDS:Server",			# AD
	"top"				=> "Top",				# X.500
	"user"				=> "User",				# AD
	"volume"			=> "mSDS:Volume",			# AD
	"applicationEntity"		=> "applicationEntity",			# NDS
	"applicationProcess"		=> "applicationProcess",		# NDS
	"domain"			=> "domain",				# NDS
	"groupOfNames"			=> "Group",				# NDS
	"groupOfUniqueNames"		=> "Group",				# NDS
	"organizationalRole"		=> "Organizational Role",		# X.500
	"dSA"				=> "dSA",				# NDS
	"inetOrgPerson"			=> "User",				# AD
	);

@NDS_NOT_CONTAINER_CLASSES = ( "group", "user" );
@NDS_NOT_CONTAINBLE_CLASSES = ( "user");
@NDS_AUXILIARY_CLASSES = ( "certificationAuthority" , "domainDNS" );
@NDS_CONTAINER_EXCEPTIONS = ( "domainDNS", "groupOfNames", "groupOfUniqueNames", "group" );

# These attributes will be marked read-only also.
@NDS_HIDDEN_ATTRIBUTES = (
	"currentValue",
	"dBCSPwd",
	"unicodePwd",
	"ntPwdHistory",
	"priorValue",
	"supplementalCredentials",
	"trustAuthIncoming",
	"trustAuthOutgoing",
	"lmPwdHistory",
	"initialAuthIncoming",
	"initialAuthOutgoing",
	"pekList",
);

@NDS_READ_ONLY_ATTRIBUTES = (
	"badPasswordTime",	# actually computed
	"badPwdCount",		# actually computed
	"lastLogoff",		# actually computed
	"lastLogon",		# actually computed
	"logonCount",		# actually computed
	"domainReplica",
	"nextRid",
	"objectSid",
	"rid",
	"ridAvailablePool",
	"ridAllocationPool",
	"rIDNextRID",
	"ridPreviousAllocationPool",
	"ridUsedPool",
	"sAMAccountType",
	"securityIdentifier",
	"serverState",
	"sidHistory",
	"uniqueDomainID",
);

@NDS_SERVER_READ_ATTRIBUTES = (
	"groupType",
	"objectSid",
	"wellKnownObjects",
	"serverReference",
);

1;
