#!/usr/bin/perl  -I.. -I/opt/novell/xad/lib64/perl -I/opt/novell/xad/lib/perl
################################################################################
# Domain Services for Windows Provisioning Tool
#
# See the file LICENSE for redistribution information.
#
# Copyright (c) 2001-2006
#	Novell, Inc.  All rights reserved.
################################################################################
#
#   NAME:
#         schema.pm
#
#   FACILITY:
#
#         Domain Controller Initialization
#
#   ABSTRACT:
#
#         This module is responsible for mapping an Active Directory 
#         schema to an LDAPv3 schema.
#
#   REVISION:
#
#         $Id$
#
#################################################################################

package schema;

use XAD::dclib;
use XAD::ndsschema;
use XAD::registry;
use warnings;

my $rcsId = '$Id$';

#
# LDAP syntax table
#
$SYNTAXES{"2.5.5.1"} = "1.3.6.1.4.1.1466.115.121.1.12";    # distinguishedName
#$SYNTAXES{"2.5.5.2"} = "1.3.6.1.4.1.1466.115.121.1.38";    # objectIdentifier
$SYNTAXES{"2.5.5.2"} = "1.3.6.1.4.1.1466.115.121.1.15";    # directoryString (Object-Identifier)
$SYNTAXES{"2.5.5.3"} = "1.3.6.1.4.1.1466.115.121.1.15";    # directoryString
$SYNTAXES{"2.5.5.4"} = "1.3.6.1.4.1.1466.115.121.1.15";    # directoryString
$SYNTAXES{"2.5.5.5:19"} = "1.3.6.1.4.1.1466.115.121.1.44"; # printableString
$SYNTAXES{"2.5.5.5:22"} = "1.3.6.1.4.1.1466.115.121.1.26"; # IA5String
$SYNTAXES{"2.5.5.6"} = "1.3.6.1.4.1.1466.115.121.1.36";    # numericString
$SYNTAXES{"2.5.5.7:VgYBAgUEHQ=="} = "1.2.840.113556.1.4.1221";          # orName
$SYNTAXES{"2.5.5.7:KoZIhvcUAQEBCw=="} = "1.2.840.113556.1.4.903"; # dnWithOctetString
$SYNTAXES{"2.5.5.8"} = "1.3.6.1.4.1.1466.115.121.1.7";     # boolean
$SYNTAXES{"2.5.5.9"} = "1.3.6.1.4.1.1466.115.121.1.27";    # integer
$SYNTAXES{"2.5.5.10"} = "1.3.6.1.4.1.1466.115.121.1.40";   # octetString
$SYNTAXES{"2.5.5.11:23"} = "1.3.6.1.4.1.1466.115.121.1.53";# utcTime
$SYNTAXES{"2.5.5.11:24"} = "1.3.6.1.4.1.1466.115.121.1.24";# generalizedTime
$SYNTAXES{"2.5.5.12"} = "1.3.6.1.4.1.1466.115.121.1.15";   # directoryString (Unicode)
$SYNTAXES{"2.5.5.13"} = "1.3.6.1.4.1.1466.115.121.1.43";   # presentationAddress
$SYNTAXES{"2.5.5.14"} = "1.3.6.1.4.1.1466.115.121.1.2";    # accessPoint
$SYNTAXES{"2.5.5.15"} = "1.2.840.113556.1.4.907";          # Object-Security-Descriptor
$SYNTAXES{"2.5.5.16"} = "1.2.840.113556.1.4.906";          # Large-Integer
$SYNTAXES{"2.5.5.17"} = "1.3.6.1.4.1.1466.115.121.1.40";   # Object-SID

#
# Equality matching rule table
#
$EQUALITY_MATCHINGRULES{"2.5.5.1"} = "distinguishedNameMatch";
#$EQUALITY_MATCHINGRULES{"2.5.5.2"} = "objectIdentifierMatch";
$EQUALITY_MATCHINGRULES{"2.5.5.2"} = "caseIgnoreMatch";
$EQUALITY_MATCHINGRULES{"2.5.5.3"} = "caseExactMatch";
$EQUALITY_MATCHINGRULES{"2.5.5.4"} = "caseIgnoreMatch";
$EQUALITY_MATCHINGRULES{"2.5.5.5:22"} = "caseExactIA5Match";
$EQUALITY_MATCHINGRULES{"2.5.5.6"} = "numericStringMatch";
$EQUALITY_MATCHINGRULES{"2.5.5.8"} = "booleanMatch";
$EQUALITY_MATCHINGRULES{"2.5.5.9"} = "integerMatch";
$EQUALITY_MATCHINGRULES{"2.5.5.10"} = "octetStringMatch";
$EQUALITY_MATCHINGRULES{"2.5.5.11:24"} = "generalizedTimeMatch";
$EQUALITY_MATCHINGRULES{"2.5.5.12"} = "caseIgnoreMatch";
$EQUALITY_MATCHINGRULES{"2.5.5.13"} = "presentationAddressMatch";
$EQUALITY_MATCHINGRULES{"2.5.5.16"} = "largeIntegerMatch";
$EQUALITY_MATCHINGRULES{"2.5.5.17"} = "octetStringMatch";

#
# Substring matching rule table
#
$SUBSTR_MATCHINGRULES{"2.5.5.3"} = "caseExactSubstringsMatch";
$SUBSTR_MATCHINGRULES{"2.5.5.4"} = "caseIgnoreSubstringsMatch";
$SUBSTR_MATCHINGRULES{"2.5.5.5:22"} = "caseExactIA5SubstringsMatch";
$SUBSTR_MATCHINGRULES{"2.5.5.10"} = "octetStringSubstringsMatch";
$SUBSTR_MATCHINGRULES{"2.5.5.12"} = "caseIgnoreSubstringsMatch";

#
# Ordering matching rule table
#
$ORDERING_MATCHINGRULES{"2.5.5.9"} = "integerOrderingMatch";
$ORDERING_MATCHINGRULES{"2.5.5.11:24"} = "generalizedTimeOrderingMatch";
$ORDERING_MATCHINGRULES{"2.5.5.16"} = "largeIntegerOrderingMatch";

#
# Adjust exceptions according to internally defined schema
# (these are for OpenLDAP)
#
@ATTRIBUTEID_EXCEPTIONS = (
	"2.5.4.0", # objectClass
	"2.5.4.1", # aliasedObjectName
	"2.5.4.3", # commonName
	"2.5.4.13", # description
	"2.5.4.34", # seeAlso
	"2.5.4.35", # userPassword
	"2.5.4.41", # name
	"2.5.4.49", # distinguishedName
	"2.5.18.1", # createTimestamp
	"2.5.18.2", # modifyTimestamp
	"2.5.18.3", # creatorsName
	"2.5.18.4", # modifersName
	"2.5.18.10", # hasSubordinates
	"2.5.18.12", # collectiveSubentries
	"2.5.21.1", # dITStructureRules
	"2.5.21.2", # dITContentRules
	"2.5.21.4", # matchingRules
	"2.5.21.5", # attributeTypes
	"2.5.21.6", # objectClasses
	"2.5.21.7", # nameForms
	"2.5.21.8", # matchingRuleUse
	"2.5.21.5", # administrativeRole
	"2.5.21.6", # subtreeSpecification
	"2.5.21.9", # structuralObjectClass
	"1.3.6.1.4.1.1466.101.119.3", # entryTTL
	"0.9.2342.19200300.100.1.1", # uid
#	"0.9.2342.19200300.100.1.3",
#	"0.9.2342.19200300.100.1.37",
#	"0.9.2342.19200300.100.1.25",
	"1.3.6.1.4.1.250.1.57", # labeledURI
	"1.3.6.1.1.1.1.0", # uidNumber
	"1.3.6.1.1.1.1.1" # gidNumber
	);

@BINARY_SYNTAXES = ( "2.5.5.7:KoZIhvcUAQEBCw==",
	"2.5.5.10",
	"2.5.5.15",
	"2.5.5.17" );

# MS have assigned different OIDs to these
@LDAPDISPLAYNAME_EXCEPTIONS = ( "name" );

@GOVERNSID_EXCEPTIONS = (
	"2.5.6.1", # alias
	"2.5.17.0", # subEntry
	"2.5.20.1", # subschema
	"1.3.6.1.4.1.1466.101.120.111", # extensibleObject
	"1.3.6.1.4.1.1466.101.119.2" # dynamicObject
	);

@INDEX_EXCEPTIONS = (
	"2.5.5.11" # generalizedTimeMatch
	);

@INDEX_ADDITIONS = (
	"schemaIDGUID",
	"attributeSecurityGUID",
	"entryUUID",
	"entryCSN",
	"possSuperiors",
	"systemPossSuperiors",
	"defaultHidingValue",
	"isDeleted"
	);

@OPERATIONAL_ADDITIONS = (
	"nTSecurityDescriptor",
	"replPropertyMetaData"
	);

@PAS_ADDITIONS = (
	"creatorsName",
	"createTimestamp",
	"modifiersName",
	"modifyTimestamp"
	);

@NRA_EXCEPTIONS = (
	"objectGUID",
	);

sub emitAttributeSchema
{
	my ($ldif) = @_;
	my $syntax;
	my ($omSyntax, $omObjectClass);
	my $except;
#	my $isOperational;
	my $desc;
	my $attributeID;
	my $isOperational = 0;

	if (!defined($ldif->{"ATTRIBUTESYNTAX"})) {
		die $ldif->{'DN'}->[0] . ": Attribute syntax undefined";
	}

	if (!defined($ldif->{"ATTRIBUTEID"})) {
		die $ldif->{'DN'}->[0] . ": Attribute OID undefined";
	}

	foreach $except (@ATTRIBUTEID_EXCEPTIONS) {
		if ($ldif->{"ATTRIBUTEID"}->[0] eq $except) {
			return;
		}
	}

	if ($ldif->{"ATTRIBUTEID"}->[0] eq "2.16.840.1.113730.3.1.34") {
		# 'ref' attribute, MS think it is 'middleName'
		# XXX this is a kludge
		$attributeID = "2.16.840.1.113719.1.513.2.28";
	} else {
		$attributeID = $ldif->{"ATTRIBUTEID"}->[0];
	}

	if (!defined($ldif->{"LDAPDISPLAYNAME"})) {
		die $ldif->{'DN'}->[0] . ": LDAP display name undefined";
	}

	foreach $except (@LDAPDISPLAYNAME_EXCEPTIONS) {
		if (uc($ldif->{"LDAPDISPLAYNAME"}->[0]) eq uc($except)) {
			return;
		}
	}

	$syntax = $ldif->{"ATTRIBUTESYNTAX"}->[0];
	$omSyntax = $ldif->{"OMSYNTAX"}->[0];
	$omObjectClass = XAD::encodeBase64($ldif->{"OMOBJECTCLASS"}->[0]);

	print "attributetype ( " . $attributeID;
	print " NAME '" . $ldif->{"LDAPDISPLAYNAME"}->[0] . "'\n";

	if (defined($ldif->{"DESCRIPTION"})) {
		$desc = $ldif->{"DESCRIPTION"}->[0];
	} elsif (defined($ldif->{"ADMINDESCRIPTION"})) {
		$desc = $ldif->{"ADMINDESCRIPTION"}->[0];
	}
#	if (defined($desc)) {
#		$desc =~ s/'//g;
#		print "\tDESC '$desc'\n";
#	}

	if (defined($EQUALITY_MATCHINGRULES{$syntax})) {
		print "\tEQUALITY " . $EQUALITY_MATCHINGRULES{$syntax} . "\n";
	} elsif (defined($EQUALITY_MATCHINGRULES{"$syntax:$omSyntax"})) {
		print "\tEQUALITY " . $EQUALITY_MATCHINGRULES{"$syntax:$omSyntax"} . "\n";
	}

	if (defined($ORDERING_MATCHINGRULES{$syntax})) {
		print "\tORDERING " . $ORDERING_MATCHINGRULES{$syntax} . "\n";
	} elsif (defined($ORDERING_MATCHINGRULES{"$syntax:$omSyntax"})) {
		print "\tORDERING " . $ORDERING_MATCHINGRULES{"$syntax:$omSyntax"} . "\n";
	}

	if (defined($SUBSTR_MATCHINGRULES{$syntax})) {
		print "\tSUBSTR " . $SUBSTR_MATCHINGRULES{$syntax} . "\n";
	} elsif (defined($SUBSTR_MATCHINGRULES{"$syntax:$omSyntax"})) {
		print "\tSUBSTR " . $SUBSTR_MATCHINGRULES{"$syntax:$omSyntax"} . "\n";
	}

	print "\t";

	if (defined($SYNTAXES{$syntax})) {
		print "SYNTAX " . $SYNTAXES{$syntax};
	} elsif (defined($SYNTAXES{"$syntax:$omSyntax"})) {
		print "SYNTAX " . $SYNTAXES{"$syntax:$omSyntax"};
	} elsif (defined($SYNTAXES{"$syntax:$omObjectClass"})) {
		print "SYNTAX " . $SYNTAXES{"$syntax:$omObjectClass"};
	} else {
		print STDERR "No syntax map for $syntax\n";
		exit 1;
	}
	if (defined($ldif->{"RANGEUPPER"})) {
		print "{" . $ldif->{"RANGEUPPER"}->[0] . "}";
	}
	if (grep(/TRUE/i, @{$ldif->{"ISSINGLEVALUED"}})) {
		print " SINGLE-VALUE";
	}

	if ($ldif->{"SYSTEMFLAGS"}->[0] & 0x00000004) {
		$isOperational = 1;
	} else {
		foreach $except (@OPERATIONAL_ADDITIONS) {
			if (uc($ldif->{'LDAPDISPLAYNAME'}->[0]) eq uc($except)) {
				$isOperational = 1;
				last;
			}
		}
	}

	if ($isOperational) {
		print " USAGE dSAOperation";
	}

	print " )\n\n";
}

sub emitAttributeIndex
{
	my ($ldif) = @_;
	my $searchFlags;

	if (!defined($ldif->{"LDAPDISPLAYNAME"})) {
		die $ldif->{'DN'}->[0] . ": attribute display name undefined";
	}

	$searchFlags = $ldif->{"SEARCHFLAGS"}->[0];

	if (uc($ldif->{"LDAPDISPLAYNAME"}->[0]) eq "CN" ||
	    uc($ldif->{"LDAPDISPLAYNAME"}->[0]) eq "UID" ||
	    uc($ldif->{"LDAPDISPLAYNAME"}->[0]) eq "MAIL" ||
	    uc($ldif->{"LDAPDISPLAYNAME"}->[0]) eq "LINKID" ||
	    uc($ldif->{"LDAPDISPLAYNAME"}->[0]) eq "MAPIID" ||
	    $ldif->{"LDAPDISPLAYNAME"}->[0] =~ /name/i) {
		$searchFlags |= 5;
	}
	if (uc($ldif->{"LDAPDISPLAYNAME"}->[0]) eq "OBJECTCLASS" ||
	    uc($ldif->{"LDAPDISPLAYNAME"}->[0]) eq "OBJECTCATEGORY") {
		$searchFlags |= 1;
	}

	# Don't index linked attributes
	if (uc($ldif->{"LINKID"}->[0]) ne "") {
		$searchFlags = 0;
	}

	# 0x1 - index over attribute
	# 0x2 - index over attribute and container
	# 0x4 - add attribute to ANR (ambiguous name resolution) set
	# 0x8 - preserve attribute when deleted
	# 0x16 - preserve attribute when copied
	if ($searchFlags & (0x1 | 0x2)) {
		my $index = "eq";
		my $syntax = $ldif->{"ATTRIBUTESYNTAX"}->[0];
		my $omSyntax = $ldif->{"OMSYNTAX"}->[0];
		my $except;

		# First, check that syntax has an equality matching rule
		if (!defined($EQUALITY_MATCHINGRULES{$syntax}) &&
		    !defined($EQUALITY_MATCHINGRULES{"$syntax:$omSyntax"})) {
#			print STDERR "WARNING: no equality matching rule for indexed attribute " .
#				$ldif->{"LDAPDISPLAYNAME"} . "; ignoring.\n";
			return;
		}

		foreach $except (@INDEX_EXCEPTIONS) {
			if ($syntax eq $except || "$syntax:$omSyntax" eq $except) {
#				print STDERR "WARNING: attribute " .  $ldif->{"LDAPDISPLAYNAME"} .
#					 " has equality matching rule but cannot be indexed; ignoring.\n";
				return;
			}
		}

		# always presence index
		$index .= ",pres";
		if (defined($SUBSTR_MATCHINGRULES{$syntax}) ||
		    defined($SUBSTR_MATCHINGRULES{"$syntax:$omSyntax"})) {
			$index .= ",sub";
		}
		print "index\t" . $ldif->{"LDAPDISPLAYNAME"}->[0] . "\t" . $index . "\n";
	}
}

sub emitPartialAttributeSet
{
	my ($ldif) = @_;
	my $partial;

	if (!defined($ldif->{"LDAPDISPLAYNAME"})) {
		die $ldif->{'DN'}->[0] . ": attribute display name undefined";
	}

	$partial = (uc($ldif->{"ISMEMBEROFPARTIALATTRIBUTESET"}->[0]) eq "TRUE");
	if ($partial) {
		print $ldif->{"LDAPDISPLAYNAME"}->[0] . "\n";
	}
}

sub emitNonReplicatedAttributes
{
	my ($ldif) = @_;
	my $systemFlags;
	my $except;
	my $nr;

	if (!defined($ldif->{"LDAPDISPLAYNAME"})) {
		die $ldif->{'DN'}->[0] . ": attribute display name undefined";
	}

	foreach $except (@NRA_EXCEPTIONS) {
		return if (uc($ldif->{"LDAPDISPLAYNAME"}->[0]) eq uc($except));
	}

	$systemFlags = $ldif->{"SYSTEMFLAGS"}->[0];

#	if (defined($ldif->{"LINKID"})) {
#		# linked attributes are constructed in our impl
#		$systemFlags |= 0x00000004;
#	}
#	Omit constructued attributes and attributes marked as non-replicated
#	$nr = (($systemFlags & 0x00000001) && !($systemFlags & 0x00000004));

#	Omit attributes marked as non-replicated
	$nr = ($systemFlags & 0x00000001);	# non-replicated

	if ($nr) {
		print $ldif->{"LDAPDISPLAYNAME"}->[0] . "\n";
	}
}

sub emitObjectClassSchema
{
	my ($ldif) = @_;
	my $type;
	my $except;
	my $top = 0;
	my $desc;
	my @extraOptionalAttrs;
	# security descriptors are optional unless native security enabled
	my $nativeSecurity = ($registry::REGISTRY{"Native Security"} eq "TRUE");

	if (!defined($ldif->{"OBJECTCLASSCATEGORY"})) {
		die $ldif->{'DN'}->[0] . ": object category name undefined";
	}
	if (!defined($ldif->{"LDAPDISPLAYNAME"})) {
		die $ldif->{'DN'}->[0] . ": object class display name undefined";
	}
	if (!defined($ldif->{"GOVERNSID"})) {
		die $ldif->{'DN'}->[0] . ": object class OID undefined";
	}

	foreach $except (@GOVERNSID_EXCEPTIONS) {
		if ($ldif->{"GOVERNSID"}->[0] eq $except) {
			return;
		}
	}
	$type = $ldif->{"OBJECTCLASSCATEGORY"}->[0];

	if ($type == 0) {
#		print STDERR "88-Class not supported; will map to STRUCTURAL: " .  $ldif->{"LDAPDISPLAYNAME"}->[0] . "\n";
		$type = 1;
	}

	foreach $except (@LDAPDISPLAYNAME_EXCEPTIONS) {
		if (uc($ldif->{"LDAPDISPLAYNAME"}->[0]) eq uc($except)) {
			return;
		}
	}

	if ($ldif->{"GOVERNSID"}->[0] eq "2.5.6.0") {
		$ldif->{"GOVERNSID"}->[0] = "2.16.840.1.113719.1.513.1.3"; # msDS-Top
		$ldif->{"LDAPDISPLAYNAME"}->[0] = "mSDS-Top";
		$top = 1;
	}

	print "objectclass ( " . $ldif->{"GOVERNSID"}->[0];
	print " NAME '" .  $ldif->{"LDAPDISPLAYNAME"}->[0] . "'";

	if (defined($ldif->{"SUBCLASSOF"}->[0])) {
		print " SUP ";
		if (!$top && uc($ldif->{"SUBCLASSOF"}->[0]) eq "TOP") {
			print "mSDS-Top";
		} else {
			print $ldif->{"SUBCLASSOF"}->[0];
		}
	}
	if ($type == 1) {
		print " STRUCTURAL";
	} elsif ($type == 2) {
		print " ABSTRACT";
	} elsif ($type == 3) {
		print " AUXILIARY";
	}

	if (defined($ldif->{"DESCRIPTION"})) {
		$desc = $ldif->{"DESCRIPTION"}->[0];
	} elsif (defined($ldif->{"ADMINDESCRIPTION"})) {
		$desc = $ldif->{"ADMINDESCRIPTION"}->[0];
	}
#	if (defined($desc)) {
#		$desc =~ s/'//g;
#		print "\n\tDESC '$desc'";
#	}

	if (defined($ldif->{"MUSTCONTAIN"}) || defined($ldif->{"SYSTEMMUSTCONTAIN"})) {
		my ($attr, $first);
		$first = 1;

		foreach $attr (@{$ldif->{"MUSTCONTAIN"}}, @{$ldif->{"SYSTEMMUSTCONTAIN"}}) {
			# XXX workaround for LinkEngine schema checking bug
			if ($SCHEMAHASH{uc($attr)}->{'LINKID'} != 0) {
				push(@extraOptionalAttrs, $attr);
				next;
			}
			if ($nativeSecurity == 0 && (uc($attr) eq "NTSECURITYDESCRIPTOR")) {
				push(@extraOptionalAttrs, $attr);
				next;
			}
			if ($first) {
				print "\n\tMUST ( ";
				$first = 0;
			} else {
				print ' $ ';
			}
			print $attr;
		}
		if ($first == 0) {
			print " )";
		}
	}
	if (defined($ldif->{"MAYCONTAIN"}) || defined($ldif->{"SYSTEMMAYCONTAIN"})) {
		my ($attr, $first);
		$first = 1;

		foreach $attr (@{$ldif->{"MAYCONTAIN"}}, @{$ldif->{"SYSTEMMAYCONTAIN"}}, @extraOptionalAttrs) {
			if ($first) {
				print "\n\tMAY ( ";
				$first = 0;
			} else {
				print ' $ ';
			}
			print $attr;
		}
		if ($first == 0) {
			print " )";
		}
	}
	print " )\n\n";
}

sub emitDitContentRuleSchema
{
	my (@auxiliaryClassNames, %classes);
	my $key;
	my ($ldifEntries) = @_;

	#
	# First, collect auxiliary classes, because they can be AUX
	# on to any object. (Weird, yes, but look at the AD aggregate
	# dITContentRules and you will see that this is the case.)
	#
	foreach $ldif (@$ldifEntries) {
		next unless (isKindOf($ldif, "classSchema"));
		next if (!defined($ldif->{"LDAPDISPLAYNAME"}));
		$classes{$ldif->{"LDAPDISPLAYNAME"}->[0]} = $ldif;

		my $type = $ldif->{"OBJECTCLASSCATEGORY"}->[0];
		next if ($type != 3);
		push(@auxiliaryClassNames, $ldif->{"LDAPDISPLAYNAME"}->[0]);
	}
	foreach $key (keys %classes) {
		emitDitContentRuleSchemaElement($key, {%classes}, \@auxiliaryClassNames);
	}
}

sub emitDitContentRuleSchemaElement
{
	my ($key, $classes, $auxiliaryClassNames) = @_;
	my $ldif = $classes->{$key};
	my $type;
	my $except;
	my $top = 0;
	my $auxiliaryClassName;
	my $first;
	my @contentClasses;
	my $contentClass;
	my @contentMusts;
	my $contentMust;
	my @contentMays;
	my $contentMay;

	if (!defined($ldif->{"OBJECTCLASSCATEGORY"})) {
		die $ldif->{'DN'}->[0] . ": object category name undefined";
	}
	if (!defined($ldif->{"LDAPDISPLAYNAME"})) {
		die $ldif->{'DN'}->[0] . ": object class display name undefined";
	}
	if (!defined($ldif->{"GOVERNSID"})) {
		die $ldif->{'DN'}->[0] . ": object class OID undefined";
	}

	foreach $except (@GOVERNSID_EXCEPTIONS) {
		if ($ldif->{"GOVERNSID"}->[0] eq $except) {
			return;
		}
	}
	$type = $ldif->{"OBJECTCLASSCATEGORY"}->[0];

	if ($type == 0) {
		$type = 1;
	}

	if ($type != 1) {
		# structural classes need only apply
		return;
	}

	foreach $except (@LDAPDISPLAYNAME_EXCEPTIONS) {
		if (uc($ldif->{"LDAPDISPLAYNAME"}->[0]) eq uc($except)) {
			return;
		}
	}

	if (uc($ldif->{"LDAPDISPLAYNAME"}->[0]) eq "TOP") {
		$top = 1;
	}

	print "ditcontentrule ( " . $ldif->{"GOVERNSID"}->[0];
	print " NAME '" . ($top ? "mSDS-Top" : $ldif->{"LDAPDISPLAYNAME"}->[0]) . "'";

	print "\n\tAUX ( ";
	$first = 1;
	foreach $auxiliaryClassName (@$auxiliaryClassNames) {
		my $auxiliaryClass = $classes->{$auxiliaryClassName};
	
		if ($first) {
			$first = 0;
		} else {
			print ' $ ';
		}
		print $auxiliaryClass->{"LDAPDISPLAYNAME"}->[0];
	}

	while (defined($ldif) &&
		$ldif->{"SUBCLASSOF"}->[0] ne $ldif->{"LDAPDISPLAYNAME"}->[0])
	{
		my $superClassName;

		push(@contentClasses, @{$ldif->{'AUXILIARYCLASS'}});
		push(@contentClasses, @{$ldif->{'SYSTEMAUXILIARYCLASS'}});

		$superClassName = $ldif->{"SUBCLASSOF"}->[0];
		$ldif = $classes->{$superClassName};
	}

	foreach $contentClass (@contentClasses) {
		# Find Content Class in Auxiliary Classes
		my $foundCC = 0;

		$contentClass =~ s/\s//g;
		next if ($contentClass eq "");
		foreach $auxiliaryClassName (@$auxiliaryClassNames) {
			if (uc($auxiliaryClassName) eq uc($contentClass)) {
				$foundCC = 1;
			}
		}
		if (!$foundCC) {
			if ($first) {
				$first = 0;
			} else {
				print ' $ ';
			}
			print $contentClass;
		}
	}
	print " )";

	foreach $contentClass (@contentClasses) {
		my @auxiliaryClasses;

		$contentClass =~ s/\s//g;
		next if ($contentClass eq "");

		$ldif = $classes->{$contentClass};

		push(@auxiliaryClasses, $contentClass);
		push(@auxiliaryClasses, @{$ldif->{'AUXILIARYCLASS'}});
		push(@auxiliaryClasses, @{$ldif->{'SYSTEMAUXILIARYCLASS'}});

		foreach $auxiliaryClassName (@auxiliaryClasses) {
			my $auxiliaryClass = $classes->{$auxiliaryClassName};
			my $at;

			foreach $at (@{$auxiliaryClass->{"MUSTCONTAIN"}}) {
				my $needAdd = 1;
				$at =~ s/\s//g;
				foreach $contentMust (@contentMusts) {
					if (uc($contentMust) eq uc($at)) {
						$needAdd = 0;
						last;
					}
				}
				if ($needAdd) {
					push (@contentMusts, $at);
				}
			}
			foreach $at (@{$auxiliaryClass->{"SYSTEMMUSTCONTAIN"}}) {
				my $needAdd = 1;
				$at =~ s/\s//g;
				foreach $contentMust (@contentMusts) {
					if (uc($contentMust) eq uc($at)) {
						$needAdd = 0;
						last;
					}
				}
				if ($needAdd) {
					push (@contentMusts, $at);
				}
			}
		}
	}
	$first = 1;
	foreach $contentMust (@contentMusts) {
		if ($first) {
			print "\n\tMUST ( ";
			$first = 0;
		} else {
			print " \$ ";
		}
		print $contentMust;
	}
	if (!$first) {
		print " )";
	}

	foreach $contentClass (@contentClasses) {
		my @auxiliaryClasses;

		$contentClass =~ s/\s//g;
		next if ($contentClass eq "");

		$ldif = $classes->{$contentClass};

		push(@auxiliaryClasses, $contentClass);
		push(@auxiliaryClasses, @{$ldif->{'AUXILIARYCLASS'}});
		push(@auxiliaryClasses, @{$ldif->{'SYSTEMAUXILIARYCLASS'}});

		foreach $auxiliaryClassName (@auxiliaryClasses) {
			my $auxiliaryClass = $classes->{$auxiliaryClassName};
			my $at;

			foreach $at (@{$auxiliaryClass->{"MAYCONTAIN"}}) {
				my $needAdd = 1;
				$at =~ s/\s//g;
				foreach $contentMay (@contentMays) {
					if (uc($contentMay) eq uc($at)) {
						$needAdd = 0;
						last;
					}
				}
				if ($needAdd) {
					push (@contentMays, $at);
				}
			}
			foreach $at (@{$auxiliaryClass->{"SYSTEMMAYCONTAIN"}}) {
				my $needAdd = 1;
				$at =~ s/\s//g;
				foreach $contentMay (@contentMays) {
					if (uc($contentMay) eq uc($at)) {
						$needAdd = 0;
						last;
					}
				}
				if ($needAdd) {
					push (@contentMays, $at);
				}
			}
		}
	}
	$first = 1;
	foreach $contentMay (@contentMays) {
		if ($first) {
			print "\n\tMAY ( ";
			$first = 0;
		} else {
			print " \$ ";
		}
		print $contentMay;
	}
	if (!$first) {
		print " )";
	}
	print " )\n\n";
}

sub isKindOf
{
	my ($ldif, $oc) = @_;
	my $p;

	foreach $p (@{$ldif->{"OBJECTCLASS"}}) {
		if (uc($p) eq uc($oc)) {
			return 1;
		}
	}

	return 0;
}

sub loadSchema
{
	my ($ldifEntries) = @_;
	my %schemaHash;
	my $ldif;

	foreach $ldif (@$ldifEntries) {
		next unless defined($ldif->{'LDAPDISPLAYNAME'});
		$schemaHash{uc($ldif->{'LDAPDISPLAYNAME'}->[0])} = $ldif;
	}

	return %schemaHash;
}

sub getAncestorDepth
{
	my ($clazz) = @_;
	my $rc = 0;

	my $ldapDisplayName = uc($clazz->{'LDAPDISPLAYNAME'}->[0]);
	my $subClassOf = uc($clazz->{'SUBCLASSOF'}->[0]);

	if ($ldapDisplayName ne "") {
		if ($ldapDisplayName ne $subClassOf) {
			$clazz = $SCHEMAHASH{$subClassOf};
			$rc = 1 + getAncestorDepth($clazz);
		}
	}

	return $rc;
}

sub byAncestor
{
	my ($a, $b) = @_;

	if (isKindOf($a, "classSchema") && isKindOf($b, "classSchema")) {
		if ($a->{'LDAPDISPLAYNAME'}->[0] eq $b->{'LDAPDISPLAYNAME'}->[0]) {
			return 0;
		}

		return getAncestorDepth($a) - getAncestorDepth($b);
	}
	return 0;
}

sub byAncestorOCAndName
{
	my $rc = 0;

	# first sort by ancestor, then name
	$rc = byAncestor($a, $b);

#	if ($rc == 0) {
#		$rc = $a->{"OBJECTCLASSCATEGORY"}->[0] - $b->{"OBJECTCLASSCATEGORY"}->[0];
#	}
	if ($rc == 0) {
		# conveniently puts attrs first 
		$rc = uc($a->{'OBJECTCLASS'}->[0]) cmp uc($b->{'OBJECTCLASS'}->[0]);
	}

	if ($rc == 0) {
		$rc = uc($a->{'LDAPDISPLAYNAME'}->[0]) cmp uc($b->{'LDAPDISPLAYNAME'}->[0]);
	}

	return $rc;
}

sub parseLdif
{
	my ($ldifEntries, $flags) = @_;
	my $ldif;

	if ($flags eq "--schema") {
		foreach $ldif (@$ldifEntries) {
			if (isKindOf($ldif, "attributeSchema")) {
				emitAttributeSchema($ldif);
			} elsif (isKindOf($ldif, "classSchema")) {
				emitObjectClassSchema($ldif);
			}
		}
	} elsif ($flags eq "--index") {
		my $attr;

		foreach $attr (@INDEX_ADDITIONS) {
			print "index\t$attr\teq,pres\n";
		}

		foreach $ldif (@$ldifEntries) {
			if (isKindOf($ldif, "attributeSchema")) {
				emitAttributeIndex($ldif);
			}
		}
	} elsif ($flags eq "--partialattributeset") {
		foreach $attr (@PAS_ADDITIONS) {
			print "$attr\n";
		}
		foreach $ldif (@$ldifEntries) {
			if (isKindOf($ldif, "attributeSchema")) {
				emitPartialAttributeSet($ldif);
			}
		}
	} elsif ($flags eq "--contentrules") {
		emitDitContentRuleSchema($ldifEntries);
	} elsif ($flags eq "--nonreplicatedattributes") {
		foreach $ldif (@$ldifEntries) {
			if (isKindOf($ldif, "attributeSchema")) {
				emitNonReplicatedAttributes($ldif);
			}
		}
	} elsif ($flags eq "--ndsschema") {
		emitNdsSchema($ldifEntries);
	} elsif ($flags eq "--ndsindex") {
		foreach $ldif (@$ldifEntries) {
			if (isKindOf($ldif, "attributeSchema")) {
				emitNdsAttributeIndex($ldif);
			}
		}
	} else {
		print STDERR "Unknown usage $flags\n";
		exit 1;
	}
}

# if $schemaCompat == TRUE then multi-valued attributes are
# stored as a single attribute separated by a '$' sign

sub loadLdif
{
	my ($fileName) = @_;
	my %ldif;
	my @data;

	my ($key, $val);
	my $isBase64Encoded = 0;

	open(DATA, $fileName) || die "Could not open $fileName\n";

	while(<DATA>) {
		chop;

		unless (/^\s/) {
			if ($isBase64Encoded) {
				push(@{$ldif{uc($key)}}, XAD::decodeBase64($val));
			} else {
				if($key) { push(@{$ldif{uc($key)}}, $val); }
			}
		}

		if ($_ eq "") {
			push(@data, {%ldif});
			undef %ldif;
			undef $key;
			undef $val;
			undef $isBase64Encoded;
		} elsif (/^([^:]+): (.*)/) {
			($key, $val) = ($1, $2);
			$isBase64Encoded = 0;
		} elsif (/^([^:]+):: (.*)/) {
			($key, $val) = ($1, $2);
			$isBase64Encoded = 1;
		} elsif (/^\s/) {
			s/^\s//;
			$val .= $_;
		} elsif (/^#/) {
			# NOOP
		} else {
			die "Can't parse line '$_'";
		}
	}

	close(DATA);

	return @data;
}

sub aggregateSchema
{
	my ($fileName, $flags) = @_;

	@SCHEMA = loadLdif($fileName);
	%SCHEMAHASH = loadSchema(\@SCHEMA);

	@SCHEMA = sort byAncestorOCAndName @SCHEMA;

# 	foreach $ldif (@SCHEMA) {
# 		next unless defined($ldif->{'LDAPDISPLAYNAME'});
# 		print "$ldif->{'LDAPDISPLAYNAME'}->[0]\n";
# 	}

	parseLdif(\@SCHEMA, $flags);
}

1;
