#!/usr/bin/perl -I. -I.. -I/opt/novell/xad/lib64/perl -I/opt/novell/xad/lib/perl
################################################################################
# Domain Services for Windows Cross Partition NTP Setup Module
#
# See the file LICENSE for redistribution information.
#
# Copyright (c) 2001-2012
#       Novell, Inc.  All rights reserved.
################################################################################

package crosspartitionntp;

use strict;
use warnings;
use XAD::registry;
use NDS::stage;
use util;
use File::Copy;
use File::Path;
use Config::IniFiles;

my $localtime=localtime;
my ($day,$month,$date,$time,$year)=split(/ /,$localtime);
my $stamp=$year.$month.$date."-".$time;
my @edir_versions = ("");

#NDSDCINIT SECTION
#ntp related changes
sub update_xad_profile_script{
	my $ncalrpcdir=registry::getReg("NcalRpcDir");
	my $xad_profile_file="/etc/profile.d/novell-xad.sh";
	
	eval{
		if(registry::getReg('NcalRpcDir') ne ""){
			print "The new sockets location for xadsd is [$ncalrpcdir]\n";

			#Adding the NCALRPCDIR=new NTP partition
			util::update_ntp_values($ncalrpcdir, 'novell-xad.sh', '/etc/profile.d', 0, 'MANPATH=');
		}
	};

	if($@) {
		print "$@\n"; #Printing ERR message actually dies with proper error message and a backtrace
	}

	return 0;
}

#STAGE SECTION
#Creating directory structure for ntp at mounted location
sub create_directory_structure_for_ntp{
	print "Creating the directory structure for NTP at new xadsd location\n";

	eval{
		if(registry::getReg('NcalRpcDir') ne ""){
			mkpath(registry::getReg('NcalRpcDir')."/PIPE");
			if (grep(/^edir2_oes11$/i, @edir_versions) or grep(/^edir_oes11_sp1$/i, @edir_versions)) {
				mkpath(registry::getReg('NcalRpcDir')."/PIPE/np");
			}
		}
	};

	if($@) {
		print "$@\n"; #Printing ERR message actually dies with proper error message and a backtrace
	}

	return 0;
}

#Replace an oldline with a new line in a specified file
#Need to leave a space before the line begins with a
# TAB or multiple spaces
sub replace_the_line($$$) {
	my $oldLine = shift;
	my $newLine = shift;
	my $inFile = shift;
	$oldLine =~ s/\s+/\\s\\+/g;
	`sed -i -e 's:$oldLine:$newLine:g' $inFile`;
}

#XADSD, RPCD, XAD-KRB5KDC init script creation
#Case to consider
#If NCALRPCDIR is available in the script file
#	Update the value for it
#If NCALRPCDIR is unavailable in the script file
#	Update the value for it
#	Update the places where RPC_NCALRPC_DIR
#	needs to be used in place of LOCALSTATEDIR
sub update_init_scripts{
	my %file_list= ("start-xadsd" => "/opt/novell/xad/libexec/",
                        "stop-xadsd" => "/opt/novell/xad/libexec/",
                        "stop-rpcd" => "/opt/novell/xad/bin/",
                        "start-xad-krb5kdc" => "/opt/novell/xad/sbin/");
	my $new_ncalrpcdir=registry::getReg("NcalRpcDir");

	print "Updating systemd service files/scripts\n";
	eval {
		foreach my $file (keys %file_list) {
			my $updating_file="$file_list{$file}"."$file";

			#Creating the backup file
			copy($updating_file, $updating_file.".".$stamp) or die("Failed to backup $updating_file : $!");
			#Adding the NCALRPCDIR=new NTP partition
			util::update_ntp_values($new_ncalrpcdir, $file, $file_list{$file}, 1, 'KRB5CCNAME');
			#Replacing xadsd specific changes needed in the xadsd.init
			if ($file eq "start-xadsd" or $file eq "stop-xadsd") {
				my $count_of_chroot_removal = `grep '.*rm -f /var/lib/chrony/.*/xadsd' $updating_file | wc -l`;
				chomp($count_of_chroot_removal);
				replace_the_line(' chown ntp $LOCALSTATEDIR/rpc/xadsd', '\tchown ntp $RPC_NCALRPC_DIR/xadsd', $updating_file);

				replace_the_line(' rm -f /var/lib/chrony/.*/xadsd', '\trm -f /var/lib/chrony/$LOCALSTATEDIR/rpc/xadsd', $updating_file);

				#Replacing the line with
				#ln -f ($RPC_NCALRPC_DIR|$LOCALSTATEDIR)/xadsd /var/lib/chrony/$LOCALSTATEDIR/rpc/xadsd
				#May be preceeded by 'then' and followed by 'fi'
				my $preceeded_with = `grep -B 1 '.*ln -f .*/xadsd /var/lib/chrony/\$LOCALSTATEDIR/rpc/xadsd' $updating_file | grep "then" | sed 's/^\\s\\+//g'`;
				my $followed_by = `grep -A 1 '.*ln -f .*/xadsd /var/lib/chrony/\$LOCALSTATEDIR/rpc/xadsd' $updating_file | grep "fi" | sed 's/^\\s\\+//g'`;
				chomp($preceeded_with);
				chomp($followed_by);
				if ($preceeded_with eq 'then' and $followed_by eq 'fi') {
					replace_the_line(' if.*ln -f .*/xadsd /var/lib/chrony/$LOCALSTATEDIR.*/xadsd.*fi', '\tif [ "X${IS_NTP_CHROOT}" == "Xyes" ]\n\tthen\n\t\tln -f $RPC_NCALRPC_DIR/xadsd /var/lib/chrony/$LOCALSTATEDIR/rpc/xadsd\n\tfi', $updating_file);
				} else {
					replace_the_line(' ln -f .*/xadsd /var/lib/chrony/$LOCALSTATEDIR.*/xadsd', '\tif [ "X${IS_NTP_CHROOT}" == "Xyes" ]\n\tthen\n\t\tln -f $RPC_NCALRPC_DIR/xadsd /var/lib/chrony/$LOCALSTATEDIR/rpc/xadsd\n\tfi', $updating_file);
				}

				replace_the_line(' rm -f $LOCALSTATEDIR/rpc/xadsd', '\trm -f $RPC_NCALRPC_DIR/xadsd', $updating_file);
				if (grep(/^edir2_sp3$/i, @edir_versions)) {
					if ($count_of_chroot_removal lt 2) {
						replace_the_line(' rm -f $LOCALSTATEDIR/rpc/PIPE/lsass', '\trm -f $RPC_NCALRPC_DIR/PIPE/lsass\n\trm -f /var/lib/chrony/$LOCALSTATEDIR/rpc/xadsd', $updating_file);
					} else {
						replace_the_line(' rm -f $LOCALSTATEDIR/rpc/PIPE/lsass', '\trm -f $RPC_NCALRPC_DIR/PIPE/lsass', $updating_file);
					}
				} else {
					replace_the_line(' rm -f $LOCALSTATEDIR/rpc/PIPE/np/samr', '\trm -f $RPC_NCALRPC_DIR/PIPE/np/samr', $updating_file);
					replace_the_line(' rm -f $LOCALSTATEDIR/rpc/PIPE/np/lsarpc', '\trm -f $RPC_NCALRPC_DIR/PIPE/np/lsarpc', $updating_file);

					#NETLOGON(OES11) followed by netlogon(OES11SP1)
					my $is_NETLOGON_present = `grep "rm -f .*/PIPE/np/NETLOGON" $updating_file | wc -l`;
					my $is_netlogon_present = `grep "rm -f .*/PIPE/np/netlogon" $updating_file | wc -l`;
					chomp($is_NETLOGON_present);
					chomp($is_netlogon_present);

					#if 'NETLOGON' or 'netlogon' is not present
					if ($is_NETLOGON_present eq 0 or $is_netlogon_present eq 0) {
						if (grep(/^edir2_oes11$/i, @edir_versions)) {
							replace_the_line(' rm -f $LOCALSTATEDIR/rpc/PIPE/np/NETLOGON', '\trm -f $RPC_NCALRPC_DIR/PIPE/np/NETLOGON\n\trm -f $RPC_NCALRPC_DIR/PIPE/np/netlogon', $updating_file);
						} else {
							replace_the_line(' rm -f $LOCALSTATEDIR/rpc/PIPE/np/netlogon', '\trm -f $RPC_NCALRPC_DIR/PIPE/np/NETLOGON\n\trm -f $RPC_NCALRPC_DIR/PIPE/np/netlogon', $updating_file);
						}
					} else {
						replace_the_line(' rm -f $LOCALSTATEDIR/rpc/PIPE/np/NETLOGON', '\trm -f $RPC_NCALRPC_DIR/PIPE/np/NETLOGON', $updating_file);
						replace_the_line(' rm -f $LOCALSTATEDIR/rpc/PIPE/np/netlogon', '\trm -f $RPC_NCALRPC_DIR/PIPE/np/netlogon', $updating_file);
					}

					#To append the command for remove chroot jail
					if ($count_of_chroot_removal lt 2) {
						replace_the_line(' rm -f $LOCALSTATEDIR/rpc/PIPE/np/netlogon', '\trm -f $RPC_NCALRPC_DIR/PIPE/np/netlogon\n\trm -f /var/lib/chrony/$LOCALSTATEDIR/rpc/xadsd', $updating_file);
					}
				}
			}
			#Replacing rpcd specific changes needed in the xadsd.init
			elsif ($file eq "stop-rpcd") {
				replace_the_line(' rm -f $LOCALSTATEDIR/rpc/epmapper', '\trm -f $RPC_NCALRPC_DIR/epmapper', $updating_file);
			}
			chmod 0755, "$updating_file";
		}
	};

	if($@) {
		print "$@\n"; #Printing ERR message actually dies with proper error message and a backtrace
	}

	return 0;
}

#SMB.CONF file modification
sub update_smb_conf{
	my $smb_file = "/etc/samba/smb.conf";

	eval {
		copy($smb_file, $smb_file.$stamp) or print "Failed to backup the /etc/smb.conf file: $!";
		my $cfg = Config::IniFiles->new (-file=> $smb_file, -nocase=>1);
		if (!defined $cfg) {
			print "Failed to open $smb_file : @Config::IniFiles::errors\n";
			exit(-1);
		}
		print "Updating $smb_file file with new xadsd sockets location\n";
		print "Updating [global] section\n";

		#  dce funnel directory = $REGISTRY=NcalRpcDir
		my $var_value = "dce funnel directory";
		my $val = $cfg->val('dce funnel directory');
		if (! defined $val) {
			$cfg->newval('global', 'dce funnel directory', registry::getReg("NcalRpcDir"));
		}
		else{
			$cfg->setval('global', 'dce funnel directory', registry::getReg("NcalRpcDir"));
		}

		#  external_rpc_pipe : socket_dir = $REGISTRY=NcalRpcDir/PIPE
		if (grep(/^edir2_oes11$/i, @edir_versions) or grep(/^edir_oes11_sp1$/i, @edir_versions)) {
			$var_value = "external_rpc_pipe : socket_dir";
			$val = $cfg->val('external_rpc_pipe : socket_dir');
			if (! defined $val) {
				$cfg->newval('global', 'external_rpc_pipe : socket_dir', registry::getReg("NcalRpcDir")."/PIPE");
			}
			else{
				$cfg->setval('global', 'external_rpc_pipe : socket_dir', registry::getReg("NcalRpcDir")."/PIPE");
			}
		}

		#  dce_funnel_winbind : socket = $REGISTRY=NcalRpcDir/xadsd
		if (grep(/^edir_oes11_sp1$/i, @edir_versions)) {
			$var_value = "dce_funnel_winbind : socket";
			$val = $cfg->val('dce_funnel_winbind : socket');
			if (! defined $val) {
				$cfg->newval('global', 'dce_funnel_winbind : socket', registry::getReg("NcalRpcDir")."/xadsd");
			}
			else{
				$cfg->setval('global', 'dce_funnel_winbind : socket', registry::getReg("NcalRpcDir")."/xadsd");
			}
		}

		$cfg->RewriteConfig();
		`sed -i 's/^[a-zA-Z]/\t&/g' $smb_file`;
		`sed -i 's/=/ = /g' $smb_file`;
	};

	if($@) {
		print "$@\n"; #Printing ERR message actually dies with proper error message and a backtrace
	}

	return 0;
}

sub update_config_files{

	if(registry::getReg('NcalRpcDir') ne "") {
		update_init_scripts();
		update_smb_conf();
		print "Successfully updated the configuration files\n";
	}
	else{
		print "Failed to update some or all configuration files\n";
	}
}

INIT {
	@edir_versions = `/bin/ls -1 /etc/sysconfig/novell | grep -i '^edir'`;
}

1;
