#!/usr/bin/perl -I. -I/opt/novell/xad/lib64/perl -I/opt/novell/xad/lib/perl
################################################################################
# Domain Services for Windows Health Check Utility
#
# See the file LICENSE for redistribution information.
#
# Copyright (c) 2008-2009
#   Novell, Inc.  All rights reserved.
################################################################################

package helpers;

use warnings;
use strict ;


my @listofDSfWContainers = ('dn: cn=Computers',
                            'dn: cn=Users',
                            'dn: ou=Domain Controllers',
                            'dn: cn=DefaultMigrationContainer',
                            'dn: cn=Deleted Objects',
                            'dn: cn=ForeignSecurityPrincipals',
                            'dn: cn=Infrastructure',
                            'dn: cn=LostAndFound',
                            'dn: cn=NTDS Quotas',
                            'dn: cn=Program Data',
                            'dn: cn=System',
                            'dn: cn=Container'); 

# Checking the existence of the DSfW specific containers in the case of the name mapped installation.

sub topLevelContainersCheck {

    my ($treeIPAddress, $adminDN, $sslPort) = @_;

    my (@LDAPSearchOutput, $error, $CurrentDN, $searchBase, $containerRDN, $password);

#TODO get the NDSEXISTINGDOMAINNAME and NDSEXISTINGADMINPASSWD from command line. # get them from ENV
# get the certs only if this is the first time or check their existance before getting everytime.
    $password = $ENV {'NDSEXISTINGADMINPASSWD'};
    if (!$password) {
        logWrite(1, "The password for the admin DN [$adminDN] is not supplied");
        return 142;
    }
        
    ($error) = getCert($adminDN, $password);

    if ($error) {
        logWrite(1, "Failed to get the certificate for the server. ");
        return 143;
    }

    if (!$treeIPAddress) {
        $treeIPAddress = getTreeIPAddress();
        if (!$treeIPAddress) {
            $treeIPAddress = $ENV{'TREE_IP_ADDRESS'};
            if (!$treeIPAddress) {
                logWrite(1, "Don't have the tree ip address to do the search.");
                return 144;
            }
        }
        
        chomp($treeIPAddress);
        if (validateIP($treeIPAddress) != 0) {
            logWrite(1, "The IP of server running the tree [$treeIPAddress] is not proper.");
            return 145;
        }
    }

    if (!$adminDN) {
        $adminDN = $ENV {'NDSEXISTINGADMINNAME'};
        if (!$adminDN) {
            logWrite(1, "Don't have the admin DN to do the search.");
            return 146;
        }
    }

    $searchBase = `/opt/novell/xad/share/dcinit/printConfigKey.pl "Mapped Domain NC"`;
    
    if (!$searchBase) {
        logWrite(1, "The search base for the top level container check is empty");
        return 147;
    }
    chomp($searchBase);
    $searchBase = lc($searchBase);

    logWrite(5, "The LDAP Search inputs are : TR-IP [$treeIPAddress], adm-dn[$adminDN], SRCH-BS [$searchBase]");
    
    # These are read only variables so admin password is not required.
    @LDAPSearchOutput = `LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem /opt/novell/eDirectory/bin/ldapsearch -H "ldaps://$treeIPAddress:$sslPort" -D "$adminDN" -b "$searchBase" -s one dn LLL`;  
    
    if ($?) {
        logWrite(1, "Failed to get the top level containers. <$?>");
        return 148;
    }

    foreach $containerRDN (@listofDSfWContainers) {
        foreach $CurrentDN (@LDAPSearchOutput) {
            my $compDN = $containerRDN.",".$searchBase;
            if ($CurrentDN =~ /$compDN/) {
		logWrite(1, "The container [$containerRDN,$searchBase] is already present in the tree.");
		return 149;
            }
        }     
        
    }
    return 0;
} 
