#!/usr/bin/perl -I. -I/opt/novell/xad/lib64/perl -I/opt/novell/xad/lib/perl
################################################################################
# Domain Services for Windows Health Check Utility
#
# See the file LICENSE for redistribution information.
#
# Copyright (c) 2008-2009
#   Novell, Inc.  All rights reserved.
################################################################################

package helpers;

use warnings;
use strict ;
use helpers::utility;

my $myLocalhost = "127.0.0.1";

sub versionCheck {
    my ($ipAddress) = @_;
    my (@output,$minorVer, $majorVer, $spVer, $hotnpatchVer) ;

    @output = `/opt/novell/eDirectory/bin/ndsstat -h $ipAddress`; 
    
    logWrite(5, "The output of the ndsstat -h $ipAddress,: \n @output");
    chomp($output[4]);
    if ($output[4] =~ /(\s)v(\d)\.(\d)(\s)SP(\d)/) { 
        $majorVer = $2;
        $minorVer = $3;
        $spVer = $5;

        if ($majorVer == 8) {
            if ($minorVer >= 8) {
                if ($spVer >= 2) {
		    $output[4] =~ s/^.*: //g;
                    logWrite(1, "Successfully checked the version [$output[4]]");
                    return 0;
                }
            }
        }
	elsif ($majorVer > 8) {
		    $output[4] =~ s/^.*: //g;
                    logWrite(1, "Successfully checked the version [$output[4]]");
                    return 0;
	}

        logWrite(1, "Version check failed - [$majorVer.$minorVer SP$spVer]");
        return -1;
    }
    elsif ($output[4] =~ /(\s)v(\d)\.(\d)\.(\d).(\d{4})(\s)/) {
                    $majorVer = $2;
                    $minorVer = $3;
                    $spVer = $4;
		    $hotnpatchVer = $5;
                    $output[4] =~ s/^.*: //g;

                    if ($majorVer > 8) {
                        $output[4] =~ s/^.*: //g;
                            logWrite(1, "Successfully checked the version [$output[4]]");
                            return 0;
                    }
    }
    elsif ($output[4] =~ /(\s)v(\d)\.(\d)(\s)/) {
                    $majorVer = $2;
                    $minorVer = $3;
                    $output[4] =~ s/^.*: //g;

                    if ($majorVer > 8) {
                        $output[4] =~ s/^.*: //g;
                            logWrite(1, "Successfully checked the version [$output[4]]");
                            return 0;
                    }
    }
    else {
        logWrite(1, "Failed to get the version information");
        return -1;
    }
        
    logWrite(4, "The version check unsuccessfull");
    return -1;
}


# The subroutine to check the domain partition list is present locally and 
# the state of the replicas is healthly.
sub checkDomainPartitionList($$) {

	my $adminDN = shift;
	my $passwd = shift;
	my $rc = 0;
	my $ldapbase = `/opt/novell/xad/share/dcinit/printConfigKey.pl "CrossRef_DN"`;
	chomp($ldapbase);

	logWrite(1, "Checking status and local replica for partitions part of domain partition list");
	my $inst = getInstallationType();
	if ($inst ne "nmfrd" && $inst ne "nnmfrd") {
		my $forestnc = `/opt/novell/xad/share/dcinit/provisionTools.sh actual-forest-nc`;
		chomp($forestnc);
		my $netbiosname = `/opt/novell/xad/share/dcinit/printConfigKey.pl "Netbios Name"`;
		chomp($netbiosname);
		$ldapbase = "cn=$netbiosname,cn=Partitions,cn=Configuration,$forestnc";
	}

	$ldapbase =~ s/s=/st=/gi;

	if ($ldapbase eq "" || not defined ($ldapbase)) {
		logWrite(3, "The ldap base is not defined. It should have the crossref dn for the domain.");
		return -1;
	}

	# Get the list of domainPartitionList;
	my @output = ();
	if ($inst eq "nmadc" || $inst eq "nnmadc") 
	{
	 # For ADC do the ldapsearch for getting the domain partition list 
		@output = `/usr/bin/ldapsearch -D '$adminDN' -w '$passwd' -s base -b '$ldapbase' -x -Z domainpartitionlist -LLL | sed ' /^ / {; H; d; }; /^ /! {; x; s/\\n //; }; '`;
		if ($? != 0) {
			my $error = $? >> 8;
			logWrite(3, "Failed to get the domain partitions list attribute, [$error]");
			return $error;
		}
	}
	else {
		# Look into the $live_dsstatedir/domain/custom_domain_boundary.ldif
		my $live_dsstatedir = `/opt/novell/xad/share/dcinit/printConfigKey.pl "LIVE_DSSTATEDIR"`;
		chomp($live_dsstatedir);
		if (-e "$live_dsstatedir/domain/custom_domain_boundary.ldif") {
			@output = `cat $live_dsstatedir/domain/custom_domain_boundary.ldif | grep ^domainpartitionlist -i`;
		} else {
			# Need not do anything when the custom partition list is not selected
			return 0;
		}
	}

	my @partitionList = grep (/domainPartitionList/i, @output);

	logWrite(5, "The partition list is [@partitionList]");

	if (@partitionList eq "") {
		logWrite(1, "There are no partitions present in domain partition list.");
		return 0;
	}

	my $ncp_dn = `/opt/novell/xad/share/dcinit/printConfigKey.pl "NCP_DN"`;
	chomp($ncp_dn);
	$ncp_dn =~ s/,/\./g;
	
	my $tree_name =  `/opt/novell/xad/share/dcinit/ndsConfigGet.sh "n4u.base.tree-name"`;
	chomp ($tree_name);
	my $mapped_domain_nc = `/opt/novell/xad/share/dcinit/printConfigKey.pl "Mapped Domain NC"`;
	chomp($mapped_domain_nc);
	foreach my $partition (@partitionList)  {
		$partition =~ s/domainPartitionList: //i;
		chomp($partition);
		$partition =~ s/st=/s=/gi;
		if (lc($partition) eq lc($mapped_domain_nc)) {
			logWrite(1, "The mapped domain partition need not be local, continuing");
			next;
		}

		# Checking if $partition is actually a partition (BUG:#819364)
		# If not skip the check.
		my @output = `LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem /opt/novell/eDirectory/bin/ldapsearch -D '$adminDN' -w '$passwd' \\
					-b '$partition' -s base 'objectclass=partition' -x -ZZ -LLL 2>&1`;
		my $partition_query_error = $? >> 8;
		if ($partition_query_error != 0) {
			logWrite(1, "Failed to contact eDirectory Server. Error:- [$partition_query_error] [@output]\n");
			next;
		}
		my $count = $#output + 1;
		if ($count < 1) {
			logWrite(1, "The [$partition] is no longer a partition. Skipping status check for [$partition]");
			next;
		}

		$partition =~ s/,/\./g;
		$partition =~ s/^/./;
		$partition .= ".T=$tree_name.";
		my $server_ip = getWorkingReplicaIP ($partition);
		my $error = replicaStatus($adminDN, $passwd, $server_ip, $partition);
		if ($error) {
			logWrite(1, "The replica status check failed for partition [$partition]");
			return -1;
		}

		logWrite(1, "Executing /opt/novell/eDirectory/bin/ndsstat -d -p '$partition' | grep -i '$ncp_dn'");
		`/opt/novell/eDirectory/bin/ndsstat -d -p '$partition' | grep -i '$ncp_dn' | grep -ie "Read/Write" -ie "Master"`;
		if ($?) {
			logWrite(1, "ERROR : The replica of [$partition] partition is missing on the local server. Add Read/Write or Master replica to proceed."); 
			$rc = -1;
		}
	}

	return $rc;
}


# The subroutine doing the replica status check for the
# NM Additional Domain Controller (ADC)
# NNM Additional Domain Controller (ADC)

#The partition to be verified here are :
#  1. Local Partition
#  2. Configuration Partition
#  3. Schema partition


   
sub adcReplicaStatus {

    my ($adminDN, $parentIP, $inst, $sslPort) = @_;
    my ($password, $ipAddress, %listOfPartitions, @listOfReplicas, $error);
    my ($localDomPart, $forestDom); 

    logWrite(4, "The additional domain controller(ADC) replica status check"); 

# The differentiation between nm and nnm adc is only for internal purpose.
    $localDomPart = `/opt/novell/xad/share/dcinit/printConfigKey.pl "Mapped Domain NC"| sed 's/,/\./g' | sed 's/^/\./g'| sed 's/\$/\./g' `;

    if (!$localDomPart) {
        logWrite(1, "Failed to get the mapped domain nc for the ADC "); 
        return -1;
    }

    if (!$parentIP) {
        logWrite(1, "The IP Address of the parent server is not available");
        return -1;
    }

    # The password of the server contacting is not proper
    $password = $ENV{'ADM_PASSWD_PARENT'};
    if (!$password)  {
        logWrite(1, "The admin password of the server at [$ipAddress] is not set");
        return -1;
    }

    if (!$adminDN) {
        logWrite(1, "The admin DN is not set for the non-name mapped frd");
        return -1;
    }

    chomp($localDomPart, $parentIP);

    $localDomPart = lc ($localDomPart);
    $ipAddress = $parentIP;

    if ($inst eq "nmadc") {
        $forestDom = getNMForestName($parentIP, $sslPort);
        if (!$forestDom) {
            logWrite(1, "Failed to get the parent's domain partition information");
            return -1;
        }

        chomp($forestDom);

        $forestDom =~ s/,/\./g;
        $forestDom =~ s/^/\./g;
        $forestDom =~ s/\$/\./g;

        $forestDom = lc($forestDom);
    }
    elsif ($inst eq "nnmadc") { # again its just plain ADC.. maybe need to rename the inst types...
        
        
            $forestDom = `/opt/novell/xad/share/dcinit/printConfigKey.pl "Forest NC" | sed 's/,/\./g' | sed 's/^/\./g' | sed 's/\$/\./g'`;
        if (!$forestDom) {
            logWrite(1, "Failed to get the forest domain information");
            return -1;
        }

        chomp($forestDom);

        $forestDom = lc($forestDom);
    }
    else {
        logWrite(1, "The additional domain installation type is not proper");
        return -1;
    }
    
    logWrite(3, "The replica status [Admin Dn - $adminDN], parentIP [$parentIP] \
            , child Dom [$localDomPart], frstDom[$forestDom] check parameters are calculated now");
        
    ($error, %listOfPartitions) = partitionList($inst);
    if ($error) {
	logWrite(0, "Failed to get the list of partition");
	return $error;
    }

    foreach my $partition (keys %listOfPartitions) {

        chomp($partition);
        logWrite(3, "The partition for the ADC replica check [$partition]");
        my $temp = lc($partition);

        # Getting the configuration and schema partition verified.
        if ($temp =~ /^\.cn=configuration$forestDom/ || $temp =~ /^\.cn=schema\.cn=configuration$forestDom/) {
            logWrite(2, "For [$inst], the partition [$partition] is being checked for replica status");
            goto REP_CHECK;
        }
        elsif ($temp =~ /^$localDomPart/) { # Local partition should be verified here.
            logWrite(2, "For [$inst] the replica status check is done only for local partition [$partition]");
            goto REP_CHECK;

        }
        else { logWrite(3, "The partition [$partition] didn't match the local domain, configuration or schema partiton"); next;}

REP_CHECK:
        
        $error = replicaStatus($adminDN, $password, $listOfPartitions{$partition}, $partition);
        if ($error) {
            logWrite(1, "The replica status check failed for partition [$partition]");
            return -1;
        }
    }


    # Get the partitions present in the domainpartitionlist
    # Check their status and that they are present on local server

    $error = checkDomainPartitionList($adminDN, $password);
    if ($error) {
	logWrite(1, "The validation for the domain partition list on ADC failed.");
	return -1;	
    }
}

# The subroutine doing the replica status check for the
# NM Additional Domain 
# NNM Additional Domain 

sub addnlDomReplicaStatus {
    logWrite(4, "The additional domain replica status check"); 

    my ($adminDN, $parentIP, $inst, $sslPort) = @_;
    my ($password, $ipAddress, %listOfPartitions, @listOfReplicas, $error);
    my ($localDomPart, $parentDomPart, $forestDom); 

    $localDomPart = `/opt/novell/xad/share/dcinit/printConfigKey.pl "Mapped Domain NC"| sed 's/,/\./g' | sed 's/^/\./g'| sed 's/\$/\./g' `;
    if (!$localDomPart) {
        logWrite(1, "Failed to get the mapped domain nc for the child domain");
        return -1;
    }

    chomp($localDomPart);
    $localDomPart = lc($localDomPart);

    # The password of the server contacting is not proper
    $password = $ENV{'ADM_PASSWD_PARENT'};
    if (!$password)  {
        logWrite(1, "The parent's admin password of the server at [$ipAddress] is not set");
        return -1;
    }

    if (!$adminDN) {
        logWrite(1, "The admin DN is not set for the non-name mapped frd");
        return -1;
    }

    if (!$parentIP) {
        $parentIP = `/opt/novell/xad/share/dcinit/printConfigKey.pl "DNS_SERVER_IP"`;
        if (!$parentIP) {
            $parentIP = $ENV{'DNS_SERVER_IP'};
            if (!$parentIP) {
                logWrite(1, "The IP Address of the parent is not supplied");
                return -1;
            }
        }
    }

    chomp($parentIP);
    $ipAddress = $parentIP;
        
    if ($inst eq "nmchild") {

        $parentDomPart = `/opt/novell/xad/share/dcinit/printConfigKey.pl "Mapped Domain NC" | sed 's/,/\./g' | sed 's/^OU=[A-Za-z0-9]*.//g' | sed 's/^/\./g' | sed 's/\$/\./g'`;
        if (!$parentDomPart) {
            logWrite(1, "Failed to get the parent's domain partition information");
            return -1;
        }

        $forestDom = getNMForestName($parentIP, $sslPort);
        if (!$forestDom) {
            logWrite(1, "Failed to get the forest NC name");
            return -1;
        }

        $forestDom =~ s/,/./g;
        $forestDom =~ s/^/./g;
        $forestDom =~ s/$/./g;

        chomp($parentDomPart, $forestDom);
        $parentDomPart = lc($parentDomPart);
        $forestDom = lc($forestDom);

    }
    elsif ($inst eq "nnmchild") {

        $parentDomPart = `/opt/novell/xad/share/dcinit/printConfigKey.pl "Parent NC" | sed 's/,/\./g' | sed 's/^/\./g' | sed 's/\$/\./g'`;
        if (!$parentDomPart) {
            logWrite(1, "Failed to get the parent's domain partition information");
            return -1;
        }

        $forestDom = `/opt/novell/xad/share/dcinit/printConfigKey.pl "Forest NC" | sed 's/,/\./g' | sed 's/^/\./g' | sed 's/\$/\./g'`;
        if (!$forestDom) {
            logWrite(1, "Failed to get the forest domain information");
            return -1;
        }

        chomp($parentDomPart, $forestDom);

        $parentDomPart = lc($parentDomPart);
        $forestDom = lc($forestDom);
    }
    else {
        logWrite(1, "The additional domain installation type is not proper");
        return -1;
    }

    logWrite(3, "The replica status [Admin Dn - $adminDN], parentIP [$parentIP] \
            Parent Dom [$parentDomPart], child Dom [$localDomPart], frstDom[$forestDom] check parameters are calculated now");
        
    ($error, %listOfPartitions) = partitionList($inst);
    if ($error) {
	logWrite(0, "Failed to get the list of partition");
	return $error;
    }

    foreach my $partition (keys %listOfPartitions) {

        chomp($partition);
        logWrite(5, "The partition returned by ndsstat [$partition]");

        # Getting the configuration and schema partition verified.
        my $temp = lc ($partition);
        if ($temp =~ /^\.cn=configuration$forestDom/) {
            logWrite(2, "For [$inst], the configuration partition [$partition] is being checked for replica status");
            goto REP_CHECK;
        }
        elsif ($temp =~ /^\.cn=schema\.cn=configuration$forestDom/) {
            logWrite(2, "For [$inst], the schema partition [$partition] is being checked for replica status");
            goto REP_CHECK;
        }
        else { logWrite(5, "The partition [$partition] didn't match the configuration or schema partiton"); }

        # For nm frd checking only the replica status of tree root partition.
        if (($inst eq "nmchild")) {

            my $temp = lc($partition);
            if ($temp =~ /^$parentDomPart/) { # Parent domain partition should be checked here.
                logWrite(2, "For [$inst] the replica status check is done only for parent partition [$partition]");
                goto REP_CHECK;
            }
            elsif ($temp =~ /^$localDomPart/) { # Local partition should be verified here.
                logWrite(2, "For [$inst] the replica status check is done only for local partition [$partition]");
                goto REP_CHECK;

            }
            else {
                logWrite(3, "For [$inst], replica status check is not done for partition[$partition]");
                next;
            }
        }
        elsif ($inst eq "nnmchild") {
            logWrite(5, "Checking the partition [$partition]");

            my $temp = lc($partition);
            if ($partition =~ /^$parentDomPart/) { 
                logWrite(2, "For [$inst], the parent's domain partition [$partition] is being checked for replica status");
                goto REP_CHECK;
            } 
            else 
            { logWrite(5, "For [$inst], partition [$partition] is not checked for any replica status") ; next;}
        }
    

REP_CHECK:
        
        $error = replicaStatus($adminDN, $password, $listOfPartitions{$partition}, $partition);
        if ($error) {
            logWrite(1, "The replica status check failed for partition [$partition]");
            return -1;
        }
    }

    # Get the partitions present in the domainpartitionlist
    # Check their status and that they are present on local server

    $error = checkDomainPartitionList($adminDN, $password);
    if ($error) {
	logWrite(1, "The validation for the domain partition list on ADC failed.");
	return -1;	
    }
}

# The Subroutine doing the replica status check for 
# NM FRD 
# NNM FRD
sub frdReplicaStatus {

    my ($adminDN, $parentIP, $inst) = @_;
    my ($password, $ipAddress, %listOfPartitions, @listOfReplicas, $error);

    if ($inst eq "nmfrd") {
        if (!$adminDN) {
            $adminDN = $ENV{'$NDSEXISTINGADMINNAME'};
            if (!$adminDN) {
                logWrite(1, "The admin DN for the NM FRD setup is not supplied");
                return -1;
            }
        }

        $password = $ENV {'NDSEXISTINGADMINPASSWD'};
        if (!$password) {
            logWrite(1, "The password for tree admin in NM frd setup is not supplied");
            return -1;
        }

        if (!$parentIP) {
            $parentIP = $ENV {'TREE_IP_ADDRESS'};
            if (!$parentIP) {
                logWrite(1, "The IP Address of the server holding the tree is not provided");
                return -1;
            }
        }
        $ipAddress = $parentIP;
    }
    elsif ($inst eq "nnmfrd") {
        
        if (!$parentIP) {
            $parentIP = `/opt/novell/xad/share/dcinit/printConfigKey.pl "IP Address"`;
            if (!$parentIP) {
                logWrite(1, "The IP Address of the current server is not available");
                return -1;
            }
            chomp($parentIP);
        }
        $ipAddress = $parentIP;
        
        # The password of the server contacting is not proper
        $password = $ENV{'ADM_PASSWD'};
        if (!$password)  {
            logWrite(1, "The admin password of the server at [$ipAddress] is not set");
            return -1;
        }
        
        if (!$adminDN) {
            logWrite(1, "The admin DN is not set for the non-name mapped frd");
            return -1;
        }
    }
    else { logWrite (0, "The replica status check is done for invalid installation [$inst]"); exit(-1); }

    logWrite(5, "The replica status [Admin Dn - $adminDN], parentIP [$parentIP] check parameters are calculated now");

    ($error, %listOfPartitions) = partitionList($inst);

    if ($error) {
        logWrite(1, "Failed while getting the list of Partitions for admin DN [$adminDN], [$ipAddress]");
        return $error;
    }

    foreach my $partition (keys %listOfPartitions) {

        chomp($partition);
        logWrite(5, "The partition for the frd is [$partition]");

        # For nm frd checking only the replica status of tree root partition.
        if (($inst eq "nmfrd")) {
            my ($domPart, $temp);

            $domPart = `/opt/novell/xad/share/dcinit/printConfigKey.pl "Mapped Domain NC" | sed 's/,/\./g'| sed 's/^/\./g' | sed 's/\$/\./g'`;
            if (!$domPart) {
                logWrite(1, "The mapped domain NC is not obtained for the NM FRD");
                return -1;
            }
            
            chomp($domPart);
            $domPart = lc($domPart);

            $temp = lc($partition);
            logWrite(5, "The domain partition is [$domPart] match against [$temp]");
            if ($temp =~ /^$domPart/) {
                logWrite(2, "The domain partition being checked is $partition");
                goto REP_CHECK;
            }
            else {
                logWrite(5, "For NNM FRD, replica status check is not done for partition[$partition]");
                next; 
            }
        }
        elsif ($inst eq "nnmfrd") {
            logWrite(5, "Checking the partition [$partition]");

            if ($partition =~ /^\.T/) {
                logWrite(2, "For NNM FRD the replica status check is done only for tree root partition [$partition]");
                goto REP_CHECK;
            }
            else {
                logWrite(5, "For NNM FRD, replica status check is not done for partition[$partition]");
                next;
            }
        }
    
REP_CHECK:
               
        $error = replicaStatus($adminDN, $password, $listOfPartitions{$partition}, $partition);
        if ($error) {
            logWrite(1, "The replica status check failed for partition [$partition]");
            return -1;
        }
    }

    # Get the partitions present in the domainpartitionlist
    # Check their status and that they are present on local server

    $error = checkDomainPartitionList($adminDN, $password);
    if ($error) {
	logWrite(1, "The validation for the domain partition list on ADC failed.");
	return -1;	
    }

    return 0;
}

# process the ndscheck output for each partition and
# verifies the server/replica state

sub processReplicaStatus {

	my ($ServerNameArrayRef, $PartitionStatusArrayRef) = @_;

	my $ServerNameArrayCount = @$ServerNameArrayRef;
	my $PartitionStatusArrayCount = @$PartitionStatusArrayRef;

	my $serverfound = 0;
	my $rc = 0;

	for(my $counter1=0;$counter1< $ServerNameArrayCount;$counter1++)
	{
		if($$ServerNameArrayRef[$counter1] ne "")
		{
			for(my $counter2=0;$counter2< $PartitionStatusArrayCount;$counter2++)
			{
				if($$PartitionStatusArrayRef[$counter2] =~ /^\.$$ServerNameArrayRef[$counter1]/)
				{
					logWrite(3, "entry for server $$ServerNameArrayRef[$counter1] found");
					logWrite(3, "$$PartitionStatusArrayRef[$counter2]");
					$serverfound = 1;
				}

				if($serverfound)
				{
					my $temp = $$PartitionStatusArrayRef[$counter2]; 
					$temp =~ s/^\.CN=.*\.+\s//g;
					my @replicastatus = split("\\s+", $temp);
					logWrite(3, "replicastatus: @replicastatus");
					#the data is in format Status, Time Sync, Time Delta, Replica State
					my $replica_status = $replicastatus[1];
					my $replica_time_sync = $replicastatus[2];
					my $replica_time_delta_mins = $replicastatus[3];
					my @time = split(":", $replicastatus[4]);
					my $replica_time_delta_secs = $time[1];
					my $replica_state = $replicastatus[6];

					if($replica_status ne "UP")
					{
						logWrite(1, "Replica Status is $replica_status. For proper functioning of DSfW provisioning and installation it should be UP. Verify the state of replica on server $$ServerNameArrayRef[$counter1] and try again once it is UP");
						$rc = -1;
						goto out;
					}
					if(($replica_time_sync ne "YES") and ($replica_time_delta_mins > 5))
					{
						logWrite(1, "Replica time sync status is $replica_time_sync. For proper functioning of DSfW provisioning and installation it should be 'YES'. Verify the state of replica on server $$ServerNameArrayRef[$counter1] and try again once it is YES");
						$rc = -1;
						goto out;
					}
					if(($replica_time_delta_mins > 5) and ($replica_time_sync ne "YES"))
					{
						logWrite(1, "Replica time sync delta is greate than 5 Minutes. For proper functioning of DSfW provisioning and installation it should be less than 5 minutes. Verify the state of replica on server $$ServerNameArrayRef[$counter1] and try again once it is less than 5 minutes");
						$rc = -1;
						goto out;
					}
					if(($replica_time_delta_secs > 59) and ($replica_time_sync ne "YES"))
					{
						logWrite(1, "Replica time sync delta is greater than the permissible limit. For proper functioning of DSfW provisioning and installation it should be well within the eDirectory time delta permissible limits. Verify the state of replica on server $$ServerNameArrayRef[$counter1] and try again once it is under permissible limits");
						$rc = -1;
						goto out;
					}
					if($replica_state ne "ON")
					{
						logWrite(1, "Replica state is $replica_state. For proper functioning of DSfW provisioning and installation it should be ON. Verify the state of replica on server $$ServerNameArrayRef[$counter1] and try again once it is ON");
						$rc = -1;
						goto out;
					}

				$serverfound = 0;
				}
			}
		}
	}
out:
	return $rc;
}


# Subroutine to get the status of replicas
# 1. Replica Status (UP/DOWN)
# 2. Time Synchronized (YES/NO)
# 3. Replica State  (ON/OFF)

sub replicaStatus {
    my ($adminDN, $password, $ipAddress, $partition) = @_;
    my (@output, @listOfReplicas, $error);

    my $Treename =  `/opt/novell/xad/share/dcinit/ndsConfigGet.sh "n4u.base.tree-name"`;
    chomp $Treename;
    $partition =~ s/\.st=/\.s=/gi;
    $adminDN =~ s/\.st=/\.s=/gi;

    logWrite(5, "The replica status check is being done for [$adminDN, $ipAddress, $partition]");


    eval {
        @listOfReplicas = `/opt/novell/eDirectory/bin/ndsstat -p "$partition" -h $ipAddress -d 2>&1 | grep -iv "Subordinate" | grep \"^\.CN\" | awk -F \".T\" \'{print \$1.\".T=$Treename\"}\'`;
	chomp(@listOfReplicas);            
        logWrite(5, "The list of replicas for partition [$partition] [@listOfReplicas]");
    };
    
    if ($@) {
        logWrite(1, "Failed to get the list of replicas");     
        return -1;
    }

    $adminDN =~ s/,/./g;

    $error = checkCredentials($ipAddress, $adminDN, $password);
    if ($error != 0) {
        logWrite(1, "The credential check failed for the replica status check failed. <$error>");
        return -1;
    }
    
	my $PartitionSplitPattern = "";
	if($partition ne "")
	{
		$PartitionSplitPattern = "\"" . $partition . "\"";
	}

	# run ndscheck on the designated server
	my @ndscheck_output_arr;
	my $ndscheck_filename = "/var/opt/novell/xad/log/ndscheck.log";

	logWrite(3, "Executing [ndscheck] a[$adminDN], ip [$ipAddress] ");
	eval {
		@ndscheck_output_arr = `/opt/novell/eDirectory/bin/ndscheck -h $ipAddress -a '$adminDN'  -w '$password' -F $ndscheck_filename`;
		die("ndscheck error: failed for partition [$partition] ") if ($? != 0);
	};
	
	if ($@)
	{
		logWrite(5, $@);
	}

	my @ndscheck_output_display = grep(/^Status of partition/ || /^\.CN/i || /^\+/ || /^\s*Server Name/, @ndscheck_output_arr);
	logWrite(3, "Displaying ndscheck returned output:\n@ndscheck_output_display");

	my $ndscheck_output_str = join("", @ndscheck_output_arr);
	my @PartitionStatusArray = split("Status of partition", $ndscheck_output_str);
	my $PartitionStatusArrayCount= @PartitionStatusArray;
	my @PartitionStatus = "";

	for(my $i=0;$i<$PartitionStatusArrayCount;$i++)
	{
		if($PartitionStatusArray[$i] =~ /^\s+$PartitionSplitPattern/i)
		{
			@PartitionStatus = split("\\n", $PartitionStatusArray[$i]);
		}
	}

	my $server = "";
	my @PartitionHoldingServersNames = "";

	foreach $server (@listOfReplicas)
	{
		chomp $server;
		my @temp = split(/\./, $server);
		push(@PartitionHoldingServersNames, $temp[1]);
	}

	logWrite(3, "PartitionHoldingServersNames array contains: @PartitionHoldingServersNames");

	# process partition replica status
	my $rc = processReplicaStatus(\@PartitionHoldingServersNames, \@PartitionStatus);
	return $rc;
}
			
# Subroutine to get the working server's ip address from the ncp
# server list obtained by 'ndsstat -p <partition-name>' which would be
# used for checking the partition's replicas status later.
sub getWorkingReplicaIP ($) {
	my $partition = shift;

	$partition =~ s/\.st=/\.s=/gi;
	my @ncpDN = `/opt/novell/eDirectory/bin/ndsstat -p '$partition' -n 2>&1 | grep ^.CN`;
	if ($? ne 0) {
		logWrite(0, "Failed to get the list of replicas for partition [$partition]");
		return (-1, "");
	}
	chomp(@ncpDN);
   
	my $ip_addr  = "";
        for (my $index = 0; $index <= $#ncpDN ; $index++) {
            my $ncp_dn = $ncpDN[$index];
            logWrite (5, "Getting the IP Address for the NCP DN [$ncp_dn]");

            $ncp_dn =~ s/^\.//g;
            $ncp_dn =~ s/\.T=.*//g;

            $ncp_dn =~ s/\./,/g;
            my $IS_REPLICA = `/opt/novell/xad/share/dcinit/printConfigKey.pl "IS_REPLICA"`;
            chomp($IS_REPLICA);

	    #getIpFromNcp does not work with "127.0.0.1" in oes2018-edir9
	    #Change due to -h changing into -H for ldapsearch inside
	    my $local_server_ip = `/bin/hostname -i`;
	    chomp($local_server_ip);
            if ($IS_REPLICA ne "TRUE" and defined $ENV{NDSEXISTINGADMINPASSWD}) {
		if (defined $ENV{ADM_PASSWD}) {
	                my $temp_admpasswd = $ENV{ADM_PASSWD};
	            	$ENV{ADM_PASSWD} = $ENV{NDSEXISTINGADMINPASSWD};
			$ip_addr = util::getIpFromNcp($ncp_dn, "$local_server_ip");
                	$ENV{ADM_PASSWD} = $temp_admpasswd;
		}else {
	            	$ENV{ADM_PASSWD} = $ENV{NDSEXISTINGADMINPASSWD};
			$ip_addr = util::getIpFromNcp($ncp_dn, "$local_server_ip");
			delete $ENV{ADM_PASSWD};
		}
            }else {
                $ip_addr = util::getIpFromNcp($ncp_dn, "$local_server_ip");
            }

	    logWrite(1, "The ip address is [$ip_addr]\n");
            if ($ip_addr eq "") {
                logWrite(1, "failed to get the ip address from ncpdn $ncp_dn");
                next; 
            }

            `/opt/novell/eDirectory/bin/ndsstat -h $ip_addr`;
            if ($? eq 0) {
		logWrite(3, "Found the server IP [$ip_addr] for partition [$partition]");
                last;
            }
            else {
                logWrite(3, "The Edirectory is not running or the server is not rechable on [$ip_addr]");
                next;
            }

        }

	my $error = validateIP($ip_addr);
	if ($error) {
		logWrite(1, "The IP Address validation failed for IP [$ip_addr]");
		return (-1, "");
	}

	return (0, $ip_addr);
}

# Subroutine to get the list of partitions
sub partitionList {
    my ($inst) = shift;
    my (%output, $error); 
    my $ip;

    %output = ();
    if (not defined ($inst)) {
	$inst = getInstallationType();
	logWrite(5, "Got the installation type as [$inst]");
    }


    my $domainNC = `/opt/novell/xad/share/dcinit/printConfigKey.pl "Mapped Domain NC"`;
    chomp $domainNC;
    $domainNC =~ s/st=/s=/gi;
    logWrite(5, "domainNC : $domainNC");

    my $ParentNC = `/opt/novell/xad/share/dcinit/printConfigKey.pl "Parent NC"`;
    chomp $ParentNC;
    logWrite(5, "ParentNC: $ParentNC");

    my $forestDNSName = `/opt/novell/xad/share/dcinit/printConfigKey.pl  "Forest Root"`;
    chomp $forestDNSName;
    logWrite(5, "forestDNSName : $forestDNSName");

    my $Treename =  `/opt/novell/xad/share/dcinit/ndsConfigGet.sh "n4u.base.tree-name"`;
    chomp $Treename;

	# create partition names
	my $domainPartitionNC = "." . lc ($domainNC) . ".T=$Treename.";
	my $TreeRootPartitionNC = ".T=$Treename.";
	my $ParentDomainPartitionNC = "";
	my $configurationPartitionNC = "";
	my $schemaPartitionNC = "";

	$domainPartitionNC =~ s/,/\./g;
	if($inst eq "nnmchild")
	{
		$ParentDomainPartitionNC = "." . lc ($ParentNC) . ".T=$Treename.";
		$ParentDomainPartitionNC =~ s/,/\./g;
		logWrite(5, "ParentDomainPartitionNC : $ParentDomainPartitionNC");
	}

	logWrite(5, "domainPartitionNC: $domainPartitionNC");
	logWrite(5, "TreeRootPartitionNC: $TreeRootPartitionNC");

	if ($inst ne "nnmfrd" && $inst ne "nmfrd")
	{
		my $forestNC = `/opt/novell/xad/share/dcinit/provisionTools.sh actual-forest-nc`;
		chomp $forestNC;
		logWrite(5, "forestNC : $forestNC");
		$forestNC =~ s/st=/s=/gi;

		my $configurationNC = "CN=Configuration,". $forestNC;
		logWrite(5, "configurationNC: $configurationNC");

		my $schemaNC = "CN=Schema," . $configurationNC;
		logWrite(5, "schemaNC : $schemaNC");

		# create partition names
		$configurationPartitionNC = "." . lc ($configurationNC) . ".T=$Treename.";
		$schemaPartitionNC = "." . lc ($schemaNC) . ".T=$Treename.";
		$configurationPartitionNC =~ s/,/\./g;
		$schemaPartitionNC =~ s/,/\./g;
		logWrite(5, "configurationPartitionNC : $configurationPartitionNC");
		logWrite(5, "schemaPartitionNC: $schemaPartitionNC");
	}

	# we will create a list of partitions on the basis of install type
	# as the server (in NM cases) can hold N number of other replicas 
	# of other partitions that would not effect the DSfW realted partitioning
	# operations

	if($inst eq "nnmfrd")
	{
		# return only the tree root partition after validating the same
		eval {
			($error, $ip ) = getWorkingReplicaIP($TreeRootPartitionNC); 
			if ($error) {die ("Failed to get parent domain partition") }
		};
		if ($@)
		{ 
			logWrite(1, " Failed to locate tree root partition $TreeRootPartitionNC on server");
			goto out;
		}
		else
		{
			# push the tree root partition in the output array and send back
			$output {$TreeRootPartitionNC} = $ip;
			logWrite(5, " List of relevent partitions for install type $inst validated on server");
		}
	} elsif($inst eq "nmfrd")
	{
		# return domain partition only after validating the same 
		eval {
			($error, $ip) = getWorkingReplicaIP($domainPartitionNC); 
			if ($error) {die ("Failed to get parent domain partition") }
		};
		if ($@)
		{
			logWrite(1, " Failed to locate domain partition $domainPartitionNC");
			goto out;
		}
		else
		{
			# push the domain partition in the output array and send back
			$output {$domainPartitionNC} = $ip;
			logWrite(5, " List of relevent partitions for install type $inst validated on server");
		}
	} elsif($inst eq "nnmchild")
	{
		# return the parent domain partition, config partition and schema partition
		# after validating 
		if(defined($ParentDomainPartitionNC))
		{
			eval {
				($error, $ip) = getWorkingReplicaIP($ParentDomainPartitionNC);
				if ($error) {die ("Failed to get parent domain partition") }
			};
			if ($@)
			{
				logWrite(1, " Failed to locate parent domain partition $ParentDomainPartitionNC");
				goto out;
			}
			else
			{
				# push the parent partition and master ip in output hash 
				$output {$ParentDomainPartitionNC} = $ip;
			}
		}
		eval {
			($error, $ip) = getWorkingReplicaIP($configurationPartitionNC); 
			if ($error) {die ("Failed to get configuration partition") }
		};
		if ($@)
		{
			logWrite(1, " Failed to locate configuration partition $configurationPartitionNC");
			goto out;
		}
		else
		{
			$output {$configurationPartitionNC} = $ip;
		}
		
		eval {
			($error, $ip) = getWorkingReplicaIP($schemaPartitionNC);
			if ($error) {die ("Failed to get schema partition") }
		};
		if ($@)
		{
			logWrite(1, " Failed to locate schema partition $schemaPartitionNC on server");
			goto out;
		}
		else
		{
			$output{$schemaPartitionNC} = $ip;
			logWrite(5, " List of relevent partitions for install type $inst validated on server");
		}
	} elsif($inst eq "nmchild")
	{
		# return back domain, config and schema partition
		eval {
			($error, $ip) = getWorkingReplicaIP($domainPartitionNC);
			if ($error) {die ("Failed to get parent domain partition") }

		};
		if ($@)
                {
                        logWrite(1, " Failed to locate domain partition $domainPartitionNC on server");
			goto out;
		}
		else
		{
			$output{$domainPartitionNC} = $ip;
		}

		eval {
			($error, $ip) = getWorkingReplicaIP($configurationPartitionNC);
			if ($error) {die ("Failed to get configuration partition") }
		};
		if($@)
		{
			logWrite(1, "Failed to locate configuration partition $configurationPartitionNC on server ");
			goto out;
		}
		else
		{
			$output{$configurationPartitionNC} = $ip;
		}

		eval {
			($error, $ip) = getWorkingReplicaIP($schemaPartitionNC);
			if ($error) {die ("Failed to get schema partition") }
		};
		if($@)
		{
			logWrite(1, " Failed to locate schema partition $schemaPartitionNC on server");
			goto out;
		}
		else
		{
			$output {$schemaPartitionNC} = $ip;
		}

	} elsif(($inst eq "nnmadc") or ($inst eq "nmadc"))
	{
		# we should return domain, config and schema partition
		eval {
			($error, $ip) = getWorkingReplicaIP($domainPartitionNC);
			if ($error) {die ("Failed to get parent domain partition") }
		};
		if($@)
		{
			logWrite(1, " Failed to locate domain partition $domainPartitionNC on server");
			goto out;
		}
		else
		{
			$output{$domainPartitionNC} = $ip;
		}
	        my $configSchemaReplica = `/opt/novell/xad/share/dcinit/printConfigKey.pl "CONFIG_SCHEMA_REPLICAS"`;
		chomp($configSchemaReplica);
		if ($configSchemaReplica eq "FALSE")  {
			logWrite(1, "Don't require to validate schema and configuration partition");
			goto out;
		}
	
		eval {
			($error, $ip) = getWorkingReplicaIP($schemaPartitionNC);
			if ($error) {die ("Failed to get schema domain partition") }
                };
                if($@)
                {
                        logWrite(1, " Failed to locate schema partition $schemaPartitionNC on server");
			goto out;
                }
                else
                {
			$output{$schemaPartitionNC} = $ip;
                }

		eval {
			($error, $ip) = getWorkingReplicaIP($configurationPartitionNC);
			if ($error) {die ("Failed to get configuration partition") }
		};
		if($@)
		{
			logWrite(1, " Failed to locate configuration partition $configurationPartitionNC on server ");
			goto out;
		}
		else
		{
			$output{$configurationPartitionNC} = $ip;
		}

	}
out:
	return($error, %output);
}

# The subroutine multiplexing the replica status check.
sub instTypeReplicaStatus {
    my ($adminDN, $installation, $parentIP, $sslPort) = @_;
    my $error ;

    $error = 0;

    logWrite(5, "The replica status check for [$installation], adminDN [$adminDN], ParentIP [$parentIP]");

    if (lc($adminDN) !~ /^cn=/) { # check not sufficient.. enhance it
        logWrite(1, "The admin DN is not in LDAP format while checking the replica status");
        return 153;
    }

    if ($installation eq "nmfrd" || $installation eq "nnmfrd") {
        $error = frdReplicaStatus($adminDN, $parentIP, $installation);
        if ($error) {return 154;}
    } 
    elsif ($installation eq "nmchild" || $installation eq "nnmchild") {
        $error = addnlDomReplicaStatus($adminDN, $parentIP, $installation, $sslPort);
        if ($error) {return 155;}
    }
    elsif ($installation eq "nmadc" || $installation eq "nnmadc") {
        $error = adcReplicaStatus($adminDN, $parentIP, $installation, $sslPort);
        if ($error) {return 156;}
    }
    else {
        logWrite(1, "The type of installation is not matching.");
    }

    return $error;
}

# Subroutine to check that the current server has the 
# latest schema.
sub schemaCheck() {
    my ($rc , $counter) = 0;
    do {
        `/opt/novell/eDirectory/bin/ldapsearch  -s base -b cn=schema -x attributetypes=uniquedomainid -LLL >&/dev/null`;
	$rc = $?;
        if ($rc) {
            sleep (5);
	    logWrite(5, "Schema not synched, waiting");
        }
        $counter++;
    } while ($rc != 0 && $counter < 5);

    if ($rc) {
        logWrite(2,"The current server doesn't have the copy of the latest schema yet");
        return -1;
    }
    logWrite(3,"The schema on the local server is uptodate");
    return $rc;
}

# The subroutine to check the version and the replica status;
# The arguments supplied :
# 1. admin DN  - This is container DN of the administrator in case of NNM and DN of tree admin.
# 2. instType - NM / NNM
# 3. domType - frd/child/adc
# 4. parentIP - Tree IP in case of NM and parent domain IP or DNS_SERVER_IP in case of child/adc. 
#               In case of nnm frd it is the local IP. 


sub ndsServerStatusCheck {
    my ($error, $instType) ;
    my ($adminDN, $installationType, $domType, $parentIP, $sslPort) = @_;
    
    if (!$installationType || !$domType) {
        logWrite (1, "The installation type is not properly given, \
                installation Type [$installationType], server Type [$domType] ");
        return 151;
    }

    $instType = getInstallationType($installationType, $domType);

    $error = versionCheck ($parentIP);
    if ($error) {
       logWrite(1, "The version check for the server failed");
      return 152;
    }

    #This checks replica time synchronization and its state.
    $error = instTypeReplicaStatus ($adminDN, $instType, $parentIP, $sslPort);
    if ($error) {
        logWrite(1, "The replca status check failed.");
        return $error;
    }

    # Check the schema of the current server.
    $error = schemaCheck();
    if ($error) {
	    logWrite(1,"The schema check on the current server failed.");
	    return 157;
    }

#    TODO : This check will be done as part of yast input validation.
#    $error = hierarchyCheck($instType);
#    if ($error) {
#        logWrite(1, "The heirarchy rule for the dsfw installation is not maintained. <$error>");
#        return 157
#    }

    logWrite(0, "Successfully health checked the NDS server status");
    return 0;
}

# Getting the partition status

sub getPartitionStatus {
    my ($ipAddress, $partition) = @_;
    my (@repList, $error) ; 

    logWrite(1, "The parameter passed to partition [$partition] and ipAddress [$ipAddress]");

    $error = validateIP($ipAddress);
    if ($error) {
        logWrite(1, "The IP Address supplied is not in proper format. [$ipAddress]");
        return -1;
    }

    if ($ipAddress) {
        @repList = `/opt/novell/eDirectory/bin/ndsstat -p $partition -h $ipAddress`; # absolute path needed... 
    }
    else {
        @repList = `/opt/novell/eDirectory/bin/ndsstat -p $partition`;
    }

    foreach my $replica (@repList) {
        chomp($replica) ;

        logWrite(5, "Obtained the replica [$replica] ");
        if ($replica =~ /^.CN(\S*)(\s*)(\S*)(\s*)(\S*)(\s*)/) {
            logWrite(5, "The parsed output of the replica status : [$1], [$2], [$3], [$4], [$5]");
            if ($5 eq "On") {
                logWrite(3, "The replica [$replica] is in ON state");
            }
            else {
                logWrite(1, "Failed The replica [$replica] state is [$5]");
                return -1;
            }
        }
    }

    logWrite(1, "For the partition [$partition] all the replicas are okay");
    return 0;
}
1;
