#!/usr/bin/perl -I. -I/opt/novell/xad/lib64/perl -I/opt/novell/xad/lib/perl
################################################################################
# Domain Services for Windows Health Check Utility
#
# See the file LICENSE for redistribution information.
#
# Copyright (c) 2008-2009
#   Novell, Inc.  All rights reserved.
################################################################################

package helpers;

use warnings;
use strict ;

# The remote status check for the kerberos service.
sub kerberosStatus {
    my $output = "";
    my ($principalName, $password) = @_;
    logWrite (5, "The principal name for doing kinit [$principalName]");
    $ENV {'KRB5_CONFIG'} = `/bin/mktemp`;
    $ENV{'ADM_PASSWD'} = $password;
    $output = `/opt/novell/xad/bin/kinit -E $principalName 2>&1` ;
    unlink ($ENV{'KRB5_CONFIG'});
    $ENV{'KRB5_CONFIG'} = "";
    if ($output =~ /^kinit\(v5\): (.*)/) {
        logWrite (1, " Kinit for [$principalName] Failed: [$1] \n");
        $output = $1;
# TODO : look for other error messages also, like cannot contact KDC in realm etc etc
        if ($output =~ /Clock skew/) {
            logWrite(1, "Clock skew detected between servers, please sync time between them using tools such as NTP");
        } 
        return ($output, $?);
    }
    else {
        logWrite (2, "Successfully health checked the kerberos service.\n");
        return ("Success", 0);
    }
}

# The remote status check subroutine for checking :
# 1. RPC service
# 2. Samba service
sub rpcNSambaStatus {
    my ($principal, $password, $ipAddress, $hostName) = @_;
    my ($output, @rpcclientOut, $expectedOutput, $error);
    my $count = 0;

    $expectedOutput = 'Machine Role = ';

    logWrite(5, "The kinit for rpcclient : principal [$principal]");
    # From klist manpage : The exit status is `0' if klist finds a credentials cache, and 
    #`1' if it does not or if the tickets are expired.
    `/opt/novell/xad/bin/klist -s` ;
    if ($? != 0) { 
        $ENV{'KRB5_CONFIG'} = `/bin/mktemp`;
        $ENV{'ADM_PASSWD'} = $password;
        $output = `/opt/novell/xad/bin/kinit $principal`;
        $ENV{'KRB5_CONFIG'} = "";
        $ENV{'ADM_PASSWD'} = ""; 
        if ($?) {
            logWrite(1, "Failed to obtain the kerberos TGT for $principal");
            return ($1, $?);
        }
        else {
            logWrite(4, " Sucessfully obtained the kerberos TGT for $principal");
        }

    }

    logWrite(5, "Got the kerberos ticket for domain [$principal]");
    do {
        ## get UID for the current user.
        my $uid = `id -u`;
        chomp($uid);

        @rpcclientOut = `KRB5CCNAME=/tmp/krb5cc_$uid /usr/bin/rpcclient -k $hostName -c dsroledominfo 2>&1`;
        chomp($rpcclientOut[0]);
        $error = $?;
        if ($rpcclientOut[0] =~ /$expectedOutput/) {
            logWrite(2, "Health check of RPC and Samba service succeeded. \n");
            return ("Success", 0);
        }
        else {
            logWrite (1, "Failed to get the DC information for $hostName.\n The output is [$rpcclientOut[0]] \n");
            #return ($rpcclientOut[0], $?);
            if ($rpcclientOut[0] =~ /NT_STATUS_LOGON_FAILURE/) {
                logWrite(1, "Ensure that the time on this server is in sync with other DC. Rerun the task once the time is rectified.\n");
            }
        }
        $count++;
    }
    while ($error != 0 and $count < 5);
    logWrite (1, "Health check of RPC and Samba service failed: ... \n");
    return ($rpcclientOut[0], 0);
}


# The remote status check for the LDAP service.
sub ldapStatus {
    my ($domainName) = @_;
    my (@output);

    @output = `/opt/novell/xad/sbin/provision -l 60 --locate-dc $domainName 2>&1`;
    if ($? != 0) {
        my $error = $? >> 8;
	    return ("Failed to get the domain information [$domainName]. <$error>", $error);
    }

    chomp($output[3]);
    logWrite (5, "The output is [$output[3]] \n");

    if ($output[6] =~ /Dom Name: $domainName/){
        logWrite (2, "Successfully health checked the LDAP server status on remote domain. \n");
        return ("Success", 0);
    }
    else {
        logWrite (1, "Failed to get the remote LDAP server status on [$domainName] \n");
        return ($output[0], $?);
    }


}

# The remote status check for the NMB service which does the 
# name resolution.
sub nmbStatus {
    my ($ipAddress) = @_; 
    my ($error, $output);

    $output = `/usr/bin/nmblookup -A $ipAddress > /dev/null`;

    if ($?) {
	$error = $? >> 8;
        logWrite(1, "Failed to run the nmblookup for the remote domain[$ipAddress]. [$output]. <$error>");
        return ("The nmb status check failed for [$ipAddress]", $error);
    }

    logWrite(5, "Successfully checked the nmb service status on the remote domain [$ipAddress]");

    return ("The nmb status check completed successfully", 0);
}
1;
