#!/usr/bin/perl -I. -I.. -I/opt/novell/xad/lib64/perl -I/opt/novell/xad/lib/perl
################################################################################
# DSfW Migration script from sp3 to oes11
#
# See the file LICENSE for redistribution information.
#
# Copyright (c) 2011
#       Novell, Inc.  All rights reserved.
################################################################################
#
# Note:
# Please do the following before executing this script
# Execute the command on target machine "ssh-keygen -t rsa"
# Copy it to source "scp /root/.ssh/id_rsa.pub source:/tmp"
# At source check for the directory /root/.ssh if not present create it
# Append the key details from /tmp/id_rsa.pub to /root/.ssh/authorized_keys as
# "cat /tmp/id_rsa.pub >> /root/.ssh/authorized_keys"
#
################################################################################
package migration;
$ENV{'PATH'} = "/opt/novell/xad/share/dcinit:$ENV{'PATH'}";

use strict;
use warnings;
use XAD::registry;
use NDS::stage;
use Logger qw(:DEFAULT);
use File::Copy;
use File::Path;
use util;
use Config::IniFiles;
use migration::migratesp2DSfW2oes11;

use vars qw($LOCAL_SBIN_DIR $target_backup_dir);
$LOCAL_SBIN_DIR="/opt/novell/xad/sbin/";
$target_backup_dir = "/var/opt/novell/xad/.migrate_dsfw/";

sub replace_pattern_in_file {
        my $replaced_pattern = shift;
        my $replacing_pattern = shift;
        my $file_name = shift;

        Log(INFO,"Replacing line pattern $replaced_pattern by $replacing_pattern in file $file_name");

        eval{
                if (-f $file_name) {
                        my $tempfile = "/tmp/tempfile.tmp";
                        open (FILE, $file_name) or die("Could not open the file $file_name");
                        open (TEMPFILE, ">$tempfile") or die("Could not open the temporary file $tempfile");
                        while (my $line=<FILE>) {
                                if(grep(/$replaced_pattern/, $line) and !grep(/$replacing_pattern/, $line)) {
                                        $line =~ s/$replaced_pattern/$replacing_pattern/;
                                }
                                print TEMPFILE  $line;
                        }
                        close(FILE);
                        close(TEMPFILE);

                        unlink($file_name) or die("Could not remove the file $file_name $!");
                        move($tempfile, $file_name) or die("Could not update the file $file_name $!");
                }
                else {
                        die("Could not find the $file_name");
                }
        };

	if ($@) { Log(ERR, $@) }

}

sub update_xad_files {
        my $file_name = shift;

	Log(INFO,"Updating the file $file_name");

        eval{
                if (-f $file_name) {
                        my $tempfile = "/tmp/tempfile.tmp";
                        open (FILE, $file_name) or die("Could not open the file $file_name");
                        open (TEMPFILE, ">$tempfile") or die("Could not open the temporary file $tempfile");
                        while (my $line=<FILE>) {
                                if(grep(/checkproc \$CHECKPROC_ARGS/, $line) and !grep(/checkproc \$CHECKPROC_ARGS \$(SBIN|BIN|LIBEXEC)DIR*/, $line)){
                                        $line =~ s/\-p .* //;
                                }
                                print TEMPFILE  $line;
                        }
                        close(FILE);
                        close(TEMPFILE);

                        unlink($file_name) or die("Could not remove the file $file_name $!");
                        move($tempfile, $file_name) or die("Could not update the file $file_name $!");
			chmod(0755, $file_name) or die("Could not change the  permission of the file $file_name $!");
                }
                else {
                        die("Could not find the $file_name");
                }
        };

	if ($@) { Log(ERR, $@) }
}

#change the smb.conf file
sub migrate_smb_conf_file() {
        my $smb_file = "/etc/samba/smb.conf";
        my $section = "";
        my $val = "";

        my $cfg = Config::IniFiles->new (-file=> $smb_file, -nocase=>1);
        if (!defined $cfg) {
                Log(WARN, "Could not open the file $smb_file");
                return -1;
        }

        Log(INFO, "Updating $smb_file file with latest entries");
        Log(INFO, "Upgrading section - global");

        #removing use kerberos keytab
        $val = $cfg->val('global', 'use kerberos keytab');
        if (defined $val) {
                $cfg->delval ('global', 'use kerberos keytab');
        }

        #idmap config addition
        # Remove 'idmap config <NETBIOSNAME> : backend'
        my $var_value = "idmap config ".uc(registry::getReg("Netbios Name"))." : backend";
        $val = $cfg->val('global', $var_value);
        if (defined $val) {
                $cfg->delval ('global', $var_value);
        }

        #idmap config <NETBIOS NAME> : backend -> idmap config * : backend
        $var_value = "idmap config : backend";
        $val = $cfg->val('global', $var_value);
        if (!defined $val) {
              $cfg->newval ('global', $var_value, 'ad');
        }else {
               $cfg->setval ('global', $var_value, 'ad');
        }

        #kerberos method
        $val = $cfg->val('global', 'kerberos method');
        if (!defined $val) {
                $cfg->newval ('global', 'kerberos method', 'system keytab');
        }else {
                $cfg->setval ('global', 'kerberos method', 'system keytab');
        }

        #np : proxy
        $val = $cfg->val('global', 'np : proxy');
        if (!defined $val) {
                $cfg->newval ('global', 'np : proxy', 'samr lsarpc netlogon');
        }else {
                $cfg->setval ('global', 'np : proxy', 'samr lsarpc netlogon');
        }

        #external_rpc_pipe : socket_dir
        $val = $cfg->val('global', 'external_rpc_pipe : socket_dir');
        if (!defined $val) {
                $cfg->newval ('global', 'external_rpc_pipe : socket_dir', '/var/opt/novell/xad/rpc/PIPE');
        }else {
                $cfg->setval ('global', 'external_rpc_pipe : socket_dir', '/var/opt/novell/xad/rpc/PIPE');
        }

        $cfg->RewriteConfig();
        `sed -i 's/^[a-zA-Z]/\t&/g' $smb_file`;
        `sed -i 's/=/ = /g' $smb_file`;

        Log(INFO, "Updated $smb_file file with latest entries");
        return 0;
}

sub dsfw_installation() {
	my $zypper_cmd = "/usr/bin/zypper -r -n --no-gpg-checks";
	EXEC_COMMAND(cmd => "$zypper_cmd in -y -l -t pattern novell-domain-services");
}

sub remove_samba_tdbs() {
	my $samba_tdb_file_path="/var/lib/samba";

	eval{
		foreach my $tdbfile (`find $samba_tdb_file_path -iname "*.tdb*"`){
			chomp($tdbfile);
	 		unlink("$tdbfile") or warn("Could not remove $tdbfile ($!)");
		}
	};

	if ($@) { Log(ERR, $@) }
}

sub casa_account_transfer{
	my $source_ip=shift;
	my ($cmd, $status);

	eval{
		$ENV{'LD_PRELOAD'}="/usr/$lib/libgssapi_krb5.so.2";
		unlink("/tmp/adm.txt");
		EXEC_COMMAND(cmd => "${SSH_CMD} $source_ip /opt/novell/named/bin/dns_retrieve_proxy_cred Username /tmp/adm.txt");
		EXEC_COMMAND(cmd => "${SCP_CMD} -rp $source_ip:/tmp/adm.txt /tmp/adm.txt");
		unlink("/tmp/pwd.txt");
		EXEC_COMMAND(cmd => "${SSH_CMD} $source_ip /opt/novell/named/bin/dns_retrieve_proxy_cred Password /tmp/pwd.txt");
		EXEC_COMMAND(cmd => "${SCP_CMD} -rp $source_ip:/tmp/pwd.txt /tmp/pwd.txt");
		delete $ENV{'LD_PRELOAD'};
	};

	if ($@) { Log(ERR, $@) }
}

sub casa_account_creation($){
	my $eDirVersion = shift;
	my ($cmd, $status);
	my ($admContext, $admPwd);

	eval{
		$admContext = `cat /tmp/adm.txt`;
		unlink("/tmp/adm.txt");

		$admPwd = `cat /tmp/pwd.txt`;
		unlink("/tmp/pwd.txt");
		EXEC_COMMAND(cmd => "/usr/sbin/oescredstore -s -n dns-ldap -u $admContext -p $admPwd");
	};

	if ($@) { Log(ERR, $@) }
}

sub prepare_named($) {
	my $eDirVersion = shift;
	my @casa_output = ();
	my $casa_count = 0;

	@casa_output = `/usr/sbin/oescredstore -l 2>&1`;
	if (grep(/Name: common-proxy/, @casa_output)) {
		Log(INFO, "common-proxy found...");
		$casa_count += 1;
	}

	if (grep(/Name: dns-ldap/, @casa_output)) {
		Log(INFO, "dns-ldap found...");
		$casa_count += 1;
	}

	if(registry::getReg("DNS Master") eq "TRUE"){
		#CASA credential creation
		if($eDirVersion eq "edir2_sp2") {
			migration::createCASACreds();
		} else {
			if ($casa_count < 2) {
				Log(INFO, "Creating CASA accounts common-proxy-casa/common-proxy and dns-ldap ");
				casa_account_creation($eDirVersion);
			} else {
				Log(INFO, "CASA accounts common-proxy-casa/common-proxy and dns-ldap exists");
			}
		}

		#Restart the novell-named
		util::novell_named_restart();

		`/usr/bin/systemctl status novell-named.service`;
		if($?){
			Log(WARN, "Failed to start novell-named");
		}
	}
}

sub migrate2_oes11 {
	my $eDirVersion = shift;
	my $target = `hostname -i`;
        Log(INFO,"Starting DSfW server migration to OES11");

	registry::clearReg();
	registry::reg_init();

	&prepare_named($eDirVersion);

	# Check if source server is at SP2 level
	if($eDirVersion eq "edir2_sp2") {
		migration::migratesp2();
	}

	#create a link to krb5.conf
	my $krb5_path = registry::getReg("LIVE_ETCDIR");
	if(-f $krb5_path."/krb5.conf"){
		eval{
			symlink($krb5_path."/krb5.conf", "/etc/krb5.conf");
		};
	        if($@) {
        	        Log(ERR,$@); #Printing ERR message actually dies with proper error message and a backtrace
        	}
	}

	#update the xad file by removing -p option files
	my @xad_files = ("xad-kpasswdd", "xad-krb5kdc", "rpcd");
	foreach my $xadfile (@xad_files){
		update_xad_files("/etc/init.d/$xadfile");
	}

	#remove rpcdep.dat
	my $rpcdep_path = registry::getReg("LIVE_LOCALSTATEDIR");
	if(-f $rpcdep_path."/run/rpcdep.dat"){
		unlink($rpcdep_path."/run/rpcdep.dat") or warn("Could not remove the file $rpcdep_path"."/run/rpcdep.dat ($!)");
	}

	#create the "np" folder
        my $pipe_dir = registry::getReg("LIVE_LOCALSTATEDIR")."/rpc/PIPE/";
        if (-d $pipe_dir and not -d $pipe_dir."/np")  {
		eval{
                	mkpath($pipe_dir."/np");
		};
	        if($@) {
        	        Log(ERR,$@); #Printing ERR message actually dies with proper error message and a backtrace
        	}
        }

        #/etc/SuSEConfig/profile changes
        util::process_suseconfig_profile();

        my $rc = 0;
        $rc = migrate_smb_conf_file();
        if ($rc == -1) {
                Log(WARN, "Failed to migrate smb.conf file: $rc");
        }

	#nsswitch.conf modification
	util::update_nsswitch();

	#delete samba .tdb files
	remove_samba_tdbs();

        Log(INFO, "Successfully migrated the DSfW server to OES11");
}

1;
