#!/usr/bin/perl -I. -I.. -I/opt/novell/xad/lib64/perl -I/opt/novell/xad/lib/perl
################################################################################
# DSfW migrate script for migrating from oes11fp0 to oes11sp1
#
# See the file LICENSE for redistribution information.
#
# Copyright (c) 2001-2012
#	Novell, Inc.  All rights reserved.
################################################################################
package migration;
$ENV{'PATH'} = "/opt/novell/xad/share/dcinit:$ENV{'PATH'}";

use strict;
use warnings;
use File::Copy;
use File::Temp;
use File::Path;
use Config::IniFiles;

use XAD::registry;
use Logger qw(:DEFAULT);
use NDS::stage;
use migration::migrateDSfW2oes11;

my $debug =0;
my $lib = registry::getReg("_LIB");
my $LDAP_ERR_DUP = 20;
my $LDAP_ERR_NO_SUCH_ATTR = 16;
my $localtime=localtime;
my ($day,$month,$date,$time,$year)=split(/ /,$localtime);
my $stamp=$year.$month.$date."-".$time;
my $marker_file = "/var/opt/novell/xad/ds/UpdateDC.run.1";
my $msdfs_dir = "/var/opt/novell/xad/msdfs";
my $xad_ini = "/etc/opt/novell/xad/xad.ini";


# This will delete the ACLs for group policy creator owners group from domain root
# 7#subtree#CN=Group Policy Creator Owners,7#subtree#CN=Group Policy Creator Owners,<user-context>#gplink
# 7#subtree#CN=Group Policy Creator Owners,7#subtree#CN=Group Policy Creator Owners,<user-context>#gpOptions
sub migrate_gpo_rights() {

	Log("Migrating GPO rights related to Group Policy Creator Admin");
	my $error = 0;
	my $templatedir = registry::getReg("TEMPLATEDIR");
	my $LIVE_DSSTATEDIR = registry::getReg("LIVE_DSSTATEDIR");
	my @output = ();
	my $ldapmodify = "LDAPCONF=/etc/opt/novell/xad/openldap/ldap.conf /usr/bin/ldapmodify -Y EXTERNAL -c -Q";

	`/opt/novell/xad/share/dcinit/genLdif.pl $templatedir/domain-gporights.ini --rootnc > $LIVE_DSSTATEDIR/domain/domain-gporights.ldif`;
	if($? != 0){
		Log("Generation of domain-gporights.ldif failed!!!");
	}

	Log("Executing .. $ldapmodify -f $LIVE_DSSTATEDIR/domain/domain-gporights.ldif 2>&1");
	@output = `$ldapmodify -f $LIVE_DSSTATEDIR/domain/domain-gporights.ldif 2>&1`;
	$error = $? >> 8;
	if ($error && $error != $LDAP_ERR_NO_SUCH_ATTR){
		chomp(@output);
		Log("Failed to migrate GPO rights on current server: [@output]");
		return -1;
	} elsif ($error == $LDAP_ERR_NO_SUCH_ATTR){
		Log ("Skipping the GPO rights related ACL update");
	} else {
		Log ("Updated the GPO rights related to Group Policy Creator Admin");
	}
	
	return 0;
}

# Subroutine to update the ACL on CN=Computers container with update rights on its subtree.
sub update_domain_acl ()
{
	my $mapped_domain_nc = registry::getReg("Mapped Domain NC");
	my $temp_file = "/var/opt/novell/xad/ds/domain/domain_nc_acl.ldif";
	my $ldapmodify = "LDAPCONF=/etc/opt/novell/xad/openldap/ldap.conf /usr/bin/ldapmodify -Y EXTERNAL -c -Q";

	if (not (-e $temp_file)) {
		open LDIFFILE, ">$temp_file" or die ("Could not open the LDIF file\n");
		print LDIFFILE "dn: $mapped_domain_nc\n";
		print LDIFFILE "changetype: modify\n";
		print LDIFFILE "add: ACL\n";
		print LDIFFILE "ACL: 7#subtree#[This]#[Entry Rights]\n";
		print LDIFFILE "ACL: 3#subtree#[This]#[All Attributes Rights]\n";
		close LDIFFILE;
	}

	Log ("Executing $ldapmodify -f $temp_file 2>&1") ;
	my @output = `$ldapmodify -f $temp_file 2>&1`;
	my $err = $? >> 8;
	if (($err != 0) and ($err != $LDAP_ERR_DUP))
	{
		chomp (@output);
		Log ("Could not update the ACLs on the domain container: [@output] : [$err]");
	}
	else
	{
		Log("Successfully updated the subtree ACLs on the domain container");
	}
}

sub migrate_domain() {

	Log("Migrating Domain internals to OES11SP1");
	my $rc = 0;

	$rc = migrate_gpo_rights();
	if ($rc == -1) {
		Log("Failed to migrate gpo rights: $rc");
	}

	# Updating the domain container and subtree with ACLs to allow computer container to 
	# search, create,delete inside its own container. This is needed for remote licensing, vmware view etc
	update_domain_acl();

			
	return 0;
}

sub migrate_init_script () {

	Log("   Creating xadsd init script: stamp - $year-$month-$date-$time");

	my $template_dir = registry::getReg("TEMPLATEDIR");
	my $xadsd_file_tmp = mktemp("/tmp/xadsd.XXXXXXXX");
	my $rpcd_file_tmp = mktemp("/tmp/rpcd.XXXXXXXX");
	my $xadkdc_file_tmp = mktemp("/tmp/xadkdc.XXXXXXXX");

	`/etc/init.d/xadsd stop`; # To remove the NETLOGON (caps) pipe
	# This is having recent changes need to work with samba -3.6
	copy("/etc/init.d/xadsd", "/etc/init.d/xadsd.$stamp") or
	Log(ERR, "Failed to backup xadsd init script: $!");

	copy("/etc/init.d/rpcd", "/etc/init.d/rpcd.$stamp") or
	Log(ERR, "Failed to backup rpcd init script: $!");

	# Creating directory structure /var/opt/novell/xad/rpc in ntp chroot jail
	mkpath("/var/lib/chrony".registry::getReg("LIVE_LOCALSTATEDIR")."/rpc" );

	stage::reg_substitute("$template_dir/xadsd.init", "$xadsd_file_tmp");
	copy("$xadsd_file_tmp", "/etc/init.d/xadsd") or
	Log(ERR, "Failed to copy the xadsd init script: $!");

	unlink($xadsd_file_tmp);
	chmod 0755, "/etc/init.d/xadsd";

	stage::reg_substitute("$template_dir/rpcd.init", "$rpcd_file_tmp");
        copy ("$rpcd_file_tmp", "/etc/init.d/rpcd") or Log(ERR, "Failed to copy the rpcd init script: $!");
        unlink($rpcd_file_tmp);
        chmod 0755, "/etc/init.d/rpcd";

	# The xad-krb5kdc dependency on xadsd running status is removed in latest init script
	copy ("/etc/init.d/xad-krb5kdc", "/etc/init.d/xad-krb5kdc.$stamp") or Log(ERR, "Failed to backup xad-krb5kdc init script: $!");
	stage::reg_substitute("$template_dir/xad-krb5kdc.init", "$xadkdc_file_tmp");
	copy ("$xadkdc_file_tmp", "/etc/init.d/xad-krb5kdc") or Log(ERR, "Failed to copy the xad-krb5kdc init script: $!");
	unlink($xadkdc_file_tmp);
	chmod 0755, "/etc/init.d/xad-krb5kdc";

}

sub migrate_smb_conf () {
	my $smb_file = "/etc/samba/smb.conf";
	copy($smb_file, $smb_file.$stamp) or Log (ERR, "Failed to backup the samba conf file: $!");
	 
	my $cfg = Config::IniFiles->new (-file=> $smb_file, -nocase=>1);
	if (!defined $cfg) {
		Log ("The open of $smb_file failed : @Config::IniFiles::errors");
		return -1;
	}
	Log ("Updating $smb_file file with latest entries");

	Log ("Upgrading section - [global]");
	# Remove 'idmap backend'
	my $val = $cfg->val('global', 'idmap backend');
	if (defined $val) {
		$cfg->delval ('global', 'idmap backend');
	}

	# idmap config * : range = 1 - 4294967295
	my $var_value = "idmap config : range";
	$val = $cfg->val('global', $var_value);
	if (!defined $val) {
		$cfg->newval ('global', $var_value, '1 - 4294967295');
	}

	#  send spnego principal = Yes
	$val = $cfg->val('send spnego principal');
	if (!defined $val) {
		$cfg->newval ('global', 'send spnego principal', 'Yes');
	}
	#  client use spnego principal = Yes
	$val = $cfg->val('client use spnego principal');
	if (!defined $val) {
		$cfg->newval ('global', 'client use spnego principal', 'Yes');
	}
	#  cifs instanceless spn = Yes
	$val = $cfg->val('cifs instanceless spn');
	if (!defined $val) {
		$cfg->newval ('global', 'cifs instanceless spn', 'Yes');
	}

	#  dce_funnel_winbind : socket = $REGISTRY=LocalStateDir/rpc/xadsd
	$val = $cfg->val('dce_funnel_winbind : socket');
	if (! defined $val) {
		my $localstatedir = registry::getReg("LIVE_LOCALSTATEDIR");
		$cfg->newval('global', 'dce_funnel_winbind : socket', $localstatedir."/rpc/xadsd");
	}

	$cfg->RewriteConfig();
	`sed -i 's/^[a-zA-Z]/\t&/g' $smb_file`;
	`sed -i 's/=/ = /g' $smb_file`;

	Log ("Updated $smb_file file with latest entries");
	return 0;
}

sub dumpregistry($) {
	my $filename = shift;
	open CONFIG,">$filename" or die ("cannot open file");
	print CONFIG "\n\n\n --------- now writing DEFAULTMAP \n\n";
	my %hash1 = %registry::DEFAULTMAP;
	foreach my $key (%hash1) {
		print CONFIG "$key -> ".$registry::DEFAULTMAP{'$key'}." \n";
	}
	print CONFIG "\n\n\n --------- now writing registry \n\n";
	foreach my $key (%registry::REGISTRY) {
		print CONFIG "$key ==>> ".$registry::REGISTRY{'$key'}." \n";

	}

	close (CONFIG);
}

#
# Correct the DFS link to point one level lesser b'cos of fix Bug#703655
# old link : msdfs:adcserver\sysvol-msdfs\domain.com,frdserver\sysvol\domain.com
# new link: msdfs:adcserver\sysvol-msdfs,frdserver\sysvol -- Removed suffixed domain name
#
sub updateDFSLink() {

        my $root_domain = `grep "Root Domain" $xad_ini |  awk '{print \$NF}'`;
        chomp($root_domain);

        if ($root_domain eq "") {
                # Fallback to dnsdomainname and expect it to be set right
                $root_domain = `/bin/dnsdomainname`;
                chomp($root_domain);
                if ($root_domain eq "") {
                        Log("Unable to deduce Root Domain name. Exiting..");
                        return 0;
                }
        }

        my $msdfs_link = "$msdfs_dir/$root_domain";
        if (-l "$msdfs_link") {

                my $group = `getfacl -n $msdfs_dir  2>/dev/null | grep " group:" | awk '{print \$NF}'`;
                my $owner = `getfacl -n $msdfs_dir  2>/dev/null | grep " owner:" | awk '{print \$NF}'`;
                chomp($group);
                chomp($owner);

                if ( $group eq "" || $owner eq "" ) {
                        Log("Unable to deduce owner or group information. Exiting..");
                        return 0;
                }

                my $current_link = `ls -l  "$msdfs_link" | awk  -F" " '{print \$NF}'`;
                my $new_link = `ls -l  "$msdfs_link" | awk '{print \$NF}' | sed  's/\\\\$root_domain,/,/i;s/\\\\$root_domain\$//i'`;
                chomp($current_link);
                chomp($new_link);

                if ($new_link eq "") {
                        Log("Unable to deduce DFS link value. Exiting..");
                        return 0;
                }
                if ($new_link eq $current_link) {
                        Log("DFS Link already updated..");
                        return 0;
                }
                Log("Changing msdfs link from $root_domain -> $current_link");
                Log("   to  $root_domain -> $new_link");
                `ln -sf "$new_link" "$msdfs_link"`;
                `chown -R "$owner"\:"$group" $msdfs_dir`;

                my $new_owner = `ls -n "$msdfs_link" | awk '{print \$3}'`;
                chomp($new_owner);

                if ($new_owner eq $owner) {
                        Log("DFS link Updated Succesfully");

                        # Make sure this marker is set at end of execution
                        `touch $marker_file`;

                        return 0;
                } else {
                        Log("DFS link Update Failed.. Follow TID");
                        return 0;
                }

        } else {
                # Do nothing
                return 0;
        }
}


sub migrate2_oes11sp1 ($) {
	my $eDirVersion = shift;
	my $rc = 0;

	# Script to migrate the target server to OES11FP0 level
	Log ("Now starting DSfW server migrate to OES11SP1");

	registry::clearReg();
	registry::reg_init();

	Log ("Reloaded the registry");

	migration::prepare_named($eDirVersion);

	$rc = migrate_domain();
	if ($rc == -1) {
		Log("Failed to migrate domain: $rc");
	}
	
	# Update xadsd and xad-krb5kdc init script
	migrate_init_script();	
	
	# Update samba smb.conf with latest changes wrt to 3.6
	migrate_smb_conf();

	#Updating the DFS link
	updateDFSLink();
	
	Log ("Successfully migrated the DSfW server to OES11SP1");
}
1;
