#!/usr/bin/perl -I. -I.. -I/opt/novell/xad/lib64/perl -I/opt/novell/xad/lib/perl
################################################################################
# DSfW Upgrade script for migrating to OES2018SP1
#
# See the file LICENSE for redistribution information.
#
# Copyright (c) 2018-2019
#	Novell, Inc.  All rights reserved.
################################################################################
package migration;
$ENV{'PATH'} = "/opt/novell/xad/share/dcinit:$ENV{'PATH'}";

use strict;
use warnings;
use XAD::registry;
use Logger qw(:DEFAULT);
use File::Copy;
use XAD::registry;
use util;
use util_update;
use Config::IniFiles;
use NDS::stage;

sub migrate2_oes2018sp1 ($) {
	my $eDirVersion = shift;

	# Script to migrate the target server to OES2018SP1 level
	Log ("Now starting DSfW server migrate to OES2018SP1");

	registry::clearReg();
	registry::reg_init();

	Log ("Reloaded the registry");

	migration::prepare_named($eDirVersion);

	eval {
		#update AES256 capabilities on the server
		util_update::update_domain_to_aes();

		# Update samba smb.conf with latest changes wrt to 4.6
		util_update::upgrade_samba_smb_conf();

		# Updating the usr.sbin.nmbd apparmor profile
		util::update_apparmor_nmbd();

		# Updating the usr.sbin.smbd apparmor profile
		util::update_apparmor_smbd();

		# Updating the usr.sbin.winbindd apparmor profile
		util::update_apparmor_winbindd();

		#Restarting apparmor to get the apparmor profiles changes of nmbd, smbd and winbindd
		my $service_start_status;

		`/usr/bin/systemctl restart apparmor.service`;
		$service_start_status = $?;
		Log(WARN, "Service apparmor restarted with code $service_start_status");

	};

	if($@){
		Log(WARN, $@);
	}
}
1;
