#!/usr/bin/perl -I. -I.. -I/opt/novell/xad/lib64/perl -I/opt/novell/xad/lib/perl
################################################################################
# DSfW Upgrade script for migrating to OES2018SP2
#
# See the file LICENSE for redistribution information.
#
# Copyright (c) 2018-2020
#	Micro Focus or one of its affiliates.
################################################################################
package migration;
$ENV{'PATH'} = "/opt/novell/xad/share/dcinit:$ENV{'PATH'}";

use strict;
use warnings;
use XAD::registry;
use Logger qw(:DEFAULT);
use File::Copy;
use XAD::registry;
use util;
use util_update;
use Config::IniFiles;
use NDS::stage;

sub migrate2_oes2018sp2 ($) {
	my $eDirVersion = shift;

	# Script to migrate the target server to OES2018SP2 level
	Log ("Now starting DSfW server migrate to OES2018SP2");

	registry::clearReg();
	registry::reg_init();

	Log ("Reloaded the registry");

	migration::prepare_named($eDirVersion);

	eval {

		# This code is called for add string in /etc/krb5.conf file.
                util::add_cache_string_krb5();

		# This subroutine for creating symlink for etc/opt/novell/xad/krb5.conf file.
                # This code is called for migrate2oes2018sp2.
                util::symlink_krb5_conf_file();

		# This subroutine for checking and changing NTPD_RUN_CHROOTED variable in /etc/sysconfig/chronyd file.
		util::ntpd_run_chrooted_var_change();

		# This code is called for update site name in /etc/opt/novell/xad/xad.ini file.
		util::update_site_name_in_xad_ini();

		# Updating the ACLs for NSSAD integration
		util_update::apply_2018sp2_acls();

		# Updating the server to AD 2016 level
		util_update::update_server_to2016("MIGRATE");
		#Enable fips
		util::enable_fips();

		# Update samba smb.conf with latest changes.
                util_update::upgrade_samba_smb_conf();

		# This code is called for add 'Zone Container' varaible in /etc/opt/novell/xad/xad.ini file.
                util::add_zone_container_in_xad_ini();
	};

	if($@){
		Log(WARN, $@);
	}
}
1;
