#!/usr/bin/perl -I. -I.. -I/opt/novell/xad/lib64/perl -I/opt/novell/xad/lib/perl
################################################################################
# DSfW migrate module 
#
# See the file LICENSE for redistribution information.
#
# Copyright (c) 2001-2020
#	Micro Focus or one of its affiliates
################################################################################
package migration;
$ENV{'PATH'} = "/opt/novell/xad/share/dcinit:$ENV{'PATH'}";


use strict;
use warnings;
use v5.10;# at least for Perl 5.10
no warnings 'experimental::smartmatch'; #Avoid known warnings with given-when
use XAD::registry;
use migration::migratesp2DSfW2oes11;
use migration::migrateDSfW2oes11;
use migration::migrateDSfW2oes11sp1;
use migration::migrateDSfW2oes11sp2;
use migration::migrateDSfW2oes2015;
use migration::migrateDSfW2oes2015sp1;
use migration::migrateDSfW2oes2018;
use migration::migrateDSfW2oes2018sp1;
use migration::migrateDSfW2oes2018sp2;
use migration::migrateDSfW2oes2018sp3;
use migration::migrateDSfW2oes2023;
use migration::migrateDSfW2oes2023sp1;
use Logger qw(:DEFAULT);
use File::Path;
use File::Copy;
use XAD::nds;
use File::Temp;
use Term::ReadLine;
use Term::ReadKey;
use util;
use util_update;

my @eDirVersion = (0);
my @target_eDirVersion = (0);
my $migration_status_file = "/var/opt/novell/migration/migration.status";
my $nds_modules_dir = "/opt/novell/xad/lib64/nds-modules";
my $pre_mig_file = "";
my $post_mig_file = "";

my $localtime=localtime;
my ($day,$month,$date,$time,$year)=split(/ /,$localtime);
my $stamp=$year.$month.$date."-".$time;

sub isDSfWInstalled{
	my @mesg=();

	$ENV{'LD_PRELOAD'}="/usr/$lib/libgssapi_krb5.so.2";
        @mesg = EXEC_COMMAND(cmd => "zypper search -t pattern -i novell-domain-services", redir => 1);
	delete $ENV{'LD_PRELOAD'};

        my @result = grep (/^i\s|\snovell-domain-services/, $mesg[2]);
        if (@result) {
		Log(INFO,"DSfW is installed on this machine.");
        }
	else
	{
		Log(INFO,"DSfW is not installed on this machine.");
		Log(INFO,"Please install DSfW and then proceed for migration.");
		Log(ERR, "Please chek the log file ($migrate_log_file)");
	}
}

# Generate and Copy the SSh key to the Source Server
sub generateAndCopySSHKey($){
	my $source_ip=shift;	
	$ENV{'LD_PRELOAD'}="/usr/$lib/libgssapi_krb5.so.2";
	system("ssh-keygen -t rsa");
	displayCommandStatus($?, $!);
	system("ssh-copy-id -i /root/.ssh/id_rsa.pub root\@$source_ip");
	displayCommandStatus($?, $!);
	Log(INFO,"SSH public key is copied to $source_ip successfully.");
	delete $ENV{'LD_PRELOAD'};
}

# Check if SSH/SCP is setup in batch mode
sub checkSSHAccess
{
        my $source_ip=shift;
        my @mesg=();
	my $error_ssh="Host key verification failed";

	$ENV{'LD_PRELOAD'}="/usr/$lib/libgssapi_krb5.so.2";
        Log(INFO,"Checking 'ssh' access to the source server '$source_ip'...");
        @mesg = EXEC_COMMAND(cmd => "${SSH_CMD} root\@$source_ip echo \"Testing SSH/SCP access...\"", redir => 1);
	delete $ENV{'LD_PRELOAD'};

        if ( grep(/$error_ssh/, $mesg[2])){
                Log(INFO,"User would be prompted for the 'root' password of the source server '$source_ip'");
                Log(INFO,"during the migration when 'ssh/scp' service is necessary since it is not set in");
                Log(INFO,"'BatchMode' for passwordless access. Which is not allowed in the current migration");
                Log(INFO,"process.");
                Log(INFO,"*****************************************************************************");
                Log(INFO,"Please do the following before executing this script");
                Log(INFO,"Execute the command on target machine \"ssh-keygen -t rsa\"");
                Log(INFO,"Copy it to $source_ip as \"scp /root/.ssh/id_rsa.pub $source_ip:/tmp\"");
                Log(INFO,"At $source_ip check for the directory /root/.ssh if not present create it");
                Log(INFO,"Append the key details from /tmp/id_rsa.pub to /root/.ssh/authorized_keys as");
                Log(INFO,"\"cat /tmp/id_rsa.pub >> /root/.ssh/authorized_keys\"");
                Log(INFO,"*****************************************************************************");
                Log(INFO,"Rerun the script at the target.");
		Log(ERR, "Please chek the log file ($migrate_log_file)");
        }
	else{
		Log(INFO,"'ssh' access to the source server '$source_ip'... Success");
	}
}

sub transfer_from_source{
	my $source_ip = shift || die("Invalid source");
	my ($cmd, $status);
        my %hash_source_target=(
                registry::getReg("LIVE_LOCALSTATEDIR")  =>      "var_opt_novell_xad",
                registry::getReg("LIVE_ETCDIR")         =>      "etc_opt_novell_xad",
                "/etc/samba"                            =>      "etc_samba",
                "/var/lib/samba"                        =>      "var_lib_samba",
                "/etc/init.d"                           =>      "etc_init_d",
                #$LOCAL_SBIN_DIR                         =>      "etc_init_d_rc",
                registry::getReg("NdsConfDir")          =>      "nds_conf",
		"/var/spool/cron/tabs"			=>	"root_cron",
		"/etc/sysconfig"			=>	"etc_sysconfig",
		"/etc/sysconfig/novell"			=>	"etc_sysconfig_novell",
		"/etc"					=>	"etc"
        );

	$ENV{'LD_PRELOAD'}="/usr/$lib/libgssapi_krb5.so.2";
	
	Log(INFO,"Transfering files from source $source_ip");

	eval{
#create the backup directory location
		if(-d $target_backup_dir){
			`rm -rf $target_backup_dir`;
		}
		mkpath($target_backup_dir) or die("Warning - couldn't create directory");

		while (my ($source_local_dir, $target_local_dir) = each %hash_source_target){
			Log(INFO,"Transfering data at location $source_local_dir .....");
			given("$target_local_dir"){
				when("etc_init_d") {
					mkpath("$target_backup_dir$target_local_dir")
					or die("Warning - couldn't create directory $target_backup_dir$target_local_dir");
					EXEC_COMMAND(cmd => "${SCP_CMD} -rp $source_ip:/etc/init.d/xad* $target_backup_dir$target_local_dir/");
					EXEC_COMMAND(cmd => "${SCP_CMD} -rp $source_ip:/etc/init.d/rpcd $target_backup_dir$target_local_dir/");
					EXEC_COMMAND(cmd => "${SCP_CMD} -rp $source_ip:/etc/init.d/winbind* $target_backup_dir$target_local_dir/");
					EXEC_COMMAND(cmd => "${SCP_CMD} -rp $source_ip:/etc/init.d/apache2* $target_backup_dir$target_local_dir/");
				}
				when("etc_init_d_rc") {
					mkpath("$target_backup_dir$target_local_dir")
					or die("Warning - couldn't create directory $target_backup_dir$target_local_dir");
					EXEC_COMMAND(cmd => "${SSH_CMD} $source_ip ${TARC_CMD} /tmp/${target_local_dir}.tgz $LOCAL_SBIN_DIR/rcxad* $LOCAL_SBIN_DIR/rcrpcd*");
					EXEC_COMMAND(cmd => "${SCP_CMD} $source_ip:/tmp/${target_local_dir}.tgz $target_backup_dir$target_local_dir/");
					EXEC_COMMAND(cmd => "${SSH_CMD} $source_ip ${RM_CMD} /tmp/${target_local_dir}.tgz");
				}
				when("nds_conf") {
					mkpath("$target_backup_dir$target_local_dir")
					or die("Warning - couldn't create directory $target_backup_dir$target_local_dir");
					EXEC_COMMAND(cmd => "${SCP_CMD} -rp $source_ip:$source_local_dir/nds.conf $target_backup_dir$target_local_dir/");
					EXEC_COMMAND(cmd => "${SCP_CMD} -rp $source_ip:$source_local_dir/ndsmodules.conf $target_backup_dir$target_local_dir/");
				}
				when("root_cron") {
					mkpath("$target_backup_dir$target_local_dir")
					or die("Warning - couldn't create directory $target_backup_dir$target_local_dir");
					EXEC_COMMAND(cmd => "${SCP_CMD} -rp $source_ip:$source_local_dir/root $target_backup_dir$target_local_dir/");
				}
				when("etc_sysconfig") {
					mkpath("$target_backup_dir$target_local_dir")
					or die("Warning - couldn't create directory $target_backup_dir$target_local_dir");
					foreach my $sysfiles ("samba", "named", "xadsd", "SuSEfirewall2"){
						EXEC_COMMAND(cmd => "${SCP_CMD} -rp $source_ip:$source_local_dir/$sysfiles $target_backup_dir$target_local_dir/");
					}
				}
				when("etc") {
					mkpath("$target_backup_dir$target_local_dir")
					or die("Warning - couldn't create directory $target_backup_dir$target_local_dir");
					foreach my $etcfiles ("resolv.conf", "hosts", "chrony.conf"){
						EXEC_COMMAND(cmd => "${SCP_CMD} -rp $source_ip:$source_local_dir/$etcfiles $target_backup_dir$target_local_dir/");
					}
				}
				default{
					mkpath("$target_backup_dir$target_local_dir")
					or die("Warning - couldn't create directory $target_backup_dir$target_local_dir");
					EXEC_COMMAND(cmd => "${SSH_CMD} $source_ip ${TARC_CMD} /tmp/${target_local_dir}.tgz ${source_local_dir}/");
					EXEC_COMMAND(cmd => "${SCP_CMD} $source_ip:/tmp/${target_local_dir}.tgz $target_backup_dir$target_local_dir/");
					EXEC_COMMAND(cmd => "${SSH_CMD} $source_ip ${RM_CMD} /tmp/${target_local_dir}.tgz");
					if ($target_local_dir eq "var_opt_novell_xad") {
						#Backup the ACL related operation of source at the target
						EXEC_COMMAND(cmd => "${SSH_CMD} $source_ip '(/usr/bin/getfacl -pnR ${source_local_dir} > /tmp/getacls.file)'");
						EXEC_COMMAND(cmd => "${SCP_CMD} $source_ip:/tmp/getacls.file $target_backup_dir$target_local_dir/");
						EXEC_COMMAND(cmd => "${SSH_CMD} $source_ip ${RM_CMD} /tmp/getacls.file");
					}
				}
			}
		}
	};
	delete $ENV{'LD_PRELOAD'};

	if ($@) { Log(ERR, $@) }
}

sub restore_etc_krb5_conf {
	`mv /etc/krb5.conf{,.before-migrate}`;

	`ln -s /etc/opt/novell/xad/krb5.conf /etc/krb5.conf`;

}

sub restore_to_target{
	my ($cmd, $status);
        my %hash_source_target=(
                registry::getReg("LIVE_LOCALSTATEDIR")  =>      "var_opt_novell_xad",
                registry::getReg("LIVE_ETCDIR")         =>      "etc_opt_novell_xad",
                "/etc/samba"                            =>      "etc_samba",
                "/var/lib/samba"                        =>      "var_lib_samba",
                "/etc/init.d"                           =>      "etc_init_d",
                #$LOCAL_SBIN_DIR                         =>      "etc_init_d_rc",
                registry::getReg("NdsConfDir")          =>      "nds_conf",
		"/var/spool/cron/tabs"			=>	"root_cron",
		"/etc/sysconfig"			=>	"etc_sysconfig",
		"/etc/sysconfig/novell"			=>	"etc_sysconfig_novell",
		"/etc"					=>	"etc"
        );

	$ENV{'LD_PRELOAD'}="/usr/$lib/libgssapi_krb5.so.2";

	my $host_ip = `/bin/hostname -i`;
	chomp($host_ip);
	Log(INFO,"Restoring files to target $host_ip");

	eval{
		#create the backup directory location
		if(! -d $target_backup_dir){
			die("$target_backup_dir is not present");
		}

		while (my ($source_local_dir, $target_local_dir) = each %hash_source_target){
			if(-d "$target_backup_dir$target_local_dir"){
				Log(INFO,"Preparing $source_local_dir .....");
				if(-d $source_local_dir){
					if (-f "$target_backup_dir$target_local_dir/${target_local_dir}.tgz") {
						EXEC_COMMAND(cmd => "${TARX_CMD} $target_backup_dir$target_local_dir/${target_local_dir}.tgz -C /");
					} else {
						EXEC_COMMAND(cmd => "cp -pr $target_backup_dir$target_local_dir/* $source_local_dir/");
					}
				} else {
					if (-f "$target_backup_dir$target_local_dir/${target_local_dir}.tgz") {
						EXEC_COMMAND(cmd => "${TARX_CMD} $target_backup_dir$target_local_dir/${target_local_dir}.tgz -C /");
					} else {
						EXEC_COMMAND(cmd => "cp -pr $target_backup_dir$target_local_dir/ $source_local_dir/");
					}
				}
				if ($target_local_dir eq "var_opt_novell_xad") {
					#Restore the ACL at the target
					EXEC_COMMAND(cmd => "/usr/bin/setfacl --restore=$target_backup_dir$target_local_dir/getacls.file");
				}
			}
		}

		restore_etc_krb5_conf();

		#update the nds.conf file and replace lib with lib64
		my $nds_c_file = registry::getReg("NdsConfDir")."/nds.conf";
		replace_pattern_in_file("/lib", "/$lib", $nds_c_file);
		`gawk -i inplace '!a[\$0]++' $nds_c_file`;

		#create the symbolic links for files
		my @rc_files = ("xad-kpasswdd", "xad-krb5kdc", "rpcd", "xadsd");
		foreach my $rcfile (@rc_files){
			symlink("/etc/init.d/$rcfile", $LOCAL_SBIN_DIR."rc".$rcfile);
		}

		#create symbolic link for krb5.keytab
		symlink(registry::getReg("LIVE_DSSTATEDIR")."/krb5kdc//krb5.keytab", "/etc/krb5.keytab");
	};
	delete $ENV{'LD_PRELOAD'};

	if ($@) { Log(ERR, $@) }

	#restart NDSD
	`/usr/sbin/rcndsd restart`;
}

sub create_xadsd_conf () {
	my $template_dir = registry::getReg("TEMPLATEDIR");
	my $etc_dir = registry::getReg("LIVE_ETCDIR");

	Log(INFO, "Creating xadss.conf");
	#Backing up the old
	copy("$etc_dir/xadss.conf", "$etc_dir/xadss.conf.$stamp") or
		Log(ERR, "Failed to backup xadss.conf: $!");
	unlink ("$etc_dir/xadss.conf");
	stage::reg_substitute("$template_dir/xadss.conf", "$etc_dir/xadss.conf");
}

sub geteDirectoryVersion($){
	my $source_ip = shift;
	my $target_ip = `hostname -i`;
	my %spec_hash = (	'-nothing-'		=> 0,
				'edir2_sp2'		=> 1,
				'edir2_sp3'		=> 2,
				'edir2_oes11'		=> 3,
				'edir_oes11_sp1'	=> 4,
				'edir_oes11_sp2'	=> 5,
				'edir_oes2015'		=> 6,
				'edir_oes11_sp3'	=> 7,
				'edir_oes2015_sp1'	=> 8,
				'edir_oes2018'		=> 9,
				'edir_oes2018_sp1'	=> 10,
				'edir_oes2018_sp2'      => 11,
# Its possible that we do not do any changes in future release that requires new migration script module.
# we need to work out a logic so that prepare_named($edirVersion) need not be added in each release.
				'edir_oes2018_sp3'      => 12,
				'edir'                  => 13
			);
	my $version_found = "-nothing-";
	my $edir_vers_temp_file = mktemp("/var/opt/novell/xad/edv.XXXXXXXX");
	my @mesg=();
	chomp($target_ip);

	$ENV{'LD_PRELOAD'}="/usr/$lib/libgssapi_krb5.so.2";
	if($source_ip){
		if($source_ip eq $target_ip){
			@mesg = EXEC_COMMAND(cmd => "ls /etc/sysconfig/novell | grep -i edir", redir => 1);
		}else{
			@mesg = EXEC_COMMAND(cmd => "${SSH_CMD} $source_ip ls /etc/sysconfig/novell | grep -i edir", redir => 1);
		}
		my @edir_versions=split('\n', $mesg[2]);
		chomp(@edir_versions);

		foreach my $current_version (@edir_versions) {
			if ($spec_hash{$current_version} gt $spec_hash{$version_found}) {
				$version_found = $current_version;
			}
		}
	}
	delete $ENV{'LD_PRELOAD'};

	if ($version_found eq "-nothing-") {
		die("Could not able to fetch the source server OES details");
	}

	`echo "$version_found" >> $edir_vers_temp_file`;

	return $version_found;
}

#Confirmation on input
sub confirmInput {
        my($str, $key) = @_;
        Log("$str");
        ReadMode ('cbreak');
        while (1) {
                $key = lc(ReadKey(0));
                Log("$key") if(($key eq "y") || ($key eq "n" ));
                last unless defined $key;
                last if (($key eq "y") || ($key eq "n" ));
        }
        ReadMode 'restore';
        return($key);
}

#Is Pre Migration done?
sub isPreMigrationDone($){
        my $src_ip = shift;
        my $tgt_ip = `/bin/hostname -i`;
	my $local_state_dir = registry::getReg("LIVE_LOCALSTATEDIR");
        my $formatted_ip;

        chomp($tgt_ip);
        ($formatted_ip=$src_ip)=~s/\./_/g;
        $pre_mig_file = "PRE_S".$formatted_ip."T";
        ($formatted_ip=$tgt_ip)=~s/\./_/g;
        $pre_mig_file =$pre_mig_file.$formatted_ip ;

	#Someone runs the script on a migrated target server
	if($src_ip eq $tgt_ip){
		Log(INFO, "Pre Migration stage already done.");
		return 1;
	}

	#Check for the pre migration status files already present or not
        if( -e "$local_state_dir/$pre_mig_file"){
                Log(INFO, "Pre Migration stage already done.");
                return 1;
        }

	return 0;
}

#Check the Migration through MIGGUI is completed
sub isMIGGUIDone{
	if(open(FH, "<$migration_status_file")){
		my @mig_status=<FH>;
		if(grep(/MIGRATION\s*:\s*SUCCESS/, @mig_status)){
			close(FH);
			return 1;
		}
		close(FH);
	}
	return 0;
}

#Is Post Migration done?
sub isPostMigrationDone($){
        my $src_ip = shift;
        my $tgt_ip = `/bin/hostname -i`;
	my $local_state_dir = registry::getReg("LIVE_LOCALSTATEDIR");
        my $formatted_ip;

        chomp($tgt_ip);
        ($formatted_ip=$src_ip)=~s/\./_/g;
        $post_mig_file = "POST_S".$formatted_ip."T";
        ($formatted_ip=$tgt_ip)=~s/\./_/g;
        $post_mig_file =$post_mig_file.$formatted_ip ;

	#If the script is rerun on a migrated server check for the post
	#migration status file already exists or not
	if($src_ip eq $tgt_ip){
		my @post_status_files = (`find $local_state_dir/ -name \"POST_S$formatted_ip*\"`);
		chomp(@post_status_files);
		if(($#post_status_files+1) >=1){
			Log(INFO, "Post Migration stage already done.");
			return 1;
		}
	}

	#Check for the post migration status files already present or not
        if( -e "$local_state_dir/$post_mig_file"){
                Log(INFO, "Post Migration stage already done.");
                return 1;
        }
	return 0;
}

#set the run level for each daemon
sub setRunLevelForNonXADServices{
	my ($cmd, $status);
	my @xad_services=(	"novell-named",
				"nscd",
				"rpcd",
				"xad-krb5kdc",
				"xad-kpasswdd",
				"xadsd",
				"nmb",
				"winbind",
				"smb",
				"ntp",
				"rsyncd",
				"sshd");

	Log(INFO,"DSfW services run levels.");
	if($target_eDirVersion[0] ne "edir_oes2018" or $target_eDirVersion[0] ne "edir_oes2018_sp1" or $target_eDirVersion[0] ne "edir_oes2018_sp2" or $target_eDirVersion[0] ne "edir_oes2018_sp3" or $target_eDirVersion[0] ne "edir"){
		foreach my $service (@xad_services){
			if($service ne "ndsd"){
				EXEC_COMMAND(cmd => "/sbin/chkconfig --level 35 $service on");
			}
			EXEC_COMMAND(cmd => "/sbin/chkconfig --list $service");
		}
	}else{
		foreach my $service (@xad_services){
			my $serv_res = "";
			my $command_result = 0;
			my $result = `/usr/bin/systemctl is-enabled $service 2>&1`;
			$command_result = ($?>>8);

			chomp($result);
			if($command_result == 0){
				#Corner case, may not happen any time
				if($result ne "enabled"){
					EXEC_COMMAND(cmd => "/usr/bin/systemctl enable $service");
				}
			}else{
				Log(INFO,"/usr/bin/systemctl is-enabled $service failed. Enabling $service.");
				EXEC_COMMAND(cmd => "/usr/bin/systemctl enable $service");
			}
		}
	}
}

sub pre_migration_of_DSfW($) {

	my $source_ip = shift;
	my $target = `/bin/hostname -i`;
	my $user_confirmation;
	my $local_state_dir = registry::getReg("LIVE_LOCALSTATEDIR");

        Log(INFO,"Starting DSfW server migration to OES23.4");

	if(isPostMigrationDone($source_ip)){
		Log(INFO,"DSfW migration is already completed on this server");
		exit(-1);
	}

	#DSfW is installed or not
	#isDSfWInstalled();

	#Generate and copy the SSh key to source
	#generateAndCopySSHKey($source_ip);

	#check for ssh/scp access to the source
	checkSSHAccess($source_ip);

	#Collect the eDir version existing on the source server before shutting down it in miggui phase
	@eDirVersion = geteDirectoryVersion($source_ip);

	if(!isPreMigrationDone($source_ip)){
		#copy the files from source to destination to a separate folder
		transfer_from_source($source_ip);

		#Read the xad.ini file of source server
		Log(INFO, "Executing ... ".
		"${TARX_CMD} ${target_backup_dir}etc_opt_novell_xad/etc_opt_novell_xad.tgz".
		" -C ${target_backup_dir}etc_opt_novell_xad/ etc/opt/novell/xad/xad.ini");

		EXEC_COMMAND(cmd => "${TARX_CMD}".
		" ${target_backup_dir}etc_opt_novell_xad/etc_opt_novell_xad.tgz".
		" -C ${target_backup_dir}etc_opt_novell_xad/ etc/opt/novell/xad/xad.ini");

		my $xad_file_name = "${target_backup_dir}etc_opt_novell_xad".
					"/etc/opt/novell/xad/xad.ini";
		eval{
			open(FH, "<$xad_file_name") or die("Could not open the file $xad_file_name : $!");
		};
		my @xad_arr=<FH>;
		close(FH);
		EXEC_COMMAND(cmd => "/bin/rm -rf ${target_backup_dir}etc_opt_novell_xad/etc");

		#Is the source a DNS Master
		my $is_dns_master = (split('=', (grep(/DNS Master/, @xad_arr))[0]))[1];
		chomp($is_dns_master);
		$is_dns_master=~s/\s//g;
		if($is_dns_master eq "TRUE") {

			if($eDirVersion[0] ne "edir2_sp2") {
				#CASA credential creation
				migration::casa_account_transfer($source_ip);
			}

		}

		move ($nds_modules_dir, "$nds_modules_dir.temp");

		#Pre migration Done Flag
                `touch $local_state_dir/$pre_mig_file`;
                `echo "done" >$local_state_dir/$pre_mig_file`;
        }

	print "Precheck for DSfW migration done successfully\n";
	Log(INFO, "Precheck for DSfW migration done successfully");
}

sub miggui_DSfW($) {

	my $source_ip = shift;

	if(isPostMigrationDone($source_ip)){
		Log(INFO,"DSfW migration is already completed on this server");
		exit(-1);
	}

	#Launch the migration tool to migrate eDirectory and other services
	if(-e $migration_status_file){
		if(isMIGGUIDone()){
			Log("Migration through MIGGUI is already completed");
		}
	}
	else{
		Log("Proceeding for Migration through MIGGUI");
		my $ld_lib_path = $ENV{'LD_LIBRARY_PATH'};
		delete $ENV{'LD_LIBRARY_PATH'};
		system("$ENV{SHELL} /opt/novell/migration/sbin/miggui");
		displayCommandStatus($?, $!);
		$ENV{'LD_LIBRARY_PATH'} = $ld_lib_path;

		#Warning Befoer Post-Migration Stage
		if(!isMIGGUIDone()){
			Log("Cannot proceed with DSfW Post-migration, as all the".
			" tasks in the MIGGUI are not completed successfully.");
			exit(-1);
		}
	}
	print "Associated Service migration for DSfW done successfully\n";
	Log(INFO, "Associated Service migration for DSfW done successfully");
}

sub fetch_edir_version(){
	my @edv_files = `/bin/ls /var/opt/novell/xad/edv.*`;
	my $edv_file = $edv_files[$#edv_files];
	open(FH, $edv_file) or
		die("Could not open file $edv_file");
	@eDirVersion = <FH>;
	chomp(@eDirVersion);
	Log(INFO, "OES version details : $eDirVersion[0]");
	close(FH);
}

sub read_password
{
        my $password;
	my $ndsadminname = shift;

	my $old_fh = select STDOUT;
	$| = 1;
	select $old_fh;

        print STDERR "Enter your password for $ndsadminname: ";
        ReadMode 'noecho';
        $password = ReadLine 0;
        chomp $password;
	print STDERR "\n";
        ReadMode 'normal';

        return $password;

}

sub password_from_terminal {
        my $password_entered;
	my $ndsadminname = shift;

        $password_entered = read_password($ndsadminname);

        return $password_entered;
}

sub getTreeAdmin() {
        my $tree_admin = "";

        #non-name mapped tree administrator
        $tree_admin = registry::getReg("LDAPDOMAINADMINNAME");
        my $type = registry::getReg("INSTALL_TYPE");
        my $is_replica = registry::getReg("IS_REPLICA");

        if ($type eq "frd" and $is_replica eq "FALSE") {
                if (not registry::getReg("MAPPED")) {
                        #name mapped tree administrator
                        $tree_admin = registry::getReg("LDAPDOMAINADMIN");
                }
        }

        #print("From registry - Returning Tree Administrator $tree_admin");
        if ($tree_admin eq "") {
		$tree_admin=`grep CONFIG_LDAP_ADMIN_CONTEXT /etc/sysconfig/novell/oes-ldap | cut -d '"' -f2 | sed 's/,/./g'`;
		chomp($tree_admin);
	}
	return $tree_admin;
}

sub validate_password {
	my $ndsadminname = shift;
	my $admin_password = shift;

	`/opt/novell/eDirectory/bin/ndslogin $ndsadminname -p $admin_password >/dev/null 2>&1`;
        if($?!=0){
             print STDERR "Wrong password\n";
	     return 0;	
        }
	return 1;
}

sub get_and_validate_password {
	my $ldapadminname = getTreeAdmin();
        my $login_success;
        chomp($ldapadminname);
        my $ndsadminname = $ldapadminname;
        $ndsadminname =~ s/,/\./g;
	

	if(not defined($ENV{ADM_PASSWD}) or $ENV{ADM_PASSWD} eq ""){
	        while(not $login_success) {
		        $ENV{ADM_PASSWD}=password_from_terminal($ndsadminname);
			$login_success = validate_password($ndsadminname,$ENV{ADM_PASSWD});
		}
        }
        else {
                Log("Password taken from environment variable for $ndsadminname");
		validate_password($ndsadminname,$ENV{ADM_PASSWD});
        }

}

sub w32tm_related_changes() {
	Log("   w32tm_client related changes");
	mkpath( "/var/lib/chrony/var/opt/novell/xad/rpc" );
}


sub post_migration_of_DSfW($) {
	my $source_ip = shift;
	my $stagedir = registry::getReg("STAGEDIR");
	my $target = `hostname -i`;
	my $local_state_dir = registry::getReg("LIVE_LOCALSTATEDIR");
	chomp($target);

	if(isPostMigrationDone($source_ip)){
		Log(INFO,"DSfW migration is already completed on this server");
		exit(-1);
	}

	&fetch_edir_version();

	@target_eDirVersion = geteDirectoryVersion($target);

	Log(INFO, "Target server details : $target_eDirVersion[0]");

	Log("Proceeding to Post Migration of DSfW server");

	#restore the copied filed to the respective locations
	restore_to_target();

	#Write new xadsd location in a cross partition NTP setup
	util::write_newxadsd_location();

	#update the xadss.conf
	&create_xadsd_conf();

	# Move the nds-modules / plugin directory and restart ndsd
	move ("$nds_modules_dir.temp", "$nds_modules_dir");

#	eval {
		#May be required
		#`system daemon-reload`;

		#fips disable
		#Stop the ndsd service
		util::stop_ndsd();

#		`pgrep ndsd`;
#		if ($? == 0) {
#			`pkill -9 ndsd`;
#		}

		#Replace the exact environment variable
#		`/usr/bin/grep -i "^n4u.server.fips_tls=" /etc/opt/novell/eDirectory/conf/nds.conf`;
#		if($? == 0){
#			`/usr/bin/sed -i -e 's/^n4u.server.fips_tls=[0-9]/n4u.server.fips_tls=0/g' /etc/opt/novell/eDirectory/conf/nds.conf`;

#			if(($?>>8) != 0) {
#				Log(WARN, "Unable to disable the fips mode. Check the /etc/opt/novell/eDirectory/conf/nds.conf file.");
#			}
#		}else{
#			`echo "n4u.server.fips_tls=0" >> /etc/opt/novell/eDirectory/conf/nds.conf`;
#		}

		#Start the ndsd service
		util::start_ndsd();
#	};

#	if($@){
#		Log(WARN, $@);
#	}



	get_and_validate_password();

	eval {
		stage::stage_rc();

		$stagedir = registry::getReg("STAGEDIR");

		my @stage_dir_list = ( "/opt/novell/xad/libexec",
				"/opt/novell/xad/sbin",
				"/opt/novell/xad/bin",
				"/usr/lib/systemd/system");

		foreach my $dir_stg (@stage_dir_list){
			Log(INFO, "rm ${stagedir}$dir_stg/*");
			`rm -v ${stagedir}$dir_stg/* 2>&1`;
		}

		stage::stage_rc();

		foreach my $dir_stg (@stage_dir_list){
			Log(INFO, "cp ${stagedir}$dir_stg/* $dir_stg/");
			`cp -v ${stagedir}$dir_stg/* $dir_stg/ 2>&1`;
		}

		if(!-e "/usr/lib/systemd/system/xadsd.service"){
			Log(INFO, "systemd files not created");
		}

	};

	if($@){
		Log(WARN, $@);
	}

	#set services with run levels
	setRunLevelForNonXADServices();

	w32tm_related_changes();

	#Reload xad services
	util::restart_services();

	# Migrate from sp2/sp3/OES11 level
	if (($eDirVersion[0] eq "edir2_sp2" or
		$eDirVersion[0] eq "edir2_sp3" or
		$eDirVersion[0] eq "edir2_oes11")) {
		migration::migrate2_oes11($eDirVersion[0]);
		migration::migrate2_oes11sp2($eDirVersion[0]);
		migration::migrate2_oes2015($eDirVersion[0]);
		migration::migrate2_oes2015sp1($eDirVersion[0]);
		migration::migrate2_oes2018($target_eDirVersion[0]);
		migration::migrate2_oes2018sp1($target_eDirVersion[0]);
		migration::migrate2_oes2018sp2($target_eDirVersion[0]);
		migration::migrate2_oes2018sp3($target_eDirVersion[0]);
		migration::migrate2_oes2023($target_eDirVersion[0]);
	}

	# Migrate from OES11SP1 level
	if (
		$eDirVersion[0] eq "edir_oes11_sp1") {
		migration::migrate2_oes11sp1($eDirVersion[0]);

		#update winbind and apache2 init script
		#Bug : 825911 This update is done to take care of the
		#WBC_ERR_DOMAIN_NOT_FOUND error, when 'xadcntrl validate' is executed.
		#Bug : 834773 Also updating the performance tuning parameters.
		util_update::update_winbind_script();
		migration::migrate2_oes11sp2($eDirVersion[0]);
		migration::migrate2_oes2015($eDirVersion[0]);
		migration::migrate2_oes2015sp1($eDirVersion[0]);
		migration::migrate2_oes2018($target_eDirVersion[0]);
		migration::migrate2_oes2018sp1($target_eDirVersion[0]);
		migration::migrate2_oes2018sp2($target_eDirVersion[0]);
		migration::migrate2_oes2018sp3($target_eDirVersion[0]);
		migration::migrate2_oes2023($target_eDirVersion[0]);
	}

	# Migrate from OES11SP2 level
	if (
		$eDirVersion[0] eq "edir_oes11_sp2") {
		migration::migrate2_oes11sp2($eDirVersion[0]);
		#update winbind and apache2 init script
		util_update::update_winbind_script();
		migration::migrate2_oes2015($eDirVersion[0]);
		migration::migrate2_oes2015sp1($eDirVersion[0]);
		migration::migrate2_oes2018($target_eDirVersion[0]);
		migration::migrate2_oes2018sp1($target_eDirVersion[0]);
		migration::migrate2_oes2018sp2($target_eDirVersion[0]);
		migration::migrate2_oes2018sp3($target_eDirVersion[0]);
		migration::migrate2_oes2023($target_eDirVersion[0]);
	}

	# Migrate from OES2015 level
	if (
		$eDirVersion[0] eq "edir_oes2015") {
		migration::migrate2_oes2015($eDirVersion[0]);
		migration::migrate2_oes2015sp1($eDirVersion[0]);
		migration::migrate2_oes2018($target_eDirVersion[0]);
		migration::migrate2_oes2018sp1($target_eDirVersion[0]);
		migration::migrate2_oes2018sp2($target_eDirVersion[0]);
		migration::migrate2_oes2018sp3($target_eDirVersion[0]);
		migration::migrate2_oes2023($target_eDirVersion[0]);
	}

	# Migrate from OES2015SP1 level
	if (
		$eDirVersion[0] eq "edir_oes2015_sp1" or
		$eDirVersion[0] eq "edir_oes11_sp3") {
		migration::migrate2_oes2015sp1($eDirVersion[0]);
		migration::migrate2_oes2018($target_eDirVersion[0]);
		migration::migrate2_oes2018sp1($target_eDirVersion[0]);
		migration::migrate2_oes2018sp2($target_eDirVersion[0]);
		migration::migrate2_oes2018sp3($target_eDirVersion[0]);
		migration::migrate2_oes2023($target_eDirVersion[0]);
	}

	# Migrate from OES2018 level or Migrate from OES2018sp1 level
	# During oes2018sp1,  migration::migrate2_oes2018 was removed to ensure that schema update is not repeated during
	# migration from oes2018 to oes2018sp1 See bug #1120588
	if (
		$eDirVersion[0] eq "edir_oes2018" or
		$eDirVersion[0] eq "edir_oes2018_sp1") {
		migration::migrate2_oes2018sp1($target_eDirVersion[0]);
		migration::migrate2_oes2018sp2($target_eDirVersion[0]);
		migration::migrate2_oes2018sp3($target_eDirVersion[0]);
		migration::migrate2_oes2023($target_eDirVersion[0]);
	}
	
	 # Migrate from OES2018SP2 level
	if (
                $eDirVersion[0] eq "edir_oes2018_sp2") {
                migration::migrate2_oes2018sp2($target_eDirVersion[0]);
                migration::migrate2_oes2018sp3($target_eDirVersion[0]);
		migration::migrate2_oes2023($target_eDirVersion[0]);
        }
	 # Migrate from OES2018SP3 level
	if (
                $eDirVersion[0] eq "edir_oes2018_sp3") {
                migration::migrate2_oes2018sp3($target_eDirVersion[0]);
		migration::migrate2_oes2023($target_eDirVersion[0]);
        }
	 # Migrate from OES2023 level
	if (
                $eDirVersion[0] eq "edir_oes2023") {
		migration::migrate2_oes2023($target_eDirVersion[0]);
        }
	
	# Migrate to OES2023SP1 level
	migration::migrate2_oes2023sp1($target_eDirVersion[0]);
	
	eval {

	# Updating the usr.sbin.nmbd apparmor profile
        util::update_apparmor_nmbd();
	
	# Updating the usr.sbin.smbd apparmor profile
	util::update_apparmor_smbd();

	# Updating the usr.sbin.winbindd apparmor profile
	util::update_apparmor_winbindd();
	
	#Restarting apparmor to get the apparmor profiles changes of nmbd, smbd and winbindd
         my $service_start_status;

         `/usr/bin/systemctl restart apparmor.service`;
         $service_start_status = $?;
         Log(WARN, "Service apparmor restarted with code $service_start_status");
	
	#Updating the configuration files to make sysvolsync work after migrating multiple DCs
	#Fix for the bug 1160127

	#Updating the /etc/ssh/ssh_config for GSSAPI-Authentication	
	util::process_ssh_config();
	
	#Updating the /etc/ssh/sshd_config for GSSAPI-Authentication
	util::process_sshd_config();

	#Updating the /etc/rsyncd.conf for multiple sections including sysvol-sync
        util::process_rsyncd_config();

	#Updating the /etc/nsswitch.conf 
	util::update_nsswitch();

	#Restarting the ssh and rsync services
        util::ssh_rsync_restart();

	};
	if($@){
                Log(WARN, $@);
        }

	#Reload xad services
	util::restart_services();

	#Rsync restart
	util::ssh_rsync_restart();

	#Post migration Done Flag
	`touch $local_state_dir/$post_mig_file`;
	`echo "done" >$local_state_dir/$post_mig_file`;

	print STDERR "Postcheck for DSfW migration done successfully\n";
	print STDERR "You now need to reboot the server\n";
}

sub migrate_now($) {

	my $source_ip = shift;
	# First do base migration from source to target
	#migrate_base($source_ip);

	#Pre migration
	if ($premig) {
		&pre_migration_of_DSfW($source_ip);
	}

	#Migration of related services though MIGGUI
	if ($miggui) {
		&miggui_DSfW($source_ip);
	}

	#Post migration
	if ($postmig) {
		&post_migration_of_DSfW($source_ip);
	}

}
1;
