#!/usr/bin/perl -I. -I.. -I/opt/novell/xad/lib64/perl -I/opt/novell/xad/lib/perl
################################################################################
# DSfW Migration script from sp2 to oes11
#
# See the file LICENSE for redistribution information.
#
# Copyright (c) 2011
#       Novell, Inc.  All rights reserved.
################################################################################
#
# Note:
# Please do the following before executing this script
# Execute the command on target machine "ssh-keygen -t rsa"
# Copy it to source "scp /root/.ssh/id_rsa.pub source:/tmp"
# At source check for the directory /root/.ssh if not present create it
# Append the key details from /tmp/id_rsa.pub to /root/.ssh/authorized_keys as
# "cat /tmp/id_rsa.pub >> /root/.ssh/authorized_keys"
#
################################################################################
package migration;
$ENV{'PATH'} = "/opt/novell/xad/share/dcinit:$ENV{'PATH'}";

use strict;
use warnings;
use XAD::registry;
use XAD::nds;
use NDS::stage;
use XAD::secure;
use Logger qw(:DEFAULT);
use File::Copy;
use File::Temp;
use File::Path;
use util;
use Config::IniFiles;
use Install::frd_install;
use Install::child_install;
use Install::adc_install;
use Term::ReadKey;

#BEGIN{
#	our (@ISA, @EXPORT_OK, @EXPORT);
#	require Exporter;
#	@migration::migratesp2DSfW2oes11::ISA = qw(Exporter);
#	@migration::migratesp2DSfW2oes11::EXPORT = qw($migrate_log_file $SSH_CMD $SCP_CMD $lib);
#	@migration::migratesp2DSfW2oes11::EXPORT_OK = qw();
#}

my $LDAP_ERR_DUP = 20;
my $LDAP_ERR_NO_SUCH_ATTR = 16;
my $LDAP_ERR_ENTRY_ALREADY_EXIST = 68;
my $ldapmodify = registry::getReg("LDAPMODIFY");
my $password_gen_tool="/opt/novell/proxymgmt/bin/randpolicypwd";

use vars qw($lib $migrate_log_file $SSH_CMD $SCP_CMD $TARC_CMD $TARX_CMD $RM_CMD);
use vars qw($premig $sourceIp $postmig $miggui);
$lib = registry::getReg("_LIB");
$migrate_log_file = "/var/opt/novell/xad/log/dsfw_migrate.log";
$SSH_CMD = "ssh -o BatchMode=yes -o ConnectionAttempts=5";
$SCP_CMD = "scp -o BatchMode=yes -o ConnectionAttempts=5";
$TARC_CMD = "/bin/tar -jcf";
$TARX_CMD = "/bin/tar -xf";
$RM_CMD = "/bin/rm -f";
($premig, $sourceIp, $postmig, $miggui) = (0, 0, 0, 0);


#Log Routine and Execution Command Routine
sub EXEC_COMMAND{
	my %args = @_;
        my $log_file  = $args{file} || $migrate_log_file;
	my $cmd  = $args{cmd} || die("No command is provided");
	my $redirect = $args{redir} || 0;
	my ($result, $status, $error);

	if($redirect){
		$result = `$cmd 2>&1`;
	}
	else{
		$result = `$cmd >>$log_file 2>&1`;
	}
	$status = $?;
	$error = $status>>8;

	return ($status, $error, $result);
}

# Display the command status after execution
sub displayCommandStatus($$){
	my ($st , $er) = ($?, $!);
	if($st == -1){
		Log(WARN,"Failed to execute : $er");
	}
	elsif($st & 127){
		Log(WARN,"Child died with signal ".($st & 127)." ".(($st & 128) ? 'with' : 'without')." coredump");
	}
	else{
		Log(INFO,"Child exited with value ".($st>>8));
	}
}

# Need to restrict the 'other' user access to sysvol (SP2 -> SP3 specific)
sub migrate_sysvol_permission {
        my $DOMAIN_COMPUTERS_GID_OFFSET = 15;
        my $DOMAIN_USERS_GID_OFFSET = 13;
        my $live_local_statedir = registry::getReg("LIVE_LOCALSTATEDIR");
        my $administrator_uid = registry::getReg("ADMINISTRATOR_UID");
        my $domain_admins_gid = registry::getReg("DOMAIN_ADMINS_GID");
        my $gp_creator_owner_gid = registry::getReg("GP_CREATOR_OWNER_GID");
        my $domain_computers_gid =$administrator_uid + $DOMAIN_COMPUTERS_GID_OFFSET;
        my $domain_users_gid=$administrator_uid + $DOMAIN_USERS_GID_OFFSET;

        Log(INFO, "Migrating Sysvol Volume access permissions: $administrator_uid, $domain_admins_gid");

        eval{
                `setfacl -R -m g:$domain_computers_gid:r-x $live_local_statedir/sysvol`;
                `setfacl -R -m g:$domain_users_gid:r-x $live_local_statedir/sysvol`;
                `setfacl -R -m o:--- $live_local_statedir/sysvol`;
                `setfacl -R -m d:g:$gp_creator_owner_gid:rwx $live_local_statedir/sysvol`;
                `setfacl -R -m d:g:$domain_computers_gid:r-x $live_local_statedir/sysvol`;
                `setfacl -R -m d:g:$domain_users_gid:r-x $live_local_statedir/sysvol`;
                `setfacl -R -m d:o:--- $live_local_statedir/sysvol`;
                `chmod -R o-r $live_local_statedir/sysvol`;
                `chmod -R o-x $live_local_statedir/sysvol`;
        };

        if ($@) {  Log(ERR, $@); }

        Log(INFO, "Sysvol folder Successfully migrated");

}

# Migrate Edir to latest sp3 (SP2 -> SP3 specific)
sub migrate_dsfw_domain() {
        my $temp_ldif = mktemp ("/tmp/tmp.XXXXXXXX");
        my $dn = registry::getReg("Domain NC");
        my $LIVE_DSSTATEDIR = registry::getReg("LIVE_DSSTATEDIR");
        my $templatedir = registry::getReg("TEMPLATEDIR");
        my $mapped_domain_nc = registry::getReg("Mapped Domain NC");
        my @mesg=();
        my $rc = 0;
        my $counter = 10;
        my $error = 0;
        my $confdir = "/etc/opt/novell/eDirectory/conf";
        my $bindir = "/opt/novell/eDirectory/bin";
	my $migrate_acl_ini = "/opt/novell/xad/share/dcinit/templates/nds-domain-upgrade-acls.ini";

        eval {
                my @password_policy_dn = ("Domain", "Default", "Builtin Domain");
                open FILE, ">$temp_ldif" or die ("failed to open the file");
                for (my $i = 0 ; $i <= $#password_policy_dn ; $i++) {
                        print FILE "dn:CN=$password_policy_dn[$i] Password Policy,CN=Password Policies,CN=System,$dn\n";
                        print FILE "replace: nspmSpecialCharactersAllowed\n";
                        print FILE "nspmSpecialCharactersAllowed: TRUE\n";
                        print FILE "-\n";
                        print FILE "replace: nspmNumericCharactersAllowed\n";
                        print FILE "nspmNumericCharactersAllowed: TRUE\n";
                        if ($i <= $#password_policy_dn -1 ) {
                                print FILE "\n";
                        }
                }
                close(FILE);

                Log(INFO, "Executing ... LDAPCONF=/etc/opt/novell/xad/openldap/ldap.conf $ldapmodify -f $temp_ldif");
                @mesg = EXEC_COMMAND(cmd => "LDAPCONF=/etc/opt/novell/xad/openldap/ldap.conf $ldapmodify -Q -Y EXTERNAL -f $temp_ldif");
                if ($mesg[0]) {
                        die("Failed to update entry for Password Policy: [$mesg[1]]");
                }
                Log(INFO, "Updated the password policy ");
                unlink($temp_ldif);

	        # Update the five attributes related to Password Policy (Xenserver 5.6 issue)
		# Check for pre-existing values and if present OR the existing values with the new ones
                open FILE, ">$migrate_acl_ini" or die ("failed to open the file $migrate_acl_ini");
                print FILE "[DEFAULTROOTDOMAIN]\n";
                print FILE "CHILD=Domain NC ACLs and Policy\n";
                print FILE "[Domain NC ACLs and Policy]\n";
                print FILE "RDN-Of-Object=DEFAULTROOTDOMAIN\n";
                print FILE "changetype=modify\n";
                print FILE "add=ACL\n";
                print FILE "ACL=3#subtree#CN=Domain Computers,CN=users,$mapped_domain_nc#PasswordExpirationInterval\n";
                print FILE "ACL=3#subtree#CN=Domain Computers,CN=users,$mapped_domain_nc#PasswordMinimumLength\n";
                print FILE "ACL=3#subtree#CN=Domain Computers,CN=users,$mapped_domain_nc#nspmConfigurationOptions\n";
                print FILE "ACL=3#subtree#CN=Domain Computers,CN=users,$mapped_domain_nc#nspmMinPasswordLifetime\n";
                print FILE "ACL=3#subtree#CN=Domain Computers,CN=users,$mapped_domain_nc#pwdInHistory\n";
                print FILE "ACL=3#subtree#[Root]#ms-net-ieee-80211-GP-PolicyData\n";
                print FILE "ACL=3#subtree#[Root]#ms-net-ieee-80211-GP-PolicyGUID\n";
                close(FILE);

		my $xad_ini = "/etc/opt/novell/xad/xad.ini";
		my $upgrade = registry::getReg("UPGRADE") ;

		`echo "UPGRADE = YES" >> $xad_ini` if ($upgrade eq "");
		`/opt/novell/xad/share/dcinit/ComputeEffectiveAcls.pl -n $mapped_domain_nc`;
		util::replace_line_in_file("UPGRADE = YES", "", $xad_ini) if ($upgrade eq "");
;
		`/opt/novell/xad/share/dcinit/genLdif.pl /tmp/effective_acls --defaultrootdomain > $LIVE_DSSTATEDIR/domain/nds-domain-upgrade-acls.ldif`;
		`rm -f /tmp/effective_acls`;
		$error = $? >> 8;
		Log(INFO, "Executing ... LDAPCONF=/etc/opt/novell/xad/openldap/ldap.conf $ldapmodify -c -Q -Y EXTERNAL -f $LIVE_DSSTATEDIR/domain/nds-domain-upgrade-acls.ldif");
		@mesg = EXEC_COMMAND(cmd => "LDAPCONF=/etc/opt/novell/xad/openldap/ldap.conf $ldapmodify -c -Q -Y EXTERNAL -f $LIVE_DSSTATEDIR/domain/nds-domain-upgrade-acls.ldif");
		if ($mesg[1] and $mesg[1] != $LDAP_ERR_DUP) {
		        Log(WARN, "Failed to update ACLs on the domain object");
		}
		Log(INFO, "Updated the  ACLs on the domain object ");
        };

        if ($@) {
                Log(ERR, $@);
        }
}

# Update the keytab file with new entries (SP2 -> SP3 Specific)
sub migrate_keytab() {
        my $LIVE_DSSTATEDIR = registry::getReg("LIVE_DSSTATEDIR");

        if( -f "$LIVE_DSSTATEDIR/krb5kdc/krb5.keytab") {
                Log(INFO, "Moving existing krb5.keytab to $LIVE_DSSTATEDIR/krb5kdc/krb5.keytab.old");
                move ("$LIVE_DSSTATEDIR/krb5kdc/krb5.keytab", "$LIVE_DSSTATEDIR/krb5kdc/krb5.keytab.old");
        }

        # Enable if not done
        secure::enable_krb5_local_lookup();

        my $tempdir = registry::getReg("TEMPDIR");
        my $stagedir = registry::getReg("STAGEDIR");
        `mkdir -p $tempdir`;

        # generate the keytab entries;
        secure::bootstrap_machine();
        `/opt/novell/xad/bin/kinit -R >&/dev/null`;
        `rm -rf $stagedir`;

        Log(INFO, "Updated keytab entries with latest changes");
}

#updat the password policies (SP2 -> SP3 Specific)
sub update_attribute {
        my $dn = shift;
        my $attribute = shift;
        my $value = shift;
        my $force = shift;
	my @mesg=();

        my $ldapsearch = "/usr/bin/ldapsearch";

        chomp ($lib);
	@mesg = EXEC_COMMAND(cmd => "LDAPCONF=/etc/opt/novell/xad/openldap/ldap.conf $ldapsearch -Q -Y EXTERNAL -b '$dn' -s base -LLL $attribute 2>/dev/null | grep -q '$attribute'");

        if ($mesg[0] != 0 or $force eq "TRUE") {
                Log(INFO, "\t Updating the value of the attribute $attribute to $value");

                my $tmpfile = mktemp("/tmp/tmp.XXXXXXXXXX");
                open FILE, ">$tmpfile" or die ("failed to open the file $tmpfile for write");
                print FILE "dn: $dn\n";
                print FILE "changetype: modify\n";

                if ($mesg[0] == 0) {
                    print FILE "replace: $attribute\n";
                }
                else {
                    print FILE "add: $attribute\n";
                }

                print FILE "$attribute: $value\n";
                close (FILE);

                @mesg = EXEC_COMMAND(cmd => "LDAPCONF=/etc/opt/novell/xad/openldap/ldap.conf $ldapmodify -Q -Y EXTERNAL -f $tmpfile");
                if ($mesg[0]) {
                    Log(WARN, "Failed to update the attribute $attribute to the value $value in the object $dn");
                }
                else {
                    unlink ($tmpfile);
                }
        }
}

sub update_password_policy {

        my $policydn = shift;
        my $force = shift;
        my %listofattrs = ( nspmLowerAsLastCharacter => 'TRUE', nspmLowerAsFirstCharacter => 'TRUE',
                            nspmExtendedCharactersAllowed => 'TRUE', nspmCaseSensitive => 'FALSE',
                            nspmSpecialAsLastCharacter => 'TRUE', nspmSpecialAsFirstCharacter => 'TRUE',
                            nspmNumericAsLastCharacter => 'TRUE', nspmNumericAsFirstCharacter => 'TRUE',
                            passwordUniqueRequired => 'FALSE', nspmSpecialCharactersAllowed => 'TRUE',
                            nspmNumericCharactersAllowed => 'TRUE' );

        Log(INFO, "Updating the policy $policydn");
        while( my ($key, $value) = each (%listofattrs)) {
                update_attribute ($policydn, $key, $value, $force);
        }

}

sub migrate_password_policies {
        my $dn = registry::getReg("Domain NC");
        my $ldapsearch = "/usr/bin/ldapsearch";
        my $ldap_base = "cn=Password Policies,cn=System,$dn";
        my $ldap_filter = ' | sed " /^ / {; H; d; }; /^ /! {; x; s/\n //g; }; "';
	my @mesg = ();

        chomp ($lib);

        @mesg = EXEC_COMMAND(cmd => "LDAPCONF=/etc/opt/novell/xad/openldap/ldap.conf $ldapsearch -Q -Y EXTERNAL -b '$ldap_base' -s one -LLL \"(objectclass=nspmPasswordPolicy)\" dn $ldap_filter", redir => 1);
        Log(INFO, "Finding the password policies");

        my @list=split('\n', $mesg[2]);
        foreach my $ele (@list){
                chomp($ele);
                next if($ele eq "");
                $ele=~s/dn:\s+//;
        }

        foreach my $policy (@list) {
            chomp ($policy);
	    next if($policy eq "");
            if ($policy =~ /^cn=Builtin Domain Password Policy,/ or $policy =~ /^cn=Default Password Policy,/) {
                update_password_policy ($policy, "TRUE"); #Force all default attribute
            }
            else {
                update_password_policy ($policy, "FALSE"); #Do not replace if any attribute already exist
            }
        }
}

sub create_dnsproxy_user($$) {
	my $cn=shift;
	my $dn=shift;
	my $tmp_ldif = mktemp("/tmp/add_user_dnsdhcp.XXXX");
	my @mesg=();

	eval {
		# Creating proxy user
		open FILE, ">$tmp_ldif" or die ("Cannot create LDIF for DNS proxy user creation\n");
		print FILE "dn: $dn\n";
		print FILE "changeType: add\n";
		print FILE "objectclass: user\n";
		print FILE "cn: $cn\n";
		print FILE "sn: -\n";
		print FILE "useraccountcontrol: 512\n";
		close FILE;

		Log(INFO, "Executing ... LDAPCONF=/etc/opt/novell/xad/openldap/ldap.conf".
				" /usr/bin/ldapmodify -Y EXTERNAL -f $tmp_ldif");
		@mesg = EXEC_COMMAND(cmd => "LDAPCONF=/etc/opt/novell/xad/openldap/ldap.conf".
				" /usr/bin/ldapmodify -Y EXTERNAL -f $tmp_ldif");
		if ($mesg[0] and $mesg[1] != $LDAP_ERR_ENTRY_ALREADY_EXIST) {
			die("Unable to create the dnsproxy user : [$mesg[0]] [$mesg[1]]");
		}

		unlink($tmp_ldif);

		$tmp_ldif = mktemp("/tmp/add_nspmpwdpoldn.XXXX");
		open FILE, ">$tmp_ldif" or
			die ("Cannot create LDIF for adding the nspmPasswordPolicyDN attribute\n");
		print FILE "dn: $dn\n";
		print FILE "changetype: modify\n";
		print FILE "replace: nspmPasswordPolicyDN\n";
		print FILE "nspmPasswordPolicyDN: cn=common_proxy_policy,cn=password policies,cn=security\n";
		close FILE;

		Log(INFO, "Executing ... LDAPCONF=/etc/opt/novell/xad/openldap/ldap.conf".
				" /usr/bin/ldapmodify -Y EXTERNAL -f $tmp_ldif");
		@mesg = EXEC_COMMAND(cmd => "LDAPCONF=/etc/opt/novell/xad/openldap/ldap.conf".
				" /usr/bin/ldapmodify -Y EXTERNAL -f $tmp_ldif");
		if ($mesg[0] and $mesg[1] != $LDAP_ERR_DUP) {
			die("Unable to update the nspmPasswordPolicyDN attribute on".
					" the common proxy user : [$dn] [$mesg[0]] [$mesg[1]]");
		}

		unlink($tmp_ldif);

		$tmp_ldif = mktemp("/tmp/add_nsimassignment.XXXX");
		open FILE, ">$tmp_ldif" or
			die ("Cannot create LDIF for adding the nsimAssignments attribute\n");
		print FILE "dn: cn=common_proxy_policy,cn=password policies,cn=security\n";
		print FILE "changetype: modify\n";
		print FILE "add: nsimAssignments\n";
		print FILE "nsimAssignments: $dn\n";
		close FILE;

		Log(INFO, "Executing ... LDAPCONF=/etc/opt/novell/xad/openldap/ldap.conf".
				" /usr/bin/ldapmodify -Y EXTERNAL -f $tmp_ldif");
		@mesg = EXEC_COMMAND(cmd => "LDAPCONF=/etc/opt/novell/xad/openldap/ldap.conf".
				" /usr/bin/ldapmodify -Y EXTERNAL -f $tmp_ldif");
		if ($mesg[0] and $mesg[1] != $LDAP_ERR_DUP) {
			die("Unable to update the nsimAssignments attribute on".
					" the common proxy password policy : [$mesg[0]] [$mesg[1]]");
		}

		unlink($tmp_ldif);
	};

	if ($@) {
		Log(ERR, $@);
	}
}
sub set_dnsproxy_user_password($$) {
	my $dn=shift;
	my $err="";
	my $count=0;
	my $retry_count = 20;
	my $status=1;
	my @mesg=();
	my $tmp_ldif = mktemp("/tmp/add_pwd_dnsdhcp.XXXX");
	my $pwd=shift;

	eval {
		Log("Generating password for user [$dn] using tool /opt/novell/proxymgmt/bin/randpwd.sh");
		@mesg = EXEC_COMMAND(cmd => "/opt/novell/proxymgmt/bin/randpwd.sh", redir => 1);
		$$pwd = $mesg[2];

		if (!$mesg[0]) {
			chomp($$pwd);
			# Setting the user password
			open FILE, ">$tmp_ldif" or
				die ("Cannot create LDIF for setting DNS proxy user password\n");
			print FILE "dn: $dn\n";
			print FILE "changeType: modify\n";
			print FILE "add: userpassword\n";
			print FILE "userpassword: $$pwd\n";
			close FILE;
			Log(INFO, "Executing ... LDAPCONF=/etc/opt/novell/xad/openldap/ldap.conf".
					" /usr/bin/ldapmodify -Y EXTERNAL -f $tmp_ldif");
			@mesg = EXEC_COMMAND(cmd => "LDAPCONF=/etc/opt/novell/xad/openldap/ldap.conf".
					" /usr/bin/ldapmodify -Y EXTERNAL -f $tmp_ldif");
			if ($mesg[0]) {
				$err="Unable to set the password for user [$dn] [$mesg[0]] [$mesg[1]]";
			} else {
				unlink($tmp_ldif);
				Log(INFO, "Successfully set the password for user [$dn]");
				return;
			}
		} else {
			$err="Unable to generate the password for user [$dn] [$mesg[0]] [$mesg[1]]";
		}

		Log(ERR, "$err");
=comment
		# This part of the code is intentionally commented and
		# expecting that the common proxy policy is not changed
		# on the server.
		if($err ne ""){
			Log(WARN,"$err");
			Log(INFO, "Generating password for user [$dn] using tool $password_gen_tool");

			#Enter the credential for tree admin
			my $ndsdomainadmin = registry::getReg("NDSDOMAINADMIN");
			$ndsdomainadmin =~ s/\./,/g;
			my $ndsdomainadmin_pwd = "";
			print "\nPlease enter the password for [$ndsdomainadmin] : ";
			ReadMode('noecho');
			$ndsdomainadmin_pwd = <STDIN>;
			chomp($ndsdomainadmin_pwd);
			ReadMode('normal');
			print "\n";

			do{
				Log(INFO, "Executing ... $password_gen_tool localhost 636 1".
						" \"$ndsdomainadmin\" **** \"$dn\"");
				@mesg = EXEC_COMMAND(cmd => "$password_gen_tool localhost 636 1".
						" \"$ndsdomainadmin\" \"$ndsdomainadmin_pwd\" \"$dn\"", redir => 1);
				if($mesg[0]){
					goto continue_loop;
				}
				$$pwd=$mesg[2];
				chomp($$pwd);

				# Setting the user password
				open FILE, ">$tmp_ldif" or die ("Cannot create the ldif for creating dnsproxy user\n");
				print FILE "dn: $dn\n";
				print FILE "changeType: modify\n";
				print FILE "add: userpassword\n";
				print FILE "userpassword: $$pwd\n";
				close FILE;
				Log(INFO, "Executing ... LDAPCONF=/etc/opt/novell/xad/openldap/ldap.conf".
						" /usr/bin/ldapmodify -Y EXTERNAL -f $tmp_ldif");
				@mesg = EXEC_COMMAND(cmd => "LDAPCONF=/etc/opt/novell/xad/openldap/ldap.conf".
						" /usr/bin/ldapmodify -Y EXTERNAL -f $tmp_ldif");

				if ($mesg[0]) {
continue_loop:
					$count++;
					sleep(2);
					Log(INFO, "Retrying to generate the password [retry count : $count]");
				}
				else{
					unlink($tmp_ldif);
					Log(INFO, "Successfully set the password for user [$dn]");
					return;
				}
			} while ($count<$retry_count);
		}

		if ($count == $retry_count) {
			die("Cannot set the password for user [$dn] [$mesg[1]] [$mesg[2]]");
		}
=cut
	};

	if ($@) {
		Log(ERR, $@);
	}
}

sub create_dns_proxy_ldap_object($$$$) {
	#my $source_ip = shift;
	my $cn = shift;
	my $dn = shift;
	my $group = shift;
	my $pwd = shift;
	my $cmd="";
	my $status="";
	my @mesg=();

	my $tmp_ldif = mktemp("/tmp/add_dnsdhcp.XXXX");

	$ENV{'LD_PRELOAD'}="/usr/$lib/libgssapi_krb5.so.2";
	eval {
		# Creating the user and Setting Password
		create_dnsproxy_user($cn, $dn);
		set_dnsproxy_user_password($dn, $pwd);

		# Add the new user to the dns-dhcp group
		open FILE, ">$tmp_ldif" or die ("Cannot create the ldif for creating dnsproxy user\n");
		print FILE "dn: $group\n";
		print FILE "changetype: modify\n";
		print FILE "add: member\n";
		print FILE "member: $dn\n";
		close FILE;

		#EXEC_COMMAND(cmd => "${SCP_CMD} -rp $tmp_ldif $source_ip:$tmp_ldif");
		@mesg = EXEC_COMMAND(cmd => "LDAPCONF=/etc/opt/novell/xad/openldap/ldap.conf /usr/bin/ldapmodify -Y EXTERNAL -f $tmp_ldif");
		if ($mesg[0] and $mesg[1] != $LDAP_ERR_DUP) {
			die ("Failed while adding the user [$dn] to dns-dhcp group [$group]: $mesg[1]");
		}
		unlink $tmp_ldif;

	};
	delete $ENV{'LD_PRELOAD'};

	if ($@) {
		Log(ERR, $@);
	}
}

sub createCASACreds (){
	my $hostname = `/bin/hostname -s`;
	chomp($hostname);
	my $rand_name="OESCommonProxy_${hostname}";
	my $dnsdhcp_group = registry::getReg("DNSDHCP_GROUP");
	my $dns_named_user = $dnsdhcp_group;
	my $pwd = "";
	$dns_named_user=~s/DNSDHCP-GROUP/$rand_name/;

	Log(INFO, "RAND NAME : $rand_name DNS NAMED USER : $dns_named_user");

	create_dns_proxy_ldap_object($rand_name, $dns_named_user, $dnsdhcp_group, \$pwd);
	`CASAcli -d -n dns-ldap`;
	`KEYVALUE="$dns_named_user" CASAcli -s -n dns-ldap -k CN`;
	`KEYVALUE="$pwd" CASAcli -s -n dns-ldap -k Password`;
}


sub migratesp2 {
		# Initializing the registry after migrate
		registry::clearReg();
		registry::reg_init();

		Log(INFO, "Migration starts from OES2 SP2 to OES11");

		my $install_type = registry::getReg("INSTALL_TYPE");
		Log(INFO, "The install type is [$install_type]");

		my $inst ;
		if (registry::getReg("IS_REPLICA")) {
			$inst = adc_install->new();
		}elsif ($install_type eq "frd") {
			$inst = frd_install->new();
		}
		elsif ($install_type eq "child") {
			$inst = child_install->new();
		}
		# Protect sysvol - ACL permission restrictive
		migrate_sysvol_permission();

		# Update DSfW related data
		migrate_dsfw_domain();

		# Migrate the keytab file
		migrate_keytab();

		# Migrate password policies
		migrate_password_policies();

		# Loading dsearch plugin
		util::load_dsearch_plugin();
}
1;
