#!/usr/bin/perl -I. -I.. -I/opt/novell/xad/lib64/perl -I/opt/novell/xad/lib/perl
################################################################################
# DSfW Upgrade script for upgrading from sp3 to oes11
#
# See the file LICENSE for redistribution information.
#
# Copyright (c) 2001-2010
#	Novell, Inc.  All rights reserved.
################################################################################
package upgrade2oes11;
$ENV{'PATH'} = "/opt/novell/xad/share/dcinit:$ENV{'PATH'}";

use strict;
use warnings;
use XAD::registry;
use Logger qw(:DEFAULT);
use File::Copy;
use File::Temp;
use NDS::stage;
use File::Path;
use util;
use Config::IniFiles;

my $debug = 0;
my $lib = registry::getReg("_LIB");
my $LDAP_ERR_DUP = 20;
# Upgrade the smb.conf file to latest oes11 changes
sub upgrade_smb_conf_file() {
	my $smb_file = "/etc/samba/smb.conf";
	my $section = "";
	my $val = "";

	my $cfg = Config::IniFiles->new (-file=> $smb_file, -nocase=>1);
	if (!defined $cfg) { 
		Log("Could not open the file $smb_file");
		return -1;
	}

	Log ("Updating $smb_file file with latest entries");
	Log ("Upgrading section - global");

	#removing use kerberos keytab
	$val = $cfg->val('global', 'use kerberos keytab');
	if (defined $val) {
		$cfg->delval ('global', 'use kerberos keytab');
	}

	#idmap config addition
	my $var_value = "idmap config : backend";
	$val = $cfg->val('global', $var_value);
	if (!defined $val) {
		$cfg->newval ('global', $var_value, 'ad');
	}else {
		$cfg->setval ('global', $var_value, 'ad');
	}

	#kerberos method
	$val = $cfg->val('global', 'kerberos method');
	if (!defined $val) {
		$cfg->newval ('global', 'kerberos method', 'system keytab');
	}else {
		$cfg->setval ('global', 'kerberos method', 'system keytab');
	}

	#np : proxy
	$val = $cfg->val('global', 'np : proxy');
	if (!defined $val) {
		$cfg->newval ('global', 'np : proxy', 'samr lsarpc netlogon');
	}else {
		$cfg->setval ('global', 'np : proxy', 'samr lsarpc netlogon');
	}

	#external_rpc_pipe : socket_dir
	$val = $cfg->val('global', 'external_rpc_pipe : socket_dir');
	my $ncalrpcdir = registry::getReg("NcalRpcDir");
	if (!defined $val) {
		$cfg->newval ('global', 'external_rpc_pipe : socket_dir', $ncalrpcdir.'/PIPE');
	}else {
		$cfg->setval ('global', 'external_rpc_pipe : socket_dir', $ncalrpcdir.'/PIPE');
	}
	
	$cfg->RewriteConfig();
	`sed -i 's/^[a-zA-Z]/\t&/g' $smb_file`;
	`sed -i 's/=/ = /g' $smb_file`;

	Log ("Updated $smb_file file with latest entries");
	return 0;
}


# update xadss.conf file
sub update_xadsd_files () {
	my $template_dir = registry::getReg("TEMPLATEDIR");
	my $etc_dir = registry::getReg("LIVE_ETCDIR");

	unlink ("$etc_dir/xadss.conf");
	stage::reg_substitute("$template_dir/xadss.conf", "$etc_dir/xadss.conf");
	
	Log("   Creating xadsd init script");
	unlink ("/etc/init.d/xadsd");
	stage::reg_substitute("$template_dir/xadsd.init", "/etc/init.d/xadsd");
	chmod 0755, "/etc/init.d/xadsd";	
}
sub update_index (){

	my $LIVE_DSSTATEDIR = registry::getReg("LIVE_DSSTATEDIR");
	my $ldapmodify = registry::getReg("LDAPMODIFY");
	my $upgrade_index_ini = "/opt/novell/xad/share/dcinit/templates/ncp-index.ini";

	# Add value index for uniqueDomainID and SecurityIdentifier
	open FILE, ">$upgrade_index_ini" or die ("failed to open the file $upgrade_index_ini");
	print FILE "[NDSTEMPCONTAINER]\n";
	print FILE "CHILD=Old Server\n";
	print FILE "[Old Server]\n";
	print FILE "RDN-Of-Object=\$REGISTRY=InstallMachineName\n";
	print FILE "changetype=modify\n";
	print FILE "add=indexDefinition\n";
	print FILE "indexDefinition=0\$securityIdentifier\$2\$0\$0\$1\$securityIdentifier\n";
	print FILE "indexDefinition=0\$uniqueDomainID\$2\$0\$0\$1\$uniqueDomainID\n";
	close(FILE);
	stage::gen_ldif($upgrade_index_ini, '>', "$LIVE_DSSTATEDIR/domain/ncp-index.ldif", "--offline", "--ndstempcontainer");
	Log("Executing ... SASL_PATH=/opt/novell/xad/$lib/sasl2 $ldapmodify -c -Q -Y GSSAPI -f $LIVE_DSSTATEDIR/domain/ncp-index.ldif");
	my @output = `SASL_PATH=/opt/novell/xad/$lib/sasl2 $ldapmodify -c -Q -Y GSSAPI -f $LIVE_DSSTATEDIR/domain/ncp-index.ldif 2>&1`;
	my $error = $? >> 8;
	if ($error and $error != $LDAP_ERR_DUP) {
		chomp(@output);
		Log("Failed to update index on the NCP server object [@output]");
	}
	Log("Updated the index on the NCP server object ");
}

sub apply_password_policy_acls (){

        my $upgrade_acl_ini = "/opt/novell/xad/share/dcinit/templates/nds-domain-upgrade-acls.ini";
        my $LIVE_DSSTATEDIR = registry::getReg("LIVE_DSSTATEDIR");
        my $ldapmodify = registry::getReg("LDAPMODIFY");
	my $mapped_domain_nc = registry::getReg("Mapped Domain NC");

	if (-e $upgrade_acl_ini) {
		Log("Skipping the updation of password policy attributes as nds-domain-upgrade-acls.ini is already present");
		return;
	}
	# Logic : Check for pre-existing values and if present OR the existing values with the new ones  
	Log("About to upgrade password policy attributes as a part of oes11 upgrade");
	open FILE, ">$upgrade_acl_ini" or die ("failed to open the file $upgrade_acl_ini");
        print FILE "[DEFAULTROOTDOMAIN]\n";
        print FILE "CHILD=Domain NC ACLs and Policy\n";
        print FILE "[Domain NC ACLs and Policy]\n";
        print FILE "RDN-Of-Object=DEFAULTROOTDOMAIN\n";
        print FILE "changetype=modify\n"; 
        print FILE "add=ACL\n";
        print FILE "ACL=3#subtree#CN=Domain Computers,CN=users,$mapped_domain_nc#PasswordExpirationInterval\n";
        print FILE "ACL=3#subtree#CN=Domain Computers,CN=users,$mapped_domain_nc#PasswordMinimumLength\n";
        print FILE "ACL=3#subtree#CN=Domain Computers,CN=users,$mapped_domain_nc#nspmConfigurationOptions\n";
        print FILE "ACL=3#subtree#CN=Domain Computers,CN=users,$mapped_domain_nc#nspmMinPasswordLifetime\n";
        print FILE "ACL=3#subtree#CN=Domain Computers,CN=users,$mapped_domain_nc#pwdInHistory\n";
        print FILE "ACL=3#subtree#[Root]#ms-net-ieee-80211-GP-PolicyData\n";
        print FILE "ACL=3#subtree#[Root]#ms-net-ieee-80211-GP-PolicyGUID\n";
        close(FILE);

       `/opt/novell/xad/share/dcinit/ComputeEffectiveAcls.pl -n $mapped_domain_nc`;
       `genLdif.pl /tmp/effective_acls --defaultrootdomain > $LIVE_DSSTATEDIR/domain/nds-domain-upgrade-acls.ldif`;
       `rm -f /tmp/effective_acls`;
        Log("Executing ... SASL_PATH=/opt/novell/xad/$lib/sasl2 $ldapmodify -c -Q -Y GSSAPI -f $LIVE_DSSTATEDIR/domain/nds-domain-upgrade-acls.ldif");
        my @output = `SASL_PATH=/opt/novell/xad/$lib/sasl2 $ldapmodify -c -Q -Y GSSAPI -f $LIVE_DSSTATEDIR/domain/nds-domain-upgrade-acls.ldif 2>&1`;
        my $error = $? >> 8;
        if ($error and $error != $LDAP_ERR_DUP) {
        	chomp(@output);
                Log("Failed to update ACLs on the domain object [@output]");
        }
        Log("Updated the  ACLs on the domain object ");
}

sub update_ntp_conf_file {
        my $file_name = shift;

        Log("Updating the file $file_name");

        eval{
                if (-f $file_name) {
			my $ntpconf_backup = mktemp("/etc/chrony.conf.XXXXXXXXXX");
			Log("Backup of file chrony.conf taken into $ntpconf_backup");
			copy("/etc/chrony.conf", $ntpconf_backup) or
                		Log("Coult not copy the /etc/chrony.conf to $ntpconf_backup");		 
			`sed -i -e '/^authprov/{/\\s*global\\s*/! s/.*/& global/}' $file_name`;
                }
                else {
                        die("Could not find the $file_name");
                }
        };

        if ($@) { Log(ERR, $@) }
}

sub upgrade_oes11 () {

	Logger::set_filename("/var/opt/novell/xad/log/ndsdcinit.log");
	Log("Starting DSfW server upgrade to OES11");

	my $pipe_dir = registry::getReg("LIVE_LOCALSTATEDIR")."/rpc/PIPE/";
	if (-d $pipe_dir and not -d $pipe_dir."/np")  {
		mkpath($pipe_dir."/np");
	}

	#/etc/SuSEConfig/profile changes
	util::process_suseconfig_profile();

	#Creating directory structure /var/opt/novell/xad/rpc in ntp chroot jail
	eval{
 		mkpath( "/var/lib/chrony".registry::getReg("LIVE_LOCALSTATEDIR")."/rpc" );
	};
	if($@) {
		Log(ERR,$@); #Printing ERR message and a backtrace
	}

	#Update chrony.conf file
	update_ntp_conf_file("/etc/chrony.conf");

	#upgrade the xadss.conf
	update_xadsd_files();
	
	if (registry::getReg ("IS_REPLICA") eq "TRUE") {
		registry::clearReg();
		my $out = `/opt/novell/xad/share/dcinit/provisionTools.sh get-default-container 2>/dev/null`;
		if ($? == 0) {
			chomp($out);
			registry::setReg("DEFAULT_CONTAINER", $out);
		}
		else {
			registry::setReg("DEFAULT_CONTAINER", "ou=OESSystemObjects");
		}
		registry::reg_init();
	}

	update_index();
	# Update the five attributes related to Password Policy (Xenserver 5.6 issue)
	apply_password_policy_acls();
	my $rc = 0;
	$rc = upgrade_smb_conf_file();
	if ($rc == -1) {
		Log("Failed to upgrade smb.conf file: $rc");
	}
	# Update the ServicePrincipalName attribute on the DC object with ldap/host/netbios value to support Microsoft SharePoint
	util::update_spn();
  
	Log ("Successfully upgraded the DSfW server to OES11");
}
1;
