#!/usr/bin/perl -I. -I.. -I/opt/novell/xad/lib64/perl -I/opt/novell/xad/lib/perl
################################################################################
# DSfW Upgrade script for upgrading from oes11fp0 to oes11sp1
#
# See the file LICENSE for redistribution information.
#
# Copyright (c) 2001-2012
#	Novell, Inc.  All rights reserved.
################################################################################
package upgrade2oes11sp1;
$ENV{'PATH'} = "/opt/novell/xad/share/dcinit:$ENV{'PATH'}";

use strict;
use warnings;
use XAD::registry;
use Logger qw(:DEFAULT);
use NDS::stage;
use File::Copy;
use Config::IniFiles;

my $debug = 0;
my $lib = registry::getReg("_LIB");
my $LDAP_ERR_DUP = 20;
my $LDAP_ERR_NO_SUCH_ATTR = 16;
my $ldapmodify = registry::getReg("LDAPMODIFY");
my $ldapmodifyflags = registry::getReg("LDAPMODIFY_FLAGS");
my $localtime=localtime;
my ($day,$month,$date,$time,$year)=split(/ /,$localtime);
my $stamp=$year.$month.$date."-".$time;

# This will delete the ACLs for group policy creator owners group from domain root
# 7#subtree#CN=Group Policy Creator Owners,7#subtree#CN=Group Policy Creator Owners,<user-context>#gplink
# 7#subtree#CN=Group Policy Creator Owners,7#subtree#CN=Group Policy Creator Owners,<user-context>#gpOptions
sub upgrade_gpo_rights() {

	Log("upgrading GPO rights related to Group Policy Creator Admin");
	my $error = 0;
	my $templatedir = registry::getReg("TEMPLATEDIR");
	my $LIVE_DSSTATEDIR = registry::getReg("LIVE_DSSTATEDIR");
	my @output = {};

	stage::gen_ldif("$templatedir/domain-gporights.ini", '>', "$LIVE_DSSTATEDIR/domain/domain-gporights.ldif", "--rootnc");

	Log("Executing .. $ldapmodify -f $LIVE_DSSTATEDIR/domain/domain-gporights.ldif 2>&1");
	@output = `$ldapmodify -f $LIVE_DSSTATEDIR/domain/domain-gporights.ldif 2>&1`;
	$error = $? >> 8;
	if ($error && $error != $LDAP_ERR_NO_SUCH_ATTR){
		chomp(@output);
		Log("Failed to upgrade GPO rights on current server: [@output]");
		return -1;
	} elsif ($error == $LDAP_ERR_NO_SUCH_ATTR){
		Log ("Skipping the GPO rights related ACL update");
	} else {
		Log ("Updated the GPO rights related to Group Policy Creator Admin");
	}
	
	return 0;
}

sub upgrade_domain() {

	Log("Upgrading Domain internals to OES11SP1");
	my $mapped_domain_nc = registry::getReg("Mapped Domain NC");
	my $rc = 0;
	my $error = 0;
	my $templatedir = registry::getReg("TEMPLATEDIR");
	my $LIVE_DSSTATEDIR = registry::getReg("LIVE_DSSTATEDIR");
	my @output = {};
	my $upgrade_acl_ini = "$templatedir/nds-domain-upgrade-acls.ini";

	# Check for pre-existing values and if present OR the existing values with the new ones  
	if ( -f $upgrade_acl_ini) {
		# backup the old file which may have sp2 or sp3 upgrade details 
		move($upgrade_acl_ini, $upgrade_acl_ini.".orig");
	}

	open FILE, ">$upgrade_acl_ini" or die ("failed to open the file $upgrade_acl_ini");
	print FILE "[DEFAULTROOTDOMAIN]\n";
	print FILE "CHILD=Domain NC ACLs and Policy\n";
	print FILE "[Domain NC ACLs and Policy]\n";
	print FILE "RDN-Of-Object=DEFAULTROOTDOMAIN\n";
	print FILE "changetype=modify\n";
	print FILE "add=ACL\n";
	print FILE "ACL=3#subtree#[Public]#uniqueDomainID\n";
	print FILE "ACL=7#subtree#[This]#[Entry Rights]\n";
	print FILE "ACL=3#subtree#[This]#[All Attributes Rights]\n";
	close(FILE);

	`/opt/novell/xad/share/dcinit/ComputeEffectiveAcls.pl -n $mapped_domain_nc`;
	`/opt/novell/xad/share/dcinit/genLdif.pl /tmp/effective_acls --defaultrootdomain > $LIVE_DSSTATEDIR/domain/nds-domain-upgrade-acls.ldif`;
	unlink("/tmp/effective_acls");
	Log("Executing ... SASL_PATH=/opt/novell/xad/$lib/sasl2 $ldapmodify -c -Q -Y GSSAPI -f $LIVE_DSSTATEDIR/domain/nds-domain-upgrade-acls.ldif");
	@output = `SASL_PATH=/opt/novell/xad/$lib/sasl2 $ldapmodify -c -Q -Y GSSAPI -f $LIVE_DSSTATEDIR/domain/nds-domain-upgrade-acls.ldif 2>&1`;
	$error = $? >> 8;
	if ($error and $error != $LDAP_ERR_DUP) {
		chomp(@output);
		Log("Failed to update ACLs on the domain object [@output]");
	}
	Log("Updated the  ACLs on the domain object ");
		
	$rc = upgrade_gpo_rights();
	if ($rc == -1) {
		Log("Failed to upgrade gpo rights: $rc");
	}
			
	return 0;
}

sub upgrade_init_script () {
	
	Log("   Creating xadsd init script: stamp - $year-$month-$date-$time");

	my $template_dir = registry::getReg("TEMPLATEDIR");
	`/etc/init.d/xadsd stop`; # To remove the NETLOGON (caps) pipe
	# This is having recent changes need to work with samba -3.6
	move ("/etc/init.d/xadsd", "/etc/init.d/xadsd.$stamp") or Log(ERR, "Failed to backup xadsd init script: $!");
	stage::reg_substitute("$template_dir/xadsd.init", "/etc/init.d/xadsd");
	chmod 0755, "/etc/init.d/xadsd";	

	# The xad-krb5kdc dependency on xadsd running status is removed in latest init script
	move ("/etc/init.d/xad-krb5kdc", "/etc/init.d/xad-krb5kdc.$stamp") or Log(ERR, "Failed to backup xad-krb5kdc init script: $!");
	stage::reg_substitute("$template_dir/xad-krb5kdc.init", "/etc/init.d/xad-krb5kdc");
	chmod 0755, "/etc/init.d/xad-krb5kdc";
}

sub upgrade_smb_conf () {
	my $smb_file = "/etc/samba/smb.conf";
	copy($smb_file, $smb_file.$stamp) or Log (ERR, "Failed to backup the samba conf file: $!");
	 
	my $cfg = Config::IniFiles->new (-file=> $smb_file, -nocase=>1);
	if (!defined $cfg) {
		Log ("The open of $smb_file failed : @Config::IniFiles::errors");
		return -1;
	}
	Log ("Updating $smb_file file with latest entries");

	Log ("Upgrading section - [global]");
	# Remove 'idmap backend'
	my $val = $cfg->val('global', 'idmap backend');
	if (defined $val) {
		$cfg->delval ('global', 'idmap backend');
	}

	# idmap config : range = 1 - 4294967295
	# Remove 'idmap config <NETBIOSNAME> : backend'
	my $var_value = "idmap config ".uc(registry::getReg("Netbios Name"))." : backend";
	$val = $cfg->val('global', $var_value);
	if (defined $val) {
		$cfg->delval ('global', $var_value);
	}

        #idmap config : backend
        $var_value = "idmap config : backend";
        $val = $cfg->val('global', $var_value);
        if (!defined $val) {
              $cfg->newval ('global', $var_value, 'ad');
        }else {
               $cfg->setval ('global', $var_value, 'ad');
        }

	$var_value = "idmap config : range";
	$val = $cfg->val('global', $var_value);
	if (!defined $val) {
		$cfg->newval ('global', $var_value, '1 - 4294967295');
	}

	#  send spnego principal = Yes
	$val = $cfg->val('send spnego principal');
	if (!defined $val) {
		$cfg->newval ('global', 'send spnego principal', 'Yes');
	}
	#  client use spnego principal = Yes
	$val = $cfg->val('client use spnego principal');
	if (!defined $val) {
		$cfg->newval ('global', 'client use spnego principal', 'Yes');
	}
	#  cifs instanceless spn = Yes
	$val = $cfg->val('cifs instanceless spn');
	if (!defined $val) {
		$cfg->newval ('global', 'cifs instanceless spn', 'Yes');
	}

	#  dce_funnel_winbind : socket = $REGISTRY=LocalStateDir/rpc/xadsd
	$val = $cfg->val('dce_funnel_winbind : socket');
	my $ncalrpcdir = registry::getReg("NcalRpcDir");
	if (! defined $val) {
		$cfg->newval('global', 'dce_funnel_winbind : socket', $ncalrpcdir."/xadsd");
	} else {
		$cfg->setval('global', 'dce_funnel_winbind : socket', $ncalrpcdir."/xadsd");
	}

	$cfg->RewriteConfig();
	`sed -i 's/^[a-zA-Z]/\t&/g' $smb_file`;
	`sed -i 's/=/ = /g' $smb_file`;

	Log ("Updated $smb_file file with latest entries");
	return 0;
}


sub update_msdfs_link() {

	my $live_local_statedir = registry::getReg("LIVE_LOCALSTATEDIR");
	my $dns_root = registry::getReg("DNSROOT");
	my $localdc_fqdn = registry::getReg("DNS Host Name");

	my $pdcmaster_fqdn = `/usr/bin/dig -t SRV _ldap._tcp.pdc._msdcs.$dns_root +short | awk '{print \$4}'`;

	chomp($pdcmaster_fqdn);

	if (($pdcmaster_fqdn ne "") and (uc($pdcmaster_fqdn) ne uc($localdc_fqdn))) {
		my $owner_gid = `/usr/bin/getfacl -n $live_local_statedir/msdfs  2>/dev/null | grep " group:" | awk '{print \$NF}'`;
		my $owner_uid = `/usr/bin/getfacl -n  $live_local_statedir/msdfs 2>/dev/null | grep " owner:" | awk '{print \$NF}'`;
		chomp($owner_gid);
		chomp($owner_uid);

		Log(" Creating msdfs Link $live_local_statedir/msdfs/");
		`(cd $live_local_statedir/msdfs; rm $dns_root ; ln -sf "msdfs:$localdc_fqdn\\sysvol-msdfs,$pdcmaster_fqdn\\sysvol" $live_local_statedir/msdfs/$dns_root)`;

		Log("   Changing ownership for $live_local_statedir/msdfs to $owner_uid:$owner_gid");
		`chown -R $owner_uid:$owner_gid $live_local_statedir/msdfs`;
	} else {
		Log ("It is a PDC master, skipping msdfs link update : $pdcmaster_fqdn, $localdc_fqdn");
	}

}

sub upgrade_oes11sp1 () {

	Logger::set_filename("/var/opt/novell/xad/log/ndsdcinit.log");
	Log("Starting DSfW server upgrade to OES11SP1");

	my $rc = 0;
	$rc = upgrade_domain();
	if ($rc == -1) {
		Log("Failed to upgrade domain: $rc");
	}
	
	# Update xadsd and xad-krb5kdc init script
	upgrade_init_script();	
	
	# Update samba smb.conf with latest changes wrt to 3.6
	upgrade_smb_conf();
	
	#Update the msdfs link
	update_msdfs_link();
	
	Log ("Successfully upgraded the DSfW server to OES11SP1");
}
1;
