#!/usr/bin/perl -I. -I.. -I/opt/novell/xad/lib64/perl -I/opt/novell/xad/lib/perl
################################################################################
# DSfW Upgrade script for upgrading from OES11/2015 to OES2018SP1
#
# See the file LICENSE for redistribution information.
#
# Copyright (c) 2018-2020
#	Micro Focus or one of its affiliates
################################################################################
package upgrade2oes2018sp1;
$ENV{'PATH'} = "/opt/novell/xad/share/dcinit:$ENV{'PATH'}";

use strict;
use warnings;
use XAD::registry;
use Logger qw(:DEFAULT);
use util;
use util_update;

sub upgrade_oes2018sp1() {
	Logger::set_filename("/var/opt/novell/xad/log/ndsdcinit.log");
	Log("Starting DSfW server upgrade to OES2018SP1");

	eval {
		#restart NDSD
		`/usr/sbin/rcndsd restart`;
		if ($?) {
			my $error = $? >> 8;
			Log (ERR, "Failed to restart the service NDSD: $error");
		}
		sleep(2);
		# Retrieve smb.conf file
		util::retrieve_smb_conf();

		# update AES256 capabilities on the server
		util_update::update_domain_to_aes();

		# Update samba smb.conf with latest changes wrt to 4.6
		util_update::upgrade_samba_smb_conf();

		# Updating the usr.sbin.nmbd apparmor profile
		util::update_apparmor_nmbd();

		# Updating the usr.sbin.smbd apparmor profile
		util::update_apparmor_smbd();

		# Updating the usr.sbin.winbindd apparmor profile
		util::update_apparmor_winbindd();

		#Restarting apparmor to get the apparmor profiles changes of nmbd, smbd and winbindd
		my $service_start_status;

		`/usr/bin/systemctl restart apparmor.service`;
		$service_start_status = $?;
		Log(WARN, "Service apparmor restarted with code $service_start_status");
	};

	if($@){
		Log(WARN, $@);
	}
}
1;
