#!/usr/bin/perl -I. -I.. -I/opt/novell/xad/lib64/perl -I/opt/novell/xad/lib/perl
################################################################################
# DSfW Upgrade script for upgrading from OES11/2015 to OES2018SP2
#
# See the file LICENSE for redistribution information.
#
# Copyright (c) 2018-2020
#	Micro Focus or one of its affiliates
################################################################################
package upgrade2oes2018sp2;
$ENV{'PATH'} = "/opt/novell/xad/share/dcinit:$ENV{'PATH'}";

use strict;
use warnings;
use XAD::registry;
use Logger qw(:DEFAULT);
use util;
use util_update;

sub upgrade_oes2018sp2() {
	Logger::set_filename("/var/opt/novell/xad/log/ndsdcinit.log");
	Log("Starting DSfW server upgrade to OES2018SP2");

	eval {

		# This code is called for add string in /etc/krb5.conf file.
                util::add_cache_string_krb5();

		# This code is called for update site name in /etc/opt/novell/xad/xad.ini file.
		util::update_site_name_in_xad_ini();

		# This subroutine for creating symlink for etc/opt/novell/xad/krb5.conf file.
                # This code is called for upgrade2oes2018sp2.
                util::symlink_krb5_conf_file();

		# This subroutine for checking and changing NTPD_RUN_CHROOTED variable in /etc/sysconfig/chronyd file.
                util::ntpd_run_chrooted_var_change();

		# Update samba smb.conf with latest changes.
                util_update::upgrade_samba_smb_conf();

		my $upgrade_from = registry::getReg("UPGRADE_FROM");

		# The condition check is to ensure that samba 4 specific parameters added during upgrade_oes2018sp1() upgrade stage,
		# and  are not overwritten for upgrades that are from oes2015sp1 or oes2018.
		# The upgrade_oes2018sp1() involves addition of samba 4 specific parameters.

		if ($upgrade_from eq "oes2018sp1"){
	                util::retrieve_smb_conf();
                }

		#updating the server to 2016 AD schema level
                util_update::update_server_to2016("UPGRADE");

		#Enable fips in nds.conf
		util::enable_fips();

		# This code is called for add 'Zone Container' varaible in /etc/opt/novell/xad/xad.ini file.
		util::add_zone_container_in_xad_ini();

		# Updating the ACLs for NSSAD integration
		my $type = registry::getReg("INSTALL_TYPE");
                my $is_replica = registry::getReg("IS_REPLICA");
		if( (lc($type) eq "frd" or lc($type) eq "child") and $is_replica eq "FALSE" ){
			util_update::apply_2018sp2_acls();
		}

	};

	if($@){
		Log(WARN, $@);
	}
}
1;
