#!/usr/bin/perl -I. -I/opt/novell/xad/lib64/perl -I/opt/novell/xad/lib/perl -I /opt/novell/xad/share/dcinit
################################################################################
# DSfW Upgrade script for upgrading from sp1 to sp2
#
# See the file LICENSE for redistribution information.
#
# Copyright (c) 2001-2010
#	Novell, Inc.  All rights reserved.
################################################################################
package upgrade2sp2;
use strict;
use warnings;

use File::Copy;
use File::Path;
use File::Temp;
use Sys::Hostname;

use lib "GSSAPI-0.12";
use GSSAPI;
use Net::DNS;
use Locale::gettext;
use POSIX;

use Logger qw(:DEFAULT);
use Install::frd_install;
use Install::adc_install;
use Install::child_install;
use Test::More;


my $debug = 0;
# Setting the Locale
setlocale(LC_MESSAGES, "");
bindtextdomain("xad", "/opt/novell/xad/share/locale");
textdomain("xad");

my @Existing_PDC_Masters = ();
my %Existing_PDC_Masters_record_values;
my $domain = `/opt/novell/xad/share/dcinit/printConfigKey.pl "DNS Root"`;
chomp $domain;

my $localhost_fqdn = `/opt/novell/xad/share/dcinit/printConfigKey.pl "DNS Host Name"`;
chomp $localhost_fqdn;

my $key_name1 = int(rand 10000000000000);
my $key_name = "$key_name1" ."-DSfW-123451";
# get the local machine's hostname in uppercase
my $hostname = uc(hostname());
chomp $hostname;

my $status = 0;
my $dnsserver = "";
my $domain_server_name = "";
my $nameserver = "";
my $pdc_master_dn = "";

#Error Codes :
my $ERR_NO_PDC_ENTRY_IN_DNS = -1;
my $ERR_NO_AUTHORITATIVE_PDC_QUERY_ANSWERS_FROM_DNS = -2;
my $ERR_DNS_RR_PACKET_HANDLE_CREATION_FAILED = -3;
my $ERR_NO_REPLY_TO_DYNAMIC_UPDATE = -4;

sub is_pdc_master {

	my $success = 1;
	my $return_val = 0;

	my $FSMOroleowner = "";

	my $ldapsearch = "/usr/bin/ldapsearch";
	my $ldap_filter = ' | sed " /^ / {; H; d; }; /^ /! {; x; s/\n //; }; "';
	my $ldap_host = "localhost";
	my $ldap_base = `/opt/novell/xad/share/dcinit/printConfigKey.pl "Mapped Domain NC"`;
	my $live_etcdir = `/opt/novell/xad/share/dcinit/printConfigKey.pl "LIVE_ETCDIR"`;
	chomp ($ldap_base, $live_etcdir);

	my $pipe_status_file = mktemp("/tmp/pipe_status.XXXXXXXX");

	my $Lib = `/opt/novell/xad/share/dcinit/printConfigKey.pl "_Lib"`;
	chomp $Lib;

	# convert to ldap format
	$ldap_base =~ s/\.([a-z,A-Z,0-9]+)=/,$1=/g;

      
	# The administrator's ticket should be present before running this command.
	my $command = 'SASL_PATH=/opt/novell/xad/'. $Lib . '/sasl2/ '. $ldapsearch . ' -h '. $ldap_host . ' -Y GSSAPI -b '. $ldap_base.' -s base'. ' fsmoroleowner -LLL ' . $ldap_filter . ' | grep -i "fsmoroleowner: " |  awk -F ": " \'{print $2}\' | awk -F "," \'{print $2}\' | awk -F "cn=" \'{print $2}\' 2>/dev/null';
	

	print "Enquiring PDC Master status of the local server with Directory backend.... $command \n";
	$FSMOroleowner = `$command ;  echo \${PIPESTATUS[0]} > $pipe_status_file`;
	my $ldapsearch_err = `cat $pipe_status_file`;
	chomp($ldapsearch_err);

	chomp $FSMOroleowner;

	if($ldapsearch_err)
	{
		# error encountered
		Log("Cannot perform LDAP search on $ldap_base by contacting $ldap_host. Check if LDAP server is running properly on $ldap_host. Error encountered: $ldapsearch_err");
		$return_val = -1;
		goto out;
	}

	$pdc_master_dn = lc($FSMOroleowner.".".$domain);

	# check if $FSMOroleowner points to $hostname DC
	if($FSMOroleowner =~ /$hostname/)
	{
		# so the current DC is stored as FSMO Role owner in Directory backend
		# de-ref the Is_PDC_Master_Flg flag and set it
		return $success;
	}

out:
	unlink($pipe_status_file);

	return $return_val;

}

sub frd_smb_changes($$) {
	my ($smb_file, $temp_file) = @_;
	my $towrite;
	my $templatefound = 0;
	my $server_signing_found = 0;
	my $ntaclfound = 0;
	my $template = "\ttemplate shell = /bin/bash";
	my $server_signing = "\tserver signing = auto";
	my $Is_global_section = 0;
	my $Is_homes_section = 0;
	my $Is_profiles_section = 0;
	my $readline;
	my $rc;
	my $return_val;

	$rc = open(SMB_FILE, "$smb_file");
	if(!($rc))
	{
		# error occured while opening the file
		$return_val = -1;
		goto out;
	}
	$rc = open(TEMP_FILE, "+>$temp_file");
	if(!($rc))
	{
		# error occured while opening the file
		$return_val = -1;
		goto out;
	}
	while($readline = <SMB_FILE>) {
		chomp $readline;
		if(($readline =~ m/^\[global\]/) or ($Is_global_section == 1)) {
			next if($readline !~ /\S/);
			if ($readline =~ /$template/) {
				$templatefound = 1;
			}
			if ($readline =~ /$server_signing/) {
				$server_signing_found = 1;
			}
			if(!($readline =~ m/^\[global\]/) and ($readline =~ m/^\[/)) {
				$Is_global_section = 0;
				$Is_homes_section = 1 if($readline =~ /homes/);
				if ($templatefound == 0 && $server_signing_found == 0){
					$towrite = "$template\n$server_signing\n\n" ;
					$readline =~ s/^\[/$towrite\[/;
				} elsif ($templatefound == 0 && $server_signing_found != 0){
					$towrite = "$template\n\n";
					$readline =~ s/\[/$towrite\[/;
				} elsif ($templatefound != 0 && $server_signing_found == 0){
					$towrite = "$server_signing\n\n";
					$readline =~ s/\[/$towrite\[/;
				} else { 
					$readline = "\n".$readline; 
				}
			} else {
				$Is_global_section = 1;
			}
			
			if ($readline =~ m/map to guest = Bad User/) {
				$readline = "\tmap to guest = Never";	
			}

			next if ($readline =~ m/nt acl support = No/);

		} elsif(($readline =~ m/^\[homes\]/) or ($Is_homes_section == 1)) {
			next if($readline !~ /\S/);
			if ($readline =~ /nt acl support/) {
				$ntaclfound = 1;
			}
			if(!($readline =~ m/^\[homes\]/) and ($readline =~ m/^\[/)) {
				$Is_homes_section = 0;
				$Is_profiles_section = 1 if($readline =~ /profiles/);
				if($ntaclfound == 0) {
					$towrite = "\tnt acl support = No \n\n" ;
					$readline =~ s/^\[/$towrite\[/;
				}
				else { $readline = "\n".$readline; }
				$ntaclfound = 0;
			} else {
				$Is_homes_section = 1;
			}
		} elsif(($readline =~ m/^\[profiles\]/) or ($Is_profiles_section == 1)) {
			next if($readline !~ /\S/);
			if ($readline =~ /nt acl support/) {
				$ntaclfound = 1;
			}
			if(!($readline =~ m/^\[profiles\]/) and ($readline =~ m/^\[/)) {
				$Is_profiles_section = 0;
				if($ntaclfound == 0) {
					$towrite = "\tnt acl support = No \n\n" ;
					$readline =~ s/^\[/$towrite\[/;
				}
				else { $readline = "\n".$readline; }
				$ntaclfound = 0;
			} else {
				$Is_profiles_section = 1;
			}
		}
		# add the same in the TEMP_FILE
		print TEMP_FILE "$readline\n";
	}
out:
	close(SMB_FILE);
	close(TEMP_FILE);
	return $rc;
}

sub adc_smb_changes($$) {
	my ($smb_file, $temp_file) = @_;
	my $readline;
	my $towrite;
	my $Is_sysvol_section = 0;
	my $templatefound = 0;
	my $server_signing_found = 0;
	my $ntaclfound = 0;
	my $template = "\ttemplate shell = /bin/bash";
	my $server_signing = "\tserver signing = auto";
	my $Is_global_section = 0;
	my $Is_homes_section = 0;
	my $Is_profiles_section = 0;
	my $rc;
	my $return_val;

	$rc = open(SMB_FILE, "$smb_file");
	if(!($rc))
	{
		# error occured while opening the file
		$return_val = -1;
		goto out;
	}
	$rc = open(TEMP_FILE, "+>$temp_file");
	if(!($rc))
	{
		# error occured while opening the file
		$return_val = -1;
		goto out;
	}
	while($readline = <SMB_FILE>) {
		chomp $readline;
		if(($readline =~ m/^\[global\]/) or ($Is_global_section == 1)) {
			next if($readline !~ /\S/);
			if ($readline =~ /$template/) {
				$templatefound = 1;
			}
			if ($readline =~ /$server_signing/) {
				$server_signing_found = 1;
			}
			if(!($readline =~ m/^\[global\]/) and ($readline =~ m/^\[/)) {
				$Is_global_section = 0;
				$Is_sysvol_section = 1 if($readline =~ /sysvol/);
				if ($templatefound == 0 && $server_signing_found == 0){
					$towrite = "$template\n$server_signing\n\n" ;
					$readline =~ s/^\[/$towrite\[/;
				} elsif ($templatefound == 0 && $server_signing_found != 0){
					$towrite = "$template\n\n";
					$readline =~ s/^\[/$towrite\[/;
				} elsif ($templatefound != 0 && $server_signing_found == 0){
					$towrite = "$server_signing\n\n" ;
					$readline =~ s/^\[/$towrite\[/;
				} else { 
					$readline = "\n".$readline; 
				}
			} else {
				$Is_global_section = 1;
			}

			if ($readline =~ m/map to guest = Bad User/) {
				$readline = "\tmap to guest = Never";	
			}

			next if ($readline =~ m/nt acl support = No/);

		} elsif(($readline =~ m/^\[sysvol\]/) or ($Is_sysvol_section == 1)){
			next if($readline !~ /\S/);
			if(!($readline =~ m/^\[sysvol\]/) and ($readline =~ m/^\[/)) {
				$Is_sysvol_section = 0;
				$Is_homes_section = 1 if($readline =~ /homes/);
				$towrite = "[sysvol-msdfs]\n\tcomment = Group Policies\n\tpath = /var/opt/novell/xad/sysvol/sysvol\n\twritable = No\n\tshare modes = No\n\tnt acl support = No\n\n[sysvol]\n\tcomment = msdfs link to Group Policies\n\tpath = /var/opt/novell/xad/msdfs\n\tmsdfs root = Yes\n\tnt acl support = No\n\n";
				$readline =~ s/^\[/$towrite\[/;
			} elsif ($Is_sysvol_section == 1) {
				next;
			} else {
				$Is_sysvol_section = 1;
				next;
			}
		} elsif(($readline =~ m/^\[homes\]/) or ($Is_homes_section == 1)) {
			next if($readline !~ /\S/);
			if ($readline =~ /nt acl support/) {
				$ntaclfound = 1;
			}
			if(!($readline =~ m/^\[homes\]/) and ($readline =~ m/^\[/)) {
				$Is_homes_section = 0;
				$Is_profiles_section = 1 if($readline =~ /profiles/);
				if($ntaclfound == 0) {
					$towrite = "\tnt acl support = No \n\n" ;
					$readline =~ s/^\[/$towrite\[/;
				}
				else { $readline = "\n".$readline; }
				$ntaclfound = 0;
			} else {
				$Is_homes_section = 1;
			}
		} elsif(($readline =~ m/^\[profiles\]/) or ($Is_profiles_section == 1)) {
			next if($readline !~ /\S/);
			if ($readline =~ /nt acl support/) {
				$ntaclfound = 1;
			}
			if(!($readline =~ m/^\[profiles\]/) and ($readline =~ m/^\[/)) {
				$Is_profiles_section = 0;
				if($ntaclfound == 0) {
					$towrite = "\tnt acl support = No \n\n" ;
					$readline =~ s/^\[/$towrite\[/;
				}
				else { $readline = "\n".$readline; }
				$ntaclfound = 0;
			} else {
				$Is_profiles_section = 1;
			}
		}
		# add the same in the TEMP_FILE
		print TEMP_FILE "$readline\n";
	}
out:
	close(SMB_FILE);
	close(TEMP_FILE);
	return $rc;

}

sub update_samba_config($) {

	my $type = shift;
	my $temp_file = "/tmp/smb.conf";
	my $smb_file = "/etc/samba/smb.conf";

	chomp ($temp_file);
	chomp ($smb_file);

	eval{

		if ($type == 0) {
        		adc_smb_changes($smb_file, $temp_file);
		} else { 
        		frd_smb_changes($smb_file, $temp_file);
		}

		Log("Copying /etc/samba/smb.conf to /etc/samba/smb.conf.org");
		move("/etc/samba/smb.conf", "/etc/samba/smb.conf.org");

		Log("Copying /tmp/smb.conf to /etc/samba/smb.conf");
		move("/tmp/smb.conf", "/etc/samba/smb.conf");

	};

	if ($@) { Log(ERR, $@) }

	Log("Successfully staged samba");
}

# set the nameserver for the domain######
##########################################
sub set_nameserver($)
{
	my $dnsserver = shift;
	my $res = Net::DNS::Resolver->new;
	$res->tcp_timeout(60);
	$res->udp_timeout(60);
	$res->nameservers($dnsserver);
	return $res;
}

###signing callback function for TSIG module####
################################################
sub gss_sign($$)
{
	my $key = shift;
	my $data = shift;
	my $sig;
	$key->get_mic(0, $data, $sig);
	return $sig;
}

## Find an authoritative nameserver for the domain ##
#####################################################
sub find_nameserver_for_domain($)
{
	my $domain = shift;

	# we need to parse the /etc/resolv.conf file, picking up
	# each nameserver entry and querying it to the domain passed
	# incase an authoritative answer is found, the nameserver in 
	# account is to be used as the authoritative nameserver.

	my $file = "/etc/resolv.conf";
	my $readline = "";
	my @Nameservers = "";

	if(!(-e "$file"))
	{
		Log(ERR, "Hard Error!, exiting ... ");
	}
	open(Filename, $file) or die "cannot open $file:$@";
	while ($readline = <Filename>)
	{
		if($readline =~ /^nameserver/)
		{
			#my $nameserver = "";
			print "nameserver entry found:" if($debug);
			#print "$readline \n" if($debug);
			#push the nameserver IP in the nameservers array
			$readline =~ s/nameserver //g;
			chomp $readline;
			push(@Nameservers, $readline);
		}
	}
	close(Filename);
	print "Nameservers array contain:@Nameservers\n" if($debug); 

	# parse through the nameserver array and check for an authoritative
	# nameserver for the passed domain.
	my $Nameservers_count = $#Nameservers;
	while($Nameservers_count != 0)
	{

		# check if this Namservers[$Nameservers_count] is an authoritative
		# nameserver for the domain or not
		my $handle = Net::DNS::Resolver->new;
		$handle->tcp_timeout(60);
		$handle->udp_timeout(60);
		
		# set the nameserver to $Nameservers[$Nameservers_count]
		$handle->nameservers("$Nameservers[$Nameservers_count]");
		my $domain_query = $handle->send("$domain", "A");
		if(defined($domain_query))
		{
			print "nameserver $Nameservers[$Nameservers_count] found for $domain\n" if($debug);
			
			# check for authoritative answer
			my $Is_Authoritative = $domain_query->header->aa;
			if($Is_Authoritative)
			{
				# this is the nameserver for the $domain, check the 
				# returned IPs
				
				print "printing the domain_query response\n" if($debug);
				print "======================================\n" if($debug);
				$domain_query->print if($debug);
				print "======================================\n" if($debug);

				my $server_name = ($domain_query->authority)[0]->{"nsdname"};
				chomp $server_name;
				if(defined($server_name))
				{
					print "server found : $server_name\n" if($debug);
					# check if the $Nameservers[$Nameservers_count]
					# also resolves to the same name
					my $server_query = $handle->send("$Nameservers[$Nameservers_count]", "PTR");
					if(defined($server_query))
					{
						# an answer is recieved, check for validity
						#printing the response receieved
						print "Printing the server_query response\n" if($debug);
						print "====================================\n" if($debug);
						$server_query->print if($debug);
						print "====================================\n" if($debug);
						my $server_fdn = ($server_query->answer)[0]->{"ptrdname"};
						if(defined($server_fdn))
						{
							if(lc("$server_name") eq lc("$server_fdn"))
							{
								# this is the authoritative nameserver
								# set it as the authoritative DNS server
								$dnsserver = $Nameservers[$Nameservers_count];
							}

						}
					}
					# set the authoritative NS entry found as the name of the 
					# authoritative DNS server	
					$domain_server_name = "$server_name";
					last;
				}
			}
		
		}
		$Nameservers_count--;

	}
	print "The domain_server_name is :$domain_server_name\n" if($debug);
	print "The Authoritative DNS server is :$dnsserver\n" if($debug);
}

# Get the dns entries
#######################
sub get_existing_dns_server_entries {
	# Query the Authorised Nameserver for the SRV record
	# of the PDC master. In the returned answers, verify if
	# the PDC master is set to the local DC. If set, return
	# success(1), or else return 0.

	find_nameserver_for_domain($domain);

	my $success = 0;


	# create the DNS query handle
	my $query_handle = Net::DNS::Resolver->new;
	$query_handle->tcp_timeout(60);
	$query_handle->udp_timeout(60);

	if ($dnsserver eq "") {
		Log("The dns server is not updated");
		return -1;
	}
	# set the nameserver as the authorized nameserver $dnsserver
	$query_handle->nameservers("$dnsserver");

	my $SRV_query = $query_handle->send("_ldap._tcp.pdc._msdcs.$domain", "SRV");

	# check for authoritative answer
	my $Is_authoritative = $SRV_query->header->aa;

	if($Is_authoritative)
	{
		# authoritative answers are obtained

		print "DNS answer packet for DNS_PDC_Master_query function\n" if($debug);
		print "++++++++++++++++++++++++++++++++++++++++++++++++++++\n" if ($debug);
		$SRV_query->print if($debug);
		print "=======================================================\n" if($debug);

		# get the answer count
		my $answer_count = $SRV_query->header->ancount;

		print "answer count $answer_count\n" if ($debug);

		# get all the answers and match them with the local DC's name
		if($answer_count)
		{
			my $count = 0;

			for($count = 0; $count<$answer_count; $count++)
			{
				# get the returned DC name
				my $server_dn = ($SRV_query->answer)[$count]->{"target"};
				print "server returned is :$server_dn\n" if($debug);

				# remove newline characters, if any
				chomp $server_dn;
				if("$server_dn" eq "$pdc_master_dn")
				{
					Log("$server_dn the server is PDC master");
					next;
				}
				# get the returned record's weight
				my $record_weight = ($SRV_query->answer)[$count]->{"weight"};
				print "record weight is : $record_weight\n" if($debug);

				# get the returned record's priority
				my $record_priority = ($SRV_query->answer)[$count]->{"priority"};
				print "record priority is: $record_priority\n" if($debug);

				# get the returned record's port
				my $record_port = ($SRV_query->answer)[$count]->{"port"};
				print "record port is : $record_port\n" if($debug);

				# push the $server_dn into @Existing_PDC_Masters array
				push(@Existing_PDC_Masters, $server_dn);

				# populate the %Existing_PDC_Masters_record_values hash
				$Existing_PDC_Masters_record_values{"$server_dn"} = {'weight' => "$record_weight",
					'priority' => "$record_priority",
					'port' => "$record_port"
				};
			}

			return $answer_count;
		}
		# no answers returned by DNS server, log an error
		Log("The authorized DNS server at returned no answers for '_ldap._tcp.pdc._msdcs.$dnsserver' query");
		return $ERR_NO_PDC_ENTRY_IN_DNS;

	}
	# no authoritative answers returned by DNS server
	Log("No authoritative answers returned by DNS server for '_ldap._tcp.pdc._msdcs.$dnsserver' query");
	return $ERR_NO_AUTHORITATIVE_PDC_QUERY_ANSWERS_FROM_DNS;
}


sub check_dns_status {

	`/etc/init.d/novell-named status`;
	if ($?) {

		Log("novell-named not running yet, starting it.");

		`/etc/init.d/namcd status`;
		if ($?) {
			Log("namcd not running, starting it");
			`/etc/init.d/namcd start`;
		}

		`/etc/init.d/novell-xregd status`;
		if ($?) {
			Log("novell-xregd not running, starting it");
			`/etc/init.d/novell-xregd start`;
		}

		`/etc/init.d/novell-named start`;
	}

}

# Deleting the entries existing already
########################################
sub update_dns_entries {

	my $return_val = 0;

	check_dns_status();

	# Get the list of dns entries and their count
	$return_val = get_existing_dns_server_entries();
	if ($return_val < 0) {
		Log(ERR, "The list of servers couldn't be obtained, exiting ...");
	}

	# get the count of the of Existing_PDC_Masters records in DNS database
	my $Existing_PDC_Master_entries_count = @Existing_PDC_Masters;

	if ($Existing_PDC_Master_entries_count == 0) {
		Log("DNS has proper PDC SRV records");
		return 0;
	}
	
	# create the GSSAPI contexts
	my $name = GSSAPI::Name->new;
	my $context = GSSAPI::Context->new;

	# Set the nameserver
	$nameserver = set_nameserver("$dnsserver");


	$status = $name->import($name, "DNS/" . $domain_server_name . "@" . uc($domain));
	#$status || gss_exit("unable to get the name imported..", $status);
	if(($status->major != 0) or ($status->minor != 0))
	{
		# error encountered
		Log("Error encountered while importing the name");
		$return_val = gss_exit($status);
	}


	# specify the GSS flags
	my $gss_flags = GSS_C_REPLAY_FLAG | GSS_C_MUTUAL_FLAG |
		GSS_C_SEQUENCE_FLAG | GSS_C_CONF_FLAG |
		GSS_C_INTEG_FLAG | GSS_C_DELEG_FLAG;


	# acquire credentails
	my ($cred, $oidset, $time) = "";

	$status = GSSAPI::Cred::acquire_cred(undef, 120, undef, GSS_C_INITIATE,
			$cred, $oidset, $time);
	#$status || gss_exit("unable to acquire credentials..", $status);

	if(($status->major != 0) or ($status->minor != 0))
	{
		# error encountered
		Log("Error encountered while acquiring credentials");
		$return_val = $status;
		goto out;
	}

	print "cedentials acquired\n" if($debug);

	# call gss_init_sec_context
	my $token = "";

	my $token2 = "";

	$status = $context->init($cred, $name, undef, $gss_flags,
			0, undef, "", undef, $token, undef, undef);

	#  here, the permitted gss major status values are 0 and 1
	if(($status->major != 1) or ($status->minor != 0))
	{
		if($status->major != 0)
		{
			# error encountered
			Log("   Error encountered during the first call to gss_init_sec_context for establishing the secure context");
			$return_val = $status;
			goto out;
		}
	}

	print "first call to gss_init_sec_context done\n" if($debug);

	# check the Major status for GSS_S_CONTINUE_NEEDED and length of $token
	# if (( Major status == 'GSS_S_CONTINUE_NEEDED') || (length($token)))
	# then we need to send the token back to the server.

	my $counter = 1;
	my $reply = "";

	do
	{
		$counter++;

		my $gss_query = Net::DNS::Packet->new("$key_name", "TKEY", "IN");

		if(!defined($gss_query))
		{
			Log("Error encountered while creating the DNS RR packet handler");
			$return_val = $ERR_DNS_RR_PACKET_HANDLE_CREATION_FAILED;
			goto out;
		}

		# create the DNS RR packet
		print "calling RR new\n" if($debug);

		$a = Net::DNS::RR->new(
				Name    => "$key_name",
				Type    => "TKEY",
				TTL     => 0,
				Class   => "ANY",
				mode => 3,
				algorithm => "gss-tsig",
				inception => time,
				expiration => time + 24*60*60,
				key => "$token",
				other_data => "",
				);
		$gss_query->push("additional", $a);

		# set the recusrion desired to 0 by default it is set to 1
		$gss_query->header->rd(0);

		# print the RR data format
		print "RR data format\n" if($debug);
		print "----------------\n" if($debug);
		$a->print if($debug);
		print "===============\n" if($debug);

		#print the DNS packt format
		print "DNS packet format\n" if($debug);
		print "------------------\n" if($debug);

		$gss_query->print if($debug);
		print "==================\n" if($debug);



		# verify the packet before sending it
		if($debug)
		{
			my $z = ($gss_query->zone)[0];

			plan tests => 6;
			ok($gss_query,                                'new() returned packet');  #1
			is($gss_query->header->opcode, 'QUERY',       'header opcode correct');  #2
			is($z->zname,  $key_name,                      'zname correct');          #3
			is($z->zclass, "IN",                       'zclass correct');         #4
			is($z->ztype,  'TKEY',                     'ztype correct');          #5
			is($gss_query->header->rd, '0',         'header rd correct');   #6
		}

		#sending the RR query packet to DNS server
		$reply = $nameserver->send($gss_query);

		if (!defined($reply) || $reply->header->{'rcode'} ne 'NOERROR')
		{
			$return_val = $reply->header->{'rcode'};
			Log("Error $return_val sending the TKEY packet");
			print "encountered $return_val error\n";
			print "returned packt is:\n" if($debug);
			print "----------------------\n" if($debug);
			$reply->print if($debug);
			print "======================\n" if($debug);
			goto out;
		}

		# printing the returned packet
		print "The DNS server response is:\n" if($debug);
		print "-------------------------------\n" if($debug);
		$reply->print if($debug);
		print "======================\n" if($debug);


		$token2 = ($reply->answer)[0]->{"key"};

                # calling gss_init_sec_context again
		$status = $context->init($cred, $name, undef, $gss_flags,
				0, undef, $token2, undef, $token,
				undef, undef);

		if(($status->major != 0) or ($status->minor != 0))
		{
			# error encountered
			$return_val = gss_exit($status);
			Log("Error $return_val during GSS Handshake");
			goto out;
		}

	} while (($status->major eq GSS_S_CONTINUE_NEEDED) || (defined($token)));

	if(($status->major eq GSS_S_COMPLETE) && !(defined($token)))
	{
		print "contect can be used for Dynamic DNS update\n" if($debug);

		print "Negotiated TKEY $key_name\n" if($debug);

		# construct a signed update
		my $update = "";

		$update = Net::DNS::Update->new("$domain");

		$update = Net::DNS::Update->new("$domain");

		$update->push("pre", yxdomain("$domain"));

		$update->push("pre", yxrrset("_ldap._tcp.pdc._msdcs.$domain SRV "));

		# parse the @Existing_PDC_Masters array and add the existing
		# SRV records for deletion
		if($Existing_PDC_Master_entries_count)
		{
			# there does exists PDC Master entries in the DNS data base
			my $deletion_entry_count = 0;
			for($deletion_entry_count = 0;$deletion_entry_count<$Existing_PDC_Master_entries_count;$deletion_entry_count++)
			{
				# create an rr_del update packet entry for each array entry
				$update->push("update", rr_del("_ldap._tcp.pdc._msdcs.$domain SRV $Existing_PDC_Masters_record_values{$Existing_PDC_Masters[$deletion_entry_count]}->{'priority'} $Existing_PDC_Masters_record_values{$Existing_PDC_Masters[$deletion_entry_count]}->{'weight'} $Existing_PDC_Masters_record_values{$Existing_PDC_Masters[$deletion_entry_count]}->{'port'} $Existing_PDC_Masters[$deletion_entry_count]"));
			}
		}

		my $sig = Net::DNS::RR->new(Name    => $key_name,
				Type    => "TSIG",
				TTL     => 0,
				Class   => "ANY",
				Algorithm => "gss-tsig",
				Time_Signed => time,
				Fudge => 36000,
				Mac_Size => 0,
				Mac => "",
				Key => $context,
				Sign_Func => \&gss_sign,
				Other_Len => 0,
				Other_Data => "",
				Error => 0,
				mode => 3,
				);

		$update->push("additional", $sig);

		# print the DNS packet being sent to DNS server to update SRV record
		print "SRV record update packet\n" if($debug);
		print "===========================================\n" if($debug);
		$update->print if($debug);
		print "=============================================\n" if($debug);


		# send the dynamic update
		my $update_reply = $nameserver->send($update);

		if (! defined($update_reply))
		{
			Log("No reply from DNS server for dynamic update packet sent");
			$return_val = $ERR_NO_REPLY_TO_DYNAMIC_UPDATE;
			goto out;
		}

		# make sure it worked
		$return_val = $update_reply->header->{"rcode"};
		print "Update gave rcode $return_val\n" if($debug);

		if ($return_val ne 'NOERROR')
		{
			# printing the returned packet
			print "The DNS server response is:\n" if($debug);
			print "-------------------------------\n" if($debug);
			$update_reply->print if($debug);
			print "====================================\n" if($debug);
			Log("Dynamic DNS Update failed, [$return_val]");
			goto out;
		} elsif($return_val eq 'NOERROR')
		{
			Log("Successfully updated the DNS");
			$return_val = 0;
		}

	}

out:
	return $return_val;
}

#Function to update the gPCFileSysPath Attribute
sub update_attribute {
        my $ldapmodify = registry::getReg("LDAPMODIFY");
        my $gPCFileSysPath_value = registry::getReg("GPODCFilePath").'\{31B2F340-016D-11D2-945F-00C04FB984F9}';
        my $dn = registry::getReg("Domain NC");
        my $lib = registry::getReg("_LIB");
        my $tempLdif = "/tmp/temp.ldif";

        print "ldapmodify : $ldapmodify \n" if ($debug);
        print "gPCFileSysPath : $gPCFileSysPath_value \n" if ($debug);

        open FILE, ">$tempLdif" or die ("failed to open the file");
        print FILE "dn:CN={31B2F340-016D-11D2-945F-00C04FB984F9},CN=Policies,CN=System,$dn\n";
        print FILE "changetype:modify\n";
        print FILE "gPCFileSysPath:$gPCFileSysPath_value\n";
        close(FILE);

        print "Executing ... SASL_PATH=/opt/novell/xad/$lib/sasl2 $ldapmodify -f $tempLdif \n" if ($debug);
        `SASL_PATH=/opt/novell/xad/$lib/sasl2 $ldapmodify -Y GSSAPI -f $tempLdif`;
        if ($?) {
                unlink($tempLdif);
		Log(ERR, "Failed to update entry of gPCFileSysPath");
        }
    
	unlink($tempLdif);

	open FILE, ">$tempLdif" or die ("failed to open the file");
	print FILE "dn: $dn\n";
	print FILE "changetype:modify\n";
	print FILE "add:ACL\n";
	print FILE "ACL:3#subtree#[Root]#gPOptions\n";
	print FILE "ACL:3#subtree#[Root]#tokenGroups\n";
	close(FILE);

	print "Executing ... SASL_PATH=/opt/novell/xad/$lib/sasl2 $ldapmodify -Y GSSAPI -f $tempLdif \n" if ($debug);
	my @output=`SASL_PATH=/opt/novell/xad/$lib/sasl2 $ldapmodify -Y GSSAPI -f $tempLdif 2>&1`;
	if ($?) {
		unlink($tempLdif);
		if (grep (/-614/, @output) ) {
			Log("The ACL for gPOptions already present");
		}
		else {
			Log(ERR, "Failed to update entry of gPOptions");
		}
	}
    
        unlink($tempLdif);
        Log("Successfully updated the attribute");
}

sub remove_gpo2nmas_from_crontab
{

	my $tempfile = mktemp("/tmp/temp_crontab.XXXXXXXX");
	my $tempfile2 = mktemp("/tmp/temp_crontab.XXXXXXXX");

	system("/usr/bin/crontab -l > $tempfile");
	system("grep -vi gpo2nmas $tempfile > $tempfile2");
	system("/usr/bin/crontab $tempfile2");

}

# Update the password policy on Domain Controller and Computer
# container as "Default Password Policy"

sub update_password_policy {
	my $temp_ldif = mktemp ("/tmp/tmp.XXXXXXXX");
	my $mapped_computers_dn = registry::getReg("Mapped Computers Container");
	my $mapped_dc_dn = registry::getReg("Mapped Domain Controllers Container");
	my $mapped_nc = registry::getReg("Mapped Domain NC");
	my $lib = registry::getReg("_LIB");
	my $ldapmodify = "/usr/bin/ldapmodify";

	open (TEMPLDIF, ">$temp_ldif") or die ("Failed to open the file");
	print TEMPLDIF "dn: $mapped_computers_dn\n";
	print TEMPLDIF "changetype: modify\n";
	print TEMPLDIF "add: objectClass\n";
	print TEMPLDIF "objectClass: nspmPasswordAux\n";
	close TEMPLDIF;
	
	my @output=`SASL_PATH=/opt/novell/xad/$lib/sasl2 $ldapmodify -Y GSSAPI -f $temp_ldif 2>&1`;
	if ($?) {
		if (grep(/-614/,@output)) {
			Log ("Computers container already had nspmPasswordAux class");
		}else {
			Log ("Updating Computers Container with nspmPasswordAux class failed: [@output]");
		}
	}else {
		Log ("Computers container was not having nspmPasswordAux Class");
	}

		
	unlink ($temp_ldif);
	$temp_ldif = mktemp ("/tmp/tmp.XXXXXXXX");
	open (TEMPLDIF, ">$temp_ldif") or die ("failed to open the file");
	print TEMPLDIF "dn:$mapped_computers_dn\n";
	print TEMPLDIF "changetype:modify\n";
	print TEMPLDIF "nspmPasswordpolicyDN: cn=Default Password Policy,cn=Password Policies,cn=System,$mapped_nc\n";
	print TEMPLDIF "\ndn:$mapped_dc_dn\n"; # Domain controller dn
	print TEMPLDIF "changetype:modify\n";
	print TEMPLDIF "nspmPasswordpolicyDN: cn=Default Password Policy,cn=Password Policies,cn=System,$mapped_nc\n";
	close(TEMPLDIF);

	@output=`SASL_PATH=/opt/novell/xad/$lib/sasl2 $ldapmodify -Y GSSAPI -f $temp_ldif 2>&1`;
	if ($?) {
		Log ("Failed to update entry of password policy: [@output]");
	}
	else {
		Log ("Successfully updated the password policy on domain controller and computers container");
	}

	unlink ($temp_ldif);
}
# Main subroutine
#######################
sub upgrade_sp2 () {
	my $type = registry::getReg('INSTALL_TYPE');
	my $is_replica = registry::getReg("IS_REPLICA");
	my $live_local_statedir = registry::getReg("LIVE_LOCALSTATEDIR");
	my $inst = 0;
	my $administrator_uid = 0;
	my $domain_admins_gid = 0;
	my $install_type = registry::getReg("INSTALL_TYPE");


	if ($is_replica eq "TRUE") {
		$inst = adc_install->new();
	}
	else {
		if ($type eq "frd") {
			$inst = frd_install->new();
		}
		elsif ($type eq "child") {
			$inst = child_install->new();
		}
	}

	$administrator_uid = registry::getReg("ADMINISTRATOR_UID");
	$domain_admins_gid = registry::getReg("DOMAIN_ADMINS_GID");

	# Changes on both receiver and sender
	util::process_krb5_config();
	util::process_ssh_config();
	util::process_sshd_config();
	util::process_rsyncd_config();

	my $gp_creator_owner_gid = registry::getReg("GP_CREATOR_OWNER_GID");

	`setfacl -m g:$domain_admins_gid:rwx $live_local_statedir/sysvol`;
	`getfacl --access $live_local_statedir/sysvol | setfacl -d -M- $live_local_statedir/sysvol`;


	`setfacl -m g:$gp_creator_owner_gid:rwx $live_local_statedir/sysvol/domain/Policies`;
	`chmod +t $live_local_statedir/sysvol/domain/Policies`;

	`chown -R $administrator_uid:$domain_admins_gid $live_local_statedir/sysvol`;

	Log("Creating $live_local_statedir/msdfs");
	mkpath ("$live_local_statedir/msdfs");

	Log("Changing ownership for $live_local_statedir/msdfs to $administrator_uid:$domain_admins_gid");
	`chown -R $administrator_uid:$domain_admins_gid $live_local_statedir/msdfs`;

	# Restarting the services 
	util::restart_services();

	my $pdc_master = is_pdc_master();

	if ($pdc_master == -1) {
		Log(ERR, "Cannot contact LDAP server, exiting ...");
	}

        if ($is_replica eq "FALSE") {
            update_attribute();
        }

	update_samba_config($pdc_master);
	if ($pdc_master == 1) { 
		remove_gpo2nmas_from_crontab();
	}
	else { 
		my $dns_root = registry::getReg("DNSROOT");
		my $pdcmaster_fqdn = registry::getReg("PDCMASTERFQDN");

		$inst->stage_sysvol();

		Log("Creating msdfs Link $live_local_statedir/msdfs/");
		`(cd $live_local_statedir/msdfs; ln -sf "msdfs:$localhost_fqdn\\sysvol-msdfs\\$dns_root,$pdcmaster_fqdn\\sysvol\\$dns_root" $live_local_statedir/msdfs/$dns_root)`;
		Log("Changing ownership for $live_local_statedir/msdfs to $administrator_uid:$domain_admins_gid");
		`chown -R $administrator_uid:$domain_admins_gid $live_local_statedir/msdfs`;
		`chown -R $administrator_uid:$domain_admins_gid $live_local_statedir/sysvol`;

	}

	# Update the password policy for computer and domain controller container.
	update_password_policy();

	my $rc;
	Log("Updating the crontab entry to add gpo2sync");
	$rc = system("/opt/novell/xad/share/dcinit/provisionTools.sh  update-crontab-GPO2NMAS");
	if ($rc != 0) {
		Log("Could not update the crontab for gposync");
	}
	$rc = 0;


	# Remove the entries if it is not the local host from the pdc entries;
	update_dns_entries();

	# Starting the sysvolsync utility 
	Log("Updating the crontab entry for sysvolsync utility");
	$rc = system("/opt/novell/xad/share/dcinit/provisionTools.sh update-crontab-SYSVOLSYNC");
	if ($rc != 0) {
		Log(ERR, "Could not update the cron entry for sysvol sync, exiting ...");
	}

}

1;
