#!/usr/bin/perl -I. -I.. -I/opt/novell/xad/lib64/perl -I/opt/novell/xad/lib/perl
################################################################################
# DSfW Upgrade script for upgrading from sp2 to sp3
#
# See the file LICENSE for redistribution information.
#
# Copyright (c) 2001-2010
#	Novell, Inc.  All rights reserved.
################################################################################
package upgrade2sp3;
$ENV{'PATH'} = "/opt/novell/xad/share/dcinit:$ENV{'PATH'}";
use v5.10;
use strict;
use warnings;
use XAD::registry;
use Logger qw(:DEFAULT);
use File::Copy;
use File::Temp;
use NDS::stage;
use Install::frd_install;
use Install::child_install;
use Install::adc_install;
use util;
use Config::IniFiles;
no warnings 'experimental::smartmatch';

my $debug = 0;
my $LDAP_ERR_DUP = 20;
my $LDAP_ERR_NO_SUCH_ATTR = 16;
my $lib = registry::getReg("_LIB");
my $ldapmodify = registry::getReg("LDAPMODIFY");

# Check that is a pdc master 
sub is_pdc_master() {
	my $domain_name = registry::getReg("DNS Root");
	my $dig_output = `/usr/bin/dig -t SRV _ldap._tcp.pdc._msdcs.$domain_name +short`;
	chomp($dig_output);
	my @split_output = split(' ', $dig_output);
	my $pdc_dns_hostname = lc($split_output[3]);
	my $hostname = `/bin/hostname`;
	chomp($hostname);
	my $local_dns_hostname = lc("$hostname.$domain_name.");
	if ($pdc_dns_hostname eq $local_dns_hostname) {
		Log ("The local server [$local_dns_hostname] is a pdc master");
		return 1;
	}
	return 0;
}

sub can_samify () {
	my $adminname = registry::getReg("Administrator DN");
	my $counter = 0;
	my $retry_count = 5;
	my $ldapadminname = registry::getReg("LDAPADMINNAME");
	my $rc = 0;

	do {
		$rc = 0;
		if ($counter > 0) {
			Log (INFO, "Retrying to samify administrator object, retry count:$counter");
			sleep (10);
		}
		`echo -e "dn:$adminname\nchangetype:modify\nadd: Description\nDescription: xxxxxx\n" | /usr/bin/ldapmodify -Q -Y GSSAPI`;
		`echo -e "dn:$adminname\nchangetype:modify\ndelete: Description\nDescription: xxxxxx\n" | /usr/bin/ldapmodify -Q -Y GSSAPI`;
		sleep (5);
		`/usr/bin/ldapsearch -Q -Y GSSAPI -s base -LLL -b $adminname uniquedomainid | grep -i uniquedomainid`;
		if ($? != 0) {
			$rc = $?;
			#The samification did not happen, update the domain now
			util::send_domain_update_now_signal ();

		}
		$counter ++;
	}while ($rc != 0 and $counter < $retry_count);

	if ($rc != 0) {
		Log("The samification cannot happen, try it later");
	}
	return $rc;
}

sub samify_all_objects () {

	my $admin_name = registry::getReg("LDAPADMINNAME");
	my $adm_principal = registry::getReg("ADM_PRINCIPAL");
	my $default_root_domain = registry::getReg("DEFAULTROOTDOMAIN");
	my $ldap_search = registry::getReg("LDAPSEARCH");
	my $ldap_search_flags = registry::getReg("LDAPSEARCH_FLAGS");
	my $livedsstatedir = registry::getReg("LIVE_DSSTATEDIR");
	my $template_dir = registry::getReg("TEMPLATEDIR");
	my $rc;
        my $violate_domain_boundary_OID = "2.16.840.1.113719.1.513.4.6";
	my $search_partition_boundary_OID = "2.16.840.1.113719.1.513.4.8";
	eval {
		Log("Adding UniqueDomainID to the existing objects");

		`$ldap_search -x -s sub -LLL -b "$default_root_domain" "(objectclass=*)" -E $violate_domain_boundary_OID -E $violate_domain_boundary_OID  dn | sed ' /^ / {; H; d; }; /^ /! {; x; s/\\n //g; s/\\n //g ;}; ' | grep -i "^dn:" > $livedsstatedir/samify/intermediate_file1`;
		`awk '{printf "%s", \$0}' $livedsstatedir/samify/intermediate_file1 > $livedsstatedir/samify/intermediate_file2`;
		`sed -i -e 's/dn: /\\ndn: /g' $livedsstatedir/samify/intermediate_file2`;
		`sed -i -e '/^\\s*\$/d' $livedsstatedir/samify/intermediate_file2`;

		`makeLdif.pl -t $template_dir/samify_user.ldif -a $livedsstatedir/samify/intermediate_file2 -o $livedsstatedir/samify/final_usr.ldif`;

		my @output = `SASL_PATH=/opt/novell/xad/$lib/sasl2 LDAPCONF=/etc/opt/novell/xad/openldap/ldap.conf $ldapmodify -Q -Y GSSAPI -f $livedsstatedir/samify/final_usr.ldif 2>&1`;
		my $error = $? >> 8;
		if ($error and $error != $LDAP_ERR_DUP) {
			die("Could not samify the objects: $error: [@output]");
		}
	};

	if ($@) { Log(ERR, $@) }
	Log("All the objects under this domain are now samified");

}

# Check the value for the attribute on a given DN is already present
sub is_already_present($$$) {
	my $dn = shift;
	my $attribute = shift;
	my $value = shift;

	my @output = `SASL_PATH=/opt/novell/xad/$lib/sasl2/ /usr/bin/ldapsearch -Y GSSAPI -s base -b $dn $attribute -LLL -Q 2>&1`;
	if ($?) {
		Log ("The ldap search failed to find [$attribute=$value] in $dn");
		return 1;
	}

	$value =~ s/([\[\]])/\\$1/g;

	my @result = grep (/^$attribute: $value$/i, @output); 
	if (@result) {
		Log ("Found [$attribute=$value] in $dn");
		return 0;
	}

	return 1;
}

# Update the xad-flags in objectclass and xad-domain-flags attribute
sub xadFlags_and_xadDomainFlag_update() {
	my $mapped_nc = registry::getReg("Mapped Domain NC");
	my $flag = registry::getReg ("xad-domain-flag");
	my $is_replica = registry::getReg ("IS_REPLICA");
	my @output = {};

	if (is_already_present ($mapped_nc, "objectclass", "xadFlags") != 0) {
		my $temp_ldif = mktemp ("/tmp/tmp.XXXXXXXX");
		open FILE, ">$temp_ldif" or die ("failed to open the file");
		print FILE "dn:$mapped_nc\n";
		print FILE "changetype: modify\n";
		print FILE "add: objectclass\n";
		print FILE "objectclass: xadFlags\n";
		close FILE;

		Log("Adding xadFlags auxiliary objectclass to domain root");
		eval {
			Log("Executing .. SASL_PATH=/opt/novell/xad/$lib/sasl2 $ldapmodify -Q -Y GSSAPI -f $temp_ldif 2>&1");
			@output = `SASL_PATH=/opt/novell/xad/$lib/sasl2 LDAPCONF=/etc/opt/novell/xad/openldap/ldap.conf $ldapmodify -Q -Y GSSAPI -f $temp_ldif 2>&1`;
			my $error = $? >> 8;
			if ($error and $error != $LDAP_ERR_DUP) {
				chomp(@output);
				die("Failed to add xadFlags auxiliary objectclass to $mapped_nc: [@output]");
			}
		};
		if ($@) { Log (ERR, $@); }
		unlink($temp_ldif);
	}
	if ($is_replica eq "FALSE") {#Skip it for ADC - It is not generated properly for ADC
		my $temp_ldif = mktemp ("/tmp/tmp.XXXXXXXX");
		open FILE, ">$temp_ldif" or die ("failed to open the file");
		print FILE "dn:$mapped_nc\n";
		print FILE "changetype: modify\n";
		print FILE "replace: xad-Domain-Flag\n";
		print FILE "xad-Domain-Flag: $flag\n";
		close FILE;

		Log ("Adding xad-domain-flags to [$mapped_nc] ");
		eval {
			Log("Executing .. SASL_PATH=/opt/novell/xad/$lib/sasl2 $ldapmodify -Q -Y GSSAPI -f $temp_ldif 2>&1");
			@output = `SASL_PATH=/opt/novell/xad/$lib/sasl2 LDAPCONF=/etc/opt/novell/xad/openldap/ldap.conf $ldapmodify -Q -Y GSSAPI -f $temp_ldif 2>&1`;
			my $error = $? >> 8;
			if ($error and $error != $LDAP_ERR_DUP) {
				chomp(@output);
				die("Failed to update xad-Domain-Flag: $flag on $mapped_nc: [@output]");
			}
		};
		if ($@) { Log (ERR, $@); }
		unlink ($temp_ldif);
	}
}

# Update the cross ref dn with domainPartitionList
sub crossRef_update() {
	my $dn = registry::getReg("CrossRef_DN");
	my $mapped_domain_nc = registry::getReg("Mapped Domain NC");

	if (is_already_present($dn, "domainPartitionList", $mapped_domain_nc) == 0) {
		Log("The crossRef DN already has domainPartitionList populated with $mapped_domain_nc");
		return;
	}

	my $temp_ldif = mktemp ("/tmp/tmp.XXXXXXXX");
	open FILE, ">$temp_ldif" or die("failed to open the file");
	print FILE "dn: $dn\n";
	print FILE "changeType: modify\n";
	print FILE "add: domainPartitionList\n";
	print FILE "domainPartitionList: $mapped_domain_nc\n";
	close FILE;

	eval {
		Log("Executing ... SASL_PATH=/opt/novell/xad/$lib/sasl2 $ldapmodify -f $temp_ldif");
		my @output = `SASL_PATH=/opt/novell/xad/$lib/sasl2 $ldapmodify -Q -Y GSSAPI -f $temp_ldif 2>&1`;
		my $error = $? >> 8;
		if ($error and $error != $LDAP_ERR_DUP) {
			chomp(@output);
			die("Failed to update [$dn] with domainPartitionList:$error [@output]");
		}
	};
	if ($@) { Log(ERR, $@); }
	unlink ($temp_ldif);

	Log("Updated the domainPartitionList for mapped partition on CrossRef object");
}

# Update the unique domain id
sub uniqueDomainId_update() {
	my $rc = 0;
	my $templatedir = registry::getReg("TEMPLATEDIR");
	my $LIVE_DSSTATEDIR = registry::getReg("LIVE_DSSTATEDIR");
	my @output = {};

	# Checking the schema extension for uniqueDomainID attribute before setting it.
	my $counter = 0;
	do {
		$rc = system("/opt/novell/eDirectory/bin/ldapsearch -s base -b cn=schema attributetypes=uniquedomainid -LLL >&/dev/null");
		if ($rc) {
			Log ("The schema update for unique domain ID is not synched yet, retrying ...");
			sleep(5);
		}
		$counter++;
	} while($rc != 0 and $counter < 10);

	my $type = registry::getReg("INSTALL_TYPE");
	if ($type eq "frd") {
		registry::setReg("UniqueDomainID", 1049076);
	}
	else {
		registry::setReg("UniqueDomainID", registry::getReg("TRUSTPOSIXOFFSET"));
	}
	
	my $value = registry::getReg("UniqueDomainID");
	my $domain_nc = registry::getReg("Domain NC");

	if (is_already_present($domain_nc, "UniqueDomainID", $value) == 0) {
		Log("The Domain Root is already updated with UniqueDomainID attribute");
		return ;
	}


	stage::gen_ldif("$templatedir/domain-uniquedomainid.ini", '>', "$LIVE_DSSTATEDIR/domain/domain-uniquedomainid.ldif", "--defaultrootdomainobject");
	eval {
		Log("Executing .. LDAPCONF=/etc/opt/novell/xad/openldap/ldap.conf $ldapmodify -Q -Y EXTERNAL -f $LIVE_DSSTATEDIR/domain/domain-uniquedomainid.ldif 2>&1");
		@output = `LDAPCONF=/etc/opt/novell/xad/openldap/ldap.conf $ldapmodify -Q -Y EXTERNAL -f $LIVE_DSSTATEDIR/domain/domain-uniquedomainid.ldif 2>&1`;
		if ($?) {
			chomp(@output);
			die("Failed to upload uniqueDomainID on current server: [@output]");
		}
	};

	if ($@) { Log (ERR, $@); }

	Log ("Updated the uniqueDomainID on the domain Root");
}

# Update the ACL for unique domain ID on mapped domain nc
sub acl_for_uniqueDomainID_update {
	my $mapped_domain_nc = registry::getReg("Mapped Domain NC");
	my $temp_ldif = mktemp ("/tmp/tmp.XXXXXXXX");
	my $counter = 0;
	my $error = 0;

	if (is_already_present($mapped_domain_nc, "ACL", "3#subtree#[Root]#uniqueDomainId") == 0) {
		Log ("The ACL entry for uniqueDomainID is already present on $mapped_domain_nc");
		return;
	}

	# Updating ACL for uniqueDomainID
	open FILE, ">$temp_ldif" or die("failed to open the file");
	print FILE "dn: $mapped_domain_nc\n";
	print FILE "changetype: modify\n";
	print FILE "add: ACL\n";
	print FILE "ACL: 3#subtree#[Root]#uniqueDomainId\n";	
	close FILE;

	eval {
		do {
			Log("Executing ... SASL_PATH=/opt/novell/xad/$lib/sasl2 $ldapmodify -f $temp_ldif");
			my @output = `SASL_PATH=/opt/novell/xad/$lib/sasl2 $ldapmodify -Q -Y GSSAPI -f $temp_ldif 2>&1`;
			$error = $? >> 8;
			if ($error and $error != $LDAP_ERR_DUP) {
			    chomp(@output);
			    Log("Failed to update $mapped_domain_nc with ACL of uniqueDomainID: $error - [@output], retrying...");
			    $counter ++;
			    sleep (20);
			}
		} while ($error && $error != $LDAP_ERR_DUP && $counter < 10);

		if ($error && $error == $LDAP_ERR_NO_SUCH_ATTR) { #Schema is not in sync
		    die ("Failed to update the acl, the schema may not be synchronized yet. \n".
			 "Synchronize the schema manually and re-attempt the upgrade by calling upgrade_dsfw.pl script.");
		}
		elsif ($error && $error != $LDAP_ERR_DUP) { #Some other error 
		    die ("Failed to update the acl, aborting. Call upgrade_dsfw.pl post upgrade");
		}
		else { #acl update succeeded 
		    Log ("Updated domain with ACL assignment for uniquedomainid");
		}
	};

	if ($@) { Log (ERR, $@); }

	unlink($temp_ldif);
}

# Upgrade Edir to latest sp3 
sub upgrade_dsfw_domain() {
	my $temp_ldif = mktemp ("/tmp/tmp.XXXXXXXX");
	my $dn = registry::getReg("Domain NC");
	my $LIVE_DSSTATEDIR = registry::getReg("LIVE_DSSTATEDIR");
	my $templatedir = registry::getReg("TEMPLATEDIR");
	my $mapped_domain_nc = registry::getReg("Mapped Domain NC");
	my @output ;
	my $rc = 0;
	my $counter = 10;
	my $error = 0;
	my $confdir = "/etc/opt/novell/eDirectory/conf";
	my $bindir = "/opt/novell/eDirectory/bin";
	my $upgrade_acl_ini = "/opt/novell/xad/share/dcinit/templates/nds-domain-upgrade-acls.ini";

	eval {
		my @password_policy_dn = ("Domain", "Default", "Builtin Domain");
		open FILE, ">$temp_ldif" or die ("failed to open the file");
		for (my $i = 0 ; $i <= $#password_policy_dn ; $i++) {
			print FILE "dn:CN=$password_policy_dn[$i] Password Policy,CN=Password Policies,CN=System,$dn\n";
			print FILE "replace: nspmSpecialCharactersAllowed\n";
			print FILE "nspmSpecialCharactersAllowed: TRUE\n";
			print FILE "-\n";
			print FILE "replace: nspmNumericCharactersAllowed\n";
			print FILE "nspmNumericCharactersAllowed: TRUE\n";
			if ($i <= $#password_policy_dn -1 ) {
				print FILE "\n";
			}
		}
		close(FILE);

		Log("Executing ... SASL_PATH=/opt/novell/xad/$lib/sasl2 $ldapmodify -f $temp_ldif");
		@output = `SASL_PATH=/opt/novell/xad/$lib/sasl2 $ldapmodify -Q -Y GSSAPI -f $temp_ldif 2>&1`;
		if ($?) {
			chomp(@output);
			die("Failed to update entry for Password Policy: [@output]");
		}
		Log("Updated the password policy ");
		unlink($temp_ldif);

		# Waiting for LDAP server to be ready for further operations
		do {
			`/usr/bin/ldapsearch -s base -b "" -x >&/dev/null`;
			if ($? != 0) {
				Log ("LDAP server not started yet, retrying ..");
				sleep (2);
			}
			$counter ++;
		} while ($? != 0 && $counter < 10);

		# Update the ACL entry on the domainRoot for uniqueDomainID attribute.
	        acl_for_uniqueDomainID_update();

		# Adding external reference to the ncp server object for domainPartitionList
		my $default_container = `/opt/novell/xad/share/dcinit/provisionTools.sh get-default-container`;
		if ($? == 0) {
			chomp($default_container);
			registry::setReg("DEFAULT_CONTAINER", $default_container);
		}
		else {
			registry::setReg("DEFAULT_CONTAINER", "ou=OESSystemObjects");
		}
 
		$dn = "cn=".registry::getReg("DNS Unqualified Host Name").
				",$default_container,$mapped_domain_nc";
					
		open FILE, ">$temp_ldif" or die("failed to open the file");
		print FILE "dn: $dn\n";
		print FILE "changetype: modify\n";
		print FILE "add: CachedAttrsOnExtRefs\n";
		print FILE "CachedAttrsOnExtRefs: domainPartitionList\n";
		close FILE;

		Log("Executing ... SASL_PATH=/opt/novell/xad/$lib/sasl2 $ldapmodify -f $temp_ldif");
		@output = `SASL_PATH=/opt/novell/xad/$lib/sasl2 $ldapmodify -Q -Y GSSAPI -f $temp_ldif 2>&1`;
		$error = $? >> 8;
		if ($error and $error != $LDAP_ERR_DUP) {
			chomp(@output);
			die("Failed to update $dn with domainPartitionList as cachedAttrOnExtRef:$error [@output]");
		}

		unlink($temp_ldif);
		Log("Updated domainpartitionList to be cached on ExtRef on $dn");

		# Updating the crossRef object with domain partition list only on name mapped and non-replica
		crossRef_update();

		# Update the ServicePrincipalName attribute on the DC object with ldap/host/netbios value to support Microsoft SharePoint
		util::update_spn();

		# Add the crossrefreference to the ncp server object
		$registry::DEFAULTMAP{"NCPDN"} = $dn; 
		stage::gen_ldif("$templatedir/config-bl.ini", '>', "$LIVE_DSSTATEDIR/config/crossrefreference.ldif", "--ncpdn");
		Log("Executing .. SASL_PATH=/opt/novell/xad/$lib/sasl2 $ldapmodify -Q -Y GSSAPI -f $LIVE_DSSTATEDIR/config/crossrefreference.ldif");
		@output = `SASL_PATH=/opt/novell/xad/$lib/sasl2 $ldapmodify -Q -Y GSSAPI -f $LIVE_DSSTATEDIR/config/crossrefreference.ldif 2>&1`;
		if ($?) {
			chomp(@output);
			die("Failed to update crossrefreference for current server: [@output]");
		}
		Log("Updated crossRefReference on NCPDN $dn");

		# xad-flags and xad-domain-flag updates
		xadFlags_and_xadDomainFlag_update();

		# Setting uniequeDomainID on domainRoot
		uniqueDomainId_update();

		# Samify all the objects only for name mapped and non adc scenarios
		if (can_samify() == 0) {
			samify_all_objects();
		}
	        # Update the five attributes related to Password Policy (Xenserver 5.6 issue)
		# Check for pre-existing values and if present OR the existing values with the new ones  
                open FILE, ">$upgrade_acl_ini" or die ("failed to open the file $upgrade_acl_ini");
                print FILE "[DEFAULTROOTDOMAIN]\n";
                print FILE "CHILD=Domain NC ACLs and Policy\n";
                print FILE "[Domain NC ACLs and Policy]\n";
                print FILE "RDN-Of-Object=DEFAULTROOTDOMAIN\n";
                print FILE "changetype=modify\n";
                print FILE "add=ACL\n";
                print FILE "ACL=3#subtree#CN=Domain Computers,CN=users,$mapped_domain_nc#PasswordExpirationInterval\n";
                print FILE "ACL=3#subtree#CN=Domain Computers,CN=users,$mapped_domain_nc#PasswordMinimumLength\n";
                print FILE "ACL=3#subtree#CN=Domain Computers,CN=users,$mapped_domain_nc#nspmConfigurationOptions\n";
                print FILE "ACL=3#subtree#CN=Domain Computers,CN=users,$mapped_domain_nc#nspmMinPasswordLifetime\n";
                print FILE "ACL=3#subtree#CN=Domain Computers,CN=users,$mapped_domain_nc#pwdInHistory\n";
                print FILE "ACL=3#subtree#[Root]#ms-net-ieee-80211-GP-PolicyData\n";
                print FILE "ACL=3#subtree#[Root]#ms-net-ieee-80211-GP-PolicyGUID\n";
                close(FILE);

                `ComputeEffectiveAcls.pl -n $mapped_domain_nc`;
		`genLdif.pl /tmp/effective_acls --defaultrootdomain > $LIVE_DSSTATEDIR/domain/nds-domain-upgrade-acls.ldif`;
		`rm -f /tmp/effective_acls`;
		Log("Executing ... SASL_PATH=/opt/novell/xad/$lib/sasl2 $ldapmodify -c -Q -Y GSSAPI -f $LIVE_DSSTATEDIR/domain/nds-domain-upgrade-acls.ldif");
		@output = `SASL_PATH=/opt/novell/xad/$lib/sasl2 $ldapmodify -c -Q -Y GSSAPI -f $LIVE_DSSTATEDIR/domain/nds-domain-upgrade-acls.ldif 2>&1`;
		$error = $? >> 8;
		if ($error and $error != $LDAP_ERR_DUP) {
	        	chomp(@output);
		        Log("Failed to update ACLs on the domain object [@output]");
		}
		Log("Updated the  ACLs on the domain object ");
		

	};

	if ($@) {
		Log (ERR, $@);
	}
}

# Upgrade the smb.conf file to latest sp3 changes
sub upgrade_smb_conf_file() {
	my $smb_file = "/etc/samba/smb.conf";
	my $loop = 2;
	my $section = "";
	my $val = "";

	my $cfg = Config::IniFiles->new (-file=> $smb_file, -nocase=>1);
	if (!defined $cfg) { 
		print "Could not open the file $smb_file \n";
		return -1;
	}
	Log ("Updating $smb_file file with latest entries");
	Log ("Upgrading section - global");
	$val = $cfg->val('global', 'unix extensions');
	if (!defined $val) {
		$cfg->newval ('global', 'unix extensions', 'no');
	}else {
		$cfg->setval ('global', 'unix extensions', 'no');
	}

	
	if (is_pdc_master() == 1)   {
		# We are not on ADC so there is no sysvol-msdfs
		$loop = 1;
	}

	do {
		given($loop){
			when($_ == 1) {$section = 'sysvol'; Log ("Upgrading section - $section");}
			when($_ == 2) {$section = 'sysvol-msdfs'; Log ("Upgrading section - $section");}
			default {last;}
		}

		$loop--;
		$val = $cfg->val($section, 'wide links');
		if (!defined $val) {
			$cfg->newval ($section, 'wide links', 'yes');
		} else {
			$cfg->setval ($section, 'wide links', 'yes');
		}
		$val = $cfg->val($section, 'create mask');
		if (defined $val) {
			$cfg->delval ($section, 'create mask');
		}
		$val = $cfg->val($section, 'directory mask');
		if (!defined $val) {
			$cfg->newval ($section, 'directory mask', '750');
		} else {
			$cfg->setval ($section, 'directory mask', '750');
		}
	} while ($loop > 0);

	$cfg->RewriteConfig();
	`sed -i 's/^[a-zA-Z]/\t&/g' $smb_file`;
	`sed -i 's/=/ = /g' $smb_file`;
	Log ("Updated $smb_file file with latest entries");
	return 0;
}

# Update the keytab file with new entries
sub upgrade_keytab() {
	my $is_replica = registry::getReg("IS_REPLICA");
	my $type = registry::getReg("INSTALL_TYPE");
	my $LIVE_DSSTATEDIR = registry::getReg("LIVE_DSSTATEDIR");
	my $inst = "";

	if ($is_replica eq "true") {
		$inst = adc_install->new();
	}
	else {
		if ($type eq "frd") {
			chdir("/opt/novell/xad/share/dcinit");
			$inst = frd_install->new();
		}
		else {
			$inst = child_install->new();
		}
	}
	
	if( -f "$LIVE_DSSTATEDIR/krb5kdc/krb5.keytab") {
		Log("Moving existing krb5.keytab to $LIVE_DSSTATEDIR/krb5kdc/krb5.keytab.old");
		move ("$LIVE_DSSTATEDIR/krb5kdc/krb5.keytab", "$LIVE_DSSTATEDIR/krb5kdc/krb5.keytab.old");
	}

	# Enable if not done
	if ($is_replica eq "TRUE") {
		$inst->enable_krb5_local_lookup();
	}

	my $tempdir = registry::getReg("TEMPDIR");
	my $stagedir = registry::getReg("STAGEDIR");
	`mkdir -p $tempdir`;
	# generate the keytab entries;
	$inst->bootstrap_machine();
	`/opt/novell/xad/bin/kinit -R >&/dev/null`;
	`rm -rf $stagedir`;
	Log ("Updated keytab entries with latest changes");
}

sub update_attribute {
	my $dn = shift;
	my $attribute = shift;
	my $value = shift;
	my $force = shift;

	my $ldapsearch = "/usr/bin/ldapsearch";

	chomp ($lib);
	my $command = 'SASL_PATH=/opt/novell/xad/' . $lib . '/sasl2/ ' . $ldapsearch . ' -Y GSSAPI -b \'' . $dn . '\' -s base ' . ' -LLL ' . $attribute;

	`$command 2>/dev/null | grep -q '$attribute'`; #gssapi header (2) is sent to /dev/null, the output (1) is grepped
	my $rc = $?;

	if ($rc != 0 or $force eq "TRUE") {
		Log ("\t Updating the value of the attribute $attribute to $value");

		my $tmpfile = mktemp("/tmp/tmp.XXXXXXXXXX");

		open FILE, ">$tmpfile" or die ("failed to open the file $tmpfile for write");
		print FILE "dn: $dn\n";
		print FILE "changetype: modify\n";

		if ($rc == 0) {
		    print FILE "replace: $attribute\n";
		}
		else {
		    print FILE "add: $attribute\n";
		}

		print FILE "$attribute: $value\n";
		close (FILE);

		`SASL_PATH=/opt/novell/xad/$lib/sasl2 $ldapmodify -Y GSSAPI -f $tmpfile 2>/dev/null 1>&2`;
		if ($?) {
		    Log ("Failed to update the attribute $attribute to the value $value in the object $dn");
		}
		else {
		    unlink ($tmpfile);
		}
	}
}

sub update_password_policy {

	my $policydn = shift;
	my $force = shift;
	my %listofattrs = ( nspmLowerAsLastCharacter => 'TRUE', nspmLowerAsFirstCharacter => 'TRUE', 
			    nspmExtendedCharactersAllowed => 'TRUE', nspmCaseSensitive => 'FALSE',
			    nspmSpecialAsLastCharacter => 'TRUE', nspmSpecialAsFirstCharacter => 'TRUE',
			    nspmNumericAsLastCharacter => 'TRUE', nspmNumericAsFirstCharacter => 'TRUE',
			    passwordUniqueRequired => 'FALSE', nspmSpecialCharactersAllowed => 'TRUE',
			    nspmNumericCharactersAllowed => 'TRUE' );

	Log ("Updating the policy $policydn");
	while( my ($key, $value) = each (%listofattrs)) {
		update_attribute ($policydn, $key, $value, $force);
	}

}

sub update_password_policies {
	my $dn = registry::getReg("Domain NC");
	my $ldapsearch = "/usr/bin/ldapsearch";
	my $ldap_base = "cn=Password Policies,cn=System,$dn";
	my $ldap_filter = ' | sed " /^ / {; H; d; }; /^ /! {; x; s/\n //g; }; "';

	chomp ($lib);
	my $command = 'SASL_PATH=/opt/novell/xad/' . $lib . '/sasl2/ ' . $ldapsearch . ' -Y GSSAPI -b \'' . $ldap_base . '\' -s one ' . ' -LLL "(objectclass=nspmPasswordPolicy)" dn '. $ldap_filter;

	Log ("Finding the password policies, Executing: $command");

	my @list = `$command | grep 'dn:' | awk -F 'dn: ' '{print \$2}'`;

	foreach my $policy (@list) {
	    chomp ($policy);
	    if ($policy =~ /^cn=Builtin Domain Password Policy,/ or $policy =~ /^cn=Default Password Policy,/) {
		update_password_policy ($policy, "TRUE"); #Force all default attribute
	    }
	    else {
		update_password_policy ($policy, "FALSE"); #Do not replace if any attribute already exist
	    }
	}
	
}

# Need to restrict the 'other' user access to sysvol 
sub upgrade_sysvol_permission {
	my $DOMAIN_COMPUTERS_GID_OFFSET = 15;
	my $DOMAIN_USERS_GID_OFFSET = 13;
	my $live_local_statedir = registry::getReg("LIVE_LOCALSTATEDIR");
	my $administrator_uid = registry::getReg("ADMINISTRATOR_UID");
	my $domain_admins_gid = registry::getReg("DOMAIN_ADMINS_GID");
	my $gp_creator_owner_gid = registry::getReg("GP_CREATOR_OWNER_GID");
	my $domain_computers_gid =$administrator_uid + $DOMAIN_COMPUTERS_GID_OFFSET;
	my $domain_users_gid=$administrator_uid + $DOMAIN_USERS_GID_OFFSET;

	Log("Upgrading Sysvol Volume access permissions");

	eval{
		`setfacl -R -m g:$domain_computers_gid:r-x $live_local_statedir/sysvol`;
		`setfacl -R -m g:$domain_users_gid:r-x $live_local_statedir/sysvol`;
		`setfacl -R -m o:--- $live_local_statedir/sysvol`;
		`setfacl -R -m d:g:$gp_creator_owner_gid:rwx $live_local_statedir/sysvol`;
		`setfacl -R -m d:g:$domain_computers_gid:r-x $live_local_statedir/sysvol`;
		`setfacl -R -m d:g:$domain_users_gid:r-x $live_local_statedir/sysvol`;
		`setfacl -R -m d:o:--- $live_local_statedir/sysvol`;
		`chmod -R o-r $live_local_statedir/sysvol`;
		`chmod -R o-x $live_local_statedir/sysvol`;
	};

	if ($@) {  Log(ERR, $@); }

	Log("Sysvol folder Successfully upgraded");

}

sub upgrade_sp3 () {
	my $type = registry::getReg("INSTALL_TYPE");
	my $is_replica = registry::getReg("IS_REPLICA");
	my $inst = "";

	Logger::set_filename("/var/opt/novell/xad/log/ndsdcinit.log");
	Log("Starting DSfW server upgrade to SP3");

	if ($is_replica eq "true") {
		$inst = adc_install->new();
	}
	else {
		if ($type eq "frd") {
			chdir("/opt/novell/xad/share/dcinit");
			$inst = frd_install->new();
		}
		else {
			$inst = child_install->new();
		}
	}

	my $rc = 0;
	$rc = upgrade_smb_conf_file();
	if ($rc == -1) {
		Log("Failed to upgrade smb.conf file: $rc");
	}
  
	# Protect sysvol - ACL permission restrictive
	upgrade_sysvol_permission();

	# This updates dsfw related data
	upgrade_dsfw_domain();

	# Update keytab file
	upgrade_keytab();

	# update the password polices
	update_password_policies();
	Log ("Successfully upgraded the DSfW server to SP3");
}
1;
