#!/usr/bin/perl -I/opt/novell/xad/lib/perl -I/opt/novell/xad/lib64/perl

use strict;
use warnings;
use v5.10;# at least for Perl 5.10
no warnings 'experimental::smartmatch'; #Avoid known warnings with given-when
use Sys::Hostname;
use Net::Ping;
use File::Copy;
use Logger qw(:DEFAULT);

use lib "GSSAPI-0.12";
use GSSAPI;
use Net::DNS;

use File::Temp;
use Getopt::Long;
use POSIX;
use Locale::gettext;
use Test::More qw(no_plan);
use Getopt::Std;
use Data::Dumper;
use XAD::dns;
# Setting the Locale
#setlocale(LC_MESSAGES, "");
#bindtextdomain("UpdatePDCMaster", "/opt/novell/xad/share/locale");
#textdomain("UpdatePDCMaster");


my $log_file_name = "/var/opt/novell/xad/log/dnsrrupdate.log";
# Debug flag, setting it to 1 throws lots of debug messages
my $debug = 0;
my $operation = undef;
my $domain_name = undef;
my $record_name = undef;
my $priority = undef;
my $weight = undef;
my $port = undef;
my $target = undef;
my $record_type = undef;
my $ip_address = undef;
my $host_fqdn = undef;
my $time_to_live = undef;
my $site = undef;

my $domain = `/opt/novell/xad/share/dcinit/printConfigKey.pl "DNS Root"`;
chomp ($domain);
my $key_name1 = int(rand 10000000000000);
my $key_name = "$key_name1" ."-DSfW-123451";
my $nameserver = "";
my @authservers = ();
my $authservercount = 0;
my $status = 0;
my $localhost_fqdn = `/opt/novell/xad/share/dcinit/printConfigKey.pl "DNS Host Name"`;
chomp $localhost_fqdn;
#my $pipe_status_file = mktemp("/tmp/pipe_status.XXXXXXXX");
#if(defined($pipe_status_file)) {
#	$ENV{'PIPE_STATUS_FILE'} = $pipe_status_file;
#}
my $admin_sam_name = `/opt/novell/xad/share/dcinit/provisionTools.sh get-admin-sam-name`;
chomp $admin_sam_name;

#######################
### Error codes #######
my $ERR_NOT_A_ROOT_USER = -11;
my $ERR_GSSAPI_BIND_ERR = -12;
my $ERR_NO_PDC_ENTRY_IN_DNS = -13;
my $ERR_NO_AUTHORITATIVE_PDC_QUERY_ANSWERS_FROM_DNS = -14;
my $ERR_DNS_RR_PACKET_HANDLE_CREATION_FAILED = -15;
my $ERR_SENDING_TKEY_PACKET = -16;
my $ERR_VALIDATION_TKEY_FAILED = -17;
my $ERR_NO_REPLY_TO_DYNAMIC_UPDATE = -18;
my $ERR_BAD_LAST_PDC_MASTER_COUNT = -19;
my $ERR_CREATING_TEMP_TRANSFER_FILE = -20;
my $ERR_CHANGING_OWNERSHIP_TEMP_TRASNFER_FILE = -21;
my $ERR_FILE_NOT_FOUND = -22;
my $ERR_TEMP_FILE_PERMISSION_CHANGE = -23;
my $ERR_FILE_MOVE_ERROR = -24;
my $ERR_CANNOT_OPEN_FILE = -25;
my $ERR_CANNOT_RESOLVE_LAST_PDC_MASTER = -26;
my $ERR_CANNOT_REACH_TO_LAST_PDC_MASTER = -27;
my $ERR_INVALID_SAM_NAME_TKT = -28;
my $ERR_NO_CREDENTIALS_CACHE_FOUND = -29;
my $ERR_NO_RESOLVE_CONF_FILE_FOUND = -30;
my $ERR_HELP_MESSAGE_INFO = -31;
my $ERR_NO_FINAL_UPDATE = -32;
my $ERR_INVALID_ARGUMENT_PASSED = -33;

####################################################
###########Functions################################

sub gss_exit {
	my $status = shift;
	my @major_errors = $status->generic_message();
	my @minor_errors = $status->specific_message();
	my ($major_err, $minor_err) = 0;

	foreach $major_err (@major_errors) {
		next;
	}

	foreach $minor_err (@minor_errors) {
		next;
	}
	return $major_err;
}

# Validate acquired credentials for valid Administrator SAM-Account-Name
###########################################################

sub validate_tkt_for_admin_sam_name {
	my $klist_command = '/opt/novell/xad/bin/klist 2>/dev/null | grep "Default principal: " | awk -F": " \'{print $2}\' | awk -F@ \'{print $1}\'';
	my $default_princ = `$klist_command`;
	chomp($default_princ);
	if($? or $default_princ eq "") {
		return $ERR_NO_CREDENTIALS_CACHE_FOUND;
	}
	else {
		if (lc($admin_sam_name) ne lc($default_princ)) {
			return $ERR_INVALID_SAM_NAME_TKT;
		}
		# there might be a chance that the admin_sam_name and the default_princ values
		# just differ in case. So assigning default_princ to admin_sam_name
		$admin_sam_name = $default_princ
	}

	return 0;
}

## Find an authoritative nameserver for the domain ##
#####################################################
sub find_nameserver_for_domain($)
{
	my $domain = shift;
	my $rc = 0;
	# we need to parse the /etc/resolv.conf file, picking up
	# each nameserver entry and querying it to the domain passed
	# incase an authoritative answer is found, the nameserver in
	# account is to be used as the authoritative nameserver.
	my $file = "/etc/resolv.conf";
	my $readline = "";
	my @Nameservers = ();

	if(!(-e "$file"))
	{
		exit $ERR_NO_RESOLVE_CONF_FILE_FOUND;
	}
	open(Filename, $file) or die "cannot open $file:$@";
	while ($readline = <Filename>)
	{
		if($readline =~ /^nameserver/)
		{
			#push the nameserver IP in the nameservers array
			$readline =~ s/^nameserver\s+//g;
			chomp $readline;
			push(@Nameservers, $readline);
		}
	}
	close(Filename);

	# parse through the nameserver array and check for an authoritative
	# nameserver for the passed domain.
	my $Nameservers_count = $#Nameservers + 1;
	while($Nameservers_count > 0)
	{
		# check if this Namservers[$Nameservers_count] is an authoritative
		# nameserver for the domain or not
		my $handle = Net::DNS::Resolver->new;
		$handle->tcp_timeout(60);
		$handle->udp_timeout(60);

		# set the nameserver to $Nameservers[$Nameservers_count]
		$handle->nameservers("$Nameservers[$Nameservers_count - 1]");
		my $domain_query = $handle->send("$domain", "A");
		if(defined($domain_query))
		{
			# check for authoritative answer
			my $Is_Authoritative = $domain_query->header->aa;
			if($Is_Authoritative)
			{
				# this is the nameserver for the $domain, check the
				# returned IPs
				my $authcnt = $domain_query->authority;
				if ($authcnt > 0) {
					$authservercount = $authcnt;
					@authservers = $domain_query->authority;
					last;
				}
			}
		}
		$Nameservers_count--;
	}
}

# set the nameserver for the domain######
##########################################
sub set_nameserver($)
{
    my $dnsserver = shift;
    my $res = Net::DNS::Resolver->new;
    $res->tcp_timeout(60);
    $res->udp_timeout(60);
    $res->nameservers($dnsserver);
    return $res;
}

###signing callback function for TSIG module####
################################################
sub gss_sign($$)
{
	my $key = shift;
	my $data = shift;
	my $sig;
	$key->get_mic(0, $data, $sig);
	return $sig;
}

# Update PDC Master SRV record in DNS database ####
###################################################
sub Update_RR_Record
{
	my %args=@_;
	my $op = $args{op};
	my $d_name = $args{d_name};
	my $r_name = $args{r_name};
	my $pri = $args{pri};
	my $wght = $args{wght};
	my $prt = $args{prt};
	my $tgt = $args{tgt};
	my $h_fqdn = $args{host_name};
	my $host_ip = $args{host_ip};
	my $time_2live = $args{ttl};
	my $rec_type = $args{rec_type};
	my $return_val = 0;

NAMESERVER:
	foreach my $serv (@authservers) {
		# call the function to check and update the DNS database
		my $dnsserver = $serv->nsdname;

		if ($debug) {
			print "DNS Server = $dnsserver\n";
		}

		$return_val = 0;

		# create the GSSAPI contexts
		my $name = GSSAPI::Name->new;
		my $context = GSSAPI::Context->new;

		# Set the nameserver
		$nameserver = set_nameserver("$dnsserver");

		$status = $name->import($name, "DNS/" . $dnsserver . "@" . uc($d_name));
		if(($status->major != 0) or ($status->minor != 0))
		{
			# error encountered
			$return_val = gss_exit($status);
			next;
		}

		# specify the GSS flags
		my $gss_flags = GSS_C_REPLAY_FLAG | GSS_C_MUTUAL_FLAG |
			GSS_C_SEQUENCE_FLAG | GSS_C_CONF_FLAG |
			GSS_C_INTEG_FLAG | GSS_C_DELEG_FLAG;

		# acquire credentails
		my ($cred, $oidset, $time) = "";

		$status = GSSAPI::Cred::acquire_cred(undef, 120, undef, GSS_C_INITIATE,
				$cred, $oidset, $time);

		if(($status->major != 0) or ($status->minor != 0))
		{
			# error encountered
			$return_val = $status;
			next;
		}

		# call gss_init_sec_context
		my $token = "";
		my $token2 = "";

		$status = $context->init($cred, $name, undef, $gss_flags,
				0, undef, "", undef, $token, undef, undef);

		#  here, the permitted gss major status values are 0 and 1
		if(($status->major != 1) or ($status->minor != 0))
		{
			if($status->major != 0)
			{
				# error encountered
				$return_val = $status;
				next;
			}
		}

		# check the Major status for GSS_S_CONTINUE_NEEDED and length of $token
		# if (( Major status == 'GSS_S_CONTINUE_NEEDED') || (length($token)))
		# then we need to send the token back to the server.

		my $counter = 1;
		my $reply = undef;

		do
		{
			$counter++;
			my $gss_query = Net::DNS::Packet->new("$key_name", "TKEY", "IN");

			if(!defined($gss_query))
			{
				$return_val = $ERR_DNS_RR_PACKET_HANDLE_CREATION_FAILED;
				goto out;
			}

			# create the DNS RR packet
			$a = Net::DNS::RR->new(
					Name	=> "$key_name",
					Type	=> "TKEY",
					TTL	=> 0,
					Class	=> "ANY",
					mode => 3,
					algorithm => "gss-tsig",
					inception => time,
					expiration => time + 24*60*60,
					key => "$token",
					other_data => "",
					);

			$gss_query->push("additional", $a);

			# set the recusrion desired to 0 by default it is set to 1
			$gss_query->header->rd(0);

			# verify the packet before sending it
			if($debug)
			{
				my $z = ($gss_query->zone)[0];
				ok($gss_query, 'new() returned packet');#1
				is($gss_query->header->opcode, 'QUERY', 'header opcode correct');#2
				is($z->zname,  $key_name, 'zname correct'); #3
				is($z->zclass, "IN", 'zclass correct');#4
				is($z->ztype, 'TKEY', 'ztype correct');#5
				is($gss_query->header->rd, '0',	'header rd correct');#6
			}

			#sending the RR query packet to DNS server
			$reply = $nameserver->send($gss_query);

			if (!defined ($reply)) {
				$return_val = $ERR_SENDING_TKEY_PACKET;
				next NAMESERVER;
			}
			if ($reply->header->rcode ne 'NOERROR')
			{
				$return_val = $reply->header->{'rcode'};
				next NAMESERVER;
			}

			$token2 = ($reply->answer)[0]->{"key"};

			# calling gss_init_sec_context again
			$status = $context->init($cred, $name, undef, $gss_flags,
					0, undef, $token2, undef, $token,
					undef, undef);

			if(($status->major != 0) or ($status->minor != 0))
			{
				# error encountered
				$return_val = gss_exit($status);
				next NAMESERVER;
			}

		} while (($status->major eq GSS_S_CONTINUE_NEEDED) || (defined($token)));

		if(($status->major eq GSS_S_COMPLETE) && !(defined($token)))
		{
			my $update_hndl = "";
			my $update_rr = "";
			$update_hndl = Net::DNS::Update->new("$d_name");
			$update_hndl->push("pre", yxdomain("$d_name"));

			$update_rr = ${Process_Record(op=>$op,
						rec_type => $rec_type,
						d_name=>$d_name,
						r_name=>$r_name,
						pri=>$pri,
						wght=>$wght,
						prt=>$prt,
						tgt=>$tgt,
						host_name=>$h_fqdn,
						host_ip=>$host_ip,
						ttl=>$time_2live,
						update=>\$update_hndl)};

			my $sig = Net::DNS::RR->new(Name    => $key_name,
					Type	=> "TSIG",
					TTL	=> 0,
					Class	=> "ANY",
					Algorithm => "gss-tsig",
					Time_Signed => time,
					Fudge => 36000,
					Mac_Size => 0,
					Mac => "",
					Key => $context,
					Sign_Func => \&gss_sign,
					Other_Len => 0,
					Other_Data => "",
					Error => 0,
					mode => 3,
					);

			$update_rr->push("additional", $sig);

			if ($debug) {
				print "\nUpdate packet with signature:\n", Data::Dumper->Dump([$update_rr]), "\n";
			}

			# send the dynamic update
			my $update_reply = $nameserver->send($update_rr);

			if (! defined($update_reply))
			{
				$return_val = $ERR_NO_REPLY_TO_DYNAMIC_UPDATE;
				next NAMESERVER;
			}

			# make sure it worked
			$return_val = $update_reply->header->rcode;

			if ($return_val ne 'NOERROR')
			{
				print "ERR-RETURN VAL : $return_val\n";
				$return_val = $ERR_NO_FINAL_UPDATE;
				next NAMESERVER;
			} elsif($return_val eq 'NOERROR')
			{
				$return_val = 0;
				last NAMESERVER;
			}
		}#End of GSS_S_COMPLETE if condition
	}#End of For loop

out:
	return $return_val;
}

sub Process_Record {
	my %args=@_;
	my $op = $args{op};			#add, mod, del
	my $rec_type = uc($args{rec_type});	#can be SRV NS A etc.
	my $h_name = $args{host_name};		#A
	my $ipaddr = $args{host_ip};		#A
	my $d_name = $args{d_name};		#SRV
	my $r_name = $args{r_name};		#SRV
	my $pri = $args{pri};			#SRV
	my $wght = $args{wght};			#SRV
	my $prt = $args{prt};			#SRV
	my $ttl = $args{ttl};			#can be SRV A NS etc.
	my $tgt = $args{tgt};			#SRV
	my $update = ${$args{update}};
	my %prerequisite = ('A'=>undef, 'NS'=>undef, 'SRV'=>undef);
	my %update_data = ('A'=>undef, 'NS'=>undef, 'SRV'=>undef);
	my $host_server = undef;
	my @nslookup_output = ();
	my @server_detail = ();

	if ($debug) {
		print "\nArguments recieved : \n";
		while(my ($k, $v) = each(%args)){
			if (defined($v) and $v ne "") {
				printf("%20s => %-50s\n",$k,$v);
			}
		}
	}
	if ($op eq 'add' or $op eq 'mod') {
		given($rec_type) {
			when('A') {
				$prerequisite{$rec_type} = ($op eq 'add') ? nxrrset("$h_name A") :
					yxrrset("$h_name A");
				$update_data{$rec_type} = rr_add("$h_name $ttl A $ipaddr");
			}
			when('RR') {
				$update_data{'A'} = rr_add("$d_name A $ipaddr");
				$update_data{'NS'} = rr_add("$d_name $ttl NS $h_name");
			}
			when('SRV') {
				$host_server = (split(/\./, $tgt))[0];
				@nslookup_output = `/usr/bin/nslookup -querytype=srv $r_name.$d_name 2>&1`;
				@server_detail = grep(/$host_server/, @nslookup_output);

				if ($debug) {
					print "\nHOST MACHINE : $host_server\n";
					print "\nNSLOOKUP OUTPUT :\n", @nslookup_output, "\n";
					print "\nSERVER DETAIL : ", @server_detail, "\n";
					print "\nSERVER COUNT : ", $#server_detail+1, "\n";
				}

#				$prerequisite{$rec_type} = ($op eq 'add') ? nxrrset("$r_name.$d_name SRV ") :
#					yxrrset("$r_name.$d_name SRV ");
				$update_data{$rec_type} = rr_add("$r_name.$d_name $ttl SRV $pri $wght $prt $tgt");
			}
		}

		if ($ttl == 0) {
			#This a HACK into the instance of the object returned from the
			#rr_add when TTL value passed for SRV record is 0.
			#There is a BUG in the new_from_string subroutine inside RR.pm
			#provided by the Net::DNS. Due to this bug, whenever TTL value
			#is provided as 0 in any SRV record it is internally set to
			#86400 seconds. There is a statement '$ttl=86400 unless $ttl;'
			#which validates TTL value inside the new_from_string subroutine.
			#Here 'unless' will get condition status as 'FALSE' when TTL is 0.
			#The 'unless' statement checks for the 'FALSE' condition compared
			#to 'if', which checks for 'TRUE' condition. In case where TTL
			#value is 0(ZERO), 0 is always 'FALSE' under condition check block,
			#'unless' will execute the block and set the TTL to 86400.
			#Which is not CORRECT. Rectifying HERE.

			$update_data{$rec_type}->{'ttl'} = $ttl;
		}
	}

	if($op eq 'del') {
		given($rec_type) {
			when('A') {
				$prerequisite{$rec_type} = yxrrset("$h_name A");
				$update_data{$rec_type} = rr_del("$h_name A");
			}
			when('RR') {
				$prerequisite{'A'} = yxrrset("$d_name A ");
				$prerequisite{'NS'} = yxrrset("$d_name NS ");
				$update_data{'A'} = rr_del("$d_name A $ipaddr");
				$update_data{'NS'} = rr_del("$d_name NS $h_name");
			}
			when('SRV') {
				$prerequisite{$rec_type} = yxrrset("$r_name.$d_name SRV ");
				$update_data{$rec_type} = rr_del("$r_name.$d_name SRV $pri $wght $prt $tgt");
			}
		}

	}

	if ($rec_type eq 'RR') {
		given($op) {
			when('add') {
				$update->push("update", $update_data{'A'});
				$update->push("update", $update_data{'NS'});
			}
			when('del') {
				$update->push("pre", $prerequisite{'A'});
				$update->push("update", $update_data{'A'});
				$update->push("pre", $prerequisite{'NS'});
				$update->push("update", $update_data{'NS'});
			}
		}
	} else {
		if ($debug) {
			print "\nPrerequisite :\n", Data::Dumper->Dump([$prerequisite{$rec_type}]), "\n";
			print "\nUpdate :\n", Data::Dumper->Dump([$update_data{$rec_type}]), "\n";
		}

		#Only for 'A' record
		if ($rec_type ne 'SRV') {
			$update->push("pre", $prerequisite{$rec_type});
		}

		if ($op eq 'add' and $#server_detail+1>0 and $rec_type eq 'SRV') {
			#Nothing to do
		} else {
			$update->push("update", $update_data{$rec_type});
		}
	}
	return \$update;
}

sub usage()
{
	exit $ERR_HELP_MESSAGE_INFO;
}

# To update SOA record.
sub update_dns_soa() {
        print "Starting update_dns_soa" if($debug);

	my $proxy_username = `/opt/novell/proxymgmt/bin/cp_retrieve_proxy_cred username`;
        chomp($proxy_username);
        my $proxy_password = `/opt/novell/proxymgmt/bin/cp_retrieve_proxy_cred password`;
        chomp($proxy_password);
        my $frdDNS_provider = get_frdDNS_provider();
        my $dns_zone_context = get_zone_context();
        my $zone_container = `/opt/novell/xad/share/dcinit/printConfigKey.pl DNSZONE`;
        chomp($zone_container);
        my $dnsforwardzonefdn = "cn=$zone_container,$dns_zone_context";
        my $_LIB = registry::getReg("_Lib");

	#SOA-validation
        my @soa_output=`LD_PRELOAD=/opt/novell/xad/$_LIB/libadmpasswd.so.2 LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem /opt/novell/eDirectory/bin/ldapsearch -H 'ldaps://${frdDNS_provider}:1636' -x -D '$proxy_username' -w '$proxy_password' -b '$dnsforwardzonefdn' -s base dNIPSOASerial -LLL`;

        if ($? != 0) {
                print "Failed to get the SOA value : [@soa_output]";
        }

        @soa_output=grep(/dNIPSOASerial:/, @soa_output);
        chomp(@soa_output);
        my $soa_serial=`echo -n $soa_output[0] | grep -i "dNIPSOASerial: " | awk -F "dNIPSOASerial: " '{print \$2}'`;

	#SOA-validation
        chomp($soa_serial);
        $soa_serial +=10;
        my $dns_soa_serial_ldif = mktemp("/tmp/soaserial.XXXXXXXX");

        open FILE, ">$dns_soa_serial_ldif" or die ("failed to open the file $dns_soa_serial_ldif");
        print FILE "dn: $dnsforwardzonefdn\nchangetype: modify\nreplace: dNIPSOASerial\ndNIPSOASerial: $soa_serial";

        print "Executing... LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem /opt/novell/eDirectory/bin/ldapmodify  -H 'ldaps://${frdDNS_provider}:1636' -D '$proxy_username' -w ***** -f '$dns_soa_serial_ldif' -x -LLL\n" if($debug);

        `LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem /opt/novell/eDirectory/bin/ldapmodify -H 'ldaps://${frdDNS_provider}:1636' -D '$proxy_username' -w '$proxy_password' -f '$dns_soa_serial_ldif' -x 2>/dev/null`;
        my $ret = $?;

        if ($ret != 0) {
                #To maintain repeatable state after this failure so that same error happens in retry.
                print "Error while updating zone SOA";
        }

        #return here
        unlink($dns_soa_serial_ldif);
}

#Getting the IP address of the server where different zones' objects are located
#Currently this is the pdc of the "root domain" of DSfW forest
sub get_frdDNS_provider() {
        my $frdDNS_provider;
        my $dig_output;
        my $install_type = registry::getReg("INSTALL_TYPE");
        my $dns_server_ip =  registry::getReg("DNS_SERVER_IP");
        my $is_replica = registry::getReg("IS_REPLICA");
        my $ret = 0;


        if($install_type eq "frd" and $is_replica eq "FALSE") {
                $frdDNS_provider = $dns_server_ip;
        }
        else {
                my $forest_root = registry::getReg("FORESTROOT");
                $dig_output = `/usr/bin/dig -t SRV _ldap._tcp.pdc._msdcs.$forest_root  +short`;
                $ret = $?;
                chomp($dig_output);
                if ($ret || $dig_output eq "") {
                        my $error = $ret >> 8;
                        print ("Failed to get the domain provider for domain [$forest_root] [$error]");
                }
                $frdDNS_provider = `echo $dig_output | cut -d " " -f4 | sed 's/\.\$//'`;
                chomp($frdDNS_provider);
        }

        return $frdDNS_provider;
}

# To get zone context.
sub get_zone_context() {
        print "Fetching the Zone Context\n" if($debug);

	my $proxy_username = `/opt/novell/proxymgmt/bin/cp_retrieve_proxy_cred username`;
        chomp($proxy_username);

        my $proxy_password = `/opt/novell/proxymgmt/bin/cp_retrieve_proxy_cred password`;
        chomp($proxy_password);

        my $server_object = `/opt/novell/xad/share/dcinit/printConfigKey.pl DNSSERVER`;
        chomp($server_object);

        my $dns_server_ip = `/opt/novell/xad/share/dcinit/printConfigKey.pl DNS_SERVER_IP`;
        chomp($dns_server_ip);


        my $zone_name = `/opt/novell/xad/share/dcinit/printConfigKey.pl DNSZONE`;
        chomp($zone_name);


        my $certFile_pem = "/etc/opt/novell/certs/SSCert.pem";

        #Read FQDN of the domain zone from the server object,
        #and get its FQDN by removing the entry before first ','
        #TBD Finally change to "-p 636"
        my $zone_context = `LDAPTLS_CACERT=$certFile_pem /opt/novell/eDirectory/bin/ldapsearch -D '$proxy_username' -w '$proxy_password' -h '$dns_server_ip' -p 389 -b '$server_object' -s base dNIPZoneList -N -LLL -Z | sed ' /^ / {; H; d; }; /^ /! {; x; s/\\n //; }; '| grep -i 'dNIPZoneList:' | grep -iw '$zone_name' | awk -F 'dNIPZoneList: \+' '{print \$2}' | perl -pe 's/.*?,//' `;
        if ($? != 0) {
                print "Failed to get zone context\n";
        }

        print "Zone context - $zone_context\n" if($debug);

        chomp($zone_context);
        return $zone_context;
}

# Update PDC Master SRV record in DNS database ####
###################################################
sub Update_RR_Record_ldap
{
        my %args=@_;
        my $op = $args{op};
        my $r_name = $args{r_name};
	my $tgt = $args{tgt};
	my $st = $args{st};
	my $domain_name_global = $args{domain_name_global};
        my $return_val = 0;
	my $site_exists = "FALSE";

	my $frdDNS_provider = get_frdDNS_provider();
	my $dns_zone_context = get_zone_context();
	my $hostname_fqdn = `hostname --fqdn`;
	chomp($hostname_fqdn);
	my $proxy_username = `/opt/novell/proxymgmt/bin/cp_retrieve_proxy_cred username`;
	chomp($proxy_username);
	my $proxy_password = `/opt/novell/proxymgmt/bin/cp_retrieve_proxy_cred password`;
	chomp($proxy_password);

	# To check if any RR record exist with the site.
	my $srv_last_record = "";
	# Sometimes this command gives empty result even though this record is present in the edir.
	# This is beacuse <domain>.db file take time to sync-up with edir.
	$srv_last_record = `dig -t SRV _gc._tcp.$st._sites.$domain_name_global +short`;
	if ($srv_last_record ne "")
	{
		$site_exists = "TRUE";
	}
	print"is_site_exist = $site_exists\n" if($debug);

	if ($op eq 'add') {

	        my $new_ldif = "/var/opt/novell/xad/ds/dns/RR/$r_name".".ldif";

		dns::Process_RR("$r_name IN SRV  0 100 389 $tgt",0,"$dns_zone_context","$site_exists","add",86400);

		print "Executing... LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem /opt/novell/eDirectory/bin/ldapmodify -H ldaps://$frdDNS_provider:1636 -D '$proxy_username' -w ***** -f '$new_ldif' -x \n" if($debug);

		`LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem /opt/novell/eDirectory/bin/ldapmodify -H ldaps://$frdDNS_provider:1636 -D '$proxy_username' -w '$proxy_password' -f '$new_ldif' -x`;

		my $ret = $?;
	}

	if ($op eq 'del') {
		my $delete_ldif = "/var/opt/novell/xad/ds/dns/RR/"."Delete_"."$r_name.ldif";

                dns::Process_RR("$r_name IN SRV  0 100 389 $tgt",0,"$dns_zone_context","TRUE","delete",86400);

                print "Executing... LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem /opt/novell/eDirectory/bin/ldapmodify -H ldaps://$frdDNS_provider:1636 -D '$proxy_username' -w ***** -f '$delete_ldif' -x \n" if($debug);

                `LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem /opt/novell/eDirectory/bin/ldapmodify -H ldaps://$frdDNS_provider:1636 -D '$proxy_username' -w '$proxy_password' -f '$delete_ldif' -x`;

                my $ret = $?;
                unlink($delete_ldif);
        }

	# Updating SOA
	update_dns_soa;

	return $return_val;
}

## MAIN ##
##########
sub main
{
	my $rc = 0;
	my %options=();


	Logger::set_filename($log_file_name);

	# For modifying a Address (A) record following options should be provided:
	# o -> For specifying wether it is a add operation or delete(-o add/mod/del)
	# T -> Type of resource record (i.e. 'A' or 'SRV' or 'NS')
	# H -> fqdn of the host machine (newhost.example.com)
	# L -> Time for which the record is valid (86400 one day), only for Add operation.
	# I -> Ip address of the Host Machine (172.16.94.111), only for add operation.
	# s -> To specify a site name. Using the site name is to check if any type of SRV record exists.

	getopts("hdo:D:r:p:w:P:t:s:T:H:L:I:",\%options) or usage;
	usage() if defined $options{h};

	if(!exists $options{o} || not defined $options{o} || $options{o} eq ""){
		print "-o invalid operation (specify add|del)\n";
		$rc = $ERR_INVALID_ARGUMENT_PASSED;
		goto out;
	}
	$operation =  $options{o};

	if(exists $options{D} && (not defined $options{D} || $options{D} eq "")){
		print "-D invalid domain name\n";
		$rc = $ERR_INVALID_ARGUMENT_PASSED;
		goto out;
	}
	$domain_name = $options{D};

	if(exists $options{r} && (not defined $options{r} || $options{r} eq "")){
		print "-r invalid SRV record\n";
		$rc = $ERR_INVALID_ARGUMENT_PASSED;
		goto out;
	}
	$record_name = $options{r};

	if(exists $options{p} && (not defined $options{p} || $options{p} < 0)){
		print "-p invalid priority\n";
		$rc = $ERR_INVALID_ARGUMENT_PASSED;
		goto out;
	}
	$priority = $options{p};

	if(exists $options{w} && (not defined $options{w} || $options{w} < 0)){
		print "-w invalid weight\n";
		$rc = $ERR_INVALID_ARGUMENT_PASSED;
		goto out;
	}
	$weight = $options{w};

	if(exists $options{P} && (not defined $options{P} || $options{P} <= 0)){
		print "-P invalid port\n";
		$rc = $ERR_INVALID_ARGUMENT_PASSED;
		goto out;
	}
	$port = $options{P};

	if(exists $options{t} && (not defined $options{t} || $options{t} eq "")){
		print "-t invalid target\n";
		$rc = $ERR_INVALID_ARGUMENT_PASSED;
		goto out;
	}
	$target = $options{t};

	if(exists $options{s} && (not defined $options{s} || $options{s} eq "")){
                print "-s invalid target\n";
                $rc = $ERR_INVALID_ARGUMENT_PASSED;
                goto out;
        }
	$site = $options{s};

	if(exists $options{T} && (not defined $options{T} || $options{T} eq "")){
		print "-T invalid resource type\n";
		$rc = $ERR_INVALID_ARGUMENT_PASSED;
		goto out;
	}
	$record_type = $options{T};

	if(exists $options{H} && (not defined $options{H} || $options{H} eq "")){
		print "-H invalid FQDN of the host\n";
		$rc = $ERR_INVALID_ARGUMENT_PASSED;
		goto out;
	}
	$host_fqdn = $options{H};

	if(exists $options{I} && (not defined $options{I} || $options{I} eq "")){
		print "-I invalid IP address of the server\n";
		$rc = $ERR_INVALID_ARGUMENT_PASSED;
		goto out;
	}
	$ip_address = $options{I};

	if(exists $options{L} && (not defined $options{L} || $options{L} eq "")){
		print "-L invalid time to live value\n";
		$rc = $ERR_INVALID_ARGUMENT_PASSED;
		goto out;
	}
	$time_to_live = $options{L};

	$debug = $options{d} if defined $options{d};

	# Validate acquired credentials for valid Administrator SAM-Account-Name
	$rc = validate_tkt_for_admin_sam_name();
	if ($rc == $ERR_INVALID_SAM_NAME_TKT) {
		$rc = 0;
	}

	if($rc){
		print "ERR - No credential cache found\n";
	}
	else{

		# get the authoritative nameserver
		find_nameserver_for_domain($domain);

		# call the function to check and update the DNS database
		if($debug) {
			print "Values Passed :\n",
				(defined($operation)?"\tOperation : $operation\n":""),
				(defined($domain_name)?"\tDomain Name : $domain_name\n":""),
				(defined($record_name)?"\tRecord Name : $record_name\n":""),
				(defined($priority)?"\tPriority : $priority\n":""),
				(defined($weight)?"\tWeight : $weight\n":""),
				(defined($port)?"\tPort : $port\n":""),
				(defined($target)?"\tTarget : $target\n":""),
				(defined($host_fqdn)?"\tHost FQDN : $host_fqdn\n":""),
				(defined($ip_address)?"\tIP Address : $ip_address\n":""),
				(defined($time_to_live)?"\tTTL : $time_to_live\n":""),
				(defined($record_type)?"\tRecord Type : $record_type\n":""),
				"\n";
		}

#		$rc = Update_RR_Record(op=>$operation,
#				d_name=>$domain_name,
#				r_name=>$record_name,
#				pri=>$priority,
#				wght=>$weight,
#				prt=>$port,
#				tgt=>$target,
#				host_name=>$host_fqdn,
#				host_ip=>$ip_address,
#				ttl=>$time_to_live,
#				rec_type=>$record_type
#				);

		$rc = Update_RR_Record_ldap(op=>$operation, r_name=>$record_name, tgt=>$target, st=>$site, domain_name_global=>$domain_name);


	}

out:
	if ($rc !~ /^[+-]?\D+$/) {
		exit($rc);
	} else {
		print "ERR - $rc\n";
		exit (-1);
	}
}
main
