#!/usr/bin/perl -I. -I.. -I/opt/novell/xad/lib64/perl -I/opt/novell/xad/lib/perl
################################################################################
# Domain Services for Windows Cross Partition NTP Setup Tool
#
# See the file LICENSE for redistribution information.
#
# Copyright (c) 2001-2012
#       Novell, Inc.  All rights reserved.
################################################################################

use strict;
use warnings;
use v5.10;# at least for Perl 5.10
no warnings 'experimental::smartmatch'; #Avoid known warnings with given-when
use XAD::registry;
use util;
use crosspartitionntp;

my $DCINIT_CONFIG = "/etc/opt/novell/xad/xad.ini";

sub main{
	my $rc = 0;
	my $old_ncalrpcdir=registry::getReg("NcalRpcDir");

	# Populate the new xadsd location
	eval{
		$rc = util::populate_newxadsd_location();
		if($rc){
			given($rc){
				when(-101){
					print "This is a default NTP configuration, no cross partition NTP configuration is needed.\n";
				}
				when(-102){
					print "The directory structure /var/lib/chrony and ...newxad/rpc are in different partitions.\n";
					print "...newxad/rpc should either be moved or deleted\n";
				}
			}
			exit(-1);
		}

		#Compare the old value to new value
		#populate the right value into /etc/opt/novell/xad/xad.ini
		my $new_ncalrpcdir=registry::getReg("NcalRpcDir");

		if ($old_ncalrpcdir ne $new_ncalrpcdir) {

			if ($old_ncalrpcdir eq "")
			{
				open(CONFIG,">>$DCINIT_CONFIG") || die("could not open $DCINIT_CONFIG\n");
				print CONFIG "NcalRpcDir = $new_ncalrpcdir\n";
				close(CONFIG);
			}
			else
			{
                		util::replace_line_in_file('^NcalRpcDir.*', "NcalRpcDir = $new_ncalrpcdir", $DCINIT_CONFIG);
			}
		}
	};

	if($@) {
		print "$@\n"; #Printing ERR message actually dies with proper error message and a backtrace
	}

	# Update conf,init files in phases
	crosspartitionntp::update_xad_profile_script();
	crosspartitionntp::create_directory_structure_for_ntp();
	crosspartitionntp::update_config_files();
}
main;
