#!/usr/bin/perl -I/opt/novell/xad/lib64/perl

use Getopt::Long;
use strict;
use warnings;
use XAD::registry;
use XAD::schema;
use NDS::import;
use Term::ReadKey;
use File::Temp;
use Logger qw(:DEFAULT);

use File::Copy;
use XAD::secure;

my $schemaTool="/opt/novell/eDirectory/bin/ndssch";
my $schemaLogFile="/var/opt/novell/eDirectory/log/schema.log";
my $treeName="";
my $schemaFile="/etc/opt/novell/xad/msds.sch";
my $vardir = "/var/opt/novell/xad/ds";
my $templatedir = "/opt/novell/xad/share/dcinit/templates/";
my $ldapadminname=`/opt/novell/xad/share/dcinit/provisionTools.sh get-admin-name`;
my $admin_password = "";
my ($frd, $nm_frd, $child, $nm_child, $adc, $nm_adc)=0;
my ($attr_mixed_val, $attr_behav_val) = 0;
my ($domainAt2016l, $totalServersFound, $forestAt2016l) = 0;
my $log_file_name = "/var/opt/novell/xad/log/domainupgrade.log";

sub Usage(){
	print <<USAGE;
	Usage:/opt/novell/xad/sbin/domainUpgrade.pl [options]

	options:
		-h or --help : Help on the command line options
		-D or --domainlevel : Updates Domain Functional Level to the latest
		-F or --forestlevel : Updates Forest Functional Level to the latest
		-p or --principal : option to configure the msDS-PrincipalName

	examples:
		Execute with the ADM_PASSWD environment variable as

		ADM_PASSWD=<password> /opt/novell/xad/sbin/domainUpgrade.pl

		where ADM_PASSWD is domain admin password.

		Execute without ADM_PASSWD environment variable will prompt
		you for domain admin password i.e

		/opt/novell/xad/sbin/domainUpgrade.pl

		For Help:
		/opt/novell/xad/sbin/domainUpgrade.pl -h
		/opt/novell/xad/sbin/domainUpgrade.pl --help
		
		For Domain Functional Level update:
		/opt/novell/xad/sbin/domainUpgrade.pl -D
		/opt/novell/xad/sbin/domainUpgrade.pl --domainlevel
		
		For Forest Functional Level update:
		/opt/novell/xad/sbin/domainUpgrade.pl -F
		/opt/novell/xad/sbin/domainUpgrade.pl --forestlevel

		For msDS-PrincipalName:
		/opt/novell/xad/sbin/domainUpgrade.pl -p
		/opt/novell/xad/sbin/domainUpgrade.pl --principal
USAGE
	exit 0
}

#Adds AD schema under schema partition
sub schema_online_serv
{
	#my $NATIVE_REPL = registry::getReg("NATIVE_REPL");

	#Added for ldif modification
	my $schema_ldif="$vardir/schema/schema.ldif";

	Log(INFO, ">>>Importing Schema Naming Context into LDAP Server");
	eval{
		Log(INFO,"Importing $schema_ldif");
		#Added for ldif modification
		my $rc = import::import_file(host=>registry::getReg("LDAPI_URL"),
				admin=>$ldapadminname,
				ssl=>"yes",
				file=>$schema_ldif,
				cont_if_exist=>"true");
		if ($rc != 0) {
			die("Could not import $schema_ldif");
		}
	};

	if ($@) { Log(WARN, $@); }
	Log(INFO,"Schema Naming Context objects are imported successfully into LDAP Server");
}

sub schema_bl_online_serv
{
        #Added for ldif modifications
        my $schema_bl_ldif = "$vardir/schema/schema-bl.ldif";

        Log(INFO,">>>Importing Schema Naming Context (Back-Links) into LDAP Server");

	eval{
		#Added for ldif modification
		Log(INFO,"Importing $schema_bl_ldif");

		my $rc = import::import_file(host=>registry::getReg("LDAPI_URL"),
				admin=>$ldapadminname,
				file=>$schema_bl_ldif,
				ssl=>"yes",
				cont_if_exist=>"true");
		if ($rc != 0) {
			die("Could not import $schema_bl_ldif");
		}

	};

	if ($@) { Log(WARN,$@); }
	Log(INFO,"Schema Naming Context (Back-Links) are imported successfully into LDAP Server");

}

sub schema_aggregate {
	eval{
		{#Lets get the output of this module to msdsfile.
			local *STDOUT;
			open STDOUT, '>', $schemaFile or die("Could not open the file $schemaFile");
			schema::aggregateSchema("$vardir/schema/schema.ldif", "--ndsschema");
		}

		if (util::chkfor_reg("$schemaFile") == 0) { die("Not all the REGISTRY values are replaced in $schemaFile")};

	};
	if ($@) { Log(WARN,$@) }
	Log(INFO,"$schemaFile is generated successfully");
}

sub generateSchemaLdif(){
	Log(INFO,"Generating schema.ldif");
	stage::gen_ldif("$templatedir/schema.ini", '>', "$vardir/schema/schema.ldif", "--offline", "--schema");

	if (util::chkfor_reg("$vardir/schema/schema.ldif") == 0) {
		die("Not all the REGISTRY values are replaced in $vardir/schema/schema.ldif $?")
	};

        Log(INFO,"Generating schema-bl.ldif");
        stage::gen_ldif("$templatedir/schema-bl.ini", '>', "$vardir/schema/schema-bl.ldif", "--schema");

        if (util::chkfor_reg("$vardir/schema/schema-bl.ldif") == 0) {
                die("Not all the REGISTRY values are replaced in $vardir/schema/schema-bl.ldif")
        };

	schema_aggregate;
}

sub ExtendSchema(){
	my $tAdmin="";
	Log(INFO,"Extending schema to 2016 level");
	$treeName = `/opt/novell/xad/share/dcinit/ndsConfigGet.sh "n4u.base.tree-name"`;
	if(not defined($treeName) or $treeName eq ""){
		Log(WARN, "failed to fetch the tree name.");
		return 1;
	}
	chomp($treeName);

	$tAdmin=$ldapadminname;
	chomp($tAdmin);
	$tAdmin=~s/,/./g;

	my @status = ();
	if(not defined($ENV{ADM_PASSWD}) or $ENV{ADM_PASSWD} eq ""){
		@status = `$schemaTool $tAdmin -F $schemaLogFile -t $treeName $schemaFile 2>&1`;
	} else {
		@status = `$schemaTool $tAdmin -F $schemaLogFile -t $treeName $schemaFile -p $ENV{ADM_PASSWD} 2>&1`;
	}
        my $error = $? >> 8;
        if ($error){
                chomp(@status);
                Log(ERR,"Failed to extend schema to 2016 level : [@status]");
                return 1;
        }
	Log(INFO,"Schema extension to 2016 level completed");
}

sub dcSchemaVersionUpdate(){
	my $mapped_server_dn_name =`/opt/novell/xad/share/dcinit/printConfigKey.pl "Mapped Server DN Name"`;
	chomp($mapped_server_dn_name);
	my $dcupdate_ldif = mktemp("/tmp/dcupdate_ldif.XXXXXXXXXX");

	my $dcver = `ldapsearch -D $ldapadminname -w  $admin_password -s base -b "cn=NTDS Settings,$mapped_server_dn_name" msDS-Behavior-Version -o ldif-wrap=no -Z -LLL | grep msDS-Behavior-Version | awk -F": " '{print \$2}'`;

	if($dcver < 7){

		open FILE, ">$dcupdate_ldif" or die ("failed to open the file $dcupdate_ldif");

		#dn: cn=NTDS Settings,cn=TEST1,cn=Servers,cn=Default-First-Site-Name,cn=Sites,cn=Configuration,dc=nonschem,dc=edu
		print FILE "dn: cn=NTDS Settings,$mapped_server_dn_name\n";
		print FILE "changetype: modify\n";
		print FILE "replace: msDS-Behavior-Version\n";
		print FILE "msDS-Behavior-Version: 7\n";
		print FILE "\n";
		close FILE;

		Log(INFO,"Updating the DC to 2016 level");
		eval{
			Log(INFO, "Executing...(post) ldapmodify -D $ldapadminname -w  ***** -Z -f $dcupdate_ldif");
			my @output = `ldapmodify -D $ldapadminname -w  $admin_password -Z -f $dcupdate_ldif`;
			my $error = $? >> 8;
			if ($error) {
				die("Could not modify the objects: $error: [@output]");
			}
		};

		if ($@) { Log(ERR,$@) }

		unlink($dcupdate_ldif);
	}
}

sub isDomainAtLevel2016(){
	my $at2016level = 1;
	my $rootDomain = registry::getReg("Root Domain");;

	chomp($rootDomain);
	$rootDomain =~ s/\./,dc=/g;
	$rootDomain = "dc=".$rootDomain;

	eval{
		#Find all server objects in the domain
		my @server_result =`ldapsearch -D $ldapadminname -w $admin_password -b "ou=Domain Controllers,$rootDomain" -s sub "(objectClass=computer)" -Z -o ldif-wrap=no -LLL serverReferenceBL 2>&1 | grep -i "^serverReferenceBL"`;

		my $error = $? >> 8;
		if ($error) {
			Log(WARN,"Could not get the domain controllers: $error: [@server_result]");
			return;
		}

		$totalServersFound = $#server_result + 1;
		foreach my $serv (@server_result){
			my $serverDn = (split(/^serverReferenceBL: /, $serv))[1];
			chomp($serverDn);
			my $name = (split(/=/, ((split(/,/, $serv))[0])))[1];

			#Search for msDS-Behavior-Version on each of the server object
			my $output = `ldapsearch -D $ldapadminname -w $admin_password -b "cn=NTDS Settings,$serverDn" -s base -Z -o ldif-wrap=no -LLL msDS-Behavior-Version 2>&1 | grep -i "msDS-Behavior-Version"`;
			if($? == 0){
				my $version = (split(/:/, $output))[1];
				if($version <7){
					Log(WARN, "Server $name is not at 2016 level.");
					$at2016level = 0;
				}else{
					Log(INFO, "Server $name is at 2016 level.");
				}
			}else{
				Log(INFO, "Server $name is not a valid server.");
			}
		}

		if(!$at2016level){
			Log(WARN, "Domain functional level it not updated to 2016, as not all domain servers are at level 2016 or greater.");
			print "Domain functional level it not updated to 2016, as not all domain servers are at level 2016 or greater.\n";
			$domainAt2016l=0;
		}else{
			$domainAt2016l=1;
		}


		#if it is single server and is the PDC
		#we are going to anyway allow the server to update msDS-Behavior-Version
		#hence checking the server
		my $type = registry::getReg("INSTALL_TYPE");
		my $is_replica = registry::getReg("IS_REPLICA");

		if($type eq "frd" and $is_replica eq "FALSE" and $totalServersFound==1){
			$domainAt2016l = 1;
		}
	};

	if ($@) { Log(WARN,$@); }
}

sub post_schema_extension(){
	my $mapped_nc = `/opt/novell/xad/share/dcinit/printConfigKey.pl "Mapped Domain NC"`;
	my $post_extn_ldif = mktemp("/tmp/post_behav_ldif.XXXXXXXXXX");

	Log(INFO,"Preparing domain for extended schema at 2016 level(post)");
	chomp($mapped_nc);

	open FILE, ">$post_extn_ldif" or die ("failed to open the file $post_extn_ldif");

	print FILE "dn:$mapped_nc\n";
	print FILE "changetype: modify\n";
	print FILE "replace: msDS-Behavior-Version\n";
	print FILE "msDS-Behavior-Version: 7\n";
	print FILE "-\n";
	print FILE "replace: nTMixedDomain\n";
	print FILE "nTMixedDomain: 0\n";
	print FILE "\n";

	close FILE;

	eval{
		Log(INFO, "Executing...(post) ldapmodify -D $ldapadminname -w  ***** -Z -f $post_extn_ldif");
		my @output = `ldapmodify -D $ldapadminname -w  $admin_password -Z -f $post_extn_ldif`;
		my $error = $? >> 8;
		if ($error) {
			die("Could not modify the objects: $error: [@output]");
		}
	};
	if ($@) { Log(ERR,$@) }

	unlink($post_extn_ldif);
}

#For the forest level validation
# First all of the servers present in the tree needs
# to be at the correct schema level
# Second all of the domains should be at the same
# updated schema level
sub isForestAtLevel2016(){
        my $at2016level = 1;
        my $totalServers = 0;
        my $forestNC = registry::getReg("Forest NC");

	#Server object validation
        eval{
                #Find all server objects in the domain
                my @server_result =`ldapsearch -D $ldapadminname -w $admin_password -b "cn=Configuration,$forestNC" -s sub "(objectClass=server)" -Z -o ldif-wrap=no -LLL dn 2>&1 | grep -i "^dn:"`;

                my $error = $? >> 8;
                if ($error) {
                        Log(WARN,"Could not get the domain controllers: $error: [@server_result]");
                        return;
                }

                my $totalServers = $#server_result + 1;
                foreach my $serv (@server_result){
                        my $serverDn = (split(/^dn: /, $serv))[1];
                        chomp($serverDn);
                        my $name = (split(/=/, ((split(/,/, $serv))[0])))[1];

                        #Search for msDS-Behavior-Version on each of the server object
                        my $output = `ldapsearch -D $ldapadminname -w $admin_password -b "cn=NTDS Settings,$serverDn" -s base -Z -o ldif-wrap=no -LLL msDS-Behavior-Version 2>&1 | grep -i "msDS-Behavior-Version"`;
                        if($? == 0){
                                my $version = (split(/:/, $output))[1];
                                if($version <7){
                                        Log(WARN, "Server $name is not at 2016 level.");
                                        $at2016level = 0;
                                }else{
                                        Log(INFO, "Server $name is at 2016 level.");
                                }
                        }else{
                                Log(INFO, "Server $name is not a valid server.");
                        }
                }

		if(!$at2016level){
			Log(WARN, "Forest functional level is not updated to 2016, as not all forest servers are at level 2016 or greater.");
			print "Forest functional level is not updated to 2016, as not all forest servers are at level 2016 or greater.\n";
			$forestAt2016l=0;
		}else{
			$forestAt2016l=1;
                }
        };

	if ($@) { Log(WARN,$@); }

	# Domain validation
	eval{
		#Find all domain contexts in the forest
		my @domain_contexts = `ldapsearch -D $ldapadminname -w $admin_password -b "$forestNC" -s sub "(objectClass=Partition)" xad-Domain-Flag -Z -o ldif-wrap=no -LLL dn 2>&1 | grep -B 1 xad-Domain-Flag | grep -i "^dn:"`;

		my $error2 = $? >>8;
		if ($error2){
			Log(WARN, "Could not get the Domain Contexts in the forest: $error2: [@domain_contexts]");
			return;
		}

		my $total_contexts = $#domain_contexts +1;
		foreach my $context (@domain_contexts){
			my $contextDn = (split(/^dn: /, $context))[1];
			chomp($contextDn);

			#Search for msDS-Behavior-Version on each of the domain context
			my $output = `ldapsearch -D $ldapadminname -w $admin_password -b "$contextDn" -s base -Z -o ldif-wrap=no -LLL msDS-Behavior-Version 2>&1 | grep -i "msDS-Behavior-Version"`;
			if($? == 0){
				my $version = (split(/:/, $output))[1];
				if($version <7){
					Log(WARN, "Domain $contextDn is not at 2016 level.");
					$at2016level = 0;
				}else{
					Log(INFO, "Domain $contextDn is at 2016 level.");
				}
			}else{
				Log(INFO, "Domain $contextDn is not a valid domain.");
			}
		}

		if(!$at2016level){
			Log(WARN, "Forest functional level is not updated to 2016, as not all domains in the Forest are at 2016 level.");
			print "Forest functional level is not updated to 2016, as not all domains in the Forest are at 2016 level.\n";
			$forestAt2016l=0;
		}else{
			$forestAt2016l=1;
		}
	};

        if ($@) { Log(WARN,$@); }

        return $forestAt2016l;
}

sub post_forest_functional_level(){
	my $post_forest_func_ldif = mktemp("/tmp/post_forest_func_ldif.XXXXXXXXXX");
	my $netbios_name = `/opt/novell/xad/share/dcinit/printConfigKey.pl "Netbios Name"`;
	my $mapped_server_dn_name =`/opt/novell/xad/share/dcinit/printConfigKey.pl "Mapped Server DN Name"`;
	my $forest_nc = `/opt/novell/xad/share/dcinit/provisionTools.sh actual-forest-nc`;

	Log(INFO,"Preparing forest for extended schema at 2016 level(post)");
	chomp($netbios_name);
	chomp($mapped_server_dn_name);
	chomp($forest_nc);

	open FILE, ">$post_forest_func_ldif" or die ("failed to open the file $post_forest_func_ldif");
	#dn: cn=Partitions,cn=Configuration,dc=nonschem,dc=edu
	print FILE "dn: cn=Partitions,cn=Configuration,$forest_nc\n";
	print FILE "changetype: modify\n";
	print FILE "replace: msDS-Behavior-Version\n";
	print FILE "msDS-Behavior-Version: 7\n";
	print FILE "\n";
	#dn: cn=NONSCHEM,cn=Partitions,cn=Configuration,dc=nonschem,dc=edu
	print FILE "dn: cn=$netbios_name,cn=Partitions,cn=Configuration,$forest_nc\n";
	print FILE "changetype: modify\n";
	print FILE "replace: msDS-Behavior-Version\n";
	print FILE "msDS-Behavior-Version: 7\n";
	print FILE "-\n";
	print FILE "replace: ntMixeddomain\n";
	print FILE "ntMixeddomain: 0\n";
	close FILE;

	eval{
		Log(INFO, "Executing...(post forest) ldapmodify -D $ldapadminname -w  ***** -Z -f $post_forest_func_ldif");
		my @output = `ldapmodify -D $ldapadminname -w  $admin_password -Z -f $post_forest_func_ldif`;
		my $error = $? >> 8;
		if ($error) {
			die("Could not modify the objects: $error: [@output]");
		}
	};
	if ($@) { Log(ERR,$@) }
	
	unlink($post_forest_func_ldif);
}

#AES update related
my %replacement_maps_aes =(
    'permitted_enctypes[\h]*=[\h]*arcfour-hmac-md5[\h]+des-cbc-md5[\h]+des-cbc-crc' => 'permitted_enctypes = aes256-cts-hmac-sha1-96 aes128-cts-hmac-sha1-96 arcfour-hmac-md5 des-cbc-md5 des-cbc-crc',
    'default_tgs_enctypes[\h]*=[\h]*arcfour-hmac-md5' => 'default_tgs_enctypes = aes256-cts-hmac-sha1-96 aes128-cts-hmac-sha1-96 arcfour-hmac-md5',
    'default_tkt_enctypes[\h]*=[\h]*arcfour-hmac-md5' => 'default_tkt_enctypes = aes256-cts-hmac-sha1-96 aes128-cts-hmac-sha1-96 arcfour-hmac-md5',
    'permitted_enctypes[\h]*=[\h]*aes128-cts[\h]+arcfour-hmac-md5[\h]+des-cbc-md5[\h]+des-cbc-crc[\h]+aes128-cts-hmac-sha1-96' => 'permitted_enctypes = aes256-cts-hmac-sha1-96 aes128-cts-hmac-sha1-96 arcfour-hmac-md5 des-cbc-md5 des-cbc-crc',
    'default_tgs_enctypes[\h]*=[\h]*aes128-cts[\h]+arcfour-hmac-md5' => 'default_tgs_enctypes = aes256-cts-hmac-sha1-96 aes128-cts-hmac-sha1-96 arcfour-hmac-md5',
    'default_tkt_enctypes[\h]*=[\h]*aes128-cts[\h]+arcfour-hmac-md5' => 'default_tkt_enctypes = aes256-cts-hmac-sha1-96 aes128-cts-hmac-sha1-96 arcfour-hmac-md5'
);

my @replacement_strings_aes = keys %replacement_maps_aes;

sub replace_in_file_aes {
    my $file = shift;
    open(IN,"<$file") or die $!;
    my @input_lines = <IN>;
    close(IN);

    my @output_lines;
    foreach(@input_lines)
    {
       $_=~s/$replacement_strings_aes[0]/$replacement_maps_aes{$replacement_strings_aes[0]}/g;
       $_=~s/$replacement_strings_aes[1]/$replacement_maps_aes{$replacement_strings_aes[1]}/g;
       $_=~s/$replacement_strings_aes[2]/$replacement_maps_aes{$replacement_strings_aes[2]}/g;
       $_=~s/$replacement_strings_aes[3]/$replacement_maps_aes{$replacement_strings_aes[3]}/g;
       $_=~s/$replacement_strings_aes[4]/$replacement_maps_aes{$replacement_strings_aes[4]}/g;
       $_=~s/$replacement_strings_aes[5]/$replacement_maps_aes{$replacement_strings_aes[5]}/g;
       push(@output_lines,$_);
    }

    open(OUT,">$file") or die $!;
    print OUT @output_lines;
    close(OUT);
}

#Current update supported only from non-AES level
sub update_krb5Conf_aes {

	copy("/etc/krb5.conf","/tmp/krb5.conf-aes") or die "Copy failed: $!";
	copy("/etc/krb5.conf","/etc/krb5.conf-orig") or die "Copy failed: $!";
	replace_in_file_aes("/tmp/krb5.conf-aes");	
	copy("/tmp/krb5.conf-aes","/etc/krb5.conf") or die "Copy failed: $!";
	print "Domain Controller config file updated to support AES encryption\n";

}

sub update_domain_to_aes {

	update_krb5Conf_aes;
	
	#bootstrap_krbtgt;
	my $command = "/opt/novell/xad/sbin/setpassword -NDOS -r -u krbtgt";

	Log(INFO, "Executing : $command");
	my @command_output = `$command 2>&1`;
	if (($?>>8) != 0) {
		Log("Could not set password for krbtgt");
		Log("error : [@command_output]");
		Log("Please try rerunning");
	}

        # setpassword command for server object
	util_update::execute_setpassword_command;
}

#Subroutine will hep you to create the Password Settings container in a DSfW
#environment.
sub createPasswordSettingsContainer(){
	my $mapped_nc = `/opt/novell/xad/share/dcinit/printConfigKey.pl "Mapped Domain NC"`;
	my $psc_creation_ldif = mktemp("/tmp/post_behav_ldif.XXXXXXXXXX");

	Log(INFO,"Creating the Password Settings Container");
	chomp($mapped_nc);

	#check if the container is already created.
	my $output = `ldapsearch -D $ldapadminname -w $admin_password -b "cn=System,$mapped_nc" -s sub "(cn=Password Settings Container*)" -Z -o ldif-wrap=no -LLL dn 2>&1 | grep -i "Password Settings Container"`;
	my $e = $? >> 8;
	if($e==0){
		Log(INFO, "Executing...(Password Settings Container) container already exists.");
		return 0;
	}

	open FILE, ">$psc_creation_ldif" or die ("failed to open the file $psc_creation_ldif");
	print FILE "dn: CN=Password Settings Container,CN=System,$mapped_nc\n";
	print FILE "changetype: add\n";
	print FILE "cn: Password Settings Container\n";
	print FILE "objectClass: top\n";
	print FILE "objectClass: msDS-PasswordSettingsContainer\n";
	print FILE "instanceType: 4\n";
	print FILE "showInAdvancedViewOnly: TRUE\n";
	print FILE "name: Password Settings Container\n";
	print FILE "systemFlags: -1946157056\n";
	print FILE "objectCategory: CN=ms-DS-Password-Settings-Container,CN=Schema,CN=Configuration,$mapped_nc\n";
	print FILE "\n";
	close FILE;

	eval{
		Log(INFO, "Executing...(Password Settings Container) ldapmodify -D $ldapadminname -w  ***** -Z -f $psc_creation_ldif");
		my @output = `ldapmodify -D $ldapadminname -w  $admin_password -Z -f $psc_creation_ldif 2>&1`;
		my $error = $? >> 8;
		if ($error) {
			die("Failed to create Password Settings Container: $error: [@output]");
		}
	};

	if ($@) { Log(WARN,$@); }

	unlink($psc_creation_ldif);
}

sub createMsDSPrincipalNameAndACL(){
	my $principal_domain_acls_ini = "$templatedir/principal-domain-acls.ini";
	my $principal_domain_acls_ldif = "$vardir/domain/principal-domain-acls.ldif";

	open FILE, ">$principal_domain_acls_ini" or die ("failed to open the file $principal_domain_acls_ini");
	print FILE "[DEFAULTROOTDOMAIN]\n";
	print FILE "CHILD=Ms DS Principal Name\n";
	print FILE "\n";
	print FILE "[Ms DS Principal Name]\n";
	print FILE "RDN-Of-Object=DEFAULTROOTDOMAIN\n";
	print FILE "changetype=modify\n";
	print FILE "add=msDS-PrincipalName\n";
	print FILE ";\n";
	print FILE "; Adding the msDS-PrincipalName with value netbios name\n";
	print FILE "; This is specifically used by the NSS AD\n";
	print FILE ";\n";
	print FILE "msDS-PrincipalName=\$REGISTRY=Netbios Name\\\n";
	print FILE "-\n";
	print FILE "add=ACL\n";
	print FILE ";\n";
	print FILE "; Allow public to have compare access to cn attribute\n";
	print FILE ";\n";
	print FILE "; DS_ATTR_COMPARE\n";
	print FILE ";\n";
	print FILE ";ACL=1#subtree#[Public]#cn\n";
	print FILE ";\n";
	print FILE "; Allow authenticated users compare/read access\n";
	print FILE ";\n";
	print FILE "; DS_ATTR_COMPARE | DS_ATTR_READ\n";
	print FILE ";\n";
	print FILE "ACL=3#subtree#[Root]#msDS-PrincipalName\n";
	close FILE;
	Log(INFO,"Generating principal-domain-acls.ldif");
	stage::gen_ldif("$principal_domain_acls_ini", '>', "$principal_domain_acls_ldif", "--offline", "--defaultrootdomain");

	eval{
		Log(INFO, "Executing...(principal) ldapmodify -D $ldapadminname -w  ***** -Z -f $principal_domain_acls_ldif");
		my @output = `ldapmodify -D $ldapadminname -w  $admin_password -Z -f $principal_domain_acls_ldif`;
		my $error = $? >> 8;
		if ($error) {
			die("Could not modify the objects: $error: [@output]");
		}
	};

	if ($@) { Log(WARN,$@) }
}

sub checkServices(){
	my @services=qw(ndsd);
	my $m_status = 0;

	foreach my $service (@services){
                `/usr/bin/systemctl status $service >/dev/null 2>&1`;
		if($?>0){
			Log(WARN, "$service is not up and running, bringing up the service.");
			`/usr/bin/systemctl start $service >/dev/null 2>&1`;
			if($? > 0){
				Log(WARN, "$service is not coming up, brining it manually.");
				$m_status = 1;
			}else{
				Log(INFO, "$service is up and running.");
			}
		}
	}
	return $m_status;
}

sub getTreeAdmin() {
        my $tree_admin = "";

        if (defined $ENV{LDAPEXISTINGADMINNAME} and $ENV{LDAPEXISTINGADMINNAME} ne "") {
                $tree_admin = $ENV{LDAPEXISTINGADMINNAME};
                return $tree_admin;
        }

        #non-name mapped tree administrator
        $tree_admin = registry::getReg("LDAPDOMAINADMINNAME");
        my $type = registry::getReg("INSTALL_TYPE");
        my $is_replica = registry::getReg("IS_REPLICA");

        if ($type eq "frd" and $is_replica eq "FALSE") {
                if (not registry::getReg("MAPPED")) {
                        #name mapped tree administrator
                        $tree_admin = registry::getReg("LDAPDOMAINADMIN");
                }
        }

        #print("From registry - Returning Tree Administrator $tree_admin");
        return $tree_admin;
}

sub main(){
	my %options=();
	my ($domainLevel, $forestLevel, $principalName) = 0;
	my $help = 0;
	my $login_success = 0;
	my $test_admin = "";
	my $statement = "Enter password for";
	my $forest_nc = `/opt/novell/xad/share/dcinit/provisionTools.sh actual-forest-nc`;
	my $ndsadminname="";
	chomp($forest_nc);

	Logger::set_filename($log_file_name);
	Logger::unset_mask(2);

	Getopt::Long::Configure("no_ignore_case");
	GetOptions(\%options,
			'D|domainlevel' => \$domainLevel,
			'F|forestlevel' => \$forestLevel,
			'p|principal' => \$principalName,
			'h|help' => \$help) or eval
	{
		Usage();
		exit (1);
	};

	if ($help) {
		Usage();
		exit(0);
	}

	my $stat  = checkServices();

	if($stat>0){
		print "Either ndsd or novell-named service is not up and running.\n
			please bring up these services manually and then re-run the script.\n";
		exit(1);
	}

	$ldapadminname = getTreeAdmin();
	chomp($ldapadminname);
	$ndsadminname = $ldapadminname;
	$ndsadminname =~ s/,/\./g;

	if(not defined($ENV{ADM_PASSWD}) or $ENV{ADM_PASSWD} eq ""){
		while(not $login_success){
			print STDERR "$statement $ldapadminname : ";
			ReadMode('noecho');
			$admin_password = ReadLine(0);
			chomp($admin_password);
			print STDERR "\n";
			ReadMode('normal');

			`/opt/novell/eDirectory/bin/ndslogin $ndsadminname -p $admin_password >/dev/null 2>&1`;
			if($?==0){
				$login_success = 1;
			}else{
				$statement="Wrong password, please re-enter password for";
			}
		}

		$ENV{ADM_PASSWD}=$admin_password;
	}

	print "Please wait, server is getting updated with 2016 schema level.\n\n";
	my $type = registry::getReg("INSTALL_TYPE");
        my $is_replica = registry::getReg("IS_REPLICA");

	#Update domain functional level
	if($domainLevel){
		#Are all the DC's at same schema level?
		isDomainAtLevel2016;

		if($is_replica eq "FALSE"){
			if ($domainAt2016l){
				#Domain updated with the latest schema version
				#if and only all dc's are the the same schema level
				post_schema_extension;
				exit(0);
			}else{
				exit(1);
			}
		}else{
			Log(WARN, "This server is not domain's PDC. So not updating the Domain Functional Level.");
			print "This server is not domain's sPDC. So not updating the Domain Functional Level.\n";
			exit(1);
		}
	}

	#Update forest functional level
	if($forestLevel){
		#check all the DC's for 2016 schema level across the forest
		isForestAtLevel2016;

		if($type eq "frd" && $is_replica eq "FALSE"){
			if ($forestAt2016l){
				#Update the forest to the latest schema level
				post_forest_functional_level;
				exit(0);
			}else{
				exit(1);
			}
		}else{
			Log(WARN, "This server is not FRD PDC. So not updating the Forest Functional Level.");
			print "This server is not FRD PDC. So not updating the Forest Functional Level.\n";
			exit(1);
		}
	}
		
	#add msDS-PrincipalName on the root domain controller
	if($principalName){
		isDomainAtLevel2016;
		if($domainAt2016l){
			createMsDSPrincipalNameAndACL;
			exit(0);
		}else{
			print "ms-DS-PrincipalName is not set as not all domain servers are at 2016 level.\n";
			exit(1);
		}
	}

	# Only FRD
	if($type eq "frd" and $is_replica eq "FALSE"){
		generateSchemaLdif;
		ExtendSchema();
		$test_admin=`echo "$ndsadminname" | sed  's/^CN=//gi' | sed 's/\.[a-zA-Z]*=/./g'`;
		chomp($test_admin);
		`/opt/novell/eDirectory/bin/ldapconfig set "Require TLS for Simple Binds with Password=no" -a $test_admin -w $admin_password >/dev/null 2>&1`;
		sleep(2);
		schema_online_serv;
		schema_bl_online_serv;
		`/opt/novell/eDirectory/bin/ldapconfig set "Require TLS for Simple Binds with Password=yes" -a $test_admin -w $admin_password >/dev/null 2>&1`;
		sleep(1);
	}

	#Update the schema version to the level on DC.
	dcSchemaVersionUpdate;

	#update AES capabilities on the server
	update_domain_to_aes;

	#update the crontab file
	Log(INFO, "Adding the entry for fine grained password policy to crontab.");
	my $rc = system("/opt/novell/xad/share/dcinit/provisionTools.sh  update-crontab-FGP2NMAS");
	if ($rc != 0) {
		die("Could not update the crontab for Fine grained password policy.");
	}

	#Creating PasswordSettings Container for Fine Grained Policies
	createPasswordSettingsContainer;

	delete $ENV{ADM_PASSWD};
	print "\nSystem is updated to 2016 schema level, please check the log file $log_file_name for more details.\n\n";
}
main;

