#!/usr/bin/env python

# (c) Copyright [2020 - 2021] Micro Focus or one of its affiliates.
#
# The only warranties for products and services of Micro Focus and its affiliates and licensors
# (Micro Focus) are as may be set forth in the express warranty statements accompanying such
# products and services. Nothing herein should be construed as constituting an additional
# warranty. Micro Focus shall not be liable for technical or editorial errors or omissions
# contained herein. The information contained herein is subject to change without notice.
#
# Except as specifically indicated otherwise, this document contains confidential information
# and a valid license is required for possession, use or copying. If this work is provided to the
# U.S. Government, consistent with FAR 12.211 and 12.212, Commercial Computer Software,
# Computer Software Documentation, and Technical Data for Commercial Items are licensed
# to the U.S. Government under vendor's standard commercial license.

import os
import glob
import json
import gettext
import fcntl
import commands
import time
from collections import OrderedDict

telemetry_result_loc = "/var/opt/novell/telemetry/data/"
telemetry_config = "/etc/opt/novell/telemetry/config/telemetry.json"
timestamp = int(time.time())


#Write the telemetry data of DSfW service to /var/opt/novell/telemetry/data/DSfW_*.json
def writeJSON(dictionary):
	files = glob.glob(telemetry_result_loc + 'DSfW_*.json')
	for file in files:
		try:
			os.remove(file)
		except Exception:
			raise Exception("Unable to delete old data files of DSfW telemetry from /var/opt/novell/telemetry/data/")

	try:
		with open("/var/opt/novell/telemetry/data/DSfW_" + str(timestamp) + ".json", "w") as dsfwtelemetryfile:
			fcntl.flock(dsfwtelemetryfile, fcntl.LOCK_EX | fcntl.LOCK_NB)
			json.dump(dictionary, dsfwtelemetryfile, indent=4, separators=(", ", ": "))
			fcntl.flock(dsfwtelemetryfile, fcntl.LOCK_UN)
	except IOError:
		raise Exception("Failed to open/write the Json file /var/opt/novell/telemetry/data/DSfW_" + str(timestamp) + ".json")

#To Get DSfW Telemetry status data
def generateTelemetryStatus():

	jsondsfw_stream = OrderedDict(
		[
			('OESService', 'DSfW'),
			('timestamp', timestamp),
			('Domain Type',''),
			('OES VERSION',''),
			('No. of Users in Domain','0'),
			('No. of Active Users','0'),
			('No. of Computers in Domain','0'),
			('No. of Active Computers','0'),
			('No. of Groups','0'),
			('No. of Policy','0'),
			('No. of Sites','0'),
			('No. of Subnet','0'),
			('WINS Configured',''),
			('No. of Domains','0'),
			('No. of DC','0'),
			('No. of DNS Servers','0'),
			('No. of DNS Zones','0')
		])

	admin_name = commands.getoutput("/opt/novell/xad/share/dcinit/printConfigKey.pl NDSADMINNAME")
	if(admin_name.startswith('CN=XAD_PROVISIONING_USER')):
		jsondsfw_stream["Domain Type"] = ("Non-Name-Mapped")
	else:
		jsondsfw_stream["Domain Type"] = ("Name-Mapped")

	domain_name = commands.getoutput("hostname -d")
	no_of_dc = commands.getoutput("dig -t SRV _ldap._tcp.dc._msdcs." + domain_name  + " +short | wc -l")
	jsondsfw_stream["No. of DC"] = no_of_dc

	oes_version = commands.getoutput("cat \/etc\/novell-release | grep -i VERSION")
	oes_version = oes_version.split(" = ")[1]
	jsondsfw_stream["OES VERSION"] = oes_version

	no_of_domains = commands.getoutput("ldapsearch -x -b \"\" '(objectclass=Domain)' | grep -i \"distinguishedName:\" | wc -l")
	jsondsfw_stream["No. of Domains"] = no_of_domains

	no_of_users = commands.getoutput("wbinfo -u | wc -l")
	jsondsfw_stream["No. of Users in Domain"] = no_of_users

	no_of_groups = commands.getoutput("wbinfo -g | wc -l")
	jsondsfw_stream["No. of Groups"] = no_of_groups

	base_domain__context = "dc="+domain_name.replace(".",",dc=")
	no_of_computers = commands.getoutput("ldapsearch -x -b \""+base_domain__context+"\" '(objectclass=computer)' | grep -i \"distinguishedName:\" | grep -i \"cn=Computers,\" | wc -l")
	jsondsfw_stream["No. of Computers in Domain"] = no_of_computers

	no_of_sites = commands.getoutput("ldapsearch -x -b \"\" '(objectclass=site)' |  grep -i \"distinguishedName:\" | grep -i \"cn=Sites,\" | wc -l")
	jsondsfw_stream["No. of Sites"] = no_of_sites

	no_of_subnet = commands.getoutput("ldapsearch -x -b \"\" '(objectclass=subnet)' | grep -i \"distinguishedName:\" | grep -i \"cn=Sites,\" | grep -i \"cn=Subnets,\" | wc -l")
	jsondsfw_stream["No. of Subnet"] = no_of_subnet

	no_of_dns = commands.getoutput("dig " + domain_name + " NS +short | wc -l")
	jsondsfw_stream["No. of DNS Servers"] = no_of_dns

	wins_configured_data = commands.getoutput("/opt/novell/xad/share/dcinit/configure_dsfw_wins.pl --isconfigured")
	if wins_configured_data.find("WINS not configured") == -1:
		if wins_configured_data.find("WINS configured on same server") == -1:
			jsondsfw_stream["WINS Configured"] = "WINS configured on other server"
		else:
			jsondsfw_stream["WINS Configured"] = "WINS configured on same server"
	else:
		jsondsfw_stream["WINS Configured"] = "WINS not configured"

	no_of_policy = commands.getoutput("ls \/var\/opt\/novell\/xad\/sysvol\/sysvol\/Policies | wc -l")
	jsondsfw_stream["No. of Policy"] = no_of_policy

	no_of_active_users = commands.getoutput("grep \"Impersonated user\" /var/log/messages | awk '{print $7}' | cut -d '@' -f1 | grep -v '\\\\' | sort -u | grep -v '$$' | grep -v times: | wc -l")
	jsondsfw_stream["No. of Active Users"] = no_of_active_users

	no_of_active_computers = commands.getoutput("grep \"Impersonated user\" /var/log/messages | awk '{print $7}' | cut -d '@' -f1 | grep -v '\\\\' | sort -u | grep '$$' | grep -v times: | wc -l")
	jsondsfw_stream["No. of Active Computers"] = no_of_active_computers

	no_domain_zone = commands.getoutput("ldapsearch -x -b \"\" '(objectclass=dNIPDNSZone)' | grep distinguishedName | grep -v 'RootServerInfo' | grep -v 'in-addr_arpa' | wc -l")
	jsondsfw_stream["No. of DNS Zones"] = no_domain_zone

	return jsondsfw_stream

def telemetry_check():
	try:
		with open(telemetry_config, "r") as file:
			status = json.load(file)
			if status["opt-in"].lower() == "yes":
				return True
	except IOError:
		pass
	return False

# generate only if telemetry is enabled
enabled = telemetry_check()
if enabled:
	dsfwTelemetryData = generateTelemetryStatus()
	try:
		system_status = commands.getoutput("systemctl is-system-running")
		if system_status == "initializing" or system_status == "starting":
			print("Cannot Execute the script as the system is not fully up and running")
		else:
			writeJSON(dsfwTelemetryData)
	except Exception:
		exit(1)
else:
	print("Telemetry is not configured.")
