#!/usr/bin/perl -I. -I.. -I/opt/novell/xad/lib64/perl -I/opt/novell/xad/lib/perl
################################################################################
# DSfW Migration script from sp3 to oes11
#
# See the file LICENSE for redistribution information.
#
# Copyright (c) 2011
#       Novell, Inc.  All rights reserved.
################################################################################
#
# Note:
# Please do the following before executing this script
# Execute the command on target machine "ssh-keygen -t rsa"
# Copy it to source "scp /root/.ssh/id_rsa.pub source:/tmp"
# At source check for the directory /root/.ssh if not present create it
# Append the key details from /tmp/id_rsa.pub to /root/.ssh/authorized_keys as
# "cat /tmp/id_rsa.pub >> /root/.ssh/authorized_keys"
#
################################################################################
$ENV{'PATH'} = "/opt/novell/xad/share/dcinit:$ENV{'PATH'}";

use strict;
use warnings;

use Getopt::Long;
use helpers::utility qw(validateIP);
use Logger qw(:DEFAULT);
use migration::migrate_server;

main(@ARGV);

sub croak { die "$0: @_: $!\n" }

sub Usage {

        print <<USAGE;
        Usage: migrate_dsfw.pl [options] [attributes]

	options:
	--source=ip     takes (ip) address of the source server
	--miggui	enabling this option does the migration of associated services
	--premig	enabling this option does the premig task for DSfW migration
	--postmig	enabling this option does the postmig task for DSfW migration
	--all		enables premig miggui postmig task for DSfW migration
USAGE
        exit 0

}

sub prompt {
  my ($query) = @_; # take a prompt string as argument
  local $| = 1; # activate autoflush to immediately show the prompt
  print STDERR $query;
  chomp(my $answer = <STDIN>);
  return $answer;
}

sub prompt_yn {
  my ($query) = @_;
  my $answer = prompt("$query (Y/N): ");
  return lc($answer) eq 'y';
}



sub mixed_mode_check() {
    my $mixed_mode_warning_message;

    $mixed_mode_warning_message="All Domain Controllers must have the same version of OES.
After this migration, ensure to upgrade/migrate all the other domain controllers in this domain and in other domains to the same version of OES.\n";
    print STDERR "$mixed_mode_warning_message";
    if (prompt_yn("Do you want to continue migration")){
        print STDERR "Continuing migration\n";
    }
    else {
        print STDERR "Quitting migration\n";
        exit 3;
    }
}

sub main{
	my ($error, $help, $all);

	Logger::set_filename($migration::migrate_log_file);

	($help, $all) = (0, 0);
	my $result = GetOptions("--premig" => \$migration::premig,
				"--postmig" => \$migration::postmig,
				"--help" => \$help,
				"--all" => \$all,
				"--miggui" => \$migration::miggui,
				"--source=s" => \$migration::sourceIp) or Usage();
	if ($help) {
		Usage();
	}

	if ($migration::sourceIp eq 0) {
		print "\tError - IP Address of the source server is not provided\n\n";
		Usage();
	}

	$error = helpers::validateIP($migration::sourceIp);
        if ($error) {
                Log(ERR, "The IP Address validation failed for IP [$migration::sourceIp]");
        }

	if ($all) {
		$migration::premig = 1;
		$migration::miggui = 1;
		$migration::postmig = 1;
	}

	mixed_mode_check();
	migration::migrate_now($migration::sourceIp);
}
