#!/bin/sh
################################################################################
# Domain services for Windows tool for fine-grained password policy
# 
#
# See the file LICENSE for redistribution information.
#
# Copyright (c) 2019-2020
#       Micro Focus or one of its affiliates
################################################################################

# This script modifies or updates the XAD.ini for Resultant-PSO value to enable 
# or disable the execution fine-grained password policy code path

xad_ini_value=NULL
XAD_INI_FILE="/etc/opt/novell/xad/xad.ini"

check_resultant_pso_in_xad_ini(){
   string="RESULTANT_PSO"
   while read line; do
        if [[ $line == *$string* ]]
           then
		xad_ini_value=$(echo $line | awk -F " = " '{print $2'})	
                return 1
        fi
   done < $XAD_INI_FILE
   return 0
}

# If the xad.ini already has an entry for RESULTANT_PSO, the function tries to modify the entry value
# e.g. if xad.ini contains RESULTANT_PSO = Disabled, and the script is run with option --enable
# xad.ini gets modified with RESULTANT_PSO = Enabled
modify_resultant_pso(){
   value=$1
   sed -i "s/RESULTANT_PSO = ${xad_ini_value}/RESULTANT_PSO = ${value}/g" "$XAD_INI_FILE"
   if [ $? -eq  0 ]
   then
	echo "The Resultant-PSO in xad.ini has been modified."
   else
	echo "Failed to modify the xad.ini for Resultant-PSO as requested."
	exit 1
   fi   
}

# The function updates the xad.ini for the RESULTANT_PSO entry initially if file is missing the entry;
# later modification is dealt by modify_resultant_pso
# e.g. If xad.ini does not contain RESULTANT_PSO entry, by running the script with --enable option
# xad.ini gets updated with  RESULTANT_PSO = Enabled
write_resultant_pso(){
   value=$1
   string="RESULTANT_PSO = $value"

   echo $string >> $XAD_INI_FILE
   EXIT_STATUS=$?
   if [ "$EXIT_STATUS" -eq "0" ]
   then
	echo "Successfully update the Resultant_PSO in the xad.ini as $value"
   else
	echo "Failed to update Resultant_PSO in the xad.ini"
   fi
}

main(){
   check_resultant_pso_in_xad_ini
   exists=$?
   if [ "$exists" == "1" ]; then
	if [ "$parameter_passed" == "Enabled" -a "$xad_ini_value" == "Enabled" ]
	then
		echo "Value is already updated as $xad_ini_value"

	elif [ "$parameter_passed" == "Enabled" -a "$xad_ini_value" == "Disabled" ]
	then
		modify_resultant_pso $parameter_passed

	elif [ "$parameter_passed" == "Disabled" -a "$xad_ini_value" == "Enabled" ]
	then
		modify_resultant_pso $parameter_passed

	elif [ "$parameter_passed" == "Disabled" -a "$xad_ini_value" == "Disabled" ]
	then
		echo "Value is already updated as $xad_ini_value"

	else
		echo " Something is wrong here !!! "
		echo " To enable fine grained policy, Run script.sh --enabled "
		echo " Or to disable fine grained policy, Run script.sh --diabled \n"
	fi
   elif [ "$exists" == "0" ]; then
	if [ -z "$parameter_passed" ]
	then
           echo " Something is wrong here !!! "
           echo " To enable fine grained policy, Run script.sh --enabled "
           echo " Or to disable fine grained policy, Run script.sh --diabled"
	   exit 1
	else
	   write_resultant_pso $parameter_passed
	fi
   else
	echo " Something is wrong here !!! "
	echo " To enable fine grained policy, Run script.sh --enabled "
	echo " Or to disable fine grained policy, Run script.sh --diabled" 
	exit 1
   fi
}

# Checking for the options to set the value in xad.ini
case "$1" in
    --e | --enable)
	parameter_passed=Enabled
	;;
    --d | --disable)
	parameter_passed=Disabled
	;;
    -h | h | --help | help)
	echo -e " Usage: "
	echo -e " Script options: "
	echo -e " --disable | --d       disables the execution of compute_resultantPSO"
	echo -e " --enable | --e        enables the execution of compute_resultantPSO"
	echo -e " Restart of the services is necessary for the cahnges to take place"
	exit 0
	;;
esac

# Calling the main function
main
