#!/usr/bin/perl -I. -I.. -I/opt/novell/xad/lib64/perl -I/opt/novell/xad/lib/perl
################################################################################
# DSfW Upgrade script
#
# See the file LICENSE for redistribution information.
#
# Copyright (c) 2001-2020
#	Micro Focus or one of its affiliates.
################################################################################
use upgrade2sp2;
use upgrade2sp3;
use upgrade2oes11;
use upgrade2oes11sp1;
use upgrade2oes11sp2;
use upgrade2oes2015;
use upgrade2oes2015sp1;
use upgrade2oes2018;
use upgrade2oes2018sp1;
use upgrade2oes2018sp2;
use upgrade2oes2018sp3;
use upgrade2oes2023;
use upgrade2oes2023sp1;
use Logger qw(:DEFAULT);
use util;
use util_update;

sub check_and_start_ndsd() {

`systemctl status ndsd`;
my $ndsd_status = $?>>8;

if ($ndsd_status == 3) {
		Log("ndsd not up. Starting it");
		`systemctl start ndsd`;
	}

}

Logger::set_filename("/var/opt/novell/xad/log/ndsdcinit.log");
my $upgrade_from = registry::getReg("UPGRADE_FROM");
my $upgrade_to = registry::getReg("UPGRADE_TO");

if ($upgrade_from eq "sp1") {
	upgrade2sp2::upgrade_sp2();
	upgrade2sp3::upgrade_sp3();
	upgrade2oes11::upgrade_oes11();
	upgrade2oes11sp1::upgrade_oes11sp1();
	upgrade2oes11sp2::upgrade_oes11sp2();
	upgrade2oes2015::upgrade_oes2015();
	upgrade2oes2015sp1::upgrade_oes2015sp1();
	upgrade2oes2018::upgrade_oes2018();
	upgrade2oes2018sp1::upgrade_oes2018sp1();
	upgrade2oes2018sp2::upgrade_oes2018sp2();
	upgrade2oes2018sp3::upgrade_oes2018sp3();
	upgrade2oes2023::upgrade_oes2023();
} elsif ($upgrade_from eq "sp2") {
	upgrade2sp3::upgrade_sp3();
	upgrade2oes11::upgrade_oes11();
	upgrade2oes11sp1::upgrade_oes11sp1();
	upgrade2oes11sp2::upgrade_oes11sp2();
	upgrade2oes2015::upgrade_oes2015();
	upgrade2oes2015sp1::upgrade_oes2015sp1();
	upgrade2oes2018::upgrade_oes2018();
	upgrade2oes2018sp1::upgrade_oes2018sp1();
	upgrade2oes2018sp2::upgrade_oes2018sp2();
	upgrade2oes2018sp3::upgrade_oes2018sp3();
	upgrade2oes2023::upgrade_oes2023();
} elsif ($upgrade_from eq "sp3") {
	upgrade2oes11::upgrade_oes11();
	upgrade2oes11sp1::upgrade_oes11sp1();
	upgrade2oes11sp2::upgrade_oes11sp2();
	upgrade2oes2015::upgrade_oes2015();
	upgrade2oes2015sp1::upgrade_oes2015sp1();
	upgrade2oes2018::upgrade_oes2018();
	upgrade2oes2018sp1::upgrade_oes2018sp1();
	upgrade2oes2018sp2::upgrade_oes2018sp2();
	upgrade2oes2018sp3::upgrade_oes2018sp3();
	upgrade2oes2023::upgrade_oes2023();
} elsif ($upgrade_from eq "oes11") {
	upgrade2oes11sp1::upgrade_oes11sp1();
	upgrade2oes11sp2::upgrade_oes11sp2();
	upgrade2oes2015::upgrade_oes2015();
	upgrade2oes2015sp1::upgrade_oes2015sp1();
	upgrade2oes2018::upgrade_oes2018();
	upgrade2oes2018sp1::upgrade_oes2018sp1();
	upgrade2oes2018sp2::upgrade_oes2018sp2();
	upgrade2oes2018sp3::upgrade_oes2018sp3();
	upgrade2oes2023::upgrade_oes2023();
} elsif ($upgrade_from eq "oes11sp1") {
	upgrade2oes11sp2::upgrade_oes11sp2();
	upgrade2oes2015::upgrade_oes2015();
	upgrade2oes2015sp1::upgrade_oes2015sp1();
	upgrade2oes2018::upgrade_oes2018();
	upgrade2oes2018sp1::upgrade_oes2018sp1();
	upgrade2oes2018sp2::upgrade_oes2018sp2();
	upgrade2oes2018sp3::upgrade_oes2018sp3();
	upgrade2oes2023::upgrade_oes2023();
} elsif ($upgrade_from eq "oes11sp2") {
	upgrade2oes2015::upgrade_oes2015();
	upgrade2oes2015sp1::upgrade_oes2015sp1();
	upgrade2oes2018::upgrade_oes2018();
	upgrade2oes2018sp1::upgrade_oes2018sp1();
	upgrade2oes2018sp2::upgrade_oes2018sp2();
	upgrade2oes2018sp3::upgrade_oes2018sp3();
	upgrade2oes2023::upgrade_oes2023();
} elsif ($upgrade_from eq "oes2015"){
	upgrade2oes2015sp1::upgrade_oes2015sp1();
	upgrade2oes2018::upgrade_oes2018();
	upgrade2oes2018sp1::upgrade_oes2018sp1();
	upgrade2oes2018sp2::upgrade_oes2018sp2();
	upgrade2oes2018sp3::upgrade_oes2018sp3();
	upgrade2oes2023::upgrade_oes2023();
} elsif ($upgrade_from eq "oes2015sp1" or $upgrade_from eq "oes11sp3"){
	upgrade2oes2018::upgrade_oes2018();
	upgrade2oes2018sp1::upgrade_oes2018sp1();
	upgrade2oes2018sp2::upgrade_oes2018sp2();
	upgrade2oes2018sp3::upgrade_oes2018sp3();
	upgrade2oes2023::upgrade_oes2023();
} elsif ($upgrade_from eq "oes2018"){
        upgrade2oes2018sp1::upgrade_oes2018sp1();
	upgrade2oes2018sp2::upgrade_oes2018sp2();
	upgrade2oes2018sp3::upgrade_oes2018sp3();
	upgrade2oes2023::upgrade_oes2023();
} elsif ($upgrade_from eq "oes2018sp1"){
	upgrade2oes2018sp2::upgrade_oes2018sp2();
	upgrade2oes2018sp3::upgrade_oes2018sp3();
	upgrade2oes2023::upgrade_oes2023();
} elsif ($upgrade_from eq "oes2018sp2"){
	upgrade2oes2018sp3::upgrade_oes2018sp3();
	upgrade2oes2023::upgrade_oes2023();
} elsif ($upgrade_from eq "oes2018sp3"){
	upgrade2oes2023::upgrade_oes2023();
}

#Upgrade to OES2023
upgrade2oes2023sp1::upgrade_oes2023sp1();

# Updating the usr.sbin.nmbd apparmor profile
util::update_apparmor_nmbd();

# Updating the usr.sbin.smbd apparmor profile
util::update_apparmor_smbd();

# Updating the usr.sbin.samba-bggd apparmor profile
util::update_apparmor_samba_bgqd();

# Updating the usr.sbin.winbindd apparmor profile
util::update_apparmor_winbindd();

#Restarting apparmor to get the apparmor profiles changes of nmbd, smbd and winbindd
my $service_start_status;

`/usr/bin/systemctl restart apparmor.service`;
$service_start_status = $?;
Log(WARN, "Service apparmor restarted with code $service_start_status");

#Restart Services
util::restart_services();

#Make "gppdate /force" work in workstation
util_update::update_for_win10();

#Restart Services
util::restart_services();
