#!/usr/bin/perl -I. -I.. -I/opt/novell/xad/lib64/perl -I/opt/novell/xad/lib/perl
################################################################################
# DSfW  DC install parameter fixes
#
#
# See the file LICENSE for redistribution information.
#
# Copyright (c)
#	Novell, Inc.  All rights reserved.
################################################################################
$ENV{'PATH'} = "/opt/novell/xad/share/dcinit:$ENV{'PATH'}";

use strict;
use warnings;
use v5.10;# at least for Perl 5.10
no warnings 'experimental::smartmatch'; #Avoid known warnings with given-when
use XAD::registry;
use NDS::stage;
use Logger qw(:DEFAULT);
use File::Copy;
use File::Temp;
use File::Path;
use Config::IniFiles;

# This lock needs to updated for successive patches if the script
# needs to be executed for any specific patch cycle. It will point
# to a prev successful run level.
my $marker_file = "/var/opt/novell/xad/ds/UpdateDC.run_sept2012.lock";

my $msdfs_dir = "/var/opt/novell/xad/msdfs";
my $xad_ini = "/etc/opt/novell/xad/xad.ini";

#
# Correct the DFS link to point one level lesser b'cos of fix Bug#703655
# old link : msdfs:adcserver\sysvol-msdfs\domain.com,frdserver\sysvol\domain.com
# new link: msdfs:adcserver\sysvol-msdfs,frdserver\sysvol -- Removed suffixed domain name
#
sub updateDFSLink() {

	my $root_domain = `grep "Root Domain" $xad_ini |  awk '{print \$NF}'`;
	chomp($root_domain);

	if ($root_domain eq "") {
		# Fallback to dnsdomainname and expect it to be set right
		$root_domain = `/bin/dnsdomainname`;
		chomp($root_domain);
		if ($root_domain eq "") {
			Log("Unable to deduce Root Domain name. Exiting..");
			Log("DFS link Update Failed.. Follow TID 7009466");
			return 0;
		}
	}

	my $msdfs_link = "$msdfs_dir/$root_domain";
	if (-l "$msdfs_link") {

		my $group = `ls -n $msdfs_link 2>/dev/null | awk '{print \$4}'`;
		my $owner = `ls -n $msdfs_link  2>/dev/null | awk '{print \$3}'`;
		chomp($group);
		chomp($owner);

		if ( $group eq "" || $owner eq "" ) {
			Log("Unable to deduce owner or group information. Exiting..");
			Log("DFS link Update Failed.. Follow TID 7009466");
			return 0;
		}

		my $current_link = `ls -l  "$msdfs_link" | awk  -F" " '{print \$NF}'`;
		my $new_link = `ls -l  "$msdfs_link" | awk '{print \$NF}' | sed  's/\\\\$root_domain,/,/i;s/\\\\$root_domain\$//i'`;
		chomp($current_link);
		chomp($new_link);

		if ($new_link eq "") {
			Log("Unable to deduce DFS link value. Exiting..");
			Log("DFS link Update Failed.. Follow TID 7009466");
			return 0;
		}

		if ($new_link eq $current_link) {
			Log("DFS Link already updated..");
			return 0;
		}
		Log("Changing msdfs link from $root_domain -> $current_link");
		Log("	to  $root_domain -> $new_link");
		`ln -sf "$new_link" "$msdfs_link"`;
		`chown -R "$owner"\:"$group" $msdfs_dir`;
		
		my $new_owner = `ls -n "$msdfs_link" | awk '{print \$3}'`;
		chomp($new_owner);

		if ($new_owner eq $owner) {
			Log("DFS link Updated successfully");
			return 0;
		} else {
			Log("DFS link Update Failed.. Follow TID 7009466");
			return 0;
		}

	} else {
		# Do nothing as this is not ADC
		return 0; 
	}
}

# This is needed on OES2SP3 upgraded setups to fix the create mask and directory
# mask values such the domain users wont be able to edit the POLICIES folder under SYSVOL.
# Upgrade the smb.conf file to latest sp3 changes
# mask values such that domain users wont be able to edit the POLICIES folder under SYSVOL.
sub upgrade_smb_conf_file() {
    my $smb_file = "/etc/samba/smb.conf";
    my $loop = 2;
    my $section = "";
    my $val = "";

    my $cfg = Config::IniFiles->new (-file=> $smb_file, -nocase=>1);
    if (!defined $cfg) {
	Log("Could not open the file $smb_file");
        return 0;
    }
	my $smbconf_backup = mktemp("/var/opt/novell/xad/ds/smb.conf.XXXXXXXXXX");
	Log("Current smb.conf backed-up to $smbconf_backup");
	copy($smb_file, $smbconf_backup); #Ignore failure. 
    do {
        given($loop) {
            when(1) {$section = 'sysvol';}
            when(2) {$section = 'sysvol-msdfs';}
            default {last;}
        }
        $loop--;
        $val = $cfg->val($section, 'create mask');
        if (defined $val) {
            $cfg->delval ($section, 'create mask');
        }
        $val = $cfg->val($section, 'directory mask');
        if (defined $val) {
            $cfg->setval ($section, 'directory mask', '0750');
        }
        $val = $cfg->val($section, 'path');
        if ((!defined $val) && ($section eq 'sysvol-msdfs')) {
            $cfg->DeleteSection($section);
        }
    } while ($loop > 0);

    $cfg->RewriteConfig();
    `sed -i 's/^[a-zA-Z]/\t&/g' $smb_file`;
    `sed -i 's/=/ = /g' $smb_file`;
    `setfacl -R -m g::r-x  /var/opt/novell/xad/sysvol/domain/Policies/ 2>&1 > /dev/null`;
    Log("Modified $smb_file file successfully");
    return 0;
}

#Update the /etc/chrony.conf file with 'global' keyword
sub update_ntp_conf_file {
	my $file_name = shift;

	Log("Updating NTP configuration");

	if (-f $file_name) {
		my $ntpconf_backup = mktemp("/etc/chrony.conf.XXXXXXXXXX");
		Log("Current chrony.conf backed-up to $ntpconf_backup");
		copy("/etc/chrony.conf", $ntpconf_backup) or
		Log("Could not backup /etc/chrony.conf to $ntpconf_backup");

		`sed -i -e '/^authprov/{/\\s*global\\s*/! s/.*/& global/}' $file_name`;
    	Log("Modified $file_name file successfully");
	} 

    return 0;
}

#Adding commands to create link to the newly created directory structure
#under /var/lib/chrony i.e. ntp chroot
sub update_xadsd_file(){
	my $xadsd_backup=mktemp("/etc/init.d/xadsd.XXXXXXXXXX");
	my $template_dir="/opt/novell/xad/share/dcinit/templates/";
	my $file_name="/etc/init.d/xadsd";

	copy("/etc/init.d/xadsd", $xadsd_backup) or
	Log("Could not copy the /etc/init.d/xadsd to $xadsd_backup");
	unlink ("/etc/init.d/xadsd");

	Log("Updating file /etc/init.d/xadsd");
	stage::reg_substitute ("$template_dir/xadsd.init", $file_name);

	Log("Changing the permission of file /etc/init.d/xadsd");
	chmod(0755, $file_name);
   	Log("Modified $file_name file successfully");
}

sub update_ntp_changes() {

	#Creating directory structure /var/opt/novell/xad/rpc in ntp chroot jail
	eval{
		mkpath( "/var/lib/chrony/var/opt/novell/xad/rpc" );
	};
	if($@) {
		Log(WARN,$@); #Printing ERR message and a backtrace
	}

	#Update chrony.conf file
	update_ntp_conf_file("/etc/chrony.conf");

	#Update xadsd file
	update_xadsd_file();

	Log ("Successfully applied ntp changes");
}

sub usage(){
		print <<USAGE
Usage:	UpdateDC.pl poststage
USAGE

}

#
# This script is called from rpm post install portion, so it should never error out in the 'rpm' section..
# Anything that requires LDAP operation or edirectory interaction must be called only with in the 'post' section..
#
sub main() {

	# Do it only if DSFW is configured 
	if (!-e $xad_ini) {
		return 0;
	}

	Logger::set_filename("/var/opt/novell/xad/log/config_changes.log");
	if (scalar(@ARGV) == 0) {
		usage();
		return 0;
	}
	my $action = shift @ARGV;
	if ($action eq "rpmstage") {
		# Execution during the YAST patching in rpm spec,
		# this portion should require no dependency on edir to be running.
		# This should never error out. 

		if ( -e $marker_file ){
			return 0;
		}

		if ( -e "/etc/sysconfig/novell/xad_oes11_sp1") {
			upgrade_smb_conf_file();
		} elsif (-e "/etc/sysconfig/novell/xad2_oes11"){
			# The upgraded and migrated system do have the old sysconfig files
			# Hence keeping it in else loop rather than in another if check 
			# checking for oes2sp3 and oes11fp0
			upgrade_smb_conf_file();
			updateDFSLink();
		} elsif (-e "/etc/sysconfig/novell/xad2_sp3"){
			upgrade_smb_conf_file();
			update_ntp_changes();
			`grep -q  "PATCHLEVEL \\+= \\+4" /etc/SuSE-release`;
			if ($? eq 0) {
				# Corresponding Samba patch is available only in SLES10 SP4
				updateDFSLink();
			}
		}
		
		# Make sure this marker is set at end of execution
		`touch $marker_file`;
	} elsif ($action eq "poststage") {
		# Execution after YAST patching,
		# this portion requires edir to be running.
		util::update_spn();
	} else {
		usage();
	}
	return 0;
}

main
