#!/usr/bin/perl -I/opt/novell/xad/lib/perl -I/opt/novell/xad/lib64/perl

use strict;
use warnings;
use Sys::Hostname;
use Net::Ping;
use File::Copy;

use lib "GSSAPI-0.12";
use GSSAPI;
use Net::DNS;

use File::Temp;
use Getopt::Long;
use POSIX;
use Locale::gettext;
use Test::More qw(no_plan);
use Getopt::Std;

use XAD::registry;
use Term::ReadKey;
use XAD::dns;

# Setting the Locale
setlocale(LC_MESSAGES, "");
bindtextdomain("UpdatePDCMaster", "/opt/novell/xad/share/locale");
textdomain("UpdatePDCMaster");


# Debug flag, setting it to 1 throws lots of debug messages
my $debug = 0;

my $domain = `/opt/novell/xad/share/dcinit/printConfigKey.pl "DNS Root"`;
chomp ($domain);
my $domain_server_name = "";
my $key_name1 = int(rand 10000000000000);
my $key_name = "$key_name1" ."-DSfW-123451";

my $MS_DNS = 0; # Not an MS-DNS server

my $nameserver = "";
my @authservers = ();
my $authservercount = 0;
my $myLocalhost = "127.0.0.1";

# get the local machine's hostname in uppercase
my $hostname = uc(hostname());
chomp $hostname;

my $status = 0; 
my @Existing_PDC_Masters = ();
my $weight = 0;
my $priority=  100;
my $port=  389;
my %Existing_PDC_Masters_record_values;

my $localhost_fqdn = `/opt/novell/xad/share/dcinit/printConfigKey.pl "DNS Host Name"`;
chomp $localhost_fqdn;

my $pdc_master;
my $Is_PDC_Master_LDAP_Backend = 0;
my $Is_PDC_Master_DNS = 0;
my $role_seizure = 0;
my $pipe_status_file = mktemp("/tmp/pipe_status.XXXXXXXX");
if(defined($pipe_status_file)) {
	$ENV{'PIPE_STATUS_FILE'} = $pipe_status_file;
}	

my $admin_sam_name = `/opt/novell/xad/share/dcinit/provisionTools.sh get-admin-sam-name`;
chomp $admin_sam_name;

#######################
### Error codes #######

my $ERR_NOT_A_ROOT_USER = -11;
my $ERR_GSSAPI_BIND_ERR = -12;
my $ERR_NO_PDC_ENTRY_IN_DNS = -13;
my $ERR_NO_AUTHORITATIVE_PDC_QUERY_ANSWERS_FROM_DNS = -14;
my $ERR_DNS_RR_PACKET_HANDLE_CREATION_FAILED = -15;
my $ERR_SENDING_TKEY_PACKET = -16;
my $ERR_VALIDATION_TKEY_FAILED = -17;
my $ERR_NO_REPLY_TO_DYNAMIC_UPDATE = -18;
my $ERR_BAD_LAST_PDC_MASTER_COUNT = -19;
my $ERR_CREATING_TEMP_TRANSFER_FILE = -20;
my $ERR_CHANGING_OWNERSHIP_TEMP_TRASNFER_FILE = -21;
my $ERR_FILE_NOT_FOUND = -22;
my $ERR_TEMP_FILE_PERMISSION_CHANGE = -23;
my $ERR_FILE_MOVE_ERROR = -24;
my $ERR_CANNOT_OPEN_FILE = -25;
my $ERR_CANNOT_RESOLVE_LAST_PDC_MASTER = -26;
my $ERR_CANNOT_REACH_TO_LAST_PDC_MASTER = -27;
my $ERR_INVALID_SAM_NAME_TKT = -28;
my $ERR_NO_CREDENTIALS_CACHE_FOUND = -29;


####################################################
###########Functions################################

sub gss_exit {
	my $status = shift;

	my @major_errors = $status->generic_message();
	my @minor_errors = $status->specific_message();

	my ($major_err, $minor_err) = 0;

	foreach $major_err (@major_errors) {
		print STDERR "	MAJOR::$major_err\n" if($debug);
	}

	foreach $minor_err (@minor_errors) {
		print STDERR "	MINOR::$minor_err\n" if($debug);
	}

	return $major_err;
}

# Check EUID ##
###############

sub Check_EUID() {

	my $EUID = $>;

	# check if the EUID is of root (0) or not
	if($EUID != 0)
	{
		# Only a user with effective id as root can run the 
		# process, abort stating the same.
		print STDERR gettext("Run this script as root user. Permission Denied"),"\n";
		exit $ERR_NOT_A_ROOT_USER;
	}
}

# Validate acquired credentials for valid Administrator SAM-Account-Name
###########################################################

sub validate_tkt_for_admin_sam_name {
	my $klist_command = '/opt/novell/xad/bin/klist 2>/dev/null | grep "Default principal: " | awk -F": " \'{print $2}\' | awk -F@ \'{print $1}\'';
	my $default_princ = `$klist_command`;
	chomp($default_princ);
	if($?) {
		print STDERR gettext("No credentials cache found."), "\n";
		exit $ERR_NO_CREDENTIALS_CACHE_FOUND;
	}
	else {
		if (lc($admin_sam_name) ne lc($default_princ)) {
#			$default_princ =~ s/Default principal:\s+(.*)\@.*/$1/g;
			print STDERR sprintf (gettext("Invalid Administrator SAM-Account-Name [%s] in the ticket.\n"),$default_princ);
			exit $ERR_INVALID_SAM_NAME_TKT;
		}
		# there might be a chance that the admin_sam_name and the default_princ values
		# just differ in case. So assigning default_princ to admin_sam_name

		$admin_sam_name = $default_princ

	}
}


## Check if the local DC is the PDC master ##
#############################################

sub Is_PDC_Master($)
{
	my $Is_PDC_Master_Flg_Ref = shift;

	my $return_val = 0;

	my $ldapsearch = "/usr/bin/ldapsearch";
	my $ldap_filter = ' | sed " /^ / {; H; d; }; /^ /! {; x; s/\n //; }; "';
	my $ldap_host = "$myLocalhost";

	my $Lib = `/opt/novell/xad/share/dcinit/printConfigKey.pl "_Lib"`;
	chomp $Lib;

	my $mapped_domainnc = `/opt/novell/xad/share/dcinit/printConfigKey.pl "Mapped Domain NC"`;
	chomp($mapped_domainnc);
	my $domain_provider = `/opt/novell/xad/sbin/provision -q -q -l 60 --locate-dc $domain`;
	if ($?) {
		$return_val = $? >> 8;
		print STDERR sprintf (gettext("Error returned: %s"), $return_val), "\n";
		goto out;
	}
	chomp($domain_provider);

	my $command = "SASL_PATH=/opt/novell/xad/$Lib/sasl2/ $ldapsearch -h $ldap_host -Y GSSAPI -b \'$mapped_domainnc\' -s base fsmoroleowner -LLL 2>/dev/null $ldap_filter  | grep -i \"fsmoroleowner: \" |  awk -F \": \" \'{print \$2}\' | awk -F \"cn=NTDS Settings,\" \'{print \$2}\'";

	my $pdc_server = `$command; echo \${PIPESTATUS[0]} > $ENV{'PIPE_STATUS_FILE'}`;
	my $ldapsearch_err = `cat $ENV{'PIPE_STATUS_FILE'}`;
	chomp $ldapsearch_err;
	chomp $pdc_server;

	if($ldapsearch_err)
	{
		# error encountered
		print STDERR sprintf(gettext("Error returned: %s"), $ldapsearch_err),"\n";
		$return_val = $ldapsearch_err;
		goto out;
	}

	$command = "SASL_PATH=/opt/novell/xad/$Lib/sasl2/ $ldapsearch -h $ldap_host -Y GSSAPI -b \'$pdc_server\' -s base dnsHostName -LLL 2>/dev/null $ldap_filter  | grep -i \"dnsHostName: \"  |  awk -F \": \" \'{print \$2}\' | awk -F \".\" \'{print \$1}\'";
	
	my $pdc_master = `$command; echo \${PIPESTATUS[0]} > $ENV{'PIPE_STATUS_FILE'}`;
	$ldapsearch_err = `cat $ENV{'PIPE_STATUS_FILE'}`;
	chomp $ldapsearch_err;
        chomp($pdc_master);

	if($ldapsearch_err)
	{
		# error encountered
		print STDERR sprintf(gettext("Error returned: %s"), $ldapsearch_err),"\n";
		$return_val = $ldapsearch_err;
		goto out;
	}

	# check if $FSMOroleowner points to $hostname DC
	if(lc($pdc_master) eq lc($hostname))
	{
		# current DC is stored as FSMO Role owner in Directory backend
		# de-ref the Is_PDC_Master_Flg flag and set it
		$$Is_PDC_Master_Flg_Ref = 1;

	} else {
		# match not found.. return 0
		$$Is_PDC_Master_Flg_Ref = 0;
	}

out:
	return $return_val;

}


## Find an authoritative nameserver for the domain ##
#####################################################
sub find_nameserver_for_domain($)
{
	my $domain = shift;
	my $rc = 0;

	# we need to parse the /etc/resolv.conf file, picking up
	# each nameserver entry and querying it to the domain passed
	# incase an authoritative answer is found, the nameserver in 
	# account is to be used as the authoritative nameserver.

	my $file = "/etc/resolv.conf";
	my $readline = "";
	my @Nameservers = "";

	if(!(-e "$file"))
	{	
		$rc = 1;
		print STDERR sprintf(gettext("Hard Error! %s %s"), $rc, $file),"\n";
		exit $rc;
	}
	open(Filename, $file) or die "cannot open $file:$@";
	while ($readline = <Filename>)
	{
		if($readline =~ /^nameserver/)
		{
			#my $nameserver = "";
			print "nameserver entry found:" if($debug);
			#print "$readline \n" if($debug);
			#push the nameserver IP in the nameservers array
			$readline =~ s/nameserver //g;
			chomp $readline;
			push(@Nameservers, $readline);
			print "$readline \n" if($debug);
		}
	}
	close(Filename);
	print "Nameservers array contain:@Nameservers\n" if($debug); 

	# parse through the nameserver array and check for an authoritative
	# nameserver for the passed domain.
	my $Nameservers_count = $#Nameservers;
	while($Nameservers_count != 0)
	{

		# check if this Namservers[$Nameservers_count] is an authoritative
		# nameserver for the domain or not
		my $handle = Net::DNS::Resolver->new;
		$handle->tcp_timeout(60);
		$handle->udp_timeout(60);
		
		# set the nameserver to $Nameservers[$Nameservers_count]
		$handle->nameservers("$Nameservers[$Nameservers_count]");
		my $domain_query = $handle->send("$domain", "A");
		if(defined($domain_query))
		{
			print "nameserver $Nameservers[$Nameservers_count] found for $domain\n" if($debug);
			
			# check for authoritative answer
			my $Is_Authoritative = $domain_query->header->aa;
			if($Is_Authoritative)
			{
				# this is the nameserver for the $domain, check the 
				# returned IPs
				
				print "printing the domain_query response\n" if($debug);
				print "======================================\n" if($debug);
				$domain_query->print if($debug);
				print "======================================\n" if($debug);

				my $replyNS = $handle->query("$domain","NS");
				my $authcnt = $replyNS->answer;
				print "Found $authcnt authoritative server\n" if($debug && $authcnt <= 0);
				print "Found $authcnt authoritative servers\n" if($debug && $authcnt > 1);

				if ($authcnt > 0) {
					$authservercount = $authcnt;
					@authservers =  $replyNS->answer;
					last;
				}
			}
		}
		$Nameservers_count--;

	}
	if ($debug) {
		print "The Authoritative DNS servers are :\n" if $authservercount > 1;
		print "The Authoritative DNS server is :\n" if $authservercount == 1;
		foreach my $serv (@authservers) {
			print $serv->nsdname;
			print "\n";
		}
	}
}


# Verify the PDC master DC with DNS server ##
#############################################
sub DNS_PDC_Master_query()
{
	# Query the Authorised Nameserver(s) for the SRV record
	# of the PDC master. In the returned answers, verify if
	# the PDC master is set to the local DC. If set, return
	# success(1), or else return 0.

	if ($authservercount <= 0) {
		return -1;
	}

	foreach my $serv (@authservers) {

		my $success = 0;
		my $dnsserver = $serv->name;

		# create the DNS query handle
		my $query_handle = Net::DNS::Resolver->new;

		$query_handle->tcp_timeout(60);
		$query_handle->udp_timeout(60);

		# set the nameserver as the authorized nameserver $dnsserver
		$query_handle->nameservers("$dnsserver");

		my $SRV_query = $query_handle->send("_ldap._tcp.pdc._msdcs.$domain", "SRV");
		next if ! defined ($SRV_query);
		# check for authoritative answer
		my $Is_authoritative = $SRV_query->header->aa;

		if($Is_authoritative)
		{
			# authoritative answers are obtained
		
			print "DNS answer packet for DNS_PDC_Master_query function\n" if($debug);
			print "++++++++++++++++++++++++++++++++++++++++++++++++++++\n" if ($debug);
			$SRV_query->print if($debug);
			print "=======================================================\n" if($debug);

			# get the answer count
			my $answer_count = $SRV_query->header->ancount;

			print "answer count $answer_count\n" if ($debug);
			# get all the answers and match them with the local DC's name
			if($answer_count)
			{
				my $count = 0;

				for($count= 0; $count<$answer_count; $count++)
				{
					# get the returned DC name
					my $server_dn = ($SRV_query->answer)[$count]->{"target"};
					print "server returned is :", $server_dn->name,"\n" if($debug);
				
					# remove newline characters, if any
					chomp $server_dn;

					# get the returned record's weight
					my $record_weight = ($SRV_query->answer)[$count]->{"weight"};
					print "record weight is : $record_weight\n" if($debug);

					# get the returned record's priority
					my $record_priority = ($SRV_query->answer)[$count]->{"priority"};
					print "record priority is: $record_priority\n" if($debug);

					# get the returned record's port
					my $record_port = ($SRV_query->answer)[$count]->{"port"};
					print "record port is : $record_port\n" if($debug);

					# push the $server_dn into @Existing_PDC_Masters array
					push(@Existing_PDC_Masters, $server_dn->name);

					# populate the %Existing_PDC_Masters_record_values hash
					$Existing_PDC_Masters_record_values{$server_dn->name} = {'weight' => "$record_weight",
											'priority' => "$record_priority",
											'port' => "$record_port",
											};
	

					# match it with the hostname of local DC if $success != 1
					# hostname is an uppercase, lower it before matching
					if(!($success))
					{
						if(lc($server_dn->name) eq lc($localhost_fqdn))
						{
							# match found, return success;
							$success = 1;
						}
					}
				}
				return $success;
			}
		}
		# no answers returned by DNS server, move to next authoritative server
		print STDERR sprintf(gettext("Authorized DNS server %s returned no response for '_ldap._tcp.pdc._msdcs.%s' query"), $dnsserver, $domain),"\n";
	}
	# no authoritative answers returned by DNS server
	print STDERR sprintf(gettext("Authorized DNS servers returned no response for '_ldap._tcp.pdc._msdcs.%s' query"), $domain),"\n";
	return $ERR_NO_AUTHORITATIVE_PDC_QUERY_ANSWERS_FROM_DNS;
}


# set the nameserver for the domain######
##########################################
sub set_nameserver($)
{
    my $dnsserver = shift;
    my $res = Net::DNS::Resolver->new;
    $res->tcp_timeout(60);
    $res->udp_timeout(60);
    $res->nameservers($dnsserver);
    return $res;
}


###signing callback function for TSIG module####
################################################
sub gss_sign($$)
{
	my $key = shift;
	my $data = shift;
	my $sig;
	$key->get_mic(0, $data, $sig);
	return $sig;
}

# verify a TSIG signature from a DNS server reply ##
#####################################################
sub sig_verify($$)
{
	my $context = shift;
	my $packet = shift;

	if($debug)
	{
		my @packet_additional = $packet->additional;
		print "packet_additional array contains:", @packet_additional, "\n";
		my $tsig_handler = ($packet->additional)[0];
		print "algorithm: ",  $tsig_handler->algorithm, "\n";
		print "time signed:",  $tsig_handler->time_signed, "\n";
		print "fudge:",	 $tsig_handler->fudge, "\n";
		print "MAC size :",  $tsig_handler->mac_size, "\n";
		print "MAC:",  $tsig_handler->mac, "\n";
		print "other len:",  $tsig_handler->other_len, "\n";
		if(($tsig_handler->other_len) != 0)
		{
			print "other data:",  $tsig_handler->other_data, "\n";
		}

	}

	my $tsig = ($packet->additional)[0];
	print "calling sig_data\n" if($debug);
	my $sigdata = $tsig->sig_data($packet);
	
	print "sig_data_done\n" if ($debug);

	return $context->verify_mic($sigdata, $tsig->{"mac"}, 0);
}

# Update PDC Master SRV record in DNS database ####
###################################################
sub Update_PDC_Master_SRV_record()
{
#	plan tests => no_plan;
	my $return_val = 0;

	# get the count of the of Existing_PDC_Masters records in DNS database
	my $Existing_PDC_Master_entries_count = @Existing_PDC_Masters;

	NAMESERVER:
	foreach my $serv (@authservers) {

		my $dnsserver = $serv->{nsdname};
		$return_val = 0;

		print STDOUT sprintf (gettext ("Sending updates to the server %s\n"), $dnsserver);
		# create the GSSAPI contexts
		my $name = GSSAPI::Name->new;
		my $context = GSSAPI::Context->new;

		# Set the nameserver
		$nameserver = set_nameserver("$dnsserver");


		$status = $name->import($name, "DNS/" . $dnsserver . "@" . uc($domain));
		#$status || gss_exit("unable to get the name imported..", $status);
		if(($status->major != 0) or ($status->minor != 0))
		{
			# error encountered
			print STDERR "Error encountered while importing the name DNS/$dnsserver\@". uc($domain)."\n" if($debug);
			$return_val = gss_exit($status);
			next;
		}


		# specify the GSS flags
		my $gss_flags = GSS_C_REPLAY_FLAG | GSS_C_MUTUAL_FLAG |
				GSS_C_SEQUENCE_FLAG | GSS_C_CONF_FLAG |
				GSS_C_INTEG_FLAG | GSS_C_DELEG_FLAG;


		# acquire credentails
		my ($cred, $oidset, $time) = "";

		$status = GSSAPI::Cred::acquire_cred(undef, 120, undef, GSS_C_INITIATE,
							$cred, $oidset, $time);
		#$status || gss_exit("unable to acquire credentials..", $status);

		if(($status->major != 0) or ($status->minor != 0))
		{
			# error encountered
			print STDERR "Error encountered while acquiring credentials. \n" if($debug);
			$return_val = $status;
			next;
		}

		print "cedentials acquired\n" if($debug);

		# call gss_init_sec_context
		my $token = "";
		my $token2 = "";

		$status = $context->init($cred, $name, undef, $gss_flags,
					0, undef, "", undef, $token, undef, undef);

		#$status || gss_exit("failed in the first call to gss_init_sec_context...", $status);

		#  here, the permitted gss major status values are 0 and 1
		#if(($status->major != 0) or ($status->minor != 0))
		if(($status->major != 1) or ($status->minor != 0))
		{
			if($status->major != 0)
			{
				# error encountered
				print STDERR "Error encountered during the first call to gss_init_sec_context for establishing the secure context. \n";
				$return_val = $status;
				next;
			}
		}

		print "first call to gss_init_sec_context done\n" if($debug);

		# check the Major status for GSS_S_CONTINUE_NEEDED and length of $token 
		# if (( Major status == 'GSS_S_CONTINUE_NEEDED') || (length($token)))
		# then we need to send the token back to the server.

		my $counter = 1;
		my $reply = undef;

		do
		{
			$counter++;

			my $gss_query = Net::DNS::Packet->new("$key_name", "TKEY", "IN");

			if(!defined($gss_query))
			{
				print STDERR "Error encountered while creating the DNS RR packet handler. exiting...\n";
				$return_val = $ERR_DNS_RR_PACKET_HANDLE_CREATION_FAILED;
				goto out;
			}

			# create the DNS RR packet
			print "calling RR new\n" if($debug);
	
			$a = Net::DNS::RR->new(
						Name	=> "$key_name",
						Type	=> "TKEY",
						TTL	=> 0,
						Class	=> "ANY",
						mode => 3,
						algorithm => "gss-tsig",
						inception => time,
						expiration => time + 24*60*60,
						key => "$token",
						other_data => "",
						);

			$gss_query->push("additional", $a);

			# set the recusrion desired to 0 by default it is set to 1
			$gss_query->header->rd(0);

			# print the RR data format
			print "RR data format\n" if($debug);
			print "----------------\n" if($debug);
			$a->print if($debug);
			print "===============\n" if($debug);

			#print the DNS packt format
			print "DNS packet format\n" if($debug);
			print "------------------\n" if($debug);

			$gss_query->print if($debug);
			print "==================\n" if($debug);



			# verify the packet before sending it
			if($debug)
			{
				my $z = ($gss_query->zone)[0];
				ok($gss_query,				      'new() returned packet');	 #1
				is($gss_query->header->opcode, 'QUERY',	      'header opcode correct');	 #2 
				is($z->zname,  $key_name,		       'zname correct');	  #3
				is($z->zclass, "IN",			   'zclass correct');	      #4
				is($z->ztype,  'TKEY',			   'ztype correct');	      #5		
				is($gss_query->header->rd, '0',		'header rd correct');	#6
			}
	
			#sending the RR query packet to DNS server
			$reply = $nameserver->send($gss_query);

			if (!defined ($reply)) {
				$return_val = $ERR_SENDING_TKEY_PACKET; 
				print STDERR sprintf(gettext("Error sending the TKEY packet")),"\n";
				next NAMESERVER;
			}
			if ($reply->header->{'rcode'} ne 'NOERROR')
			{
				$return_val = $reply->header->{'rcode'};
				print STDERR sprintf(gettext("Error %s sending the TKEY packet"),$return_val),"\n";
				print "encountered $return_val error\n";
				print "returned packt is:\n" if($debug);
				print "----------------------\n" if($debug);
				$reply->print if($debug);
				print "======================\n" if($debug);
				next NAMESERVER;
			}

			# printing the returned packet
			print "The DNS server response is:\n" if($debug);
			print "-------------------------------\n" if($debug);
			$reply->print if($debug);
			print "======================\n" if($debug);
	

			$token2 = ($reply->answer)[0]->{"key"};

			# calling gss_init_sec_context again
			$status = $context->init($cred, $name, undef, $gss_flags,
						0, undef, $token2, undef, $token,
						undef, undef);

			#$status || gss_exit("failed in the $counter call to gss_init_sec_context...", $status);
			if(($status->major != 0) or ($status->minor != 0))
			{
				# error encountered
				$return_val = gss_exit($status);
				print STDERR sprintf(gettext("Error %s during GSS Handshake"),$return_val),"\n";
				next NAMESERVER;
			}
			
		} while (($status->major eq GSS_S_CONTINUE_NEEDED) || (defined($token)));

		if(($status->major eq GSS_S_COMPLETE) && !(defined($token)))
		{
			if($MS_DNS)
			{
				# signature verification is required for MS DNS as it 
				# sends signature for check in the additional section
				# of the DNS reply packet. Novell-Named does not does 
				# the same .i.e. doesn't sends any additional info.
				# check the signature on the TKEY reply
				print "verifying the TKEY signature..\n" if($debug);
				my $rc = sig_verify($context, $reply);
				if(!($rc))
				{
					$return_val = $ERR_VALIDATION_TKEY_FAILED;
					print STDERR sprintf(gettext("Error %s verifying TKEY reply"),$return_val),"\n";
					next NAMESERVER;
				}
	
				print "verifying done\n" if($debug);
			}

			print "context can be used for Dynamic DNS update\n" if($debug);

			print "Negotiated TKEY $key_name\n" if($debug);

			# construct a signed update
			my $update = "";

			if($MS_DNS)
			{
				# the pdc SRV record is held in the forest level DNS
				# zone and not in the domain level DNS zone for MS DNS
				$update = Net::DNS::Update->new("_msdcs.$domain");
			} else {
				$update = Net::DNS::Update->new("$domain");
			}

			if($MS_DNS)
			{
				# the pdc SRV record is held in the forest level DNS
				# zone and not in the domain level DNS zone for MS DNS
				$update->push("pre", yxdomain("_msdcs.$domain"));
			} else {
				$update->push("pre", yxdomain("$domain"));
			}
	
			$update->push("pre", yxrrset("_ldap._tcp.pdc._msdcs.$domain SRV "));
	
			# create the record add entry first
			$update->push("update", rr_add("_ldap._tcp.pdc._msdcs.$domain SRV 0 100 389 ". lc($hostname)."\.$domain"));

			# parse the @Existing_PDC_Masters array and add the existing 
			# SRV records for deletion
			if($Existing_PDC_Master_entries_count)
			{
				# there does exists PDC Master entries in the DNS data base
				my $deletion_entry_count = 0;
				for($deletion_entry_count = 0;$deletion_entry_count<$Existing_PDC_Master_entries_count;$deletion_entry_count++)
				{
					# create an rr_del update packet entry for each array entry
					$update->push("update", rr_del("_ldap._tcp.pdc._msdcs.$domain SRV $Existing_PDC_Masters_record_values{$Existing_PDC_Masters[$deletion_entry_count]}->{'priority'} $Existing_PDC_Masters_record_values{$Existing_PDC_Masters[$deletion_entry_count]}->{'weight'} $Existing_PDC_Masters_record_values{$Existing_PDC_Masters[$deletion_entry_count]}->{'port'} $Existing_PDC_Masters[$deletion_entry_count]"));
				}
			}

			my $sig = Net::DNS::RR->new(Name    => $key_name,
						Type	=> "TSIG",
						TTL	=> 0,
						Class	=> "ANY",
						Algorithm => "gss-tsig",
						Time_Signed => time,
						Fudge => 36000,
						Mac_Size => 0,
						Mac => "",
						Key => $context,
						Sign_Func => \&gss_sign,
						Other_Len => 0,
						Other_Data => "",
						Error => 0,
						mode => 3,
						);

			$update->push("additional", $sig);

			# print the DNS packet being sent to DNS server to update SRV record
			print "SRV record update packet\n" if($debug);
			print "===========================================\n" if($debug);
			$update->print if($debug);
			print "=============================================\n" if($debug);

	
			# send the dynamic update
			my $update_reply = $nameserver->send($update);

			if (! defined($update_reply))
			{
				print STDERR gettext("No reply from DNS server for dynamic update"),"\n";
				$return_val = $ERR_NO_REPLY_TO_DYNAMIC_UPDATE;
				next NAMESERVER;
			}

			# make sure it worked
			$return_val = $update_reply->header->{"rcode"};
			print "Update gave rcode $return_val\n" if($debug);
	
			if ($return_val ne 'NOERROR')
			{
				# printing the returned packet
				print "The DNS server response is:\n" if($debug);
				print "-------------------------------\n" if($debug);
				$update_reply->print if($debug);
				print "====================================\n" if($debug);
				print STDERR gettext("Dynamic DNS Update failed. Check to see if the DNS server is up"),"\n";
				next NAMESERVER;
			} elsif($return_val eq 'NOERROR')
			{
				$return_val = 0;
				last NAMESERVER;
			}
		
		}
	}
out:
	return $return_val;
}

# Update the smb.conf file being passed ##
##########################################

sub Process_SMB_Conf_File($$)
{
	my $smb_conf_file = shift;
	my $action = shift;
	my $return_val = 0;
	my $readline = "";
	my $rc = 0;
	my $Is_sysvol_section = 0;
	my $Is_netlogon_section = 0;
	my $Is_sysvol_msdfs_section = 0;


	# check if the file passed exists or not
	if(!(-e "$smb_conf_file"))
	{
		# the file passed does not exists.. error out
		$return_val = $ERR_FILE_NOT_FOUND;
		print STDERR sprintf(gettext("Hard Error! %s %s"),$return_val, $smb_conf_file),"\n";
		goto out;
	}

	# file found open it for processing
	$rc = open(SMB_FILE, "$smb_conf_file");
	if(!($rc))
	{
		# error occured while opening the file
		$return_val = $ERR_CANNOT_OPEN_FILE;
		print STDERR sprintf(gettext("Hard Error! %s %s"),$return_val, $smb_conf_file),"\n";
		goto out;
	}

	# open a temporary file
	my $temp_file = `/bin/mktemp`;
	chomp ($temp_file);
	$rc = open(TEMP_FILE, "+>$temp_file");
	if(!($rc))
	{
		# error occured while opening the file
		$return_val = $ERR_CANNOT_OPEN_FILE;
		print STDERR sprintf(gettext("Hard Error! %s %s"),$return_val, $temp_file),"\n";
		goto out;
	}

	if($action eq "makewritable") {
		while($readline = <SMB_FILE>) {
			chomp $readline;
			if(($readline =~ m/^\[netlogon\]/) or ($Is_netlogon_section == 1)) {
				if(!($readline =~ m/^\[netlogon\]/) and ($readline =~ m/^\[/)) {
					$Is_netlogon_section = 0;
				}
				elsif (($readline =~ m/^\#*\s*writable/) and ($Is_netlogon_section == 1)) {
					$readline =~ s/\#*\s*writable\s*=\s*(.*)/\twritable = Yes/;

				}
				else {
					$Is_netlogon_section = 1;
				}
			}
			if(($readline =~ m/^\[sysvol\]/) or ($Is_sysvol_section == 1)) {
				if(!($readline =~ m/^\[sysvol\]/) and ($readline =~ m/^\[/)) {
					$Is_sysvol_section = 0;
				} else {
					$Is_sysvol_section = 1;
					next;
				}
			}
			if($readline =~ m/^\[sysvol-msdfs\]/ or ($Is_sysvol_msdfs_section == 1)) {
				if(!($readline =~ m/^\[sysvol-msdfs\]/) and ($readline =~ /^\[/)) {
					$Is_sysvol_msdfs_section = 0;
				} else {
					$Is_sysvol_msdfs_section = 1;
					$readline =~ s/\-msdfs//;
				}
			}
			if ($Is_sysvol_msdfs_section == 1) {
				if($readline =~ m/^\#*\s*writable/) {
					$readline =~ s/\#*\s*writable\s*=\s*(.*)/\twritable = Yes/;
				}
				elsif($readline =~ m/^\[sysvol\]/) {
					$readline = $readline . "\n\t" . "wide links = Yes";	#Set the wide link option at the start of new sysvol section.
				}
				elsif($readline =~ m/^\#*\s*wide links/) {	#Ignore if wide link option already set.
					next;
				}
			}	
			# add the same in the TEMP_FILE
			print TEMP_FILE "$readline\n";
		}
	}
	elsif($action eq "makereadable") {
		# read the $smb_conf_file file
		$Is_sysvol_section = 0;
		$Is_sysvol_msdfs_section = 0;	
		while($readline = <SMB_FILE>) {
			chomp $readline;

			if(($readline =~ m/^\[netlogon\]/) or ($Is_netlogon_section == 1)) {
				if(!($readline =~ m/^\[netlogon\]/) and ($readline =~ m/^\[/)) {
					$Is_netlogon_section = 0;
				}
				elsif (($readline =~ m/^\#*\s*writable/) and ($Is_netlogon_section == 1)) {
					$readline =~ s/\#*\s*writable\s*=\s*(.*)/\twritable = No/;

				}
				else {
					$Is_netlogon_section = 1;
				}
			}

			if($readline =~ m/^\[sysvol\]/) {
				# Found sysvol entry, set the Is_sysvol_section flg
				$readline =~ s/\[sysvol\]/\[sysvol-msdfs\]/;
				$Is_sysvol_section = 1;
			}
			elsif(($readline =~ m/^\[/) and ($Is_sysvol_section == 1)) {
				# another section starts, reset the Is_sysvol_section flag
				my $tmpread = $readline;

				my $towrite = "[sysvol] \n \t comment = msdfs link to Group Policies \n \t wide links = yes \n \t path = /var/opt/novell/xad/msdfs \n \t msdfs root = Yes \n \n" ;
				$readline =~ s/^\[/$towrite\[/;
				$Is_sysvol_section = 0;
			}

			if ($Is_sysvol_section == 1) {
				if($readline =~ m/^\#*\s*writable/) {
					$readline =~ s/\#*\s*writable\s*=\s*(.*)/\twritable = No/;
				}
			}

			# add the same in the TEMP_FILE
			print TEMP_FILE "$readline\n";
		}
	}
	
	close(SMB_FILE);
	close(TEMP_FILE);

	# change the permission of the $temp_file
	$rc = chmod(0644, $temp_file);
	if(!($rc))
	{
		# error encountered, return error
		$return_val = $ERR_TEMP_FILE_PERMISSION_CHANGE;
		print STDERR sprintf(gettext("Hard Error! %s %s"),$return_val, $temp_file),"\n";
		goto out;
	}

	# backup the original $smb_conf_file file
	$rc = move($smb_conf_file, "$smb_conf_file"."_org");
	if($rc != 1)
	{
		# error encountered
		$return_val = $ERR_FILE_MOVE_ERROR;
		print STDERR sprintf(gettext("Hard Error! %s %s"),$return_val, $smb_conf_file),"\n";
		goto out;
	}

	# overwrite the $smb_conf_file with the $temp_file 
	$rc = move($temp_file, $smb_conf_file);
	if($rc != 1)
	{
		# error encountered
		$return_val = $ERR_FILE_MOVE_ERROR;
		print STDERR sprintf(gettext("Hard Error! %s %s"),$return_val, $temp_file),"\n";
		goto out;
	}

out:
	return $return_val;
		
}

# Update the remote server's samba configuration #
##################################################

sub Update_Remote_Samba_Configuration()
{
	my $return_val = 0;
	my $rc = 0;

	# Check the remote server, which was the last PDC master
	my $Last_PDC_Master_count = @Existing_PDC_Masters;
	if($Last_PDC_Master_count != 1)
	{
		# last PDC Master count cannot be greater then 1,
		# something is wrong, error out
		print STDERR sprintf(gettext("PDC Master count %s is not correct. Check the PDC SRV record in the DNS server"),$Last_PDC_Master_count), "\n";
		$return_val = $ERR_BAD_LAST_PDC_MASTER_COUNT;
		goto out;
	}

	# check if the remote server is up and reachable
	my $ping_check = Net::Ping->new("icmp") or die "Cannot create a Ping object:$!\n";
	my $Is_server_reachable = $ping_check->ping("$Existing_PDC_Masters[0]");

	if(!(defined($Is_server_reachable)))
	{
		# cannot locate the server/hostname
		$return_val = $ERR_CANNOT_RESOLVE_LAST_PDC_MASTER;
		print STDERR sprintf(gettext("Not able to contact the old PDC Master %s. Server might be down"),$Existing_PDC_Masters[0]),"\n";
		goto out;
	}
	if(!($Is_server_reachable))
	{
		# server resolved, but is not responding to ping..
		$return_val = $ERR_CANNOT_REACH_TO_LAST_PDC_MASTER;
		print STDERR sprintf(gettext("Not able to reach old PDC Master %s"), $Existing_PDC_Masters[0]), "\n";
		goto out;
	}

	`rmdir "/tmp/PDC-MASTER-CONF/"` if(-e "/tmp/PDC-MASTER-CONF/");
	mkdir("/tmp/PDC-MASTER-CONF/","0775");
	# pull remote samba configuration
	print STDERR sprintf(gettext("Updating %s samba Configuration file"),$Existing_PDC_Masters[0]),"\n";
	#my $remote_smb_pull_status = SyncChanges($Existing_PDC_Masters[0],"pdc-role-change/smb.conf","/tmp/PDC-MASTER-CONF/", 0);
	my $old_pdc_fqdn = $Existing_PDC_Masters[0];
        my @fqdn_split = split('\.',$old_pdc_fqdn);
        my $old_pdc_server = $fqdn_split[0];
	`/opt/novell/xad/sbin/updatePdcHelper -s $old_pdc_server -d /tmp/PDC-MASTER-CONF/smb.conf -m 127.0.0.1::pdc-role-change/smb.conf -t pull`;
	my $remote_smb_pull_status = $?;

	if($remote_smb_pull_status != 0) {
		$return_val = $remote_smb_pull_status;
		goto out;
	}

	# remote smb.conf file is pulled to local, process it to change
	# the sysvol share's 'writable' setting
	my $smb_conf_process_status = Process_SMB_Conf_File("/tmp/PDC-MASTER-CONF/smb.conf", "makereadable");

	if($smb_conf_process_status != 0) {
		# error encountered while processing smb.conf file of remote server
		$return_val = $smb_conf_process_status;
		goto out;
	}

	# remote smb.conf updated, send it back to the remote server
	#my $remote_smb_push_status = SyncChanges($Existing_PDC_Masters[0],"/tmp/PDC-MASTER-CONF/smb.conf","pdc-role-change", 1);
	`/opt/novell/xad/sbin/updatePdcHelper -s $old_pdc_server -d /tmp/PDC-MASTER-CONF/smb.conf -m 127.0.0.1::pdc-role-change/smb.conf -t push`;
	my $remote_smb_push_status = $?;

	if($remote_smb_push_status != 0) {
		# error encountered while pushing the smb.conf file to the remote server
		 print STDERR sprintf(gettext("Unable to synchronize /etc/samba/smb.conf file to server %s"),$Existing_PDC_Masters[0]),"\n";
		$return_val = $remote_smb_push_status;
		goto out;
	}

out:
	unlink("/tmp/PDC-MASTER-CONF/smb.conf");
	return $return_val;
}

# Update the local server's samba configuration #
#################################################

sub Update_Local_Samba_Configuration()
{

	my $return_val = 0;

	# Update the local smb.conf file to reflect that this DC is the new PDC Master
	# and holds the 'writable' share of SYSVOL
	print STDERR sprintf(gettext("Updating %s samba Configuration file"),$localhost_fqdn),"\n";

	$return_val = Process_SMB_Conf_File("/etc/samba/smb.conf", "makewritable");
	return $return_val;
}

sub OldPDCMaster_DNS_Query()
{
	#DNS query to know old PDC Master
	print "local server is PDC master\n" if($debug);

	# get the authoritative nameserver
	find_nameserver_for_domain($domain);

	# verify the PDC Master as per the DNS server
	$Is_PDC_Master_DNS = DNS_PDC_Master_query();

	if($Is_PDC_Master_DNS > 0) {
		# Local DC is registered as PDC master in the DNS database
		print STDOUT gettext("This Domain controller is already registered as a PDC Master in the DNS database"),"\n";
		return $Is_PDC_Master_DNS;
	} elsif($Is_PDC_Master_DNS < 0) {
		# error occured .. print the error encountered and exit out
		print STDERR gettext("Unable to query DNS for PDC Master record. Check to see if DNS server is up"),"\n";
		return $Is_PDC_Master_DNS;
	}
	return 0;
}

# Update the samba configuration ##
###################################

sub Update_Samba_Configuration()
{
	my $rc = 0;
	my $continue = 0;

	# update the remote samba configuration
	print "Updating the remote samba configuration..\n" if($debug);
	$rc = Update_Remote_Samba_Configuration();

	$continue = 1 if(!($rc));
	if (($rc) and ($role_seizure == 1)){
		$continue = 1;
	}

	if($continue) {
		# proceed with local samba configuration update
		print "Updating the local samba configuration..\n" if($debug);
		$rc = Update_Local_Samba_Configuration();
		RollBack (1) if($rc); # RollBack remote samba configuartion file
	}

	return $rc;
}

sub SyncToOtherServer($$$)
{
	my ($data, $module, $pullorpush) = @_;
	system ("/usr/bin/rsync -AazoXp --numeric-ids --port=8731 $data $myLocalhost\:\:$module") if($pullorpush == 1);
	system("/usr/bin/rsync -AazoXp --numeric-ids --port=8731 $myLocalhost\:\:$data $module") if($pullorpush == 0);
	return $?;
}

sub CreateTunnel($)
{
	my ($dc) = @_;

	system("LD_LIBRARY_PATH=/usr/lib/ /usr/bin/ssh -q -f -N -o StrictHostKeyChecking=no -o UserKnownHostsFile=/dev/null -o PreferredAuthentications=gssapi-with-mic -o ConnectTimeout=5 -MS /var/opt/novell/xad/sysvol_sock -L 8731:$myLocalhost:873 '$admin_sam_name'\@$dc >/dev/null 2>&1");
	return $?;
}

sub CloseTunnel($)
{
	my ($dc) = @_;

	system("LD_LIBRARY_PATH=/usr/lib/ /usr/bin/ssh -q -S /var/opt/novell/xad/sysvol_sock -O exit '$admin_sam_name'\@$dc >/dev/null 2>&1");
	return $?;
}

sub SyncChanges($$$$)
{
	my $return_val = 0;
	my ($dc, $data, $module, $pullorpush) = @_;
	my $tunnel_status = CreateTunnel($dc);
	if($tunnel_status) {
		# error while creating tunnel
		print STDERR sprintf(gettext("SSH tunnel creation between localhost and %s failed %s"),$dc, (($tunnel_status >> 8)& 0xff)),"\n";
		$return_val = $tunnel_status >> 8;
		goto out;
	}

	my $sync_status = SyncToOtherServer($data,$module,$pullorpush);

	if($sync_status) {
		# error encountered while pushing the smb.conf file to the remote server
		print STDERR sprintf(gettext("Unable to synchronize %s to %s server"),$data,$dc),"\n";
		$return_val = $sync_status;
	}

	my $tunnel_closure_status = CloseTunnel($dc);

	if($tunnel_closure_status) {
		# error while terminating tunnel
		print STDERR sprintf(gettext("SSH tunnel closure between localhost and %s failed %s"),$dc, (($tunnel_closure_status >> 8)& 0xff)),"\n"; 
		$return_val = $tunnel_closure_status >> 8;
		goto out;
	}
out:
	return $return_val;
}


# Update msdfs link ##
######################

sub Update_msdfsLink()
{
	my $rc = 0;
	my $Is_PDC_Master_Flg_Ref = shift;

	my $failure = 0;
	my $return_val = 0;

	my $msdfs_dir = "/tmp/msdfs";
	my $ldapsearch = "/usr/bin/ldapsearch";
	my $ldap_filter = ' | sed " /^ / {; H; d; }; /^ /! {; x; s/\n //; }; "';
	my $ldap_host = "$myLocalhost";
	my $ldap_base = `/opt/novell/xad/share/dcinit/printConfigKey.pl "Domain Controllers Container"`;
	my $admin_dn = `/opt/novell/xad/share/dcinit/provisionTools.sh get-admin-name`;
	my $mapped_users_dn = `/opt/novell/xad/share/dcinit/printConfigKey.pl "Mapped Users Container"`;
	my $ldap_objectclass = "mSDS:Computer";	
	my @domaincontroller_fqdn;
	chomp $ldap_base;
	my $dc_fqdn;
	my $Lib = `/opt/novell/xad/share/dcinit/printConfigKey.pl "_Lib"`;
	chomp $Lib;
	my $inst = 0;
	my $administrator_uid = 0;
	my $domain_admins_gid = 0;

	# convert to ldap format
	$ldap_base =~ s/\.([a-z,A-Z,0-9]+)=/,$1=/g;

	# ldapsearch to get all the Domain controllers in the replcia ring
	my $command = "SASL_PATH=/opt/novell/xad/$Lib/sasl2/ $ldapsearch -h $ldap_host -Y GSSAPI -b \'$ldap_base\' \"(objectclass=$ldap_objectclass)\" dnsHostName -LLL 2>/dev/null $ldap_filter | grep -i \"dnsHostName: \" |  awk -F \": \" \'{print \$2}\'";
	@domaincontroller_fqdn = `$command; echo \${PIPESTATUS[0]} > $ENV{'PIPE_STATUS_FILE'}`;
	my $ldapsearch_err = `cat $ENV{'PIPE_STATUS_FILE'}`;
	chomp $ldapsearch_err;
        if($ldapsearch_err)
        {
                print STDERR sprintf(gettext("Error returned: %s"), $ldapsearch_err),"\n";
                $return_val = $ldapsearch_err;
                goto out;
        }
	chomp @domaincontroller_fqdn;

	# ldapsearch to get Administrators uidNumber
	chomp ($admin_dn, $mapped_users_dn);
	 $command = "SASL_PATH=/opt/novell/xad/$Lib/sasl2/ $ldapsearch -h $ldap_host -Y GSSAPI -b \'$admin_dn\' -s base uidNumber -LLL 2>/dev/null $ldap_filter  | grep -i \"uidNumber: \" |  awk -F \": \" \'{print \$2}\'";

	my $uidNumber = `$command; echo \${PIPESTATUS[0]} > $ENV{'PIPE_STATUS_FILE'}`;
	$ldapsearch_err = `cat $ENV{'PIPE_STATUS_FILE'}`;
	chomp $ldapsearch_err;
        if($ldapsearch_err)
        {
                print STDERR sprintf(gettext("Error returned: %s"), $ldapsearch_err),"\n";
                $return_val = $ldapsearch_err;
                goto out;
        }
	chomp $uidNumber;

	$administrator_uid = $uidNumber;
	my $base_dn = "cn= domain admins,$mapped_users_dn";
	# GID Number of Domain Admins Group
	 $command = "SASL_PATH=/opt/novell/xad/$Lib/sasl2/ $ldapsearch -h $ldap_host -Y GSSAPI -b \'$base_dn\' -s base gidNumber -LLL 2>/dev/null $ldap_filter  | grep -i \"gidNumber: \" |  awk -F \": \" \'{print \$2}\'";
	$domain_admins_gid = `$command; echo \${PIPESTATUS[0]} > $ENV{'PIPE_STATUS_FILE'}`;
	$ldapsearch_err = `cat $ENV{'PIPE_STATUS_FILE'}`;
	chomp $ldapsearch_err;
        if($ldapsearch_err)
        {
                print STDERR sprintf(gettext("Error returned: %s"), $ldapsearch_err),"\n";
                $return_val = $ldapsearch_err;
                goto out;
        }
	chomp $domain_admins_gid;
	
	`rmdir $msdfs_dir` if(-e $msdfs_dir);
	`mkdir $msdfs_dir`;

	foreach (@domaincontroller_fqdn) {
		# create msdfs link and sync it to the DC.
		$dc_fqdn = lc($_);

		if ($dc_fqdn ne $localhost_fqdn) {
			`(ln -sf "msdfs:$dc_fqdn\\sysvol-msdfs,$localhost_fqdn\\sysvol" $msdfs_dir/$domain)`;
			`chown -R $administrator_uid:$domain_admins_gid $msdfs_dir`;
			#$return_val = SyncChanges($dc_fqdn, "$msdfs_dir/$domain", "msdfs", 1);
		        my @fqdn_split = split('\.',$dc_fqdn);
		        my $dc_server = $fqdn_split[0];
			`/opt/novell/xad/sbin/updatePdcHelper -s $dc_server -d $msdfs_dir/$domain -m 127.0.0.1::msdfs/ -t push`;
			$return_val = $?;
			$failure = $return_val if(($return_val) and (($role_seizure == 0) or ($Existing_PDC_Masters[0] ne $dc_fqdn)));
			print STDOUT sprintf(gettext("msdfs link synchronized to %s. Error %s"),$dc_fqdn, $return_val),"\n";
			`unlink $msdfs_dir/$domain`;
		}
	}
	$return_val = $failure;

	if($return_val == 0) {
		`rm -f /var/opt/novell/xad/msdfs/$domain`;
	}
out:
	`rmdir $msdfs_dir` if (-e $msdfs_dir);
	return $return_val;
}

sub usage()
{
	print STDERR gettext("Domain Services for Windows PDC Master Update Tool"),"\n";
        print STDERR gettext("Copyright (c) 2001-2011 Novell, Inc. All rights reserved."),"\n";

        print STDERR gettext("Usage: UpdatePDCMaster [-h] [-d] [-M] [-S]"),"\n";
	print STDERR gettext("-d                      Debug"),"\n";
	print STDERR gettext("-S                      Role Seizure"),"\n";
	print STDERR gettext("-M                      MS DNS"),"\n";
	print STDERR gettext("-h                      help"),"\n";
	exit 0;
}

## Cleanup temp files ##
########################

sub CleanUp
{
	unlink("/tmp/PDC-MASTER-CONF/smb.conf_org");
	unlink("/tmp/PDC-MASTER-CONF/smb.conf");
	unlink("/etc/samba/smb.conf_org");
	rmdir("/tmp/PDC-MASTER-CONF");
}

## RollBack ##
##############

sub RollBack($)
{
	my ($option)= @_;
	# RollBack Remote samba configuration file
	move("/tmp/PDC-MASTER-CONF/smb.conf_org", "/tmp/PDC-MASTER-CONF/smb.conf");
	#SyncChanges($Existing_PDC_Masters[0],"/tmp/PDC-MASTER-CONF/smb.conf","pdc-role-change", 1);
	my $old_pdc_fqdn = $Existing_PDC_Masters[0];
        my @fqdn_split = split('\.',$old_pdc_fqdn);
        my $old_pdc_server = $fqdn_split[0];
	`/opt/novell/xad/sbin/updatePdcHelper -s $old_pdc_server -d /tmp/PDC-MASTER-CONF/smb.conf -m 127.0.0.1::pdc-role-change/smb.conf -t push`;

	# RollBack local samba configuration file.
	if ($option == 0) {
		move("/etc/samba/smb.conf_org", "/etc/samba/smb.conf");
	}
}
sub Update_xad_ini()
{
	my $return_val = 0;
	print STDOUT gettext ("Updating the local xad.ini file with XADRETAINPOLICIES flag from PDC\n");
	#my $remote_pull_status = SyncChanges($Existing_PDC_Masters[0],"xad-ini-change/xad.ini", "/tmp/PDC-MASTER-CONF/", 0);
	my $old_pdc_fqdn = $Existing_PDC_Masters[0];
        my @fqdn_split = split('\.',$old_pdc_fqdn);
        my $old_pdc_server = $fqdn_split[0];
	`/opt/novell/xad/sbin/updatePdcHelper -s $old_pdc_server -d /tmp/PDC-MASTER-CONF/ -m 127.0.0.1::xad-ini-change/xad.ini -t pull`;
	my $remote_pull_status = $?;

	if($remote_pull_status != 0) {
		$return_val = $remote_pull_status;
		print STDERR gettext ("Failed to pull PDCs xad.ini file. rsyncd conf on PDC may not be up-to-date.\n");
		print STDERR gettext ("If XADRETAINPOLICIES is present in PDC's xad.ini file, manually update it to local /etc/opt/novell/xad/xad.ini file\n");
		print STDERR gettext ("Ignoring the error and continuing...\n");
		goto out;
	}
	print "Removing old XAD_RETAIN_POLICIES reference in local xad.ini.\n" if($debug);
	`sed -i  '/XADRETAINPOLICIES/d' /etc/opt/novell/xad/xad.ini`;
	`grep -i "XADRETAINPOLICIES" /tmp/PDC-MASTER-CONF/xad.ini >> /etc/opt/novell/xad/xad.ini`;
	print "Updating the local xad.ini with XADRETAINPOLICIES succeeded.\n" if($debug);
out:
	return $return_val;
}


sub create_updatePdcHelper_softLink()
{
	if(! -e "/opt/novell/xad/sbin/updatePdcHelper") {
                `ln /opt/novell/xad/sbin/sysvolsync /opt/novell/xad/sbin/updatePdcHelper`;
        }
}

#Getting the IP address of the server where different zones' objects are located
#Currently this is the pdc of the "root domain" of DSfW forest
sub get_frdDNS_provider() {
        my $frdDNS_provider;
        my $dig_output;
        my $install_type = registry::getReg("INSTALL_TYPE");
        my $dns_server_ip =  registry::getReg("DNS_SERVER_IP");
        my $is_replica = registry::getReg("IS_REPLICA");
        my $ret = 0;


        if($install_type eq "frd" and $is_replica eq "FALSE") {
                $frdDNS_provider = $dns_server_ip;
        }
        else {
                my $forest_root = registry::getReg("FORESTROOT");
                $dig_output = `/usr/bin/dig -t SRV _ldap._tcp.pdc._msdcs.$forest_root  +short`;
                $ret = $?;
                chomp($dig_output);
                if ($ret || $dig_output eq "") {
                        my $error = $ret >> 8;
                        print ("Failed to get the domain provider for domain [$forest_root] [$error]");
                }
                $frdDNS_provider = `echo $dig_output | cut -d " " -f4 | sed 's/\.\$//'`;
                chomp($frdDNS_provider);
        }

        return $frdDNS_provider;
}

sub getTreeAdministrator() {
        my $tree_administrator = "";

        if (defined $ENV{LDAPEXISTINGADMINNAME} and $ENV{LDAPEXISTINGADMINNAME} ne "") {
                #Get from environment variable
                $tree_administrator = $ENV{LDAPEXISTINGADMINNAME};
                return $tree_administrator;
        }

        #non-name mapped tree administrator
        $tree_administrator = registry::getReg("LDAPDOMAINADMINNAME");
        my $type = registry::getReg("INSTALL_TYPE");
        my $is_replica = registry::getReg("IS_REPLICA");

        if ($type eq "frd" and $is_replica eq "FALSE") {
                if (not registry::getReg("MAPPED")) {
                        #name mapped tree administrator
                        $tree_administrator = registry::getReg("LDAPDOMAINADMIN");
                }
        }

        print "From registry - Returning Tree Administrator $tree_administrator\n" if($debug);
        return $tree_administrator;
}

my $global_password = "";

sub getPassword() {

	print "Retrieving password for Administrator" if($debug);
	my $password;
	my $export_variable;
	my $tree_administrator;

	my $type = registry::getReg("INSTALL_TYPE");
	my $is_replica = registry::getReg("IS_REPLICA");

	if ($global_password ne "") {
                return $global_password;
	}

	$password = $ENV{NDSEXISTINGADMINPASSWD};
	$export_variable = "NDSEXISTINGADMINPASSWD";

	if ($type eq "frd" and $is_replica eq "FALSE") {
		if (not registry::getReg("MAPPED") and $ENV{ADM_PASSWD}) {
			$password = $ENV{ADM_PASSWD};
			$export_variable = "ADM_PASSWD";
			$global_password = $password;
			chomp($global_password);
			if ($global_password ne "") {
				return $password;
			}
		}
	}

	print "Taking password input from user\n" if($debug);

	$tree_administrator = getTreeAdministrator();
	print STDERR "Enter password for $tree_administrator :";
	ReadMode('noecho');
	$password = ReadLine(0);
	chomp($password);
	print STDERR "\n";
	ReadMode('normal');

	$global_password = $password;
	return $password;
}

#FQDN where forward zone of the domain is located
sub get_zone_context() {
	print "Fetching the Zone Context\n" if($debug);
	my $tree_administrator = "";
	my $password = "";

	my $server_object = `/opt/novell/xad/share/dcinit/printConfigKey.pl DNSSERVER`;
	chomp($server_object);

	my $dns_server_ip = `/opt/novell/xad/share/dcinit/printConfigKey.pl DNS_SERVER_IP`;
	chomp($dns_server_ip);

	$tree_administrator = getTreeAdministrator();

	my $zone_name = `/opt/novell/xad/share/dcinit/printConfigKey.pl DNSZONE`;
	chomp($zone_name);

	$password = getPassword();

	my $certFile_pem = "/etc/opt/novell/certs/SSCert.pem";

	#Read FQDN of the domain zone from the server object,
	#and get its FQDN by removing the entry before first ','
	#TBD Finally change to "-p 636"
	my $zone_context = `LDAPTLS_CACERT=$certFile_pem /opt/novell/eDirectory/bin/ldapsearch -D '$tree_administrator' -w '$password' -h '$dns_server_ip' -p 389 -b '$server_object' -s base dNIPZoneList -N -LLL -Z | sed ' /^ / {; H; d; }; /^ /! {; x; s/\\n //; }; '| grep -i 'dNIPZoneList:' | grep -iw '$zone_name' | awk -F 'dNIPZoneList: \+' '{print \$2}' | perl -pe 's/.*?,//' `;
	if ($? != 0) {
		print "Failed to get zone context\n";
	}

	print "Zone context - $zone_context\n" if($debug);

	chomp($zone_context);
	return $zone_context;
}

sub update_dns_soa() {
	print "Starting update_dns_soa" if($debug);

	my $admin_passwd = getPassword();
	my $frdDNS_provider = get_frdDNS_provider();
	my $admin_name = getTreeAdministrator();
	my $dns_zone_context = get_zone_context();
	my $zone_container = `/opt/novell/xad/share/dcinit/printConfigKey.pl DNSZONE`;
	chomp($zone_container);
	my $dnsforwardzonefdn = "cn=$zone_container,$dns_zone_context";
	my $_LIB = registry::getReg("_Lib");
#SOA-validation
	my @soa_output=`LD_PRELOAD=/opt/novell/xad/$_LIB/libadmpasswd.so.2 LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem /opt/novell/eDirectory/bin/ldapsearch -H 'ldaps://${frdDNS_provider}:1636' -x -D '$admin_name' -w '$admin_passwd' -b '$dnsforwardzonefdn' -s base dNIPSOASerial -LLL`;
	if ($? != 0) {
		print "Failed to get the SOA value : [@soa_output]";
	}
	@soa_output=grep(/dNIPSOASerial:/, @soa_output);
	chomp(@soa_output);
	my $soa_serial=`echo -n $soa_output[0] | grep -i "dNIPSOASerial: " | awk -F "dNIPSOASerial: " '{print \$2}'`;
#SOA-validation
	chomp($soa_serial);
	$soa_serial +=10;
	my $dns_soa_serial_ldif = mktemp("/tmp/soaserial.XXXXXXXX");

	open FILE, ">$dns_soa_serial_ldif" or die ("failed to open the file $dns_soa_serial_ldif");
	print FILE "dn: $dnsforwardzonefdn\nchangetype: modify\nreplace: dNIPSOASerial\ndNIPSOASerial: $soa_serial";

	print "Executing... LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem /opt/novell/eDirectory/bin/ldapmodify  -H 'ldaps://${frdDNS_provider}:1636' -D '$admin_name' -w ***** -f '$dns_soa_serial_ldif' -x -LLL\n" if($debug);

	`LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem /opt/novell/eDirectory/bin/ldapmodify -H 'ldaps://${frdDNS_provider}:1636' -D '$admin_name' -w '$admin_passwd' -f '$dns_soa_serial_ldif' -x 2>/dev/null`;
	my $ret = $?;

	if ($ret != 0) {
		#To maintain repeatable state after this failure so that same error happens in retry.
		print "Error while updating zone SOA";
	}

	#return here
	unlink($dns_soa_serial_ldif);
}



sub create_delete_rr_file {
	my $ldif_file = shift;
	my $zone_context = shift;
	my $zone_contiainer = shift;

	open(my $fh, '>', $ldif_file);
	print $fh "dn: cn=\\#ldap_#tcp_pdc_#msdcs,cn=$zone_contiainer,$zone_context\n";
	print $fh "changetype: delete\n";
	close $fh;
	print "Created file $ldif_file\n" if($debug);
}

sub Update_PDC_Master_SRV_record_ldap()
{
	my $frdDNS_provider = get_frdDNS_provider();
	my $hostname_fqdn = `hostname --fqdn`;
	my $dns_zone_context = get_zone_context();

	chomp($hostname_fqdn);

	my $zone_container = `/opt/novell/xad/share/dcinit/printConfigKey.pl DNSZONE`;
        chomp($zone_container);

	my $pdc_srv="";
	my $password = "";

	my $tree_administrator = getTreeAdministrator();

	my $domain = `hostname -d`;
	chomp($domain);
	my $old_pdc_ldif = "/var/opt/novell/xad/ds/dns/RR/delete_pdc_master.ldif";
	my $old_pdc_fqdn = `dig -t SRV _ldap._tcp.pdc._msdcs.$domain +short | awk '{print \$4}' | sed 's/.\$//g'`;
	chomp($old_pdc_fqdn);

	#my $old_pdc_srv = "_ldap._tcp.pdc._msdcs.$old_pdc_fqdn";
	$pdc_srv = "_ldap._tcp.pdc._msdcs";
	#my $old_pdc_srv_result = "";
	#$old_pdc_srv_result = "$old_pdc_srv IN SRV 0 100 389 $old_pdc_fqdn";
	#my $ldif_file = "/var/opt/novell/xad/ds/dns/RR/$old_pdc_srv".".ldif";
	my $ldif_file = "/var/opt/novell/xad/ds/dns/RR/$pdc_srv".".ldif";

	create_delete_rr_file($old_pdc_ldif,$dns_zone_context,$zone_container);


	$password = getPassword();

        print "Executing... LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem /opt/novell/eDirectory/bin/ldapmodify -H ldaps://$frdDNS_provider:1636 -D '$tree_administrator' -w ***** -f '$old_pdc_ldif' -x \n" if($debug);

        `LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem /opt/novell/eDirectory/bin/ldapmodify -H ldaps://$frdDNS_provider:1636 -D '$tree_administrator' -w '$password' -f '$old_pdc_ldif' -x 2>/dev/null`;
	my $ret = $?;


	#Identical code with 0 ttl but original pdc
	$pdc_srv = "_ldap._tcp.pdc._msdcs";
	my $pdc_srv_result = "$pdc_srv IN SRV 0 100 389 $hostname_fqdn";
	my $new_pdc_ldif = "/var/opt/novell/xad/ds/dns/RR/$pdc_srv".".ldif";

        dns::Process_RR("$pdc_srv IN SRV  0 100 389 $hostname_fqdn",0,
                "$dns_zone_context","FALSE","add",86400);

        print "Executing.. LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem /opt/novell/eDirectory/bin/ldapmodify -H ldaps://$frdDNS_provider:1636 -D '$tree_administrator' -w ***** -f '$new_pdc_ldif' -x \n" if($debug);

        `LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem /opt/novell/eDirectory/bin/ldapmodify -H ldaps://$frdDNS_provider:1636 -D '$tree_administrator' -w '$password' -f '$new_pdc_ldif' -x `;

	$ret = $?;

	print "Updating SOA\n";
	update_dns_soa;

	return $ret;

}

## MAIN ##
##########

sub main
{
	my $rc = 0;
	my %options=();

	getopts("hSdM",\%options) or usage;
	$role_seizure = 1 if defined $options{S};
	usage() if defined $options{h};
	$MS_DNS = 1 if defined $options{M};
	$debug = 1 if defined $options{d};

	# Check EUID
	Check_EUID();

	# Validate acquired credentials for valid Administrator SAM-Account-Name
	validate_tkt_for_admin_sam_name();

	# check if the local DC is the current PDC master as per the Directory backend
	goto out if($rc = Is_PDC_Master(\$Is_PDC_Master_LDAP_Backend));

	if ($Is_PDC_Master_LDAP_Backend == 0){
		print STDOUT gettext("This Domain Controller is not a PDC Master. Run this script on the PDC Master server"),"\n";
		$rc = 1;
		goto out;
	}	
	
	if ($rc = OldPDCMaster_DNS_Query()) {
		if ($Is_PDC_Master_DNS == 1) {
			$rc = 0;
			goto out;
		}
	}
	goto out if($rc != 0);

	create_updatePdcHelper_softLink();
	
	goto out if($rc = Update_Samba_Configuration());
	goto cleanup if($rc = Update_msdfsLink());
	
	# call the function to check and update the DNS database
	print "calling Update_PDC_Master_SRV_record() \n" if($debug);
	#$rc = Update_PDC_Master_SRV_record();
	$rc = Update_PDC_Master_SRV_record_ldap();

	if ($rc) {
		print STDERR gettext ("Can not update the pdc record using dynamic dns update method.\n");
		print STDERR sprintf (gettext ("Update the _ldap._tcp.pdc._msdcs.%s record manually to new pdc server.\n"), $domain);
		$rc = 0;
	}	 
	Update_xad_ini();
cleanup:
	RollBack(0) if($rc);

out:
	print STDOUT sprintf(gettext("PDC Configuration changes are done. Error %s"),$rc),"\n" if($rc);
	print STDOUT sprintf(gettext("PDC Configuration changes are done")),"\n" if (!($rc));
	print STDOUT gettext("Please refer to DSfW Administration Guide"),"\n" if($rc);
	unlink($ENV{'PIPE_STATUS_FILE'});
	delete ($ENV{'PIPE_STATUS_FILE'});
	CleanUp();
	exit $rc;
}

main;
