#!/usr/bin/perl -I. -I.. -I/opt/novell/xad/lib64/perl -I/opt/novell/xad/lib/perl
################################################################################
# DSFW version comparision script. The script will check the current DC version and all DC's version under the domain. 
# If all DCs are equal then the script will allow the installation otherwise will block it.
# Copyright (c) 2001-2020
#	Micro Focus or one of its affiliates.
################################################################################
use Getopt::Std;
use XAD::registry;
use File::Copy;
my $current_oes_version = "";
my $admin_name = "";
my $admin_password = "";
my $domain_name = "";
my $domain_netbios_name = "";
my $action = "";
my @all_dc_hostname_list = ();
my @lowerVersion_server_list = ();
my @unreachable_server_list = ();
my $server_name_list_string = "";
my $log_file_location = "/var/opt/novell/xad/log/ndsdcinit.log";
use Logger qw(:DEFAULT);
use constant EQUAL_OES_VERSION => 0;
use constant UNEQUAL_OES_VERSION => 1;
use constant UNREACHABLE_SERVER => 2;

my $localtime=localtime;
my ($day,$month,$date,$time,$year)=split(/ /,$localtime);
my $stamp=$year."-".$month."-".$date." ".$time;

# If ndsdcinit.log not exist then creation of the ndsdcinit.log file.
my $logdir = '/var/opt/novell/xad/log';
mkdir $logdir unless -d $logdir;
open(FFWL, ">>$logdir/ndsdcinit.log") or Log("cannot open $logdir/ndsdcinit.log file");

# To get the domain name or BIOS name.
sub get_domain {
        if ($action eq "adc") {
                $domain_name = `/bin/hostname -d`;
                chomp($domain_name);
		$domain_netbios_name = uc(substr( $domain_name, 0, index( $domain_name, '.' )));
        }
        if ($action eq "cdc") {
                $domain_name1 = `/bin/hostname -d`;
                chomp($domain_name1);
                my $domain_name2 = substr( $domain_name1, 0, index( $domain_name1, '.' ));
                $domain_name = `/bin/hostname -d | grep $domain_name2 | awk -F "$domain_name2." '{print \$2}'`;
                chomp($domain_name);
		$domain_netbios_name = uc(substr( $domain_name, 0, index( $domain_name, '.' )));
        }
	print FFWL "\n$stamp FRD Domain : $domain_name";
	print FFWL "\n$stamp NETBIOS Name : $domain_netbios_name";
}

# To Get the current server's OES version from /etc/novell-release file.
sub get_current_dc_version {
        my $localDCVersion  = `cat /etc/novell-release | grep -i VERSION`;
	$current_oes_version = (split /\ /,$localDCVersion)[2];
        chomp($current_oes_version);
	print FFWL "\n$stamp Current DC Version : $current_oes_version";
}

# For converting the FQDN of domain controller to IP address.
# The subroutine calls with one required parameter. @param {string} type
sub get_ip_from_hostname {
        my $ip_address = "";
        my $host_name = @_[0];
        $ip_address = `dig $host_name  +short`;
        chomp($ip_address);
        return $ip_address;
}

# To check remote server is power on or not.
sub server_up_or_not {
        my @server_response = `ping -c1 @_[0] && echo yes || echo no`;
        my $server_status = $server_response[@server_response-1];
        chomp($server_status);
        return ($server_status);
}

# To get all DCs list based on domain name in the form of IP Addresses.
sub get_all_dc_hostname_list {
        my @fqdn_list = ();
	print FFWL "\n$stamp Executing :dig -t SRV _ldap._tcp.dc._msdcs.$domain_name +short";
        my @dig_fqdn_list = `dig -t SRV _ldap._tcp.dc._msdcs.$domain_name +short`;
        # Looping for traversing and slicing every element from @fqdn_list array list, @fqdn_list array list contains all DCs FQDN list with some other info of current domain.
        for(@dig_fqdn_list) {
                my $fqdn = (split / /,$_)[3];
                chomp($fqdn);
                chop($fqdn);
                push @fqdn_list, $fqdn;
        }
        @all_dc_hostname_list = @fqdn_list;
	print FFWL "\n$stamp All DCs : @all_dc_hostname_list";
}

# This function is for retrieving the oes version of the remote server.
sub get_remote_oes_version {
	my $remote_server_ip = @_[0];
	my $remote_server_response = "";
	my $remote_server_response_status = "";
	my $domain_found = 0;
	# If server is powered off.
	if(server_up_or_not("$remote_server_ip") eq "no"){
		return("serverdown");
	}
	# Looping because sometimes this command gives the NT_STATUS_LOGON_FAILURE status with new IP in first attempt.
	for (my $tries = 1; $tries <= 3; $tries++) {
		# Output of rpcclient command :
		# Domain:         DSFW
		# Server:         (null)
		# Comment:        VERSION = 2018.1
		# ..., Based on the output status, will filter the Comment section value.
                $remote_server_response = qx "rpcclient -U '$domain_netbios_name\\\\$admin_name%$admin_password' $remote_server_ip -c querydominfo";
		# Here we are taking first word of the string.
		if ( index($remote_server_response, "Domain" ) != -1 ){
			$domain_found = 1;
			last;
		}
	}
	# Checking if string starts with Domain 
	# In which case we will split
	# to take required string from output.
	if ($domain_found) {
		my $indx = index($remote_server_response, "Comment");
		my $version_search = substr($remote_server_response, $indx);
		my $remote_oes_version = (split /=/, (split /\n/, $version_search)[0])[1];
		$remote_oes_version =~ s/^\s+|\s+$//g;
		return $remote_oes_version;
	} else{
		return("unreachable");
	}
}

# Based on the server status, separating the lists.
# This subroutine split the @all_dc_hostname_list
# parameter into two separate list
# @unreachable_server_list and @lowerVersion_server_list.
sub get_separate_dc_list {
	# Looping to get OES version for every FQDN
	# in the list @all_dc_hostname_list,
	# and seprate them into two different list.
        for(@all_dc_hostname_list){
                my $vr = get_remote_oes_version(get_ip_from_hostname($_));
                if( ($vr eq "unreachable") || ($vr eq "serverdown")) {
                        push @unreachable_server_list, $_;
                } elsif( $vr ne $current_oes_version || $vr eq "") {
                        push @lowerVersion_server_list, $_;
                }
        }
}

sub main {
	print FFWL "\n$stamp >>>comparing the all DC version and FRD version.(/opt/novell/xad/share/dcinit/compareToPdcOes.pl)";
	my @arg = @ARGV;
	# If the current DC is Additoin Domain Controller then
	# at the time of calling passed arg[0] should be --forAdc.
	if (@arg[0] eq "--forAdc") {
		$action = "adc";
        }
	# If the current DC is Child Domain Controller then
	# at the time of calling passed arg[0] should be --forCdc.
        if (@arg[0] eq "--forCdc") {
		$action = "cdc";
        }
	$admin_name = @arg[1];
	$admin_password = @arg[2];

	get_domain();
        get_current_dc_version();
        get_all_dc_hostname_list();
        get_separate_dc_list();

        my $lowerVersion_dc_count = @lowerVersion_server_list;
        my $unreachable_dc_countc = @unreachable_server_list;
        if( $lowerVersion_dc_count != 0 ) {
                print FFWL "\n$stamp These DCs are not equal as current DC :";
                for(@lowerVersion_server_list){
			print FFWL "\n $_";
			my $host_name = (split /\./,$_)[0];
			$server_name_list_string = $server_name_list_string." ".$host_name;
                }
		if( length($server_name_list_string) <= 80 ) {
			print "$server_name_list_string";
		} else {
			print $log_file_location;
		}
		print FFWL "\n$stamp Exit(1)";
                exit UNEQUAL_OES_VERSION;
        } elsif( $unreachable_dc_countc != 0 ) {
                print FFWL "\n$stamp These DCs are not reachable :";
                for(@unreachable_server_list){
			print FFWL "\n $_";
			my $host_name = (split /\./,$_)[0];
			$server_name_list_string = $server_name_list_string." ".$host_name;
                }
		if( length($server_name_list_string) <= 80 ) {
                        print "$server_name_list_string";
                } else {
                        print $log_file_location;
                }
		print FFWL "\n$stamp Exit(2)";
                exit UNREACHABLE_SERVER;
        } else {
                print FFWL "\n$stamp All DCs are equal.";
		print FFWL "\n$stamp Exit(0)";
                exit EQUAL_OES_VERSION;
        }
close(FFWL);
}
main
