#!/usr/bin/perl -I. -I/opt/novell/xad/lib64/perl -I/opt/novell/xad/lib/perl
################################################################################
# Domain Services for Windows Provisioning Tool
#
# See the file LICENSE for redistribution information.
#
# Copyright (c) 2001-2004
#	Novell, Inc.  All rights reserved.
################################################################################

#push(@INC, "/opt/novell/xad/lib64/perl", "/opt/novell/xad/lib/perl");

use XAD::registry;
use XAD::dclib;
use POSIX;     # for setlocale()
use Locale::gettext;

use Socket;

#
# Usage: genConfig.pl [--nds] [--dns] [--replica] [Configuration-File] [Dns-Domain] [Netbios-Domain] [Forest-Root-Domain] [Parent-Domain] [Mapped-Domain-Naming-Context]
#

sub ValidateHostNameAndAddress
{
	my $guid;
	my @subauth;
	my $iaddr;
	my $reversename, $fqdn;

	$HostName = `/bin/hostname` || die "/bin/hostname failed\n";
	chop($HostName);

	$HostName =~ s/\..*//; # strip trailing domain

	$iaddr = gethostbyname($HostName);
	if (!defined($iaddr)) {
		print STDERR sprintf(gettext("Could not resolve IP address for %s."), $HostName), "\n";
		exit 1;
	}

	if ((unpack("C4", $iaddr))[0] == 127) {
		print STDERR sprintf(gettext("IP address for %s is a loopback address; check /etc/hosts."), $HostName), "\n";
		exit 1;
	}

	$IpAddress = inet_ntoa($iaddr); # requires Socket library

	$reversename = gethostbyaddr($iaddr, AF_INET);
	if (!defined($reversename) || !$reversename) {
		print STDERR sprintf(gettext("The IP address %s does not resolve to a name."), $IpAddress), "\n";
		print STDERR sprintf(gettext("Please modify /etc/hosts so that it resolves to %s and retry."), $HostName), "\n";
		exit 1;
	}

	$reversename = lc($reversename); # lower-case for comparison
	$fqdn = $HostName . "." . $DomainName;

	if ($reversename ne lc($fqdn)) {
		print STDERR sprintf(gettext("The IP address %s resolves to %s."),$IpAddress,$reversename), "\n";
		print STDERR sprintf(gettext("Please correct /etc/hosts so that it resolves to %s.%s."),$HostName,$DomainName), "\n";
		exit 1;
	}
}

sub GenerateGuidAndSid
{
	$DomainGuid = `uuidgen` || die "uuidgen failed";
	chop($DomainGuid);

	# Generate the SID subauthorities from the GUID,
	# it is a suitable source of randomness. We do it
	# in reverse order to preserve some semblance of
	# hierarchy; the Ethernet address is last in the
	# GUID.
	$guid = XAD::packGuid($DomainGuid);
	@subauth = unpack("V4", $guid);
	$DomainSid = "S-1-5-21-$subauth[3]-$subauth[2]-$subauth[1]";
}

sub ReadDomainName
{
	$DomainName = `/bin/domainname` || die "/bin/domainname failed\n";
	chop($DomainName);
	if ($DomainName eq "(none)") { $DomainName = "" };
	if ($DomainName ne "") {
		$DomainName = $HostName;
	}
	if ($DomainName =~ /\./) {
		$DomainName =~ s/^(\w+)\.//;
	} else {
		$DomainName = "";
	}

	print sprintf(gettext("Enter the domain name for this domain: [%s] "), $DomainName);
	$_ = <STDIN>;
	chop($_);
	if ($_ ne "") {
		$DomainName = $_;
	}
}

sub GenerateConfig()
{
	if ($ARGV[0] eq "--nds") {
		shift @ARGV;
		$Nds = 1;
	}
	if ($ARGV[0] eq "--dns") {
		shift @ARGV;
		$DnsMaster = 1;
	}
	if ($ARGV[0] eq "--replica") {
		shift @ARGV;
		$Replica = 1;
	}

	if (defined($ARGV[1])) {
		$DomainName = $ARGV[1];
	} else {
		ReadDomainName();
	}

	if ($DomainName eq "") {
		print STDERR gettext("You must specify a domain name."), "\n";
		exit 2;
	}

	if (!($DomainName =~ /\./)) {
		print STDERR gettext("The domain name must have a period in it."), "\n";
		exit 3;
	}
#Max size allowed for Domain Name is 255
#eDirectory's Max DN size =255
#The size of longest default object in aquila tree =144 + Host Name
#So we are checking for 111 - Host Name  =(eDirectory's Max DN size - 144) characters 
#for Domain name

	$HostName = `/bin/hostname` || die "/bin/hostname failed\n";
        chop($HostName);
        $HostName =~ s/\..*//; # strip trailing domain

	my $Length_DOMAINNAME = 111 - length($HostName);
	if (length($DomainName) > $Length_DOMAINNAME ) {
		print sprintf(gettext("Domain names must be %s characters or less."), $Length_DOMAINNAME);
		exit 4;
	}
	@DnsLabels = split (/\./,$DomainName);
	foreach $Label (@DnsLabels) {
		if (length($Label) > 63) {
                	print STDERR gettext("Domain name Label must be 63 characters or less."), "\n";
	                exit 4;
        	}
	}

	# Valid characters are [A-Z][0-9]-\. (case-insensitive)
	if ($DomainName =~ /[^A-Za-z0-9-\.]/) {
		print STDERR gettext("The domain name contained invalid characters."), "\n";
		exit 5;
	}

	$DomainName = lc($DomainName);

	ValidateHostNameAndAddress();

	$NetbiosName = $DomainName;
	$NetbiosName =~ s/\.(.*)//;

	$NetbiosName = defined($ARGV[2]) ? uc($ARGV[2]) : uc($NetbiosName);

	if (!defined($ARGV[1])) {
		print sprintf(gettext("Enter the downlevel name for this domain: [%s] "), $NetbiosName);
		$_ = <STDIN>;
		chop($_);
		if ($_ ne "") {
			$NetbiosName = uc($_);
		}
	}

	if (length($NetbiosName) > 15) {
		print STDERR gettext("Downlevel domains must be 15 characters or less."), "\n";
		exit 6;
	}

	if (!defined($ARGV[1])) {
		print gettext("Do you want this machine to be a primary DNS server: [no] ");
		$_ = <STDIN>;
		chop($_);
		if (lc($_) eq "yes") {
			$DnsMaster = 1;
		}
	}

	$ForestRoot = defined($ARGV[3]) ? lc($ARGV[3]) : $DomainName;

	if (defined($ARGV[3])) {
		if (!($ForestRoot =~ /\./)) {
			print STDERR gettext("The forest root must have a period in it."), "\n";
			exit 3;
		}

		if (length($ForestRoot) > 120) {
			print STDERR gettext("Forest names must be 120 characters or less."), "\n";
			exit 4;
		}
	}

	if(defined($ARGV[5])) {
		$MappedDomainNC = $ARGV[5];

#Max size allowed for Domain Name is 255
#eDirectory's Max DN size =255
#The size of longest default object in aquila tree =144 + Host Name
#So we are checking for 111 - Host Name =(eDirectory's Max DN size - 144 ) characters

#For Mapped container name
		if(length($MappedDomainNC) > $Length_DOMAINNAME) {
			print sprintf(gettext("Domain names must be %s characters or less."), $Length_DOMAINNAME);
			exit 7;		
		}
	}
	else
	{
		$MappedDomainNC = "";
	}

	$MappedDomainNC = uc($MappedDomainNC);
	
	$ParentDomain = defined($ARGV[4]) ? lc($ARGV[4]) : $ForestRoot;	

	if (defined($ARGV[4])) {
		if (!($ParentDomain =~ /\./)) {
			print STDERR gettext("The parent domain name must have a period in it."), "\n";
			exit 3;
		}
	}

	$MachineGuid = `uuidgen` || die "uuidgen failed\n";
	chop($MachineGuid);

	$Date = `/bin/date`;
	chop($Date);

	if ($ARGV[0] eq "") {
		$Config = "config/" . lc($NetbiosName) . ".ini";
	} else {
		$Config = $ARGV[0];
	}

	if ($Nds && $Replica) {
		$ReplicationProvider = `/opt/novell/xad/sbin/provision -q -q --locate-dc $DomainName` || die "Failed to locate domain controller for domain $DomainName";
		chop($ReplicationProvider);
	}
}

sub DumpConfigToFile
{
	open(CONFIG,">$Config") || die "could not open $Config\n";

	print CONFIG "#\n";
	print CONFIG "# XAD configuration file for domain $DomainName\n";
	print CONFIG "# Generated by dcinit on $Date\n";
	print CONFIG "#\n"; 

	print CONFIG "[DefaultXADConfig]\n";
	print CONFIG "Domain GUID = $DomainGuid\n";
	print CONFIG "Domain SID = $DomainSid\n";
	print CONFIG "Netbios Name = $NetbiosName\n";
	print CONFIG "Root Domain = $DomainName\n";
	print CONFIG "Forest Root = $ForestRoot\n";
	print CONFIG "Parent Domain = $ParentDomain\n";
	print CONFIG "IP Address = $IpAddress\n";
	print CONFIG "Machine GUID = $MachineGuid\n";

	if ($MappedDomainNC ne "") {
		print CONFIG "Mapped Domain NC = $MappedDomainNC\n";
	}

	print CONFIG "DNS Master = ";
	if ($DnsMaster != 0) {
		print CONFIG "TRUE";
	} else {
		print CONFIG "FALSE";
	}
	print CONFIG "\n";

	if (defined($ReplicationProvider)) {
		print CONFIG "Replication Provider = $ReplicationProvider\n";
	}

	if ($Nds == 0) {
		# for XAD 2.0, default behaviour is to use NT security
		# for XAD 1.0 compat remove or set to FALSE
		print CONFIG "Native Security = TRUE\n";
	} else {
		print CONFIG "NDS = TRUE\n";
	}

	close(CONFIG);
}

sub main
{
	setlocale(LC_MESSAGES, "");
	bindtextdomain("xad", "/opt/novell/xad/share/locale");
	textdomain("xad");

	$DnsMaster = 0;

	GenerateGuidAndSid();
	GenerateConfig();
	DumpConfigToFile();
}

main();

