#!/usr/bin/perl -I. -I/opt/novell/xad/lib64/perl -I/opt/novell/xad/lib/perl
################################################################################
# Domain Services for Windows Provisioning Tool
#
# See the file LICENSE for redistribution information.
#
# Copyright (c) 2001-2004
#	Novell, Inc.  All rights reserved.
################################################################################

#push(@INC, "/opt/novell/xad/lib64/perl", "/opt/novell/xad/lib/perl");

use XAD::registry;
use XAD::dclib;

use Socket;

sub ValidateHostNameAndAddress
{
	my $iaddr;
	my $reversename, $fqdn;

	$iaddr = inet_aton($XAD::REGISTRY{"IP Address"}); # requires Socket library

	$reversename = gethostbyaddr($iaddr, AF_INET);
	if (!defined($reversename) || !$reversename) {
		print STDERR "WARNING: The IP address " . $XAD::REGISTRY{"IP Address"} . " does not resolve to a name. ";
		print STDERR "Please check for correct reverse DNS configuration.\n";
	}

	$reversename = lc($reversename); # lower-case for comparison

	$fqdn = $XAD::REGISTRY{"InstallMachineName"} . "." . $XAD::REGISTRY{"DNS Root"};
	$fqdn = lc($fqdn);

	if ($reversename ne $fqdn) {
		print STDERR "WARNING: The IP address " . $XAD::REGISTRY{"IP Address"} . " resolves to $reversename. ";
		print STDERR "Please correct reverse DNS so it resolves to $fqdn.\n";
	}
}

sub main
{
	$REPZONEFILE = $XAD::REGISTRY{"Prefix"} . "/share/dcinit/templates/dns-addlmachinereplica.zone";

	if ($#ARGV < 1) {
	        print STDERR "Usage: genReplicaDNSConfig.pl [InstallMachineName] [IP Address] [Site] [DNS SRV Priority] [DNS SRV Weight]\n";
		exit 1;
	}

	$XAD::REGISTRY{"InstallMachineName"} = $ARGV[0];
	$XAD::REGISTRY{"IP Address"} = $ARGV[1];

	if ($#ARGV > 1) {
		$XAD::REGISTRY{"Site"} = $ARGV[2];
	}

	if ($#ARGV > 2) {
		$XAD::REGISTRY{"DNS SRV Priority"} = $ARGV[3];
	} else {
		$XAD::REGISTRY{"DNS SRV Priority"} = 10; # greater than default of zero
	}
	if ($#ARGV > 3) {
		$XAD::REGISTRY{"DNS SRV Weight"} = $ARGV[4];
	}

	ValidateHostNameAndAddress();

	XAD::registryConfig();

	open(REPZONE, $REPZONEFILE) || die "Could not open $REPZONEFILE";
	while(<REPZONE>) {
		print XAD::regSubstitute($_);
	}
	close(REPZONE);
}

main();

