#!/bin/sh

LDAPSEARCH="/opt/novell/eDirectory/bin/ldapsearch"
SS_CERT="/etc/opt/novell/certs/SSCert.der"
SS_PEM="/etc/opt/novell/certs/SSCert.pem"

#Description : Both create_certificate and create_certificate1 functions are same but create_certificate 
# is using openssl command to get CA certificate and create_certificate1 is using ldap commands to get CA certificate.

#create_certificate1 is using ldap commands to get CA certificate
create_certificate1() {
        # Check the file exist and file size is not zero
        if [ -f "/etc/opt/novell/certs/SSCert.pem" -a -s "/etc/opt/novell/certs/SSCert.pem" ]
        then
                echo "/etc/opt/novell/certs/SSCert.pem CA certificate is already present"
                exit
        fi

        # Obtain the certificate authority DN
        certificate_authority=$($LDAPSEARCH -x -h $edir_ip_address -b cn=security -s one  objectclass=ndspkicertificateauthority dn -LLL 2>&1)
        if [ $? -ne 0 ]
        then
                echo "LDAP search failed to obtain certificate authority DN - [$certificate_authority]"
                exit
        fi
        certificate_authority=`echo $certificate_authority | sed 's/dn: //g'`
        echo "Certificate authority : $certificate_authority"

        # Create the certificate
        TEMPFILE=$(/bin/mktemp /tmp/tmp.XXXXXXXX)
        e_output=$($LDAPSEARCH -x -h $edir_ip_address -b "$certificate_authority" -s base cacertificate -LLL > $TEMPFILE 2>&1)
        if [ $? -ne 0 ]
        then
                echo "LDAP search failed to find the CA certificate with error - [`cat $TEMPFILE`]"
                exit
        fi

        `cat $TEMPFILE | sed -e '/dn:.*/d' | sed 's/cacertificate::/-----BEGIN CERTIFICATE-----\n/g' | grep -v ^$ | sed '$a-----END CERTIFICATE-----' | tee /tmp/SSCert.pem | /usr/bin/openssl x509 -outform DER -out /tmp/cert.der`
        if [ $? -ne 0 ]
        then
                echo "Failed to create the CA certificate."
                exit
        fi
        `/bin/cp /tmp/cert.der $SS_CERT`
        `rm $TEMPFILE`
        `rm /tmp/cert.der`
	openssl x509 -in $SS_CERT -inform DER > $SS_PEM
        # Test the certificate
        #output=$($LDAPSEARCH -x -h $edir_ip_address -p 636 -b "" -s base -e "$SS_CERT" >/dev/null 2>&1)
	#LDAPTLS_CACERT=$SERVER_PEM $LDAPCommand -b '' -D "$LdapAdminName" -w "$ADM_PASSWD" -s base 
	output=`LDAPTLS_CACERT=$SS_PEM $LDAPSEARCH -x -h $edir_ip_address -b "" -s base >/dev/null 2>&1`
        if [ $? -ne 0 ]
        then
                echo "CA certificate validation failed. error - [$output]"
                exit
        fi

        echo "CA certificate created successfully."
}

#create_certificate is using openssl command to get CA certificate 
create_certificate() {
        # Check the file exist and file size is not zero
        if [ -f "/etc/opt/novell/certs/SSCert.pem" -a -s "/etc/opt/novell/certs/SSCert.pem" ]
        then
                echo "/etc/opt/novell/certs/SSCert.pem CA certificate is already present"
                exit
        fi

        # Create the certificate
        TEMPFILE_INIT=$(/bin/mktemp /tmp/tmp.XXXXXXXX)
        `/usr/bin/openssl s_client -showcerts -connect $edir_ip_address:636 </dev/null > $TEMPFILE_INIT 2>&1`

        TEMPFILE_CERT=$(/bin/mktemp /tmp/tmp.XXXXXXXX)
        depth=$(cat $TEMPFILE_INIT | grep -i -m1 "^depth=[0-9]*" | cut -d' ' -f1 |cut -d'=' -f2)
        out_from_depth=$(cat $TEMPFILE_INIT | sed -n "/^ $depth s:/,/-----END CERTIFICATE-----/p" | sed '1,2d')
        echo "$out_from_depth" > $TEMPFILE_CERT

        `cat $TEMPFILE_CERT | tee /tmp/SSCert.pem | /usr/bin/openssl x509 -outform DER -out /tmp/cert.der`
        if [ $? -ne 0 ]
        then
                echo "Failed to create the CA certificate."
                exit
        fi
        `/bin/cp /tmp/cert.der $SS_CERT`
        `rm $TEMPFILE_INIT`
        `rm $TEMPFILE_CERT`
        `rm /tmp/cert.der`
	openssl x509 -in $SS_CERT -inform DER > $SS_PEM

        # Test the certificate
        #output=$($LDAPSEARCH -x -h $edir_ip_address -p 636 -b "" -s base -e "$SS_CERT" >/dev/null 2>&1)
	output=`LDAPTLS_CACERT=$SS_PEM $LDAPSEARCH -x -h $edir_ip_address -b "" -s base >/dev/null 2>&1`
        if [ $? -ne 0 ]
        then
                echo "CA certificate validation failed. error - [$output]"
                exit
        fi

        echo "CA certificate created successfully."
}

main() {
        edir_ip_address=$1
        if [ "X$edir_ip_address" = "X" ]
        then
                echo "Usage: ./getsscert_script.sh <eDirectory server IP address>"
                exit
        fi

        create_certificate
}
main "$@"
