#!/usr/bin/perl -I. -I/opt/novell/xad/lib64/perl -I/opt/novell/xad/lib/perl
################################################################################
# Domain Services for Windows Provisioning Tool
#
# See the file LICENSE for redistribution information.
#
# Copyright (c) 2001-2007
#       Novell, Inc.  All rights reserved.
################################################################################

use v5.10; # at least for perl 5.10
no warnings 'experimental::smartmatch';

use Getopt::Std;

$version = "1.1.1";
$err_file = "Can't open file: $!";

### functions ###

sub getprogramname()
{
	@appdir = split(/\\/, $0);
	return $appdir[$#appdir];	
}

sub help() 
{
	print "\n" .
        "Usage: $progname -t <template> -a <attrib> [-o <output>] [-s <seperator>] [-h]\n" .
        "\n" .
        " Options:\n" . 
        "   -t file  Template file. This file is the template of the LDIF code to\n" .
        "            be generated. Use *1..*n as variables to be replaces from the\n" .
        "            attribute file.\n" .
        "\n" . 
        "   -a file  \"Something\"-separaded file with attributes to be replaced by the\n" .
        "            variables in the template. Ie.:\n" .
        "\n" .
        "              attrib.txt file\n" .
        "              ---------------\n" .
        "              user1;locationA\n" .
        "\n" .
        "              template.ldif file\n" .
        "              ------------------\n" .
        "               dn: cn=*1,ou=users,o=myorg\n" .
        "               changetype: modify\n" .
        "               replace: l\n" .
        "               l: *2\n" .
        "\n" .
        "            This will generate:\n" .
        "\n" . 
        "               dn: cn=user1,ou=users,o=myorg\n" .
        "               changetype: modify\n" .
        "               replace: l\n" .
        "               l: locationA\n" .  
        "\n" .
        "   -o file  The file to write LDIF code to. If not specified it will be\n" .
        "            written to <STDOUT>.\n" .
        "\n" .
        "   -s sep   The separator for the attribute file. If not specified ';' will\n" . 
        "            be used. Use tab for \\t (tabulator) and space for whitespace.\n" .
        "\n" . 
        "   -h       Prints this help text." .
        "\n\n" .
        "Ie.: makeldif.pl -t template.file -a attrib.file -o new.ldif -s tab\n" .
        "\n";
}

sub usage() 
{
	print "\n" .
	      "Usage: $progname [-t file -a file -o file -s sep] [-h]\n" .
	      "Try `$progname -h' for more information.\n";
	      	
	exit(-1);	
}


### The main program ###

$progname = getprogramname();

my %copt=(); # commandline options

getopts('t:a:o:s:h', \%copt);

# Do the commandline swithes
if ($copt{t} && $copt{a}) { 

	my @userlist = ();
	my $userCount = 0;
	my $sep = "";
	my $attribCount = 0;
	my ($ldif, $new_ldif) = "";
	my @attrib = ();

	open(LDIFTEMPLATE, $copt{t}) or die $err_file;
	open(USERS, $copt{a}) or die $err_file;

	if ($copt{o}) {
		open(LDIFFILE, ">$copt{o}") or die $err_file;
	} else {
		print "version: 1\n\n";
	}
	
	# Getting seperator
	if ($copt{s}) {
		given($copt{s}) {
			when ("tab")  		{ $sep = '\t'; }
			when ("space")	{	 $sep = ' '; }
			when ("newline")	{ $sep = '\n'; }
			default			{ $sep = $copt{s}; }
		}
	} else {
		$sep=";";
	}
		
	while (<USERS>) {
		chomp;
		if($_ =~ /^#/) {
			# referral/commented line found, ignore it
			next;
		}
		else {
			$userlist[$userCount++] = $_;
		}
	}
	
	my @TempAttributecount = split(/$sep/, $userlist[$userCount - 1]);
	$attribCount = scalar @TempAttributecount;
	#$attribCount = scalar @userlist;
	
	while (<LDIFTEMPLATE>) {
		if($_ =~ /^dn:/) {
			#while reading the ldif template, replace the dn: with null
			$_ =~ s/dn: //g;
		}
		$ldif .= $_;
	}
	
	for (my $counter1 = 0; $counter1 < $userCount; $counter1++) {
		$new_ldif = $ldif;

		for (my $counter2 = 0; $counter2 < $attribCount; $counter2++) {
			@attrib = split(/$sep/, $userlist[$counter1]);
			my $counter3 = $counter2 + 1;
			$new_ldif =~ s/\*$counter3/$attrib[$counter2]/g;
		}

		if ($copt{o}) {
			print LDIFFILE "$new_ldif\n\n";
		} else {
			print "$new_ldif\n\n";
		}
	}
	
	close USERS;
	close LDIFFILE;
	close LDIFTEMPLATE;
	
} elsif ($copt{h}) {
	help();
} else {
	usage();
}

