#!/bin/sh
################################################################################
# Domain Services for Windows Provisioning Tool
# Verify the supported server versions in the tree root during YaST install
#
# See the file LICENSE for redistribution information.
#
# Copyright (c) 2006-2007
#       Novell, Inc.  All rights reserved.
################################################################################

# Return Values:
#---------------
# 0   The given server meets the minimum version requirement for Domain
#     Services for Windows.  Export the environment variable 
#     ROOT_SERVER before calling ndsdcinit.
# 1   Patch required for 64 bit syntax is not applied in the given server.
# 2   The given server does not have a read/write or master replica of 
#     the root partition.
# 3   Server is not reachable using the given IP address.
# 4   Not able to get the server version of the server.

ROOT_SERVER="$1"
treeName=`echo $2 | tr "[:lower:]" "[:upper:]"`
NcpServerDN=`/usr/bin/ldapsearch -h $ROOT_SERVER -b "" -s base -LLL -x dsaName | sed ' /^ / {; H; d; }; /^ /! {; x; s/\n //; }; ' | grep dsaName |awk -F"dsaName: " '{print $2}'`
if [ "$NcpServerDN" = "" ]
then
	exit 3
fi

NcpServerName=`echo $NcpServerDN | awk -F"," '{print $1}' | awk -F"cn=" '{print $2}'`
NcpServerDN_NDS=`echo $NcpServerDN | sed -e 's/\,\([a-z,A-Z,0-9]*\)=/.\1=/g' -e 's/st=/s=/ig'`;
ReplicaRing=`/opt/novell/eDirectory/bin/ndsstat -h $ROOT_SERVER -p ".T=$treeName." -d | grep -i "^.$NcpServerDN_NDS"`

test -z "$ReplicaRing" && exit 2

ReplicaType=`echo "$ReplicaRing" | sed "s/$NcpServerDN_NDS//i" |  awk '{print $2}'`

test -z "$ReplicaType"  && exit 2

ServerMajorVersion=0
ServerMajorVersion=`/usr/bin/ldapsearch -h $ROOT_SERVER -b "" -s base -x | sed ' /^ / {; H; d; }; /^ /! {; x; s/\n //; }; ' | grep -i version | tail -1 |awk -F "(" '{print $2}' | awk -F "." '{print $1}'`
if [ $ServerMajorVersion -eq 0 ]
then
	exit 4
fi

if [ "$ReplicaType" = "Master" -o "$ReplicaType" = "Read/Write" ]
then
	if [ -n $ServerMajorVersion ]
	then
		if [ $ServerMajorVersion -ge 20214 ]
		then
			exit 0
		else
			exit 1
		fi
	else
		exit 2
	fi
else
	exit 2
fi
