#!/usr/bin/perl -I. -I.. -I/opt/novell/xad/lib64/perl -I/opt/novell/xad/lib/perl

use strict;
use warnings;

use XAD::registry;
use util;
use Logger qw(:DEFAULT);
use wins_common;

my $wins = registry::getReg("WINSSUPPORT");
my $LIVE_ETCDIR = registry::getReg("LIVE_ETCDIR");
my $error = 0;
my $provisioning_log_file = registry::getReg("LIVE_LOCALSTATEDIR")."/log/provisioning.log";

if ((not defined $wins) or ($wins eq "") or (defined $wins and $wins eq "NO")) {
	Log(INFO, "Configuring WINS on this server");
	wins_common::clear_and_set(cnfg => 1);
	$error = wins_common::ConfigureWINS();
	if ($error == 0) {
		Log (INFO, "Successfully configured wins");
	} elsif ($error == 1 or $error == 2) {
		Log(INFO, "WINS already configured in the domain");
	} else {
		Log(ERR, "Failed to configure WINS, see wins_configure.log for more details");
	}

	if (not defined $wins or $wins eq "") {
		`echo "WINSSUPPORT = YES" >> /etc/opt/novell/xad/xad.ini`;
	}
	else {
		Log ("Updating the entry from NO to YES\n");
		util::replace_line_in_file ('^WINSSUPPORT = NO', "WINSSUPPORT = YES", "$LIVE_ETCDIR/xad.ini")
	}
	Log ("Doing post check for WINS");
	wins_common::clear_and_set(postchk => 1);
	$error = wins_common::ConfigureWINS();
	if ($error != 0) {
		Log ("Post check of WINS successfully done.");
	}

}elsif (( defined $wins and $wins eq "YES")) {
	# Here the tool takes care of performing a postcheck. So need not done again. 
	$error = wins_common::UnconfigureWINS();
	if ($error) {
		Log (ERR, "Failed to unconfigure WINS, see wins_configure.log for more details");
	}
	Log ("Successfully de-configured wins");
	util::replace_line_in_file ('^WINSSUPPORT = YES', "WINSSUPPORT = NO", "$LIVE_ETCDIR/xad.ini")
}


