#!/usr/bin/perl -I. -I/opt/novell/xad/lib64/perl -I/opt/novell/xad/lib/perl
################################################################################
# Domain Services for Windows Provisioning Tool
#
# See the file LICENSE for redistribution information.
#
# Copyright (c) 2001-2020
#       Micro Focus or one of its affiliates.
################################################################################

$ENV_PATH = "/opt/novell/xad/sbin:/opt/novell/xad/bin:/opt/novell/xad/share/dcinit:/opt/novell/eDirectory/bin:";
$ENV{'PATH'} = $ENV_PATH.$ENV{'PATH'};

use XAD::registry;
use Install::frd_install;
use Install::child_install;
use Logger qw(:DEFAULT);
use Install::check;
#use Install::check_util;
use Install::adc_install;
use util_update;

my $_LIB = registry::getReg("_Lib");
my $SASL_PATH = "/opt/novell/xad/$_LIB/sasl2";
$ENV{'_LIB'} = $_LIB;
$ENV{'SASL_PATH'} = $SASL_PATH;

my $type = registry::getReg('INSTALL_TYPE');
my $is_replica = registry::getReg("IS_REPLICA");
my $require_tls = registry::getReg("Require TLS");
my $inst;

sub update_DC_configuration() {
    #operations taken from crontab entry addition task
    $inst->stage_ntp();
    $inst->stage_sysvol();
    $inst->stage_gpo();
    $inst->sync_gpo_nmas();
    $inst->update_cron_tab();

    util::process_suseconfig_profile();
    util::process_ssh_config();
    util::process_sshd_config();
    util::process_krb5_config();
    util::process_rsyncd_config();
    util::ssh_rsync_restart();
}

if ($is_replica eq "TRUE") {
    $inst = adc_install->new();
    &update_DC_configuration();
}
elsif ($type eq "frd") {
    $inst = frd_install->new();
}
elsif ($type eq "child") {
    $inst = child_install->new();
}

my ($status, $msg) = check::pre_check ("CLEANUP");
if ($status == 1) {
	#Operations taken from crontab entry addition task
	&update_DC_configuration();
	util::update_pamd_sshd();

	Log("Pre-check of cleanup Passed");
	if ($require_tls eq "TRUE") {
		$inst->nldap_enable_requiretls();
	}

	if ($type eq "frd" and $is_replica eq "FALSE") {
		$inst->delete_temporary_admin();
	}

	$inst->stage_clean();

	# Create the xad_provisioned file
	Log("Creating the file /opt/novell/oes-install/xad_provisioned");
	`touch "/opt/novell/oes-install/xad_provisioned"`;
	
	# Delete the file "start-xad-provisioning.desktop" to stop auto-launch
	# of provisioning plugin.
	Log("Removing the file /etc/xdg/autostart/start-xad-provisioning.desktop");
	unlink("/etc/xdg/autostart/start-xad-provisioning.desktop");
 
	# Remove the temporary ticket cache files
	$inst->remove_ticket_cache();

	if (registry::getReg("DNS Master") eq "TRUE") {
		$inst->update_resolv_conf();
	}

	util_update::dc_schema_version_update_2019();
	util_update::update_functional_levels_2019();
	util_update::update_for_win10();
	util_update::update_max_smbd_param();

	util::restart_services();

	#namcd is unable to fetch the group information details of dsfw 
	#users. so restarting namcd service(Bug 1156769).
	my $service_start_status = 0;
	if (-e "/etc/nam.conf") {
		`/usr/bin/systemctl status namcd.service`;
		if ($?) {
			Log(WARN, "The namcd daemon was not running so starting it.");
			`/usr/bin/systemctl start namcd.service`;
			$service_start_status = $?;
			Log(WARN, "Service namcd started with code $service_start_status");
		} else {
			Log(WARN, "The namcd daemon is running so re-starting it.");
			`/usr/bin/systemctl restart namcd.service`;
			$service_start_status = $?;
			Log(WARN, "Service namcd started with code $service_start_status");
		}
	}

	#This call is made during fresh install for adding "default_ccache_name = FILE:/tmp/krb5cc_0" string in /etc/krb5.conf file.
        util::add_cache_string_in_krb5_conf_file();

	($status, $msg) = check::post_check ("CLEANUP");
	Log("Post-check of cleanup Passed") if ($status);
	Log(ERR, "Post-check of cleanup Failed:[$msg]") if (!$status);

	# Completed the provisioning, updating the status in xad.ini
	util::file_append_str("PROVISIONED = YES\n", registry::getReg("LIVE_ETCDIR")."/xad.ini");
	
} else {
	Log(ERR, "Pre-check of cleanup Failed:[$msg], not proceeding for cleanup");
}
