#!/usr/bin/perl -I. -I/opt/novell/xad/lib64/perl -I/opt/novell/xad/lib/perl
################################################################################
# Domain Services for Windows Provisioning Tool
#
# See the file LICENSE for redistribution information.
#
# Copyright (c) 2001-2009
#	Novell, Inc.  All rights reserved.
################################################################################


$ENV{'PATH'} = "/opt/novell/xad/share/dcinit:$ENV{'PATH'}";
use XAD::registry;
use Install::frd_install;
use Install::child_install;
use Logger qw(:DEFAULT);
use Install::check;
#use Install::check_util;
use Install::adc_install;
use XAD::secure;
use util;

my $type = registry::getReg('INSTALL_TYPE');
my $is_replica = registry::getReg("IS_REPLICA");
my $is_mapped = int(registry::getReg("MAPPED"));
my $retain_policy = lc(registry::getReg("XADRETAINPOLICIES"));
my $inst;

if ($is_replica eq "TRUE") {
    $inst = adc_install->new();

    #Setting sasl path
    my $_LIB = registry::getReg("_Lib");
    $ENV{SASL_PATH} = "/opt/novell/xad/$_LIB/sasl2";

    $inst->refresh_nldap();
    # Stage1 to add crossrefreference attribute
    $inst->add_custom_objects ("stage1");

    $inst->import_adc_acls();
}
else {

	my ($status, $msg) = check::pre_check ("ACLCONFIGURATION");
	if ($status == 1) {
		Log("Pre-check of ACL configuration Passed");
		if ($type eq "frd") {
			$inst = frd_install->new();

			$inst->compute_effective_acls();
			$inst->import_nds_super_rights_acls();
			$inst->import_nds_admin_acls();
		}
		elsif ($type eq "child") {
			$inst = child_install->new();
		}

		if ($is_replica eq "FALSE") {
			$inst->import_domain_server_reference();

			$inst->refresh_nldap();
			# Stage1 of add_custom_objects
			$inst->add_custom_objects ("stage1");

			util::stop_ndsd();
			sleep(10);
			util::start_ndsd();

			util::send_domain_update_now_signal();	

			$inst->import_nds_domain_acl();
			#
			# Kamalesh - FIXME
			# Commenting this portion for now. Needs a re-look.
			# Overwrites PP at whole sub-tree unexpectedly
			# Needs to
			# 1. Follow domain boundary
			# 2. Dump the existing PP associations
			# 3. Update the PP with Domain's
			#
			#if ($is_mapped == 1 and $retain_policy eq "no") {
			#	$inst->import_nds_policies()
			#}

			secure::associate_passwd_policy($retain_policy);

			# Stage1 of add_custom_objects
			$inst->add_custom_objects ("stage2");

			util::send_domain_update_now_signal ();
		}
		($status, $msg) = check::post_check ("ACLCONFIGURATION");
		Log("Post-check of ACL configuration Passed") if ($status);
		Log(ERR, "Post-check of ACL configuration Failed:[$msg]") if (!$status);
	}
	else {
		Log(ERR, "Pre-check of ACL configuration Failed:[$msg], not proceeding for ACL configuration");
	}

}
