#!/usr/bin/perl -I. -I/opt/novell/xad/lib64/perl -I/opt/novell/xad/lib/perl
################################################################################
# Domain Services for Windows Provisioning Tool
#
# See the file LICENSE for redistribution information.
#
# Copyright (c) 2001-2009
#	Novell, Inc.  All rights reserved.
################################################################################


$ENV{'PATH'} = "/opt/novell/xad/share/dcinit:$ENV{'PATH'}";
use XAD::registry;
use Install::frd_install;
use Install::child_install;
use Logger qw(:DEFAULT);
use Install::check;
#use Install::check_util;
use Install::adc_install;
use wins_common;
use checkdupws;

my $type = registry::getReg('INSTALL_TYPE');
my $is_replica = registry::getReg("IS_REPLICA");
my $inst;
my $provisioning_log_file = registry::getReg("LIVE_LOCALSTATEDIR")."/log/provisioning.log";
my $wins_script_status = -1;


sub wins_configuration(){
	#WINS related changes
	Log(INFO, "Initializing environment for checkdupws.pl");

	checkdupws::clear_and_set(init => 1);
	my $ret = checkdupws::checkDuplicateWorkStation(checkdupws::XAD_PROVISIONING);
	if ($ret != 0) {
		Log(ERR, "Failed to initialize environment for".
		" checkdupws.pl [$ret]");
	}

	if ( lc(registry::getReg("WINS")) eq 'no'){
		Log("WINS not selected to be configured");
		return 0;
	}

	#This code is for retry comaptibility of WINS in DSfW provisioning
	#Uses LDAP search to check if WINS is configured or not
	wins_common::clear_and_set(is_cnfg_upg => 1);
	$wins_script_status = wins_common::ConfigureWINS();
	Log(INFO, "Status obtained : $wins_script_status");
	if ($wins_script_status == 1 or $wins_script_status == 2) {
		Log(INFO, "WINS already configured in the domain");
		return 0;
	} elsif ($wins_script_status != 0) {
		Log(INFO, "Unable to determine status of WINS in the domain");
		return 0;
	}

	#We are going to configure WINS ....
	Log(INFO, "Configuring WINS on this server");
	wins_common::clear_and_set();
	$wins_script_status = wins_common::ConfigureWINS();
	Log(INFO, "Status obtained : $wins_script_status");
	if($wins_script_status != 0){
		Log(ERR, "Failed to configure WINS on this server");
	}
}

if ($is_replica eq "TRUE") {
	$inst = adc_install->new();
	$inst->configure_zone_object();
	$inst->configure_zone_rr_object();
	if (!($type eq "frd" && $is_replica eq "FALSE") &&
			registry::getReg("DNS Master") eq "TRUE") {
		util::novell_named_restart();
	}

	&wins_configuration();

	exit (0);
}
elsif ($type eq "frd") {
    $inst = frd_install->new();
}
elsif ($type eq "child") {
   $inst = child_install->new();
}

my ($status, $msg) = check::pre_check ("DNSCONFIGURATION");
if ($status == 1) {
	Log("Pre-check of DNS configuration Passed");
	$inst->configure_zone_object();
	$inst->configure_zone_rr_object();

	&wins_configuration();

	($status, $msg) = check::post_check ("DNSCONFIGURATION");
	Log("Post-check of DNS configuration Passed") if ($status);
	Log(ERR, "Post-check of DNS configuration Failed:[$msg]") if (!$status);
} else {
	Log(ERR, "Pre-check of DNS configuration Failed:[$msg], not proceeding for DNS configration");
}
