#!/usr/bin/perl -I. -I/opt/novell/xad/lib64/perl -I/opt/novell/xad/lib/perl
################################################################################
# Domain Services for Windows Provisioning Tool
#
# See the file LICENSE for redistribution information.
#
# Copyright (c) 2001-2020
#	Micro Focus or one of its affiliates.
################################################################################


$ENV{'PATH'} = "/opt/novell/xad/share/dcinit:$ENV{'PATH'}";
use XAD::registry;
use Logger qw(:DEFAULT);
use Install::frd_install;
use Install::child_install;
use Install::adc_install;
use Logger qw(:DEFAULT);
use util;

my $type = registry::getReg('INSTALL_TYPE');
my $is_replica = registry::getReg("IS_REPLICA");
my $launch_config_wizard = registry::getReg("LAUNCH_CONFIG_WIZARD");
my $parentdomain;
my $inst ;
my ($adminDN, $ipAddress);

#For setting the DSfW path variable.

`systemctl stop ndsd`;
#util::disable_fips();
`systemctl start ndsd`;

my $file="/etc/bash.bashrc.local";
`grep -q -i "novell-xad.sh" $file > /dev/null 2>&1`;
if ($?) {
    `echo ". /etc/profile.d/novell-xad.sh" >> $file`;
}

if ($is_replica eq "TRUE") {
        $inst = adc_install->new();
	$adminDN = registry::getReg("LDAPADMINNAME");
	$ENV{NDSEXISTINGADMINNAME} = registry::getReg("Mapped Administrator DN");
	if ($launch_config_wizard eq "YES") {
		$adminDN = `/opt/novell/xad/share/dcinit/provisionTools.sh get-admin-name`;;
		chomp($adminDN);
		$ENV{NDSEXISTINGADMINNAME} = $adminDN;
	}
        $ipAddress = registry::getReg('DNS_SERVER_IP'); # Only for non-name mapped frd
        $ENV{ADM_PASSWD_PARENT} = $ENV{ADM_PASSWD};
	$ENV{NDSEXISTINGADMINPASSWD} = $ENV{ADM_PASSWD};

	Log(MESG, ">>>Starting the DSfW pre-install health check");
        `/opt/novell/xad/share/dcinit/healthCheck.pl -r -n -b -c -e -A '$adminDN'`;
        my $error = $? >> 8;
        if ($?) {
		Log(MESG, ">>>DSfW pre-install health check failed. Check /var/opt/novell/xad/log/healthCheck.log for more details.");
		die "Failed to run the DSfW pre-install health check.";
	}
        Log(MESG, "   Successfully health checked the [$type-adc] [$ipAddress]");
}
elsif ($type eq "frd") {
	$inst = frd_install->new();
	$adminDN = registry::getReg('LDAPADMINNAME');
	$ipAddress = registry::getReg('IP Address'); # Only for non-name mapped frd
	$isMapped = int(registry::getReg("MAPPED"));

	Log(MESG, ">>>Starting the health check");
	if ($isMapped) {
		if ($ENV{LDAPEXISTINGADMINNAME} ne "") {
			$adminDN = $ENV{LDAPEXISTINGADMINNAME};
		}
		system("/opt/novell/xad/share/dcinit/healthCheck.pl -r -n -b -c -e -A '$adminDN' -I $ipAddress");
	}
	else {
		if ($launch_config_wizard eq "YES") {
			$adminDN = `/opt/novell/xad/share/dcinit/provisionTools.sh get-admin-name`;
			chomp($adminDN);
		}
		Log ("The admin name as [$adminDN]\n");
		system("/opt/novell/xad/share/dcinit/healthCheck.pl -a -A '$adminDN' -I $ipAddress");
	}
	my $error = $? >> 8;
        if ($?) {
		Log(MESG, ">>>DSfW pre-install health check failed. Check /var/opt/novell/xad/log/healthCheck.log for more details.");
		die "Failed to run the DSfW pre-install health check.";
	}
	Log(MESG, "   Successfully health checked the FRD [$ipAddress]");
}
elsif ($type eq "child") {
	$inst = child_install->new();
	$adminDN = registry::getReg("NDSPARENTDOMAINADMIN");
	my $parent_domain = registry::getReg("Parent Domain");
	if (uc($launch_config_wizard) eq "YES") {
		$adminDN = `/opt/novell/xad/share/dcinit/provisionTools.sh get-admin-name -d $parent_domain`;
		chomp($adminDN);
	}
	$adminDN =~ s/\./,/g;

        $parentdomain = registry::getReg("Parent Domain");
        util::send_domain_update_now_signal ($parentdomain);
	
	Log(MESG, ">>>Starting the health check");
	system("/opt/novell/xad/share/dcinit/healthCheck.pl -r -n -b -c -e -A '$adminDN'");
	my $error = $? >> 8;
        if ($?) {
		Log(MESG, ">>>DSfW pre-install health check failed. Check /var/opt/novell/xad/log/healthCheck.log for more details.");
		die "Failed to run the DSfW pre-install health check.";
	}
	Log(MESG, "   Succesfully health checked the child domain");
}


