#!/usr/bin/perl -I. -I/opt/novell/xad/lib64/perl -I/opt/novell/xad/lib/perl
################################################################################
# Domain Services for Windows Provisioning Tool
#
# See the file LICENSE for redistribution information.
#
# Copyright (c) 2001-2009
#	Novell, Inc.  All rights reserved.
################################################################################

$ENV{'PATH'} = "/opt/novell/xad/share/dcinit:$ENV{'PATH'}";
use XAD::registry;
use Install::frd_install;
use Install::child_install;
use Install::adc_install;
use Logger qw(:DEFAULT);
use Install::check;
#use Install::check_util;

my $type = registry::getReg('INSTALL_TYPE');
my $is_replica = registry::getReg("IS_REPLICA");
my $is_upgrade = registry::getReg("UPGRADE");
my $inst;

# Replacing the ldap.conf
util::replace_ldapconf();

# Updating the usr.sbin.nmbd apparmor profile
util::update_apparmor_nmbd();

# Updating the usr.sbin.smbd apparmor profile
util::update_apparmor_smbd();

# Updating the usr.sbin.samba-bggd apparmor profile
util::update_apparmor_samba_bgqd();

# Updating the usr.sbin.winbindd apparmor profile
util::update_apparmor_winbindd();

#Restarting apparmor to get the apparmor profiles changes of nmbd, smbd and winbindd
my $service_start_status;

`/usr/bin/systemctl restart apparmor.service`;
$service_start_status = $?;
Log(WARN, "Service apparmor started with code $service_start_status");

if ($is_replica eq "TRUE") {
	$inst = adc_install->new();
}
elsif ($type eq "frd") {
    $inst = frd_install->new();
}
elsif ($type eq "child") {
    $inst = child_install->new();
}

my ($status, $msg) = check::pre_check ("RESTARTDSFW");
if ($status == 1) {
	Log("Pre-check to restart DSFW services Passed");

	if ($is_upgrade ne "" && $is_upgrade eq "YES") {
		$inst->restart_services();
	}
	else 
	{
		$inst->update_nsswitch();
		$inst->restart_services();

		if (($is_replica eq "FALSE") && ($type eq "child") 
			&& (!registry::getReg("MAPPED"))) {
			$inst->touch_all_users_groups();
		}
	}

	($status, $msg) = check::post_check ("RESTARTDSFW");
	Log("Post-check of DSFW services status Passed") if ($status);
	Log(ERR, "Post-check of DSFW services status Failed:[$msg]") if (!$status);
} else {
	Log(ERR, "Pre-check to restart DSFW services Failed:[$msg], not proceeding to restart DSFW services");
}
