#!/usr/bin/perl -I. -I/opt/novell/xad/lib64/perl -I/opt/novell/xad/lib/perl
################################################################################
# Domain Services for Windows Provisioning Tool
#
# See the file LICENSE for redistribution information.
#
# Copyright (c) 2001-2009
#	Novell, Inc.  All rights reserved.
################################################################################


$ENV{'PATH'} = "/opt/novell/xad/share/dcinit:$ENV{'PATH'}";
use XAD::registry;
use XAD::nds;
use Install::frd_install;
use Install::child_install;
use Install::check;
use Logger qw(:DEFAULT);

my $type = registry::getReg('INSTALL_TYPE');
my $is_mapped = int(registry::getReg("MAPPED"));
my $is_replica = registry::getReg("IS_REPLICA");
my $adm_principal = registry::getReg("Domain Administrator Principal");
my $inst;

my ($status, $msg) = check::pre_check ("SAMIFICATION");
if ($status == 1) {
	Log("Pre-check of container object samification Passed");

	if ($type eq "frd") {
		$inst = frd_install->new();
	}
	elsif ($type eq "child") {
		$inst = child_install->new();
		$ENV{KRB5CCNAME} = registry::getReg("KRB5CCNAME_DOMAIN");

		if ($is_replica eq "FALSE") {
			$adm_principal = uc($adm_principal);
		}
	}
	
	nds::wait_for_samspm();

	registry::setReg("LDAPSEARCH_FLAGS", " -Y GSS-SPNEGO");
	registry::setReg("LDAPMODIFY_FLAGS", " -Y GSS-SPNEGO");
	registry::setReg("ADM_PRINCIPAL", $adm_principal);

	if ($inst->can_samify () == 0) {
		$inst->samify_all_domain_objects();
	}
	else {
		Log(ERR,"Samification failed, try again");
	}		

	#Before provisioning the child domain the administrator objects and few more
	#objects gets created and those objects are provisioned by the parent module
	#Hence they hold the parent domain's uniqueDomainID

	#This happens only during non-name mapped child domain controller installation
	#and those tainted objects should be cleaned up to avoid further failures.
	if ($type eq "child" && $is_mapped == 0 &&  $is_replica eq "FALSE") {
		$inst->remove_invalid_uniquedomainid();
	}

	# As the eDirectory is restarted in the previous task, it is observed
	# that the AD keys generation (by samspm module) is affected and 
	# the key generation is not completing successfully in all cases

	# this block will check for the unicodePwd attribute generation, and if 
	# missing will attempt to re-generate it for a while

	$inst->check_and_generate_AD_keys();		

	($status, $msg) = check::post_check ("SAMIFICATION");
	Log("Post-check of container objects samification Passed") if ($status);
	Log(ERR, "Post-check of container objects samification Failed:[$msg]") if (!$status);
} else {
	Log(ERR, "Pre-check of container objects samification Failed:[$msg], not proceeding for samification");
}
