#!/bin/sh
# This script is used for various replica removal operations
############
# Variables
############
	
_LIB=`/opt/novell/xad/share/dcinit/printConfigKey.pl "_Lib"`
export _LIB
SASL_PATH=/opt/novell/xad/$_LIB/sasl2
export SASL_PATH
LDAP_FLAGS="-Y GSS-SPNEGO"
config_file=`/opt/novell/xad/share/dcinit/printConfigKey.pl "NdsConfDir"`
treeName=`/opt/novell/xad/share/dcinit/ndsConfigGet.sh "n4u.base.tree-name"`
mappedDomainNC=`/opt/novell/xad/share/dcinit/printConfigKey.pl "Mapped Domain NC"`
mappedDomainNCnds=`echo $mappedDomainNC | sed 's/,/\./g'`
adminNameLdapFormat=`/opt/novell/xad/share/dcinit/printConfigKey.pl "NDSEXISTINGADMINNAME" | sed 's/\./,/g'`
currentServerContextNdsName=`ndsstat | grep "Server Name:" | awk -F": " '{print $2}'`

##############
#Functions
##############

removeConfigReplica()
{
	configPartitionNdsName="CN=Configuration.$mappedDomainNCnds"
	configPartitionFullNdsName=".$configPartitionNdsName.T=$treeName."
	ndsstat -n -p $configPartitionFullNdsName --config-file $config_file/nds.conf 2>/dev/null | while read serverFullNdsName
        do
		if [ "$serverFullNdsName" != "$currentServerContextNdsName" ]
	        then
			serverLdapName=`echo $serverFullNdsName | sed 's/^\.//g' | awk -F".T=" '{print $1}' | sed 's/\./,/g'`
			
			/opt/novell/xad/sbin/replops -s "$serverLdapName" -n "CN=Configuration,$mappedDomainNC" -t "writable" -D "$adminNameLdapFormat" -w "env:NDSEXISTINGADMINPASSWD" --remove-replica;
	fi	
		done

}
removeSchemaReplica()
{
	schemaPartitionNdsName="CN=Schema.CN=Configuration.$mappedDomainNCnds"
	schemaPartitionFullNdsName=".$schemaPartitionNdsName.T=$treeName."
	ndsstat -n -p $schemaPartitionFullNdsName --config-file $config_file/nds.conf 2>/dev/null | while read serverFullNdsName
	do
		if [ "$serverFullNdsName" != "$currentServerContextNdsName" ]
		then
			serverLdapName=`echo $serverFullNdsName | sed 's/^\.//g' | awk -F".T=" '{print $1}' | sed 's/\./,/g'`
			/opt/novell/xad/sbin/replops -s "$serverLdapName" -n "CN=Schema,CN=Configuration,$mappedDomainNC" -t "writable" -D "$adminNameLdapFormat" -w "env:NDSEXISTINGADMINPASSWD" --remove-replica;
		fi
	done
		
}
removeReplicaFromOtherDomains()
{
	parentDomain=`/opt/novell/xad/share/dcinit/printConfigKey.pl "Parent Domain"`
	dnsRoot=`/opt/novell/xad/share/dcinit/printConfigKey.pl "DNS Root"`
	parentDomainProvider=`/opt/novell/xad/sbin/provision -q -q -l 60 --locate-dc $parentDomain`
	if [ $? -ne  0 ]
	then
		error=$?>>8
		exit $error
	fi
	ldapParentContext=`/opt/novell/xad/share/dcinit/printConfigKey.pl "Parent Domain NC"`
	ndsParentContext=`echo $ldapParentContext | sed 's/,/./g'`
	ndsParentAdminName="CN=Administrator.CN=Users.$ndsParentContext"
	export ADM_PASSWD=$ADM_PASSWD_PARENT
	partitionNdsName=`/opt/novell/xad/share/dcinit/ndsConfigServerContext.sh $arg $dnsRoot $ndsParentAdminName $parentDomainProvider`
	if [ "$action" = "--child" ]
	then
		defaultContainerName=`/opt/novell/xad/share/dcinit/printConfigKey.pl "DEFAULT_CONTAINER"`;
		if [ $defaultContainerName -eq "ou=novell" ]
		then
			partitionNdsName=`echo $partitionNdsName | awk -F"^ou=novell." '{ print $2}'`
		else
			partitionNdsName=`echo $partitionNdsName | awk -F"^ou=OESSystemObjects." '{ print $2}'`
		fi
	fi
	partitionLdapName=`/opt/novell/xad/share/dcinit/printConfigKey.pl "Mapped Domain NC"`
	partitionFullNdsName=".$partitionNdsName.T=$treeName."

	ndsstat -n -p $partitionFullNdsName --config-file $config_file/nds.conf 2>/dev/null | while read serverFullNdsName 
	do
		serverRDN=`echo "$serverFullNdsName" | awk -F"." '{print $2}'`
		replicaType=`ndsstat -p "$partitionFullNdsName" | grep -i $serverRDN | awk '{print $2}'`
		if [ "$replicaType" = "Subordinate" ]
		then
			continue
		fi

		if [ "$serverFullNdsName" != "$currentServerContextNdsName" ]
		then
			serverLdapName=`echo $serverFullNdsName | sed 's/^\.//g' | awk -F".T=" '{print $1}' | sed 's/\./,/g'`
			/usr/bin/ldapsearch $LDAP_FLAGS -H ldap://$parentDomainProvider -b $serverLdapName -s base | sed ' /^ / {; H; d; }; /^ /! {; x; s/\n //; }; ' | grep serverReference > /dev/null 
			if [ $? = 0 ]
			then
				/opt/novell/xad/sbin/replops -s "$serverLdapName" -n "$partitionLdapName" -D "$adminNameLdapFormat" -w "env:NDSEXISTINGADMINPASSWD" --remove-replica;
				sleep 40
			fi
		fi
	done
}

main() 
{
	action=$1
	shift
	if [ "$action" = "--nm-child" ] 
	then
		arg="--nm-child"		
	fi
	
	case $action in
		--child |--nm-child) removeReplicaFromOtherDomains ;;
		--remove-schema-replica) removeSchemaReplica ;;
		--remove-config-replica) removeConfigReplica ;;
		*) gettext -s "Invalid option" ;;
	esac
}

main $@
